-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.strings.wide_maps.spc,v $ 
-- $Revision: 1.11 $ $Date: 96/10/21 15:19:11 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

package Ada.Strings.Wide_Maps is
    pragma Suppress( Elaboration_Check );
    pragma Preelaborate (Wide_Maps);

    --  Representation for a set of Wide_Character values:
    type Wide_Character_Set is private;

    Null_Set : constant Wide_Character_Set;

    type Wide_Character_Range is record
       Low  : Wide_Character;
       High : Wide_Character;
    end record;
    --  Represents Wide_Character range Low .. High

    type Wide_Character_Ranges is array (Positive range <>)
        of Wide_Character_Range;

    function To_Set    (Ranges : in Wide_Character_Ranges)
       return Wide_Character_Set;

    function To_Set    (Span : in Wide_Character_Range)
       return Wide_Character_Set;

    function To_Ranges (Set :  in Wide_Character_Set)
       return Wide_Character_Ranges;

    function "=" (Left, Right : in Wide_Character_Set) return Boolean;

    function "not" (Right  : in Wide_Character_Set) 
       return Wide_Character_Set;
    function "and" (Left, Right : in Wide_Character_Set)
       return Wide_Character_Set;
    function "or"  (Left, Right : in Wide_Character_Set)
       return Wide_Character_Set;
    function "xor" (Left, Right : in Wide_Character_Set)
       return Wide_Character_Set;
    function "-" (Left, Right : in Wide_Character_Set)
       return Wide_Character_Set;

    function Is_In (Element : in Wide_Character;
                    Set     : in Wide_Character_Set)
       return Boolean;

    function Is_Subset (Elements : in Wide_Character_Set;
                        Set      : in Wide_Character_Set)
       return Boolean;

    function "<=" (Left  : in Wide_Character_Set;
                   Right : in Wide_Character_Set)
       return Boolean renames Is_Subset;

    --  Alternative representation for a set of character values:
    subtype Wide_Character_Sequence is Wide_String;

    function To_Set (Sequence  : in Wide_Character_Sequence)
       return Wide_Character_Set;

    function To_Set (Singleton : in Wide_Character)
       return Wide_Character_Set;

    function To_Sequence (Set  : in Wide_Character_Set)
       return Wide_Character_Sequence;

    --  Representation for a wide character to wide character mapping:
    type Wide_Character_Mapping is private;

    function Value (Map     : in Wide_Character_Mapping;
                    Element : in Wide_Character)
       return Wide_Character;

    Identity : constant Wide_Character_Mapping;

    function To_Mapping (From, To : in Wide_Character_Sequence)
       return Wide_Character_Mapping;

    function To_Domain (Map  : in Wide_Character_Mapping)
       return Wide_Character_Sequence;

    function To_Range (Map  : in Wide_Character_Mapping)
       return Wide_Character_Sequence;

    type Wide_Character_Mapping_Function is
       access function (From : in Wide_Character) return Wide_Character;

private

    type Wide_Character_Position is mod 2**16;

    subtype Wide_Character_Set_Index is integer range
  	     Wide_Character'Pos(Wide_Character'First) ..
  	     Wide_Character'Pos(Wide_Character'Last);

    type Wide_Character_Set_Array is array (Wide_Character_Set_Index range <>) 
				     of Wide_Character_Position;
    pragma pack(Wide_Character_Set_Array);

    type Wide_Character_Set is access all Wide_Character_Set_Array;

    Null_Set : constant Wide_Character_Set := Null;

    type Mapping_Sequence_Access is access all Wide_Character_Sequence;

    type Wide_Character_Mapping is record
       From : Mapping_Sequence_Access;
       To   : Mapping_Sequence_Access;
    end record;

    Identity : constant Wide_Character_Mapping := ( Null, Null );

end Ada.Strings.Wide_Maps;
