-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.calendar.bdy,v $
-- $Revision: 1.8 $ $Date: 96/11/01 17:52:25 $ $Author: stt $
--        Copyright (C) 1995 by Intermetrics, Inc.   

with Ada.Calendar.TGT;

pragma ada_child;
package body Ada.Calendar is

    pragma Suppress( Elaboration_Check );

---------------------------------------------------------------------
-- Local types, variables, and routines
---------------------------------------------------------------------

    days_in_the_month : constant array (month_number, Boolean) of
      day_number :=
        ((31,31), (28,29), (31,31), (30,30), (31,31), (30,30),
         (31,31), (31,31), (30,30), (31,31), (30,30), (31,31));
           --| Days in month, given month num and is-leap-year

    days_since_year   : constant array (month_number, Boolean) of
      integer :=
       ((0,0), (31,31), (59,60), (90,91), (120,121), (151,152),
        (181,182), (212,213), (243,244), (273,274), (304,305), (334,335));
           --| Days since start of year, given month num and is-leap-year

    days_in_year      : constant array (Boolean) of integer :=
      (365, 366);
        --| Days in year, given is-leap-year

    secs_per_day      : constant duration := 86400.0;
    integer_secs_per_day : constant := 86400;

    days_in_epoch     : constant Day_Count := 72685.0;

    subtype Good_Day is Day_Count range 1.0 .. days_in_epoch;
       -- Range of legal normalized Day_Count

    One_Day           : constant Day_Count := 1.0;

    Zero_Duration     : constant duration := 0.0;

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Local routines specs
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

    function leap_year (the_year : year_number) return Boolean;

    function days_since_epoch (the_year : year_number) return Day_Count;
       -- the number of days from Jan 1, 1901 til the beginning
       -- of the_year.

    function normalize (the_time : time) return time;
       -- the_time is a result of some additition or subtraction
       -- where we just added the time components. Normalization
       -- turns it into something where the seconds_since_day
       -- will fit into something of size day_duration. If this
       -- is not possible, or if the_time overflows/underflows the bounds
       -- of the epoch, then we will raise time_error.

---------------------------------------------------------------------
-- Global routines
---------------------------------------------------------------------

    function Clock return time is
	result : Time;
    begin
	return TGT.GMT_Clock;
    end Clock;

    function Year (date : time) return year_number is
        The_Year  : Year_Number;
        The_Month : Month_Number;
        The_Day   : Day_Number;
        The_Seconds : Day_Duration;
    begin
    --* IMPLEMENTATION
        Split (Date, The_Year, The_Month, The_Day, The_Seconds);
        return (The_Year);
    end Year;

    function month (date : time) return month_number is
        the_year  : year_number;
        the_month : month_number;
        the_day   : day_number;
        the_seconds : day_duration;
    begin
    --* IMPLEMENTATION
        Split (date, the_year, the_month, the_day, the_seconds);
        return (the_month);
    end month;

    function day (date : time) return day_number is
        the_year  : year_number;
        the_month : month_number;
        the_day   : day_number;
        the_seconds : day_duration;
    begin
    --* IMPLEMENTATION
        Split (date, the_year, the_month, the_day, the_seconds);
        return (the_day);
    end day;

    function seconds (date: time) return day_duration is
        the_year  : year_number;
        the_month : month_number;
        the_day   : day_number;
        the_seconds : day_duration;
    begin
    --* IMPLEMENTATION
        Split (date, the_year, the_month, the_day, the_seconds);
        return (the_seconds);
    end seconds;

    procedure Split (date    : in time;
                     year    : out year_number;
                     month   : out month_number;
                     day     : out day_number;
                     seconds : out day_duration) is
	Local_Date : Time := date - TGT.Local_Time_Zone;
        day_num : integer;
        the_year   : year_number;
        the_month  : month_number;
        -- temps used to compute the year, month, and day.
        days_to_next_year : integer;
        days_to_next_month: integer;
        is_leap : Boolean;
    begin
--* IMPLEMENTATION
-- Calculate an underestimate of year_number using day_in_epoch
-- and assuming 366 days per year.
-- Calculate day_number based on year_number. (It may be > 366.)
-- loop
--     exit when day_number < days_to_next_year;
--     day_number :=  day_number - days_to_next_year;
--     increment year;
-- end loop;
-- Similarly, compute month_number and day_number in month based
-- on the day_number in year obtained above.
        -- Calculate approximate year (underestimate)
        the_year := 1901 + Integer(Local_Date.day_in_epoch/366);
 
        loop
            -- Calculate day assuming year is correct
            day_num := Integer(
              Local_Date.day_in_epoch - days_since_epoch(the_year)
            );
 
            exit when day_num > 0;
 
            -- Oops, machine must have rounded the fixed-pt division
            -- Try the previous year
            the_year := the_year - 1;
        end loop;
 
        -- find the correct year
        loop
            is_leap := leap_year(the_year);
            days_to_next_year := days_in_year (is_leap);
 
            exit when day_num <= days_to_next_year;
 
            day_num := day_num - days_to_next_year;
            the_year := the_year + 1;
        end loop;
 
        -- day_num = day in year.
        -- the year = the year number in question.
 
        -- get initial (under)estimate of month
        the_month := (day_num + 30) / 31;
 
        -- calculate day assuming given month
        day_num := day_num - days_since_year(the_month, is_leap);
 
        -- loop to calculate actual month
        loop
            days_to_next_month :=
              days_in_the_month (the_month, is_leap);
 
            exit when day_num <= days_to_next_month;
 
            day_num := day_num - days_to_next_month;
            the_month := the_month + 1;
        end loop;
 
        -- Initialize OUT parameters
        year    := the_year;
        month   := the_month;
        day     := day_num;
        seconds := Local_Date.seconds_since_day;
    end split;

    function time_of (year    : year_number;
                      month   : month_number;
                      day     : day_number;
                      seconds : day_duration := 0.0) return time is
           result : time;
           is_leap : Boolean := leap_year(year);
    begin
    --* IMPLEMENTATION
    -- Calculate days_since_epoch based on year, month, and day.
    -- seconds_since_day := seconds
	if day > days_in_the_month (month, is_leap) then
	    -- the time doesn't make any sense at all
	    raise TIME_ERROR;
	end if;
 
	-- Calculate day number in epoch
	result.day_in_epoch := days_since_epoch (year) +
	    Day_Count(days_since_year (month, is_leap) + day);
 
	-- Store seconds
	result.seconds_since_day := seconds;
 
	-- May raise TIME_ERROR
	return Normalize (result + TGT.Local_Time_Zone);
    end time_of;

    function "+" (left : duration; right:time) return time is
	result : time := right;
    begin
	begin -- exception block
	    result.seconds_since_day :=
	       left + right.seconds_since_day;
	exception
	    when constraint_error =>
		-- left + right.seconds_since_day overflowed
		-- to the positive side because right.seconds_since_day
		-- is in 0..duration'last as a result of normalization.
		-- Compute the result another way.
		result.day_in_epoch :=
		  result.day_in_epoch + One_Day;
		result.seconds_since_day :=
		  (left - secs_per_day) + right.seconds_since_day;
	end;  -- exception block
 
	return Normalize (result); -- normalize will raise time error
				   -- if necessary.
    exception
	when constraint_error =>
	    raise TIME_ERROR;
    end "+";

    function "+" (left : time; right : duration) return time is
    begin
	return right + left;
    end "+";

    function "-" (left : time; right: duration) return time is
    begin
	return left + (-right);
    end "-";

    function "-" (left : time; right : time) return duration is
	Result : Duration;
	Day_Diff : Integer;
    begin
 
	Day_Diff := Integer(left.day_in_epoch - right.day_in_epoch);
	Result := left.seconds_since_day - right.seconds_since_day;
 
	-- calculate total seconds, taking day diff into account
	-- Note that the loop is used instead of multiply
	--  so we don't get overflow from multiply when the addition
	--  would succeed, and because the day diff must be in
	--  -2 .. +2 to avoid time_error anyway.
 
	if Day_Diff < -1 or Day_Diff > 1 then
	    raise time_error;
	elsif Day_Diff = -1 then
	    Result := Result - secs_per_day;
	elsif Day_Diff = 1 then
	    Result := Result + secs_per_day;
	end if;
	return Result;
    exception
	when constraint_error =>
	    raise time_error;
    end "-";

    function "<" (left, right : time) return boolean is
    begin
 
	return Left.day_in_epoch < Right.day_in_epoch or else
	  (Left.day_in_epoch = Right.day_in_epoch and then
	    Left.seconds_since_day < Right.seconds_since_day);
    end "<";

    function "<=" (left, right : time) return boolean is
    begin
	return not (Right < Left);
    end "<=";

    function ">" (left, right : time) return boolean is
    begin
	return (right < left);
    end ">";

    function ">=" (left, right : time) return boolean is
    begin
	return not (Left < Right);
    end ">=";


--------------------------------------------------------------
-- Local Routine Bodies
--------------------------------------------------------------

    function leap_year (the_year : year_number) return Boolean is
        -- This algorithm is good for 1901-2099 (LRM), but not beyond.
    begin
	return ((the_year rem 4) = 0);
    end leap_year;
 

    function days_since_epoch (the_year : year_number) return Day_Count is
 
            no_years : integer := the_year - 1901;
    begin
	-- the number of days from Jan 1, 1901 til the beginning
	-- of the_year.
	return no_years * Day_Count'(365.0) + Day_Count(no_years/4);
    end days_since_epoch;


    function normalize (the_time : time) return time is
       -- the_time is a result of some additition or subtraction
       -- where we just added the time components. Normalization
       -- turns it into something where the seconds_since_day
       -- will fit into something of size day_duration. If this
       -- is not possible, or if the_time overflows/underflows the bounds
       -- of the epoch, then we will raise time_error.
           normalized_time : time := the_time;
    begin
--      Loop reducing seconds until < Day_Duration'Last
	while normalized_time.seconds_since_day >= secs_per_day loop
	    normalized_time.day_in_epoch :=
	      normalized_time.day_in_epoch + One_Day;
 
	    normalized_time.seconds_since_day :=
	      normalized_time.seconds_since_day - secs_per_day;
	end loop;
 
--      Loop increasing seconds until >= zero.
	while normalized_time.seconds_since_day < Zero_Duration loop
	    normalized_time.day_in_epoch :=
	      normalized_time.day_in_epoch - One_Day;
	    normalized_time.seconds_since_day :=
	      normalized_time.seconds_since_day + secs_per_day;
	end loop;
 
--      Day_In_Epoch must be in range, and seconds must be zero
--      if last "good" day.
	if normalized_time.day_in_epoch not in Good_Day or else
	  (normalized_time.day_in_epoch = days_in_epoch and then
	   normalized_time.seconds_since_day > Zero_Duration) then
 
	    raise TIME_ERROR;
 
	end if;
	return normalized_time;
    end normalize;

end Ada.Calendar;
