-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.calendar.spc,v $
-- $Revision: 1.5 $ $Date: 96/08/14 13:20:46 $ $Author: jparadis $
--        Copyright (C) 1995 by Intermetrics, Inc.   

--with System.RTS.Time_Pkg;

pragma ada_child;
package Ada.Calendar is

--* OVERVIEW
-- This package implements the predefined library package
-- CALENDAR.  The operators are not shown as their implementation
-- is obvious.

    pragma Suppress( Elaboration_Check );

    type Time is private;

    subtype Year_Number  is Integer  range 1901..2099;
    subtype Month_Number is Integer  range 1..12;
    subtype Day_Number   is Integer  range 1..31;
    subtype Day_Duration is Duration range 0.0..86_400.0;

    function Clock return Time;
    --* EFFECTS
    -- Returns the current value of TIME at the time it is called.
    -- Always returns GMT time, so is unaffected by local time zone changes.
    -- Split, Time_of, Year, Month, Day, and Seconds use GMT time
    -- values but local time zone values for the the separated
    -- component values.

    function Year    (Date : Time) return Year_Number;
    --* EFFECTS
    -- Returns the YEAR_NUMBER part of a given value of type TIME.

    function Month   (Date : Time) return Month_Number;
    --* EFFECTS
    -- Returns the MONTH_NUMBER part of a given value of type TIME.

    function Day     (Date : Time) return Day_Number;
    --* EFFECTS
    -- Returns the DAY_NUMBER part of a given value of type TIME.

    function Seconds (Date : Time) return Day_Duration;
    --* EFFECTS
    -- Returns the DAY_DURATION part of a given value of type TIME.

    procedure Split  (Date    : in Time;
                      Year    : out Year_Number;
                      Month   : out Month_Number;
		      Day     : out Day_Number;
		      Seconds : out Day_Duration);
    --* EFFECTS
    -- Returns the four parts (year_number, month_number, day_number,
    -- and day_duration) of a given value of type time.

    function Time_of (Year    : Year_Number;
                      Month   : Month_Number;
		      Day     : Day_Number;
		      Seconds : Day_Duration := 0.0) 
             return Time;
    --* EFFECTS
    -- Combines the four parts (year_number, month_number, day_number,
    -- and day_duration) into a value of type TIME.

    function "+" (Left : Time;     Right : Duration) return Time;
    --* EFFECTS
    -- Effect
    -- Adds a time and a duration, returning a value of type TIME.
    -- If, for the given operands, a date whose year number is not
    -- in the range of the corresponding subtype is formed, the
    -- exception TIME_ERROR is raised.

    function "+" (Left : Duration; Right : Time)     return Time;
    --* EFFECTS
    -- Adds a duration and a time, returning a value of type TIME.
    -- If, for the given operands, a date whose year number is not
    -- in the range of the corresponding subtype is formed, the
    -- exception TIME_ERROR is raised.

    function "-" (Left : Time;     Right : Duration) return Time;
    --* EFFECTS
    -- Subtracts a time and a duration, returning a value of type TIME.
    -- If, for the given operands, a date whose year number is not
    -- in the range of the corresponding subtype is formed, the
    -- exception TIME_ERROR is raised. Also, if the result is not in
    -- the range of type duration, TIME_ERROR is raised.

    function "-" (Left : Time;     Right : Time)     return Duration;
    --* EFFECTS
    -- Subtracts a duration and a time, returning a value of type TIME.
    -- If, for the given operands, a date whose year number is not
    -- in the range of the corresponding subtype is formed, the
    -- exception TIME_ERROR is raised. Also, if the result is not in
    -- the range of type duration, TIME_ERROR is raised.

    function "<"  (Left, Right : Time) return Boolean;
    --* EFFECTS
    -- Has the conventional meaning - returns TRUE if the time
    -- corresponding to LEFT precedes the time corresponding to
    -- RIGHT, and FALSE otherwise.

    function "<=" (Left, Right : Time) return Boolean;
    --* EFFECTS
    -- Has the conventional meaning - returns TRUE if the time
    -- corresponding to LEFT precedes or it the same as
    -- the time corresponding to RIGHT, and FALSE otherwise.

    function ">"  (Left, Right : Time) return Boolean;
    --* EFFECTS
    -- Has the conventional meaning - returns TRUE if the time
    -- corresponding to LEFT follows the time corresponding to
    -- RIGHT, and FALSE otherwise.

    function ">=" (Left, Right : Time) return Boolean;
    --* EFFECTS
    -- Has the conventional meaning - returns TRUE if the time
    -- corresponding to LEFT follows or is the same as
    -- the time corresponding to RIGHT, and FALSE otherwise.

    TIME_ERROR : exception;

private
    -- Use a fixed-point type with delta 1.0 for Day number,
    --  since Integer'last may not be large enough.

    Max_Day_Count : constant := 366.0 *
	 Integer'Pos(Year_Number'last - Year_Number'first + 1);
    -- Generous count for number of Days in epoch

    type Day_Count is delta 1.0 range -Max_Day_Count .. Max_Day_Count;
    for Day_Count'Small use 1.0;

    type Time is record
        day_in_epoch : Day_Count;
	-- Day number, where number 1 is January 1, 1901
	-- Outside of this package, is always >= 1.0

	seconds_since_day : Duration;
	-- Seconds since beginning of the Day.
	-- Outside of this package, is always >= 0.0
	-- and < Day_Duration'Last.
    end record;

end Ada.Calendar;

with Ada.Calendar;
package Calendar renames Ada.Calendar;
