-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.interrupts.spc,v $
-- $Revision: 1.4 $ $Date: 96/10/25 13:35:26 $ $Author: bethc $
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System;
with System.RTS.TGT.Interrupt_Types;
pragma ada_child;
package Ada.Interrupts is 

pragma Suppress( Elaboration_Check );

    type Interrupt_Id is new System.RTS.TGT.Interrupt_Types.Interrupt_Id;

    type Parameterless_Handler is access protected procedure;

    function Is_Reserved (Interrupt : Interrupt_Id)
       return Boolean;
    --* EFFECTS
    -- Returns True if Interrupt is reserved.

    function Is_Attached (Interrupt : Interrupt_Id)
       return Boolean;
    --* EFFECTS
    -- Returns True if a user-specified handler is attached to Interrupt
    -- wither statically or dynamically. Raises Interrupt_Error if
    -- Interrupt is Reserved.

    function Current_Handler (Interrupt : Interrupt_Id)
       return Parameterless_Handler;
    --* EFFECTS
    -- Returns the user-specified handler if attached. Returns
    -- Default_Handler if not reserved and not attached.
    -- If Interrupt is reserved, Interrupt_Error is returned.

    procedure Attach_Handler
       (New_Handler : Parameterless_Handler;
        Interrupt   : Interrupt_Id);
    --* EFFECTS
    -- Attaches the specified handler to an unreserved interrupt,
    -- thereby overriding the current handler (which may be the default).
    -- Interrupt_Error will be raised if Interrupt is reserved.
    -- When implemented for Ada9X the following effects will be added:
    -- NEW_Handler = null will restore the default treatment.  If pragma
    -- Interrupt_Handler is not present in the protected object
    -- containing New_Handler, then Program_Error will be raised.


    procedure Exchange_Handler
       (Old_Handler : out Parameterless_Handler;
        New_Handler : Parameterless_Handler;
        Interrupt   : Interrupt_Id);
    --* EFFECTS
    -- Similar to Attach_Handler except that the previous handler is
    -- returned.

    procedure Detach_Handler
       (Interrupt : Interrupt_Id);
    --* EFFECTS
    -- Restores default handler.  Raises Program_Error is Reserved.

    function Reference(Interrupt: Interrupt_Id)
       return System.Address;
    --* EFFECTS
    -- Raises Program_Error since interrupt entries are not supported.

end Ada.Interrupts;
