-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.streams.stream_io.spc,v $ 
-- $Revision: 1.4 $ $Date: 95/11/15 15:05:56 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   
with Ada.IO_Exceptions;
with System.RTS.TGT.Basic_IO;

pragma ada_child;
package Ada.Streams.Stream_IO is
    pragma Suppress( Elaboration_Check );

    type Stream_Access is access all Root_Stream_Type'Class;

    type File_Type is limited private;

    type File_Mode is (In_File, Out_File, Append_File);

    type    Count          is range 0 .. Integer'Last;
    subtype Positive_Count is Count range 1 .. Count'Last;
      -- Index into file, in  stream  elements.

    procedure Create (File : in out File_Type;
                      Mode : in File_Mode := Out_File;
                      Name : in String    := "";
                      Form : in String    := "");

    procedure Open (File : in out File_Type;
                    Mode : in File_Mode;
                    Name : in String;
                    Form : in String := "");

    procedure Close  (File : in out File_Type);
    procedure Delete (File : in out File_Type);
    procedure Reset  (File : in out File_Type; Mode : in File_Mode);
    procedure Reset  (File : in out File_Type);

    function Mode (File : in File_Type) return File_Mode;
    function Name (File : in File_Type) return String;
    function Form (File : in File_Type) return String;

    function Is_Open     (File : in File_Type) return Boolean;
    function End_Of_File (File : in File_Type) return Boolean;

    function Stream (File : in File_Type) return Stream_Access;
    -- Return stream access for use with T'Input and T'Output


    -- Read array of stream elements from file
    procedure Read (File : in  File_Type;
                    Item : out Stream_Element_Array;
                    Last : out Stream_Element_Offset;
                    From : in  Positive_Count);

    procedure Read (File : in  File_Type;
                    Item : out Stream_Element_Array;
                    Last : out Stream_Element_Offset);


    -- Write array of stream elements into file
    procedure Write (File : in File_Type;
                     Item : in Stream_Element_Array;
                     To   : in Positive_Count);

    procedure Write (File : in File_Type;
                     Item : in Stream_Element_Array);


    -- Operations on position within file

    procedure Set_Index(File : in File_Type; To : in Positive_Count);

    function Index(File : in File_Type) return Positive_Count;
    function Size (File : in File_Type) return Count;

    procedure Set_Mode(File : in out File_Type; Mode : in File_Mode);

    procedure Flush(File : in out File_Type);

    -- exceptions
    Status_Error : exception renames IO_Exceptions.Status_Error;
    Mode_Error   : exception renames IO_Exceptions.Mode_Error;
    Name_Error   : exception renames IO_Exceptions.Name_Error;
    Use_Error    : exception renames IO_Exceptions.Use_Error;
    Device_Error : exception renames IO_Exceptions.Device_Error;
    End_Error    : exception renames IO_Exceptions.End_Error;
    Data_Error   : exception renames IO_Exceptions.Data_Error;

private

    Buffer_Length : constant integer := 1;

    subtype Buffer_Count is integer range 0 .. Buffer_Length;

    type File_Record is new Root_Stream_Type with record
        Mode             : File_Mode;
        Number_In_Buffer : Buffer_Count := 0;
                           --| Number of valid elements in buffer
        Number_Returned  : Buffer_Count := 0;
                           --| Number already returned by read
        End_Of_File      : boolean := false;
        Host_File        : System.RTS.TGT.Basic_IO.File_Type;
    end record;

    type File_Type is access all File_Record;

    procedure Read (File : in out File_Record;
                    Item : out Stream_Element_Array;
                    Last : out Stream_Element_Offset);
    -- Read used when Stream_IO file is treated directly as Stream

    procedure Write (File : in out File_Record;
                     Item : in Stream_Element_Array);
    -- Write used when Stream_IO file is treated directly as Stream

end Ada.Streams.Stream_IO;
