-- $Source: /home/dsd/d/aa/monoBANK/AA/j_ada.text_io.bdy,v $
-- $Revision: 1.3 $ -- $Date: 96/10/09 18:14:47 $ -- $Author: stt $
--        Copyright (C) 1995 by Intermetrics, Inc.

-- with Ada.Unchecked_Deallocation;
with System.RTS.Lexer;          
    pragma Elaborate(System.RTS.Lexer);
with System.RTS.TGT.Char_IO; 
    pragma Elaborate(System.RTS.TGT.Char_IO);
with System.RTS.String_Utilities;   use System.RTS.String_Utilities;
with System.RTS.TGT.IO_Constants;
    pragma Elaborate(System.RTS.TGT.IO_Constants);
with System.RTS.Float_IO_Support;
with System.RTS.Integer_State_Tables;  -- int lex tables
with System.RTS.Integer_Image_And_Value_Pkg;
    pragma Elaborate(System.RTS.Integer_Image_And_Value_Pkg);
with System.RTS.TGT;
with Ada.Characters.Latin_1;
with System.RTS.Unsigned;

pragma ada_child;
package body Ada.Text_IO is

    pragma Suppress( Elaboration_Check );

    package CIO renames System.RTS.TGT.Char_IO;
    package FIO renames System.RTS.Float_IO_Support;
    package TGT renames System.RTS.TGT;
    package UNS renames System.RTS.Unsigned;

    NYI : exception;

    Record_Oriented: constant Boolean :=
      System.RTS.TGT.IO_Constants.Record_Oriented;

    type PEEK_PLACE is (UNKNOWN,           --| Used for look-ahead
                        AFTER_EOL,
                        AFTER_EOP,
                        NON_TERMINATOR,
                        EOL,
                        EOL_NOTP,
                        EOP,
                        EOP_NOTF,
                        EOF,
                        EOF_ONLY);

    type File_Record is record
        Mode           : File_Mode;        --| Mode of file
        Char_IO_File   : CIO.File_Type;    --| Underlying host file
        Line_Length    : Count;            --| Maximum line length allowed
        Page_Length    : Count;            --| Maximum page length allowed
        Peeked_Char    : character;        --| One char lookahead, if valid
        Where          : Peek_Place;
        Current_Column : Positive_Count;
        Current_Line   : Positive_Count;
        Current_Page   : Positive_Count;
        Is_Special     : boolean;          --| True if file is the current
                                           --| default input file or the
                                           --| current default output file
    end record;

type Mode_Array is array (Positive range <>) of File_Mode;

Line_Terminator : constant character := Ada.Characters.Latin_1.LF;
Vertical_Tab    : constant character := Ada.Characters.Latin_1.VT;
Page_Terminator : constant character := Ada.Characters.Latin_1.FF;

subtype Interesting_Characters is character range
  Line_Terminator .. Page_Terminator;

CIO_Mode : constant array (File_Mode) of CIO.File_Mode :=
           (In_File =>     CIO.In_File,
	    Out_File =>    CIO.Out_File,
            Append_File => CIO.Append_File);

Merge_When_Unknown : constant array (Interesting_Characters) of
  Peek_Place := (
   Line_Terminator => EOL,
   Vertical_Tab    => Non_Terminator,
   Page_Terminator => EOP);
   --| Used to "Merge_In_Peek" when File.Where is unknown
   --| (for performance reasons)


subtype EOL_Range is Peek_Place range EOL .. EOF_Only;
    --| Set of places for which "End_Of_Line" is true
subtype EOP_Range is Peek_Place range EOP .. EOF_Only;
    --| Set of places for which "End_Of_Page" is true
subtype EOF_Range is Peek_Place range EOF .. EOF_Only;
    --| Set of places for which "End_Of_File" is true

Standard_Input_File,
Standard_Output_File,
Standard_Error_File : aliased File_Type;
    --| Files associated with standard input and output

Current_Input_File,
Current_Output_File,
Current_Error_File : aliased File_Type;
    --| File associated with current default input and output

Current_Input_Initialized,
Current_Output_Initialized,
Current_Error_Initialized : boolean := false;
    --| False until first reference to current input or output, then
    --| the file is initialized and the boolean becomes true

Internal_Error : exception;    --| Raised if internal inconsistency
                               --| detected

procedure Free_File(File : in out File_Type) is
      -- is new Unchecked_Deallocation(File_Record, File_Type);
begin
    -- For Java, we just set the file pointer to null
    File := null;
end Free_File;

-- Local subprograms

procedure Init_File (  --| Initializes all fields of file to
  File : in File_Type; --| their default values (except Char_IO_File),
  Mode : in File_Mode  --| and sets mode of file to Mode
);

procedure Close_Special_File (  --| Closes current input/output
  File : in File_Type           --| if associated file closed.
);

procedure Peek (               --| Initializes File.Where, File.Peeked_Char
  File       : in File_Type;   --| File to "peek"
  Peek_Until : in Peek_Place;  --| How far to peek
  Char       : out character   --|
);

procedure Advance (    --| Skip char/terminator
  File : in File_Type  --| File to advance
);

procedure Merge_In_Peek (  --| Update "Where" based on File.Peeked_Char
  File : in File_Type      --| File to be updated
);

function Get_Standard_Input return File_Type;
    --| Return standard input file

function Get_Standard_Output return File_Type;
    --| Return standard output file

function Get_Standard_Error return File_Type;
    --| Return standard error file

procedure Local_Put (
   File : in File_Type;
   Item : in string
);

procedure Local_Put_Line (
  File : in File_Type;
  Item : in string
);

procedure Local_New_Line(
  File : in File_Type;
  Spacing : in Positive_Count
);

procedure Local_New_Page( File : in File_Type );

procedure Local_Get (
  File : in File_Type;
  Item : out string
);

procedure Local_Get_Line (
  File : in File_Type;
  Item : out string;
  Last : out Natural
);

-- Local subprograms

procedure Check_Mode (File : in File_Type;
                      Mode : in File_Mode) is
begin
    if File = null then
        raise STATUS_ERROR;
    elsif File.Mode /= Mode then
        raise MODE_ERROR;
    end if;
end Check_Mode;

procedure Check_Mode (File : in File_Type;
                      Mode : in Mode_Array) is
begin
    if File = null then
        raise STATUS_ERROR;
    end if;
    for i in Mode'range loop
        if File.Mode = Mode(i) then
            return;
        end if;
    end loop;
    raise Mode_Error;
end Check_Mode;

procedure Check_Current_Input is
begin
    -- Check to make sure current_input hasn't been closed
    if Current_Input_Initialized and then
        Current_Input_File = null then
            raise STATUS_ERROR;
    end if;
end Check_Current_Input;

-- pragma Inline(Check_Current_Input);

procedure Check_Current_Output is
begin
    if Current_Output_Initialized and then
        Current_Output_File = null then
          -- The current output file has been closed
          raise STATUS_ERROR;
    end if;
end Check_Current_Output;

-- pragma Inline(Check_Current_Output);

procedure Next_Page ( File : in File_Type ) is
begin
    File.Current_Page   := File.Current_Page + 1;
    File.Current_Line   := 1;
    File.Current_Column := 1;
end Next_Page;

-- pragma Inline(Next_Page);

procedure Next_Line ( File : in File_Type ) is
begin
    File.Current_Line := File.Current_Line + 1;
    File.Current_Column := 1;
end Next_Line;

-- pragma Inline(Next_Line);

-- Visible subprogram bodies

-- File Management

procedure Create (File : in out File_Type;
                  Mode : in File_Mode := Out_File;
                  Name : in string := "";
                  Form : in string := "") is
begin
    if File /= null then
        raise STATUS_ERROR;
    else
    -- Allocate new file, fill in fields as needed
        File := new File_Record;
        Init_File(File, Mode);
        CIO.Create(File.Char_IO_File, CIO_Mode(Mode), Name, Form);
    end if;
end Create;

procedure Open (File : in out File_Type;
                Mode : in File_Mode;
                Name : in string;
                Form : in string := "") is
begin
    if File /= null then
        raise STATUS_ERROR;
    else
        File := new File_Record;
        Init_File(File, Mode);
        CIO.Open(File.Char_IO_File, CIO_Mode(Mode), Name, Form);
    end if;
end Open;


procedure Close (File : in out File_Type) is
begin
    if File = null then
        raise STATUS_ERROR;
    else
        if File.Is_Special then
            Close_Special_File(File);
        end if;
        if File.Current_Column /= 1 and then
	   (File.Mode = Out_File or else File.Mode = Append_File) then
            Local_New_Line(File, 1);
        end if;
        CIO.Close(File.Char_IO_File);
        Free_File(File);
    end if;
end Close;


procedure Delete (File : in out File_Type) is
begin
    if File = null then
        raise STATUS_ERROR;
    else
        if File.Is_Special then
            Close_Special_File(File);
        end if;
        CIO.Delete(File.Char_IO_File);
        Free_File(File);
    end if;
end Delete;


procedure Reset (File : in out File_Type;
                 Mode : in File_Mode) is
begin
    if File = null then
        raise STATUS_ERROR;
    elsif File.Is_Special and then File.Mode /= Mode then
        -- File is either the current default input file or the
        -- current default output file, and it's mode cannot be
        -- changed.
        raise MODE_ERROR;
    else
        if File.Current_Column /= 1 and then
	   (File.Mode = Out_File or else File.Mode = Append_File) then
            -- Last line in file unterminated, so terminate it
            Local_New_Line(File, 1);
        end if;
        CIO.Reset(File.Char_IO_File, CIO_Mode(Mode));
        Init_File(File, Mode);
    end if;
end Reset;

procedure Reset (File : in out File_Type) is
begin
    if File /= null then
        Reset (File, File.Mode);
    else
        raise STATUS_ERROR;
    end if;
end Reset;

function Mode (File : in File_Type) return File_Mode is
begin
    if File = null then
        raise STATUS_ERROR;
    else
        return File.Mode;
    end if;
end Mode;

function Name (File : in File_Type) return string is
begin
    if File = null then
        raise STATUS_ERROR;
    else
        return CIO.Name(File.Char_IO_File);
    end if;
end Name;

function Form (File : in File_Type) return string is
begin
    if File = null then
        raise STATUS_ERROR;
    else
        return CIO.Form(File.Char_IO_File);
    end if;
end Form;


function Is_Open (File : in File_Type) return boolean is
begin
    return File /= null;
end Is_Open;


-- Default Input and Output files

procedure Set_Input (File : in File_Type) is
begin
    Check_Mode(File, In_File);
    if Current_Input_File /= null then
        -- Old file no longer associated with current input
        Current_Input_File.Is_Special := false;
    end if;
    Current_Input_File := File;
    Current_Input_File.Is_Special := true;
    Current_Input_Initialized := true;
end Set_Input;

procedure Set_Output (File : in File_Type) is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    if Current_Output_File /= null then
        -- Old file no longer associated with current output
        Current_Output_File.Is_Special := false;
    end if;
    Current_Output_File := File;
    Current_Output_File.Is_Special := true;
    Current_Output_Initialized := true;
end Set_Output;

procedure Set_Error( File : in File_Type ) is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    if Current_Error_File /= null then
        Current_Error_File.Is_Special := false;
    end if;
    Current_Error_File := File;
    Current_Error_File.Is_Special := true;
    Current_Error_Initialized := true;
end Set_ERROR;

function Standard_Input return File_Type is
begin
    return Standard_Input.all;
end Standard_Input;

function Standard_Input return File_Access is
begin
    if Standard_Input_File = null then
        Standard_Input_File := Get_Standard_Input;
    end if;
    return Standard_Input_File'Access;
end Standard_Input;

function Standard_Output return File_Type is
begin
    return Standard_Output.all;
end Standard_Output;

function Standard_Output return File_Access is
begin
    if Standard_Output_File = null then
        Standard_Output_File := Get_Standard_Output;
    end if;
    return Standard_Output_File'Access;
end Standard_Output;

function Standard_Error return File_Type is
begin
    return Standard_Error.all;
end Standard_Error;

function Standard_Error return File_Access is
begin
    if Standard_Error_File = null then
        Standard_Error_File := Get_Standard_Error;
    end if;
    return Standard_Error_File'Access;
end Standard_Error;

function Current_Input return File_Type is
begin
    return Current_Input.all;
end Current_Input;

function Current_Input return File_Access is
begin
    if not Current_Input_Initialized then
        Current_Input_File := Standard_Input;
        Current_Input_File.Is_Special := true;
        Current_Input_Initialized := true;
    end if;
    return Current_Input_File'Access;
end Current_Input;

function Current_Output return File_Type is
begin
    return Current_Output.all;
end Current_Output;

function Current_Output return File_Access is
begin
    if not Current_Output_Initialized then
        Current_Output_File := Standard_Output;
        Current_Output_File.Is_Special := true;
        Current_Output_Initialized := true;
    end if;
    return Current_Output_File'Access;
end Current_Output;

function Current_Error return File_Type is
begin
    return Current_Error.all;
end Current_Error;

function Current_Error return File_Access is
begin
    if not Current_Error_Initialized then
        Current_Error_File := Standard_Error;
        Current_Error_File.Is_Special := true;
        Current_Error_Initialized := true;
    end if;
    return Current_Error_File'Access;
end Current_Error;

procedure Flush (File : in out File_Type) is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    CIO.Flush(File.Char_IO_File);
end FLUSH;

procedure Flush is
    FT : File_Type;
begin
    Check_Current_Output;
    FT := Current_Output;
    Flush(FT);
end FLUSH;

-- Specification of Line and Page Lengths
procedure Set_Line_Length (File : in File_Type;
                           To   : in Count) is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    File.Line_Length := To;
end Set_Line_Length;

procedure Set_Line_Length (To : in Count) is
begin
    Check_Current_Output;
    Set_Line_Length(Current_Output, To);
end Set_Line_Length;

procedure Set_Page_Length (File : in File_Type;
                           To   : in Count) is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    File.Page_Length := To;
end Set_Page_Length;


procedure Set_Page_Length (To : in Count) is
begin
    Check_Current_Output;
    Set_Page_Length(Current_Output, To);
end Set_Page_Length;

function Line_Length (File : in File_Type) return Count is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    return File.Line_Length;
end Line_Length;

function Line_Length return Count is
begin
    Check_Current_Output;
    return Line_Length(Current_Output);
end Line_Length;

function Page_Length (File : in File_Type) return Count is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    return File.Page_Length;
end Page_Length;

function Page_Length return Count is
begin
    Check_Current_Output;
    return Page_Length(Current_Output);
end Page_Length;

-- Operations on Columns, Lines, and Pages

procedure New_Line (File : in File_Type;
                    Spacing : in Positive_Count := 1) is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    Local_New_Line (File, Spacing);
end New_Line;

procedure New_Line (Spacing :  in Positive_Count := 1) is
begin
    Check_Current_Output;
    Local_New_Line(Current_Output, Spacing);
end New_Line;

procedure Skip_Line (File : in File_Type;
                     Spacing : in Positive_Count := 1) is
    Char : character;
begin
    Check_Mode(File, In_File);
    for i in 1 .. Spacing loop
        loop
            -- Find out where we are
            Peek(File, EOL, Char);
            -- Loop until a terminator is found
            exit when File.Where in EOL_Range;
            Advance(File);
        end loop;
        Advance(File);  -- Raises END_ERROR at EOF
    end loop;
end Skip_Line;

procedure Skip_Line (Spacing : in Positive_Count := 1) is
begin
    Check_Current_Input;
    Skip_Line(Current_Input, Spacing);
end Skip_Line;

function End_of_Line (File : in File_Type) return boolean is
    Char : character;
begin
    Check_Mode(File, In_File);
    Peek(File, EOL, Char);
    return File.Where in EOL_Range;
end End_Of_Line;


function End_Of_Line return boolean is
begin
    Check_Current_Input;
    return End_Of_Line(Current_Input);
end End_Of_Line;

procedure New_Page (File : in File_Type) is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    Local_New_Page(File);
end New_Page;

procedure New_Page is
begin
    Check_Current_Output;
    Local_New_Page(Current_Output);
end New_Page;

procedure Skip_Page (File : in File_Type) is
    Char : character;
begin
    Check_Mode(File, In_File);
    loop
        -- Find out where we are
        Peek(File, EOP, Char);
        -- Loop until we find end-of-page
        exit when File.Where in EOP_Range;
        Advance(File);
    end loop;
    Advance(File);  -- Gobble page terminator
end Skip_Page;

procedure Skip_Page is
begin
    Check_Current_Input;
    Skip_Page(Current_Input);
end Skip_Page;

function End_Of_Page (File : in File_Type) return boolean is
    Char : character;
begin
    Check_Mode(File, In_File);
    Peek(File, EOP, Char);
    return File.Where in EOP_Range;
end End_Of_Page;

function End_Of_Page return boolean is
begin
    Check_Current_Input;
    return End_Of_Page(Current_Input);
end End_Of_Page;

function End_Of_File (File : in File_Type) return boolean is
    Char : character;
begin
    Check_Mode(File, In_File);
    Peek(File, EOF, Char);
    return File.Where in EOF_Range;
end End_Of_File;

function End_Of_File return boolean is
begin
    Check_Current_Input;
    return End_Of_File(Current_Input);
end End_Of_File;

procedure Set_Col (File : in File_Type;
                   To   : in Positive_Count) is
    Char : character;
begin
    if File = null then
        raise STATUS_ERROR;
    else
        case File.Mode is
            when In_File =>
                loop
                    Peek(File, Non_Terminator, Char);
                    exit when File.Current_Column = To and then
                      File.Where = Non_Terminator;
                    Advance(File);
                end loop;
            when Out_File | Append_File =>
                if File.Line_Length /= Unbounded and then
                    To > File.Line_Length then
                      raise LAYOUT_ERROR;
                end if;
                if To < File.Current_Column then
                    Local_New_Line(File, 1);
                end if;
                for i in File.Current_Column .. To - 1 loop
                    Put(File, Ada.Characters.Latin_1.Space);
                end loop;
            when others =>
		null;
        end case;
    end if;
end Set_Col;

procedure Set_Col(To : in Positive_Count) is
begin
    Check_Current_Output;
    Set_Col(Current_Output, To);
end Set_Col;

procedure Set_Line(File : in File_Type;
                   To   : in Positive_Count) is
    Char : character;
begin
    if File = null then
        raise STATUS_ERROR;
    else
        case File.Mode is
            when In_File =>
                loop
                    if File.Current_Line = To then
                        Peek(File, EOL, Char);
                        exit when File.Current_Line = To;
                    end if;
                    Skip_Line(File);
                end loop;
            when Out_File | Append_File =>
                if File.Page_Length /= Unbounded and then
                    To > File.Page_Length then
                      raise LAYOUT_ERROR;
                end if;
                if To < File.Current_Line then
                    Local_New_Page(File);
                end if;
                if To > File.Current_Line then
                    Local_New_Line(File, To - File.Current_Line);
                end if;
            when others =>
		null;
        end case;
    end if;
end Set_Line;

procedure Set_Line(To : in Positive_Count) is
begin
    Check_Current_Output;
    Set_Line(Current_Output, To);
end Set_Line;

function Col (File : in File_Type) return Positive_Count is
begin
    if File = null then
        raise STATUS_ERROR;
    else
        return File.Current_Column;
    end if;
end Col;

function Col return Positive_Count is
begin
    Check_Current_Output;
    return Col(Current_Output);
end Col;

function Line(File : in File_Type) return Positive_Count is
    Char : character;
begin
    if File = null then
        raise STATUS_ERROR;
    elsif File.Where = After_EOL then
        -- Line may be wrong, find out what it should really be
        Peek(File, EOL, Char);
    end if;
    return File.Current_Line;
end Line;

function Line return Positive_Count is
begin
    Check_Current_Output;
    return Line(Current_Output);
end Line;

function Page (File : in File_Type) return Positive_Count is
    Char : character;
begin
    if File = null then
        raise STATUS_ERROR;
    elsif File.Where = After_EOL then
        -- Page may be wrong, find out what it should be
        Peek(File, EOL, Char);
    end if;
    return File.Current_Page;
end Page;

function Page return Positive_Count is
begin
    Check_Current_Output;
    return Page(Current_Output);
end Page;

procedure Get (File : in File_Type;
               Item : out character) is
begin
    Check_Mode(File, In_File);
    Peek(File, Non_Terminator, Item);
    Advance(File);
end Get;

procedure Get (Item : out character) is
begin
    Check_Current_Input;
    Get(Current_Input, Item);
end Get;

procedure Put (File : in File_Type;
               Item : in character) is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    -- Check for output of terminators
    case Item is
        when Line_Terminator =>
            Local_New_Line(File, 1);
        when Page_Terminator =>
            Local_New_Page(File);
        when others =>
            -- Not a terminator, just do the usual
            if File.Line_Length /= Unbounded and then
                File.Current_Column > File.Line_Length then
                  Local_New_Line(File, 1);
            end if;
            CIO.Write(File.Char_IO_File, Item);
            File.Current_Column := File.Current_Column + 1;
    end case;
end Put;

procedure Put (Item : in character) is
begin
    Check_Current_Output;
    Put(Current_Output, Item);
end Put;

procedure Look_Ahead( File : in File_Type;
    	    	      Item : out Character;
		      End_Of_Line : out Boolean ) is
begin
    Check_Mode(File, In_File);
    if File.Where <= Non_Terminator then
        Peek(File, EOL, Item);
	End_Of_Line := (File.Where >= EOL);
    else
	Item := Ada.Characters.Latin_1.NUL;
	End_Of_Line := true;
    end if;
end Look_Ahead;

procedure Look_Ahead( Item : Out Character;
		      End_Of_Line : out Boolean ) is
begin
    Check_Current_Input;
    Look_Ahead(Current_Input, Item, End_Of_Line);
end Look_Ahead;

procedure Get_Immediate( File : in File_Type;
    	    	         Item : out Character )  is
    Available : boolean;
begin
    Check_Mode(File, In_File);
    CIO.Get_Immediate(File      => File.Char_IO_File,
		      Wait      => false, 
		      Item      => Item,
		      Available => Available);
end Get_Immediate;

procedure Get_Immediate( Item : out Character )   is
    Available : boolean;
begin
    Check_Current_Input;
    CIO.Get_Immediate(File      => Current_Input.Char_IO_File,
		      Wait      => false, 
		      Item      => Item,
		      Available => Available);
end Get_Immediate;

procedure Get_Immediate( File : In File_Type;
			 Item : out Character;
			 Available : out Boolean )   is
begin
    Check_Mode(File, In_File);
    CIO.Get_Immediate(File      => File.Char_IO_File,
		      Wait      => false, 
		      Item      => Item,
		      Available => Available);
end Get_Immediate;

procedure Get_Immediate( Item : out Character;
			 Available : out Boolean )   is
begin
    Check_Current_Input;
    CIO.Get_Immediate(File      => Current_Input.Char_IO_File,
		      Wait      => false, 
		      Item      => Item,
		      Available => Available);
end Get_Immediate;

procedure Get (File : in File_Type;
               Item : out string) is
begin
    Check_Mode(File, In_File);
    Local_Get(File, Item);
end Get;

procedure Get (Item : out string) is
begin
    Check_Current_Input;
    Local_Get(Current_Input, Item);
end Get;

procedure Put (File : in File_Type;
               Item : in string) is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    Local_Put(File, Item);
end Put;

procedure Put(Item : in string) is
begin
    Check_Current_Output;
    Local_Put(Current_Output, Item);
end Put;

procedure Get_Line (File : in File_Type;
                    Item : out string;
                    Last : out natural) is
begin
    Check_Mode(File, In_File);
    Local_Get_Line(File, Item, Last);
end Get_Line;

procedure Get_Line(Item : out string;
                   Last : out Natural) is
begin
    Check_Current_Input;
    Local_Get_Line(Current_Input, Item, Last);
end Get_Line;

procedure Put_Line(File : in File_Type;
                   Item : in string) is
begin
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    Local_Put_Line(File, Item);
end Put_Line;

procedure Put_Line(Item : in string) is
begin
    Check_Current_Output;
    Local_Put_Line(Current_Output, Item);
end Put_Line;

--| Put out word, padded to WIDTH
--| Evoke NEW_LINE if would reach line length
--| Raise LAYOUT_ERROR if exceeds line length
--| Raise MODE_ERROR if mode(file) /= out_file
--| Raise STATUS_ERROR if file not open


-------------------------------------------------------------
--    LOCAL SUBPROGRAMS USED BY SEVERAL BODIES
-------------------------------------------------------------

procedure RTS_LEX_FILE(FILE : FILE_TYPE;
            STATE_TABLE: System.RTS.Lexer.STATE_TRANSITIONS;
            CHAR_TO_TOKEN_TABLE: System.RTS.Lexer.CHAR_TO_TOKEN_MAPPING;
            WIDTH: FIELD;
            RESULT: out STRING; -- resulting lexical unit withOUT
                -- leading or trailing blanks from file
                -- value in result(result'first .. LAST)
            LAST  : out NATURAL  ------------------/\
            );
--+-----------------------------------------------------
--| Effect     RTS_LEX_FILE
--|   Lexes the next token in FILE according to STATE_TABLE and
--|   CHAR_TO_TOKEN_TABLE.  If the token is Accept according to the
--|   tables, then then token without leading or trailing blanks
--|   is returned in RESULT(RESULT'first .. LAST).  Otherwise,
--|   DATA_ERROR or END_ERROR are raised.
--|
--|   The effect is as specified for GET(file) procedures in enum_io,
--|   integer_io, fixed_io, and float_io: as to effect of WIDTH, when
--|   DATA_ERROR and END_ERROR are raised, and which character will be
--|   the next one read from the file.
--|   RESULT is modified even if an exception is raised (though LAST isn't
--|   due to semantics of OUT Scalar Parms.)

procedure RTS_LEX_FILE (FILE : FILE_TYPE;
                    STATE_TABLE: System.RTS.Lexer.STATE_TRANSITIONS;
                    CHAR_TO_TOKEN_TABLE: System.RTS.Lexer.CHAR_TO_TOKEN_MAPPING;
                    WIDTH: FIELD;
                    RESULT: out STRING; -- resulting lexical unit withOUT
                        -- trailing blanks from file
                        -- value in result(result'first .. LAST)
                    LAST  : out NATURAL  ------------------/\
                    )
is
    --+-----------------------------------------------------
    --| Overview
    --|   Lexes the next token in FILE according to STATE_TABLE and
    --|   CHAR_TO_TOKEN_TABLE.  If the token is Accept according to the
    --|   tables, then the token without leading or trailing blanks
    --|   is returned in RESULT(RESULT'first .. LAST).  Otherwise,
    --|   DATA_ERROR or END_ERROR are raised.
    --|
    --| Effect
    --|   The effect is as specified for GET(file) procedures in enum_io,
    --|   integer_io, fixed_io, and float_io: as to effect of WIDTH, when
    --|   DATA_ERROR and END_ERROR are raised, and which character will be
    --|   the next one read from the file.
    --|
    --| Algorithm
    --|   1. SKIP_BLANKS is called to skip spaces and tabs (and when width=0,
    --|      line and page terminators), such that a call to NEXT will return
    --|      the first non-blank character.
    --|   2. LEXER is called to "lex" the desired token, using NEXT.
    --|   3. NEXT returns a single "lookahead" character to LEXER each time it
    --|      is called, and also "consumes" the previous lookahead character
    --|      removing it from the file and storing it into RESULT.
    --|   4. If WIDTH/=0, fewer than WIDTH chars have been read, and the
    --|      file is not at end-of-line, then (a) the syntax is bad, and
    --|      (b) we need to read those remaining characters, or up to EOL.
    --|   5. DATA_ERROR is raised if the final state isn't OK; else LAST
    --|      is given its appropriate value.
    --|
    --| Requires
    --|   RESULT is a non-null string, big enough to hold the lexical item
    --|   when stripped of leading/trailing blanks. The FSA specified by
    --|   TABLEs are such that all accept transitions are made whenever a
    --|   charcater which can't be a legal part of the lexical element is
    --|   read. (NEXT returns a space when it reaches EOL or reads WIDTH
    --|   characters.)

    use System.RTS.Lexer;
--  dbg : constant boolean := false;
      -- add "constant" to entirely eliminate all debugging code.

    Num_Chars_Read : Natural := 0;
    Last_Index : Natural := RESULT'first - 1;  -- Index into result string
    NEXT_CHAR : character;  -- "OWN" variable of function Next
    result_state : state;   -- result of LEXER

    Should_Advance_File : boolean := FALSE;
      -- Controls whether NEXT should actually "ADVANCE" the file, thereby
      -- "READING" the previously examined character.  Should_Advance is
      -- false under these conditions
      --      1. First time NEXT is called.  This is because SKIP_BLANKS
      --         has already examined the first non-blank, and it should
      --         not be "READ" until the Lexer has examined it.
      --      2. If EOL has been read, then the file should NOT advance.
      --         The Lexer probably won't call NEXT after hitting EOL,
      --         but if WIDTH /= 0 and the lexer finishes without reading
      --         WIDTH number of characters, then we need to perform
      --         that many calls to NEXT, and then raise DATA_ERROR
      --      3. WIDTH /= 0 and WIDTH number of characters have been read.

    function NEXT return character;

    function MY_LEXER is new LEXER (next => NEXT);

    procedure SKIP_BLANKS
    --+-----------------------------------------------------
    --| Overview
    --| Skips leading blanks (= ' ' or horizontal_tab), and if
    --|    width = 0, also line and page terminators.
    --| On completion, a call to NEXT will return the first non-blank
    --|
    --| Exceptions:
    --|   END_ERROR for width = 0 and only blanks and terminators remain
    --|     in file.
    --|   END_ERROR for width /= 0 only if positioned at end-of-file
    --|
    --|   DATA_ERROR for width /= 0 and all blanks WIDTH position, or up to
    --|        first EOL.
    is

        CHAR : Character;
        Where : PEEK_PLACE;
    begin

           -- Look for the first non blank-like character.
        if Width /= 0 then
            -- Stop at a terminator
            Where := EOL;
        else
            -- Don't stop until non-terminator
            Where := NON_TERMINATOR;
        end if;
        loop
            -- Gobble leading spaces/terminators

            PEEK(FILE, Where, CHAR);
            if FILE.WHERE in EOL_Range then
                raise DATA_ERROR;  -- Only possible if Width /= 0;
            end if;

            exit when CHAR /= Ada.Characters.Latin_1.Space and then
		      CHAR /= Ada.Characters.Latin_1.HT ;

--          if dbg then --dbg_on
--            text_io.put_line("skip_blanks: just skipped a blank");
--          end if;     --dbg_off
            ADVANCE(FILE);
            Num_Chars_Read := Num_Chars_Read + 1;

            if Num_Chars_Read = Width then
                raise DATA_ERROR;
            end if;

        end loop;
    end SKIP_BLANKS;

    function Next return character is
    --+-----------------------------------------------------
    --| Overview
    --|  "CONSUMES" the previous lookahead character and returns
    --|  the next lookahead character, incrementing Num_Chars_Read
    --|  each time a character is consumed and putting it into the
    --|  next position of RESULT
    --|
    --| Requires
    --|   SKIP_BLANKS is called before Next is called the first time.
    --|
    --|  Modifies
    --|    Num_Chars_Read, RESULT, and Last_Index
    --|    NEXT_CHAR is used as a temporary, but MUST BE an OWN variable (i.e.,
    --|      global), because its value is set on one call, and stored away in
    --|     RESULT on the next call.
    --|    Should_Advance_File

    begin
        if Should_Advance_File then
            --Gobble previous character and save it away in result
            Num_Chars_Read := Num_Chars_Read + 1;
            Last_Index := Last_Index + 1;
            RESULT (Last_Index) := NEXT_CHAR;
            ADVANCE (FILE); -- gobble up the character
        else
              -- Don't advance file this time, but DO advance it next
              -- time (unless should-advance is set false again when
              -- at EOL or width /= 0 and have already read WIDTH chars
            Should_Advance_File := TRUE;
        end if;

        if Width /= 0 and then Num_Chars_Read >= Width then
            -- End of width: Don't read any more, but return space
            -- so Lexer can ACCEPT, if string is OK
            Should_Advance_File := FALSE;
            NEXT_CHAR := Ada.Characters.Latin_1.Space;

        else
            PEEK(FILE, EOL, NEXT_CHAR);  -- Look at next character
            if FILE.WHERE in EOL_Range then
                -- Line/page/file terminator, return a space, and don't advanc
                Should_Advance_File := FALSE;
                NEXT_CHAR := Ada.Characters.Latin_1.Space;
            end if;
        end if;  -- width /= 0
--      if dbg then --dbg_on
--        text_io.put_line("NEXT: returning :" & next_char & ':' );
--      end if;     --dbg_off
        return (NEXT_CHAR);
    end Next;

begin
        if Mode(File)     -- will raise STATUS_ERROR if FILE not open
           /= In_File then
            raise MODE_ERROR;
        end if;

        SKIP_BLANKS;

        Result_State := MY_LEXER (state_table, char_to_token_table);

--      if dbg then --dbg_on
--        text_io.new_line;
--        text_io.put_line("lex_file: after calling lexer. width, num_char," &
--          "last_index, result_state");
--        text_io.put(integer'image(width));
--        text_io.put(integer'image(num_chars_read));
--        text_io.put(integer'image(last_index));
--        text_io.put_line(state'image(result_state));
--      end if;     --dbg_off

          -- For the case of width /= 0, check that WIDTH chars were
          -- read (or that EOL) was hit.  If not then READ remaining
          -- width chars up to EOL, and then raise DATA_ERROR
          -- because it is required that the ENTIRE width number of
          -- characters satisfy the syntax table: if some chars are left
          -- then there must be trailing blanks and other characters

        if width > Num_Chars_Read and then FILE.WHERE not in EOL_range
        then
            for i in Num_Chars_Read + 1 .. WIDTH loop
                ADVANCE (FILE);
                PEEK (FILE, EOL, NEXT_CHAR);
                exit when FILE.WHERE in EOL_range;  -- hit end of line
            end loop;
            raise DATA_ERROR;
        end if;

        -- Now check on final state of LEXER
        if Result_State = Accepted then
            LAST := Last_Index;
        else
            raise DATA_ERROR;
        end if;

end RTS_LEX_FILE;


procedure PUT_WORD( FILE  : in FILE_TYPE;
               	    WORD  : in STRING;
               	    WIDTH : in FIELD    ) is
    LENGTH : constant Integer := WORD'Length;
begin
    --| Algorithm
    --| If line is bounded, and output would go
    --| past end-of-line, then break line here.
    --| In any case, output the word and the padding spaces (if any).

    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));

    if FILE.Line_Length /= UNBOUNDED then
    	-- Line is bounded,Check for LAYOUT_ERROR
    	if WIDTH > INTEGER(FILE.Line_Length) or else
      	    LENGTH > INTEGER(FILE.Line_Length) then
            raise LAYOUT_ERROR;
    	end if;
   	 -- Calculate room left
    	declare
            Room_Left : INTEGER := INTEGER(
            	    	    FILE.Line_Length - FILE.Current_Column + 1);
    	begin
            if WIDTH > Room_Left or else LENGTH > Room_Left then
            	-- Output is too fat, break the line here
            	Local_New_Line(FILE, 1);
            end if;
    	end; -- declare
    end if;

    -- put out the word
    PUT(FILE, WORD);

    -- pad with spaces
    for I in LENGTH+1 .. WIDTH loop
    	PUT(FILE, Ada.Characters.Latin_1.Space);
    end loop;
end PUT_WORD;


-------------------------------------------------------------
--        INTEGER_IO_SUPPORT     SPEC
-------------------------------------------------------------

package INTEGER_IO_SUPPORT is

--| Overview
--|   INTEGER_IO_SUPPORT is the same as an instantiation of
--| INTEGER_IO for type INTEGER, except GET/PUT using default
--| files are missing.

-------------------------------------------------------------
    procedure GET(FILE  : in FILE_TYPE;
                  ITEM  : out TGT.LONGEST_INTEGER;
                  WIDTH : in FIELD);
-------------------------------------------------------------
    procedure PUT(FILE : in FILE_TYPE;
                  ITEM : in TGT.LONGEST_INTEGER;
                  WIDTH : in FIELD;
                  BASE  : in NUMBER_BASE);
-------------------------------------------------------------

    procedure GET(FROM : in STRING;
                  ITEM : out TGT.LONGEST_INTEGER;
                  LAST : out POSITIVE);
-------------------------------------------------------------

    procedure PUT(TO   : out STRING;
                  ITEM : in TGT.LONGEST_INTEGER;
                  BASE  : in NUMBER_BASE);

end INTEGER_IO_SUPPORT;


-------------------------------------------------------------
--        INTEGER_IO_SUPPORT     BODY
-------------------------------------------------------------


package body INTEGER_IO_SUPPORT is

-------------------------------------------------------------
--           LOCAL DECLARATIONS
-------------------------------------------------------------


function INTEGER_IMAGE ( --| returns integer image
                         ITEM : TGT.LONGEST_INTEGER;
                         WIDTH : INTEGER;
                         BASE  : INTEGER)
    return STRING;

int_to_char : constant string (1 .. 16) := "0123456789ABCDEF";
  -- int_to_char(i+1) is the hex_char for i
  -- used in INTEGER_IMAGE

--

-------------------------------------------------------------
    procedure GET(FILE  : in FILE_TYPE;
                  ITEM  : out TGT.LONGEST_INTEGER;
                  WIDTH : in FIELD)
    --| Effect
    --|   Lexes the next token from FILE using the syntax of
    --| integer literals, and converts it to an integer,
    --| putting the value into ITEM.  If WIDTH /= 0, then
    --| exactly that number of characters (or up to end-
    --| of-line) are used for the token; otherwise only
    --| those characters necessary to make up an integer
    --| are read.
    --|
    --| Algorithm
    --|   Use RTS_LEX_FILE to LEX the next integer token
    --| via Integer transition tables into BUFFER(1..last).
    --|   Generate the integer value via INTEGER'VALUE,
    --| converting any constraint error into DATA_ERROR.

    is
        buffer : string (1 .. 256); -- big enough for non-blank part of
                                    -- any integer
        last : natural;
    begin
        RTS_LEX_FILE (FILE,
                      System.RTS.Integer_State_Tables.Transitions,
                      System.RTS.Integer_State_Tables.Mapping,
                      WIDTH,
                      buffer,
                      last);
        --  Protect against constraint_error when converting to int, and
        -- change it to DATA_ERROR
        begin
            -- Since ITEM already lexed and legal, use a direct conversion
            -- with no check. ('value does check)    Offer
            ITEM := TGT.Longest_Integer(
		      System.RTS.Integer_Image_And_Value_Pkg.RTS_Integer_Value(
                        BUFFER(BUFFER'first .. last)));
        exception
            when CONSTRAINT_ERROR =>
                raise DATA_ERROR;
        end;
    end GET;


--|
-------------------------------------------------------------
    procedure PUT(FILE : in FILE_TYPE;
                  ITEM : in TGT.LONGEST_INTEGER;
                  WIDTH : in FIELD;
                  BASE  : in NUMBER_BASE)
    is
    --+-----------------------------------------------------
    --| Overview
    --|   Uses INTEGER_IMAGE to generate a string containing the
    --| necessary WIDTH and BASE.  PUT_WORD then outputs
    --| the string to the file, performing NEW_LINE, if necessary

    begin

        PUT_WORD (file => FILE,
                  word => INTEGER_IMAGE (item, width, base),
                  width => 0 ); -- no padding needed

    end PUT;  -- file

    --|
    -------------------------------------------------------------

    procedure GET(FROM : in STRING;
                  ITEM : out TGT.LONGEST_INTEGER;
                  LAST : out POSITIVE)
    --| Effect
    --|   Lexes the first token from FROM using the syntax of integer
    --|   literals, skips leading spaces, then converts it to an integer,
    --|   putting the value into ITEM. LAST is set to the index of the
    --|   last character used.
    --|
    --| Algorithm
    --|   Use RTS_LEX_STRING to LEX the next integer token via
    --|   Integer-Transition tables, setting My_Last. Skipping leading
    --|   spaces and then converting to integer using RTS_Integer_Value.
    --|   Convert any constraint error into DATA_ERROR.
    is
        My_Last   : natural;  -- Need local so the return value from
                              -- lex_string can be used in slice.
        My_First  : natural := From'First;  -- The first non-space character.
    begin
        System.RTS.Lexer.RTS_LEX_STRING(FROM,
                       System.RTS.Integer_State_Tables.Transitions,
                       System.RTS.Integer_State_Tables.Mapping,
                       MY_Last);
        LAST := My_Last;
        begin
            for i in FROM'first..My_Last loop
                -- Skip leading spaces
                My_First := i;
                exit when (FROM(i) /= Ada.Characters.Latin_1.Space and
			   FROM(i) /= Ada.Characters.Latin_1.HT);
            end loop;

            -- Since ITEM already lexed and legal, use a direct conversion
            -- with no check. ('value does check)    Offer
            ITEM := TGT.Longest_Integer(
		      System.RTS.Integer_Image_And_Value_Pkg.RTS_Integer_Value(
                        FROM(My_first .. My_Last)));
        exception
            when CONSTRAINT_ERROR =>
                raise DATA_ERROR;
        end;
    end GET;  -- string
--|
-------------------------------------------------------------

    procedure PUT(TO   : out STRING;
                  ITEM : in TGT.LONGEST_INTEGER;
                  BASE  : in NUMBER_BASE)
    is
    --+-----------------------------------------------------
    --| Overview
    --| INTEGER_IMAGE is employed to generate a string from ITEM,
    --| using TO'length as the WIDTH. However, INTEGER_IMAGE is required
    --| to override WIDTH if more room is needed.  Thus it is necessary
    --| to see if the returned string is too big before assigning it into
    --| TO.

   put_integer_string_internal_error : exception;
     -- raised if somehow the item_image is SMALLER than TO,
     -- implying integer_image didn't pad with blanks

       item_image : constant string :=
         INTEGER_IMAGE (item, +TO'length, base);

       diff : constant integer := ITEM_IMAGE'length - TO'length;

   begin
       -- Check that ITEM_IMAGE fits in TO

       if diff = 0 then        -- Same Size
           TO := ITEM_IMAGE;

       elsif diff > 0 then     -- Item_Image is bigger.
           raise LAYOUT_ERROR;

       else  -- TO is longer than ITEM_IMAGE. But then
             -- INTEGER_IMAGE didn't pad with enough blanks!
             raise put_integer_string_internal_error;
       end if;
    end PUT; -- string
--

-------------------------------------------------------------
-- Source: /usr9/mdavis/io/RCS/int_imb.bdy,v
-- Revision: 1.1  -- Date: 85/08/20 10:48:38  -- Author: mdavis

--------------------------------------------------------------
function INTEGER_IMAGE ( --| returns integer image
                         ITEM : TGT.LONGEST_INTEGER;
                         WIDTH : INTEGER;
                         BASE  : INTEGER)
    return STRING
is

--| Effect
--|   Returns a string with the IMAGE of ITEM.  If BASED /= 10, then
--| it has the syntax of a based number, with 1 or 2 digits for the
--| base.  There is a '-' sign if ITEM is negative, otherwise no
--| character position is reserved for a positive number.
--|   If the so-defined number consumes fewer than WIDTH positions, then
--| it is padded with leading spaces.  If WIDTH is too small, then WIDTH
--| is ignored.
--|
--| Algorithm
--|   Accumulate the result string BACKWARDS.  X holds the NEGATIVE magnitude
--| value remaining to be output. (Negative is used because it can handle
--| integer'first, which cannot be done using positive values for twos
--| complement machines.)
--|
--| Requires
--|   BASE is in range 2 .. 16.  Since this routine is a "helping" routine,
--|   PUT(file) and PUT(string) have already checked that base is correct.

    subtype index_subtype is positive range 1 .. 75;
        -- big enough for 64-bit integer in base 2

    index : index_subtype := index_subtype'last;  -- current position
      -- in BUF where next char should be put.

    buf : string (index_subtype);

    X : TGT.Longest_Integer := ITEM;

    use type TGT.Longest_Integer;

    procedure ADD (C : character)
    --+-----------------------------------------------------
    --| Overview
    --|   Adds C to buf(index), and decrements index by 1
    --| Note BUF is filled from back to front!
    is
    begin
        buf(index) := c;
        index := index - 1;
    end;

begin
    if X > 0 then X := - X; end if;  -- keep X non-positive

    if base /= 10 then
	add (Ada.Characters.Latin_1.Number_Sign);
    end if;

    for i in reverse index_subtype range buf'first .. index loop

        buf(i) := int_to_char(integer(1 - (X rem TGT.Longest_Integer(BASE))));
          -- X is negative, so we want
          -- to negate the result, because: (-X) rem BASE == -(X rem BASE)

        X := X / TGT.Longest_Integer(BASE);

        if X >= 0 then  -- (we do test here and use an EXIT instead of
                        -- a WHILE loop in order to handle the case of
                        -- ITEM = 0.)
            index := i - 1; -- set index to next avail pos in buf
            exit;
        end if;
    end loop;

    if base /= 10 then
        add (Ada.Characters.Latin_1.Number_Sign);
        add (int_to_char (1 + (BASE rem 10)));
            -- "ones" place for BASE's value
        if base > 10 then
	    add ('1');
	end if;  -- "tens" place
    end if;

    if ITEM < 0 then
	add (Ada.Characters.Latin_1.Minus_Sign);
    end if;

    -- Result is in buf (index + 1 .. buf'last)
    -- Tack on (width - (buf'last - index)) leading spaces, if necessary

    return (1 .. (width - (buf'last - index)) => Ada.Characters.Latin_1.Space) &
           buf (index + 1 .. buf'last);

end INTEGER_IMAGE;

end INTEGER_IO_SUPPORT;


-------------------------------------------------------------
--             INTEGER_IO : BODY
-------------------------------------------------------------

package body INTEGER_IO is

subtype NUM_RANGE is     --range of NUM in type integer
TGT.Longest_Integer range TGT.Longest_Integer(NUM'first) ..
			   TGT.Longest_Integer(NUM'last);
                     --used in range check of resulting integer


procedure GET( FILE : in FILE_TYPE;
     	       ITEM : out NUM;
     	       WIDTH : in FIELD := 0)  is
    INTEGER_ITEM : TGT.Longest_Integer;
begin
    INTEGER_IO_SUPPORT.GET (FILE, INTEGER_ITEM, WIDTH);

    if INTEGER_ITEM in NUM_RANGE then
        ITEM := NUM (INTEGER_ITEM);
    else
    -- If the result is not within the instantiated subtype, then
    -- DATA_ERROR must be raised.  If the value is within NUM,
    -- but outside range of ITEM's subtype, then CONSTRAINT_ERROR
    -- will be raised on return.
        raise DATA_ERROR;
    end if;
end GET;


procedure GET( ITEM : out NUM;
    	       WIDTH : in FIELD := 0 ) is
begin
    Check_Current_Input;
    Get(CURRENT_INPUT, ITEM, WIDTH);
end GET;


procedure PUT( FILE  : in FILE_TYPE;
    	       ITEM  : in NUM;
    	       WIDTH : in FIELD := default_width;
    	       BASE  : in NUMBER_BASE := default_base ) is
begin
    INTEGER_IO_SUPPORT.PUT (FILE, TGT.LONGEST_INTEGER (ITEM), WIDTH, BASE);
end PUT;


procedure PUT( ITEM : in NUM;
    	       WIDTH : in FIELD := default_width;
    	       BASE : in NUMBER_BASE := default_base ) is
begin
    Check_Current_Output;
    PUT( CURRENT_OUTPUT, ITEM, WIDTH, BASE);
end PUT;


procedure GET( FROM : in STRING;
    	       ITEM : out NUM;
    	       LAST : out POSITIVE ) is
    INTEGER_ITEM : TGT.Longest_integer;
begin
    INTEGER_IO_SUPPORT.GET (FROM, INTEGER_ITEM, LAST);

    if INTEGER_ITEM in NUM_RANGE then
        ITEM := NUM (INTEGER_ITEM);
    else
    -- If the result is not within the instantiated subtype, then
    -- DATA_ERROR must be raised.  If the value is within NUM,
    -- but outside range of ITEM's subtype, then CONSTRAINT_ERROR
    -- will be raised on return.
        raise DATA_ERROR;
    end if;
end GET;


procedure PUT( TO : out STRING;
    	       ITEM : in NUM;
    	       BASE : in NUMBER_BASE := default_base ) is
begin
   INTEGER_IO_SUPPORT.PUT (TO, TGT.LONGEST_INTEGER (ITEM), BASE);
end PUT;

end INTEGER_IO;

-------------------------------------------------------------
--        MODULAR_IO_SUPPORT     SPEC
-------------------------------------------------------------

package MODULAR_IO_SUPPORT is

--| Overview
--|   MODULAR_IO_SUPPORT is the same as an instantiation of
--| MODULAR_IO for MODULAR types, except GET/PUT using default
--| files are missing.

-------------------------------------------------------------
    procedure GET(FILE  : in FILE_TYPE;
                  ITEM  : out UNS.Unsigned_Integer;
                  WIDTH : in FIELD);
-------------------------------------------------------------
    procedure PUT(FILE : in FILE_TYPE;
                  ITEM : in UNS.Unsigned_Integer;
                  WIDTH : in FIELD;
                  BASE  : in NUMBER_BASE);
-------------------------------------------------------------

    procedure GET(FROM : in STRING;
                  ITEM : out UNS.Unsigned_Integer;
                  LAST : out POSITIVE);
-------------------------------------------------------------

    procedure PUT(TO   : out STRING;
                  ITEM : in UNS.Unsigned_Integer;
                  BASE  : in NUMBER_BASE);

end MODULAR_IO_SUPPORT;


-------------------------------------------------------------
--        MODULAR_IO_SUPPORT     BODY
-------------------------------------------------------------


package body MODULAR_IO_SUPPORT is

-------------------------------------------------------------
--           LOCAL DECLARATIONS
-------------------------------------------------------------


function MODULAR_IMAGE ( --| returns image
                         ITEM : UNS.Unsigned_Integer;
                         WIDTH : INTEGER;
                         BASE  : INTEGER)
    return STRING;

int_to_char : constant string (1 .. 16) := "0123456789ABCDEF";
  -- int_to_char(i+1) is the hex_char for i
  -- used in MODULAR_IMAGE

--

-------------------------------------------------------------
    procedure GET(FILE  : in FILE_TYPE;
                  ITEM  : out UNS.Unsigned_Integer;
                  WIDTH : in FIELD)
    --| Effect
    --|   Lexes the next token from FILE using the syntax of
    --| modular type literals, and converts it to a modular type,
    --| putting the value into ITEM.  If WIDTH /= 0, then
    --| exactly that number of characters (or up to end-
    --| of-line) are used for the token; otherwise only
    --| those characters necessary to make up a modular type
    --| are read.
    --|
    --| Algorithm
    --|   Use RTS_LEX_FILE to LEX the next modular token
    --| via modular transition tables into BUFFER(1..last).
    --|   Generate the modular value via NUM'VALUE,
    --| converting any constraint error into DATA_ERROR.

    is
        buffer : string (1 .. 256); -- big enough for non-blank part of
                                    -- any integer
        last : natural;
    begin
        RTS_LEX_FILE (FILE,
                      System.RTS.Integer_State_Tables.Transitions, 
                      System.RTS.Integer_State_Tables.Mapping,
                      WIDTH,
                      buffer,
                      last);
        --  Protect against constraint_error when converting to int, and
        -- change it to DATA_ERROR
        begin
            -- Since ITEM already lexed and legal, use a direct conversion
            -- with no check. ('value does check)    Offer
            ITEM := UNS.Unsigned_Integer(
		      System.RTS.Integer_Image_And_Value_Pkg.RTS_Integer_Value(
                        BUFFER(BUFFER'first .. last)));
        exception
            when CONSTRAINT_ERROR =>
                raise DATA_ERROR;
        end;
    end GET;


--|
-------------------------------------------------------------
    procedure PUT(FILE : in FILE_TYPE;
                  ITEM : in UNS.Unsigned_Integer;
                  WIDTH : in FIELD;
                  BASE  : in NUMBER_BASE)
    is
    --+-----------------------------------------------------
    --| Overview
    --|   Uses MODULAR_IMAGE to generate a string containing the
    --| necessary WIDTH and BASE.  PUT_WORD then outputs
    --| the string to the file, performing NEW_LINE, if necessary

    begin

        PUT_WORD (file => FILE,
                  word => MODULAR_IMAGE (item, width, base),
                  width => 0 ); -- no padding needed

    end PUT;  -- file

    --|
    -------------------------------------------------------------

    procedure GET(FROM : in STRING;
                  ITEM : out UNS.Unsigned_Integer;
                  LAST : out POSITIVE)
    --| Effect
    --|   Lexes the first token from FROM using the syntax of integer
    --|   literals, skips leading spaces, then converts it to an integer,
    --|   putting the value into ITEM. LAST is set to the index of the
    --|   last character used.
    --|
    --| Algorithm
    --|   Use RTS_LEX_STRING to LEX the next integer token via
    --|   Integer-Transition tables, setting My_Last. Skipping leading
    --|   spaces and then converting to integer using RTS_Integer_Value.
    --|   Convert any constraint error into DATA_ERROR.
    is
        My_Last   : natural;  -- Need local so the return value from
                              -- lex_string can be used in slice.
        My_First  : natural := From'First;  -- The first non-space character.
    begin
        System.RTS.Lexer.RTS_LEX_STRING(FROM,
                       System.RTS.Integer_State_Tables.Transitions, 
                       System.RTS.Integer_State_Tables.Mapping,
                       MY_Last);
        LAST := My_Last;
        begin
            for i in FROM'first..My_Last loop
                -- Skip leading spaces
                My_First := i;
                exit when (FROM(i) /= Ada.Characters.Latin_1.Space and
			   FROM(i) /= Ada.Characters.Latin_1.HT);
            end loop;

            -- Since ITEM already lexed and legal, use a direct conversion
            -- with no check. ('value does check)    Offer
            ITEM := UNS.Unsigned_Integer(
		      System.RTS.Integer_Image_And_Value_Pkg.RTS_Integer_Value(
                        FROM(My_first .. My_Last)));
        exception
            when CONSTRAINT_ERROR =>
                raise DATA_ERROR;
        end;
    end GET;  -- string
--|
-------------------------------------------------------------

    procedure PUT(TO   : out STRING;
                  ITEM : in UNS.Unsigned_Integer;
                  BASE  : in NUMBER_BASE)
    is
    --+-----------------------------------------------------
    --| Overview
    --| MODULAR_IMAGE is employed to generate a string from ITEM,
    --| using TO'length as the WIDTH. However, MODULAR_IMAGE is required
    --| to override WIDTH if more room is needed.  Thus it is necessary
    --| to see if the returned string is too big before assigning it into
    --| TO.

   put_integer_string_internal_error : exception;
     -- raised if somehow the item_image is SMALLER than TO,
     -- implying integer_image didn't pad with blanks

       item_image : constant string :=
         MODULAR_IMAGE (item, +TO'length, base);

       diff : constant integer := ITEM_IMAGE'length - TO'length;

   begin
       -- Check that ITEM_IMAGE fits in TO

       if diff = 0 then        -- Same Size
           TO := ITEM_IMAGE;

       elsif diff > 0 then     -- Item_Image is bigger.
           raise LAYOUT_ERROR;

       else  -- TO is longer than ITEM_IMAGE. But then
             -- MODULAR_IMAGE didn't pad with enough blanks!
             raise put_integer_string_internal_error;
       end if;
    end PUT; -- string

--------------------------------------------------------------

function MODULAR_IMAGE ( --| returns image
                         ITEM  : UNS.Unsigned_Integer;
                         WIDTH : INTEGER;
                         BASE  : INTEGER)
    return STRING
is

--| Effect
--|   Returns a string with the IMAGE of ITEM.  If BASED /= 10, then
--| it has the syntax of a based number, with 1 or 2 digits for the
--| base.  There is a '-' sign if ITEM is negative, otherwise no
--| character position is reserved for a positive number.
--|   If the so-defined number consumes fewer than WIDTH positions, then
--| it is padded with leading spaces.  If WIDTH is too small, then WIDTH
--| is ignored.
--|
--| Algorithm
--|   Accumulate the result string BACKWARDS.  X holds the NEGATIVE magnitude
--| value remaining to be output. (Negative is used because it can handle
--| integer'first, which cannot be done using positive values for twos
--| complement machines.)
--|
--| Requires
--|   BASE is in range 2 .. 16.  Since this routine is a "helping" routine,
--|   PUT(file) and PUT(string) have already checked that base is correct.

    subtype index_subtype is positive range 1 .. 75;
        -- big enough for 64-bit integer in base 2

    index : index_subtype := index_subtype'last;  -- current position
      -- in BUF where next char should be put.

    buf : string (index_subtype);

    X : UNS.Unsigned_Integer := ITEM;

    use type UNS.Unsigned_Integer;

    procedure ADD (C : character)
    --+-----------------------------------------------------
    --| Overview
    --|   Adds C to buf(index), and decrements index by 1
    --| Note BUF is filled from back to front!
    is
    begin
        buf(index) := c;
        index := index - 1;
    end;

begin

    if base /= 10 then
	add (Ada.Characters.Latin_1.Number_Sign);
    end if;

    for i in reverse index_subtype range buf'first .. index loop

        buf(i) := int_to_char(
		     integer(UNS."rem"(X, UNS.Unsigned_Integer(BASE))) + 1);

        if UNS."<"(X, UNS.Unsigned_Integer(BASE)) then
            index := i - 1; -- set index to next avail pos in buf
            exit;
        end if;

	X := UNS."/"(X, UNS.Unsigned_Integer(BASE));
    end loop;

    if base /= 10 then
        add (Ada.Characters.Latin_1.Number_Sign);
        add (int_to_char (1 + (BASE rem 10)));
            -- "ones" place for BASE's value
        if base > 10 then
	    add ('1');
	end if;  -- "tens" place
    end if;

    -- Result is in buf (index + 1 .. buf'last)
    -- Tack on (width - (buf'last - index)) leading spaces, if necessary

    return (1 .. (width - (buf'last - index)) => Ada.Characters.Latin_1.Space) &
           buf (index + 1 .. buf'last);

end MODULAR_IMAGE;

end MODULAR_IO_SUPPORT;

package body Modular_IO is

--range of NUM used in range check of resulting modular type
subtype NUM_RANGE is UNS.Unsigned_Integer range 
        UNS.Unsigned_Integer(NUM'first) ..  UNS.Unsigned_Integer(NUM'last);

MODULAR_ITEM : UNS.Unsigned_Integer;

procedure GET( FILE : in  File_Type;
    	       ITEM : out NUM;
    	       WIDTH : in FIELD := 0 ) is

begin
    MODULAR_IO_SUPPORT.GET (FILE, MODULAR_ITEM, WIDTH);
    if MODULAR_ITEM in NUM_RANGE then
	ITEM := NUM (MODULAR_ITEM);
    else
	raise DATA_ERROR;
    end if;
end GET;

procedure GET( ITEM: out NUM;
    	       WIDTH : in FIELD := 0 ) is
begin
    Check_Current_Input;
    Get(Current_Input, ITEM, WIDTH);
end GET;

procedure PUT( FILE  : in File_Type;
    	       ITEM  : in NUM;
    	       WIDTH : in FIELD := DEFAULT_WIDTH;
    	       BASE  : in NUMBER_BASE := DEFAULT_BASE ) is
begin
    MODULAR_IO_SUPPORT.PUT (FILE, UNS.Unsigned_Integer(ITEM), WIDTH, BASE);
end PUT;

procedure PUT( ITEM  : in NUM;
    	       WIDTH : in FIELD := DEFAULT_WIDTH;
    	       BASE  : in NUMBER_BASE := DEFAULT_BASE ) is
begin
    Check_Current_Output;
    PUT( CURRENT_OUTPUT, ITEM, WIDTH, BASE);
end PUT;

procedure GET( FROM : in String;
    	       ITEM : out NUM;
    	       LAST : out Positive ) is
begin
    MODULAR_IO_SUPPORT.GET (FROM, MODULAR_ITEM, LAST);
    if MODULAR_ITEM in NUM_RANGE then
	ITEM := NUM (MODULAR_ITEM);
    else
	raise DATA_ERROR;
    end if;
end GET;

procedure PUT( TO    : out String;
    	       ITEM  : in NUM;
    	       BASE  : in NUMBER_BASE := DEFAULT_BASE ) is
begin
    MODULAR_IO_SUPPORT.PUT (TO, UNS.Unsigned_Integer(ITEM), BASE);
end PUT;

end Modular_IO;


-------------------------------------------------------------
--        FLOAT_IO_SUPPORT
-------------------------------------------------------------

package FLOAT_IO_SUPPORT is

--| Overview
--|   FLOAT_IO_SUPPORT is like an instantiation of FLOAT_IO
--| for type FLOAT.  (The only missing routines are GET/PUT
--| with default files.)  We use LONG_FLOAT on RISCAE, so 32 bit
--| float is converted to 64 bit float before processing.

-------------------------------------------------------------

    procedure GET(FILE  : in FILE_TYPE;
                  ITEM  : out TGT.LONGEST_FLOAT;
                  WIDTH : in FIELD);

-------------------------------------------------------------
    procedure PUT (FILE : in FILE_TYPE;
                   ITEM : in TGT.LONGEST_FLOAT;
                   FORE : in FIELD;
                   AFT  : in FIELD;
                   EXP  : in FIELD);

-------------------------------------------------------------

    procedure GET(FROM : in STRING;
                  ITEM : out TGT.LONGEST_FLOAT;
                  LAST : out POSITIVE);

-------------------------------------------------------------

    procedure PUT (TO   : out STRING;
                   ITEM : in TGT.LONGEST_FLOAT;
                   AFT  : in FIELD;
                   EXP  : in FIELD);

end FLOAT_IO_SUPPORT;


-------------------------------------------------------------
--        FLOAT_IO_SUPPORT     BODY
-------------------------------------------------------------

package body FLOAT_IO_SUPPORT is

-------------------------------------------------------------

    procedure GET(FILE  : in FILE_TYPE;
                  ITEM  : out TGT.LONGEST_FLOAT;
                  WIDTH : in FIELD)

    --| Effect
    --|   Lexes the next token from FILE using the syntax of
    --| real literals, and converts it to a float,
    --| putting the value into ITEM.  If WIDTH /= 0, then
    --| exactly that number of characters (or up to end-
    --| of-line) are used for the token; otherwise only
    --| those characters necessary to make up a real
    --| are read.
    --|
    --| Algorithm
    --|   Use RTS_LEX_FILE to LEX the next real token
    --|   via REAL transition tables into BUFFER(1..last).
    --|   Generate the integer value via FLOAT_VALUE,
    --|   converting any constraint error into DATA_ERROR.

    is
        buffer : string (1 .. 512); -- big enough for non-blank part of
                                    -- any real
        last : natural;
    begin
        RTS_LEX_FILE (FILE,
                      FIO.FLOAT_TRANSIT.Transitions,
                      FIO.FLOAT_TRANSIT.Mapping,
                      WIDTH,
                      buffer,
                      last);
        --  Protect against constraint_error when converting to int, and
        -- change it to DATA_ERROR
        begin
            ITEM := FIO.FLOAT_VALUE (BUFFER(BUFFER'first .. last));
        exception
            when CONSTRAINT_ERROR =>
                raise DATA_ERROR;
        end;
    end GET;  -- file

--


-------------------------------------------------------------
    procedure PUT (FILE : in FILE_TYPE;
                   ITEM : in TGT.LONGEST_FLOAT;
                   FORE : in FIELD;
                   AFT  : in FIELD;
                   EXP  : in FIELD)
    is
    --+-----------------------------------------------------
    --| Overview
    --|   Uses FLOAT_IMAGE to generate a string containing the
    --| necessary FORE, AFT, EXP fields.  PUT_WORD then outputs
    --| the string to the file, performing NEW_LINE, if necessary

    begin

        PUT_WORD (file => FILE,
                  word => FIO.FLOAT_IMAGE (item, fore, aft, exp),
                  width => 0 ); -- no padding needed

    end PUT;  -- file

    -------------------------------------------------------------

    procedure GET(FROM : in STRING;
                  ITEM : out TGT.LONGEST_FLOAT;
                  LAST : out POSITIVE)
    --| Effect
    --|   Lexes the first token from FROM using the syntax of
    --| REAL literals, and converts it to a float,
    --| putting the value into ITEM. LAST is set to the
    --| index of the last character used.
    --|
    --| Algorithm
    --|   Use RTS_LEX_STRING to LEX the next real token
    --|   via Real transition tables, setting My_Last.
    --|   Generate the float value via FLOAT_VALUE,
    --|   converting any constraint error into DATA_ERROR.
    is
        My_Last   : natural;  -- Need local so the return value from
                              -- lex_string can be used in slice.
    begin
        System.RTS.Lexer.RTS_LEX_STRING(FROM,
                       FIO.FLOAT_TRANSIT.Transitions,
                       FIO.FLOAT_TRANSIT.Mapping,
                       My_Last);
        LAST := My_Last;
        begin
            ITEM := FIO.FLOAT_VALUE (FROM(FROM'first .. My_Last));
        exception
            when CONSTRAINT_ERROR =>
                raise DATA_ERROR;
        end;
    end GET;  -- string

    -------------------------------------------------------------

    procedure PUT (TO   : out STRING;
                   ITEM : in TGT.LONGEST_FLOAT;
                   AFT  : in FIELD;
                   EXP  : in FIELD)
    is
    --+-----------------------------------------------------
    --| Algorithm
    --| FLOAT_IMAGE is employed to generate a string from ITEM.
    --| However, first a value for FORE should be computed.
    --| This isn't strictly necessary, since float_image is required
    --| to override sizes of FORE and EXP if more room is needed.
    --| However, guessing "right" means the result can be put directly
    --| into TO; otherwise it is necessary to check that a tail portion
    --| of the string can be assigned into TO such that the head portion is
    --| ONLY spaces. This could happen in the following case:
    --|   exp=1, and a value of 20 was computed for FORE.
    --|   You always need at least 2 for exp, so that is
    --|   1 unexpected character needed, and the returned
    --|   string will be LONGER than TO.  However, FORE
    --|   was very generous (since if exp/=0, at most 2
    --|   positions are needed for FORE), so there are
    --|   at least 18 leading blanks that can be eliminated.

        FORE : integer :=   -- FORE = string_length - specified_portions
          TO'length -
          ( 1              -- '.'
          + AFT            -- fraction
          + boolean'pos(EXP /= 0)  -- 'E' if there will be an exponent
          + EXP );           -- sign and digits of exponent

         put_float_string_internal_error : exception;
           -- raised if somehow the item_image is SMALLER than TO,
           -- implying this computation of FORE is bad.

     begin
         -- TO may not have been big enough, so FORE might be negative,
         -- it might not belong to subtype FIELD, so LAYOUT_ERROR should
         -- be raised

         if FORE not in FIELD then
             raise LAYOUT_ERROR;
         end if;

         declare
             -- All parms are prepared, so convert item to a string

             item_image : constant string :=
               FIO.FLOAT_IMAGE (item, fore, aft, exp);

             diff : constant integer := ITEM_IMAGE'length - TO'length;

         begin
             -- Check that ITEM_IMAGE fits in TO, or that all omitted
             -- leading characters are blanks

             if diff = 0 then
                 -- Same Size
                 TO := ITEM_IMAGE;

             elsif diff > 0 then
                 -- Item_Image is bigger.  Check that first diff
                 -- characters are blanks

                 if ITEM_IMAGE(ITEM_IMAGE'first ..
                               ITEM_IMAGE'first + diff -1) =
                    (1 .. diff-1 => Ada.Characters.Latin_1.Space) then
                     -- It's OK, so assign a slice to TO
                     TO := ITEM_IMAGE (ITEM_IMAGE'first + diff ..
                                       ITEM_IMAGE'last);
                 else
                     -- Nope, doesn't fit!
                     raise LAYOUT_ERROR;
                 end if;

             else  -- TO is longer than ITEM_IMAGE. But we computed a
                   -- value such that this couldn't happen!
                   raise put_float_string_internal_error;
             end if;
         end;  -- declare

         return;
    end PUT; -- string

end FLOAT_IO_SUPPORT;


-------------------------------------------------------------
--          FLOAT_IO : BODY
-------------------------------------------------------------

package body FLOAT_IO is

subtype NUM_RANGE is    -- NUM range in type float
--sem FLOAT range float(NUM'first) .. float(NUM'last);
--sem dies on -------------| and --------------|
        TGT.LONGEST_FLOAT;  --sem --


procedure GET( FILE : in FILE_TYPE;
    	       ITEM : out NUM;
    	       WIDTH : in FIELD := 0) is
    FLOAT_ITEM : TGT.Longest_Float;
begin
    FLOAT_IO_SUPPORT.GET (FILE, FLOAT_ITEM, WIDTH);

    if FLOAT_ITEM in NUM_RANGE then
        ITEM := NUM (FLOAT_ITEM);
    else
    -- If the result is not within the instantiated subtype, then
    -- DATA_ERROR must be raised.  If the value is within NUM,
    -- but outside range of ITEM's subtype, then CONSTRAINT_ERROR
    -- will be raised on return.
        raise DATA_ERROR;
    end if;
exception  --sem
    when constraint_error =>  --sem  happens when convert float to num
         raise DATA_ERROR; --sem
end GET;


procedure GET( ITEM : out NUM;
    	       WIDTH : in FIELD := 0) is
begin
    Check_Current_Input;
    Get(CURRENT_INPUT, ITEM, WIDTH);
end GET;


procedure PUT( FILE : in FILE_TYPE;
    	       ITEM : in NUM;
    	       FORE : in FIELD := default_fore;
    	       AFT  : in FIELD := default_aft;
    	       EXP  : in FIELD := default_exp ) is
begin
    FLOAT_IO_SUPPORT.PUT (FILE, TGT.LONGEST_FLOAT(ITEM), FORE, AFT, EXP);
end PUT;


procedure PUT( ITEM : in NUM;
    	       FORE : in FIELD := default_fore;
    	       AFT  : in FIELD := default_aft;
    	       EXP  : in FIELD := default_exp) is
begin
    Check_Current_Output;
    PUT( CURRENT_OUTPUT, ITEM, FORE, AFT, EXP);
end PUT;


procedure GET( FROM : in STRING;
    	       ITEM : out NUM;
    	       LAST : out POSITIVE ) is
    FLOAT_ITEM : TGT.Longest_Float;
begin
    FLOAT_IO_SUPPORT.GET (FROM, FLOAT_ITEM, LAST);

    if FLOAT_ITEM in NUM_RANGE then
        ITEM := NUM (FLOAT_ITEM);
    else
    -- If the result is not within the instantiated subtype, then
    -- DATA_ERROR must be raised.  If the value is within NUM,
    -- but outside range of ITEM's subtype, then CONSTRAINT_ERROR
    -- will be raised on return.
        raise DATA_ERROR;
    end if;
exception  --sem
    when constraint_error =>  --sem  happens when convert float to num
         raise DATA_ERROR; --sem
end GET;


procedure PUT( TO   : out STRING;
    	       ITEM : in NUM;
    	       AFT  : in FIELD := default_aft;
    	       EXP  : in FIELD := default_exp ) is
begin
    FLOAT_IO_SUPPORT.PUT (TO, TGT.LONGEST_FLOAT(ITEM), AFT, EXP);
end PUT;

end FLOAT_IO;


-------------------------------------------------------------
--           FIXED IO : BODY
-------------------------------------------------------------

package body FIXED_IO is

--+-----------------------------------------------------
--| Overview
--| FIXED_IO is performed by converting to/from FLOAT, and
--| using FLOAT_IO_SUPPORT routines.  This will only
--| cause problems for Fixed values that are outside the
--| range of FLOAT, e.g., they have extremely large or small
--| values for 'SMALL.

procedure GET( FILE : in FILE_TYPE;
    	       ITEM : out NUM;
    	       WIDTH : in FIELD := 0 ) is
    FLOAT_ITEM : TGT.Longest_Float;
       subtype NUM_RANGE is TGT.LONGEST_FLOAT
    --sem   range float(NUM'first) .. float(NUM'last);
    --sem doesn't like num'first as an expression
       ; --sem
begin
    FLOAT_IO_SUPPORT.GET (FILE, FLOAT_ITEM, WIDTH);

    if FLOAT_ITEM in NUM_RANGE then
        ITEM := NUM (FLOAT_ITEM);
    else
    -- If the result is not within the instantiated subtype, then
    -- DATA_ERROR must be raised.  If the value is within NUM,
    -- but outside range of ITEM's subtype, then CONSTRAINT_ERROR
    -- will be raised on return.
        raise DATA_ERROR;
    end if;
exception  --sem
    when constraint_error =>  --sem  happens when convert float to num
        raise DATA_ERROR; --sem
end GET;


procedure GET( ITEM : out NUM;
    	       WIDTH : in FIELD := 0 ) is
begin
    Check_Current_Input;
    GET(CURRENT_INPUT, ITEM, WIDTH);
end GET;

procedure PUT( FILE : in FILE_TYPE;
               ITEM : in NUM;
               FORE : in FIELD := default_fore;
               AFT  : in FIELD := default_aft;
               EXP  : in FIELD := default_exp ) is
begin
    FLOAT_IO_SUPPORT.PUT (FILE, TGT.LONGEST_FLOAT(ITEM), FORE, AFT, EXP);
end PUT;

procedure PUT( ITEM : in NUM;
               FORE : in FIELD := default_fore;
               AFT  : in FIELD := default_aft;
               EXP  : in FIELD := default_exp ) is
begin
    Check_Current_Output;
    PUT( CURRENT_OUTPUT, ITEM, FORE, AFT, EXP);
end PUT;


procedure GET( FROM : in STRING;
               ITEM : out NUM;
               LAST : out POSITIVE ) is
    FLOAT_ITEM : TGT.Longest_Float;
    subtype NUM_RANGE is TGT.LONGEST_FLOAT
    --sem   range float(NUM'first) .. float(NUM'last);
    --sem doesn't like num'first as an expression
       ; --sem
begin
    FLOAT_IO_SUPPORT.GET (FROM, FLOAT_ITEM, LAST);

    if FLOAT_ITEM in NUM_RANGE then
        ITEM := NUM (FLOAT_ITEM);
    else
    -- If the result is not within the instantiated subtype, then
    -- DATA_ERROR must be raised.  If the value is within NUM,
    -- but outside range of ITEM's subtype, then CONSTRAINT_ERROR
    -- will be raised on return.
        raise DATA_ERROR;
    end if;
exception  --sem
    when constraint_error =>  --sem  happens when convert float to num
         raise DATA_ERROR; --sem
end GET;


procedure PUT( TO : out STRING;
               ITEM : in NUM;
               AFT : in FIELD := default_aft;
               EXP : in FIELD := default_exp ) is
begin
    FLOAT_IO_SUPPORT.PUT (TO, TGT.LONGEST_FLOAT(ITEM), AFT, EXP);
end PUT;

end FIXED_IO;


-------------------------------------------------------------
--           DECIMAL_IO
-------------------------------------------------------------

package body Decimal_IO is

procedure GET( FILE  : in File_Type;
    	       ITEM  : out NUM;
    	       WIDTH : in FIELD := 0 ) is
    FLOAT_ITEM : TGT.Longest_Float;
    subtype NUM_RANGE is TGT.LONGEST_FLOAT;
begin
    FLOAT_IO_SUPPORT.GET (FILE, FLOAT_ITEM, WIDTH);

    if FLOAT_ITEM in NUM_RANGE then
        ITEM := NUM (FLOAT_ITEM);
    else
        raise DATA_ERROR;
    end if;
exception
    when constraint_error => 
        raise DATA_ERROR;
end GET;

procedure GET( ITEM : out NUM;
    	       WIDTH : in FIELD := 0 ) is
begin
    Check_Current_Input;
    GET(CURRENT_INPUT, ITEM, WIDTH);
end GET;

procedure PUT( FILE : in File_Type;
    	       ITEM : in NUM;
    	       FORE : in FIELD := DEFAULT_FORE;
    	       AFT  : in FIELD := DEFAULT_AFT;
    	       EXP  : in FIELD := DEFAULT_EXP ) is
begin
    FLOAT_IO_SUPPORT.PUT (FILE, TGT.LONGEST_FLOAT(ITEM), FORE, AFT, EXP);
end PUT;

procedure PUT( ITEM : in NUM;
    	       FORE : in FIELD := DEFAULT_FORE;
    	       AFT  : in FIELD := DEFAULT_AFT;
    	       EXP  : in FIELD := DEFAULT_EXP ) is
begin
    Check_Current_Output;
    PUT( CURRENT_OUTPUT, ITEM, FORE, AFT, EXP);
end PUT;

procedure GET( FROM : in String;
    	       ITEM : out NUM;
    	       LAST : out Positive ) is
    FLOAT_ITEM : TGT.Longest_Float;
    subtype NUM_RANGE is TGT.LONGEST_FLOAT;
begin
    FLOAT_IO_SUPPORT.GET (FROM, FLOAT_ITEM, LAST);

    if FLOAT_ITEM in NUM_RANGE then
        ITEM := NUM (FLOAT_ITEM);
    else
        raise DATA_ERROR;
    end if;
exception 
    when constraint_error => 
         raise DATA_ERROR;
end GET;

procedure PUT( TO   : out String;
    	       ITEM : in NUM;
    	       AFT  : in FIELD := DEFAULT_AFT;
    	       EXP  : in FIELD := DEFAULT_EXP ) is
begin
    FLOAT_IO_SUPPORT.PUT (TO, TGT.LONGEST_FLOAT(ITEM), AFT, EXP);
end PUT;

end Decimal_IO;

-------------------------------------------------------------
--          ENUMERATION_IO: LOCAL DECLARATIONS
-------------------------------------------------------------
--| mapping and transitions for Enmumerations
package ENUM_TRANSIT is
use System.RTS.Lexer;

Mapping: Char_To_Token_Mapping := Char_To_Token_Mapping'
 ('0'..'9'                                     => Digit,
  'A'..'Z' | 'a'..'z'                          => Letter,
  Ada.Characters.Latin_1.Low_Line              => UnderScore,
  -- For character literal enumerations
  Ada.Characters.Latin_1.Apostrophe            => Quote,   
  -- Space can be in char literal enum...
  Ada.Characters.Latin_1.Space                 => Space,   
  -- ... but tab cannot!
  Ada.Characters.Latin_1.Ht                    => Htab,    
    -- This part uses knowledge of ASCII ordering from LRM C(13)
    -- to define other graphic characters as certain ranges.
  -- (Omit single quote)
  Ada.Characters.Latin_1.Exclamation .. Ada.Characters.Latin_1.Ampersand    |
  Ada.Characters.Latin_1.Left_Parenthesis .. Ada.Characters.Latin_1.Solidus |
  -- up to start of digits
  Ada.Characters.Latin_1.Colon .. Ada.Characters.Latin_1.Commercial_At      |
  -- to start of letters
  Ada.Characters.Latin_1.Reverse_Solidus       => Other_Printing,
  Ada.Characters.Latin_1.Right_Square_Bracket  => Other_Printing,
  Ada.Characters.Latin_1.Circumflex            => Other_Printing,
  Ada.Characters.Latin_1.Grave                 => Other_Printing,
  Ada.Characters.Latin_1.Left_Curly_Bracket    => Other_Printing,
  Ada.Characters.Latin_1.Vertical_Line         => Other_Printing,
  Ada.Characters.Latin_1.Right_Curly_Bracket   => Other_Printing,
  Ada.Characters.Latin_1.Tilde                 => Other_Printing,
    -- These other_printing chars may be character literal enumerals
  others                                       => Delim);

    -- Further state definitions for Enumerations
Expecting_Letter_Digit_Underscore_or_End  : constant state := 3;
Expecting_Letter_or_Digit                 : constant state := 4;
Expecting_Graphic_Char                    : constant state := 5;
Expecting_Quote                           : constant state := 6;
Expecting_End_After_Quote                 : constant state := 7;

  -- subtypes needed to define transitions constant:
subtype ST2 is State_transitions (Start..Expecting_End_After_Quote);
  -- ST2'last MUST be largest State constant for enumeration IO

    -- The following state transitions are what drive the
    -- Finite State Automaton (FSA).
Transitions: constant ST2 := ST2'(
  Start => (
    Space | Htab   => Start, -- Skip_Blanks() should prevent this
    Letter  => Expecting_Letter_Digit_Underscore_or_End,
    Quote   => Expecting_Graphic_Char,
    others  => Error),
  Expecting_Letter_Digit_Underscore_or_End => (
    Underscore      => Expecting_Letter_or_Digit,
    Letter | Digit  => Expecting_Letter_Digit_Underscore_or_End,
    others => Accepted),   -- End
  Expecting_Letter_or_Digit => (
    Letter | Digit  => Expecting_Letter_Digit_Underscore_or_End,
    others  => Error),

    -- transitions for character literal enumerations
  Expecting_Graphic_Char => (
    Letter | Digit | Quote | Underscore
    | Space | Other_Printing => Expecting_Quote,
    others  => Error),
  Expecting_Quote => (
    Quote   => Expecting_End_After_Quote,
    others  => Error),
  Expecting_End_After_Quote => (
    others  => Accepted),
  others => (       -- Did we forget anything?
    others => Error));
end Enum_transit;


function ENUM_IMAGE (  --| performs requested conversion lower case and
                   --| padding with trailing blanks to WIDTH
                   IMAGE : STRING;
                   WIDTH : FIELD;
                   SET   : TYPE_SET) return STRING
--+-----------------------------------------------------
--| Effect
--|   Builds a string of length max(width, image'length);
--| If SET=lower_case, AND this isn't a char literal, then
--| convert the characters to lower_case
--|
--| Requires
--|   IMAGE is a legal enum value, i.e., non-null, either is a char literal
--| or is in upper case already.
--|
--| Algorithm
--|   If size is OK, and no case conversion is needed, just return IMAGE.
--|   Else, build a BUFFER of the required size, inited to spaces.
--|   Move IMAGE into the front of it.  If case conversion is needed,
--|   change each uppercase char to lower case.
is
length : natural := IMAGE'length;
begin
if length >= width then
    -- No need to pad with blanks, if don't have to
    -- convert to lower case either, then just return
    if SET = UPPER_CASE or else
	-- No conversion on char lits
        IMAGE(IMAGE'first) = Ada.Characters.Latin_1.Apostrophe
      then
        return IMAGE;
    end if;
else
    -- need more space for spaces
    length := width;
end if;
declare
    subtype index_subtype is positive range 1 .. length;
    buffer : string (index_subtype) := (others => Ada.Characters.Latin_1.Space);
begin
    buffer (buffer'first .. IMAGE'length) := IMAGE;
    if SET = LOWER_CASE and then
       buffer(buffer'first) /= Ada.Characters.Latin_1.Apostrophe  then
        -- Have to convert to lower case
       for index in index_subtype range buffer'first..IMAGE'length
       loop
            buffer(index) := lower_case(buffer(index));
       end loop;
    end if;
    return buffer;
end;
end ENUM_IMAGE;

-------------------------------------------------------------
--          ENUMERATION_IO : BODY
-------------------------------------------------------------
-- Source: /usr9/mdavis/io/RCS/enum_iob.bdy,v
-- Revision: 1.4  -- Date: 85/08/21 17:34:12  -- Author: mdavis


package body ENUMERATION_IO is

   subtype ENUM_Width is positive range 1 .. Enum'WIDTH;

procedure GET (FILE : in FILE_TYPE;
               ITEM : out ENUM)
--| Effect
--|   Lexes the next token from FILE using the syntax of
--| Enumeration literals, and converts it to an ENUM,
--| putting the value into ITEM.
--|
--| Algorithm
--|   Use RTS_LEX_FILE to LEX the next ENUM token
--| via Enumeration transition tables into BUFFER(1..last).
--| Use WIDTH=0 which means to pick up the next token, stopping
--| as soon as the syntax is correct or in error.
--|   Generate the ENUM value via ENUM'VALUE,
--| converting any constraint error into DATA_ERROR.

is
    buffer : string (ENUM_Width); -- big enough for non-blank part of
                                -- any ENUM
    last : natural;
    first : natural := buffer'FIRST;
begin
        Check_Mode(File, In_File);  -- check for closed file/wrong mode
        RTS_LEX_FILE (FILE,
                      ENUM_TRANSIT.Transitions,
                      ENUM_TRANSIT.Mapping,
                      0,  -- width
                      buffer,
                      last);
    --  Protect against constraint_error when converting to enum, and
    -- change it to DATA_ERROR
    begin
        while first in BUFFER'first..last loop
            exit when (BUFFER(first) /= Ada.Characters.Latin_1.Space) and then
              (BUFFER(first) /= Ada.Characters.Latin_1.HT);
            first := first + 1;
        end loop;
        ITEM := ENUM'value (BUFFER(first .. last));
    exception
        when CONSTRAINT_ERROR =>
            raise DATA_ERROR;
    end;
end GET;  -- file


procedure GET( ITEM : out ENUM ) is
begin
    Check_Current_Input;
    GET (CURRENT_INPUT, ITEM);
end GET;


procedure PUT( FILE : in FILE_TYPE;
               ITEM : in ENUM;
               WIDTH : in FIELD := default_width ;
               SET : TYPE_SET := default_setting ) is
begin
    -- check for closed file/wrong mode
    Check_Mode(File, Mode_Array'(1 => Out_File, 2 => Append_File));
    PUT_WORD (file => FILE,
              word => ENUM_IMAGE (Enum'IMAGE(Item), 1, set),
              width => WIDTH);
end PUT;  -- file


procedure PUT( ITEM : in ENUM;
               WIDTH : in FIELD := default_width ;
               SET : in TYPE_SET := default_setting ) is
begin
    Check_Current_Output;
    PUT (CURRENT_OUTPUT, ITEM, WIDTH, SET);
end PUT;

procedure GET( FROM : in STRING;
               ITEM : out ENUM;
               LAST : out POSITIVE ) is
--| Effect
--|   Lexes the first token from FROM using the syntax of
--| enumeration literals, and converts it to an ENUM,
--| putting the value into ITEM. LAST is set to the
--| index of the last character used.
--|
--| Algorithm
--|   Use RTS_LEX_STRING to LEX the next enum token
--| via Enumeration transition tables, setting My_Last.
--|   Generate the enum value via ENUM'VALUE,
--| converting any constraint error into DATA_ERROR.
    My_First  : natural := FROM'first; -- for stripping lead blanks
    My_Last   : natural;  -- Need local so the return value from
                          -- lex_string can be used in slice.
begin
    System.RTS.Lexer.RTS_LEX_STRING(FROM,
                   ENUM_TRANSIT.Transitions,
                   ENUM_TRANSIT.Mapping,
                   MY_Last);
    LAST := My_Last;
    begin
        while My_First in FROM'first..My_Last loop
            exit when (FROM(My_First) /= Ada.Characters.Latin_1.Space) and then
              (FROM(My_First) /= Ada.Characters.Latin_1.HT);
            My_First := My_First + 1;
        end loop;
        ITEM := ENUM'value (FROM(My_First .. My_Last));
    exception
        when CONSTRAINT_ERROR =>
            raise DATA_ERROR;
    end;
end GET;  -- string


procedure PUT( TO : out STRING;
    	       ITEM : in ENUM;
    	       SET : in TYPE_SET := default_setting ) is
   item_image : constant string :=
           enum_image(Enum'IMAGE(Item), To'LENGTH, Set);

   diff : constant integer := ITEM_IMAGE'length - TO'length;

begin
   -- Check that ITEM_IMAGE fits in TO

   if diff = 0 then        -- Same Size
       TO := ITEM_IMAGE;

   elsif diff > 0 then     -- Item_Image is bigger.
       raise LAYOUT_ERROR;

   else  -- TO is longer than ITEM_IMAGE. But then
         -- ENUM_IMAGE didn't pad with enough blanks!
         raise Internal_Error;
   end if;
end PUT; -- string

end ENUMERATION_IO;

-- Local subprogram bodies

procedure Peek (
  File       : in File_Type;
  Peek_Until : in Peek_Place;
  Char       : out character
) is
begin
    Char := Ada.Characters.Latin_1.NUL;
    loop
        case File.Where is
            when Non_Terminator =>
                -- We have a non-terminator, just return it!
                Char := File.Peeked_Char;
                return;
            when After_EOL | After_EOP =>
                -- We need more information
                if CIO.End_Of_File(File.Char_IO_File) then
                    if File.Where = After_EOL then
                        -- Treat as end-of-page now
                        Next_Page(File);
                    end if;
                    File.Where := EOF_Only;
                else
		    begin
                        -- Get another character, and merge it
                        CIO.Read(File.Char_IO_File, File.Peeked_Char);
                        Merge_In_Peek(File);
                    exception
			when End_Error =>
                            if File.Where = After_EOL then
                                -- Treat as end-of-page now
                                Next_Page(File);
                            end if;
                            File.Where := EOF_Only;
			when others =>
			    raise;
		    end;
                end if;
            when Unknown | EOL | EOP =>
                if File.Where < Peek_Until then
                    -- We need more information
                    if CIO.End_Of_File(File.Char_IO_File) then
                        -- No more characters, indicate at EOL/EOP/EOF
                        File.Where := EOF;
                    else
			begin
                            -- Get another character and merge it
                            CIO.Read(File.Char_IO_File, File.Peeked_Char);
                            Merge_In_Peek(File);
                        exception
			    when End_Error =>
                                -- No more characters, indicate at EOL/EOP/EOF
                                File.Where := EOF;
			    when others =>
				raise;
			end;
                    end if;
                elsif Peek_Until = Non_Terminator then
                    -- We want a non-terminator, keep looking
                    Advance(File);
                else
                    -- All set, just return
                    return;
                end if;
            when others =>
                if Peek_Until = Non_Terminator then
                    -- We want a non-terminator, keep looking
                    Advance(File);
                else
                    -- All set, just return
                    return;
                end if;
        end case;
        -- "Where" has changed, loop and check again
    end loop;
end Peek;

procedure Advance ( File : in File_Type ) is
    Char : character;
begin
    case File.Where is
        when Non_Terminator =>
            -- Indicate peeked char used
            File.Where := Unknown; -- Makes File.Peeked_Char invalid
            File.Current_Column := File.Current_Column + 1;
        when EOL =>
            -- Indicate we don't know very much
            File.Where := After_EOL;
            Next_Line(File);
        when EOP =>
            -- This state indicates that if EOF follows, it should be
            -- treated as EOF_Only
            File.Where := After_EOP;
            Next_Page(File);
        when EOL_NOTP =>
            -- Clear end_of_line, merge peeked char
            File.Where := Unknown;
            Next_Line(File);
            Merge_In_Peek(File);
--             if File.Peeked_Char in Interesting_Characters then
--                 File.Where := Merge_When_Unknown(File.Peeked_Char);
--             else
--                 File.Where := Non_Terminator;
--             end if;
        when EOP_NOTF =>
            -- Clear end_of_page, merge in peeked char
            File.Where := Unknown;
            Next_Page(File);
            Merge_in_Peek(File);
            -- if File.Peeked_Char in Interesting_Characters then
            --     File.Where := Merge_When_Unknown(File.Peeked_Char);
            -- else
            --     File.Where := Non_Terminator;
            -- end if;
        when EOF =>
            File.Where := EOF_Only;
            Next_Page(File);
        when EOF_Only =>
            raise END_ERROR;
        when others =>
            -- Should never happen
            raise INTERNAL_ERROR;
    end case;
end Advance;

procedure Merge_In_Peek( File : in File_Type ) is
    Char : character;
begin
    case File.Where is
        when EOP =>
            -- Not at EOF if we get here, so indicate in File.Where
            File.Where := EOP_NOTF;
        when EOL =>
            -- We already have a line terminator, check for page term
            if File.Peeked_Char = Page_Terminator then
                File.Where := EOP;
            else
                File.Where := EOL_NOTP;  -- Not at end-of-page
            end if;
        when Unknown | After_EOL | After_EOP =>
            case File.Peeked_Char is
                when Line_Terminator =>
                    File.Where := EOL;
                when Page_Terminator =>
                    if File.Where = After_EOL then
                        File.Where := After_EOP;
                        Next_Page(File);
                    else
                        File.Where := EOP;
                    end if;
                when others =>
                    File.Where := Non_Terminator;
            end case;
        when others =>
            raise INTERNAL_ERROR;
    end case;
    if Record_Oriented then
        -- Absorb line terminator after page terminator when file is
        -- record-oriented.
        case File.Where is
            when EOP =>
                Peek(File, EOF, Char);
                if File.Where = EOP_NOTF and then
                    File.Peeked_Char = Line_Terminator then
                    -- It's a line terminator, ignore it.
                    File.Where := EOP;
                end if;
            when AFTER_EOP =>
                Peek (File, EOL, Char);
                if File.Where = EOL then
                    -- It's a line terminator, so ignore it
                    File.Where := UNKNOWN;
                end if;
            when others =>
                null;
        end case;
    end if;
end Merge_In_Peek;

procedure Close_Special_File ( File : in File_Type ) is
begin
    case File.Mode is
        when In_File =>
            Current_Input_File := null;
        when Out_File =>
            Current_Output_File := null;
        when Append_File =>
            Current_Output_File := null;
    end case;
end Close_Special_File;

procedure Init_File ( File : in File_Type;
    	    	      Mode : in File_Mode ) is
begin
    File.Mode := Mode;
    File.Current_Page := 1;
    File.Current_Line := 1;
    File.Current_Column := 1;
    File.Is_Special := false;
    File.Where := Unknown;
    if Mode = Out_File or else Mode = Append_File then
        File.Line_Length := Unbounded;
        File.Page_Length := Unbounded;
    end if;
end Init_File;

function Get_Standard_Input return File_Type is
    F : File_Type;
begin
    F := new File_Record;
    Init_File(F, In_File);
    CIO.Open_Standard_Input(F.Char_IO_File);
    return F;
end Get_Standard_Input;

function Get_Standard_Output return File_Type is
    F : File_Type;
begin
    F := new File_Record;
    Init_File(F, Out_File);
    CIO.Open_Standard_Output(F.Char_IO_File);
    return F;
end Get_Standard_Output;

function Get_Standard_Error return File_Type is
    F : File_Type;
begin
    F := new File_Record;
    Init_File(F, Out_File);
    CIO.Open_Standard_Error(F.Char_IO_File);
    return F;
end Get_Standard_Error;

procedure Local_Put ( File : in File_Type;
    	    	      Item : in string ) is
begin
    if File.Line_Length = Unbounded then
        -- Write out entire string at once, and update column number
        -- Note: this will cause the line/page number to be "incorrect"
        -- if the string contains line or page terminators, but this
        -- behavior is implementation dependent anyway.
        CIO.Write(File.Char_IO_File, Item);
        File.Current_Column := File.Current_Column + Item'LENGTH;
    else
        -- Line length is bounded, so carefully put out characters
        -- one at a time, keeping track of line/page numbers
        for i in Item'RANGE loop
            Put(File, Item(i));
        end loop;
    end if;
end Local_Put;

procedure Local_Put_Line ( File : in File_Type;
    	    	    	   Item : in string ) is
begin
    Local_Put(File, Item);
    Local_New_Line(File, 1);
end Local_Put_Line;

procedure Local_New_Line ( File : in File_Type;
    	    	    	   Spacing : in Positive_Count ) is
begin
    for I in 1 .. Spacing loop
        CIO.Write(File.Char_IO_File, Line_Terminator);
        Next_Line(File);
        if File.Page_Length /= Unbounded and then
            File.Current_Line > File.Page_Length then
              Local_New_Page(File);
        end if;
    end loop;
end Local_New_Line;

procedure Local_New_Page( File : in File_Type ) is
begin
    if File.Current_Column /= 1 or else File.Current_Line = 1 then
        -- Line not terminated or page is empty, so terminate line
        CIO.Write(File.Char_IO_File, Line_Terminator);
    end if;
    CIO.Write(File.Char_IO_File, Page_Terminator);
    if Record_Oriented then
        -- If record-oriented, line terminator follows page terminator
        CIO.Write(File.Char_IO_File, Line_Terminator);
    end if;
    Next_Page(File);
end Local_New_Page;

procedure Local_Get (
  File : in File_Type;
  Item : out string
) is
    Char : character;
begin
    for i in Item'RANGE loop
        Peek(File, Non_Terminator, Char);
        Item(i) := Char;
        Advance(File);
    end loop;
end Local_Get;

procedure Local_Get_Line (
  File : in File_Type;
  Item : out string;
  Last : out natural
) is
begin
    Last := Item'First-1;
    while Last < Item'Last loop
        if End_Of_Line (File) then
            Skip_Line (File);
            exit;
        end if;
        Last := Last + 1;
        Get (File, Item (Last));
    end loop;
end Local_Get_Line;

end Ada.Text_IO;


