-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.float_io_support.spc,v $ 
-- $Revision: 1.5 $ $Date: 96/04/01 14:54:56 $ $Author: maclaren $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Lexer;
with System.RTS.TGT;

package System.RTS.Float_IO_Support is

    pragma Suppress( Elaboration_Check );

--| Overview
--|   System.RTS.Float_IO_Support is like an instantiation of FLOAT_IO
--| for type FLOAT.  (The only missing routines are GET/PUT
--| with default files.)  

    package TGT renames System.RTS.TGT;

    function FLOAT_IMAGE( ITEM : in TGT.Longest_Float; 
					       --| value to cvt to string
                      	  FORE : in Natural;   --| size of integerpart
                      	  AFT  : in Natural;   --| size of fractional part
                      	  EXP  : in Natural)   --| size of exponent
             return STRING;
    
    --| Returns string corresponding to rules for
    --| FLOAT_IO.PUT RM 14.3.8



    function FLOAT_VALUE (IMAGE : string) return TGT.Longest_Float;
    --+-----------------------------------------------------
    --| Overview
    --| Returns the floating point value corresponding to IMAGE.
    --|
    --| Requires:
    --|   IMAGE is lexically correct (and therefore also non-null)
    --|         (leading and trailing blanks are OK)
    --|
    --| Exceptions
    --|   DATA_ERROR if based number and
    --|      (1) base is not in range 2 .. 16
    --|   or (2) any of "digits" don't belong to the base
    --|   or (3) value is too big to hold in FLOAT.  Numeric_Error is
    --|          converted to DATA_ERROR.

    -------------------------------------------------------------
    -- Stuff for FLOAT_VALUE:


    subtype legal_base is integer range 2 .. 16;
    subtype hex_values is integer range 0 .. 16;


    --| mapping and transitions for float
    package FLOAT_TRANSIT is
        use System.RTS.Lexer;
        Mapping: Char_To_Token_Mapping := Char_To_Token_Mapping'
         ('0'..'9'      => Digit,
          'e' | 'E'     => Exp, -- exponent; could also be extended digit
          'A'..'D' | 'F'..'Z' | 'a'..'d' | 'f'..'z' => ExDigit,
          '_'           => UnderScore,
          '#'           => Sharp,
          ':'           => Colon,
          '+' | '-'     => Sign,
          ' ' | Ascii.ht        => Blank,
          '.'           => Decimal,     -- radix point
          others  => Delim);

        -- Further state definitions, useful for both integer and float:
        Sign_or_Underscore          : constant state := 3;
        After_Digit                 : constant state := 4;
        Exponent    : constant state := 5; -- Have seen 'e' signaling exponent
        Digit_After_Exponent        : constant state := 6;
        Exponent_sign_or_underscore : constant state := 7;
        Base_Or_Underscore_Sharp    : constant state := 8;
            -- misnamed state???
        Based_Number_Sharp          : constant state := 9;
        Base_Or_Underscore_Colon    : constant state := 10;
            -- misnamed state???
        Based_Number_Colon          : constant state := 11;
        Complete_Base       : constant state := 12; -- finished non-decimal part
          -- Following states are added for float:
      Decimal_Point_Or_Underscore_Sharp         : constant state := 13;
      Digit_After_Decimal_In_Based_Number_Sharp : constant state := 14;
      Decimal_Point_Or_Underscore_Colon         : constant state := 15;
      Digit_After_Decimal_In_Based_Number_Colon : constant state := 16;
      Digit_After_Decimal                       : constant state := 17;
      Decimal_Point_Or_Underscore               : constant state := 18;

          -- subtypes needed to define transitions constant:

        subtype ST2 is State_transitions (Start..Decimal_Point_Or_Underscore);
          --  ST2'last MUST be largest state defined for float IO

            -- The following state transitions are what drive the
            -- Finite State Automaton (FSA).
        Transitions: constant ST2 := ST2'(
          Start => (
            Blank   => Start,        -- Skip_Blanks() should prevent this
            Digit   => After_Digit,
            Sign    => Sign_or_Underscore,
	    Decimal => Decimal_Point_Or_Underscore,
            others  => Error),
          Sign_or_Underscore => (
            Digit   => After_Digit,
            others  => Error),
          After_Digit => (
            Underscore => Sign_or_Underscore,
            Digit   => After_Digit,
            Sharp   => Base_or_Underscore_Sharp,
            Colon   => Base_or_Underscore_Colon,
            Decimal => Decimal_Point_Or_Underscore,
	    Exp     => Exponent,
	    others  => Accepted),
          Base_or_Underscore_Sharp => (
            Digit | ExDigit | Exp => Based_Number_Sharp,
            others  => Error),
          Based_Number_Sharp => (
            Digit | ExDigit | Exp => Based_Number_Sharp,
            Underscore      => Base_or_Underscore_Sharp,
            Decimal => Decimal_Point_Or_Underscore_Sharp,
            others  => Error),
          Base_or_Underscore_Colon => (
            Digit | ExDigit | Exp => Based_Number_Colon,
            others  => Error),
          Based_Number_Colon => (
            Digit | ExDigit | Exp => Based_Number_Colon,
            Underscore      => Base_or_Underscore_Colon,
            Decimal => Decimal_Point_Or_Underscore_Colon,
            others  => Error),
          Complete_Base => (
            Exp     => Exponent,
            others  => Accepted),
          Exponent => (
            Sign    => Exponent_Sign_or_Underscore,
            Digit   => Digit_After_Exponent,
            others  => Error),
          Digit_After_Exponent => (
            Digit   => Digit_After_Exponent,
            Underscore => Exponent_Sign_or_Underscore,
            others  => Accepted),
          Exponent_Sign_or_Underscore => (
            Digit   => Digit_After_Exponent,
            others  => Error),
          Decimal_Point_Or_Underscore_Sharp => (
            Digit | ExDigit |
              Exp => Digit_After_Decimal_In_Based_Number_Sharp,
            others  => Error),
          Digit_After_Decimal_In_Based_Number_Sharp => (
            Digit | ExDigit |
              Exp => Digit_After_Decimal_In_Based_Number_Sharp,
            Underscore      => Decimal_Point_Or_Underscore_Sharp,
            Sharp   => Complete_Base,
            others  => Error),
          Decimal_Point_Or_Underscore_Colon => (
            Digit | ExDigit |
              Exp => Digit_After_Decimal_In_Based_Number_Colon,
            others  => Error),
          Digit_After_Decimal_In_Based_Number_Colon => (
            Digit | ExDigit |
              Exp => Digit_After_Decimal_In_Based_Number_Colon,
            Underscore      => Decimal_Point_Or_Underscore_Colon,
            Colon   => Complete_Base,
            others  => Error),

            -- transitions for ordinary, decimal real notation:
          Decimal_Point_Or_Underscore => (
            Digit   => Digit_After_Decimal,
	    Exp     => Exponent,
	    others  => Accepted),
          Digit_After_Decimal => (
            Digit   => Digit_After_Decimal,
            Underscore      => Decimal_Point_Or_Underscore,
            Exp     => Exponent,
            others  => Accepted ),
          others => (
            others  => Error));
    end FLOAT_TRANSIT;


end System.RTS.Float_IO_Support;
