-- $Source: /home/dsd/d/aa/monoBANK/AA/j_system.rts.tgt.float_attributes_pkg.bdy,v $
-- $Revision: 1.2 $ $Date: 96/10/25 02:39:52 $ $Author: stt $
--        Copyright (C) 1995 by Intermetrics, Inc.   

-- Derived from:
--  Source: /home/dsd/d/aa/monoBANK/AA/system.rts.tgt.float_attributes_pkg.bdy,v  
--  Revision: 1.9    Date: 96/02/14 13:34:06    Author: akira  

with Interfaces;
with Ada.Unchecked_Conversion;
with Java.Lang.Float;
with Java.Lang.Double;

package body System.RTS.TGT.Float_Attributes_Pkg is
-- This package implements various Floating point attribute-functions for
-- IEEE single and double floating point representations.

    pragma Suppress( Elaboration_Check );

--  Some constants relating to single and double IEEE. Note that
--  the mantissa has an implicit high order one bit for non-denormals,
--  hence # mantissa bits is one more than # fraction bits.
--  Thus the value of an IEEE with sign s, exponent e, fraction f is

--	(-1)**s * 1.f * 2**(e-bias), with the following exceptions:

--	e = 0 and f = 0 => signed 0.0
--	e = 0 and f /= 0 => signed denormal, value = (-1)**s * 0.f * 2**(-bias)
--	e = max and f = 0 => signed infinity
--	e = max and f /= 0 => signed NANs, signaling and non-signaling

    Single_Frac_Bits	: constant := 23; -- Float'Machine_Mantissa-1
    Single_Exp_Bits	: constant := 8;
    Single_Exp_Bias	: constant := 127;
    Single_Exp_Max	: constant := 255; -- 2**Single_Exp_Bits - 1

    Double_Frac_Bits	: constant := 52; -- Long_Float'Machine_Mantissa-1
    Double_Frac_Lo_Bits	: constant := 32;
    Double_Frac_Hi_Bits	: constant := Double_Frac_Bits - Double_Frac_Lo_Bits;
    Double_Exp_Bits	: constant := 11;
    Double_Exp_Bias     : constant := 1023;
    Double_Exp_Max	: constant := 2047; -- 2**Double_Exp_Bits - 1

--  Here are some constants which define the bit positions in a portable
--  manner, depending on the bit ordering (System.Default_Bit_Order).

    Low_Is_1 : constant := Boolean'Pos(Default_Bit_Order = Low_Order_First);
    High_Is_1 : constant := Boolean'Pos(Default_Bit_Order = High_Order_First);

    Single_Sign_Pos	: constant := Low_Is_1 * 31 + High_Is_1 * 0;
    Single_Exp_Pos	: constant := Low_Is_1 * 23 + High_Is_1 * 1;
    Single_Frac_Pos 	: constant := Low_Is_1 * 0  + High_Is_1 * 9;

    Double_Sign_Pos	: constant := Low_Is_1 * 63 + High_Is_1 * 0;
    Double_Exp_Pos	: constant := Low_Is_1 * 52 + High_Is_1 * 1;
    Double_Frac_Hi_Pos	: constant := Low_Is_1 * 32 + High_Is_1 * 12;
    Double_Frac_Lo_Pos	: constant := Low_Is_1 * 0  + High_Is_1 * 32;

--  IEEE Single precision floating point representation

    type Single_Frac_Part is mod 2**Single_Frac_Bits;

    type Single_Record is record
        Sign     : Boolean;
	Exponent : Univ_Integer range 0..Single_Exp_Max;
	Frac     : Single_Frac_Part;
    end record;

--     for Single_Record use record
-- 	Sign     at 0 range Single_Sign_Pos..Single_Sign_Pos;
-- 	Exponent at 0 range Single_Exp_Pos..Single_Exp_Pos +
-- 				Single_Exp_Bits-1;
-- 	Frac     at 0 range Single_Frac_Pos..Single_Frac_Pos +
-- 				Single_Frac_Bits-1;
--     end record;

    function To_Uns32 is new Ada.Unchecked_Conversion(
	Integer, Interfaces.Unsigned_32);
    function To_Int is new Ada.Unchecked_Conversion(
	Interfaces.Unsigned_32, Integer);

    function To_Rec(From : Float) return Single_Record is
	Rec : Single_Record;
	use Interfaces;
	Bits : constant Unsigned_32 := 
	  To_Uns32(Java.Lang.Float.floatToIntBits(From));
    begin
	Rec.Sign := (Bits and 2**31) /= 0;
	Rec.Exponent := Univ_Integer(Shift_Right(Bits, Single_Frac_Bits) and 
	  (2**Single_Exp_Bits-1));
	Rec.Frac := Single_Frac_Part(Bits and (Single_Frac_Part'Modulus-1));
	return Rec;
    end To_Rec;

    function To_Flt(From : Single_Record) return Float is
	use Interfaces;
	Bits : Unsigned_32 :=
	  Shift_Left(Unsigned_32(From.Exponent), Single_Frac_Bits) or
	    Unsigned_32(From.Frac);
    begin
	if From.Sign then
	    Bits := Bits or 2**31;
	end if;
	return Java.Lang.Float.intBitsToFloat(To_Int(Bits));
    end To_Flt;


--  IEEE Double precision floating point representation

    -- type Double_Frac_Hi_Part is mod 2**Double_Frac_Hi_Bits;
    -- type Double_Frac_Lo_Part is mod 2**Double_Frac_Lo_Bits;
    type Double_Frac_Part is mod 2**Double_Frac_Bits;

    type Double_Record is record
        Sign     : Boolean;
	Exponent : Univ_Integer range 0..Double_Exp_Max;
	Frac	 : Double_Frac_Part;
	-- Frac_Hi  : Double_Frac_Hi_Part;
	-- Frac_Lo  : Double_Frac_Lo_Part;
    end record;

--     for Double_Record use record
-- 	Sign     at 0 range Double_Sign_Pos..Double_Sign_Pos;
-- 	Exponent at 0 range Double_Exp_Pos..Double_Exp_Pos +
-- 				Double_Exp_Bits-1;
-- 	Frac_Hi  at 0 range Double_Frac_Hi_Pos..Double_Frac_Hi_Pos +
-- 				Double_Frac_Hi_Bits-1;
-- 	Frac_Lo  at 0 range Double_Frac_Lo_Pos..Double_Frac_Lo_Pos +
-- 				Double_Frac_Lo_Bits-1;
--     end record;

    function To_Uns64 is new Ada.Unchecked_Conversion(
	Long_Integer, Interfaces.Unsigned_64);
    function To_Long is new Ada.Unchecked_Conversion(
	Interfaces.Unsigned_64, Long_Integer);


    function To_Rec(From : Long_Float) return Double_Record is
	Rec : Double_Record;
	use Interfaces;
	Bits : constant Unsigned_64 := 
	  To_Uns64(Java.Lang.Double.doubleToLongBits(From));
    begin
	Rec.Sign := (Bits and 2**63) /= 0;
	Rec.Exponent := Univ_Integer(Shift_Right(Bits, Double_Frac_Bits) and 
	  (2**Double_Exp_Bits-1));
	Rec.Frac := Double_Frac_Part(Bits and (Double_Frac_Part'Modulus-1));
	return Rec;
    end To_Rec;

    function To_Flt(From : Double_Record) return Long_Float is
	use Interfaces;
	Bits : Unsigned_64 :=
	  Shift_Left(Unsigned_64(From.Exponent), Double_Frac_Bits) or
	    Unsigned_64(From.Frac);
    begin
	if From.Sign then
	    Bits := Bits or 2**63;
	end if;
	return Java.Lang.Double.longBitsToDouble(To_Long(Bits));
    end To_Flt;

--------------------
-- Local subprograms
--------------------

    function Succ_Magnitude(
      Arg: Float
    ) return Float is
    -- Returns next machine number away from zero, i.e.
    -- Sign(Arg)*Float'Succ(Abs(Arg))
    -- Used to implement 'Pred and 'Succ
    -- Raises Constraint_Error or returns infinity if Arg = Float'Last
    -- Does not check for infinity or NANs.
	Arg_Rec : Single_Record := To_Rec(Arg);
    begin
	-- Use modular integer arithmentic on fraction
	Arg_Rec.Frac := Arg_Rec.Frac + 1;
	if Arg_Rec.Frac = 0 then
	    -- Increment(1.11111...) = 2.0 (mantissa overflow)
	    Arg_Rec.Exponent := Arg_Rec.Exponent + 1;
	    if Arg_Rec.Exponent = Single_Exp_Max then
		-- Exponent overflow to infinity
		if Float'Machine_Overflows then
		    raise Constraint_Error;
		end if;
		-- Arg_Rec already set to infinity
	    end if;
	end if;
	return To_Flt(Arg_Rec);
    end Succ_Magnitude;

    function Succ_Magnitude(
      Arg: Long_Float
    ) return Long_Float is
    -- Returns next machine number away from zero, i.e.
    -- Sign(Arg)*Long_Float'Succ(Abs(Arg))
    -- Used to implement 'Pred and 'Succ
    -- Raises Constraint_Error or returns infinity if Arg = Long_Float'Last
    -- Does not check for infinity or NANs.
	Arg_Rec : Double_Record := To_Rec(Arg);
    begin
	-- Use modular integer arithmentic on fraction
	Arg_Rec.Frac := Arg_Rec.Frac + 1;
	if Arg_Rec.Frac = 0 then
	    -- Increment(1.11111...) = 2.0 (mantissa overflow)
	    Arg_Rec.Exponent := Arg_Rec.Exponent + 1;
	    if Arg_Rec.Exponent = Double_Exp_Max then
		-- Exponent overflow to infinity
		if Long_Float'Machine_Overflows then
		    raise Constraint_Error;
		end if;
		-- Arg_Rec already set to infinity
	    end if;
	end if;
	return To_Flt(Arg_Rec);
    end Succ_Magnitude;

    function Pred_Magnitude(
      Arg: Float
    ) return Float is
    -- Assumes Arg /= 0.0
    -- Returns next machine number closer to zero, i.e.
    -- Sign(Arg)*Float'Pred(Abs(Arg))
    -- Used to implement 'Pred and 'Succ
    -- Result is undefined if Arg = +-0.0
	Arg_Rec : Single_Record := To_Rec(Arg);
    begin
	-- Use modular integer arithmentic on fraction
	Arg_Rec.Frac := Arg_Rec.Frac - 1;
	if Arg_Rec.Frac = Single_Frac_Part'Last then
	    -- Decrement(1.0) = 0.11111... (mantissa underflow)
	    Arg_Rec.Exponent := Arg_Rec.Exponent - 1;
	    -- Allow underflow to denormal.
	    -- Generates incorrect value if Arg = 0.0
	end if;
	return To_Flt(Arg_Rec);
    end Pred_Magnitude;

    function Pred_Magnitude(
      Arg: Long_Float
    ) return Long_Float is
    -- Assumes Arg /= 0.0
    -- Returns next machine number closer to zero, i.e.
    -- Sign(Arg)*Long_Float'Pred(Abs(Arg))
    -- Used to implement 'Pred and 'Succ
    -- Result is undefined if Arg = +-0.0
	Arg_Rec : Double_Record := To_Rec(Arg);
    begin
	-- Use modular integer arithmentic on fraction
	Arg_Rec.Frac := Arg_Rec.Frac - 1;
	if Arg_Rec.Frac = Double_Frac_Part'Last then
	    -- Decrement(1.0) = 0.11111... (mantissa underflow)
	    Arg_Rec.Exponent := Arg_Rec.Exponent - 1;
	    -- Allow underflow to denormal.
	    -- Generates incorrect value if Arg = 0.0
	end if;
	return To_Flt(Arg_Rec);
    end Pred_Magnitude;

    function Even(Arg : Float) return Boolean;
    pragma Inline(Even);

    function Even(Arg : Long_Float) return Boolean;
    pragma Inline(Even);

    function Even(Arg : Float) return Boolean is
    begin
	-- Rely on Integer precision > float precision
	return Integer(Arg) mod 2 = 0;
    exception
	when Constraint_Error =>
	    -- Exponent too large to convert to integer.
	    -- Since mantissa precision does not hold even/odd bit,
	    -- its considered even.
	    return True;
    end Even;

    function Even(Arg : Long_Float) return Boolean is
    begin
	-- Can not rely on Integer precision > long_float precision
	-- Assumes integral float
	if RTS_Long_Float_Floor(Arg/2.0)*2.0 = Arg then
	    return True;
	else
	    return False;
	end if;
    end Even;

---------------------
-- Global subprograms
---------------------

    function RTS_float_succ(
      Arg: Float
    ) return Float is
    begin
	-- Return next machine number.
	if Arg = 0.0 then
	    return Float'Succ(0.0);  -- Static
	elsif Arg < 0.0 then
	    return Pred_Magnitude(Arg);
	else
	    return Succ_Magnitude(Arg);
	end if;
    end RTS_float_succ;

    function RTS_Long_float_succ(
      Arg: Long_Float
    ) return Long_Float is
    begin
	-- Return next machine number.
	if Arg = 0.0 then
	    return Long_Float'Succ(0.0);  -- Static
	elsif Arg < 0.0 then
	    return Pred_Magnitude(Arg);
	else
	    return Succ_Magnitude(Arg);
	end if;
    end RTS_Long_float_succ;

    function RTS_float_pred(
      Arg: Float
    ) return Float is
    begin
	-- Return preceding machine number;
	-- Create appropriate increment and subtract from Arg
	if Arg = 0.0 then
	    return Float'Pred(0.0); -- Static
	elsif Arg < 0.0 then
	    return Succ_Magnitude(Arg);
	else
	    return Pred_Magnitude(Arg);
	end if;
    end RTS_float_pred;

    function RTS_Long_float_pred(
      Arg: Long_Float
    ) return Long_Float is
    begin
	-- Return preceding machine number;
	-- Create appropriate increment and subtract from Arg
	if Arg = 0.0 then
	    return Long_Float'Pred(0.0); -- Static
	elsif Arg < 0.0 then
	    return Succ_Magnitude(Arg);
	else
	    return Pred_Magnitude(Arg);
	end if;
    end RTS_Long_float_pred;

    function RTS_float_leading_part(
      Arg: Float;
      Radix_Digits: Univ_Integer
    ) return Float is
	Arg_Rec : Single_Record := To_Rec(Arg);
	Mantissa_Bits : constant := Single_Frac_Bits+1; -- 'Machine_Mantissa;
    begin
	-- Create value with only Radix_Digits of precision.
	-- Other bits of mantissa should be zero (in canonical representation)
	-- Used also in implementation of 'Truncation.
 
	-- Check if Radix_Digits is too big or too small for masking fraction
	if Radix_Digits <= 0 then
	    -- LRM A.5.3(59)
	    raise Constraint_Error;
	elsif Radix_Digits >= Mantissa_Bits then
	    -- Radix_Digits big enough so no masking needed.
	    return Arg;
	end if;
	-- Radix_Digits in range for masking fraction, in 1 .. 23
	-- We want to zero N low order fraction bits, where N in 1 .. 23
	-- N = Mantissa_Bits - Radix_Digits
	declare
	    use Interfaces;
	    Mask : constant Unsigned_32 := Single_Frac_Part'Modulus -
	      -- Shift left by # zero bits desired (N)
	      Shift_Left(1, Mantissa_Bits - Natural(Radix_Digits));
	begin
	    Arg_Rec.Frac := Arg_Rec.Frac and Single_Frac_Part(Mask);
	end;
	return To_Flt(Arg_Rec);
    end RTS_float_leading_part;

    function RTS_long_float_leading_part(
      Arg: Long_Float;
      Radix_Digits: Univ_Integer
    ) return Long_Float is
	Arg_Rec : Double_Record := To_Rec(Arg);
	Mantissa_Bits : constant := Double_Frac_Bits+1; -- 'Machine_Mantissa;
    begin
	-- Create value with only Radix_Digits of precision.
	-- Other bits of mantissa should be zero (in canonical representation)
	-- Used also in implementation of 'Truncation.
 
	-- Check if Radix_Digits is too big or too small for masking fraction
	if Radix_Digits <= 0 then
	    -- LRM A.5.3(59)
	    raise Constraint_Error;
	elsif Radix_Digits >= Mantissa_Bits then
	    -- Radix_Digits big enough so no masking needed.
	    return Arg;
	end if;
	-- Radix_Digits in range for masking fraction, in 1 .. 52
	-- We want to zero N low order fraction bits, where N in 1 .. 52
	-- N = Mantissa_Bits - Radix_Digits
	declare
	    use Interfaces;
	    Mask : constant Unsigned_64 := - Shift_Left(1,
		Mantissa_Bits - Natural(Radix_Digits));
	begin
	    Arg_Rec.Frac := Arg_Rec.Frac and
		Double_Frac_Part(Mask);
	end;
	return To_Flt(Arg_Rec);
    end RTS_long_float_leading_part;

    function RTS_float_truncation(
      Arg: Float
    ) return Float is
	Arg_Rec : Single_Record := To_Rec(Arg);
	Exponent : constant Univ_Integer :=
		Arg_Rec.Exponent - Single_Exp_Bias;
    begin
	-- Find integral value closer to zero.
	-- Truncation is used for implementing other "rounding" functions.
 
	-- Check if exponent is too small for using 'Leading_Part
	if Exponent < 0 then
	    -- Exponent small enough so result is zero
	    -- LRM A.5.3(44) Result has sign of Arg
	    if Arg_Rec.Sign then
		return -0.0;
	    else
		return 0.0;
	    end if;
	end if;
 
	-- Exponent in range to use 'Leading_Part
	-- We want to keep exponent bits of the fraction, or
	-- exponent+1 bits of the mantissa (1.fraction) .
	return RTS_float_leading_part(Arg, Exponent+1);
    end RTS_float_truncation;

    function RTS_long_float_truncation(
      Arg: Long_Float
    ) return Long_Float is
	Arg_Rec : Double_Record := To_Rec(Arg);
	Exponent : constant Univ_Integer :=
		Arg_Rec.Exponent - Double_Exp_Bias;
    begin
	-- Find integral value closer to zero.
	-- Truncation is used for implementing other "rounding" functions.
 
	-- Check if exponent is too small for using 'Leading_Part
	if Exponent < 0 then
	    -- Exponent small enough so result is zero
	    -- LRM A.5.3(44) Result has sign of Arg
	    if Arg_Rec.Sign then
		return -0.0;
	    else
		return 0.0;
	    end if;
	end if;
 
	-- Exponent in range to use 'Leading_Part
	-- We want to keep exponent bits of the fraction, or
	-- exponent+1 bits of the mantissa (1.fraction) .
	return RTS_long_float_leading_part(Arg, Exponent+1);
    end RTS_long_float_truncation;

    function RTS_float_floor(
      Arg: Float
    ) return Float is
	Trunc : constant Float := RTS_float_truncation(Arg);
    begin
	if Arg < 0.0 and then Arg /= Trunc then
	    return Trunc - 1.0;
	else
	    return Trunc;
	end if;
    end RTS_float_floor;

    function RTS_Long_float_floor(
      Arg: Long_Float
    ) return Long_Float is
	Trunc : constant Long_Float := RTS_long_float_truncation(Arg);
    begin
	if Arg < 0.0 and then Arg /= Trunc then
	    return Trunc - 1.0;
	else
	    return Trunc;
	end if;
    end RTS_Long_float_floor;

    function RTS_float_ceiling(
      Arg: Float
    ) return Float is
	Trunc : constant Float := RTS_float_truncation(Arg);
    begin
	if Arg > 0.0 and then Arg /= Trunc then
	    return Trunc + 1.0;
	else
	    return Trunc;
	end if;
    end RTS_float_ceiling;

    function RTS_long_float_ceiling(
      Arg: Long_Float
    ) return Long_Float is
	Trunc : constant Long_Float := RTS_long_float_truncation(Arg);
    begin
	if Arg > 0.0 and then Arg /= Trunc then
	    return Trunc + 1.0;
	else
	    return Trunc;
	end if;
    end RTS_long_float_ceiling;

    function RTS_float_rounding(
      Arg: Float
    ) return Float is
	Trunc : constant Float := RTS_float_truncation(Arg);
    begin
	-- Round to nearest, ties go away from zero (opposite of trunc)
	if Arg < 0.0 then
	    if Arg <= Trunc - 0.5 then
		return Trunc - 1.0;
	    end if;
	else
	    if Arg >= Trunc + 0.5 then
		return Trunc + 1.0;
	    end if;
	end if;
	return Trunc;
    end RTS_float_rounding;

    function RTS_long_float_rounding(
      Arg: Long_Float
    ) return Long_Float is
	Trunc : constant Long_Float := RTS_long_float_truncation(Arg);
    begin
	-- Round to nearest, ties go away from zero (opposite of trunc)
	if Arg < 0.0 then
	    if Arg <= Trunc - 0.5 then
		return Trunc - 1.0;
	    end if;
	else
	    if Arg >= Trunc + 0.5 then
		return Trunc + 1.0;
	    end if;
	end if;
	return Trunc;
    end RTS_long_float_rounding;

    function RTS_float_unbiased_rounding(
      Arg: Float
    ) return Float is
	Floor : constant Float := RTS_Float_Floor(Arg);
    begin
	-- Round to nearest, ties go toward even
	if Arg > Floor + 0.5 then
	    return Floor + 1.0;
	elsif Arg < Floor + 0.5 then
	    return Floor;
	elsif Even(Floor) then
	    return Floor;
	else
	    return Floor + 1.0;
	end if;
    end RTS_float_unbiased_rounding;

    function RTS_Long_float_unbiased_rounding(
      Arg: Long_Float
    ) return Long_Float is
	Floor : constant Long_Float := RTS_Long_Float_Floor(Arg);
    begin
	-- Round to nearest, ties go toward even
	if Arg > Floor + 0.5 then
	    return Floor + 1.0;
	elsif Arg < Floor + 0.5 then
	    return Floor;
	elsif Even(Floor) then
	    return Floor;
	else
	    return Floor + 1.0;
	end if;
    end RTS_Long_float_unbiased_rounding;

    function RTS_float_exponent(
      Arg: Float
    ) return Univ_Integer is
	Arg_Rec : constant Single_Record := To_Rec(Arg);
    begin
	-- Extract exponent, and subtract out bias.
	if Arg = 0.0 then
	    return 0;
	else
            -- see explanation of 'Fraction for the reason
            -- we add 1.
            return Arg_Rec.Exponent - Single_Exp_Bias + 1;
	end if;
    end RTS_float_exponent;

    function RTS_Long_float_exponent(
      Arg: Long_Float
    ) return Univ_Integer is
	Arg_Rec : constant Double_Record := To_Rec(Arg);
    begin
	-- Extract exponent, and subtract out bias.
	if Arg = 0.0 then
	    return 0;
	else
            -- see explanation of Float'Fraction for the reason
            -- we add 1.
            return Arg_Rec.Exponent - Double_Exp_Bias + 1;
	end if;
    end RTS_Long_float_exponent;

    function RTS_float_fraction(
      Arg: Float
    ) return Float is
	Arg_Rec : Single_Record := To_Rec(Arg);
    begin
	-- Extract fraction.
	if Arg = 0.0 then
	    return Arg;
	else
            -- In IEEE 754, For exponents in the range of 0 .. 255, the value is
            -- represented by (-1) ** sign * 2 ** (exponent - 127) * 1.fraction
            -- For 'Fraction, the fraction must be between 0 and 1, not 1 to 2.
            -- Therefore, we subtract 1.
            Arg_Rec.Exponent := Single_Exp_Bias - 1;
	end if;
	return To_Flt(Arg_Rec);
    end RTS_float_fraction;

    function RTS_Long_float_fraction(
      Arg: Long_Float
    ) return Long_Float is
	Arg_Rec : Double_Record := To_Rec(Arg);
    begin
	-- Extract fraction.
	if Arg = 0.0 then
	    return Arg;
	else
            -- see explanation of Float'Fraction for the reason
            -- we subtact 1.
            Arg_Rec.Exponent := Double_Exp_Bias - 1;
	end if;
	return To_Flt(Arg_Rec);
    end RTS_Long_float_fraction;

    function RTS_float_compose(
      Fraction: Float;
      Exponent: Univ_Integer
    ) return Float is
	Fraction_Rec : Single_Record := To_Rec(Fraction);
    begin
	-- Build up a floating point value given fraction and exponent.
	if Fraction = 0.0 then
	    return Fraction;
	else
	    Fraction_Rec.Exponent := Fraction_Rec.Exponent + Exponent;
	    return To_Flt(Fraction_Rec);
	end if;
    end RTS_float_compose;

    function RTS_Long_float_compose(
      Fraction: Long_Float;
      Exponent: Univ_Integer
    ) return Long_Float is
	Fraction_Rec : Double_Record := To_Rec(Fraction);
    begin
	-- Build up a floating point value given fraction and exponent.
	if Fraction = 0.0 then
	    return Fraction;
	else
	    Fraction_Rec.Exponent := Fraction_Rec.Exponent + Exponent;
	    return To_Flt(Fraction_Rec);
	end if;
    end RTS_Long_float_compose;

    function RTS_float_scaling(
      Arg: Float;
      Adjustment: Univ_Integer
    ) return Float -- renames RTS_float_compose;
    is begin
	return RTS_float_compose(Arg, Adjustment);
    end RTS_float_scaling;

    function RTS_Long_float_scaling(
      Arg: Long_Float;
      Adjustment: Univ_Integer
    ) return Long_Float -- renames RTS_Long_float_compose;
    is begin
	return RTS_Long_float_compose(Arg, Adjustment);
    end RTS_Long_float_scaling;

    function RTS_float_remainder(
      X, Y: Float
    ) return Float is
	Quotient : constant Float := RTS_Float_Unbiased_Rounding(X/Y);
    begin
	-- Determine remainder, such that |rem| <= |Y|/2
	return X - Quotient * Y;
    end RTS_float_remainder;

    function RTS_Long_float_remainder(
      X, Y: Long_Float
    ) return Long_Float is
	Quotient : constant Long_Float := 
	  RTS_Long_Float_Unbiased_Rounding(X/Y);
    begin
	-- Determine remainder, such that |rem| <= |Y|/2
	return X - Quotient * Y;
    end RTS_Long_float_remainder;

    function RTS_float_adjacent(
      X, Towards: Float
    ) return Float is
    begin
	-- Find next machine number, in direction of "Towards"
	if Towards = X then
	    return X;
	elsif Towards > X then
	    return RTS_float_succ(X);
	else
	    return RTS_float_pred(X);
	end if;
    end RTS_float_adjacent;

    function RTS_Long_float_adjacent(
      X, Towards: Long_Float
    ) return Long_Float is
    begin
	-- Find next machine number, in direction of "Towards"
	if Towards = X then
	    return X;
	elsif Towards > X then
	    return RTS_Long_float_succ(X);
	else
	    return RTS_Long_float_pred(X);
	end if;
    end RTS_Long_float_adjacent;

    function RTS_float_copy_sign(
      Value, Sign: Float
    ) return Float is
    begin
	-- Copy sign from Sign to Value
	if Sign < 0.0 then
	    if Value >= 0.0 then
		return -Value;
	    end if;
	else
	    if Value < 0.0 then
		return -Value;
	    end if;
	end if;
	return Value;
    end RTS_float_copy_sign;

    function RTS_Long_float_copy_sign(
      Value, Sign: Long_Float
    ) return Long_Float is
    begin
	-- Copy sign from Sign to Value
	if Sign < 0.0 then
	    if Value >= 0.0 then
		return -Value;
	    end if;
	else
	    if Value < 0.0 then
		return -Value;
	    end if;
	end if;
	return Value;
    end RTS_Long_float_copy_sign;

    function RTS_float_machine(
      Arg: Float
    ) return Float is
    begin
	-- Return adjacent machine-number; this is an identify 
	-- function if no extended precision intermediates are used.
	return Arg;
    end RTS_float_machine;

    function RTS_Long_float_machine(
      Arg: Long_Float
    ) return Long_Float is
    begin
	-- Return adjacent machine-number; this is an identify 
	-- function if no extended precision intermediates are used.
	return Arg;
    end RTS_Long_float_machine;

    function RTS_float_model(
      Arg: Float
    ) return Float is
    begin
	-- Return adjacent model-number; this is an identify 
	-- function if no extended precision intermediates are used,
	-- and hardware is reasonably accurate.
	return Arg;
    end RTS_float_model;

    function RTS_Long_float_model(
      Arg: Long_Float
    ) return Long_Float is
    begin
	-- Return adjacent model-number; this is an identify 
	-- function if no extended precision intermediates are used,
	-- and hardware is reasonably accurate.
	return Arg;
    end RTS_Long_float_model;

end System.RTS.TGT.Float_Attributes_Pkg;
