-- $Source: /home/dsd/d/aa/monoBANK/AA/j_system.rts.integer_image_and_value_pkg.bdy,v $ 
-- $Revision: 1.1 $ $Date: 96/04/01 14:47:28 $ $Author: maclaren $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Lexer;
with System.RTS.Integer_State_Tables;
with Ada.IO_Exceptions; use Ada.IO_Exceptions;
with Ada.Characters.Handling;
with Ada.Characters.Latin_1;
with System.RTS.Unsigned;

with Interfaces.Java;
with java.lang.Integer;
with java.lang.Long;
with java.lang.String;


package body System.RTS.Integer_Image_And_Value_Pkg is

-- WARNING: the subprogram declaration format is important, since this
-- file is read used with the rts calls from the emitter.
-- see intrintic_rts.c

    pragma Suppress( Elaboration_Check );

-- Note: The basis of this code is taken from the aie host_rts _type
-- subdirectory:
-- Source: /home/inmet/m/ada/monoBANK/RTS/RTS_IV.bdy,v 
-- Revision: 1.37  -- Date: 88/06/13 16:34:04  -- Author: callen 

    subtype LegalRange is integer range -1 .. 15 ;
    type CharValueArray is array (character) of LegalRange ;
    CharValue : constant CharValueArray := CharValueArray'
      ('0' => 0, '1' => 1, '2' => 2, '3' => 3, '4' => 4,
       '5' => 5, '6' => 6, '7' => 7, '8' => 8, '9' => 9,
       'a'|'A'=> 10, 'b'|'B' => 11, 'c'|'C'=>12, 'd'|'D'=>13,
       'e'|'E' => 14, 'f'|'F' => 15, others => -1) ;
         -- Put this here so it is only done once (since
	 --  static aggregates are still a ways off)
     
    ----------------------------------------------------------
    -- Local Scalar routines.
    ----------------------------------------------------------
   
    procedure RTS_Ivalue(
      The_Image : in  String;
      The_Value : out System.RTS.Unsigned.Unsigned_Integer;
      The_Sign  : out Integer
    )
    is
      use System.RTS.Unsigned;
        Result : System.RTS.Unsigned.Unsigned_Integer :=0;
        E_Result : System.RTS.Unsigned.Unsigned_Integer := 1 ;
        Base : integer := 10 ; -- Starts as 10 for integer or base of based
    			   --    literal
        Exponent_Base : integer := Base;
        
        Curr_Int  : System.RTS.Unsigned.Unsigned_Integer := 0;
    	--  Accumulates the negative of the "current integer",
    	-- which can be the base, integer, or exponent.
    	-- The value is always accumulated as a non-POSITIVE number,
    	-- because there is one more negative number than positive numbers
    	-- so it is possible to do value("<integer'first>").
    	-- THEREFORE you must negate curr_int to get its "TRUE" value
    	-- before use as base or exponent. Note that vbl Sign (below)
    	-- has positive== -1  negative== +1 so the conversion of curr_int
    	-- to its TRUE signed value happens via the multiply by sign.
    	
        Sharp_Encountered : Boolean := false;
        subtype LegalBaseRange is integer range 2 .. 16;
        IntValue : LegalRange ;
        Sign : integer range -1 .. +1 := +1 ;
        E_Sign : integer range -1 .. +1 := +1 ;
        
        E_is_exponent : boolean := true ; --| consider 'e' to be the
    				      --| beginning of exponent.
        Exponent_Seen : boolean := false; -- Has there been an exponent?
    begin
        --| Integer_literal ::= (integer | based_integer) exponent
        --| integer ::= digit < (:_:) digit >
        --| based_integer ::= base # extended_integer #
        --| extended_integer ::= extended_digit < (:_:) extended_digit >
        --| Requires
        --| The input string must be syntactically legal.
        --| Follows the rules in LRM Chapter 2.
        
        -- Note that if we are processing the digits between # and #
        -- that an e should be interpreted as a digit, not the
        -- indicator that an exponent is following.

        for i in The_Image'range loop
    	case The_Image(i) is
    	when '#' | ':' =>
    	    if Sharp_Encountered then
    		E_is_exponent := true ;
    		Sharp_encountered := false;
    	    else
    		Sharp_encountered := true;
    		E_is_exponent := false ;
    		Base := Integer(Curr_Int) ;
    		Exponent_Base := Base;
    		Curr_Int := 0; -- get ready for based number
    		if Base not in LegalBaseRange then
    		    raise CONSTRAINT_ERROR ;
    		end if ;
    	    end if;
    	when '_' =>
    	    null;
        when ' ' | Ada.Characters.Latin_1.HT =>
    	    -- Should not really get here, but...
    	    exit;
    	when '+' =>
	    if Exponent_Seen then
    	        E_Sign := +1 ;
            else
    	        Sign := +1 ;
	    end if;
    	when '-' =>
	    if Exponent_Seen then
    	        E_Sign := -1 ;
            else
    	        Sign := -1 ;
	    end if;
    	when others =>
    	    -- Process the digit.
    	    --| Should be either 0..9 a..f A..F
    	    IntValue := CharValue(The_Image(i)) ;
    	    case IntValue is
    	    when -1 =>
    		raise CONSTRAINT_ERROR ;
    	    when 14 =>  -- 'e' or 'E'
    		if E_is_exponent then
    		    if Exponent_Seen then
    			-- Multiple Exponents are Bad!
    			raise CONSTRAINT_ERROR;
    		    end if;
    		    Result := Curr_Int; 
    		    Exponent_Seen := true;
    		    -- Reset things for accumulating exponent
    		    E_Sign := +1;
    		    Curr_Int := 0;
    		    Base := 10;
    		    -- set IntValue to zero, so when we fall through to
    		    -- the intvalue test and multiply updating Curr_Int,
    		    -- that the effect will be to set Curr_Int to zero
    		    -- again.  Alternative is to use GOTO, or to introduce
    		    -- a boolean...
    		    IntValue := 0;
    		end if ;
    		-- E is just a digit in the number (between # and #)
    	    when others =>
    		null ;
    	    end case ;
    	    if IntValue not in 0 .. Base-1 then
    		raise CONSTRAINT_ERROR ;
    	    end if ;
    	    Curr_Int := (Curr_Int * System.RTS.Unsigned.Unsigned_Integer(Base))
			 + System.RTS.Unsigned.Unsigned_Integer(IntValue) ;
    	end case ;
        end loop ;
        
        if Exponent_Seen then
    	    if E_Sign = -1 then
    	        -- Negative exponent is illegal for integer'value
    	        raise CONSTRAINT_ERROR ;
    	    end if;
	    for i in 1 .. Integer(Curr_Int) loop
	        E_Result := E_Result * 
			System.RTS.Unsigned.Unsigned_Integer(Exponent_Base);
            end loop;
    	    Result := Result * E_Result;
        else
    	    Result := Curr_Int;
        end if;

        The_Value := Result;
	The_Sign  := Sign;
    exception
        when NUMERIC_ERROR =>
    	-- Can be raised by overflows.  But we're only supposed to
    	-- raise constraint error in this routine, so convert it.
    	raise CONSTRAINT_ERROR;
    
    end RTS_Ivalue;
    
    ----------------------------------------------------------
    ----------------------------------------------------------
        ----------------------------------------------------------
        -- Integer type routines.
        ----------------------------------------------------------
    
    function RTS_Integer_Image(
      The_Integer : Integer
    )
    return String
    is
    --+-------------------------------------------------------
    --| Overview:
    --|    Computes Integer'Image (The_Integer) 
    --|     1. Decimal
    --|     2. Leading '-' or blank
    --|     3. return string'first == 1
    --| Algorithm :
    --|  Let java.lang.Integer.toString(The_Integer) do the work.
    --|  
    --+-------------------------------------------------------

    begin
	if The_Integer < 0 then
   	   return Interfaces.Java.Java_To_Ada_String
		(java.lang.Integer.toString(The_Integer));
	else
	   return "&"(" ", Interfaces.Java.Java_To_Ada_String
			    (java.lang.Integer.toString(The_Integer)));
	end if;

    end RTS_Integer_Image;
    
    ----------------------------------------------------------
    ----------------------------------------------------------
    function RTS_Integer_Wide_Image (
      The_Integer : Integer
        ) return Wide_String 
    is
    begin
      return Interfaces.Java.Java_To_Ada_Wide_String(
                       java.lang.Integer.toString(The_Integer));
    end RTS_Integer_Wide_Image;
    
    ----------------------------------------------------------
    ----------------------------------------------------------
    function RTS_Integer_Value (
      The_Image : String
    ) return Integer
    is
        --| EFFECTS
        --|
        --| Given a string representing an integer, returns the value of
        --| the corresponding integer type ('VALUE). Raises
        --| CONSTRAINT_ERROR if the string is not a syntactically legal
        --| integer literal.
    --+-------------------------------------------------------
    --| Overview :
    --|   returns integer'value of the string.
    --|
    --| Algorithm :
    --|  Skips all leading blanks, HT's are illegal, then calls RTS_Ivalue
    --|  to return an integer value. Scan the remainder of the string (if any)
    --|  only blank are allowed.
    --|
    --+-------------------------------------------------------
        use System.RTS.Unsigned;
        My_Last : natural := 0;
        My_First: natural := 0;
	Value : Integer;
	U_Value : System.RTS.Unsigned.Unsigned_Integer;
	Sign    : Integer;
        
    begin
        
        for i in The_Image'range loop
    	if The_Image(i) /= ' ' then
        if The_Image(i) = Ada.Characters.Latin_1.HT then
    		-- Because of IO, HT is allowed in the lex tables but
    		-- not here.
   		raise Constraint_error;
    	    else
    		My_First := i;
    		exit;
    	    end if;
    	end if;
        end loop;
    	
        if My_First = 0 then
    	return 0;
        end if;
    	
        begin
            System.RTS.Lexer.RTS_lex_String (
			  The_Image(My_First .. The_Image'last), 
    			  System.RTS.Integer_State_Tables.Transitions,
    			  System.RTS.Integer_State_Tables.Mapping,
    			  My_Last);
     exception
    	when Data_Error | End_Error =>
    	    -- Since the lexer is originally part of the IO stuff, it
    	    -- sometimes raises these exceptions, all we need is
    	    -- Constraint_error.
  	    raise Constraint_error;
        end;

    				  
        if My_Last < My_First then
            raise Constraint_error;
        end if;
    	
        for i in My_Last+1..The_Image'last loop
    	-- The only character allowed here is blank.
    	if The_image(i) /= ' ' then
   	    raise Constraint_error;
    	end if;
        end loop;
       
        RTS_Ivalue(The_Image(My_First..My_Last), U_Value, Sign);

	Value := Integer(U_Value);
	if Sign = +1 then
	    if Value < 0 then
                raise Constraint_error;
            end if;
        else
	    if Value > 0 then
		Value := -Value;
            elsif Value /= Integer'First then 
                raise Constraint_error;
	    end if;
        end if;

        return Value;
        
    end RTS_Integer_Value;
    
    ----------------------------------------------------------
    ----------------------------------------------------------
    function RTS_Integer_Wide_Value (
        The_Image : Wide_String
    ) return Integer
    is
        --+-------------------------------------------------------
        --| EFFECTS
        --|
        --| Given a string representing an integer, returns the value of
        --| the corresponding integer type ('WIDE_VALUE). Raises
        --| CONSTRAINT_ERROR if the string is not a syntactically legal
        --| integer literal.
        --+-------------------------------------------------------
    begin
        return RTS_Integer_Value(Ada.Characters.Handling.To_String(The_Image));
    end RTS_Integer_Wide_Value;
    
    ----------------------------------------------------------
    ----------------------------------------------------------
   
    function RTS_Unsigned_Image (
      The_Unsigned : System.RTS.Unsigned.Unsigned_Integer
    ) return String is
	long_integer_rep : long_integer;
    --+-------------------------------------------------------
    --| Overview:
    --|    Computes Unsigned'image (The_Unsigned) 
    --|     1. Decimal
    --|     2. Leading blank
    --|     3. return string'first == 1
    --| Algorithm :
    --| TBD: get ArithmeticException for constant unsigned > integer'last.
    --| TBD: We should be  able to simply call
    --|    return  Interfaces.Java.Java_To_Ada_String(
    --|		java.lang.Long.toString(long_integer(The_Unsigned)));
    --|   However, long_integer(the_unsigned) does an integer to long
    --|	  integer conversion, so we must compensate here.
    --| 
    --+-------------------------------------------------------

    begin
	long_integer_rep := long_integer(The_Unsigned);
	if long_integer_rep >=0 then
	   return Interfaces.Java.Java_To_Ada_String(
		     java.lang.Integer.toString(Integer(The_Unsigned)));
	else 
 	   return  Interfaces.Java.Java_To_Ada_String(
		java.lang.Long.toString(long_integer_rep +
		long_integer(4294967296)));
--		java.lang.Long.toString(long_integer(The_Unsigned)));
        end if;
    end RTS_Unsigned_Image;
 
    function RTS_Unsigned_Wide_Image (
      The_Unsigned : System.RTS.Unsigned.Unsigned_Integer
    ) return Wide_String is
    begin
 	   return  Interfaces.Java.Java_To_Ada_Wide_String(
		java.lang.Long.toString(long_integer(The_Unsigned)));
    end RTS_Unsigned_Wide_Image;

    function RTS_Unsigned_Value (
      The_Image : String
    ) return System.RTS.Unsigned.Unsigned_Integer is
	use System.RTS.Unsigned;
        LL : natural := 0;
        FF : natural := 0;
        Result : Unsigned_Integer;
	Sign : Integer;
    begin

        for i in The_Image'range loop
            if The_Image(i) /= ' ' then
                if The_Image(i) = Ada.Characters.Latin_1.HT then
                -- Because of IO, HT is allowed in the lex tables but
                -- not here.
	            raise Constraint_error;
                else
                    FF := i;
                    exit;
                end if;
            end if;
        end loop;

        if FF = 0 then
            return 0;
        end if;

        begin
            System.RTS.Lexer.RTS_lex_String (
				The_image(FF..The_Image'last),
                                System.RTS.Integer_State_Tables.Transitions,
                                System.RTS.Integer_State_Tables.Mapping,
                                LL);
        exception
            when Data_Error | End_Error =>
            -- Since the lexer is originally part of the IO stuff, it
            -- sometimes raises these exceptions, all we need is
            -- Constraint_error.
            raise Constraint_error;
        end;

        if LL < FF then
            raise Constraint_error;
        end if;

        for i in LL+1..The_Image'last loop
            -- The only character allowed here is blank.
            if The_image(i) /= ' ' then
                raise Constraint_error;
            end if;
        end loop;

	RTS_Ivalue(The_Image(FF..LL),Result,Sign);

        return Result;

    end RTS_Unsigned_Value;

    function RTS_Unsigned_Wide_Value (
      The_Image : Wide_String
    ) return System.RTS.Unsigned.Unsigned_Integer is
    begin
       return RTS_Unsigned_Value(Ada.Characters.Handling.To_String(The_Image));
    end RTS_Unsigned_Wide_Value;

    ----------------------------------------------------------
    ----------------------------------------------------------
    function RTS_Integer_Width
    (
      First_Position   : Integer;
      Last_Position    : Integer
    )
    return Integer
    is
    --+-------------------------------------------------------
    --| Overview :
    --| Return maximum length of the image of the integers
    --| between First_Position and Last_Position.
    --| Only called if width is not statically known.
    --|  Note that as defined in LRM 3.5.5, if range is null, answer is zero.
    --|  
    --| Algorithm
    --|   check for null range and return zero, else
    --|   find the one with the largest number of digits, i.e.,
    --|      max Log<10> (value) + 1 <for sign>.
    --|            (define Log<10>(0) == 1)
    --+-------------------------------------------------------

      function Log_10 (i:integer)
        return integer
      --+-----------------------------------------------------
      --| Overview
      --|  Returns Log10(i)+1; where log10(0)==1.
      --|  and Log10(-i)==Log10(i).
      --|  Safely handles case where i = integer'first
      is
          Numerator : integer := i;  -- Starts as i,
    			             -- gets divided by 10 in loop, counting
    			             -- number of trips around the loop
      begin
        for Result in 2 .. 100 loop
    	  Numerator := Numerator / 10;
    	  if Numerator = 0 then
    	    return Result;
    	  end if;
        end loop;
      end Log_10;
    			    
      function max (a, b: integer) return integer;
      PRAGMA INLINE (max);
    
      function max (a, b: integer)
        return integer
      is
      begin
    	if a >= b then
    	  return a;
    	else
    	  return b;
    	end if;
      end max;
        
    begin
      if First_Position > Last_Position then
    	return 0;
      else
    	return max (log_10(First_Position), log_10(Last_Position));
      end if;
    end RTS_Integer_Width;

    function RTS_Unsigned_Width (
      Position  : System.RTS.Unsigned.Unsigned_Integer
    )
    return Integer
    is
      Numerator : System.RTS.Unsigned.Unsigned_Integer := Position;
    begin
      for Result in 2 .. 100 loop
	Numerator := System.RTS.Unsigned."/"(Numerator, 10);
        if System.RTS.Unsigned."="(Numerator, 0) then
	  return Result;
        end if;
      end loop;
    end RTS_Unsigned_Width;


end System.RTS.Integer_Image_And_Value_Pkg;
