-- $Source: /home/dsd/d/aa/monoBANK/AA/j_system.rts.tgt.char_io.bdy,v $
-- $Revision: 1.4 $ $Date: 96/10/14 18:45:34 $ $Author: stt $

with java.io.File; use java.io.File;
with java.lang.System; use java.lang.System;
with java.io.InputStream; use java.io.InputStream;
with java.io.PrintStream; use java.io.PrintStream;
with java.io.RandomAccessFile; use java.io.RandomAccessFile;
with interfaces.java; use interfaces.java;
with java.lang.String; use java.lang.String;

package body System.RTS.TGT.Char_IO is      --| IO operations underlying Text_IO

    package java renames Standard.Java;
      -- "java" is hidden by enumeral in package System called "java".

    pragma suppress( elaboration_check );

    --| OVERVIEW
    --| This package specifies the IO operations underlying Text_IO.
    --| It calls routines which act upon the actual external files,
    --| and also manipulates the file types which are used to represent
    --| the external files.  It also takes care of input and output buffering.
    -- file management
    
    type File_Record is
       record
	Java_File : RandomAccessFile_Ptr;
	Input : InputStream_Ptr;
	Output : PrintStream_Ptr;
	File : File_Ptr;	
	Mode : File_Mode;
	Java_Mode : String_Ptr;

       end record;

    ------------------------------------------------------------------------
    
    procedure CREATE (
        FILE     : in out FILE_TYPE;    --| File to be associated with
					--|   new external file
        MODE     : in FILE_MODE := OUT_FILE;
					--| Mode of new file
        NAME     : in STRING := "";     --| Name of new file
        FORM     : in STRING := ""      --| Form of new file
        ) is
     begin
	-- Makes sure file is not already open
	if File /= null then
	    raise Status_Error;
	end if;

      -- Establish the new File_Record
	File := new File_Record;

	--  TBD:
	-- If Name is not given then create a temp file 




	File.File := New_File(+Name);

	-- Set Java_Mode variable accordIng to MODE:  
	--     r = read only mode
	--     rw = read and write mode
	-- Defaults to Out_File.
	if Mode = In_File then
	    File.Java_Mode := +"r";
	elsif Mode = Out_File then
	    File.Java_Mode := +"rw";     -- note:  does not have capability to
				   --        open file for write only
	elsif Mode = Append_File then
	    File.Java_Mode := +"rw";
	else
	    File.Java_Mode := +"rw";
	end if;

	File.Java_File := New_RandomAccessFile(File.File, File.Java_Mode);
	
	File.Mode := Mode;
	
	-- If file could not be opened in the given mode, raise Use_Error
    
     exception 
	when java.io.IOException =>
	    raise Device_Error;
	when java.io.FileNotFoundException =>
	    raise Name_Error;
	when others =>
	    raise Use_Error;


	-- !! form FORM is not handled
	-- tbd: temp files

     end Create;
    
        --| OVERVIEW
        --| This procedure associates the given file type FILE with an
	--| external file having the given name NAME and form FORM, and
	--| sets the mode of the external file to the given mode
	--| MODE.  The file is left open.  If no name is given, a temporary
	--| file is created. If no mode is given, the default mode is OUT_FILE.
        --| This routine requires that the given file is not already open.
        --| If it is already open, STATUS_ERROR is raised.  If the string
        --| given as name does not allow the identification of an external
        --| file, NAME_ERROR is raised.  If the environment does not support
        --| creation of an external file with the specified name and mode,
        --| USE_ERROR is raised.
    
    
    ------------------------------------------------------------------------
    
    procedure OPEN (
        FILE     : in out FILE_TYPE;    --| File to be associated with the
                                        --|   external file.
        MODE     : in FILE_MODE;        --| Mode of external file.
        NAME     : in STRING;           --| Name of external file.
        FORM     : in STRING := ""      --| Form of external file.
        ) is
     begin
	if File /= null then
	    raise Status_Error;
	end if;

	File := new File_Record;
	File.File := New_File(+Name);
	if not Exists(File.File) then
	    raise Name_Error;
	end if;
	File.Mode := Mode;
	if File.Mode = In_File then
	    File.Java_Mode := +"r";
	else
	    File.Java_Mode := +"rw";
	end if;
	File.Java_File  := New_RandomAccessFile(File.File, File.Java_Mode);
	if Mode = Append_File then
	    seek(File.Java_File, length(File.Java_File));
	end if;

	-- If file could not be opened in the given mode, raise Use_Error

     exception 
	when java.io.IOException =>
	    raise Device_Error;
	when java.io.FileNotFoundException =>
	    raise Name_Error;
	when Constraint_Error =>
	    raise Use_Error;
	when java.io.EOFException =>
	    raise Use_Error;
	when Program_Error =>
	    raise Use_Error;

     end Open;
    
        --| OVERVIEW
        --| This procedure associates the given file type FILE with an
	--| existing external file having the given name NAME and form
	--| FORM, and sets the mode of the external file to the given mode
	--| MODE.  The file is left open. This procedure requires that the
	--| file is not already open.  If it is, STATUS_ERROR is raised.
	--| If the string given as name does not allow the identification
	--| of an external file, NAME_ERROR is raised.  If the environment
	--| does not support the opening of an external file with the given
	--| name and mode, USE_ERROR is raised.
    
    ------------------------------------------------------------------------
    
    procedure CLOSE (
        FILE     : in out FILE_TYPE     --| File associated with external file
        ) is
      begin
	if File = null then
	    raise Status_Error;
	end if;

	Close(File.Java_File);
	File := null;	

	-- tbd:  
	-- if  TEMP then
	--     delete(File.File);
	-- ?? is this necessary?   close(File.??);
	-- end if;    

     end Close;
    
        --| OVERVIEW
        --| The given file type FILE is disassociated from its associated
	--| external file.  The external file is left closed, and if it
	--| is a temporary file, it is deleted.
        --| This procedure requires that the given file is open.  If it is
	--| not, STATUS_ERROR is raised.
    
    ------------------------------------------------------------------------
    
    procedure DELETE (
        FILE     : in out FILE_TYPE     --| File associated with external file
                                        --|   to be deleted
        ) is
     begin
	if File = null then
	    raise Status_Error;
	end if;

	if not delete(File.File) then
	   raise Use_Error;
	end if;
	close(File);
     end Delete;
    
        --| OVERVIEW
        --| Deletes the external file associated with the given file type
	--| FILE.  If the given file is not open, STATUS_ERROR is raised.
	--| If the environment does not support deletion of the external file
        --| associated with the given file, USE_ERROR is raised.
    
    ------------------------------------------------------------------------
    
    procedure RESET (
        FILE     : in out FILE_TYPE;    --| File associated with external file
                                        --|   which is to be reset.
        MODE     : in FILE_MODE         --| Mode which the file's current mode
                                        --|   is to be changed to.
        ) is
     begin
	if File = null then
	    raise Status_Error;
	end if;

	Close(File.Java_File);

	File.Mode := Mode;

	-- Set Java_Mode variable according to MODE:  
	--     r = read only mode
	--     rw = read and write mode
	-- Defaults to Out_File.
	if Mode = In_File then
	    File.Java_Mode := +"r";
	elsif Mode = Out_File then
	    File.Java_Mode := +"rw";     -- note:  does not have capability to
				   --        open file for write only
	elsif Mode = Append_File then
	    File.Java_Mode := +"rw";
	else
	    File.Java_Mode := +"rw";
	end if;
			
	File.Java_File := New_RandomAccessFile(File.File, File.Java_Mode);

    exception 
	when java.io.IOException =>
	    raise Device_Error;
	when java.io.FileNotFoundException =>
	    raise Name_Error;
	when others =>
	    raise Use_Error;

     end Reset;
    
        --| OVERVIEW
        --| The current position in the external file associated with
	--| the file type FILE is reset to the beginning of the file,
	--| and the mode of the external file is changed to the given mode
	--| MODE.  If the file is not open, STATUS_ERROR is raised.
	--| If the mode of file is unable to be changed, or the current
	--| position in the file is unable to be reset to the beginning
	--| of the file, USE_ERROR is raised.
    
    --------------------------------------------------------------------
    
    function  NAME (
        FILE     : in FILE_TYPE         --| File associated with external file
                                        --|   whose name is to be returned.
        ) return STRING is              --| Name of external file.
     begin
	if File = null then
	    raise Status_Error;
	end if;

	return java_to_ada_string(getAbsolutePath(File.File));
     end Name;
    
        --| OVERVIEW
        --| The name of the external file associated with the given file is
        --| returned.  If the given file is not open, STATUS_ERROR is raised.
    
    ----------------------------------------------------------------------
    
    function  FORM (
        FILE     : in FILE_TYPE         --| File associated with external file
                                        --|   whose form is to be returned.
        ) return STRING is              --| Form of external file.
     begin
 	return "";	
     end Form;
        
        --| OVERVIEW
        --| The form string of the external file associated with the given
	--| file is returned.  If the given file is not open, STATUS_ERROR
	--| is raised.
    
    ----------------------------------------------------------------------
    
    procedure OPEN_STANDARD_INPUT (
        FILE     : in out FILE_TYPE     --| File which is to be associated
                                        --|   with external file.
        ) is
     begin
	if File /= null then
	    raise Status_Error;
	end if;

	File := new File_Record;
	File.Mode := In_File;
	File.File := New_File(+"<Standard Input>");
	File.Input := java.lang.System.Stdin;
	
     end Open_Standard_Input;
    
        --| OVERVIEW
        --| The given file is associated with the external file representing
        --| terminal input.  If the file is already open, STATUS_ERROR 
	--| is raised.
    
    --------------------------------------------------------------------

    procedure OPEN_STANDARD_OUTPUT (
        FILE     : in out FILE_TYPE     --| File to be associated with
                                        --|   external file.
        ) is
      begin
 	if File /= null then
	    raise Status_Error;
	end if;

	File := new File_Record;
	File.Mode := Out_File;
	File.File := New_File(+"<Standard Output>");
	File.Output := java.lang.System.Stdout;

     end Open_Standard_Output;

        --| OVERVIEW
        --| Associates a file with the external file representing terminal
        --| output.  STATUS_ERROR is raised if the given file is already open.
    
    --------------------------------------------------------------------

    procedure OPEN_STANDARD_ERROR (
        FILE     : in out FILE_TYPE     --| File to be associated with
                                        --|   external file.
        ) is
    begin
 	if File /= null then
	    raise Status_Error;
	end if;

	File := new File_Record;
	File.Mode := Out_File;
	File.File := New_File(+"<Standard Error>");
	File.Output := java.lang.System.Stderr;
     end Open_Standard_Error;

        --| OVERVIEW
        --| Associates a file with the external file representing terminal
        --| error.  STATUS_ERROR is raised if the given file is already open.

    --------------------------------------------------------------------
    
    procedure READ (
        FILE     : in FILE_TYPE;        --| File associated with external file
                                        --|   where element is to be read from.
        ITEM     : out ELEMENT_TYPE     --| Where element is to be put.
        ) is
	Byte : Integer;
     begin
	if File.Input /= null then
	    Byte := Read(File.Input);
	else
	    Byte := Read(File.Java_File);
	end if;

	if Byte < 0 then
	    raise End_Error;
	end if;

	item := Element_Type'Val(Byte);
     end Read;
    
        --| OVERVIEW
        --| The element at the current position in the external file
        --| associated with the given file is read into ITEM, and the
        --| current position in the file is advanced.
        --| If position in external file associated with given file is at end
        --| of file, END_ERROR will be raised.
    
    --------------------------------------------------------------------

    procedure WRITE (
        FILE     : in FILE_TYPE;        --| File associated with external file
                                        --|   which will be written into.
        ITEM     : in ELEMENT_TYPE      --| Element to be written into file.
        ) is
     begin
 	if File = null then
	    raise Status_Error;
	end if;

	if File.Mode /= Out_File then
	    raise Mode_Error;
	end if;

	if File.Output /= null then
	    Write(File.Output, Element_Type'Pos(Item));
	else
	    Write(File.Java_File, Element_Type'Pos(Item));
	end if;

     end Write;    
        --| OVERVIEW
        --| A data element is output to the output buffer associated with the
        --| given file.  If the file is not open, STATUS_ERROR will be raised.
	--| If the mode of the external file associated with the given file
	--| is not Out_File, MODE_ERROR will be raised.
    
    
    --------------------------------------------------------------------
    
    procedure WRITE(
        FILE     : in FILE_TYPE;        --| File associated with external file
                                        --|   which is to be written to.
        ITEM     : in STRING            --| String which is to be written to
                                        --|   file.
        ) is
     begin
 	if File = null then
	    raise Status_Error;
	end if;

	if File.Output /= null then
	     Write(File.Output, Item);
	else
	     Write(File.Java_File, Item);
	end if;

     end Write;    
    
        --| OVERVIEW
        --| A string is output to the output buffer associated with the given
        --| file.  If the file is not open, STATUS_ERROR is raised.
	--| If the mode of the external file associated with the given
	--| file is not Out_File, MODE_ERROR is raised.
    
    --------------------------------------------------------------------
    
    function  END_OF_FILE (
        FILE     : in FILE_TYPE         --| File associated with external file
                                        --|   where end of file is going to be
                                        --|   checked.
        ) return BOOLEAN is             --| TRUE if at end of file,
    begin
 	if File = null then
	    raise Status_Error;
	end if;

	if getFilePointer(File.Java_File) >= length(File.Java_File) then
	    return true;
	else
	    return false;
	end if;

    end;
					--| FALSE otherwise.
    
        --| OVERVIEW
        --| This file return TRUE if the external file associated with the
        --| given file is at end of file, and FALSE otherwise.
    
    ----------------------------------------------------------------------
    
    procedure Get_Line (
        FILE     : in FILE_TYPE;        --| File associated with external file
                                        --|   to be read from.
        ITEM     : out string;          --| Buffer to be filled
        LAST     : out natural          --| Last index read
        ) is
       LOCAL_LAST : NATURAL := ITEM'FIRST-1;
       C : CHARACTER;
    begin
 	if File = null then
	    raise Status_Error;
	end if;

	if File.Mode /= In_File then
	    raise Mode_Error;
	end if;

       while Local_last<Item'Last loop
          READ (File, C);
          exit when C=ascii.lf;
          LOCAL_LAST := LOCAL_LAST + 1;
          Item (Local_Last) := C;
       end loop;
       Last := Local_Last;
    end Get_Line;

        --| OVERVIEW
        --| The external file associated with the given file is read
        --| up to and including the carriage return character or until
        --| range of string ITEM is exhausted.  The current position in
        --| the file is advanced.  LAST is the position in ITEM
        --| where the last character read is stored.
        --| If the file is not open, STATUS_ERROR will be raised.
        --| If the mode of the external file associated with the given file
        --| is not IN_FILE, MODE_ERROR will be raised.
 
    
    ----------------------------------------------------------------------

    procedure FLUSH (
        FILE     : in File_Type         --| File whose output buffer is to be
                                        --|   flushed.
        ) is
    begin
       null;
    end;


        --| OVERVIEW
        --| The output buffer associated with the given file is written out
        --| to the external file associated with the given file.
        --| If all the elements in the output buffer are unable to be written
        --| to the external file associated with the given file, DATA_ERROR is
        --| raised.
    
    --------------------------------------------------------------------
    

    procedure Get_Immediate(
        FILE      : in FILE_TYPE;        --| File associated with external file
                                         --|   to be read from.
        WAIT      : in boolean;          --| Wait or No_Wait
        ITEM      : out character;       --| character to be filled
        AVAILABLE : out boolean          --| if Item is available when
                                         --|   when No_Wait
        ) is
    begin
	read(File, Item);
	Available := true;

	-- ignores concept of wait
    end Get_Immediate;

        --| OVERVIEW
        --| Reads the next character, wither control or graphic, from
        --| the specified FILE.  FILE is assumed to be In_File and the
        --| current location is assumed to be not at the end of the file.
        --| The procedure, if the value of WAIT is false, will return with
        --| AVAILABLE set to true and ITEM containing the character read or
        --| AVAILABLE set to false and ITEM unspecified.  If the value of
        --| WAIT is true then this procedure will wait until the ITEM is
        --| read and AVAILABLE is meaningless.

    ----------------------------------------------------------------------

end System.RTS.TGT.Char_IO;











