-- $Source: /home/dsd/d/aa/monoBANK/AA/j_system.rts.enumeration_image_and_value_pkg.bdy,v $
-- $Revision: 1.3 $ $Date: 96/10/25 08:51:14 $ $Author: stt $
--        Copyright (C) 1995 by Intermetrics, Inc.   

-- Derived from:
--  Source: /home/dsd/d/aa/monoBANK/AA/system.rts.enumeration_image_and_value_pkg.bdy,v  
--  Revision: 1.6    Date: 96/08/15 11:56:30    Author: akira  

with Ada.Characters.Handling;
with Ada.Characters.Latin_1;

package body System.RTS.Enumeration_Image_And_Value_Pkg is

    pragma Suppress( Elaboration_Check );

-- WARNING: the subprogram declaration format is important, since this
-- file is read used with the rts calls from the emitter.
-- see intrinsic_rts.c


  procedure Normalize_String( S : in out String;
      	    	    	      F :    out Positive;
			      L :    out Positive );

-- Note: The basis of this code is taken from the aie host_rts _type
-- subdirectory:
-- Source: /home/inmet/m/ada/monoBANK/RTS/RTS_IV.bdy,v
-- Revision: 1.37  -- Date: 88/06/13 16:34:04  -- Author: callen

    --------------------------------------------------------------------
    -- Enumeration-based Routines
    --------------------------------------------------------------------

    function RTS_Enumeration_Image (
      Enum_Position    : Integer;
      Enum_Index_Table : Enum_Integer_Array_Type;
      Enum_Image_Table : Enum_Character_Array_Type
    ) return String is
    --+-------------------------------------------------------
    --| Overview :
    --|
    --| Given the position number of an enumeral, return its
    --| normalized spelling (i.e. upper case unless char enumeral).
    --|
    --| EFFECTS
    --|
    --| Returns the image ('IMAGE) of an enumeration value in upper
    --| case.
    --+-------------------------------------------------------

	First_Char, Last_Char : Integer;
    begin
	--| Algorithm
	--| Determine appropriate slice of enum-image-table
	--| using enum-index-table, as follows:
	--| 'first is enum-index-table(enum-pos - 1) + 1
	--| 'last  is enum-index-table(enum-pos)
	--| Except if enum-pos is = 0 --> 'first = enum-image-table'first
	--|
	--| According to LRM 3.5.5 (depending on how you read it)
	--| low bound should be 1, hence we must convert 
	--| slice to get appropriate bounds.

        if (Enum_Index_Table'Length = 0 ) and
          then (Enum_Image_Table'Length = 0) then
                  return "";
        end if;

	if Enum_Position = Enum_Index_Table'First then
	    First_Char := Enum_Image_Table'First;
	else
	    First_Char := Enum_Index_Table(Enum_Position - 1) + 1;
	end if;

	Last_Char  := Enum_Index_Table(Enum_Position);	
	    
	declare
	    subtype RetStr is String( 1 .. Last_Char - First_Char + 1 );
	    result : RetStr;
	begin
	    result:= RetStr(Enum_Image_Table(First_Char .. Last_Char ));
	    return result;
	end;

    end RTS_Enumeration_Image ;


    function RTS_Enumeration_Value (
    	Enum_Image       : String;
    	Enum_Index_Table : Enum_Integer_Array_Type;
    	Enum_Image_Table : Enum_Character_Array_Type  )
    return Natural is
    	My_First    : Natural;
    	My_Last     : Natural;
    	Image_First : Natural;
	My_Length_Minus_1   : Natural;
	Str : String := Enum_Image;
	First_Char : Character;
    begin
    	--
    	-- remove leading and following spaces
    	--
	Normalize_String( Str, My_First, My_Last );

    	My_Length_Minus_1 := My_Last - My_First;

	First_Char := Str(My_First);
	
    	-- Enum string is in Enum_Image(My_First..My_Last)
    	-- now find string in enum_array

    	Image_First := 1;

    	for I in Enum_Index_Table'RANGE loop
	    --
	    -- if the sizes and the first char are the same then 
	    -- check the string contents
	    --
	    if Enum_Index_Table(I) - Image_First = My_Length_Minus_1 and then
	      Enum_Image_Table(Image_First) = First_Char then
   	    	if Enum_Image_Table(Image_First .. Enum_Index_Table(I)) =
		  Str( My_First .. My_Last ) then
	    	   return I;
		end if;
	    end if;
    	    	
	    Image_First := Enum_Index_Table(I) + 1;

    	end loop;

    	raise Constraint_Error;

    end RTS_Enumeration_Value;


    function RTS_Enumeration_Width (
    	    First_Pos        : Natural;
    	    Last_Pos         : Natural;
    	    Enum_Index_Table : Enum_Integer_Array_Type
         ) return Natural is
    	Max_Width   : Natural := 0;
	Prev_Index : Natural;
	
    begin
	    
	if First_Pos = Enum_Index_Table'FIRST then
	    Prev_Index := 0;
	else
	    Prev_Index := Enum_Index_Table(First_Pos-1);
	end if;
	    
	for I in First_Pos .. Last_Pos loop
	
	    if Enum_Index_Table( I ) - Prev_Index > Max_Width then
		Max_Width := Enum_Index_Table( I ) - Prev_Index;
	    end if;

	    Prev_Index := Enum_Index_Table( I );
	    
	end loop;
	    
	return Max_Width;
	    
    end RTS_Enumeration_Width;	


----------------------------------------------------------------------------
-- The following three functions lookup values in enumeration representation
-- tables.  RTS_Get_Enum_Val_Position does the searching and it is used by
-- RTS_Enumeration_Value_In_Table (for 'Valid) and RTS_Holey_Enumeration_Pos
-- (for 'Pos).
----------------------------------------------------------------------------

function RTS_Get_Enum_Val_Position (
             Enum_Val         : In  Integer;
	     Enum_Holey_Table : In  Enum_Holey_Array_Type
         ) return Integer is
	 
    --| OVERVIEW:
    --| This function is used by RTS_Holey_Enumeration_Pos and 
    --| RTS_Enumeration_Value_In_Table to lookup the given enumeration value 
    --| in the given enumeration representation table.  If the value is
    --| found then its position in the table is returned, otherwise -1 
    --| is return.
    
    --| ALGORITHM:
    --| Search slices of the Enum_Holey_Table by cutting in half successively,
    --| switching focus to the left or right half depending
    --| on the relation between the wanted value and the
    --| value at the mid point of the slice.
    
    --| NOTES:
    --| Enum_Holey_Table is in increasing order because "the integer codes
    --| must satisfy the predefined ordering relation" -- LRM 13.3:4.
    
    --| Pos of first rep is always zero -- LRM 3.5.1:4.
    
    --| Worst case performance is O(log2(Enum_Holey_Table'length)) thanks
    --| to binary search algorithm.

    Low                 : Natural := Enum_Holey_Table'First;
	    	    	--| Low index in slice of interest

    High                : Natural := Enum_Holey_Table'Last;
	    	    	--| High index in slice of interest
			
    Mid                 : Natural;
	    	    	--| Mid point in slice of interest
			
    Mid_Point_Enum_Val  : Integer;
	    	        --| Rep stored at mid point.

begin
    while Low <= High loop
    
        -- Find mid point between Low and High
    	Mid := (Low + High)/2;
    	Mid_Point_Enum_Val := Enum_Holey_Table(Mid);
	
    	if Enum_Val = Mid_Point_Enum_Val then
       	    -- Found the element of interest;
            -- Return its Pos
            -- (Pos of first rep is always zero LRM 3.5.1:4)
            return (Mid - Enum_Holey_Table'First);
        elsif Enum_Val < Mid_Point_Enum_Val then
            -- Desired value is in left half of slice
            High := Mid - 1;
        else
            -- Desired value is in right half of slice
            Low := Mid + 1;
         end if;
	 
    end loop;
    
    -- Not found in table.  
    return -1;
		  
end RTS_Get_Enum_Val_Position;

function RTS_Enumeration_Value_In_Table (
             Enum_Val         : Integer;
             Enum_Holey_Table : Enum_Holey_Array_Type) return Boolean is
begin	
    --| Return True if the Enum_Val in found in Enum_Holey_Table, otherwise
    --| return False
    return RTS_Get_Enum_Val_Position(Enum_Val, Enum_Holey_Table) /= -1;
end RTS_Enumeration_Value_In_Table;


function RTS_Holey_Enumeration_Pos (
             Enum_Val         : Integer;
	     Enum_Holey_Table : Enum_Holey_Array_Type
         ) return Natural is
	 
    --| Lookup the position of the Enum_Val in Enum_Holey_Table.
    --| if the value is not found, raise constraint error,
    --| otherwise return the value.
    
    Enum_Position : Integer;
    
begin
    Enum_Position := RTS_Get_Enum_Val_Position(Enum_Val, Enum_Holey_Table);
    if  Enum_Position /= -1 then
        return Enum_Position;
    else
        -- Not found in table.  Something is erroneous out there
    	raise Program_Error;
    end if;
    
    exception
        when others =>
    	    -- Something is erroneous
    	    raise Program_Error;
	    
end RTS_Holey_Enumeration_Pos;
     
     
--------------------------------------------------------------------
-- Character-based Routines
--------------------------------------------------------------------
    Char_Image_Array_00_To_1F : constant
      Enum_Character_Array_Type :=
	"NULSOHSTXETXEOTENQACKBELBSHTLFVTFFCRSOSI" &
	"DLEDC1DC2DC3DC4NAKSYNETBCANEMSUBESCFSGSRSUS";

    Char_Image_Index_Array_00_To_1F : constant
	Enum_Integer_Array_Type(0 .. 16#1F#) :=
      (
	3, 6, 9, 12, 15, 18, 21, 24,
	26, 28, 30, 32, 34, 36, 38, 40,
	43, 46, 49, 52, 55, 58, 61, 64,
	67, 69, 72, 75, 77, 79, 81, 83
      );

    Char_Image_Array_7F_To_9F : constant
      Enum_Character_Array_Type :=
	"DEL" &
	"RESERVED_128RESERVED_129BPHNBHRESERVED_132NELSSAESA" &
	"HTSHTJVTSPLDPLURISS2SS3" &
	"DCSPU1PU2STSCCHMWSPAEPA" &
	"SOSRESERVED_153SCICSISTOSCPMAPC";


    Char_Image_Index_Array_7F_To_9F : constant
	Enum_Integer_Array_Type(16#7F# .. 16#9F#) :=
      (
	3,
	15, 27, 30, 33, 45, 48, 51, 54,
	57, 60, 63, 66, 69, 71, 74, 77,
	80, 83, 86, 89, 92, 94, 97, 100,
	103, 115, 118, 121, 123, 126, 128, 131
      );


    function RTS_Character_Image( Char : Character ) return String is
    begin
	case Character'Pos(Char) is
	    when 0 .. 16#1F# =>
		return RTS_Enumeration_Image(Character'Pos(Char),
		    Char_Image_Index_Array_00_To_1F,
		    Char_Image_Array_00_To_1F);
	    when 16#7F# .. 16#9F# =>
		return RTS_Enumeration_Image(Character'Pos(Char),
		    Char_Image_Index_Array_7F_To_9F,
		    Char_Image_Array_7F_To_9F);
	    when others =>
		return ''' & Char & ''';
	end case;
    end RTS_Character_Image;

    function RTS_Character_Value( Char_Image : String ) return Character is
    	My_First : Natural;
	My_Last  : Natural;
	Str : String := Char_Image;
	Image_First : Integer;
    begin

    	--
    	-- remove leading and following spaces
    	--
	Normalize_String( Str, My_First, My_Last );
	
      	--  Accept any single non-control character enclosed in quotes

      	if My_Last - My_First = 2 and then Str( My_First ) = '''
		and then Str( My_Last ) = ''' then
	    declare
		Char : constant Character := Str(My_First + 1);
	    begin
		-- Check if is a control character
		case Character'Pos(Char) is
		    when 0 .. 16#1F# | 16#7F# .. 16#9F# =>
			raise Constraint_Error;
		    when others =>
			return Char;
		end case;
	    end;

	end if;

	--  Check control character images
	Image_First := 1;
	for C in Char_Image_Index_Array_00_To_1F'Range loop
	    if Str(My_First .. My_Last) = 
	      Char_Image_Array_00_To_1F(
	       Image_First .. Char_Image_Index_Array_00_To_1F(C)) then
		return Character'Val(C);
	    end if;
	    Image_First := Char_Image_Index_Array_00_To_1F(C) + 1;
	end loop;

	Image_First := 1;
	for C in Char_Image_Index_Array_7F_To_9F'Range loop
	    if Str(My_First .. My_Last) = 
	      Char_Image_Array_7F_To_9F(
	       Image_First .. Char_Image_Index_Array_7F_To_9F(C)) then
		return Character'Val(C);
	    end if;
	    Image_First := Char_Image_Index_Array_7F_To_9F(C) + 1;
	end loop;

	-- Illegal image
	raise Constraint_Error;

    end RTS_Character_Value;

    function RTS_Character_Width ( First_Pos : Character;
    	    	    	    	   Last_Pos  : Character ) return Natural is
	Max_Width : Integer := 0;
    begin
	-- Just do the obvious (and slow!) thing
	-- This could be more clever, but who cares?
	for C in First_Pos .. Last_Pos loop
	    declare
		Len : constant Integer := 
		  RTS_Character_Image(C)'Length;
	    begin
		if Len > Max_Width then
		    Max_Width := Len;
		end if;
	    end;
	end loop;
	return Max_Width;
    end RTS_Character_Width;
	

--------------------------------------------------------------------
-- Wide_Character-based Routines
--------------------------------------------------------------------

    Hex : constant string(1..16) := "0123456789ABCDEF";

    function RTS_Wide_Character_Image( Wide_Char : Wide_Character ) 
       return String is

        Dec : integer;
        Res : string(1..4);

    begin
        if Wide_Character'Pos(Wide_Char) < 16#0100# then
            return RTS_Character_Image(Character'Val(
                                          Wide_Character'Pos(Wide_Char)));
        else
	    Dec := Wide_Character'Pos(Wide_Char);
	    if Dec = 16#FFFE# then
		return "FFFE";
            elsif Dec = 16#FFFF# then
		return "FFFF";
            end if;
            for inx in reverse Res'range loop
		Res(inx) := Hex((Dec rem 16) + 1);
		Dec := Dec / 16;
	    end loop;
            return Ada.Characters.Latin_1.HT & Res;
        end if;
    end RTS_Wide_Character_Image;

    function RTS_Wide_Character_Wide_Image( Wide_Char : Wide_Character )
       return Wide_String is
        ws : wide_string(1..1);
    begin
	return Ada.Characters.Handling.To_Wide_String(
		   RTS_Wide_Character_Image(Wide_Char));
    end RTS_Wide_Character_Wide_Image;

    function RTS_Wide_Character_Value( Wide_Char_Image : String ) 
       return Wide_Character is
        My_First : Natural;
        My_Last  : Natural;
        Str : String := Wide_Char_Image;
        Char : Character;
        Val : integer;

        function To_Dec (S : string) return integer is
            Res : integer := 0;
        begin
            for inx in S'first .. S'last loop
                Res := Res * 16;
                if character'pos(S(inx)) in character'pos('0') ..
                                            character'pos('9') then
                    Res := Res + (character'pos(S(inx))-character'pos('0'));
                elsif
                   character'pos(S(inx)) in character'pos('a') ..
                                            character'pos('f') then
                    Res := Res + (character'pos(S(inx))-character'pos('a')+10);
                elsif
                   character'pos(S(inx)) in character'pos('A') ..
                                            character'pos('F') then
                    Res := Res + (character'pos(S(inx))-character'pos('A')+10);
                else
                    raise constraint_error;
                end if;
            end loop;
            if Res > 16#FFFF# then
                raise constraint_error;
            end if;
            return Res;
        end To_Dec;

    begin
        --
        -- remove leading and following spaces
        --
        Normalize_String( Str, My_First, My_Last );

        -- check if >= 256 (ie. wide)
        if My_Last - My_First = 4 and then
	   Str(My_First) = Ada.Characters.Latin_1.HT then
	    return Wide_Character'val(
		       To_Dec(Str(My_First + 1 .. My_Last)));
        else
            Char := RTS_Character_Value(Str);
            return Wide_Character'val(Character'pos(Char));
        end if;
    end RTS_Wide_Character_Value;


    function RTS_Wide_Character_Wide_Value( Wide_Char_Image : Wide_String )
       return Wide_Character is
	wc : Wide_Character;
    begin
	return RTS_Wide_Character_Value(
		   Ada.Characters.Handling.To_String(Wide_Char_Image));
    end RTS_Wide_Character_Wide_Value;


    function RTS_Wide_Character_Width ( First_Pos : Wide_Character;
                                        Last_Pos  : Wide_Character )
       return Natural is
        Max_Width : Integer := 0;
	From : Wide_Character;
	To   : Wide_Character;
    begin
	if Wide_Character'Pos(First_Pos) >= 16#0100# then
	    From := '	0100';
	    To   := From;
        else
	    From := First_Pos;
	    if Wide_Character'Pos(Last_Pos) >= 16#0100# then
	        To := '	0100';
            else
		To := Last_Pos;
            end if;
        end if;
        for WC in From .. To loop
            declare
                Len : constant Integer := 
                  RTS_Wide_Character_Image(WC)'Length;
            begin
                if Len > Max_Width then
                    Max_Width := Len;
                end if;
            end;
        end loop;
        return Max_Width;
    end RTS_Wide_Character_Width;


--------------------------------------------------------------------
-- Boolean-based Routines
--------------------------------------------------------------------
    function RTS_Boolean_Image ( Enum_Position : Integer ) return String is
    begin
	if Enum_Position = 0 then
	    return "FALSE";
	else
	    return "TRUE";
	end if;
    end RTS_Boolean_Image;

    function RTS_Boolean_Value ( Boolean_Image : String ) return Boolean is
      My_First : Natural;
      My_Last  : Natural;
      Str : String( Boolean_Image'RANGE ) := Boolean_Image;
    begin

    	--
    	-- remove leading and following spaces
    	--
	Normalize_String( Str, My_First, My_Last );
	
      	if Str (My_First .. My_Last) = "TRUE" then
            return True;
	elsif Str (My_First .. My_Last) = "FALSE" then
            return False;
      	end if;

      	raise Constraint_Error;

    end RTS_Boolean_Value;

    function RTS_Boolean_Width ( First_Pos : Boolean;
    	    	    	    	 Last_Pos  : Boolean ) return Natural is
    begin    				
      if First_Pos > Last_Pos then
         return 0;
      elsif First_Pos = False then
         return 5;
      else
         return 4;
      end if;

    end RTS_Boolean_Width;
-------------------------------------------------------------
-- Local Functions
--------------------------------------------------------------
  procedure Normalize_String( S : in out String;
      	    	    	      F :    out Positive;
			      L :    out Positive ) is
   begin
      F := S'First;
      L := S'Last;

      --  Scan for leading spaces

      while F <= L and then S (F) = ' ' loop
         F := F + 1;
      end loop;

      --  Check for case when the string contained no characters

      if F > L then
         raise Constraint_Error;
      end if;

      --  Scan for trailing spaces

      while S (L) = ' ' loop
         L := L - 1;
      end loop;

      if S (F) /= ''' then

         --  Upper case any lower case characters.

         for J in F .. L loop
	    S(J) := Ada.Characters.Handling.To_Upper(S(J));
         end loop;
      end if;

  end Normalize_String;

end System.RTS.Enumeration_Image_And_Value_Pkg;
