-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.enumeration_image_and_value_pkg.spc,v $
-- $Revision: 1.6 $ $Date: 96/08/14 16:12:30 $ $Author: akira $
--        Copyright (C) 1995 by Intermetrics, Inc.   

package System.RTS.Enumeration_Image_And_Value_Pkg is

    pragma Suppress( Elaboration_Check );

    subtype Enum_Character_Array_Type is String;

    type Enum_Integer_Array_Type is array (Natural range <>) of Positive;
       -- index of last char position for i'th enumeral

    type Enum_Holey_Array_Type is array (Natural range <>) of Integer;
       -- index of last char position for i'th enumeral



--------------------------------------------------------------------
-- Enumeration-based Routines
--------------------------------------------------------------------

    function RTS_Enumeration_Image (
      Enum_Position    : Integer;
      Enum_Index_Table : Enum_Integer_Array_Type;
      Enum_Image_Table : Enum_Character_Array_Type
    ) return String;

    pragma export (Ada, 
		   RTS_Enumeration_Image, 
                   External_Name => "rts_enumeration_image"); 
    pragma global (RTS_Enumeration_Image); 

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the image ('IMAGE) of an enumeration value in upper
    --| case.
    --|
    --+-------------------------------------------------------

    function RTS_Enumeration_Value (
	Enum_Image       : String;
    	Enum_Index_Table : Enum_Integer_Array_Type;
    	Enum_Image_Table : Enum_Character_Array_Type
    ) return Natural;

    pragma export (Ada,
		   RTS_Enumeration_Value, 
                   External_Name => "rts_enumeration_value"); 
    pragma global (RTS_Enumeration_Value); 

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| If the sequence of characters has the syntax of an enumeration
    --| literal and if this literal exists for the base type of T,
    --| returns the corresponding enumeration value ('VALUE).
    --| Otherwise, CONSTRAINT_ERROR is raised.
    --+-------------------------------------------------------

      function RTS_Enumeration_Width (
	First_Pos        : Natural;
	Last_Pos         : Natural;
	Enum_Index_Table : Enum_Integer_Array_Type
      ) return Natural;

    pragma export (Ada,
		   RTS_Enumeration_Width, 
                   External_Name => "rts_enumeration_width");
    pragma global (RTS_Enumeration_Width); 

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the width ('WIDTH) of an enumeration value.
    --|
    --+-------------------------------------------------------

     function RTS_Holey_Enumeration_Pos (
         	Enum_Val         : Integer;
	       	Enum_Holey_Table : Enum_Holey_Array_Type
     ) return Natural;

    pragma Export( Ada,
		   RTS_Holey_Enumeration_Pos,
       	    	   "rts_holey_enumeration_pos"); 
    pragma Global( rts_holey_enumeration_pos); 
    
     function RTS_Enumeration_Value_In_Table (
         	Enum_Val         : Integer;
	       	Enum_Holey_Table : Enum_Holey_Array_Type
     ) return Boolean;

    pragma Export(Ada,
		  RTS_Enumeration_Value_In_Table,
       	    	  "rts_enumeration_value_in_table"); 
		  
    pragma Global(rts_enumeration_value_in_table); 

--------------------------------------------------------------------
-- Character-based Routines
--------------------------------------------------------------------

    function RTS_Character_Image( Char : Character ) return String;
    
    pragma export (Ada,
		   RTS_Character_Image,
                   External_Name => "rts_character_image");
    pragma global (RTS_Character_Image);

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the image ('IMAGE) of a character enumeration value
    --| in upper case.
    --|
    --+-------------------------------------------------------
    
    function RTS_Character_Value( Char_Image : String ) return Character;
    
    pragma export (Ada,
		   RTS_Character_Value,
                   External_Name => "rts_character_value");
    pragma global (RTS_Character_Value);

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the value ('VALUE) of a character
    --|
    --+-------------------------------------------------------
    
    function RTS_Character_Width ( First_Pos : Character;
    	    	    	    	 Last_Pos  : Character ) return Natural;

    pragma export (Ada,
		   RTS_Character_Width,
                   External_Name => "rts_character_width");
    pragma global (RTS_Character_Width);

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the width ('WIDTH) of a Character enumeration value.
    --|
    --+-------------------------------------------------------


--------------------------------------------------------------------
-- Wide_Character-based Routines
--------------------------------------------------------------------

    function RTS_Wide_Character_Image( Wide_Char : Wide_Character ) 
       return String;
    
    pragma export (Ada,
		   RTS_Wide_Character_Image,
                   External_Name => "rts_wide_character_image");
    pragma global (RTS_Wide_Character_Image);

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the image ('IMAGE) of a wide_character enumeration value
    --| in upper case.
    --|
    --+-------------------------------------------------------
    
    function RTS_Wide_Character_Value( Wide_Char_Image : String ) 
       return Wide_Character;
    
    pragma export (Ada,
		   RTS_Wide_Character_Value,
                   External_Name => "rts_wide_character_value");
    pragma global (RTS_Wide_Character_Value);

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the value ('VALUE) of a wide_character value
    --|
    --+-------------------------------------------------------
    
    function RTS_Wide_Character_Width ( First_Pos : Wide_Character;
    	    	    	    	        Last_Pos  : Wide_Character ) 
        return Natural;

    pragma export (Ada,
		   RTS_Wide_Character_Width,
                   External_Name => "rts_wide_character_width");
    pragma global (RTS_Wide_Character_Width);

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the width ('WIDTH) of a Wide_Character enumeration value.
    --|
    --+-------------------------------------------------------

    function RTS_Wide_Character_Wide_Image( Wide_Char : Wide_Character ) 
       return Wide_String;
    
    pragma export (Ada,
		   RTS_Wide_Character_Wide_Image,
                   External_Name => "rts_wide_character_wide_image");
    pragma global (RTS_Wide_Character_Wide_Image);

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the wide image ('WIDE_IMAGE) of a wide_character enumeration
    --| value in upper case.
    --|
    --+-------------------------------------------------------
    
    function RTS_Wide_Character_Wide_Value( Wide_Char_Image : Wide_String ) 
       return Wide_Character;
    
    pragma export (Ada,
		   RTS_Wide_Character_Wide_Value,
                   External_Name => "rts_wide_character_wide_value");
    pragma global (RTS_Wide_Character_Wide_Value);

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the wide value ('WIDE_VALUE) of a wide_character value
    --|
    --+-------------------------------------------------------
    
--------------------------------------------------------------------
-- Boolean-based Routines
--------------------------------------------------------------------

    function RTS_Boolean_Image ( Enum_Position : Integer ) return String;

    pragma export (Ada,
		   RTS_Boolean_Image,
                   External_Name => "rts_boolean_image");
    pragma global (RTS_Boolean_Image);

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the image ('IMAGE) of a boolean enumeration value
    --| in upper case.
    --|
    --+-------------------------------------------------------

    function RTS_Boolean_Width ( First_Pos : Boolean;
    	    	    	    	 Last_Pos  : Boolean ) return Natural;

    pragma export (Ada,
		   RTS_Boolean_Width,
                   External_Name => "rts_boolean_width");
    pragma global (RTS_Boolean_Width);

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the width ('WIDTH) of a boolean enumeration value.
    --|
    --+-------------------------------------------------------

    function RTS_Boolean_Value ( Boolean_Image : String ) return Boolean;

    pragma export (Ada,
		   RTS_Boolean_Value,
                   External_Name => "rts_boolean_value");
    pragma global (RTS_Boolean_Value);

    --+-------------------------------------------------------
    --| EFFECTS
    --|
    --| Returns the value ('VALUE ) of a boolean enumeration value
    --| in upper case.
    --|
    --+-------------------------------------------------------


end System.RTS.Enumeration_Image_And_Value_Pkg;
