-- ****************************************************************************
--
--  COPYRIGHT NOTICE:
--    This software is protected as an unpublished work under the Copyright
--    Act of 1976. Created 1986 (C). All rights reserved. Alsys.
--    "Alsys" is a trademark of Alsys, S.A.
--
--  PACKAGE BODY: DOS
--  -----------------
--
--  NT-DOS Version 1.00 07 Sep 1994, 09:19
--
-- Modifications :
-- ---------------
--
-- 07 Sep 1994, 20:42 SPM: Initial version with most but not all DOS functions
--			      implemented
--
-- ****************************************************************************


with DOSE, SYSTEM, TEXT_IO, UNCHECKED_CONVERSION;
with Win32;
with Win32.WINUSER, Win32.WinCon, Win32.WinIoctl, Win32.WinNLS, Win32.WinNT;
with Interfaces.C;
with Ada.Command_Line;
-- with Interfaces.C.Strings;
package body DOS is
  
   package Wincon   renames Win32.WinCon;
   package WinIoctl renames Win32.WinIoctl;
   package WinNLS   renames Win32.WinNLS;
   package C        renames Interfaces.C;

   use type Interfaces.C.Unsigned_Long;
   use type Interfaces.C.Unsigned_Short;
   use type Interfaces.C.Char;
   use type Interfaces.C.Char_Array;
   use type Interfaces.C.wChar_Array;
   use type win32.Bool;


   Not_Implemented : exception;

   subtype C_String is Interfaces.C.Char_Array; 
   subtype wC_String is Interfaces.C.wChar_Array;

   -- %%%%%%%%%%%%%%%%% Non Reentrant Data Structures %%%%%%%%%%%%%%%%%%%%%%%% 
   FIND_DATA: aliased WINBASE.WIN32_FIND_DATA;
   FIND_ATTRIBUTE: aliased ATTR_TYPE;
   -- %%%%%%%%%%%%%%%%% Non Reentrant Data Structures %%%%%%%%%%%%%%%%%%%%%%%% 
   subtype HSTRING is STRING (1..4);

   EXIT_CODE: WORD_16 := 0;

   CONSOLE_NAME_Z: aliased C_STRING (1..7);
   KEYBOARD: HANDLE_TYPE;

   KEYBOARD_BUFFER_MAX: constant := 128;
   KEYBOARD_BUFFER: aliased array (1..KEYBOARD_BUFFER_MAX) of CHARACTER;
   KEYBOARD_HEAD, KEYBOARD_TAIL: INTEGER := 1;

-- ********************
-- * Utility Routines *
-- ********************

   -- convert an Ada string to a C string by appending a zero at the end
   function STRINGZ (S: STRING) return C_String is
      SZ: C_String (C.Size_t (S'FIRST) .. C.Size_T (S'LAST+1));
   begin
      for I in S'Range loop
         SZ (C.Size_T(I)) := win32.Char'Val (Character'Pos (S(I)));
      end loop;
      SZ (SZ'LAST) := win32.Char'val (Character'Pos (ASCII.NUL));
      return SZ;
   end STRINGZ;

   function STRINGZ (S: STRING) return wC_String is
      SZ: wC_String (C.Size_t (S'FIRST) .. C.Size_T (S'LAST+1));
   begin
      for I in S'Range loop
         SZ (C.Size_T(I)) := win32.wChar'Val (Character'Pos (S(I)));
      end loop;
      SZ (SZ'LAST) := win32.wChar'val (Character'Pos (ASCII.NUL));
      return SZ;
   end STRINGZ;

   function Ada_String (Str : C_String) return String is      
      Result : String (1 .. 1024);
      use type C.Size_t;
      use type Win32.Char;
   begin
      if Str'Length=0 then
         return "";
      end if;
      for I in Str'Range loop         
         Result (Result'First + Integer(I-Str'First)) := Character'Val (Win32.Char'Pos (Str(I)));
      end loop;
      if Str (Str'Last) = Win32.Nul then
         return Result (Result'First .. Result'First + Integer(Str'Length-2));
      else
         return Result (Result'First .. Result'First + Integer(Str'Length-1));
      end if;
   end Ada_String;

   function Ada_String (Str : Win32.Char_Array) return String is      
      Result : String (1 .. 1024);
      use type C.Size_t;
   begin
      for I in Str'Range loop         
         Result (Result'First + Integer(I-Str'First)) := Character'Val (Win32.Char'Pos (Str(I)));
      end loop;
      return Result (Result'First .. Result'First + Integer(Str'Length-1));
   end Ada_String;

   function Ada_String (Str : wC_String) return String is      
      Result : String (1 .. 1024);
      use type C.Size_t;
   begin
      for I in Str'Range loop         
         Result (Result'First + Integer(I-Str'First)) := Character'Val (Win32.wChar'Pos (Str(I)));
      end loop;
      return Result (Result'First .. Result'First + Integer(Str'Length-1));
   end Ada_String;

   function To_C_String (Str : String) return C_String is      
      Result : C_String (1 .. 1024);
   begin
      for I in Str'Range loop         
         Result (C.Size_t(I)) := Win32.Char'Val (Character'Pos (Str(I)));
      end loop;
      return Result (C.Size_t(Str'First) .. C.Size_t(Str'Last));
   end To_C_String;

  -- convert a zero-terminated string of decimal digits to a decimal number
   function STRING_TO_DECIMAL (S: C_STRING) return INTEGER is
      RESULT: INTEGER := 0;
      DIGIT: INTEGER;
   begin
      for I in S'FIRST .. S'LAST loop
         if C.Char'POS (S (I)) /= 0 then
            DIGIT := C.CHAR'POS (S (I)) - C.CHAR'POS ('0');
	         if DIGIT < 0 or DIGIT > 9 then
	            raise CONSTRAINT_ERROR;
	         end if;
	         RESULT := 10*RESULT + DIGIT;
	      end if;
      end loop;
      return RESULT;
   end STRING_TO_DECIMAL;

   -- convert a zero-terminated string of decimal digits to a decimal number
   function STRING_TO_DECIMAL (S: wC_STRING) return INTEGER is
      RESULT: INTEGER := 0;
      DIGIT: INTEGER;
   begin
      for I in S'FIRST .. S'LAST loop
         if Win32.wChar'POS (S (I)) /= 0 then
            DIGIT := win32.wCHAR'POS (S (I)) - Win32.wCHAR'POS ('0');
	         if DIGIT < 0 or DIGIT > 9 then
	            raise CONSTRAINT_ERROR;
	         end if;
	         RESULT := 10*RESULT + DIGIT;
	      end if;
      end loop;
      return RESULT;
   end STRING_TO_DECIMAL;



   -- diagnostic routine
   procedure DUMP (IR: WinCon.INPUT_RECORD) is
      use WIN32; -- for the "and" and "/=" function
      function "and" (Left, Right : Win32.Dword) return Win32.Dword is
      begin
         return Left/Right mod 2;
      end "and";
   begin
      TEXT_IO.PUT ("Event: " & WIN32.WORD'IMAGE (IR.EVENTTYPE));
      if IR.Event.KeyEVENT.bKEYDOWN /= 0 then
         TEXT_IO.PUT ("dn");
      else
         TEXT_IO.PUT ("up");
      end if;
      TEXT_IO.PUT (" RPT:" & WIN32.WORD'IMAGE (IR.Event.KeyEVENT.wREPEATCOUNT));
      TEXT_IO.PUT (" KEY: " & WIN32.WORD'IMAGE (IR.Event.KeyEVENT.wVIRTUALKEYCODE));
      TEXT_IO.PUT (" SCN: " & WIN32.WORD'IMAGE (IR.Event.KeyEVENT.wVIRTUALSCANCODE));
      TEXT_IO.PUT (" CHR: " & WIN32.CHAR'IMAGE (IR.Event.KeyEVENT.uCHAR.ASCIIChar));
      if (IR.EVENT.KeyEvent.dwCONTROLKEYSTATE and WinCon.RIGHT_ALT_PRESSED) /= 0 then
         TEXT_IO.PUT (" r_alt");
      end if;
      if (IR.EVENT.KeyEvent.dwCONTROLKEYSTATE and WinCon.LEFT_ALT_PRESSED) /= 0 then
         TEXT_IO.PUT (" l_alt");
      end if;
      if (IR.EVENT.KeyEvent.dwCONTROLKEYSTATE and WinCon.RIGHT_CTRL_PRESSED) /= 0 then
         TEXT_IO.PUT (" r_ctl");
      end if;
      if (IR.EVENT.KeyEvent.dwCONTROLKEYSTATE and WinCon.LEFT_CTRL_PRESSED) /= 0 then
         TEXT_IO.PUT (" l_ctl");
      end if;
      if (IR.EVENT.KeyEvent.dwCONTROLKEYSTATE and WinCon.SHIFT_PRESSED) /= 0 then
         TEXT_IO.PUT (" shf");
      end if;
      if (IR.EVENT.KeyEvent.dwCONTROLKEYSTATE and WinCon.NUMLOCK_ON) /= 0 then
         TEXT_IO.PUT (" num");
      end if;
      if (IR.EVENT.KeyEvent.dwCONTROLKEYSTATE and	WinCon.SCROLLLOCK_ON) /= 0 then
         TEXT_IO.PUT (" scr");
      end if;
      if (IR.EVENT.KeyEvent.dwCONTROLKEYSTATE and WinCon.CAPSLOCK_ON) /= 0 then
         TEXT_IO.PUT (" cap");
      end if;
      if (IR.EVENT.KeyEvent.dwCONTROLKEYSTATE and WinCon.ENHANCED_KEY) /= 0 then
         TEXT_IO.PUT (" enh");
      end if;
      if (IR.EVENT.KeyEvent.dwCONTROLKEYSTATE and 16#100#) /= 0 then
         TEXT_IO.PUT (" ???");
      end if;
      TEXT_IO.PUT_LINE ("");
   end DUMP;

   procedure INCREMENT (X: in out INTEGER) is
   begin
      if X = KEYBOARD_BUFFER_MAX then
	 X := 1;
      else
	 X := X+1;
      end if;
   end INCREMENT;

   procedure DECREMENT (X: in out INTEGER) is
   begin
      if X = 1 then
	 X := KEYBOARD_BUFFER_MAX;
      else
	 X := X-1;
      end if;
   end DECREMENT;

   procedure FLUSH_INPUT_BUFFER is
      IR: aliased WinCon.INPUT_RECORD;
      COUNT: aliased WIN32.DWORD;
      CHAR_COUNT: WIN32.WORD;
      use WIN32; -- for the  "=" function
      use type Win32.Bool;
   begin
      loop

	 -- see if there is some input from windows
	 if WinCon.PEEKCONSOLEINPUT (
            KEYBOARD, IR'Unchecked_Access, 1, COUNT'Unchecked_Access)
	       = 0 then
       DOSE.ERROR ("Impossible error on peeking at console input");
	    raise DOS_ERROR;
	 end if;
	 exit when COUNT = 0; -- exit if no input

	 -- see if there is room in the keyboard buffer
	 if IR.EVENTTYPE = WinCon.KEY_EVENT and then
	       IR.EVENT.KeyEvent.bKEYDOWN /= 0 then -- a new character is available
	    CHAR_COUNT := IR.EVENT.KeyEvent.wREPEATCOUNT;
	    if IR.EVENT.KeyEvent.uChar.ASCIICHAR = win32.Char'Val (Character'Pos (ASCII.NUL)) then -- 2-char sequence
	       CHAR_COUNT := 2 * CHAR_COUNT;
	    end if;
	    for I in 1 .. CHAR_COUNT loop
	       INCREMENT (KEYBOARD_TAIL);
	       if KEYBOARD_HEAD = KEYBOARD_TAIL then
		  for J in 1..I loop
		     DECREMENT (KEYBOARD_TAIL);
		  end loop;
		  return; -- no more room
	       end if;
	    end loop;
	    for I in 1..CHAR_COUNT loop
	       DECREMENT (KEYBOARD_TAIL);
	    end loop;
	 end if;

	 -- get next input
	 if WinCon.READCONSOLEINPUT (KEYBOARD, IR'Unchecked_Access, 1, COUNT'Unchecked_Access)
	       = 0 then
            DOSE.ERROR ("Impossible error on reading console input");
	    raise DOS_ERROR;
	 end if;

	 -- process the input
	 if IR.EVENTTYPE = WinCon.KEY_EVENT and then
	       IR.EVENT.KeyEvent.bKEYDOWN /= 0 then -- a new character is available
	    for I in 1..IR.EVENT.KeyEvent.wREPEATCOUNT loop
	       if IR.EVENT.KeyEvent.uChar.ASCIICHAR /= Win32.Char'Val (Character'Pos(ASCII.NUL)) then -- single ASCII character
		  KEYBOARD_BUFFER (KEYBOARD_TAIL) := Character'Val (win32.Char'Pos (IR.EVENT.KeyEvent.uChar.ASCIICHAR) mod 16#80#);
		  INCREMENT (KEYBOARD_TAIL);
	       else -- two ASCII characters, starting with NULL put into buffer
		  if IR.EVENT.KeyEvent.wVIRTUALSCANCODE /= 29 --	control
		     and IR.EVENT.KeyEvent.wVIRTUALSCANCODE /= 42 -- left shift
		     and IR.EVENT.KeyEvent.wVIRTUALSCANCODE /= 54 -- right shift
		     and IR.EVENT.KeyEvent.wVIRTUALSCANCODE /= 56 -- alt
		     and IR.EVENT.KeyEvent.wVIRTUALSCANCODE /= 58 -- caps lock
		     and IR.EVENT.KeyEvent.wVIRTUALSCANCODE /= 69 -- num lock, pause
		     and IR.EVENT.KeyEvent.wVIRTUALSCANCODE /= 70 -- scroll lock
		     then
			KEYBOARD_BUFFER (KEYBOARD_TAIL) := Character'Val (
			   Win32.Char'Pos (IR.EVENT.KeyEvent.uChar.ASCIICHAR) mod 16#80#);
			INCREMENT (KEYBOARD_TAIL);
			KEYBOARD_BUFFER (KEYBOARD_TAIL) :=
                           CHARACTER'VAL (IR.EVENT.KeyEvent.wVIRTUALSCANCODE);
			INCREMENT (KEYBOARD_TAIL);
		  end if;
	       end if;
	    end loop;
	 end if;

      end loop;
   end FLUSH_INPUT_BUFFER;

   procedure FLUSH_KEYBOARD_BUFFER is
   begin
      KEYBOARD_HEAD := 1;
      KEYBOARD_TAIL := 1;
   end FLUSH_KEYBOARD_BUFFER;

   function KBD_GET return CHARACTER is
      CHAR: CHARACTER;
   begin
      loop -- wait for buffer to be non empty
	 FLUSH_INPUT_BUFFER;
	 exit when KEYBOARD_HEAD /= KEYBOARD_TAIL;
      end loop;
      CHAR := KEYBOARD_BUFFER (KEYBOARD_HEAD);
      INCREMENT (KEYBOARD_HEAD);
      return CHAR;
   end KBD_GET;

   procedure KBD_SET (MODE: WIN32.DWORD) is
   begin
      if WinCon.SETCONSOLEMODE(KEYBOARD, MODE) = 0 then
         DOSE.ERROR ("Error on SETCONSOLE ");
	      raise DOS_ERROR;
      end if;
   end KBD_SET;

   -- function  Argv (INDEX: in NATURAL) return SYSTEM.ADDRESS;
   -- pragma INTERFACE (C, ARGV);
   -- pragma INTERFACE_NAME (ARGV, "_alsys_argv");
   -- pragma Import (C, ARGV, "_alsys_argv");

   -- function ARG_COUNT return POSITIVE;
   -- pragma INTERFACE(C, ARG_COUNT);
   -- pragma INTERFACE_NAME(ARG_COUNT, "_alsys_argc");
   -- pragma Import (C, ARG_COUNT, "_alsys_argc");

   function ARG_VALUE (INDEX : in NATURAL) return C_STRING is
      -- ARG: C_STRING (1..1024);
      -- for ARG use at ARGV (INDEX);
      -- LENGTH: C.Size_t := 0;
      -- use type C.Size_t; 
   begin
      if Index = 0 then
         return Stringz (Ada.Command_Line.Command_Name);
      else
         return Stringz (Ada.Command_Line.Argument (Index));
      end if;
      -- while ARG (LENGTH+1) /= Win32.Char'Val (Character'Pos (ASCII.NUL)) loop
	  --     LENGTH := LENGTH + 1;
      --end loop;
      -- return ARG (1 .. LENGTH);
   end ARG_VALUE;

   function HANDLE_TO_STRING is
      new UNCHECKED_CONVERSION (HANDLE_TYPE, HSTRING);

   function STRING_TO_HANDLE is
      new UNCHECKED_CONVERSION (HSTRING, HANDLE_TYPE);

   -- function ADDRESS_TO_LCID is
   --    new UNCHECKED_CONVERSION (SYSTEM.ADDRESS, WinCon.LCID);

   -- function LCID_TO_ADDRESS is
   --   new UNCHECKED_CONVERSION (WinCon.LCID, SYSTEM.ADDRESS);

   procedure DEBUG (MESSAGE: STRING) is
   begin
      -- TEXT_IO.PUT_LINE (MESSAGE);
      null;
   end DEBUG;

-- ********************************************
-- * Function 01H -- Keyboard Input with Echo *
-- ********************************************

   function READ_KBD_ECHO return CHARACTER is
      CHAR: CHARACTER;
   begin
      DEBUG ("Entering READ_KBD_ECHO");
      KBD_SET (WinCon.ENABLE_PROCESSED_INPUT);
      CHAR := KBD_GET;
      DISPLAY_CHAR (CHAR);
      DEBUG ("Leaving READ_KBD_ECHO");
      return CHAR;
   end READ_KBD_ECHO;

-- **********************************
-- * Function 02H -- Display Output *
-- **********************************

   procedure DISPLAY_CHAR (CHR: CHARACTER) is
      BUFFER: aliased Win32.Char := win32.Char'Val (Character'Pos (Chr));
      OUT_COUNT: aliased Win32.Dword;
   begin
      DEBUG ("Entering DISPLAY_CHAR");
      if WINBASE.WRITEFILE (
            STANDARD_OUTPUT, BUFFER'Address, 1,
            OUT_COUNT'Unchecked_Access, null) = 0 then
         DOSE.ERROR ("Error on displaying character");
	      raise DOS_ERROR;
      end if;
      if OUT_COUNT /= 1 then
         DOSE.ERROR ("Impossible error on displaying character");
	      raise DOS_ERROR;
      end if;
      DEBUG ("LEAVING DISPLAY_CHAR");
   end DISPLAY_CHAR;

-- **************************************
-- * Function 06H -- Direct Console I/O *
-- **************************************

   function READ_KBD_DIRECT_ECHO return CHARACTER is
      CHAR: CHARACTER;
   begin
      DEBUG ("Entering READ_KBD_DIRECT_ECHO");
      KBD_SET (0);
      CHAR := KBD_GET;
      DISPLAY_CHAR (CHAR);
      DEBUG ("Leaving READ_KBD_DIRECT_ECHO");
      return CHAR;
   end READ_KBD_DIRECT_ECHO;

-- *****************************************
-- * Function 07H -- Direct Keyboard Input *
-- *****************************************

   function READ_KBD_DIRECT_NO_ECHO return CHARACTER is
   begin
      DEBUG ("Entering READ_KBD_DIRECT_NO_ECHO");
      KBD_SET (0);
      DEBUG ("Leaving READ_KBD_DIRECT_NO_ECHO");
      return KBD_GET;
   end READ_KBD_DIRECT_NO_ECHO;

-- **********************************
-- * Function 08H -- Keyboard Input *
-- **********************************

   function READ_KBD_NO_ECHO return CHARACTER is
   begin
      DEBUG ("Entering READ_KBD_NO_ECHO");
      KBD_SET (WinCon.ENABLE_PROCESSED_INPUT);
      DEBUG ("Leaving READ_KBD_NO_ECHO");
      return KBD_GET;
   end READ_KBD_NO_ECHO;

-- *******************************************
-- * Function 0AH -- Buffered Keyboard Input *
-- *******************************************

   function READ_KBD_BUFFERED (MAX_KEYS : INTEGER) return STRING is
      BUFFER: STRING (1..MAX_KEYS);
      COUNT: INTEGER := 0;
      CHAR: CHARACTER;
   begin
      DEBUG ("Entering READ_KBD_BUFFERED");
      KBD_SET (WinCon.ENABLE_PROCESSED_INPUT);
      loop
	      CHAR := KBD_GET;
	      if CHAR = ASCII.BS then
	         if COUNT > 0 then
	            COUNT := COUNT - 1;
	            DISPLAY_CHAR (ASCII.BS);
               DISPLAY_CHAR (' ');
	            DISPLAY_CHAR (ASCII.BS);
	         end if;
	      elsif CHAR = ASCII.CR then
	         exit;
	      elsif COUNT = MAX_KEYS then
	         DISPLAY_CHAR (ASCII.BEL);
	      else
	         COUNT := COUNT+1;
	         BUFFER (COUNT) := CHAR;
	         DISPLAY_CHAR (CHAR);
	      end if;
      end loop;
      DEBUG ("Leaving READ_KBD_BUFFERED");
      return BUFFER (1..COUNT);
   end READ_KBD_BUFFERED;

-- **************************************
-- * Function 0BH -- Check Input Status *
-- **************************************

   function KBD_DATA_AVAILABLE return BOOLEAN is
   begin
      DEBUG ("Entering KBD_DATA_AVAILABLE");
      FLUSH_INPUT_BUFFER;
      DEBUG ("Leaving KBD_DATA_AVAILABLE");
      return KEYBOARD_HEAD /= KEYBOARD_TAIL;
   end KBD_DATA_AVAILABLE;

-- ****************************************************
-- * Function 0CH -- Clear Buffer and Invoke Function *
-- ****************************************************

   function READF_KBD_BUFFERED (MAX_KEYS : INTEGER) return STRING is
      BUFFER: STRING (1..MAX_KEYS);
      COUNT: INTEGER;
   begin
      DEBUG ("Entering READF_KBD_BUFFERED");
      FLUSH_KEYBOARD_BUFFER;
      DEBUG ("Leaving READF_KBD_BUFFERED");
      return READ_KBD_BUFFERED (MAX_KEYS);
   end READF_KBD_BUFFERED;

   function READF_KBD_ECHO return CHARACTER is
      CHAR: CHARACTER;
   begin
      DEBUG ("Entering READF_KBD_ECHO");
      FLUSH_KEYBOARD_BUFFER;
      KBD_SET (WinCon.ENABLE_PROCESSED_INPUT);
      CHAR := KBD_GET;
      DISPLAY_CHAR (CHAR);
      DEBUG ("Leaving READF_KBD_ECHO");
      return CHAR;
   end READF_KBD_ECHO;

   function READF_KBD_NO_ECHO return CHARACTER is
   begin
      DEBUG ("Entering READF_KBD_NO_ECHO");
      FLUSH_KEYBOARD_BUFFER;
      KBD_SET (WinCon.ENABLE_PROCESSED_INPUT);
      DEBUG ("Leaving READF_KBD_NO_ECHO");
      return KBD_GET;
   end READF_KBD_NO_ECHO;

   function READF_KBD_DIRECT_ECHO return CHARACTER is
      CHAR: CHARACTER;
   begin
      DEBUG ("Entering READF_KBD_DIRECT_ECHO");
      FLUSH_KEYBOARD_BUFFER;
      KBD_SET (0);
      CHAR := KBD_GET;
      DISPLAY_CHAR (CHAR);
      DEBUG ("Leaving READF_KBD_DIRECT_ECHO");
      return CHAR;
   end READF_KBD_DIRECT_ECHO;

   function READF_KBD_DIRECT_NO_ECHO return CHARACTER is
   begin
      DEBUG ("Entering READF_KBD_DIRECT_NO_ECHO");
      FLUSH_KEYBOARD_BUFFER;
      KBD_SET (0);
      DEBUG ("Leaving READF_KBD_DIRECT_NO_ECHO");
      return KBD_GET;
   end READF_KBD_DIRECT_NO_ECHO;

-- ********************************
-- * Function 0EH -- Select Drive *
-- ********************************

   procedure SELECT_DRIVE (DRIVE: DRIVE_LETTER)is
      DRIVE_NAME_Z: aliased C_STRING (1..3) := STRINGZ (" :");
   begin
      DEBUG ("Entering SELECT_DRIVE");
      if DRIVE /= '@' then
	      DRIVE_NAME_Z (1) := Win32.Char'Val (Drive_Letter'Pos(DRIVE));
         if WINBASE.SETCURRENTDIRECTORY (DRIVE_NAME_Z(1)'Unchecked_Access) = 0 then
            DOSE.ERROR ("Error on selecting drive");
	         raise DOS_ERROR;
	      end if;
      end if;
      DEBUG ("Leaving SELECT_DRIVE");
   end SELECT_DRIVE;

   function NUMBER_OF_DRIVES return INTEGER is
      COUNT: WIn32.Dword := 0;
      MASK: WIN32.DWORD;
      use win32;
   begin
      DEBUG ("Entering NUMBER_OF_DRIVES");
      MASK := WINBASE.GETLOGICALDRIVES;
      if MASK = 0 then
         DOSE.ERROR ("Error on getting number of drives");
	      raise DOS_ERROR;
      end if;
      for I in 1..30 loop
	      COUNT := COUNT + (MASK mod 2);
	      MASK := MASK / 2;
      end loop;
      DEBUG ("Leaving NUMBER_OF_DRIVES");
      return Integer (COUNT);
   end NUMBER_OF_DRIVES;

-- *********************************
-- * Function 19H -- Current Drive *
-- *********************************

   function CURRENT_DRIVE return DRIVE_LETTER is
      BUFFER: aliased C_String (1..1024);
   begin
      DEBUG ("Entering CURRENT_DRIVE");
      if WINBASE.GETCURRENTDIRECTORY (1024, BUFFER(1)'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on getting current directory");
	      raise DOS_ERROR;
      end if;
      DEBUG ("Leaving CURRENT_DRIVE");
      return Character'Val (Win32.Char'Pos (BUFFER (1)));
   end CURRENT_DRIVE;

-- *****************************************************************
-- * Function 1CH -- Allocation Table Information, Specified Drive *
-- *****************************************************************

-- !! implementation error: DEVICEIOCONTROL call returns with error #1
--    which is "Invalid function"

   procedure ALLOCATION_INFO (
      DRIVE : DRIVE_LETTER := '@';
	   MEDIA_DESCRIPTION_BYTE: out WORD_16;
	   NUM_ALLOCATION_UNITS: out WORD_16;
	   NUM_SECTORS_ALLOC_UNIT: out WORD_16;
	   PHYSICAL_SECTOR_SIZE: out WORD_16) is
      HANDLE: WinNT.Handle;
      DRIVE_NAME_Z: C_STRING (1..7) := STRINGZ ("\\.\ :");
      BUFFER: aliased C_STRING (1..1024);
      GEOMETRY: WINIOCTL.DISK_GEOMETRY;
      BYTES_RETURNED: aliased Win32.Dword;
      use type WinNT.Handle;
   begin
      DEBUG ("Entering CURRENT_DRIVE");

      -- get drive name

      if DRIVE = '@' then
         if WINBASE.GETCURRENTDIRECTORY (1024, BUFFER(1)'Unchecked_Access) = 0 then
            DOSE.ERROR ("Error on getting allocation information");
	         raise DOS_ERROR;
	      end if;
	      DRIVE_NAME_Z (5) := BUFFER (1);
      else
	      DRIVE_NAME_Z (5) := Win32.Char'Val (Drive_Letter'Pos(DRIVE));
      end if;

      -- get handle for drive

      HANDLE :=  WINBASE.CREATEFILE (
         DRIVE_NAME_Z(1)'Unchecked_Access,
	      WinNT.GENERIC_READ,
	      WinNT.FILE_SHARE_WRITE, -- share mode
	      null, -- pointer to optional security attributes
	      WINBASE.OPEN_EXISTING, -- open existing drive
	      ATTR_NORMAL,
	      null);  -- optional file with attributes to copy
        
      if HANDLE = WINBASE.INVALID_HANDLE_VALUE then
         DOSE.ERROR ("Error on getting allocation information");
	      raise DOS_ERROR;
      end if;

      -- get allocation information for handle

      if WINBASE.DEVICEIOCONTROL (
         HANDLE, -- handle of device to operate on
	      WinIoctl.IOCTL_DISK_GET_MEDIA_TYPES, -- control code of operation
	      null, -- address of input buffer
	      0, -- size of input buffer
         GEOMETRY'Address, -- address of output buffer
         GEOMETRY'SIZE/8, -- size of output buffer in bytes
         BYTES_RETURNED'Unchecked_Access, -- address of actual number of bytes returned
	      null -- address of overlapped structure
	      ) = 0 then
	      raise DOS_ERROR;
      end if;

      -- close the handle

      if WINBASE.CLOSEHANDLE (HANDLE) = 0 then
         DOSE.ERROR ("Error on getting allocation information");
	      raise DOS_ERROR;
      end if;

      -- return the results

      MEDIA_DESCRIPTION_BYTE := WinIOctl.Media_type'Pos (GEOMETRY.MEDIATYPE);
      NUM_ALLOCATION_UNITS := WORD_16 (GEOMETRY.TRACKSPERCYLINDER);
      NUM_SECTORS_ALLOC_UNIT := WORD_16 (GEOMETRY.SECTORSPERTRACK);
      PHYSICAL_SECTOR_SIZE := WORD_16 (GEOMETRY.BYTESPERSECTOR);

      DEBUG ("Leaving ALLOCATION_INFO");
   end ALLOCATION_INFO;

-- ****************************
-- * Function 2AH -- Get Date *
-- ****************************

   function DATE return DATE_RECORD is
      DOS_DATE: DATE_RECORD;
      NT_DATE_TIME: aliased WINBASE.SYSTEMTIME;
   begin
      DEBUG ("Entering DATE");
      WINBASE.GETLOCALTIME (NT_DATE_TIME'Unchecked_Access);
      DOS_DATE.YEAR := INTEGER (NT_DATE_TIME.WYEAR);
      DOS_DATE.MONTH := INTEGER (NT_DATE_TIME.WMONTH);
      DOS_DATE.DAY := INTEGER (NT_DATE_TIME.WDAY);
      DEBUG ("Leaving DATE");
      return DOS_DATE;
   end DATE;

   function DAY_OF_WEEK return DAY_OF_WEEK_TYPE is
      DOS_DATE: DATE_RECORD;
      NT_DATE_TIME: aliased WINBASE.SYSTEMTIME;
   begin
      DEBUG ("Entering DAY_OF_WEEK");
      WINBASE.GETLOCALTIME (NT_DATE_TIME'Unchecked_Access);
      DEBUG ("Leaving DAY_OF_WEEK");
      return INTEGER (NT_DATE_TIME.WDAYOFWEEK);
   end DAY_OF_WEEK;

-- ****************************
-- * Function 2BH -- Set Date *
-- ****************************

   procedure SET_DATE (DATE: DATE_RECORD) is
      DUMMY: BOOLEAN;
   begin
      DEBUG ("Entering SET_DATE procedure");
      DUMMY := SET_DATE (DATE);
      DEBUG ("Leaving SET_DATE procedure");
   end SET_DATE;

   function SET_DATE (DATE: DATE_RECORD) return BOOLEAN is
      NT_DATE_TIME: aliased WINBASE.SYSTEMTIME;
      TOKEN_HANDLE: aliased WinNT.HANDLE;
      -- SE_SYSTEMTIME_NAME_Z: aliased STRING
      --    (WinNT.SE_SYSTEMTIME_NAME'FIRST .. WinNT.SE_SYSTEMTIME_NAME'LAST+1)
	   --        := STRINGZ (WinNT.SE_SYSTEMTIME_NAME);
      TKP: aliased WinNT.TOKEN_PRIVILEGES;
      VALID_DATE: BOOLEAN;
      function To_PLUID is new Unchecked_Conversion (Source => WinNT.PTOKEN_PRIVILEGES, 
                                                     Target => WinNT.PLUID);
   begin
      DEBUG ("Entering SET_DATE function");

      -- get the current time/date

      WINBASE.GETLOCALTIME (NT_DATE_TIME'Unchecked_Access);

      -- modify the time/date

      NT_DATE_TIME.WYEAR := WIN32.WORD (DATE.YEAR);
      NT_DATE_TIME.WMONTH := WIN32.WORD (DATE.MONTH);
      NT_DATE_TIME.WDAY := WIN32.WORD (DATE.DAY);

      -- get the current process token handle

--      raise Not_Implemented;
       if WinBase.OPENPROCESSTOKEN (
	         WINBASE.GETCURRENTPROCESS,
	         WinNT.TOKEN_ADJUST_PRIVILEGES + WinNT.TOKEN_QUERY,
            TOKEN_HANDLE'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on setting date");
	      raise DOS_ERROR;
       end if;

      -- get the LUID for system time privileges

--      raise Not_Implemented;
      if WINBASE.LOOKUPPRIVILEGEVALUE (
	      null, -- address of string specifying the system
         WinNT.SE_SYSTEMTIME_NAME(1)'Unchecked_Access, -- addr of strng specifying the privilege
         To_PLUID (TKP'Unchecked_Access) -- address of locally unique identifier
	      ) = 0 then
            DOSE.ERROR ("Error on setting date");
	    raise DOS_ERROR;
      end if;

      -- get set-time privilege for this process

      TKP.PRIVILEGECOUNT := 1;
      TKP.PRIVILEGES(0).Attributes := WinNT.SE_PRIVILEGE_ENABLED;
--      raise Not_Implemented;
      if WinBase.ADJUSTTOKENPRIVILEGES (
	      TOKEN_HANDLE,
         0, -- don't disable all privileges
         TKP'Unchecked_Access, -- privileges to be changed
	      0, -- size of previous buffer
	      WinNT.PToken_Privileges'(null), -- old privileges
	      Win32.PDword'(null)) -- addr of req size of old privileges
         /= 0 then
         raise DOS_Error;
       end if;
      if WINBASE.GETLASTERROR /= WinNT.ERROR_SEVERITY_SUCCESS then
         DOSE.ERROR ("Error on setting date");
	      raise DOS_ERROR;
      end if;

      -- set the system time

      VALID_DATE := WINBASE.SETLOCALTIME (NT_DATE_TIME'Unchecked_Access) /= 0;

      -- now disable set-time privileges

      TKP.PRIVILEGES(0).ATTRIBUTES := 0;
--      raise Not_Implemented;
      if WinBase.ADJUSTTOKENPRIVILEGES (
	      TOKEN_HANDLE,
         0, -- don't disable all privileges
         TKP'Unchecked_Access, -- privileges to be changed
	      0, -- size of previous buffer
	      WinNT.PToken_Privileges'(null), -- old privileges
	      Win32.PDword'(null))  -- addr of req size of old privileges
         /= 0 then
         raise DOS_Error;
      end if;
     if WINBASE.GETLASTERROR /= WinNT.ERROR_SEVERITY_SUCCESS then
         DOSE.ERROR ("Error on setting date");
	      raise DOS_ERROR;
      end if;

      -- return

      DEBUG ("Leaving SET_DATE function");
      return VALID_DATE;

   end SET_DATE;

-- ****************************
-- * Function 2CH -- Get Time *
-- ****************************

   function TIME return TIME_RECORD is
      DOS_TIME: TIME_RECORD;
      NT_DATE_TIME: aliased WINBASE.SYSTEMTIME;
   begin
      DEBUG ("Entering TIME");
      WINBASE.GETLOCALTIME (NT_DATE_TIME'Unchecked_Access);
      DOS_TIME.HOURS := INTEGER (NT_DATE_TIME.WHOUR);
      DOS_TIME.MINUTES := INTEGER (NT_DATE_TIME.WMINUTE);
      DOS_TIME.SECONDS := INTEGER (NT_DATE_TIME.WSECOND);
      DOS_TIME.HUNDREDTHS := INTEGER (NT_DATE_TIME.WMILLISECONDS) / 10;
      DEBUG ("Leaving TIME");
      return DOS_TIME;
   end TIME;

-- ****************************
-- * Function 2BH -- Set Time *
-- ****************************

   procedure SET_TIME (TIME: TIME_RECORD) is
      DUMMY: BOOLEAN;
   begin
      DEBUG ("Entering SET_TIME procedure");
      DUMMY := SET_TIME (TIME);
      DEBUG ("Leaving SET_TIME procedure");
   end SET_TIME;

   function SET_TIME (TIME: TIME_RECORD) return BOOLEAN is
      NT_DATE_TIME: aliased WINBASE.SYSTEMTIME;
      TOKEN_HANDLE: aliased HANDLE_TYPE;
      -- SE_SYSTEMTIME_NAME_Z: STRING
      --    (WIN32.SE_SYSTEMTIME_NAME'FIRST .. WIN32.SE_SYSTEMTIME_NAME'LAST+1)
	   --        := STRINGZ (WIN32.SE_SYSTEMTIME_NAME);
      TKP: aliased WinNT.TOKEN_PRIVILEGES;
      VALID_TIME: BOOLEAN;
      function To_PLUID is new Unchecked_Conversion (Source => WinNT.PToken_Privileges,
                                                     Target => WinNT.PLUID); 
   begin
      DEBUG ("Entering SET_TIME function");

      -- get the current time/date

      WINBASE.GETLOCALTIME (NT_DATE_TIME'Unchecked_Access);

      -- modify the time/date

      NT_DATE_TIME.WHOUR := WIN32.WORD (TIME.HOURS);
      NT_DATE_TIME.WMINUTE := WIN32.WORD (TIME.MINUTES);
      NT_DATE_TIME.WSECOND := WIN32.WORD (TIME.SECONDS);
      NT_DATE_TIME.WMILLISECONDS := WIN32.WORD (10*TIME.HUNDREDTHS);

      -- get the current process token handle

--      raise Not_Implemented;
      if WinBase.OPENPROCESSTOKEN (
	         WINBASE.GETCURRENTPROCESS,
	         WinNT.TOKEN_ADJUST_PRIVILEGES + WinNT.TOKEN_QUERY,
            TOKEN_HANDLE'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on setting time");
	      raise DOS_ERROR;
      end if;

      -- get the LUID for system time privileges

--      raise Not_Implemented;
      if WINBASE.LOOKUPPRIVILEGEVALUE (
	      null, -- address of string specifying the system
         WinNT.SE_SYSTEMTIME_NAME(1)'Unchecked_Access, -- addr of strng specifying the privilege
         To_PLUID (TKP'Unchecked_Access) -- address of locally unique identifier
	 ) = 0 then
            DOSE.ERROR ("Error on setting time");
	    raise DOS_ERROR;
      end if;

      -- get set-time privilege for this process

      TKP.PRIVILEGECOUNT := 1;
      TKP.PRIVILEGES(0).ATTRIBUTES := WinNT.SE_PRIVILEGE_ENABLED;
--      raise Not_Implemented;
      if WinBase.ADJUSTTOKENPRIVILEGES (
	      TOKEN_HANDLE,
         0, -- don't disable all privileges
         TKP'Unchecked_Access, -- privileges to be changed
	      0, -- size of previous buffer
	      WinNT.PToken_Privileges'(null), -- old privileges
	      Win32.PDword'(null)) -- addr of req size of old privileges
         /= 0 then
         raise DOS_Error;
       end if;
      if WINBASE.GETLASTERROR /= WinNT.ERROR_SEVERITY_SUCCESS then
         DOSE.ERROR ("Error on setting time");
	      raise DOS_ERROR;
      end if;

      -- set the system time

      VALID_TIME := WINBASE.SETLOCALTIME (NT_DATE_TIME'Unchecked_Access) /= 0;

      -- now disable set-time privileges

      TKP.PRIVILEGES(0).ATTRIBUTES := 0;
--      raise Not_Implemented;
      if WinBase.ADJUSTTOKENPRIVILEGES (
	      TOKEN_HANDLE,
         0, -- don't disable all privileges
         TKP'Unchecked_Access, -- privileges to be changed
	      0, -- size of previous buffer
	      WinNT.PToken_Privileges'(null), -- old privileges
	      Win32.PDword'(null)) -- addr of req size of old privileges
         /= 0 then
         raise DOS_Error;
      end if;
      if WINBASE.GETLASTERROR /= WinNT.ERROR_SEVERITY_SUCCESS then
         DOSE.ERROR ("Error on setting time");
	      raise DOS_ERROR;
      end if;

      -- return

      DEBUG ("Leaving SET_TIME function");
      return VALID_TIME;

   end SET_TIME;

-- ***************************************
-- * Function 36H -- Get Disk Free Space *
-- ***************************************

   procedure GET_DISK_FREE_SPACE (
      DRIVE: DRIVE_LETTER := '@'; -- Drive
      CLUSTERS_AVAIL: out WORD_16; -- Available clusters
      CLUST_PER_DRV: out WORD_16; -- Clusters per drive
      BYTES_PER_SEC: out WORD_16; -- Bytes per sector
      SEC_PER_CLUST: out WORD_16) is -- Sectors per cluster
      DRIVE_NAME_Z: C_STRING (1..4) := STRINGZ (" :\");
	   BUFFER: aliased C_String (1..1024);
      Local_SEC_PER_CLUST : aliased Win32.Dword;
      Local_BYTES_PER_SEC : aliased Win32.Dword;
      Local_CLUSTERS_AVAIL : aliased Win32.Dword;
      Local_CLUST_PER_DRV : aliased Win32.Dword;
      use type Win32.Bool;
   begin

      -- generate string for root directory

      if DRIVE = '@' then
         if WINBASE.GETCURRENTDIRECTORY (1024, BUFFER(1)'Unchecked_Access) = 0 then
            DOSE.ERROR ("Error on getting allocation information");
	         raise DOS_ERROR;
	      end if;
	      DRIVE_NAME_Z (1) := BUFFER (1);
      else
	      DRIVE_NAME_Z (1) := Win32.Char'Val (Drive_Letter'Pos(DRIVE));
      end if;

      -- get disk free space
      if WINBASE.GETDISKFREESPACE (
            DRIVE_NAME_Z(1)'Unchecked_Access,
            Local_SEC_PER_CLUST'Unchecked_Access,
            Local_BYTES_PER_SEC'Unchecked_Access,
            Local_CLUSTERS_AVAIL'Unchecked_Access,
            Local_CLUST_PER_DRV'Unchecked_Access
	      ) = 0 then
         DOSE.ERROR ("Error on getting disk free space");
	      raise DOS_ERROR;
      end if;
      Sec_Per_Clust := Word_16 (Local_SEC_PER_CLUST);
      Bytes_Per_Sec := Word_16 (Local_BYTES_PER_SEC);
      Clusters_Avail := Word_16 (Local_CLUSTERS_AVAIL);
      Clust_Per_Drv := Word_16 (Local_CLUST_PER_DRV);
   end GET_DISK_FREE_SPACE;

-- ************************************************************
-- * Function 38H -- Get or Set Country Dependent Information *
-- ************************************************************

   procedure GET_COUNTRY_INFO (
      COUNTRY_CODE: in out INTEGER;
      COUNTRY_INFO: out COUNTRY_INFO_RECORD) is
	 COUNT: C.Size_t;
	 BUFFER: aliased C_String (1..1024);
	 NEW_COUNTRY_CODE: INTEGER;
	 LCID: WinNT.LCID := 0;
      Scratch : Win32.Int;
      use type Win32.Char;
      use type WinNT.LCID;
      use System;
   begin
      DEBUG ("Entering GET_COUNTRY_INFO");

      -- get LCID from country code

      if COUNTRY_CODE = 0 then
	      LCID := WinNLS.GETSYSTEMDEFAULTLCID;
	      COUNT := C.Size_T (WinNLS.GETLOCALEINFO
	           (LCID,WinNLS.LOCALE_ICOUNTRY,BUFFER(1)'Unchecked_Access,1024));
	      COUNTRY_CODE := STRING_TO_DECIMAL (BUFFER (1..COUNT));
	      if COUNTRY_CODE = 0 then
            DOSE.ERROR ("Error on getting country info");
	         raise DOS_ERROR;
	      end if;
      else
	      OUTER:
	      for I in WinNT.LCID range 1..31 loop -- try up to 31 languages
	         for J in WinNT.LCID range 1..7 loop -- try up to 7 sublanguages
	            LCID := 1024*J+I; -- compute LCID from language and sublanguage
	            COUNT := C.Size_T (WinNLS.GETLOCALEINFO
		            (LCID, WinNLS.LOCALE_ICOUNTRY, BUFFER(1)'Unchecked_Access,1024));
	            NEW_COUNTRY_CODE := STRING_TO_DECIMAL (BUFFER (1..COUNT));
	            if NEW_COUNTRY_CODE = COUNTRY_CODE then
		            exit OUTER;
	            end if;
	         end loop;
	      end loop OUTER;
	      if LCID = 0 then
            DOSE.ERROR ("Error on getting country info");
	         raise DOS_ERROR;
	      end if;
      end if;

      -- get country information from LCID

      Scratch := WinNLS.GETLOCALEINFO -- get date separator
	             (LCID, WinNLS.LOCALE_SDATE,BUFFER(1)'Unchecked_Access,1024);
      if BUFFER (1) = Win32.Char'Val (Character'Pos (':')) then -- it must be Japan Standard (h:m:s d:m:y)
	      COUNTRY_INFO.DATE_FORMAT := 2;
      elsif BUFFER (1) = Win32.Char'Val (Character'Pos ('/')) then -- either USA or Europe Standard
	      Scratch := WinNLS.GETLOCALEINFO -- get date ordering
	                  (LCID,WinNLS.LOCALE_IDATE,BUFFER(1)'Unchecked_Access,1024);
         if BUFFER (1) = Win32.Char'Val (Character'Pos('0')) then -- it must be USA Standard (h:m:s m/d/y)
	         COUNTRY_INFO.DATE_FORMAT := 0;
         elsif BUFFER (1) = WIn32.Char'Val (Character'Pos ('1')) then -- it must be European Standard
	         COUNTRY_INFO.DATE_FORMAT := 1;
	      else -- no standard possible, assume European
	         COUNTRY_INFO.DATE_FORMAT := 1;
	      end if;
      else -- no standard possible -- assume European
	      COUNTRY_INFO.DATE_FORMAT := 1;
      end if;

      Scratch := WinNLS.GETLOCALEINFO -- get currency symbol
	               (LCID, WinNLS.LOCALE_SCURRENCY,BUFFER(1)'Unchecked_Access,1024);
      COUNTRY_INFO.CURRENCY_SYMBOL (1..5) := Ada_String (BUFFER (1..5));

      Scratch := WinNLS.GETLOCALEINFO -- get thousands separator
	               (LCID,WinNLS.LOCALE_STHOUSAND,BUFFER(1)'Unchecked_Access,1024);
      COUNTRY_INFO.THOUSANDS_SEP (1..2) := Ada_String (BUFFER (1..2));

      Scratch := WinNLS.GETLOCALEINFO -- get decimal separator
	               (LCID,WinNLS.LOCALE_SDECIMAL,BUFFER(1)'Unchecked_Access,1024);
      COUNTRY_INFO.DECIMAL_SEP (1..2) := Ada_String (BUFFER (1..2));

      Scratch := WinNLS.GETLOCALEINFO -- get date separator
	               (LCID,WinNLS.LOCALE_SDATE,BUFFER(1)'Unchecked_Access,1024);
      COUNTRY_INFO.DATE_SEP (1..2) := Ada_String (BUFFER (1..2));

      Scratch := WinNLS.GETLOCALEINFO -- get time separator
	               (LCID,WinNLS.LOCALE_STIME,BUFFER(1)'Unchecked_Access,1024);
      COUNTRY_INFO.TIME_SEP (1..2) := Ada_String (BUFFER (1..2));

      COUNT := C.Size_T (WinNLS.GETLOCALEINFO -- get currency mask
	               (LCID,WinNLS.LOCALE_ICURRENCY,BUFFER(1)'Unchecked_Access,1024));
      COUNTRY_INFO.CURSYM_BEFORE_VAL :=
	               BYTE_8 (STRING_TO_DECIMAL (BUFFER (1..COUNT)));

      COUNT := C.Size_t (WinNLS.GETLOCALEINFO -- get number of decimal digits in currency
	              (LCID,WinNLS.LOCALE_ICURRDIGITS,BUFFER(1)'Unchecked_Access,1024));
      COUNTRY_INFO.CURRENCY_DIGITS :=
	               BYTE_8 (STRING_TO_DECIMAL (BUFFER (1..COUNT)));

      COUNT := C.Size_t (WinNLS.GETLOCALEINFO -- get time format
	              (LCID,WinNLS.LOCALE_ITIME,BUFFER(1)'Unchecked_Access,1024));
      COUNTRY_INFO.TIME_FORMAT :=
	              BYTE_8 (STRING_TO_DECIMAL (BUFFER (1..COUNT)));

      Scratch := WinNLS.GETLOCALEINFO -- get date separator
	             (LCID,WinNLS.LOCALE_SLIST,BUFFER(1)'Unchecked_Access,1024);
      COUNTRY_INFO.DATA_SEP (1..2) := Ada_String (BUFFER (1..2));

      COUNTRY_INFO.CASE_MAP_ADDRESS := System.Null_Address+System.Offset (LCID);  -- conversion!

      DEBUG ("Leaving GET_COUNTRY_INFO");
   end GET_COUNTRY_INFO;

   procedure SET_COUNTRY_CODE (COUNTRY_CODE: INTEGER) is
   begin
      DEBUG ("Entering SET_COUNTRY_INFO");
      DEBUG ("Leaving SET_COUNTRY_INFO");
   end SET_COUNTRY_CODE;

-- !! implementation error: UPPER_CASE doesn't appear to do anything

   function UPPER_CASE (
      CHAR: CHARACTER;
      CASE_MAP_ADDRESS: SYSTEM.ADDRESS)
      return CHARACTER is
	 SOURCE: aliased C_STRING (1..1) := (1 => Win32.Char'Val (Character'Pos (CHAR)));
	 RESULT: aliased C_STRING (1..10);
      use type Win32.Int;
      use SYstem;
   begin
      DEBUG ("Entering UPPER_CASE");
      if WinNLS.LCMAPSTRING (
	          WinNT.LCID(CASE_MAP_ADDRESS-System.Null_Address), -- locale identifier
	          WinNLS.LCMAP_UPPERCASE, -- type of mapping to be performed
             SOURCE(1)'Unchecked_Access, -- string to be mapped
	          1, -- number of characters in string to be mapped
             RESULT(1)'Unchecked_Access, -- buffer to store mapped string
	          Win32.Int (Result'Length) -- number of characters that result can hold
	      ) = 0 then
            DOSE.ERROR ("Error on mapping to upper case");
	      raise DOS_ERROR;
      end if;
      DEBUG ("Leaving UPPER_CASE");
      return Character'Val (Win32.Char'Pos (RESULT (1)));
   end UPPER_CASE;

-- ***************************************
-- * Function 39H -- Create Subdirectory *
-- ***************************************

   procedure MKDIR (DIRECTORY_NAME: STRING) is
      DIRECTORY_NAME_Z: C_STRING (C.Size_t (DIRECTORY_NAME'FIRST) .. C.Size_t (DIRECTORY_NAME'LAST+1))
	        := STRINGZ (DIRECTORY_NAME);
   begin
      DEBUG ("Entering MKDIR");
      if WINBASE.CREATEDIRECTORY
            (DIRECTORY_NAME_Z(1)'Unchecked_Access, null) = 0 then
         DOSE.ERROR ("Error on creating directory");
	      raise DOS_ERROR;
      end if;
      DEBUG ("Leaving MKDIR");
   end MKDIR;

-- ***************************************
-- * Function 3AH -- Remove Subdirectory *
-- ***************************************

   procedure RMDIR (DIRECTORY_NAME: STRING) is
      DIRECTORY_NAME_Z: aliased C_String (C.Size_t (DIRECTORY_NAME'FIRST) .. C.Size_t (DIRECTORY_NAME'LAST+1))
	      := STRINGZ (DIRECTORY_NAME);
   begin
      DEBUG ("Entering RMDIR");
      if WINBASE.REMOVEDIRECTORY (DIRECTORY_NAME_Z(1)'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on removing directory");
	      raise DOS_ERROR;
      end if;
      DEBUG ("Leaving RMDIR");
   end RMDIR;

-- ********************************************
-- * Function 3BH -- Change Current Directory *
-- ********************************************

   procedure CHDIR (DIRECTORY_NAME: STRING)is
      DIRECTORY_NAME_Z: aliased C_STRING (C.Size_T (DIRECTORY_NAME'FIRST) .. C.Size_T (DIRECTORY_NAME'LAST+1))
	      := STRINGZ (DIRECTORY_NAME);
   begin
      DEBUG ("Entering CHDIR");
      if WINBASE.SETCURRENTDIRECTORY (DIRECTORY_NAME_Z(1)'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on changing current directory");
	      raise DOS_ERROR;
      end if;
      DEBUG ("Leaving CHDIR");
   end CHDIR;

-- *******************************
-- * Function 3CH -- Create File *
-- *******************************

   function  Create_File (
	 FILE_NAME: STRING;
	 ATTRIBUTE: ATTR_TYPE := ATTR_NORMAL)
	 return HANDLE_TYPE is
      HANDLE: HANDLE_TYPE;
      FILE_NAME_Z: aliased C_STRING (C.Size_t (FILE_NAME'FIRST) .. C.Size_T (FILE_NAME'LAST+1))
	                   := STRINGZ (FILE_NAME);
      use type WinNT.Handle;
   begin
      DEBUG ("Entering CREATE_FILE");
      HANDLE :=  WINBASE.CREATEFILE (
         FILE_NAME_Z(1)'Unchecked_Access,
	      WinNT.GENERIC_WRITE + WinNT.GENERIC_READ,
	      0, -- share mode
	      null, -- pointer to optional security attributes
	      WINBASE.CREATE_ALWAYS, -- overwrite existing file
	      ATTRIBUTE,
	      null); -- optional file with attributes to copy
      if HANDLE /= WINBASE.INVALID_HANDLE_VALUE then
         DEBUG ("Leaving CREATE_FILE");
	      return HANDLE;
      else
         DOSE.ERROR ("Error on creating " & FILE_NAME);
	      raise DOS_ERROR;
      end if;
   end CREATE_FILE;

-- *****************************
-- * Function 3DH -- Open File *
-- *****************************

   function OPEN_FILE (
	 FILE_NAME: STRING;
	 OPEN_MODE: OPEN_TYPE := OPEN_READ_WRITE)
	 return HANDLE_TYPE is
      HANDLE: HANDLE_TYPE;
      FILE_NAME_Z: aliased C_STRING (C.Size_t (FILE_NAME'FIRST) .. C.Size_t (FILE_NAME'LAST+1))
	                   := STRINGZ (FILE_NAME);
      use type WinNT.Handle;
   begin
      DEBUG ("Entering OPEN_FILE");
      HANDLE :=  WINBASE.CREATEFILE (
         FILE_NAME_Z(1)'Unchecked_Access,
	      Open_Mode,
	      0, -- share mode
	      null, -- pointer to optional security attributes
	      WINBASE.Open_Existing, -- overwrite existing file
          Attr_Normal,
	      null); -- optional file with attributes to copy      
        if HANDLE /= WINBASE.INVALID_HANDLE_VALUE then
         DEBUG ("Leaving OPEN_FILE");
	      return HANDLE;
         else
         DOSE.ERROR ("Error on opening " & FILE_NAME);
	      raise DOS_ERROR;
      end if;
   end OPEN_FILE;

-- ******************************
-- * Function 3EH -- Close File *
-- ******************************

   procedure CLOSE_FILE (HANDLE: HANDLE_TYPE) is
      use type WinNT.Handle;
   begin
      DEBUG ("Entering CLOSE_FILE");
      if WINBASE.CLOSEHANDLE (HANDLE) = 0 then
         DOSE.ERROR ("Error on closing file");
	      raise DOS_ERROR;
      end if;
      DEBUG ("Leaving CLOSE_FILE");
   end CLOSE_FILE;

-- *****************************
-- * Function 3FH -- Read File *
-- *****************************

   procedure READ (
      BUFFER: out STRING;
      COUNT: out INTEGER;
      HANDLE: HANDLE_TYPE := STANDARD_INPUT) is
      Local_Count : aliased Win32.Dword;
   begin
      DEBUG ("Entering READ");
      if WINBASE.READFILE (
            HANDLE, BUFFER'ADDRESS, BUFFER'LENGTH,
            Local_COUNT'Unchecked_Access, null) = 0 then
         DOSE.ERROR ("Error on reading file");
         raise DOS_ERROR;
      end if;
      Count := Integer (Local_Count);
      DEBUG ("Leaving READ");
   end READ;

   procedure READ_ADR (
	 BUFFER: SYSTEM.ADDRESS;
	 COUNT: in out INTEGER;
	 HANDLE: HANDLE_TYPE := STANDARD_INPUT) is
      IN_COUNT: Win32.Dword := Win32.Dword (COUNT);
      Local_Count : aliased Win32.Dword := Win32.Dword (Count);
   begin
      DEBUG ("Entering READ_ADR");
      if WINBASE.READFILE (
	      HANDLE, BUFFER, IN_COUNT,
         Local_Count'Unchecked_Access, null) = 0 then
         DOSE.ERROR ("Error on reading file");
	      raise DOS_ERROR;
      end if;
      Count := Integer (Local_Count);
      DEBUG ("Leaving READ_ADR");
   end READ_ADR;

-- ******************************
-- * Function 40H -- Write File *
-- ******************************

   procedure WRITE (
	 BUFFER: STRING;
	 HANDLE: HANDLE_TYPE := STANDARD_OUTPUT) is
      OUT_COUNT: aliased Win32.Dword;
      use type WinNT.Handle;
   begin
      DEBUG ("Entering WRITE");

      -- Note that DOS treats zero-length writes as a file truncation whereas
      -- win32 does not.  So it must be special-cased in the code below.
      if HANDLE /= STANDARD_OUTPUT and then HANDLE /= STANDARD_ERROR and then
         BUFFER'LENGTH = 0 then
	      if WINBASE.SETENDOFFILE (HANDLE) = 0 then
            DOSE.ERROR ("Error on writing file -- truncation failed");
	         raise DOS_ERROR;
	      end if;
         OUT_COUNT := 0; -- avoids "disk full" error message below
      elsif WINBASE.WRITEFILE (
            HANDLE, BUFFER'ADDRESS, BUFFER'LENGTH,
            OUT_COUNT'Unchecked_Access, null) = 0 then
         DOSE.ERROR ("Error on writing file");
         raise DOS_ERROR;
      end if;
      if BUFFER'LENGTH /= OUT_COUNT then
         DOSE.ERROR ("Error on writing file -- disk full");
         raise DOS_ERROR;
      end if;
      DEBUG ("Leaving WRITE");
   end WRITE;

   procedure WRITE_ADR (
	 BUFFER: SYSTEM.ADDRESS;
	 COUNT: INTEGER;
	 HANDLE: HANDLE_TYPE := STANDARD_OUTPUT) is
      OUT_COUNT: aliased Win32.Dword;
      use type Win32.Dword;
   begin
      DEBUG ("Entering WRITE_ADR");

      -- Note that DOS treats zero-length writes as a file truncation whereas
      -- win32 does not.  So it must be special-cased in the code below.

      if COUNT = 0 then
	 if WINBASE.SETENDOFFILE (HANDLE) = 0 then
            DOSE.ERROR ("Error on writing file -- truncation failed");
	    raise DOS_ERROR;
	 end if;
         OUT_COUNT := 0; -- avoids "disk full" error message below
      elsif WINBASE.WRITEFILE (
	         HANDLE, BUFFER, Win32.Dword (COUNT),
            OUT_COUNT'Unchecked_Access, null) = 0 then
         DOSE.ERROR ("Error on writing file");
	 raise DOS_ERROR;
      end if;
      if Win32.Dword(COUNT) /= OUT_COUNT then
         DOSE.ERROR ("Error on writing file -- disk full");
	 raise DOS_ERROR;
      end if;
      DEBUG ("Leaving WRITE_ADR");
   end WRITE_ADR;

-- *******************************
-- * Function 41H -- Delete File *
-- *******************************

   procedure DELETE_FILE (FILE_NAME : STRING) is
      FILE_NAME_Z: aliased C_STRING (C.Size_t (FILE_NAME'FIRST) .. C.Size_t (FILE_NAME'LAST+1))
	 := STRINGZ (FILE_NAME);
   begin
      DEBUG ("Entering DELETE_FILE");
      if WINBASE.DELETEFILE (FILE_NAME_Z(1)'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on deleting file");
	 raise DOS_ERROR;
      end if;
      DEBUG ("Leaving DELETE_FILE");
   end DELETE_FILE;

-- ************************************************
-- * Function 42H -- Move File Read/Write Pointer *
-- ************************************************

   procedure LSEEK (
	 HANDLE: HANDLE_TYPE;
	 OFFSET: INTEGER;
	 METHOD: INTEGER) is
      POSITION: INTEGER;
   begin
      DEBUG ("Entering LSEEK procedure");
      POSITION := LSEEK (HANDLE, OFFSET, METHOD);
      DEBUG ("Leaving LSEEK procedure");
   end LSEEK;

   function LSEEK (
	 HANDLE: HANDLE_TYPE;
	 OFFSET: INTEGER;
	 METHOD: INTEGER)
	 return INTEGER is
      POSITION: Win32.Dword;
      DISTANCE_HIGH: aliased Win32.Long := 0;
      use type Win32.Dword;
      use type Win32.Long;
   begin
      DEBUG ("Entering LSEEK function");
      if OFFSET < 0 then
	 DISTANCE_HIGH := -1;
      end if;
      POSITION := WINBASE.SETFILEPOINTER (
         HANDLE, Win32.Long(OFFSET), DISTANCE_HIGH'Unchecked_Access, Win32.Dword (METHOD));
      if POSITION = -1 and DISTANCE_HIGH = -1 then
         DOSE.ERROR ("Error on seeking in file");
	 raise DOS_ERROR;
      end if;

      DEBUG ("Leaving LSEEK function");
      return Integer (POSITION);
   end LSEEK;

   function FILE_SIZE (HANDLE: HANDLE_TYPE) return INTEGER is
      CURRENT, LAST: INTEGER;
   begin
      DEBUG ("Entering FILE_SIZE");
      CURRENT := LSEEK (HANDLE, 0, FROM_CURRENT);
      LAST := LSEEK (HANDLE, 0, FROM_END);
      LSEEK (HANDLE, CURRENT, FROM_START);
      DEBUG ("Leaving FILE_SIZE");
      return LAST;
   END FILE_SIZE;

   function FILE_POSITION (HANDLE: HANDLE_TYPE) return INTEGER is
   begin
      DEBUG ("Entering and leaving FILE_POSITION");
      return LSEEK (HANDLE, 0, FROM_CURRENT);
   END FILE_POSITION;

-- ************************************
-- * Function 43H -- Change File Mode *
-- ************************************

   function GET_MODE (FILE_NAME: STRING) return ATTR_TYPE is
      FILE_NAME_Z: aliased C_STRING (C.Size_T (FILE_NAME'FIRST) .. C.Size_t (FILE_NAME'LAST+1))
	 := STRINGZ (FILE_NAME);
   begin
      DEBUG ("Entering and leaving GET_MODE");
      return WINBASE.GETFILEATTRIBUTES (FILE_NAME_Z(1)'Unchecked_Access);
   end GET_MODE;

   procedure CHMOD (FILE_NAME: STRING; ATTRIBUTE: ATTR_TYPE) is
      FILE_NAME_Z: aliased C_STRING (C.Size_t (FILE_NAME'FIRST) .. C.Size_t (FILE_NAME'LAST+1))
	 := STRINGZ (FILE_NAME);
   begin
      DEBUG ("Entering CHMOD");
      if WINBASE.SETFILEATTRIBUTES (FILE_NAME_Z(1)'Unchecked_Access, ATTRIBUTE) = 0 then
         DOSE.ERROR ("Error on changing file mode");
	 raise DOS_ERROR;
      end if;
      DEBUG ("Leaving CHMOD");
   end CHMOD;

-- *******************************************
-- * Function 44H -- I/O Control for Devices *
-- *******************************************

   procedure IOCTL_GET_DEVICE_INFO (
      HANDLE: HANDLE_TYPE;
      DEVICE_INFO: out DEVICE_INFO_RECORD) is -- not all entries supported
      use type WinNT.Handle;
   begin
      DEBUG ("Entering IOCTL_GET_DEVICE_INFO");
      DEVICE_INFO.ISDEV :=
	 WINBASE.GETFILETYPE (HANDLE) /= WINBASE.FILE_TYPE_CHAR;
      DEVICE_INFO.ISCOT := HANDLE = STANDARD_OUTPUT;
      DEVICE_INFO.ISCIN := HANDLE = STANDARD_INPUT;
      DEBUG ("Leaving IOCTL_GET_DEVICE_INFO");
   end IOCTL_GET_DEVICE_INFO;

-- *****************************************
-- * Function 45H -- Duplicate File Handle *
-- *****************************************

   function DUP (HANDLE: HANDLE_TYPE) return HANDLE_TYPE is
      NEW_HANDLE: aliased HANDLE_TYPE;
   begin
      DEBUG ("Entering DUP");
      if WINBASE.DUPLICATEHANDLE (
	    WINBASE.GETCURRENTPROCESS,	  -- source process
	    HANDLE,			  -- source handle
	    WINBASE.GETCURRENTPROCESS,	  -- target process
            NEW_HANDLE'Unchecked_Access,           -- target handle
	    0,				  -- access rights (ignored)
	    1,				  -- is handle inheritable
	    WinNT.DUPLICATE_SAME_ACCESS   -- options (access is to be ignored)
	    ) = 0 then
         DOSE.ERROR ("Error on duplicating handle");
	 raise DOS_ERROR;
      end if;
      DEBUG ("Leaving DUP");
      return NEW_HANDLE;
   end DUP;


-- *************************************************
-- * Function 46H -- Force a Duplicate of a Handle *
-- *************************************************

   procedure FORCDUP (EXISTING_HANDLE: HANDLE_TYPE; NEW_HANDLE: HANDLE_TYPE) is
      Local_New_Handle : aliased Handle_Type := New_Handle;
   begin
      DEBUG ("Entering FORCDUP");
      if WINBASE.CLOSEHANDLE (NEW_HANDLE) = 0 then
	 null; -- no error if handle was not previously opened
      end if;
      if WINBASE.DUPLICATEHANDLE (
	    WINBASE.GETCURRENTPROCESS,	  -- source process
	    EXISTING_HANDLE,		  -- source handle
	    WINBASE.GETCURRENTPROCESS,	  -- target process
            Local_NEW_HANDLE'Unchecked_Access,           -- target handle
	    0,				  -- access rights (ignored)
	    1,				  -- is handle inheritable
	    WinNT.DUPLICATE_SAME_ACCESS   -- options (access is to be ignored)
	    ) = 0 then
         DOSE.ERROR ("Error on forcing duplice handle");
	 raise DOS_ERROR;
      end if;
      DEBUG ("Leaving FORCDUP");
   end FORCDUP;

-- *****************************************
-- * Function 47H -- Get Current Directory *
-- *****************************************

   function CURRENT_DIRECTORY (DRIVE: DRIVE_LETTER := '@') return STRING is
      BUFFER: aliased C_STRING (1..1024);
      SIZE: C.Size_t;
      OLD_DRIVE_NAME_Z: aliased C_STRING (1..3) := STRINGZ (" :");
      NEW_DRIVE_NAME_Z: aliased C_STRING (1..3) := STRINGZ (" :");
      use type C.Size_t;
   begin
      DEBUG ("Entering CURRENT_DIRECTORY");

      -- change to designated drive but save original drive

      if DRIVE /= '@' then
         if WINBASE.GETCURRENTDIRECTORY (1024, BUFFER(1)'Unchecked_Access) = 0 then
            DOSE.ERROR ("Error on getting current directory");
	    raise DOS_ERROR;
	 end if;
	 OLD_DRIVE_NAME_Z (1) := BUFFER (1);

	 NEW_DRIVE_NAME_Z (1) := Win32.Char'Val (Drive_Letter'Pos(DRIVE));
         if WINBASE.SETCURRENTDIRECTORY (NEW_DRIVE_NAME_Z(1)'Unchecked_Access) = 0 then
            DOSE.ERROR ("Error on getting current directory");
	    raise DOS_ERROR;
	 end if;
      end if;

      -- get current directory on designated drive

      SIZE := C.Size_t (WINBASE.GETCURRENTDIRECTORY (1024, BUFFER(1)'Unchecked_Access));
      if SIZE = 0 then
         DOSE.ERROR ("Error on getting current directory");
	 raise DOS_ERROR;
      end if;

      -- restore original drive

      if DRIVE /= '@' then
         if WINBASE.SETCURRENTDIRECTORY (OLD_DRIVE_NAME_Z(1)'Unchecked_Access) = 0 then
            DOSE.ERROR ("Error on getting current directory");
	    raise DOS_ERROR;
	 end if;
      end if;

      DEBUG ("Leaving CURRENT_DIRECTORY");
      
      return Ada_String (Buffer (3..SIZE)); -- do not include <drive>:\ on returned value
   end CURRENT_DIRECTORY;

-- **********************************************
-- * Function 4BH -- Load and Execute a Program *
-- **********************************************

   procedure EXEC (PROGRAM_FILE: STRING; COMMAND_LINE: STRING := "") is
      PROGRAM_LENGTH: INTEGER := 0;
      COMMAND_LINE_Z: aliased C_STRING (1..C.Size_t (PROGRAM_FILE'LENGTH+1+COMMAND_LINE'LENGTH+1));
      ERRCODE: aliased WIN32.Dword;
      SI: aliased WINBASE.STARTUPINFO;
      PI: aliased WINBASE.PROCESS_INFORMATION;
      -- Cannot use WINBASE.INFINITE since it will raise constraint error.  So
      -- use the following INFINITE until the error in WINBASE is corrected.
      INFINITE: constant := 16#FFFF_FFFF#; -- 16#7FFFFFFF# - 16#80000000#;  -- what about '-1'?

      function LOWER_CASE (C: CHARACTER) return CHARACTER is
      begin
         if CHARACTER'POS (C) >= CHARACTER'POS ('A') and
               CHARACTER'POS (C) <= CHARACTER'POS ('Z') then
            return CHARACTER'VAL (CHARACTER'POS (C) -
               CHARACTER'POS ('A') + CHARACTER'POS ('a'));
	 else
	    return C;
	 end if;
      end LOWER_CASE;

      function LOWER_CASE (S: STRING) return STRING is
         RESULT: STRING (S'FIRST .. S'LAST);
      begin
         for I in S'FIRST .. S'LAST loop
	    RESULT (I) := LOWER_CASE (S (I));
	 end loop;
	 return RESULT;
      end LOWER_CASE;

      use type Win32.Bool;
   begin
      DEBUG ("Entering EXEC");
      COMMAND_LINE_Z (1..C.Size_t (PROGRAM_FILE'LENGTH)) := To_C_String (PROGRAM_FILE);
      for I in 1..PROGRAM_FILE'LENGTH loop
         exit when  COMMAND_LINE_Z (C.Size_t (I)) = Win32.Char'Val (Character'Pos (ASCII.NUL));
         PROGRAM_LENGTH := I;
      end loop;
      COMMAND_LINE_Z (C.Size_T(PROGRAM_LENGTH+1)) := Win32.Char'Val (Character'Pos (' '));
      COMMAND_LINE_Z (C.Size_T (PROGRAM_LENGTH+2)..C.Size_T (PROGRAM_LENGTH+COMMAND_LINE'LENGTH+1))
	       := To_C_String (COMMAND_LINE);
      COMMAND_LINE_Z (C.Size_T (PROGRAM_LENGTH + COMMAND_LINE'LENGTH + 2)) := Win32.Char'Val (Character'Pos(ASCII.NUL));
      WINBASE.GETSTARTUPINFO (SI'Unchecked_Access);
      if WINBASE.CREATEPROCESS (
	         null, -- image name
            COMMAND_LINE_Z(1)'Unchecked_Access,
	         null, -- process attributes
	         null, -- thread attributes
	         0, -- new process inherit handles?
	         0, -- creation flags
	         null, -- optional environment
	         null, -- new current directory
            SI'Unchecked_Access, -- startup information
            PI'Unchecked_Access) -- process information
	    = 0 then
         DOSE.ERROR ("Error on creating process");
	 raise DOS_ERROR;
      else
	 if WINBASE.WAITFORSINGLEOBJECT (PI.HPROCESS, Infinite) = -1 then
            DOSE.ERROR ("Error on waiting for created process to terminate");
	    raise DOS_ERROR;
	 else
    -- raise Not_Implemented;
	declare
  	  function Convert is new Unchecked_Conversion(Win32.lpVoid, Win32.lpDWord);
      -- Exit_Code'Unchecked_Access should do!
    begin
	  if WINBASE.GETEXITCODEPROCESS (
                  PI.HPROCESS, Convert(EXIT_CODE'Address)) = 0 then
                DOSE.ERROR ("Error on getting exit code of created process");
	       raise DOS_ERROR;
	  end if;
	end;
            if LOWER_CASE (PROGRAM_FILE) /= "command.com" then
	       -- ?? why is above test needed
	       if WINBASE.CLOSEHANDLE (PI.HTHREAD) = 0 then
                  DOSE.ERROR ("Error on closing thread of created process");
		  raise DOS_ERROR;
	       end if;
	    end if;
	    if WINBASE.CLOSEHANDLE (PI.HPROCESS) = 0 then
               DOSE.ERROR ("Error on closing created process");
	       raise DOS_ERROR;
	    end if;
	 end if;
      end if;
      DEBUG ("Leaving EXEC");
   end EXEC;

   procedure EXEC_COMMAND (DOS_COMMAND: STRING) is
   begin
      DEBUG ("Entering EXEC_COMMAND");
      EXEC ("command.com", " /c " & DOS_COMMAND);
      DEBUG ("Leaving EXEC_COMMAND");
   end EXEC_COMMAND;

   procedure EXEC_COMMANDS is
   begin
      DEBUG ("Entering EXEC_COMMANDS");
      EXEC ("command.com");
      DEBUG ("Leaving EXEC_COMMANDS");
   end EXEC_COMMANDS;

-- ***************************************
-- * Function 4CH -- Terminate a Process *
-- ***************************************

   procedure EXIT_PROCESS (RETURN_CODE: WORD_16 := 0) is
   begin
      DEBUG ("Entering EXIT_PROCESS");
      WINBASE.EXITPROCESS (WIN32.UINT (RETURN_CODE));
   end EXIT_PROCESS;

-- ***************************************************
-- * Function 4DH -- Get Return Code of a Subprocess *
-- ***************************************************

   function WAIT_PROCESS return WORD_16 is
      LOCAL_EXIT_CODE: WORD_16;
   begin
      DEBUG ("Entering WAIT_PROCESS");
      LOCAL_EXIT_CODE := EXIT_CODE;
      EXIT_CODE := 0;
      DEBUG ("Leaving WAIT_PROCESS");
      return LOCAL_EXIT_CODE;
   end WAIT_PROCESS;

-- ***********************************
-- * Function 4EH -- Find First File *
-- ***********************************

   procedure FIND_FIRST (
	 FILE_PATTERN: STRING;
	 SEARCH_BUF: out SEARCH_RECORD;
	 FILE_FOUND: out BOOLEAN;
	 ATTRIBUTE: ATTR_TYPE := ATTR_NORMAL) is
      FILE_PATTERN_Z: aliased C_STRING
         (C.Size_t (FILE_PATTERN'FIRST) .. C.Size_T (FILE_PATTERN'LAST+1))
	    := STRINGZ (FILE_PATTERN);
      LOCAL_FILE_FOUND: BOOLEAN;
      HANDLE: WinNT.HANDLE;
      File_Date : aliased Packed_Date;
      File_Time : aliased Packed_Time;
      FTLASTWRITETIME : aliased Winbase.Filetime;
      use type WinNT.Handle ;
   begin
      DEBUG ("Entering FIND_FIRST");
      FIND_ATTRIBUTE := ATTRIBUTE;
      HANDLE := WINBASE.FINDFIRSTFILE (
         FILE_PATTERN_Z(1)'Unchecked_Access, FIND_DATA'Unchecked_Access );
	   SEARCH_BUF.RESERVED (1..4) := HANDLE_TO_STRING (HANDLE);
      LOCAL_FILE_FOUND := HANDLE /= WINBASE.INVALID_HANDLE_VALUE;
      FILE_FOUND := LOCAL_FILE_FOUND;
      if not LOCAL_FILE_FOUND then
         DEBUG ("Prematurely leaving FIND_FIRST");
	 return;
      end if;

--    The following loop is necessary if we want to find only those files
--    whose attributes match the specified attribute.  However it has been
--    observed that DOS doesn't make this test although the documentation
--    says it does.  So for compatibility with what DOS does (not what it
--    should do), the loop is commented out
--
--    while FIND_DATA.DWFILEATTRIBUTES /= FIND_ATTRIBUTE loop
--	 LOCAL_FILE_FOUND :=
--	    WINBASE.FINDNEXTFILE
--            (FIND_HANDLE, FIND_DATA'ADDRESS ) = 0;
--	  FILE_FOUND := LOCAL_FILE_FOUND;
--	 if not LOCAL_FILE_FOUND then
--	    return;
--	 end if;
--	end loop;
      FTLASTWRITETIME := Find_Data.FTLASTWRITETIME;
      if WINBASE.FILETIMETODOSDATETIME (
            FTLASTWRITETIME'Unchecked_Access,
            FILE_DATE'Unchecked_Access,
            FILE_TIME'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on converting to DOS date/time");
	 raise DOS_ERROR;
      end if;
      Search_Buf.File_Date := File_Date;
      Search_Buf.File_Time := File_Time;
      Search_Buf.FILE_SIZE := Integer (FIND_DATA.NFILESIZELOW);
      Search_Buf.FILE_NAMX := Ada_String (FIND_DATA.Cfilename) (1..Max_Filename);
      DEBUG ("Leaving FIND_FIRST");
   end FIND_FIRST;

-- **********************************
-- * Function 4FH -- Find Next File *
-- **********************************

   procedure FIND_NEXT (
	 SEARCH_BUF: in out SEARCH_RECORD;
	 FILE_FOUND: out BOOLEAN) is
      File_Date : aliased Packed_Date;
      File_Time : aliased Packed_Time;
      LOCAL_FILE_FOUND: BOOLEAN;
      HANDLE: HANDLE_TYPE;
      FTLASTWRITETIME : aliased Winbase.Filetime;
      use type WinNT.Handle;
   begin
      DEBUG ("Entering FIND_NEXT");

--    The following loop is commented out for the same reasons that the
--     loop in FIND_FIRST is commented out

--    loop
	 HANDLE := STRING_TO_HANDLE (SEARCH_BUF.RESERVED (1..4));
	 if HANDLE = WINBASE.INVALID_HANDLE_VALUE then
            DOSE.ERROR ("Error on FIND_NEXT");
	    raise DOS_ERROR;
	 end if;
	 LOCAL_FILE_FOUND := WINBASE.FINDNEXTFILE (
               HANDLE, FIND_DATA'Unchecked_Access) /= 0;
	 FILE_FOUND := LOCAL_FILE_FOUND;
	 if not LOCAL_FILE_FOUND then
            DEBUG ("Prematurely leaving FIND_NEXT");
	    return;
	 end if;
--	 exit when FIND_DATA.DWFILEATTRIBUTES = FIND_ATTRIBUTE;
--    end loop;

      FTLASTWRITETIME := Find_Data.FTLASTWRITETIME;
      if WINBASE.FILETIMETODOSDATETIME (
            FTLASTWRITETIME'Unchecked_Access,
            FILE_DATE'Unchecked_Access,
            FILE_TIME'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on converting to DOS date/time");
         raise DOS_ERROR;
      end if;
      Search_Buf.File_Date := File_Date;
      Search_Buf.File_Time := File_Time;
      SEARCH_BUF.FILE_SIZE := Integer (FIND_DATA.NFILESIZELOW);
      SEARCH_BUF.FILE_NAMX := Ada_String (FIND_DATA.CFILENAMe)(1 .. MAX_FILENAME);
      DEBUG ("Leaving FIND_NEXT");
   end FIND_NEXT;

-- *******************************
-- * Function 56H -- Rename File *
-- *******************************

   procedure RENAME_FILE (OLD_FILE_NAME : STRING; NEW_FILE_NAME : STRING) is
      OLD_FILE_NAME_Z: aliased C_STRING (C.Size_t (OLD_FILE_NAME'FIRST) .. C.Size_T (OLD_FILE_NAME'LAST+1))
	                       := STRINGZ (OLD_FILE_NAME);
      NEW_FILE_NAME_Z: aliased C_STRING (C.Size_t (NEW_FILE_NAME'FIRST) .. C.Size_T (NEW_FILE_NAME'LAST+1))
	                       := STRINGZ (NEW_FILE_NAME);
   begin
      DEBUG ("Entering RENAME_FILE");
      if WINBASE.MOVEFILE (OLD_FILE_NAME_Z(1)'Unchecked_Access, NEW_FILE_NAME_Z(1)'Unchecked_Access)
	    = 0 then
         DOSE.ERROR ("Error on renaming file");
         raise DOS_ERROR;
      end if;
      DEBUG ("Leaving RENAME_FILE");
   end RENAME_FILE;

-- ******************************************
-- * Function 57H -- Get/Set File Date/Time *
-- ******************************************

   procedure GET_FILE_DATE_TIME (
	 HANDLE: HANDLE_TYPE;
	 DATE: out PACKED_DATE;
	 TIME: out PACKED_TIME) is
      Local_Date : aliased Packed_Date;
      Local_Time : aliased Packed_time;
      CREATIONTIME, LASTACCESSTIME, LASTWRITETIME: aliased WINBASE.FILETIME;
      ZONE, TEMP_ZONE: aliased WINBASE.TIME_ZONE_INFORMATION;
   begin

      -- establish a time zone with no bias

      if WINBASE.GETTIMEZONEINFORMATION (ZONE'Unchecked_Access) = -1 then
         DOSE.ERROR ("Error on converting to DOS date/time");
	      raise DOS_ERROR;
      end if;
      TEMP_ZONE := ZONE;
      ZONE.BIAS := 0;
      if WINBASE.SETTIMEZONEINFORMATION (ZONE'Unchecked_Access) = -1 then
         DOSE.ERROR ("Error on converting to DOS date/time");
	      raise DOS_ERROR;
      end if;

      -- get file date/time

      if WINBASE.GETFILETIME (
	         HANDLE,
            CREATIONTIME'Unchecked_Access,
            LASTACCESSTIME'Unchecked_Access,
            LASTWRITETIME'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on getting file date/time");
	      raise DOS_ERROR;
      end if;

      -- convert to DOS format

      if WINBASE.FILETIMETODOSDATETIME (
            LASTWRITETIME'Unchecked_Access,
            Local_DATE'Unchecked_Access,
            Local_TIME'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on converting to DOS date/time");
	      raise DOS_ERROR;
      end if;

      Date := Local_Date;
      Time := Local_Time;

      -- restore original time-zone bias

      ZONE := TEMP_ZONE;
      if WINBASE.SETTIMEZONEINFORMATION (ZONE'Unchecked_Access) = -1 then
         DOSE.ERROR ("Error on converting to DOS date/time");
         raise DOS_ERROR;
      end if;

   end GET_FILE_DATE_TIME;

   procedure SET_FILE_DATE_TIME (
      HANDLE: HANDLE_TYPE;
      DATE: PACKED_DATE;
      TIME: PACKED_TIME) is

      Local_Date : aliased constant PACKED_DATE := Date;
      Local_Time : aliased constant Packed_Time := Time; 
      CREATIONTIME, LASTACCESSTIME, LASTWRITETIME: aliased WINBASE.FILETIME;
      ZONE : aliased WINBASE.TIME_ZONE_INFORMATION;
      Temp_Zone : WINBASE.TIME_ZONE_INFORMATION;
   begin

      -- establish a time zone with no bias

      if WINBASE.GETTIMEZONEINFORMATION (ZONE'Unchecked_Access) = -1 then
         DOSE.ERROR ("Error on converting to DOS date/time");
	      raise DOS_ERROR;
      end if;
      TEMP_ZONE := ZONE;
      ZONE.BIAS := 0;
      if WINBASE.SETTIMEZONEINFORMATION (ZONE'Unchecked_Access) = -1 then
         DOSE.ERROR ("Error on converting to DOS date/time");
	      raise DOS_ERROR;
      end if;

      -- get file date/time

      if WINBASE.GETFILETIME (
	         HANDLE,
            CREATIONTIME'Unchecked_Access,
            LASTACCESSTIME'Unchecked_Access,
            LASTWRITETIME'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on getting file date/time");
	      raise DOS_ERROR;
      end if;

      -- insert new date/time

      if WINBASE.DOSDATETIMETOFILETIME (
            Local_DATE,
	         Local_TIME,
            LASTWRITETIME'Unchecked_Access) = 0 then
            DOSE.ERROR ("Error on converting from DOS date/time");
	         raise DOS_ERROR;
      end if;

      -- set file date/time

      if WINBASE.SETFILETIME (
            HANDLE,
            CREATIONTIME'Unchecked_Access,
            LASTACCESSTIME'Unchecked_Access,
            LASTWRITETIME'Unchecked_Access) = 0 then
         DOSE.ERROR ("Error on setting file date/time");
         raise DOS_ERROR;
      end if;

      -- restore original time-zone bias

      ZONE := TEMP_ZONE;
      if WINBASE.SETTIMEZONEINFORMATION (ZONE'Unchecked_Access) = -1 then
         DOSE.ERROR ("Error on converting to DOS date/time");
	 raise DOS_ERROR;
      end if;

   end SET_FILE_DATE_TIME;

-- **************************************
-- * Function 5AH -- Create Unique File *
-- **************************************

   procedure CREATE_UNIQUE_FILE (
	 PATH: STRING;
	 FILE_NAME: out STRING8;
	 HANDLE: out HANDLE_TYPE;
	 ATTRIBUTE: ATTR_TYPE := ATTR_NORMAL) is
      LOCAL_HANDLE: HANDLE_TYPE;
      VALUE: Win32.UInt;
      PATH_Z: aliased C_STRING (C.Size_T (PATH'FIRST) .. C.Size_T (PATH'LAST+1)) := STRINGZ (PATH);
      PREFIX_Z: aliased constant C_STRING := STRINGZ ("TMP");
      FILE_NAME_Z, FILE_NAME2_Z: aliased C_STRING (1..C.Size_t(PATH'LAST+MAX_FILENAME));
      use type WinNT.Handle;
      use type C.Size_t;
      subtype Size_t is C.Size_t;  -- OA Feature?
   begin
      DEBUG ("Entering CREATE_UNIQUE_FILE");

      -- create unique file with name of form tmpxxxx.tmp

      VALUE := WINBASE.GETTEMPFILENAME
         (PATH_Z(1)'Unchecked_Access, PREFIX_Z(1)'Unchecked_Access, 0, FILE_NAME_Z(1)'Unchecked_Access);

      -- remove .tmp suffix from unique file name

      -- FILE_NAME := "        "; -- initialize file name to all blanks
      -- FILE_NAME2_Z := FILE_NAME_Z;
      File_Name := (others => ' ');
      File_Name2_Z := (others => Win32.Char'Val (Character'Pos(' ')));
      for I in Size_t range 1..8 loop
         if FILE_NAME2_Z (C.Size_T (PATH'LAST) + I) = Win32.Char'Val (Character'Pos('.')) then
            FILE_NAME2_Z (C.Size_T (PATH'LAST) + I) := Win32.char'Val (Character'Pos(ASCII.NUL));
	         exit;
	      end if;
         FILE_NAME (Integer (I)) := Character'Val (Win32.Char'Pos(FILE_NAME2_Z (C.Size_t (PATH'LAST) + I)));
      end loop;

      -- rename file removing the .tmp suffix

      if WINBASE.MOVEFILE (FILE_NAME_Z(1)'Unchecked_Access, FILE_NAME2_Z(1)'Unchecked_Access) = 0 then
         -- Can't rename file which means that file without the .tmp suffix
	      -- already exists.  Therefore we try again recursively.  But we must
	      -- delete the file with the .tmp suffix.  We do this deletion after the
         -- recursion finishes so that the recursion won't try to pick this
	      -- same name which we failed on.
	      CREATE_UNIQUE_FILE (PATH, FILE_NAME, HANDLE, ATTRIBUTE);
         if WINBASE.DELETEFILE (FILE_NAME_Z(1)'Unchecked_Access) = 0 then
            DOSE.ERROR ("Error on creating unique file");
	         raise DOS_ERROR;
	      end if;
	      return;
      end if;

      -- Open the file

      LOCAL_HANDLE := WINBASE.CREATEFILE (
         FILE_NAME2_Z(1)'Unchecked_Access,
	      WinNT.GENERIC_WRITE + WinNT.GENERIC_READ,
	      0, -- share mode
	      null, -- pointer to optional security attributes
	      WINBASE.OPEN_EXISTING, -- open file that was just created
	      ATTRIBUTE,
	      null); -- optional file with attributes to copy

      -- see if file was opened successfully

      if LOCAL_HANDLE /= WINBASE.INVALID_HANDLE_VALUE then
	      HANDLE := LOCAL_HANDLE;
         DEBUG ("Leaving CREATE_UNIQUE_FILE");
      else
         DOSE.ERROR ("Error on creating unique file");
	      raise DOS_ERROR;
      end if;
   end CREATE_UNIQUE_FILE;

-- ***********************************
-- * Function 5BH -- Create New File *
-- ***********************************

   function CREATE_NEW_FILE (
	 FILE_NAME: STRING;
	 ATTRIBUTE: ATTR_TYPE := ATTR_NORMAL)
	 return HANDLE_TYPE is
      HANDLE: HANDLE_TYPE;
      FILE_NAME_Z: aliased C_STRING (C.Size_T (FILE_NAME'FIRST) .. C.Size_t (FILE_NAME'LAST+1))
	                   := STRINGZ (FILE_NAME);
      use type WinNT.Handle;
   begin
      DEBUG ("Entering CREATE_NEW_FILE");
      HANDLE :=  WINBASE.CREATEFILE (
         FILE_NAME_Z(1)'Unchecked_Access,
	 WinNT.GENERIC_WRITE + WinNT.GENERIC_READ,
	 0, -- share mode
	 null,  -- pointer to optional security attributes
	 WINBASE.CREATE_NEW, -- create new file
	 ATTRIBUTE,
	 null); -- optional file with attributes to copy
      if HANDLE /= WINBASE.INVALID_HANDLE_VALUE then
         DEBUG ("Leaving CREATE_NEW_FILE");
         return HANDLE;
      else
         DOSE.ERROR ("Error on creating " & FILE_NAME);
         raise DOS_ERROR;
      end if;
   end CREATE_NEW_FILE;

-- *******************************************
-- * Function 5CH -- Lock/Unlock File Access *
-- *******************************************

   procedure LOCK (HANDLE: HANDLE_TYPE; OFFSET: INTEGER; LENGTH: INTEGER) is
   begin
      DEBUG ("Entering LOCK");
      if WINBASE.LOCKFILE (HANDLE, Win32.Dword(OFFSET), 0, Win32.Dword (LENGTH), 0) = 0 then
         DOSE.ERROR ("Error on locking file");
         raise DOS_ERROR;
      end if;
      DEBUG ("Leaving LOCK");
   end LOCK;

   procedure UNLOCK (HANDLE: HANDLE_TYPE; OFFSET: INTEGER; LENGTH: INTEGER) is
   begin
      DEBUG ("Entering UNLOCK");
      if WINBASE.UNLOCKFILE (HANDLE, Win32.Dword (OFFSET), 0, Win32.Dword(LENGTH), 0) = 0 then
         DOSE.ERROR ("Error on unlocking file");
         raise DOS_ERROR;
      end if;
      DEBUG ("Leaving UNLOCK");
   end UNLOCK;

-- *******************************
-- * Environment Access Routines *
-- *******************************

   function GET_ENV_STRING (NAMESTR: STRING) return STRING is
      ENVSTR_Z: aliased C_STRING (1..1024);
      NAMESTR_Z: aliased C_STRING (C.Size_t (NAMESTR'FIRST) .. C.Size_t (NAMESTR'LAST+1))
	                := STRINGZ (NAMESTR);
      ENV_SIZE: C.Size_t;
      use type C.Size_t;
   begin
      DEBUG ("Entering GET_ENV_STRING");
      ENV_SIZE := C.Size_t (WINBASE.GETENVIRONMENTVARIABLE (
         NAMESTR_Z(1)'Unchecked_Access, ENVSTR_Z(1)'Unchecked_Access, Win32.Dword (ENVSTR_Z'Length)));
      DEBUG ("Leaving GET_ENV_STRING");
      return Ada_String (EnvStr_Z (1..ENV_SIZE)); -- ENV_SIZE doesn't include terminating NUL
   end GET_ENV_STRING;

   function GET_PROGRAM_NAME return STRING is
      PROGRAM_NAME: aliased C_STRING (1..1024);
      PROGRAM_SIZE: C.Size_T;
      FULL_PATH_NAME: aliased C_STRING (1..1024);
      FULL_PATH_SIZE: C.Size_t;
      FILENAME_OFFSET: aliased Win32.LPSTR;
      use type C.Size_t;
      use type C_String;
   begin
      DEBUG ("Entering GET_PROGRAM_NAME");

      -- get name by which program was invoked

      PROGRAM_SIZE := ARG_VALUE(0)'LAST;
      PROGRAM_NAME (1..PROGRAM_SIZE) := ARG_VALUE (0) (1 .. PROGRAM_SIZE);
      PROGRAM_NAME (PROGRAM_SIZE + 1) := Win32.Char'Val (Character'Pos (ASCII.NUL)); -- terminate it with 0

      -- convert to full pathname

      FULL_PATH_SIZE := C.Size_t (WINBASE.GETFULLPATHNAME (
         PROGRAM_NAME(1)'Unchecked_Access, -- name of file to find path for
	      1024, -- size of buffer to hold full path name
         FULL_PATH_NAME(1)'Unchecked_Access, -- buffer to hold full path name
         FILENAME_OFFSET'Unchecked_Access)); -- offset of filename within full path name

      -- add .exe suffix to full pathname

      FULL_PATH_NAME (FULL_PATH_SIZE+1 .. FULL_PATH_SIZE+4 ) := ".exe";
      FULL_PATH_SIZE := FULL_PATH_SIZE + 4;

      DEBUG ("Leaving GET_PROGRAM_NAME");
      return Ada_String (FULL_PATH_NAME (1	.. FULL_PATH_SIZE));

   end GET_PROGRAM_NAME;

   function GET_PARMS return STRING is
      PARMS_BUFFER: C_STRING (1..1024);
      PARMS_LENGTH: C.Size_t := 0;
      ARG_LENGTH: C.Size_t;
      use type C.Size_t;
      subtype Size_t is C.Size_T;
   begin
      DEBUG ("Entering GET_PARMS");
      for I in 1 .. Ada.Command_Line.Argument_COUNT loop
	      if I /= 1 then
	       PARMS_LENGTH := PARMS_LENGTH + 1;
          PARMS_BUFFER (PARMS_LENGTH) := Win32.Char'Val (Character'Pos (' '));
	      end if;
         ARG_LENGTH := ARG_VALUE (I)'LAST;
	      PARMS_BUFFER
	         (PARMS_LENGTH + 1 .. PARMS_LENGTH + ARG_LENGTH) :=
	          ARG_VALUE (I);
	      PARMS_LENGTH := PARMS_LENGTH + ARG_LENGTH;
      end loop;
      -- Debug (Ada_String (Parms_Buffer(1..Parms_Length)));
      DEBUG ("Leaving GET_PARMS");
      return Ada_String (Parms_Buffer(1..Parms_Length));
   end GET_PARMS;

--  ***********************
--  * Conversion Routines *
--  ***********************

   function UNPACK_DATE (DATEP : PACKED_DATE) return DATE_RECORD is
      DATEU: DATE_RECORD;
      I: INTEGER := INTEGER (DATEP);
   begin
      DATEU.DAY := I mod 32;
      I := I / 32;
      DATEU.MONTH := I mod 16;
      I := I / 16;
      DATEU.YEAR := 1980 + I;
      return DATEU;
   end UNPACK_DATE;

   function PACK_DATE (DATEU : DATE_RECORD) return PACKED_DATE is
      I: INTEGER;
   begin
      I := (DATEU.YEAR-1980)*2**9 + DATEU.MONTH*2**5 + DATEU.DAY;
      return PACKED_DATE (I);
   end PACK_DATE;

   function UNPACK_TIME (TIMEP : PACKED_TIME) return TIME_RECORD is
      TIMEU: TIME_RECORD;
      I: INTEGER := INTEGER (TIMEP);
   begin
      TIMEU.HUNDREDTHS := 0;
      TIMEU.SECONDS := 2*(I mod 32);
      I := I / 32;
      TIMEU.MINUTES := I mod 64;
      I := I / 64;
      TIMEU.HOURS := I;
      return TIMEU;
   end UNPACK_TIME;

   function PACK_TIME (TIMEU : TIME_RECORD) return PACKED_TIME is
      I: INTEGER;
   begin
      I := TIMEU.HOURS*2**11 + TIMEU.MINUTES*2**5 + TIMEU.SECONDS/2;
      return PACKED_TIME (I);
   end PACK_TIME;

   function GET_STRING (STR: STRING) return STRING is
   begin
      DEBUG ("Entering GET_STRING");
      for I in STR'FIRST .. STR'LAST loop
	 if STR (I) = ASCII.NUL then
            DEBUG ("Leaving GET_STRING");
            return STR (STR'FIRST .. I-1);
	 end if;
      end loop;
      DEBUG ("Leaving GET_STRING");
      return STR;
   end GET_STRING;


-- *******************
-- * Initializations *
-- *******************
   use type WinNT.Handle;
begin

   -- create a handle for the console input buffer
   CONSOLE_NAME_Z := STRINGZ ("CONIN$");
   KEYBOARD :=	WINBASE.CREATEFILE (
      CONSOLE_NAME_Z(1)'Unchecked_Access,
      WinNT.GENERIC_READ + WinNT.GENERIC_WRITE,
      WinNT.FILE_SHARE_READ, -- share mode
      null, -- pointer to optional security attributes
      WINBASE.OPEN_EXISTING, -- overwrite existing file
      0, -- attributes
       null); -- optional file with attributes to copy
   if KEYBOARD = WINBASE.INVALID_HANDLE_VALUE then
      DOSE.ERROR ("Error on opening console");
   end if;

end DOS;
-- VAX/DEC CMS REPLACEMENT HISTORY, Element DOS.ADB
-- *6     9-FEB-1995 15:31:18 JLO "Support long file name"
-- *5    12-DEC-1994 17:59:06 JLO "Fix error caused by new Win32 binding"
-- *4    22-NOV-1994 14:39:39 JLO "Emulate DOS zero length writes only for none standard I/O devices"
-- *3     8-SEP-1994 00:23:20 MORSE "MANY MORE FUNCTIONS IMPLEMENTED"
-- *2    25-AUG-1994 21:08:24 MORSE "ADDED DELETE_FILE AND RENAME_FILE IN ORDER TO SUPPORT COUNT AND SPLIT"
-- *1    23-JUN-1994 21:10:44 MORSE "DOS PACKAGE FOR NT"
-- VAX/DEC CMS REPLACEMENT HISTORY, Element DOS.ADB
