-- $Source: /home/dsd/d/aa/monoBANK/AA/TSP_system.rts.bitfield_utils.spc,v $ 
-- $Revision: 1.1 $ $Date: 96/04/24 14:46:35 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

package System.RTS.Bitfield_Utils is

--*OVERVIEW
-- This package provides the necessary functions and procedures to
-- manipulate bitfields.
--
--* EFFECTS
-- Bitfields are essentially considered to be arrays of one-bit elements.
-- In a "signed" bitfield, the first element of the bitfield is
-- considered to be a sign bit.
--

    pragma Suppress( Elaboration_Check );

    type Element_Size_Type is new Integer range 1..System.Word_Size;

    ------------------------------------------------------------
    --------------- Bitfield Extract Routines ------------------
    ------------------------------------------------------------
 
    function RTS_Extract_Signed_Bitfield( 
      Src_Addr  : in System.Address;
      Src_Off   : in Natural;
      Size      : in Element_Size_Type ) return Integer;
    pragma Import( ada, 
                   rts_extract_signed_bitfield, 
                   "rts_extract_signed_bitfield" ); 
    pragma Global( rts_extract_signed_bitfield); 

    --*OVERVIEW
    -- RTS_Extract_Signed_Bitfield is used to retrieve a signed bitfield.
    --*EFFECTS
    -- RTS_Extract_Signed_Bitfield accepts the base address of a
    -- bitfield, the offset of the desired bitfield, and the width of
    -- the bitfield.
    -- The return value is the value of the designated bitfield, with
    -- sign extension of the most significant bit.
    --

    function RTS_Extract_Unsigned_Bitfield( 
      Src_Addr  : in System.Address;
      Src_Off   : in Natural;
      Size      : in Element_Size_Type ) return Natural;
    pragma Import( ada, 
                   RTS_Extract_Unsigned_Bitfield, 
                   "rts_extract_unsigned_bitfield" ); 
    pragma Global( RTS_Extract_Unsigned_Bitfield); 

    --*OVERVIEW
    -- RTS_Extract_Unsigned_Bitfield is used to retrieve an unsigned bitfield.
    --*EFFECTS
    -- RTS_Extract_Unsigned_Bitfield accepts the base address of a
    -- bitfield, the offset of the desired bitfield, and the width of
    -- the bitfield.
    -- The return value is the value of the designated bitfield, with
    -- zeroes extension of the most significant bit.
    --

    ------------------------------------------------------------
    ---------------- Bitfield Assign Routines ------------------
    ------------------------------------------------------------
    
      
    procedure RTS_Assign_Bitfield(
      Src_Value : in Integer;
      Dest_Addr : in System.Address;
      Dest_Off  : in Natural;
      Size      : in Element_Size_Type );
    pragma Import( ada, 
                   RTS_Assign_Bitfield, 
                   "rts_assign_bitfield" ); 
    pragma Global( RTS_Assign_Bitfield ); 
    --*OVERVIEW
    -- RTS_Assign_Bitfield is used to assign into a bitfield.
    --*EFFECTS
    -- RTS_Assign_Bitfield accepts the base address of a bitfield,
    -- the offset of the bitfield, and the width of the bitfield.
    -- The assigned value is the contents of Src_Value.
    --

    ------------------------------------------------------------
    -------------------- Slice Assign Routines -----------------
    ------------------------------------------------------------

    procedure RTS_Copy_Bitfield( 
      Src_Addr  : in System.Address;
      Src_Off   : in Natural;
      Src_Wid   : in Integer;
      Dest_Addr : in System.Address;
      Dest_Off  : in Natural;
      Dest_Wid  : in Integer );
    pragma Import( ada, 
                   RTS_Copy_Bitfield, 
                   "rts_copy_bitfield" ); 
    pragma Global( RTS_Copy_Bitfield );
    --*OVERVIEW
    -- RTS_Bitfield is used to assign one bitfield to another.  The
    -- major difference between this operation and the others in
    -- this package is that this operation is not restricted in size
    -- to less than a word.
    --*EFFECTS
    -- RTS_Copy_Bitfield performs the following...
    --
    -- Dest( Dest_Off..Dest_Off+Numb-1 ) := Src( Src_Off..Src_Off+Numb-1 );
    -- 
    -- Numb refers to the number of bits, the minimum of the number of
    -- bits specified in the source and destination bitfields.
    -- If the source and destination overlap, this routine will act as
    -- if a temporary were used. ( sec 5.2 )
    
end System.RTS.Bitfield_Utils;
