-- == % filter "-- "
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ***************************************************************************
--
-- PACKAGE SPECIFICATION System.RTS.TGT.Exceptions.DEBUG
-- ------------------------------
--
-- OUTLINE
-- -------
-- This module defines switches controlling exception manager debug output.
--
-- ***************************************************************************

package System.RTS.TGT.Exceptions.DEBUG is

   pragma SUPPRESS (ELABORATION_CHECK);
   pragma ELABORATE_BODY;
   
   --	    *********************
   --	    * TABLE OF CONTENTS *
   --	    *********************

   --	      1. Debug Control Switches
   --	      2. Revision History


   --	    *****************************
   --	    * 1. DEBUG CONTROL SWITCHES *
   --	    *****************************

--    == % if DEBUG
--    DEBUG : constant BOOLEAN := TRUE;
--    -- This switch enables debug code. If DEBUG is FALSE then no output is
--    -- generated regardless of the setting of the other switches defined here.
--    -- The intention is that this be a BOOLEAN constant which will cause the
--    -- inclusion of debug code if it is set TRUE.
-- 
--    -- The following variables are set at program start-up.
--    
--    TRACE_MIN   : BOOLEAN := FALSE;
--    TRACE_MAX   : BOOLEAN := FALSE;
--    TRACE_TABLE : BOOLEAN := FALSE;
--    -- These switches enable the generation of debug output during propagation
--    -- and handling of exceptions.
-- 
--    DUMP_TABLE_SWITCH : BOOLEAN := FALSE;
--    -- Controls dumping of exception tables at elaboration time
-- 
--    DUMP_CONTEXT_SWITCH : BOOLEAN := FALSE;
--    -- Controls dumping of exception context
-- 
--    == % elsif ~DEBUG
   DEBUG,
   DUMP_TABLE_SWITCH,
   DUMP_CONTEXT_SWITCH : constant BOOLEAN := FALSE;
--    == % end if

   procedure INIT_EM_DEBUG;

end System.RTS.TGT.Exceptions.DEBUG;

--	    ***********************
--	    * 2. REVISION HISTORY *
--	    ***********************

--  0.01  15 Oct 1990, 07:05 (RBKD)
--        Initial version

--  0.02  20 Dec 1990, 01:37 (RBKD)
--        Initial version for common runtime source checkin (no changes)

--  6.01  20 Dec 1990 (BJD)
--        Initial introduction to CMS v6.

--  6.02  21 May 1991 (BJD)
--        Add null body in line with coding standards.

--  6.03  7 Jun 1991 (BJD)
--        Use TRACES settings to control debug output generation.

--  6.04  3rd November 1991 (BJD)
--        Prefix all CRTS library unit names with ALSYS_

--  6.05  24th February 1994 (Russell March)
--        Moved body to separate file for Ada 9X. See Change Log
--        X6.01/30 for details.
--  VAX/DEC CMS REPLACEMENT HISTORY, Element EMDEBUG.ADS
--  *5     5-MAR-1994 07:03:19 CMS_DAEMON "ltd_daemon:March::Suppress dummy body for Ada 9x"
--  *4     4-NOV-1991 23:22:16 CMS_DAEMON "ltd_daemon:DOBBING:Add ALSYS_ prefix"
--  *3    10-JUN-1991 22:41:30 CMS_DAEMON "ltd_daemon:DOBBING:Add trace constants"
--  *2    31-MAY-1991 21:29:21 CMS_DAEMON "ltd_daemon:DOBBING:Add support for traceback generation"
--  *1    21-DEC-1990 21:47:38 CMS_DAEMON "Mimic From France"
--  VAX/DEC CMS REPLACEMENT HISTORY, Element EMDEBUG.ADS
