-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ***************************************************************************
--
-- PACKAGE SPECIFICATION ALSYS_EM_HIST
-- -----------------------------
--
-- OUTLINE
-- -------
-- This module contains the subprograms to output the exception traceback.
-- No assumption is made about the whereabouts of the history data, nor
-- about its format. This information is encapsulated in separate target-
-- dependent files.
--
-- ***************************************************************************

with System.RTS.TGT.Exceptions.TDEP;
with System.Rts.Types;
with System.RTS.Exception_Types;

package System.RTS.TGT.Exceptions.HIST is

   pragma SUPPRESS (ELABORATION_CHECK);

   --	    *********************
   --	    * TABLE OF CONTENTS *
   --	    *********************

   --	      1. Type Definitions
   --	      2. Traceback Generation Subprograms
   --	      Revision History

   -- ------------------------------------------
   -- Package renamings to remove prefix:
   -- ------------------------------------------
   package EM_TDEP renames System.RTS.TGT.Exceptions.TDEP;
   package RT_TYPES renames System.Rts.Types;
   -- ------------------------------------------


   --	    ***********************
   --	    * 1. TYPE DEFINITIONS *
   --	    ***********************

   type EXCEPTION_EFFECT is (DEAD_PROGRAM, DEAD_TASK, HANDLED);


   --	    ***************************************
   --	    * 2. Traceback Generation Subprograms *
   --	    ***************************************

   procedure PRINT_TRACEBACK_HEADER (
      EFFECT     : EXCEPTION_EFFECT;
      OCCURRENCE : System.RTS.Exception_Types.Exception_Occurrence_Access);
   -----------------------------------------------------------------------
   -- OUTLINE:
   -- --------
   --    Outputs the header lines in the traceback. EFFECT indicates what
   -- the effect of the exception is, eg. program aborted, and ECODE
   -- indicates the exception code to allow for printing its identifier.
   -----------------------------------------------------------------------


   procedure PRINT_PROCEDURE_INFO (CONTEXT : EM_TDEP.REF_EXCEPTION_CONTEXT);
   -----------------------------------------------------------------------
   -- OUTLINE:
   -- --------
   --    Outputs one line in the traceback table corresponding to CONTEXT.
   -- Typically, this includes CU name, Proc name and line number.
   -----------------------------------------------------------------------


   procedure PRINT_HANDLER_INFO (CONTEXT     : EM_TDEP.REF_EXCEPTION_CONTEXT;
                                 WHEN_OTHERS : BOOLEAN);
   -----------------------------------------------------------------------
   -- OUTLINE:
   -- --------
   --    Outputs the line in the traceback table corresponding to the 
   -- location of the handler. This is only called if the handler is in
   -- user code space. CONTEXT indicates the address of the handler.
   -- This procedure can be null if such information is not included in
   -- the traceback output for a particular target.
   -----------------------------------------------------------------------


   procedure PRINT_TRACEBACK_FOOTER (EFFECT : EXCEPTION_EFFECT);
   -----------------------------------------------------------------------
   -- OUTLINE:
   -- --------
   --    Outputs the footer lines in the traceback. EFFECT indicates what
   -- the effect of the exception is.
   -----------------------------------------------------------------------

end System.RTS.TGT.Exceptions.HIST;

--	    *********************
--	    *  REVISION HISTORY *
--	    *********************


--  0.01  14 Oct 1990, 16:48 (RBKD)
--        Initial version of this module

--  0.02  20 Dec 1990, 01:37 (RBKD)
--        Initial version for common runtime source checkin (no changes)

--  6.01  20 Dec 1990 (BJD)
--        Initial introduction to CMS v6.

--  6.02  14 May 1991 (BJD)
--        HISTORY_TABLE type(s) moved to EM_TDEP since it is target-dependent.
--        Traceback generation subprogram interface added.

-- 6.??   3rd November 1991 (BJD)
--        Prefix all CRTS library unit names with ALSYS_
