-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.finalization_pkg.bdy,v $ 
-- $Revision: 1.19 $ $Date: 97/04/21 18:45:46 $ $Author: stt $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

--DBG-- with Ada.Text_IO;
with Ada.Unchecked_Conversion;
with System.RTS.Exception_Manager;
with System.RTS.Master_Pkg;
with System.RTS.Memory_Pkg;
with System.RTS.TGT.Kernel.Abort_Signals;
with System.RTS.TGT.Sec_Stack_Pkg;
with System.Storage_Elements;

package body System.RTS.Finalization_Pkg is

    pragma Suppress( Elaboration_Check );

    package EM   renames System.RTS.Exception_Manager;
    package MEM  renames System.RTS.Memory_Pkg;

    subtype Storage_Count is System.Storage_Elements.Storage_Count;
    use type Storage_Count;

    use type RTT.Master_Access;
    use type System.Address;
    use type Secondary_Mark_Ptr;

    Null_Secondary_Mark_Ptr : Secondary_Mark_Ptr renames
	System.RTS.TGT.Sec_Stack_Pkg.Null_Mark_Ptr;

    procedure Finalize_And_Pop(Master : RTT.Master_Access) renames
	System.RTS.Master_Pkg.RTS_Finalize_And_Pop;

    function SS_Copy_And_Release(
	From : System.Address;
	To   : Secondary_Mark_Ptr;
	Size : Natural
    ) return System.Address renames
	System.RTS.TGT.Sec_Stack_Pkg.Copy_And_Release;
    function SS_Allocate(Size : Natural) return System.Address renames
	System.RTS.TGT.Sec_Stack_Pkg.Allocate;

    procedure Defer_Abort renames
	TGT.Kernel.Abort_Signals.Defer_Abort_Handling;
    procedure Undefer_Abort renames
	TGT.Kernel.Abort_Signals.Undefer_Abort_Handling;

    procedure Cleanup(Rec : Simple_Cleanup) is
    begin
	-- Call cleanup operation on object (unless null)
	-- Note: Presumes Abort already deferred when called
	if Rec.Obj /= System.Null_Address then
	    Rec.Cleanup_Op(Rec.Obj);
	end if;
    exception
	when others =>
	    raise Program_Error;
    end Cleanup;

    function I(X : System.Address) return String is
	use System.Storage_Elements;
    begin
        return Integer_Address'Image(To_Integer(X));
    end I;

--DBG--     Debug : Boolean := False;

--DBG--     procedure Debug_Put_Line(Msg : String);
--DBG--     procedure Debug_Put_Line(Msg : String; Addr : System.Address);
--DBG--     pragma Inline(Debug_Put_Line);

--DBG--     procedure Debug_Put_Line(Msg : String) is
--DBG--     begin
--DBG--         if Debug then
--DBG--             Ada.Text_IO.Put_Line(Msg);
--DBG--         end if;
--DBG--     end Debug_Put_Line;

--DBG--     procedure Debug_Put_Line(Msg : String; Addr : System.Address) is
--DBG--     begin
--DBG-- 	if Debug then
--DBG-- 	    Debug_Put_Line(Msg & I(Addr));
--DBG-- 	end if;
--DBG--     end Debug_Put_Line;

    procedure Register_In_Temps_Master(Ptr : RTT.Cleanup_Ptr);
    pragma Inline(Register_In_Temps_Master);

    procedure Register_In_Temps_Master(Ptr : RTT.Cleanup_Ptr) is
	Temps_Master : RTT.Master_Record renames
	  RTT.Self.Innermost_Submaster.all;
    begin
	Ptr.Previous := Temps_Master.Cleanup_Actions;
        Temps_Master.Cleanup_Actions := Ptr;
	  -- TBD this should be atomic
    end Register_In_Temps_Master;

    procedure Register_In_Master(Ptr : RTT.Cleanup_Ptr;
				 Master : RTT.Master_Access);
    pragma Inline(Register_In_Master);

    procedure Register_In_Master(Ptr : RTT.Cleanup_Ptr;
				 Master : RTT.Master_Access) is
	Local_Master : RTT.Master_Access := Master;
    begin
	if Local_Master = null then
	    Local_Master := RTT.Default_Master'Access;
	end if;
	Ptr.Previous := Local_Master.Cleanup_Actions;
        Local_Master.Cleanup_Actions := Ptr;
	  -- TBD this should be atomic
    end Register_In_Master;

    procedure Flush_Master(Master : RTT.Master_Access);
    pragma Inline(Flush_Master);

    procedure Flush_Master(Master : RTT.Master_Access) is
    begin
	Master.Cleanup_Actions := null;
    end;


    function Caller_Simple_Cleanup(Caller_Master : RTT.Master_Access)
		 return Simple_Cleanup_Ptr is
    begin
        if Caller_Master = null then
            return Simple_Cleanup_Ptr(
              RTT.Self.Innermost_Submaster.Cleanup_Actions);
        else
            return Simple_Cleanup_Ptr(
	      Caller_Master.Enclosing_Master.Cleanup_Actions);
        end if;
    end Caller_Simple_Cleanup;

    function Caller_Array_Cleanup(Caller_Master : RTT.Master_Access)
		 return Array_Cleanup_Ptr is
    begin
	if Caller_Master = null then
	    return Array_Cleanup_Ptr(
	      RTT.Self.Innermost_Submaster.Cleanup_Actions);
	else
	    return Array_Cleanup_Ptr(
	      Caller_Master.Enclosing_Master.Cleanup_Actions);
	end if;
    end Caller_Array_Cleanup;


    procedure Check_Abort is
    begin
	Undefer_Abort;
	Defer_Abort;
    end Check_Abort;

    procedure Default_Init_And_Register(
	Master : in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Cleanup_Op : in Operation_Ptr;  -- TBD: Static link
	Init_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access)
    is
    -- Overview:
    --   This defers abort, and then calls Init_Op on Obj.
    --   If Init_Op returns normally, the cleanup record
    --   is registered with the given obj and cleanup operation
	Rec : aliased Simple_Cleanup :=
		(RTT.Cleanup_Record with Obj, Cleanup_Op);
	for Rec'Address use Cleanup_Rec;
    begin
--DBG-- 	Debug_Put_Line("Default_Init_And_Register on", Obj);

	Defer_Abort;
	    -- Abort deferred section
	    begin
		Init_Op(Obj);
	    exception
		when others =>
		    Undefer_Abort;
		    raise;
	    end; -- handler
            if Flush /= null then
              Flush_Master(Flush);
            end if;
	    Register_In_Master(Rec'Unchecked_Access, Master);
	Undefer_Abort;
    end Default_Init_And_Register;
	
    procedure Adjust_And_Register(
	Master : in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Adjust_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access)
    is
    -- Overview:
    --   This defers abort, and then calls Adjust_Op on Obj.
    --   If Adjust_Op returns normally, the cleanup record
    --   is registered with the given obj and cleanup op.
    --   Program_Error is propagated if Adjust_Op propagates
    --   an exception.

	Rec : aliased Simple_Cleanup :=
		(RTT.Cleanup_Record with Obj, Cleanup_Op);
	for Rec'Address use Cleanup_Rec;
    begin
--DBG-- 	Debug_Put_Line("Adjust_And_Register on", Obj);

	Defer_Abort;
	    -- Abort deferred section
	    begin
		Adjust_Op(Obj);
	    exception
		when others =>
		    Undefer_Abort;
		    raise Program_Error;
	    end; -- handler
            if Flush /= null then
              Flush_Master(Flush);
            end if;
	    Register_In_Master(Rec'Unchecked_Access, Master);
	Undefer_Abort;

    end Adjust_And_Register;

    procedure Register_For_Cleanup(
	Master : in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access)
    is
    -- Overview:
    --   This registers the cleanup record with the given
    --   obj and cleanup op.

	Rec : aliased Simple_Cleanup :=
		(RTT.Cleanup_Record with Obj, Cleanup_Op);
	for Rec'Address use Cleanup_Rec;
    begin
--DBG-- 	Debug_Put_Line("Register_For_Cleanup on", Obj);
	if Flush /= null then
	    Defer_Abort;
		Flush_Master(Flush);
		Register_In_Master(Rec'Unchecked_Access, Master);
	    Undefer_Abort;
	else
	    Register_In_Master(Rec'Unchecked_Access, Master);
	end if;
    end Register_For_Cleanup;


    procedure Finalize_Copy_And_Adjust(
	Target_Obj : in System.Address;
	Source_Obj : in System.Address;
	Storage_Size : in Storage_Count;
	Controlled_Part : in System.Address;
	Cleanup_Op : in Operation_Ptr;		-- TBD: Static Link
	Adjust_Op : in Operation_Ptr)		-- TBD: Static link
    is
    -- Overview:
    --    This defers abort, finalizes Controlled_Part uping Cleanup_Op,
    --    and if Cleanup_Op returns normally, copies Source_Obj to
    --    Target_Obj (Storage_Size gives number of storage elements),
    --    and then calls Adjust_Op on Controlled_Part.
    --    If either Cleanup_Op or Adjust_Op propagates an exception,
    --    Program_Error is raised.
    --    Note that abort is allowed to occur between finalize and copy, 
    --    but not between copy and adjust.
    begin
--DBG-- 	Debug_Put_Line("Finalize_Copy_And_Adjust on", Controlled_Part);
	Defer_Abort;
	    -- Abort deferred section
	    begin
		Cleanup_Op(Controlled_Part);
	    exception
		when others =>
		    Undefer_Abort;
		    raise Program_Error;
	    end; -- handler
	Check_Abort;  -- Check if aborted during finalization
	    MEM.RTS_Memmove(Source_Obj, Target_Obj, Natural(Storage_Size));
	    begin
		Adjust_Op(Controlled_Part);
	    exception
		when others =>
		    Undefer_Abort;
		    raise Program_Error;
	    end; -- handler
	Undefer_Abort;
    end Finalize_Copy_And_Adjust;
	    

    procedure Preregister_Func_Result(
	Master : in RTT.Master_Access;
	Cleanup_Rec : in System.Address)
    is
    -- Overview:
    --    This registers a cleanup record for the result of a
    --    function call.  It must be called immediately prior
    --    to calling any function returning an object requiring
    --    cleanup.  The cleanup record is only partially filled
    --    in at this point.

	Rec : aliased Simple_Cleanup :=
		(RTT.Cleanup_Record with System.Null_Address, null);
	for Rec'Address use Cleanup_Rec;
    begin
--DBG-- 	Debug_Put_Line("Preregister_Func_Result");
	Register_In_Master(Rec'Unchecked_Access, Master);
    end Preregister_Func_Result;

    procedure Reregister_Func_Result(
	Master : in RTT.Master_Access;
	Obj : in System.Address;			-- interesting part
	Obj_Cleanup_Rec : in Simple_Cleanup_Ptr)
    is
    -- Overview:
    --    This is to be called before returning a local object
    --    from a function into caller-allocated space.  It cancels
    --    the finalization for the object in the local scope
    --    (the finalization associated with Obj_Cleanup_Rec).
    --    Master is finalized and popped, and Obj is registered
    --    on a master in the caller of the function, using a
    --    cleanup record which was specified using
    --    Register_Func_Result_Before_Call.

	Caller_Cleanup : Simple_Cleanup_Ptr := Caller_Simple_Cleanup(Master);
    begin
--DBG-- 	Debug_Put_Line("Reregister_Func_Result on", Obj);
	Defer_Abort;
	    -- Abort deferred section
	    Obj_Cleanup_Rec.Obj := System.Null_Address;
	    Caller_Cleanup.Cleanup_Op := Obj_Cleanup_Rec.Cleanup_Op;
	    Caller_Cleanup.Obj := Obj;
	Undefer_Abort;
	if Master /= null then
	    Finalize_And_Pop(Master);
	end if;
    end Reregister_Func_Result;

    procedure Adjust_And_Reregister_Func_Result(
	Master : in RTT.Master_Access;
	Obj : in System.Address;			-- interesting part
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Adjust_Op : in Operation_Ptr)	-- TBD: Static link
    is
    -- Overview:
    --    This is to be called before returning a value from
    --    a function into caller-allocated space.  Obj is adjusted
    --    and registered on a master in the caller of the function,
    --    using a cleanup record which was specified using
    --    Register_Func_Result_Before_Call.  Master is finalized
    --    and popped.

	Caller_Cleanup : Simple_Cleanup_Ptr := Caller_Simple_Cleanup(Master);
    begin
--DBG-- 	Debug_Put_Line("Adjust_And_Reregister_Func_Result on", Obj);

	Defer_Abort;
	    -- Abort deferred section
	    begin
		Adjust_Op(Obj);
	    exception
		when others =>
		    Undefer_Abort;
		    raise Program_Error;
	    end; -- handler
	    Caller_Cleanup.Cleanup_Op := Cleanup_Op;
	    Caller_Cleanup.Obj := Obj;
	Undefer_Abort;
	if Master /= null then
	    Finalize_And_Pop(Master);
	end if;
    end Adjust_And_Reregister_Func_Result;

    function Reregister_Func_Result_Copy_On_SS(
	Master : in RTT.Master_Access;
	Obj : in System.Address;
	Storage_Size : in System.Storage_Elements.Storage_Count;
	Obj_Cleanup_Rec : in Simple_Cleanup_Ptr;
	Target_Mark : Secondary_Mark_Ptr
    ) return System.Address is

    -- Overview:
    --    This is to be called before returning a local object from
    --    a function.  It cancels the finalization for the object in
    --    the local scope (the finalization associated with
    --    Obj_Cleanup_Rec).  Master is finalized and popped.  Obj
    --    is copied to Target_Mark in the secondary stack, which is
    --    released to the end of the copy of Obj.  The interesting
    --    part of the copy of Obj (as determined from Obj_Cleanup_Rec)
    --    is registered on a master in the caller of the function,
    --    using a cleanup record which was specified using
    --    Register_Func_Result_Before_Call.

	Caller_Cleanup : Simple_Cleanup_Ptr := Caller_Simple_Cleanup(Master);
	Obj_Interesting_Offset : System.Storage_Elements.Storage_Offset;
	Return_Obj : System.Address := System.Null_Address;
	Exception_Raised : Boolean := False;
    begin
--DBG-- 	Debug_Put_Line("Reregister_Func_Result_Copy_On_SS on", Obj);
	Defer_Abort;
	    -- Abort deferred section
	    Obj_Interesting_Offset := Obj_Cleanup_Rec.Obj - Obj;
	    Obj_Cleanup_Rec.Obj := System.Null_Address;
	    
	    if Master /= null then
		begin
		    Finalize_And_Pop(Master);
		exception
		    when others =>
			Exception_Raised := True;
		end; -- handler
	    end if;

	    Caller_Cleanup.Cleanup_Op := Obj_Cleanup_Rec.Cleanup_Op;
	    if Target_Mark = Null_Secondary_Mark_Ptr then
		Return_Obj := SS_Allocate(Natural(Storage_Size));
		MEM.RTS_Memmove(Obj, Return_Obj, Natural(Storage_Size));
	    else
		Return_Obj :=
		  SS_Copy_And_Release(Obj, Target_Mark, Natural(Storage_Size));
	    end if;
	    Caller_Cleanup.Obj := Return_Obj + Obj_Interesting_Offset;
	Undefer_Abort;

	if Exception_Raised then
	    raise Program_Error;
	end if;

	return Return_Obj;
    end Reregister_Func_Result_Copy_On_SS;

    function Adjust_And_Reregister_Func_Result_Copy_On_SS(
	Master : in RTT.Master_Access;
	Obj : in System.Address;
	Storage_Size : in System.Storage_Elements.Storage_Count;
	Obj_Cleanup_Rec : in Simple_Cleanup_Ptr;
	Adjust_Op : in Operation_Ptr;	-- TBD: Static link
	Target_Mark : Secondary_Mark_Ptr
    ) return System.Address is

    -- Overview:
    --    This is to be called before returning a local object with
    --    aliased components from a function.  It proceeds as
    --    Reregister_Func_Result_Copy_On_SS, except that after
    --    copying Obj to the secondary stack, it adjusts the copy
    --    and finalizes the original.
    begin
	-- **** TBD ****
	return Reregister_Func_Result_Copy_On_SS(
		 Master, Obj, Storage_Size, Obj_Cleanup_Rec, Target_Mark);
    end Adjust_And_Reregister_Func_Result_Copy_On_SS;



    -----------------------------------------------------------------
    ----             A R R A Y S                                 ----
    -----------------------------------------------------------------

    procedure Cleanup(Rec : Array_Cleanup) is
      -- Overview:
      -- Cleanup an entire array; raise Program_Error if
      -- Cleanup_Op propagates any exception, after
      -- cleaning up rest of array.
      -- If Rec.Num_Components is zero, no cleanup is performed.
      -- Note: Presumes Abort already deferred when called

	Obj : System.Address := Rec.Obj;
	Exception_Was_Raised : Boolean := False;
    begin
	for I in 1..Rec.Num_Components loop
	    begin
		Rec.Cleanup_Op(Obj);
	    exception
		when others =>
		    Exception_Was_Raised := True;
	    end;  -- handler

	    -- Advance to next array component
	    Obj := Obj + Rec.Component_Size;
	end loop;
	if Exception_Was_Raised then
	    raise Program_Error;
	end if;
    end Cleanup;

    procedure Per_Component_Array_Op(
	Obj : in System.Address;			-- interesting part
	Rec : in Array_Cleanup;
	Op : in Operation_Ptr;	-- TBD: Static link
	Propagate_Exception : Boolean)
    is
    -- Call Op on each element of array given by Obj and Rec.
    -- If any Op raises an exception, finalize each element of the
    -- array, and either propagate the exception that was raised
    -- (if Propagate_Exception is True) or raise Program_Error
    -- (if Propagate_Exception is False).
	
	Component : System.Address := Obj;
    begin
	for I in 1..Rec.Num_Components loop
	    begin
		Op(Component);
	    exception
		when others =>
		    -- Clean up those already inited/adjusted, in reverse order
		    declare
			Cleanup_Succeeded : Boolean := True;
		    begin
			for J in reverse 1..I-1 loop
			    Component := Component - Rec.Component_Size;
			    begin
				Rec.Cleanup_Op(Component);
			    exception
				when others =>
				    Cleanup_Succeeded := False;
			    end; -- handler
			end loop;
			Undefer_Abort;
			if Cleanup_Succeeded and then Propagate_Exception then
			    raise;  -- propagate exception from Register_Op
			else
			    raise Program_Error;
			end if;
		    end; -- declare
	    end; -- handler

	    -- Advance to next component
	    Component := Component + Rec.Component_Size;
	end loop;
    end Per_Component_Array_Op;

    procedure Default_Init_And_Register_Array(
	Master: in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Component_Size : in Storage_Count;
	Num_Components : in Natural;
	Cleanup_Op : in Operation_Ptr;  -- TBD: Static link
	Init_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access)
    is
    -- Overview:
    --   This defers abort, and then calls Init_Op on components of Obj.
    --   If Init_Op returns normally, the cleanup record
    --   is registered with the given obj and cleanup operation

	Rec : aliased Array_Cleanup :=
		(RTT.Cleanup_Record with Obj, Cleanup_Op, Component_Size,
					 Num_Components);
	for Rec'Address use Cleanup_Rec;

	Component : System.Address := Obj;
    begin
--DBG-- 	Debug_Put_Line("Default_Init_And_Register_Array on", Obj);
	Defer_Abort;
	    -- Abort deferred section
	    Per_Component_Array_Op(Obj, Rec, Init_Op,
				   Propagate_Exception => True);
	    Register_In_Master(Rec'Unchecked_Access, Master);
	Undefer_Abort;
    end Default_Init_And_Register_Array;
                  
    procedure Adjust_And_Register_Array(
	Master: in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Component_Size : in Storage_Count;
	Num_Components : in Natural;
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Adjust_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access)
    is
    -- Overview:
    --   This defers abort, and then calls Adjust_Op on comps of Obj.
    --   If Adjust_Op returns normally, the cleanup record
    --   is registered with the given obj and cleanup op.
    --   Program_Error is propagated if Adjust_Op propagates
    --   an exception.

	Rec : aliased Array_Cleanup :=
		(RTT.Cleanup_Record with Obj, Cleanup_Op, Component_Size,
					 Num_Components);
	for Rec'Address use Cleanup_Rec;

	Component : System.Address := Obj;
    begin
--DBG-- 	Debug_Put_Line("Adjust_And_Register_Array on", Obj);

	Defer_Abort;
	    -- Abort deferred section
	    Per_Component_Array_Op(Obj, Rec, Adjust_Op,
				   Propagate_Exception => False);
	    Register_In_Master(Rec'Unchecked_Access, Master);
	Undefer_Abort;
    end Adjust_And_Register_Array;

    procedure Register_Array_For_Cleanup(
	Master: in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Component_Size : in Storage_Count;
	Num_Components : in Natural;
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access)
    is
    -- Overview:
    --   This registers the cleanup record with the given obj and cleanup op.

	Rec : aliased Array_Cleanup :=
		(RTT.Cleanup_Record with Obj, Cleanup_Op, Component_Size,
					 Num_Components);
	for Rec'Address use Cleanup_Rec;
    begin
--DBG-- 	Debug_Put_Line("Register_Array_For_Cleanup on", Obj);
	Register_In_Master(Rec'Unchecked_Access, Master);
    end Register_Array_For_Cleanup;

    procedure Bump_Array_Register(
	Cleanup_Rec : in out Array_Cleanup;
	New_Components : in Natural;
	Flush : in RTT.Master_Access)
    is
    -- Overview:
    --   This incrementally registers new elements of a previously
    --   registered array.
    begin
	if Flush /= null then
	    Defer_Abort;
		-- Abort deferred section
		Cleanup_Rec.Num_Components :=
		  Cleanup_Rec.Num_Components + New_Components;
		Flush_Master(Flush);
	    Undefer_Abort;
	else
	    Cleanup_Rec.Num_Components :=
	      Cleanup_Rec.Num_Components + New_Components;
	end if;
    end Bump_Array_Register;

    procedure Finalize_Copy_And_Adjust_Array(
	Target_Obj : in System.Address;
	Source_Obj : in System.Address;
	Component_Size : in Storage_Count;
	Num_Components : in Natural;
	Controlled_Part : in System.Address;
	Cleanup_Op : in Operation_Ptr;		-- TBD: Static Link
	Adjust_Op : in Operation_Ptr)		-- TBD: Static link
    is
    -- Overview:
    --    This defers abort, and then one component at a time,
    --    finalizes component of Controlled_Part using Cleanup_Op,
    --    and if Cleanup_Op returns normally, copies component of 
    --    Source_Obj to component of Target_Obj (Component_Size
    --    determines number of SUs to copy)
    --    and then calls Adjust_Op on component of Controlled_Part.
    --    If either Cleanup_Op or Adjust_Op propagates an exception,
    --    Program_Error is raised.
    --    Note that abort is allowed to occur between finalize and copy, 
    --    but not between copy and adjust of a given component.
    --    Operation is performed left-to-right or right-to-left
    --    as necessary, depending on addresses.  If addresses
    --    are the same, nothing is done.
	Target_Component : System.Address;
	Source_Component : System.Address;
	Controlled_Component : System.Address;
	use type System.Address;

	Total_Size : constant Storage_Count :=
	  Storage_Count(Num_Components-1) * Component_Size;
    begin
	if Target_Obj < Source_Obj then

	    -- Copy starting at beginning of arrays

	    Target_Component := Target_Obj;
	    Source_Component := Source_Obj;
	    Controlled_Component := Controlled_Part;
	    Defer_Abort;
		-- Abort deferred section
		for I in 1..Num_Components loop
		    -- First cleanup
		    begin
			Cleanup_Op(Controlled_Component);
		    exception
			when others =>
			    Undefer_Abort;
			    raise Program_Error;
			      -- see RM95 7.6.1(15)
		    end; -- handler
	    Check_Abort;
		    -- Now copy and adjust
		    MEM.RTS_Memmove(Source_Component, 
		      Target_Component, Natural(Component_Size));
		    begin
			Adjust_Op(Controlled_Component);
		    exception
			when others =>
			    Undefer_Abort;
			    raise Program_Error;
			      -- *** TBD *** It is not exactly clear from
			      -- RM95 7.6.1(16) whether we should
			      -- raise Program_Error here.
		    end; -- handler

		    -- Advance to next components
		    Target_Component := Target_Component + Component_Size;
		    Source_Component := Source_Component + Component_Size;
		    Controlled_Component := 
		      Controlled_Component + Component_Size;
		end loop;
		Undefer_Abort;
	elsif Target_Obj > Source_Obj then

	    -- Copy starting at end of arrays

	    Target_Component := Target_Obj + Total_Size;
	    Source_Component := Source_Obj + Total_Size;
	    Controlled_Component := Controlled_Part + Total_Size;
	    Defer_Abort;
		-- Abort deferred section
		for I in 1..Num_Components loop
		    -- Go back to preceding component
		    Target_Component := Target_Component - Component_Size;
		    Source_Component := Source_Component - Component_Size;
		    Controlled_Component := 
		      Controlled_Component - Component_Size;

		    -- First cleanup this component
		    begin
			Cleanup_Op(Controlled_Component);
		    exception
			when others =>
			    Undefer_Abort;
			    raise Program_Error;
			      -- see RM95 7.6.1(15)
		    end; -- handler
	    Check_Abort;

		    -- Now copy and adjust

		    MEM.RTS_Memmove(Source_Component, 
		      Target_Component, Natural(Component_Size));
		    begin
			Adjust_Op(Controlled_Component);
		    exception
			when others =>
			    Undefer_Abort;
			    raise Program_Error;
			      -- *** TBD *** It is not exactly clear from
			      -- RM95 7.6.1(16) whether we should
			      -- raise Program_Error here.
		    end; -- handler
		end loop;
	    Undefer_Abort;
	end if;

	-- Nothing to do if Target_Obj = Source_Obj

    end Finalize_Copy_And_Adjust_Array;


    procedure Preregister_Array_Func_Result(
	Master : in RTT.Master_Access;
	Cleanup_Rec : in System.Address)
    is
    -- Overview:
    --    This registers a cleanup record for the array result of
    --    a function call.  It must be called immediately prior
    --    to calling any function returning an array requiring
    --    cleanup.  The cleanup record is only partially filled
    --    in at this point.

	Rec : aliased Array_Cleanup :=
		(RTT.Cleanup_Record with System.Null_Address, null, 0, 0);
	for Rec'Address use Cleanup_Rec;
    begin
--DBG-- 	Debug_Put_Line("Preregister_Array_Func_Result");
	Register_In_Master(Rec'Unchecked_Access, Master);
    end Preregister_Array_Func_Result;

    procedure Reregister_Array_Func_Result(
	Master : in RTT.Master_Access;
	Obj : in System.Address;			-- interesting part
	Obj_Cleanup_Rec : in Array_Cleanup_Ptr)
    is
    -- Overview:
    --    This is to be called before returning a local array object
    --    from a function into caller-allocated space.  It cancels
    --    the finalization for the object in the local scope
    --    (the finalization associated with Obj_Cleanup_Rec).
    --    Master is finalized and popped, and Obj is registered
    --    on a master in the caller of the function, using a
    --    cleanup record which was specified using
    --    Register_Func_Result_Before_Call.

	Caller_Cleanup : Array_Cleanup_Ptr := Caller_Array_Cleanup(Master);
    begin
--DBG-- 	Debug_Put_Line("Reregister_Array_Func_Result on", Obj);
	Defer_Abort;
	    -- Abort deferred section
	    Obj_Cleanup_Rec.Obj := System.Null_Address;
	    Caller_Cleanup.Cleanup_Op := Obj_Cleanup_Rec.Cleanup_Op;
	    Caller_Cleanup.Obj := Obj;
	    Caller_Cleanup.Component_Size := Obj_Cleanup_Rec.Component_Size;
	    Caller_Cleanup.Num_Components := Obj_Cleanup_Rec.Num_Components;
	Undefer_Abort;
	if Master /= null then
	    Finalize_And_Pop(Master);
	end if;
    end Reregister_Array_Func_Result;

    procedure Adjust_And_Reregister_Array_Func_Result(
	Master : in RTT.Master_Access;
	Obj : in System.Address;			-- interesting part
	Component_Size : in Storage_Count;
	Num_Components : in Natural;
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Adjust_Op : in Operation_Ptr)	-- TBD: Static link
    is
    -- Overview:
    --    This is to be called before returning an array value from
    --    a function into caller-allocated space.  The components of
    --    Obj are adjusted and registered on a master in the caller
    --    of the function, using a cleanup record which was specified
    --    using Register_Func_Result_Before_Call.  Master is finalized
    --    and popped.

	Caller_Cleanup : Array_Cleanup_Ptr := Caller_Array_Cleanup(Master);

	Component : System.Address := Obj;
    begin
--DBG-- 	Debug_Put_Line("Adjust_And_Reregister_Array_Func_Result on", Obj);

	Defer_Abort;
	    -- Abort deferred section
	    Caller_Cleanup.Component_Size := Component_Size;
	    Caller_Cleanup.Num_Components := Num_Components;
	    Caller_Cleanup.Cleanup_Op := Cleanup_Op;

	    -- Don't store Caller_Cleanup.Obj until after call, since if
	    -- it raises an exception we don't want the array to be registered.
	    Per_Component_Array_Op(Obj, Caller_Cleanup.All, Adjust_Op,
				   Propagate_Exception => False);
	    Caller_Cleanup.Obj := Obj;
	Undefer_Abort;
	if Master /= null then
	    Finalize_And_Pop(Master);
	end if;
    end Adjust_And_Reregister_Array_Func_Result;

    function Reregister_Array_Func_Result_Copy_On_SS(
	Master : in RTT.Master_Access;
	Obj : in System.Address;
	Storage_Size : in System.Storage_Elements.Storage_Count;
	Obj_Cleanup_Rec : in Array_Cleanup_Ptr;
	Target_Mark : Secondary_Mark_Ptr
    ) return System.Address is

    -- Overview:
    --    This is to be called before returning a local array object
    --    from a function.  It cancels the finalization for the
    --    object in the local scope (the finalization associated with
    --    Obj_Cleanup_Rec).  Master is finalized and popped.  Obj
    --    is copied to Target_Mark in the secondary stack, which is
    --    released to the end of the copy of Obj.  The interesting
    --    part of the copy of Obj (as determined from Obj_Cleanup_Rec)
    --    is registered on a master in the caller of the function,
    --    using a cleanup record which was specified using
    --    Register_Func_Result_Before_Call.

	Caller_Cleanup : Array_Cleanup_Ptr := Caller_Array_Cleanup(Master);
	Obj_Interesting_Offset : System.Storage_Elements.Storage_Offset;
	Return_Obj : System.Address := System.Null_Address;
	Exception_Raised : Boolean := False;
    begin
--DBG-- 	Debug_Put_Line("Reregister_Array_Func_Result_Copy_On_SS on", Obj);
	Defer_Abort;
	    -- Abort deferred section
	    Obj_Interesting_Offset := Obj_Cleanup_Rec.Obj - Obj;
	    Obj_Cleanup_Rec.Obj := System.Null_Address;
	    
	    if Master /= null then
		begin
		    Finalize_And_Pop(Master);
		exception
		    when others =>
			Exception_Raised := True;
		end; -- handler
	    end if;

	    Caller_Cleanup.Cleanup_Op := Obj_Cleanup_Rec.Cleanup_Op;
	    if Target_Mark = Null_Secondary_Mark_Ptr then
		Return_Obj := SS_Allocate(Natural(Storage_Size));
		MEM.RTS_Memmove(Obj, Return_Obj, Natural(Storage_Size));
	    else
		Return_Obj :=
		  SS_Copy_And_Release(Obj, Target_Mark, Natural(Storage_Size))
		  + Obj_Interesting_Offset;
	    end if;
	    Caller_Cleanup.Obj := Return_Obj;
	Undefer_Abort;

	if Exception_Raised then
	    raise Program_Error;
	end if;

	return Caller_Cleanup.Obj;
    end Reregister_Array_Func_Result_Copy_On_SS;

    function Adjust_And_Reregister_Array_Func_Result_Copy_On_SS(
	Master : in RTT.Master_Access;
	Obj : in System.Address;
	Storage_Size : in System.Storage_Elements.Storage_Count;
	Obj_Cleanup_Rec : in Array_Cleanup_Ptr;
	Adjust_Op : in Operation_Ptr;	-- TBD: Static link
	Target_Mark : Secondary_Mark_Ptr
    ) return System.Address is

    -- Overview:
    --    This is to be called before returning a local array object
    --    with aliased components from a function.  It proceeds as
    --    Reregister_Array_Func_Result_Copy_On_SS, except that after
    --    copying Obj to the secondary stack, it adjusts the copy
    --    and finalizes the original.
    begin
	-- **** TBD ****
	return Reregister_Array_Func_Result_Copy_On_SS(
		 Master, Obj, Storage_Size, Obj_Cleanup_Rec, Target_Mark);
    end Adjust_And_Reregister_Array_Func_Result_Copy_On_SS;



    -----------------------------------------------------------------
    ----             A C C E S S  C O L L E C T I O N S          ----
    -----------------------------------------------------------------

    --type Collection_Element;
    --type Collection_Element_Ptr is access all Collection_Element;
    --type Collection_Data is null record;

    --type Collection_Element is 
	-- Object within a collection needing cleanup
	-- is doubly-linked into collection-cleanup
    --  record
    --    Next, Prev : Collection_Element_Ptr;
    --    Data : aliased Collection_Data;
    --  end record;

    --type Collection_Cleanup is new RTT.Cleanup_Record with
    --  record
    --	List : Collection_Element_Ptr;  -- Doubly linked list
    --	Cleanup_Op : Operation_Ptr;	-- Operation to cleanup one element
    --	Offset : System.Storage_Elements.Storage_Offset;
					-- Offset to "interesting" part
    --  end record;
    --type Collection_Cleanup_Ptr is access all Collection_Cleanup;

    Header_Offset : constant Storage_Count :=
	Collection_Element'Size / System.Storage_Unit;

    procedure Cleanup(Rec : Collection_Cleanup) is
      -- Overview:
      --   Cleanup a whole collection.
	Element : Collection_Element_Ptr := Rec.List;
	Exception_Was_Raised : Boolean := False;
    begin
	-- Cleanup up each element of collection
	while Element /= null loop
	    begin
		Rec.Cleanup_Op(Element.Data'Address + Rec.Offset);
	    exception
		when others =>
		    Exception_Was_Raised := True;
	    end; -- handler
	
	    Element := Element.Next;
	end loop;
	if Exception_Was_Raised then
	    raise Program_Error;
	end if;
    end Cleanup;
	

    procedure Register_Collection_For_Cleanup(
	Master: in RTT.Master_Access;
	Collection : in System.Address;
	Offset : in System.Storage_Elements.Storage_Offset;
	Cleanup_Op : in Operation_Ptr)	-- TBD: Static link
    is
    -- Overview:
    --   Adds collection cleanup record to given master
    --   (or to default master, if none).
    --   The collection is cleaned up as a whole when the
    --   corresponding scope is exited.

	Rec : aliased Collection_Cleanup :=
		(RTT.Cleanup_Record with null, Cleanup_Op, Offset);
	for Rec'Address use Collection;
    begin
--DBG-- 	Debug_Put_Line("Register_Collection_For_Cleanup for", Collection);
	Register_In_Master(Rec'Unchecked_Access, Master);
    end Register_Collection_For_Cleanup;

    protected Collection_Manager is

      -- This protected object provides synchronization
      -- while adding or removing elements from collection

        pragma Interrupt_Priority( Any_Priority'Last );

	procedure Insert(
	  Collection : in Collection_Cleanup_Ptr;
	  Element : in Collection_Element_Ptr);
	    -- Overview: Insert into doubly linked list
	procedure Remove(
	  Collection : in Collection_Cleanup_Ptr;
	  Element : in Collection_Element_Ptr);
	    -- Overview: Remove from doubly linked list
    end Collection_Manager;

    protected body Collection_Manager is
	procedure Insert(
	    Collection : in Collection_Cleanup_Ptr;
	    Element : in Collection_Element_Ptr) is
	begin
	    -- Insert at head of list
	    Element.Next := Collection.List;
	    if Element.Next /= null then
		Element.Next.Prev := Element;
	    end if;
	    Element.Prev := null;
	    Collection.List := Element;
	end Insert;

	procedure Remove(
	    Collection : in Collection_Cleanup_Ptr;
	    Element : in Collection_Element_Ptr) is
	begin
	    -- Extract from doubly linked list
	    if Element.Next /= null then
		Element.Next.Prev := Element.Prev;
	    end if;
	    if Element.Prev /= null then
		Element.Prev.Next := Element.Next;
	    else
		-- We have a new head of list
		Collection.List := Element.Next;
	    end if;
	    Element.Prev := null;
	    Element.Next := null;
	end Remove;
	    
    end Collection_Manager;

    procedure RTS_Finalize_Classwide_Object(Obj : in System.Address);
    pragma Import(Ada, RTS_Finalize_Classwide_Object, 
      External_Name => "rts_finalize_classwide_object");
	-- This is defined in Ada.Tags

    function Finalize_Op(Tagged_Obj : in System.Address) return Operation_Ptr;
	-- Return finalization operation for given tagged object
    pragma Inline(Finalize_Op);

    function Finalize_Op(Tagged_Obj : in System.Address) 
      return Operation_Ptr is
	-- Return finalization operation for given tagged object

	-- TBD: this duplicates info from Ada.Tags body
	-- Finalize_Op should be a routine in a 
	-- child package "Ada.Tags.Internals" or equivalent.
	type Tag_Type_Descriptor is record
	    Expanded_Name	: System.Address;
	    External_Tag	: System.Address;
	    Ancestor_Table	: System.Address;
	    SU_Size_Func	: System.Address;
	    Finalize		: Operation_Ptr;
	    Adjust		: Operation_Ptr;
	end record;
	type Tag is access all Tag_Type_Descriptor;
	type Normal_Tagged_Obj is record
	    Tag_Descpt : Tag;
	end record;
	
	Tagged_Obj_Rec : Normal_Tagged_Obj;
	pragma Import(Ada, Tagged_Obj_Rec);
	for Tagged_Obj_Rec'Address use Tagged_Obj;

    begin
	return Tagged_Obj_Rec.Tag_Descpt.Finalize;
    end Finalize_Op;
    
    function Needs_No_Cleanup(
	Collection : in Collection_Cleanup_Ptr;
	Element : in System.Address) return Boolean;
    pragma Inline(Needs_No_Cleanup);
	-- "Needs_No_Cleanup" is used to avoid putting objects onto
	-- collection list when the designated type is classwide, but
	-- the tag of the object implies it has no cleanup.

	-- NOTE: This "optimization" is *necessary* because the compiler
	-- does not generate a call on Link_Collection_Element when
	-- it can determine that the object needs no cleanup.
	-- However, upon unchecked deallocation, the compiler *always*
	-- generates a call on Finalize_And_Unlink...
	-- Therefore Finalize_And_Unlink has to be smart enough to
	-- decide whether to remove from collection, and hence
	-- Link_Collection_Element must avoid putting it on the list
	-- in the cases when Finalize_And_Unlink... would refuse to
	-- remove it.  This is because if Link_Collection_... put it
	-- on the list, but it wasn't removed, but yet its storage was
	-- freed, the list would have a bad element in its midst when we
	-- finally get around to walking the whole collection list on
	-- scope exit.

    function Needs_No_Cleanup(
	Collection : in Collection_Cleanup_Ptr;
	Element : in System.Address) return Boolean is
	-- "Needs_No_Cleanup" is used to avoid putting objects onto
	-- collection list when the designated type is classwide, but
	-- the tag of the object implies it has no cleanup.
    begin
	if Collection.Cleanup_Op = 
	  RTS_Finalize_Classwide_Object'Access and then
	   Collection.Offset = 0 then
	    -- We have a classwide designated type, see if this
	    -- object needs any cleanup...
	    if Finalize_Op(Element) = Null_Operation'Access then
		return True;
	    end if;
	end if;
	return False;
    end Needs_No_Cleanup;

    procedure Link_Collection_Element(
	Collection : in Collection_Cleanup_Ptr;
	Element : in System.Address;
	Flush : in RTT.Master_Access)
    is
    -- Overview:
    --   This links the element into the collection list,
    --   so that it eventually gets cleaned up.
	Header : aliased Collection_Element;
	for Header'Address use (Element - Header_Offset);
    begin
--DBG-- 	Debug_Put_Line("Link_Collection_Element for", Element);

	-- Just pass the buck to the protected procedure, basically

	-- Check for special case where object does not actually have
	-- any cleanup to be performed
	if Needs_No_Cleanup(Collection, Element) then
	    -- Has no cleanup, so must not add to collection list;
	    -- Still should flush master if non-null
	    if Flush /= null then
		Flush_Master(Flush);
	    end if;

	elsif Flush /= null then
	    -- Must flush and insert as an atomic action
	    Defer_Abort;
		-- Abort deferred section
		Flush_Master(Flush);
		Collection_Manager.Insert(Collection, Header'Unchecked_Access);
	    Undefer_Abort;
	else
	    -- No need to flush, just do the insert
	    Collection_Manager.Insert(Collection, Header'Unchecked_Access);
	end if;
    end Link_Collection_Element;

    procedure Finalize_And_Unlink_Collection_Element(
	Collection : in Collection_Cleanup_Ptr;
	Element : System.Address)
    is
    -- Overview:
    --   This defers abort, calls the Cleanup_Op on Element.Data+Offset,
    --   and then unlinks the element from the collection list,
    --   so that it doesn't get cleaned up twice.
    --   If Cleanup_Op propagates an exception, Program_Error is raised,
    --   but the element is unlinked anyway.
    --   This is intended to be called by Unchecked_Deallocation.
	Exception_Was_Raised : Boolean := False;

	function To_Pointer is new Ada.Unchecked_Conversion(
	  System.Address, Collection_Element_Ptr);
	
	Header : Collection_Element_Ptr := To_Pointer(Element - Header_Offset);
    begin
--DBG-- 	Debug_Put_Line("Finalize_And_Unlink_Collection_Element on", Element);
	-- Check for special case where object does not actually have
	-- any cleanup to be performed
	if Needs_No_Cleanup(Collection, Element) then
	    -- Has no cleanup, so was not put on collection list
	    return;
	end if;

	Defer_Abort;
	    -- Abort deferred section
	    begin
		Collection.Cleanup_Op(Element + Collection.Offset);
	    exception
		when others =>
		    Exception_Was_Raised := True;
	    end; -- handler

	    -- Extract from doubly linked list
	    Collection_Manager.Remove(Collection, Header);
	Undefer_Abort;

	if Exception_Was_Raised then
	    raise Program_Error;
	end if;
    end Finalize_And_Unlink_Collection_Element;

    -------------------------------------------------------
    --     C L A S S W I D E    C L E A N U P            --
    -------------------------------------------------------
    procedure Null_Operation(Obj : in System.Address) is
    --   This operation just returns.  It is put into
    --   Adjust_Whole_Object and Finalize_Whole_Object slots
    --   for tagged types that have no controlled parts.
    begin
	null;
    end Null_Operation;

    -- See also classwide finalization operations in the body of Ada.Tags.


    -------------------------------------------------------
    --     E X C E P T I O N S   A N D  C L E A N U P    --
    -------------------------------------------------------
    function Finalize_And_Pop_Wrapper( 
    	    	        Exc_Occurrence : ET.Exception_Occurrence_Access;
			Master         : RTT.Master_Access ) 
		return ET.Exception_Occurrence_Access is
    begin
    	System.RTS.Master_Pkg.RTS_Finalize_And_Pop( Master );
	return Exc_Occurrence;
    exception
    	when others =>
	    return EM.RTS_New_Occurrence( Standard.Program_Error'Identity );
    end Finalize_And_Pop_Wrapper;

begin

    null;         -- body eleboration needed for protected object

end System.RTS.Finalization_Pkg;
