-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.interrupts.static_handlers.spc,v $ 
-- $Revision: 1.4 $ $Date: 96/10/29 15:07:17 $ $Author: bethc $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Protected_Types_Pkg;
with System.RTS.Interface_Types;
package Ada.Interrupts.Static_Handlers is

    package PRT renames System.RTS.Protected_Types_Pkg;
    package IFT renames System.RTS.Interface_Types;

    procedure RTS_Attach_Static_Handler(
        Desc        : IFT.Handler_Desc_Handle;
        PO          : IFT.PO_Handle;
        Handler     : IFT.Code_Address;
        Interrupt   : Interrupt_ID );
    pragma Export (ada,
		   RTS_Attach_Static_Handler,
		   "rts_attach_static_handler" );
    pragma Global (RTS_Attach_Static_Handler);
    --* OVERVIEW
    -- Called for attachment of a statically defined interrupt


    procedure Detach_Static_Handlers(
        PO       : PRT.PO_Access
    );
    --* OVERVIEW
    -- Called upon finalization of PO to update state of interrupt 
    -- attachment both the kernel and RTS.  First, active static
    -- attachments are detached by making the required kernal calls.
    -- Furthermore, this procedure searches the data structure where
    -- attachment history is maintained to identify PO as having
    -- an "old" attachment.  That is, PO has an attachment that was 
    -- overridden.  The record of the old attachment has to be removed
    -- so it won't be restored in the future.  

    function Is_Statically_Attached(
         Interrupt     : Ada.Interrupts.Interrupt_ID
    ) return Boolean;
    --* OVERVIEW
    -- Returns True if Interrupt is currently statically attached to
    -- some handler.  Operations in Ada.Interrupts call this function
    -- to ensure that a dynamic attachment does not override a static
    -- one. 
    
    function RTS_Handler_Priority(
        Interrupt     : Ada.Interrupts.Interrupt_ID;
        Min_Prio      : System.Interrupt_Priority
    ) return System.Interrupt_Priority;
    pragma Export (ada, RTS_Handler_Priority, "rts_handler_priority" );
    pragma Global (RTS_Handler_Priority);
    --  OVERVIEW
    -- Used by emitter to determine the priority to use of one or
    -- more pragma Attach_Handlers is present but no pragma Interrupt_
    -- Priority. Returns maximum of Priority_From_Id(Interrupt)
    -- and Min_Prio.

end Ada.Interrupts.Static_Handlers;
