-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.protected_interface.spc,v $ 
-- $Revision: 1.12 $ $Date: 96/10/29 15:05:30 $ $Author: bethc $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Protected_Types_Pkg;
with System.RTS.Interface_Types;       use System.RTS.Interface_Types;

package System.RTS.Protected_Interface is

    pragma Suppress( Elaboration_Check );

--* OVERVIEW
-- This package contains interfaces to generated code for all
-- protected operations.

    type Queuing_Policy_Type is ( FIFO, PRIORITY );
    Queuing_Policy : Queuing_Policy_Type := FIFO;

    procedure RTS_Init_Decl_PO (
      Master        : Master_Handle;
      PO_Address    : PO_Handle;
      Priority      : Priority_Type;
      Type_Info     : Protected_Types_Pkg.PR_Type_Desc_Access;
      Static_Link   : Simple_Handle
    );
    pragma Export (ada, RTS_Init_Decl_PO, "rts_init_decl_po" );
    pragma Global (RTS_Init_Decl_PO);
    --* OVERVIEW
    -- Called to initialize the RTS-managed portion of a protected object.
    -- When the compiler is changed, this routine will be obsolete.

    procedure RTS_Init_PO (
      Master        : Master_Handle;
      Cleanup_Rec   : Simple_Handle;
      PO_Address    : PO_Handle;
      Priority      : Priority_Type;
      Type_Info     : Protected_Types_Pkg.PR_Type_Desc_Access;
      Static_Link   : Simple_Handle
    );
    pragma Export (ada, RTS_Init_PO, "rts_init_po" );
    pragma Global (RTS_Init_PO);
    --* OVERVIEW
    -- Called to initialize the RTS-managed portion of a protected object,
    -- and register it for finalization.

    procedure RTS_Finalize_PO (
      PO_Address    : PO_Handle
    );
    --* OVERVIEW
    -- Called to finalize a protected object
    pragma Export (ada, RTS_Finalize_PO, "rts_finalize_po" );
    pragma Global (RTS_Finalize_PO);

    procedure RTS_Entryless_Protected_Subp_Call(
      PO_Address  : Data_Address;
      Params      : Param_Block_Ptr;
      Pr_Subp     : Pr_Subp_Ptr
    );
    pragma Export (ada,
		   RTS_Entryless_Protected_Subp_Call,
		   "rts_entryless_protected_subp_call" );
    pragma Global (RTS_Entryless_Protected_Subp_Call);
    --* OVERVIEW
    --  Called for a protected procedure or function call, when the
    -- protected object has no entries.

    procedure RTS_Protected_Procedure_Call(
      PO_Address  : Data_Address;
      Params      : Param_Block_Ptr;
      Pr_Subp     : Pr_Subp_Ptr );
    pragma Export (ada,
		   RTS_Protected_Procedure_Call,
		   "rts_protected_procedure_call" );
    pragma Global (RTS_Protected_Procedure_Call);
    --* OVERVIEW
    --  Called for a protected procedure or function call.

    procedure RTS_Protected_Entry_Call(
      Called_PO    : PO_Handle;
      Params       : Param_Block_Ptr;
      Name_Index   : Entry_Name_Index;
      Member_Index : Entry_Member_Index );
    pragma Export (ada,
		   RTS_Protected_Entry_Call,
		   "rts_protected_entry_call" );
    pragma Global (RTS_Protected_Entry_Call);
    --* OVERVIEW
    --  Called for an unconditional entry call to a protected object.

    function RTS_Conditional_Protected_Call(
      Called_PO    : PO_Handle;
      Params       : Param_Block_Ptr;
      Name_Index   : Entry_Name_Index;
      Member_Index : Entry_Member_Index ) return Boolean;
    pragma Export (ada,
		   RTS_Conditional_Protected_Call,
		   "rts_conditional_protected_call" );
    pragma Global (RTS_Conditional_Protected_Call);
    --* OVERVIEW
    --  Called for a conditional entry call on a protected object.

    function RTS_Timed_Rel_Protected_Call(
      Called_PO    : PO_Handle;
      Params       : Param_Block_Ptr;
      Name_Index   : Entry_Name_Index;
      Member_Index : Entry_Member_Index;
      Delay_Amount : Duration ) return Boolean;
    pragma Export (ada,
		   RTS_Timed_Rel_Protected_Call,
		   "rts_timed_rel_protected_call" );
    pragma Global (RTS_Timed_Rel_Protected_Call);
    --* OVERVIEW
    --  Called for a timed entry call on a protected object.
    --  where the delay statement is delay_relative.

    function RTS_Timed_Abs_Protected_Call(
      Called_PO       : PO_Handle;
      Params          : Param_Block_Ptr;
      Name_Index      : Entry_Name_Index;
      Member_Index    : Entry_Member_Index;
      Expiration      : Absolute_Delay_Handle;
      Which_Time_Type : Time_Type_Enum ) return Boolean;
    pragma Export (ada,
		   RTS_Timed_Abs_Protected_Call,
		   "rts_timed_abs_protected_call" );
    pragma Global (RTS_Timed_Abs_Protected_Call);
    --* OVERVIEW
    --  Called for a timed entry call on a protected object
    --  where the delay statement is delay_until.

    function RTS_Async_Protected_Call(
      ECD          : ECD_Handle;
      Setjmp_Buffer: Data_Address;
      Master       : Master_Handle;
      Called_PO    : PO_Handle;
      Params       : Param_Block_Ptr;
      Name_Index   : Entry_Name_Index;
      Member_Index : Entry_Member_Index ) return Boolean;
    pragma Export (ada,
		   RTS_Async_Protected_Call,
		   "rts_async_protected_call" );
    pragma Global (RTS_Async_Protected_Call);
    --* OVERVIEW
    --  Called for an asynchronous call with protected entry call
    --  as trigger alternative.

    function RTS_Protected_Entry_Count(
      PO           : PO_Handle;
      Name_Index   : Entry_Name_Index;
      Member_Index : Entry_Member_Index ) return Caller_Count;
    pragma Export (ada,
		   RTS_Protected_Entry_Count,
		   "rts_protected_entry_count" );
    pragma Global (RTS_Protected_Entry_Count);
    --* OVERVIEW
    --  Corresponds to the attribute E'COUNT for Large model.
    --* REQUIRES
    --  RTS_Protected_Entry_Count must be called from within the
    --  protected object.

   function RTS_Protected_Entry_Caller(
        ECD : ECD_Handle
    ) return Task_Obj_Handle;
    pragma Export (ada,
		   RTS_Protected_Entry_Caller,
		   "rts_protected_entry_caller" );
    pragma Global (RTS_Protected_Entry_Caller);
    --* OVERVIEW
    --  Corresponds to the attribute E'Caller.
    --* REQUIRES
    --  RTS_Protected_Entry_Caller must be called from within the
    --  entry body.


   function RTS_Protected_Entry_Caller_Priority(
        PO : PO_Handle;
        Name_Index : Entry_Name_Index;
        Member_Index : Entry_Member_Index
    ) return Priority_Type;
    pragma export (Ada, RTS_Protected_Entry_Caller_Priority, 
			"rts_protected_entry_caller_priority");
    pragma global (RTS_Protected_Entry_Caller_Priority);
    --* OVERVIEW
    --	Corresponds to the attribute E'Caller_Priority
    --* REQUIRES
    --  RTS_Protected_Entry_Caller_Priority must be called from within the
    --  entry body.

    procedure RTS_Do_Protected_Requeue(
      Current_ECD  : ECD_Handle;
      With_Abort   : Boolean;
      Called_PO    : PO_Handle;
      Name_Index   : Entry_Name_Index;
      Member_Index : Entry_Member_Index;
      Task_To_Protected : Boolean );
    pragma Export (ada,
		   RTS_Do_Protected_Requeue,
		   "rts_do_protected_requeue" );
    pragma Global (RTS_Do_Protected_Requeue);
    --* OVERVIEW
    -- Called for a requeue statement to a protected entry.

    procedure RTS_Set_Queuing_Policy_FIFO;
    pragma Export(Ada, 
		  RTS_Set_Queuing_Policy_FIFO,
		  "rts_set_queuing_policy_fifo");
 
    pragma Global( RTS_Set_Queuing_Policy_FIFO );
    --* OVERVIEW
    --	TBD

    procedure RTS_Set_Queuing_Policy_Priority;
    pragma Export(Ada, 
		  RTS_Set_Queuing_Policy_Priority,
		  "rts_set_queuing_policy_priority");
 
    pragma Global( RTS_Set_Queuing_Policy_Priority );
    --* OVERVIEW
    --	TBD

    function Get_ECD_Priority( Pri : SYSTEM.Any_Priority )
			      return System.Any_Priority;
    --* OVERVIEW
    --	TBD

    procedure RTS_Propagate_Entry_Call_Exn (
	ECD : ECD_Handle
    );
    pragma Export(Ada, 
		  RTS_Propagate_Entry_Call_Exn,
		  "rts_propagate_entry_call_exn");
 
    pragma Global( RTS_Propagate_Entry_Call_Exn );
    --* OVERVIEW
    --	TBD
    -- Needed for async calls, so exception can be propagated after longjmp
    

end System.RTS.Protected_Interface;
