-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.sequential_io.bdy,v $ 
-- $Revision: 1.13 $ $Date: 1997/02/27 16:26:36 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

pragma revision ("$Revision: 1.13 $");

with System;
with Ada.Unchecked_Deallocation;
with Ada.Unchecked_Conversion;
with System.RTS.TGT.Basic_IO;
pragma Elaborate(System.RTS.TGT.Basic_IO);

pragma ada_child;
package body Ada.Sequential_IO is
    
    Dope_Size : Natural := 0;

    type Element_Access is access all Element_Type;
    function Conv is new Ada.Unchecked_Conversion (Element_Access,System.Address);
    function Conv is new Ada.Unchecked_Conversion (System.Address,Element_Access);

    package BIO renames System.RTS.TGT.Basic_IO;
    
    -- Convert modes back and forth
    SIO_Mode_To_BIO_Mode : constant array(File_Mode) of
          BIO.File_Mode := (In_File     => BIO.In_File,
                            Out_File    => BIO.Out_File,
                            Append_File => BIO.Append_File);
    
    BIO_Mode_To_SIO_Mode : constant array(BIO.File_Mode range
      BIO.In_File .. BIO.Out_File) of File_Mode :=
       (BIO.In_File => In_File, BIO.Out_File => Out_File);

--    Element_Size : constant Natural := 
--      (Element_Type'SIZE + (BIO.Bits_Per_IO_Element - 1)) /
--        BIO.Bits_Per_IO_Element;
--      -- I/O element type size in storage units

    procedure File_Free is new
      Ada.Unchecked_Deallocation(File_Record, File_Type);
        --| To deallocate files when closed (allocated on heap)
	
    procedure Read  (File : in File_Type; Item : out Element_Type) is
	Item_Size : Natural := 
	  (Item'SIZE + (BIO.Bits_Per_IO_Element - 1)) / BIO.Bits_Per_IO_Element;
        Size : Natural;
        procedure Read (S:in System.Address;Length:in Integer) is
           Amount_Read : Natural;
        begin
           if BIO.Read (File.Host_File, S, Length)=0 then
              raise End_Error;
           end if;
        end Read;
    begin
        if not End_Of_File ( File ) then
            if Element_Type'Constrained then
               Read (Item'Address, Item_Size); -- Read the item directly
            else
               -- Calculate the dope size if not already done:
               if Dope_Size=0 then
                  declare
                     Local_Item : aliased Element_Type := Item;
                     use System;
                  begin
                     Dope_Size := Natural(Local_Item'Address-Conv(Local_Item'Unchecked_Access));
                  end;
               end if;
               Item_Size:=Item_Size+Dope_Size;
               -- Read the size as recorded in the file
               Read (Size'Address, Size'Size/ BIO.Bits_Per_IO_Element);
               if Size>Item_Size then
                  -- Item in file greater than actual - skip the item
                  -- (without risking trying to declare an arbitrarily large object to read into)
                  declare
                     Discard : String (1..1024);
                  begin
                     for I in 1..Size/Discard'Length loop
                        Read (Discard'Address, Discard'Length);
                     end loop;
                     Read (Discard'Address, Size mod Discard'Length);
                  end;
                  raise Data_Error;
               else
	          declare
                     Buffer : String (1..Item_Size);
	          begin
                     Read(Buffer(1)'Address,Size);
                     Item:=Conv(buffer'address).all; -- Checking the contraints
                  exception
                     when Constraint_Error =>
                        raise Data_Error;
                  end;
               end if;
            end if;

        else
	    raise End_Error;
	end if;
    end Read;

    procedure Write (File : in File_Type; Item : in Element_Type) is
	Item_Size : Natural := 
	  (Item'SIZE + (BIO.Bits_Per_IO_Element - 1)) / BIO.Bits_Per_IO_Element;
	Amount_Written : Natural;
        procedure Write (A:in System.Address;Size:in Natural) is
        begin
           if BIO.Write(File.Host_File,A,Size)/=Size then
               raise DEVICE_ERROR;
           end if;
        end Write;
    begin
        if File = null then
	    raise STATUS_ERROR;  --| File not open
        end if;
    
        if File.Mode /= Out_File and then
           File.Mode /= Append_File then
	    raise MODE_ERROR;    --| File not open for writing
        end if;
    
        if Element_Type'Constrained then
	   -- Write item out directly
           Write (Item'Address, Item_Size);
        else
           declare
              Local_Item : aliased Element_Type := Item;
              use System;
           begin
              -- Record with the item's length (including dope)
              if Dope_Size=0 then
                 Dope_Size := Natural(Local_Item'Address-Conv(Local_Item'Unchecked_Access));
              end if;
              Item_Size := Item_Size + Dope_Size;
              Write (Item_Size'ADDRESS,Item_Size'Size/BIO.Bits_Per_IO_Element);
              Write (Conv(Local_Item'Unchecked_Access), Item_Size);
           end;
        end if;
    end Write;

    procedure Create(File : in out File_Type;
                     Mode : in File_Mode := Out_File;
		     Name : in String := "";
		     Form : in String := "") is
    begin
	if File /= null then
	    raise STATUS_ERROR; --| File already open
	end if;
        File := new File_Record;
	File.Mode := Mode;
	BIO.Create(File.Host_File,
		   SIO_Mode_To_BIO_Mode(Mode),
		   Name,
		   Form,
		   BIO.Sequential_IO,
		   1);
    end Create;
		     
    procedure Open(File : in out File_Type;
                   Mode : in File_Mode;
		   Name : in String;
		   Form : in String := "") is
    begin
	if File /= null then
	    raise STATUS_ERROR;  --| File already open
	end if;
        File := new File_Record;
	File.Mode := Mode;
	BIO.Open(File.Host_File,
		 SIO_Mode_To_BIO_Mode(Mode),
		 Name,
		 Form,
		 BIO.Sequential_IO,
		 1);
    end Open;
		   
    procedure Close  (File : in out File_Type) is
    begin
        if File = null then
	    raise STATUS_ERROR;  --| File is not open
	end if;
	BIO.Close(File.Host_File);
        File_Free(File);
        File := null;
    end Close;

    procedure Delete (File : in out File_Type) is
    begin
	if File = null then
	    raise STATUS_ERROR;  --| File not open
	end if;
	BIO.Delete(File.Host_File);
        File_Free(File);
        File := null;
    end Delete;

    procedure Reset  (File : in out File_Type; Mode : in File_Mode) is
    begin
        if File = null then
	    raise STATUS_ERROR;  --| File is not open
	end if;
        BIO.Switch_Mode(File.Host_File,
			SIO_Mode_To_BIO_Mode(File.Mode),
			SIO_Mode_To_BIO_Mode(Mode));
			
	if File.Mode = Mode then
	    -- Mode not changed, so just change file pointer
	    BIO.Reset(File.Host_File);
	else
	    -- Just update to reflect new mode
	    File.Mode := Mode;
	end if;
	File.End_Of_File := False;
    end Reset;

    procedure Reset  (File : in out File_Type) is
    begin
	if File = null then
	    raise STATUS_ERROR;  --| File not open
	end if;
	BIO.Reset(File.Host_File);
	File.End_Of_File := False;
    end Reset;
    
    function Mode (File : in File_Type) return File_Mode is
    begin
	if File = null then
	    raise STATUS_ERROR;
	end if;
	return File.Mode;
    end Mode;

    function Name (File : in File_Type) return String is
    begin
	if File = null then
	    raise STATUS_ERROR;  --| File not open
	end if;
	return BIO.Name(File.Host_File);
    end Name;

    function Form (File : in File_Type) return String is
    begin
	if File = null then
	    raise STATUS_ERROR;  --| File not open
	end if;
	return BIO.Form(File.Host_File);
    end Form;
    
    function Is_Open (File : in File_Type) return Boolean is
    begin
	return File /= null;
    end Is_Open;
    
    function End_Of_File (File : in File_Type) return Boolean is
    begin
        if File = null then
            raise Status_Error;  --| File not open
        end if;
   
        if File.Mode /= In_File then
            raise Mode_Error;    --| File not open for input
        end if;
   
	return File.End_Of_File or else BIO.End_Of_File (File.Host_File);

    end End_Of_File;

end Ada.Sequential_IO;
