-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ***************************************************************************
-- PACKAGE: System.RTS.Tgt_MESSAGES
--
-- OUTLINE:
--   This package contains the user error messages which are considered to be
-- target dependent. User error messages may be written in one of several
-- natural languages. Each error message is referenced by an error
-- code of type MESSAGE. Error messages in this table have error codes
-- greater or equal to MIN_MESSAGE_NUMBER.
--
-- Some natural languages require extended ascii to provide the entire
-- character set, the value of RTS_EXTENDED_ASCII.STRING determines the type
-- of the message table strings.
--
-- The specification of this package is shared (and largely common). The body
-- however is a template file since it contains the specific target-dependent
-- messages.
-- ***************************************************************************

with System.Rts.Error_IO;	    -- For MESSAGE and constants.
--with ALSYS_RTS_EXTENDED_ASCII;
--with ALSYS_TCB_PACKAGE;

package System.RTS.Tgt_MESSAGES is

   pragma SUPPRESS (ELABORATION_CHECK);

   -- -------------------------------------------------------------------
   -- OUTLINE:
   --	Provides a procedure for looking up target dependent messages and
   -- a target dependent procedure for finding out a task name.
   -- -------------------------------------------------------------------

   --	     -----------------
   --	     Table of Contents
   --	     -----------------
   --	     1. Target-dependent Declarations
   --	     2. Target-independent Declarations

   -- ------------------------------------------
   -- Package renamings to remove ALSYS_ prefix:
   -- ------------------------------------------
   package ERROR_IO renames System.Rts.Error_IO;
--   package RTS_EXTENDED_ASCII renames ALSYS_RTS_EXTENDED_ASCII;
--   package TCB_PACKAGE renames ALSYS_TCB_PACKAGE;
   -- ------------------------------------------

   -- *********************************
   -- (1) TARGET-DEPENDENT DECLARATIONS
   -- *********************************

   -- When this character appears in a entry of the message table it
   -- indicates that you should insert the parameter at this point.
   -- This is only target/language dependent in the sense that
   -- it is impossible to choose a wholly target/language independent value.

   PARAM : constant CHARACTER := '\';

   MAX_TASK_NAME_LENGTH : constant NATURAL := 0;  -- tasks don't have names

   -- ***********************************
   -- (2) TARGET-INDEPENDENT DECLARATIONS
   -- ***********************************

   use ERROR_IO;

   procedure MESSAGE_LOOKUP (MESS   : in MESSAGE;
			     TEXT   : in out MESSAGE_STRING;
			     LENGTH : out NATURAL);
   -- -------------------------------------------------------------------
   -- OUTLINE:
   --	Looks up a target-dependent message in the message table and
   -- the extended ascii string which is that message is written into
   -- TEXT. The length of the string is returned in LENGTH.
   -- -------------------------------------------------------------------

--   subtype TASK_NAME_STRING is STRING(1 .. MAX_TASK_NAME_LENGTH);
--
--   procedure GET_TASK_NAME (TASK_ID : in TCB_PACKAGE.TASK_ID;
--			    TEXT    : in out TASK_NAME_STRING;
--			    LENGTH  : out NATURAL);
   -- ----------------------------------------------------------------------
   -- OUTLINE:
   --	A target-dependent routine which finds out the name of the task which
   -- is passed as a parameter. If the name is longer than
   -- MAX_TASK_NAME_LENGTH this routine should truncate it.
   -- ----------------------------------------------------------------------

end System.RTS.Tgt_MESSAGES;

-- Modification History:

--     6.01 29th November 1990 (APC)
--	    Initial introduction to CMS.

--     6.02 17th December 1990 (BJD)
--	    Update for new name of EXTENDED_ASCII package (RTS_EXTENDED_ASCII)

--     6.03 11th February 1991 (APC)
--	    Removed unnecessary pragma ELABORATE(TCB_PACKAGE);

--     6.04 19th Feb 1991 (BJD)
--	    Adjust MAX_TASK_NAME_LENGTH to reflect using the task number image
--     as the name.

--     6.05 11th March 1991 (BJD)
--	    Upgrade from template file to common file by filtering
--     MAX_TASK_NAME_LENGTH.

-- 6.??   3rd November 1991 (BJD)
--	  Prefix all CRTS library unit names with ALSYS_

-- 6.07   7th November 1991 (CEB)
--	  For 386 only, increased the size of a task string by adding
--	  in space for an Interrupt buffer prefix. Interrupt buffers
--	  will be printed as IB@hex_addr.

-- 6.08   1st May 1992 (CDH)
--	  Retarget for 370
