-- $Source: /home/dsd/d/aa/monoBANK/AA/TSP_system.rts.tgt.kernel.threads.spc,v $ 
-- $Revision: 1.7 $ $Date: 96/10/23 13:55:19 $ $Author: stt $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

-- Derived from:
--  Source: /home/dsd/d/aa/monoBANK/AA/system.rts.tgt.kernel.threads.spc,v   
--  Revision: 1.17    Date: 96/10/23 13:51:53    Author: stt   

with System.RTS.Exception_Types;
with System.Storage_Elements;
with SYSTEM.RTS.TCB; use SYSTEM.RTS.TCB;
with System.RTS.Win32; use System.RTS.Win32;

package System.RTS.TGT.Kernel.Threads is
  -- This package defines the basic "thread" abstraction
  -- Threads can wait on at most one "suspension" object
  -- at a time, and at most one thread can wait on a given
  -- suspension object.

  -- A TCB is presumed to be an extension of a thread.

    EtsStandardEdition : Boolean := False;           
    MonoThreaded       : Boolean := False;           

    type Thread_Descriptor is abstract tagged limited private;
    type Thread_Ptr is access all Thread_Descriptor'Class;

    type Thread_Code is access procedure(Self : Thread_Ptr);

    subtype Abort_Signal_Level is Natural;
      -- A level number is passed when an abort signal is sent or
      -- scheduled to be sent at some time.
      -- See also Kernel.Timers.Signal_Abort_Upon_Timeout.
      -- When multiple aborts occur, only the outermost
      -- (lowest number) abort need be delivered.
      -- Abort_Signal_Level should be defined in the higher-level run-time
      -- and could be moved if the per-thread abort data moves to the
      -- the Intermetrics TCB.

    procedure Set_System_Thread_As_Current_Thread(
	Thread : Thread_Ptr
    );
    -- Set up so that Current_Thread may be called, even before
    -- thread structure is fully initialized, in case it is used
    -- to hold information such as current exception, secondary
    -- stack pointer, etc.

    procedure Register_System_Thread(
      Thread : Thread_Ptr;
      Main_Priority : in out System.Priority
    );
    -- The given Thread_Ptr will represent the thread that the system
    -- created to execute the program. Only a subset of the operations
    -- will be applied to the system thread. Operations that will not
    -- be used are Init_Thread, Activate_Thread, 
    -- and Remove_Unactivated_Thread. 
    -- The Main_Priority will be initialized to System.Default_Priority;
    -- on return it should contain the user-specified main priority, if any,
    -- as specified by the prelinker.
    -- This operation will be called only once and from the system thread.


    procedure Register_User_Thread(
      Thread : Thread_Ptr
    );
    -- Register a user thread;
    -- This performs anything that must be done in the context of the 
    -- thread.  It is called as the first thing in task_envelope.  For
    -- example, with a Posix-like OS, you could use this to set the thread_ptr 
    -- in the thread specific data area.
    -- For other real-time executives, each task has its own signal
    -- handlers, so these must be set up before the task body is called.
    pragma Suppress(Storage_Check, On => Register_User_Thread);
	-- Suppress storage check in case this is called
	-- before stack limit info is initialized

    procedure Init_Thread(
      Thread : Thread_Ptr; 
      Storage_Size : Storage_Elements.Storage_Count;
      Code : Thread_Code;
      Priority : Kernel_Priority);
    -- Init_Thread operates on an existing but uninitialized
    -- data structure, creating a thread that is initially inactive,
    -- represented by that data structure.
    -- Init_Thread allocates the thread's stack.
    -- Storage_Size is count of storage elements for the stack.
    -- If it is zero, the implementation should choose a default.
    -- If it is less than an implementation minimum, the implementation
    -- should use the minimum.

    procedure Activate_Thread(Thread : Thread_Ptr);
    -- Activate_Thread is used only for the initial activation.
    -- Control after that is via Suspend_Until_True.

    function Current_Thread return Thread_Ptr;
    pragma Import (Intrinsic, Current_Thread, link_name => "*CURR_ITCB");
    -- Determines the thread of the caller

    function Exception_Occurrence_Save_Area 
      return Exception_Types.Exception_Occurrence_Access;
    -- Return pointer to save area in current thread 

    procedure Finish_Thread;
    -- Stops the current thread "dead in its tracks",
    -- releasing the thread's stack.
    -- Note: the implementation of this operation is expected to call
    -- Release_Thread(Current_Thread) when the
    -- Thread_Descriptor is free to be reused for another Init_Thread.

    procedure Release_Thread( Thread : access Thread_Descriptor) is abstract;
    -- This routine is called by Finish_Thread when a
    -- Thread_Descriptor is ready to be reused, probably as
    -- Finish_Thread's last action.
    -- The purpose is to allow a TCB to be reused.

    procedure Remove_Unactivated_Thread(Thread : Thread_Ptr);
    -- Cleans up a thread that was created by Init_Thread but never
    -- activated by Activate_Thread. Further reference to the thread
    -- is an error.
    -- This situation arises when a task is aborted prior to activation.

    procedure Yield;
    -- Causes the current thread to give up the processor to another
    -- equal priority ready but not running thread, allowing round-robin
    -- scheduling.
    -- Used by RTS when D <= 0.0 in a simple delay D;

    procedure Set_Thread_Priority(
      Thread : Thread_Ptr;
      Priority : Kernel_Priority;
      Yielding : Boolean);	-- If true, puts at tail of ready Q

    procedure Get_Thread_Priority(
      Thread : in Thread_Ptr; Priority : out Kernel_Priority);

    type Suspension_Object is private;
      -- Threads can wait on suspension objects.
      -- Prior to doing so, they should set the state of the
      -- suspension object to False.

    procedure Set_True(S : in out Suspension_Object);
    pragma Inline (Set_True);
    procedure Set_False(S : in out Suspension_Object);
    pragma Inline (Set_False);
    function  Current_State(S : Suspension_Object) return Boolean;
    pragma Inline (Current_State);

    procedure Suspend_Until_True(S : in out Suspension_Object);
    pragma Inline (Suspend_Until_True);

    -- Thread is suspended until some other thread calls Set_True
    -- on its Suspension_Object
   procedure Suspend_Until_True_or_Timeout(
       S : in out Suspension_Object;
       Wait_Time : DWord;
       sts : out Dword);
private

    State_False : constant DWORD := 0;
    State_True  : constant DWORD := 16#FFFFFFFF#;
    type Suspension_Object is
       record
          State  : Dword := State_False;
       end record;

    for Thread_Ptr'Storage_Size use 0;
      -- Disallow allocators, and suppress registration of collection

    type Thread_State is (
    	Running, Abort_Setup_In_Progress );

    type Thread_Descriptor is abstract tagged limited
    	record
	    Thread_handle : Win32_Handle := 0;
            Dbg_ID    : Dword; -- Result of GetCurrentThreadID() for AdaView and also useful in debugging
	    Wake_Thread : Win32_Handle;
	    Suspension_Waiting : System.Address := System.Null_Address; -- address of suspension object state containing our wake event handle
	    TCB : Ref_TCB := null;
	    TCB_Saved_Data : Integer;
	    Thread_Start : Thread_Code;
	    
	    -- Defer_Count is the count of the Defer_Abort calls without
	    -- having yet seen the corresponding Undefer_Abort call.  A
	    -- count of 0 means aborts are not being deferred.  
	    -- Pending_Abort_Level is the lowest level of a call to
	    -- Signal_Abort while aborts were being deferred.
	    Defer_Count : Natural;
	    Pending_Abort_Level : Abort_Signal_Level;
	    Level_In_Progress : Abort_Signal_Level;
	    Waiters : System.Address := System.Null_Address; -- ets only, the address of the L.Waiters we have raised
	    Base_Priority : Kernel_Priority;
	    Active_Priority : Kernel_Priority;
	    Yielding : Boolean;
	    Mutexes_Owned : DWORD := 0;
	    State : Thread_State;
	    Save_Area : aliased Exception_Types.Exception_Occurrence;
	end record;

end System.RTS.TGT.Kernel.Threads;
