package System.RTS.TGT.Kernel.Threads.NT is

    -- Jump_Thread forces a thread to continue execution at the 
    -- location specified by Where.  If Thread is the current
    -- thread, Jump_Thread never returns.
    procedure Jump_Thread( 
	Thread : Thread_Ptr;
	Where : Thread_Code );
 
    procedure Abortable_Sleep (Wait_Time : in DWord);

    -- Wait_For_Handle_Or_Wake is a common routine used to wait
    -- on events.  It centralizes the implementation of events
    -- for aborting the thread; two events are waited upon - the one passed
    -- as parameter and the special Wake_Thread event used for abort.
    
    function Wait_For_Handle_Or_Abort (
       Handle  : in Win32_Handle; 
       Timeout : in DWORD := Infinite) return DWORD;
       
    type lock_type is private;
    -- We use this to force pass by reference (we mustn't take copies of the count component)
    type lock_access is access all lock_type;
    
    SuspensionLock,
    PO_Ceiling_Lock : constant Lock_Access;

    PO_Ceiling_Priority : constant := 30;
    function Wait_For_Handle (Handle:in Win32_Handle;Wait_Time:in DWORD:=Infinite) return DWORD;

    -- Lock the resource:
    procedure Lock (L:in Lock_access);

    -- Unlock the resource:
    procedure Unlock(L:in Lock_access);

    -- Do I own the resource?
    function  IOwn (L:in Lock_access) return Boolean;
    pragma Inline (IOwn);

    -- Lock and immediately unlock the resource; null operation if lock already owned:
    procedure Touch(L:in Lock_access); 
    pragma Inline (Touch);

    -- Initialization is only necessary if the lock is reused (declaration is sufficient)
    procedure Initializelock(L:in Lock_access);
    
    procedure DestroyLock (L:in Lock_access);

    procedure Suspension_Lock (L:in Lock_access:=SuspensionLock) renames Lock;
    procedure Suspension_Unlock(L:in Lock_access:=SuspensionLock) renames Unlock;
    
    rts_is_ets : Boolean;
    pragma export(C, rts_is_ets);

    rts_in_interrupt : Boolean := False;
    pragma export(C, rts_in_interrupt);

    procedure rts_init_interrupt_thread(Thread : Thread_Ptr);
    -- This procedure initializes the portions of the 
    -- tgt kernel thread structure that the interrupt needs
    -- initialized.
    pragma export(C, rts_init_interrupt_thread);

    procedure rts_finish_system_thread;
    pragma export(C, rts_finish_system_thread);

    procedure Set_Active_Priority( Thread : Thread_Ptr; Priority : Kernel_Priority; 
		    Yielding : Boolean := False );
    function Get_Base_Priority( Thread : Thread_Ptr) return Kernel_Priority;
    pragma Inline (Get_Base_Priority);
    
    function Get_Active_Priority( Thread : Thread_Ptr) return Kernel_Priority;
    pragma Inline (Get_Active_Priority);

    procedure EnteringMutex;
    pragma InLine(EnteringMutex);
    
    function ExitingMutex return DWORD;
    pragma Inline(ExitingMutex);
    
    Use_PO_Ceiling_Lock : Boolean;

private
    NoThread : constant DWORD := 0;
    type Lock_Type is record
       Event   : Dword := 0;
       Waiters : Dword := 0;
       Owner   : Dword := NoThread;
    end record;

    Suspension_Lock_Object : aliased Lock_Type;
    PO_Ceiling_Lock_Object : aliased Lock_Type;

    SuspensionLock        : constant Lock_Access:=Suspension_Lock_Object'access;
    PO_Ceiling_Lock        : constant Lock_Access:=PO_Ceiling_Lock_Object'access;
    
end System.RTS.TGT.Kernel.Threads.NT;
