package System.RTS.TGT.Fixed_Arithmetic_Pkg is

    pragma Suppress( Elaboration_Check );

    subtype Fixed             is Integer;
    subtype Short_Fixed       is Short_Integer;    
    subtype Short_Short_Fixed is Short_Short_Integer;    

    subtype Shift_Amount is Integer;
   

    function RTS_Fixed_Mul_Div_F(
      Start      : Fixed;
      Multiplier : Fixed;
      Shift      : Shift_Amount;
      Divisor    : Fixed )
	return Fixed;
    -- Implements   ( ( Start * Multiplier ) << Shift ) / Divisor
    -- with Don't Care rounding.
    pragma export( Ada,
                   RTS_Fixed_Mul_Div_F,
                   "rts_fixed_mul_div" );

    function RTS_Fixed_Mul_Div_Round_F(
      Start      : Fixed;
      Multiplier : Fixed;
      Shift      : Shift_Amount;
      Divisor    : Fixed )
	return Fixed;
    -- Implements   ( (Start * Multiplier ) << Shift ) / Divisor
    -- with Rounding.
       pragma Export( Ada,
		      RTS_Fixed_Mul_Div_Round_F,
                      "rts_fixed_mul_div_round" );

    function RTS_Fixed_Mul_Div_Trunc_F(
      Start      : Fixed;
      Multiplier : Fixed;
      Shift      : Shift_Amount;
      Divisor    : Fixed )
	return Fixed;
    -- Implements   ( (Start * Multiplier ) << Shift ) / Divisor
    -- with Truncating.
       pragma Export( Ada,
                      RTS_Fixed_Mul_Div_Trunc_F,
                      "rts_fixed_mul_div_trunc" );

    function SdivR_F(
      Left  : Short_Short_Fixed;
      Right : Short_Short_Fixed )
	return Short_Short_Fixed;
    -- Implements ( Left / Right )
    -- with Rounding, overflow checks suppressed.
       pragma Export( Ada, SdivR_F, "sdivr" );

    function SdivRCk_F(
      Left  : Short_Short_Fixed;
      Right : Short_Short_Fixed )
	return Short_Short_Fixed;
    -- Implements ( Left / Right )
    -- with Rounding and overflow checking.
       pragma Export( Ada, SdivRCk_F, "sdivrck" );

    function SdivEZ_F(
      Left  : Short_Short_Fixed;
      Right : Short_Short_Fixed )
	return Short_Short_Fixed;
    -- Implements ( Left / Right )
    -- with Don't Care rounding, overflow checks suppressed.
       pragma Export( Ada, SdivEZ_F, "sdivez" );

    function SdivEZCk_F(
      Left  : Short_Short_Fixed;
      Right : Short_Short_Fixed )
	return Short_Short_Fixed;
    -- Implements ( Left / Right )
    -- with Don't Care rounding and overflow checking.
       pragma Export( Ada, sdivEZCk_F, "sdivezck" );

    function DivR_F(
      Left  : Short_Fixed;
      Right : Short_Fixed )
	return Short_Fixed;
    -- Implements ( Left / Right )
    -- with Rounding, overflow checks suppressed.
       pragma Export( Ada, DivR_F, "divr" );

    function DivRCk_F(
      Left  : Short_Fixed;
      Right : Short_Fixed )
	return Short_Fixed;
    -- Implements ( Left / Right )
    -- with Rounding and overflow checking.
       pragma Export( Ada, DivRCk_F, "divrck" );

    function DivEZ_F(
      Left  : Short_Fixed;
      Right : Short_Fixed )
	return Short_Fixed;
    -- Implements ( Left / Right )
    -- with Don't Care rounding, overflow checks suppressed.
       pragma Export( Ada, DivEZ_F, "divez" );

    function DivEZCk_F(
      Left  : Short_Fixed;
      Right : Short_Fixed )
	return Short_Fixed;
    -- Implements ( Left / Right )
    -- with Don't Care rounding and overflow checking.
       pragma Export( Ada, divEZCk_F, "divezck" );

    function LdivR_F(
      Left  : Fixed;
      Right : Fixed )
	return Fixed;
    -- Implements ( Left / Right )
    -- with Rounding, overflow checks suppressed.
       pragma Export( Ada, LdivR_F, "ldivr" );

    function LdivRCk_F(
      Left  : Fixed;
      Right : Fixed )
	return Fixed;
    -- Implements ( Left / Right )
    -- with Rounding and overflow checking.
       pragma Export( Ada, LdivRCk_F, "ldivrck" );

    function LdivEZ_F(
      Left  : Fixed;
      Right : Fixed )
	return Fixed;
    -- Implements ( Left / Right )
    -- with Don't Care rounding, overflow checks suppressed.
       pragma Export( Ada, LdivEZ_F, "ldivez" );

    function LdivEZCk_F(
      Left  : Fixed;
      Right : Fixed )
	return Fixed;
    -- Implements ( Left / Right )
    -- with Don't Care rounding and overflow checking.
       pragma Export( Ada, ldivEZCk_F, "ldivezck" );

    function RTS_Fixed_Fore( Lo : Long_Float;
    	    	    	     Hi : Long_Float ) return Natural;
    -- Body will be in Ada, but must have external name generated by emitter.
    pragma export(Ada, RTS_Fixed_Fore, "rts_fixed_fore" );
			
end System.RTS.TGT.Fixed_Arithmetic_Pkg;
