-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.tgt.kernel.abort_signals.spc,v $ 
-- $Revision: 1.7 $ $Date: 96/02/09 16:49:38 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.TGT.Kernel.Threads;

package System.RTS.TGT.Kernel.Abort_Signals is

    subtype Abort_Signal_Level is Natural;
      -- A level number is passed when an abort signal is sent or
      -- scheduled to be sent at some time.
      -- See also Kernel.Timers.Signal_Abort_Upon_Timeout.
      -- When multiple aborts occur, only the outermost
      -- (lowest number) abort need be delivered.

    type Abort_Signal_Handler is 
      access procedure(Level : Abort_Signal_Level);
	-- The state of a thread when the Abort_Signal_Handler executes
	-- must be such that it can successfully use ATC.Longjmp.

    procedure Establish_Abort_Signal_Handler(Handler : Abort_Signal_Handler);
	-- This is called once at system startup, to establish
	-- abort handler for all threads.

    procedure Signal_Abort(
      Thread : Threads.Thread_Ptr; Level : Abort_Signal_Level);
	-- Send an abort signal to the specified thread, causing the
	-- Abort_Signal_Handler routine to execute in the context of
	-- the thread.
	-- Both this abort signal and ones generated by
	-- Kernel.Timers.Signal_Abort_Upon_Timeout shall release a
	-- thread from a suspended state (Suspend_Until_True or
	-- Suspend_Until_True_Or_Timeout)

    procedure Defer_Abort_Handling;
	-- While abort handling is deferred, any abort signal is
	-- not delivered until abort handling is undeferred.
	-- If multiple aborts were signalled while deferred,
	-- only the one with the lowest number need be signalled
	-- when undeferred.
	-- Defer_Abort_Handling increments a zero-initialized
	-- counter that is decremented by Undefer_Abort_Handling.
	-- Aborts are automatically deferred while a thread is
	-- holding any mutex(es).

    procedure Undefer_Abort_Handling;
	-- Decrements the abort deferral counter, and if zero,
	-- allows any deferred aborts to occur.

    procedure Reset_Abort;
        -- Resets the target-dependent state associated with
        -- an abort;

end System.RTS.TGT.Kernel.Abort_Signals;
