-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.tgt.kernel.timers.spc,v $ 
-- $Revision: 1.7 $ $Date: 96/05/08 10:32:26 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.TGT.Kernel.Threads;
with System.RTS.TGT.Kernel.Abort_Signals;
package System.RTS.TGT.Kernel.Timers is
    
    type Timer_Record is abstract tagged null record;

    function "<"(Left, Right : Timer_Record) return Boolean is abstract;
      -- This function is used to choose the shortest delay,
      -- when there are multiple delay alternatives in a single
      -- selective accept statement. All such delays must be of the
      -- same type - LRM 9.7.1(13).

    procedure Suspend_Until_True_Or_Timeout(
      S : in out Threads.Suspension_Object; 
      Timeout : in out Timer_Record
    ) is abstract;
      -- This procedure suspends the calling thread until
      -- the given suspension object is set True,
      -- or until the Timeout expires.
      -- The action of setting the suspension object to True
      -- effectively cancels the Timeout period, and no further
      -- cleanup by the client is needed.
      -- This operation can be implemented internally by setting
      -- the suspension object True from a timer interrupt handler,
      -- or by using some kind of timed-wait.
      -- If the time has already passed, no suspension occurs.

    procedure Signal_Abort_Upon_Timeout(
      Timeout : in out Timer_Record; 
      Level : Abort_Signals.Abort_Signal_Level) is abstract;
      -- This procedure requests that the calling thread
      -- be sent an abort signal when the Timeout expires.
      -- See also Kernel.Abort_Signals.Signal_Abort.
      -- Multiple abort timeouts may be started by a single thread.
      -- The Timeout object must have a lifetime that includes the
      -- period of time that the Timeout is active, since the
      -- address of the Timer_Record serves as the identification in the
      -- Cancel_Timer operation.
      -- If the time has already passed, the abort is immediate.

    procedure Cancel_Timer(
      Timeout : in out Timer_Record
    ) is abstract;
      -- This procedure requests that the given (abort) timeout
      -- on the calling thread be cancelled.
      -- Generally, if the cancellation did not take place,
      -- this procedure would not return (due to a "long jump"),
      -- unless abort handling was deferred.

    -- There must be an extension of Timer_Record for each supported
    -- time type.

end System.RTS.TGT.Kernel.Timers;
