-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.tgt.kernel.atc.spc,v $ 
-- $Revision: 1.4 $ $Date: 96/04/19 14:28:11 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

package System.RTS.TGT.Kernel.ATC is

    -- This package provides operations to support Asynchronous
    -- Transfer of Control. The generated code and portable RTS
    -- use these operations. The target dependent exception
    -- manager could also use them.

    pragma Suppress( Elaboration_Check );

    type Setjmp_Buffer_Ptr is private;

    function setjmp( 
	Save_Buffer : Setjmp_Buffer_Ptr
    ) return Integer;
    pragma Import (Ada, Setjmp, "rts_setjmp");
        --* OVERVIEW
	-- Setjmp establishes the pre-selected point of execution
	-- to which control is subseqently transferred.  There is
	-- no protection for interruption within setjmp, so the
	-- buffer may not be considered "active" until the return
	-- from setjmp.  Multiple buffers, however, may be "active".
	--
	-- It is erroneous to call setjmp from interrupt, trap, or
	-- breakpoint software.  It is erroneous to invoke longjmp
	-- against a buffer established by a procedure that has
	-- subsequently returned.
        --
        -- This routine is called only from generated code, not from
	-- the portable Run Time System.
        --
        --* EFFECTS
	-- Portions of the current machine state are preserved in
	-- the designated save buffer.  A value of 0 is returned,
	-- indicating the original call to setjmp. A non-zero
	-- value indicates return via longjmp.
        --

    procedure longjmp( 
	Save_Buffer : Setjmp_Buffer_Ptr;
	Val : Integer
    );
    pragma Import (Ada, Longjmp, "rts_longjmp");
        --* OVERVIEW
	-- Longjmp is used to restore the state at a call to setjmp.
	-- The return from longjmp circumvents the call chain built
	-- up to its call and returns as if from the setjmp call
	-- which established the buffer.  Multiple longjmp calls
	-- may be made against the same setjmp buffer.
        --
	-- It is assumed that longjmp will be called from an
	-- Abort_Signal_Handler established by
	-- System.RTS.TGT.Kernel.Abort_Signals.Establish_Abort_Signal_Handler
	-- It is erroneous to call longjmp from interrupt, trap,
	-- or breakpoint software.
        --
        -- This routine is called only from the portable Run Time
	-- System, not from the generated code.
        --
        --* EFFECTS
	-- The state defined by the given buffer is established.
	-- If applicable, abort signals are undeferred (one level).
	-- A return value of Val is established for the return
	-- to the point of setjmp invocation, indicating that
	-- return through longjmp. The portable RTS uses two different
	-- non-zero values to distinguish normal ATC and ATC followed
	-- by exception propagation.
        --

private

    type Setjmp_Buffer is
       record
           null;  -- completely defined by assembler implementation of setjmp/longjmp
       end record;
       
	-- Allocated by compiler, so target dependent size must
	-- be known to compiler.

    type Setjmp_Buffer_Ptr is access all Setjmp_Buffer;

end System.RTS.TGT.Kernel.ATC;
