-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.tgt.kernel.interrupts.spc,v $ 
-- $Revision: 1.6 $ $Date: 96/10/29 15:10:19 $ $Author: bethc $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with Ada.Interrupts;
with System.RTS.TGT.Kernel.Mutexes;
package System.RTS.TGT.Kernel.Interrupts is

--  OVERVIEW
--  This package contains the kernel interface supporting 
--  operations of Ada.Interrupts and the attachment of static handlers.

    -- The access to procedure types below 
    type Interrupt_Handler is
	access procedure(Mutex  : Mutexes.Mutex_Ptr; 
			 Params : System.Address);

    type Queue_Scanner is
	access procedure(Mutex : Mutexes.Mutex_Ptr);

    Default_Handler : constant Interrupt_Handler := null;
    	-- Represents the default treatment of interrupts in the
	-- absence of a handler.

    function Is_Reserved(
	Interrupt : Ada.Interrupts.Interrupt_ID
    ) return Boolean;
    --* OVERVIEW
    -- Returns True if Interrupt is reserved for use by the kernel.
    -- If referenced in  other operations of this package, 
    -- Program_Error is raised.

    procedure Exchange_Handler(
	Interrupt     : Ada.Interrupts.Interrupt_ID;
	New_Handler   : Interrupt_Handler;
	New_Mutex     : Mutexes.Mutex_Ptr;
        New_Scanner   : Queue_Scanner;
	Old_Handler   : out Interrupt_Handler;
	Old_Mutex     : out Mutexes.Mutex_Ptr
    );
    --* OVERVIEW
    --  Causes New_Handler to receive delivery of interrupts
    --  for the designated ID. The handler is passed the New_Mutex
    --  as its parameter.  The output parameters are used in the
    --  context of attaching a static handler.  A Scanner parameter 
    --  is not returned because the RTS can easily determine it from 
    --  Mutex.  (The kernel cannot).  If New_Handler = null, 
    --  the default treatment is restored, and the Mutex parameter is
    --  ignored.  


    procedure Current_Handler(
	Interrupt : Ada.Interrupts.Interrupt_ID;
	Handler   : out Interrupt_Handler;
	Mutex     : out Mutexes.Mutex_Ptr
    );
    --* OVERVIEW
    --  Returns the current handler for the interrupt.
    --  If there is no current handler (default treatment),
    --  both Handler and Mutex are set := null;

    function Priority_From_Id(
	Interrupt : Ada.Interrupts.Interrupt_ID
    ) return System.Interrupt_Priority;
    --* OVERVIEW
    -- Returns the priority to use for the ceiling priority
    -- of a protected object to which the interrupt is statically
    -- attached.

pragma inline(Is_Reserved);
pragma inline(Priority_From_Id);
pragma Interface(C, Exchange_Handler, External_Name=>"rts_tgt_exchange_handler");
pragma Interface(C, Current_Handler, External_Name=>"rts_tgt_current_handler");

end System.RTS.TGT.Kernel.Interrupts;










