-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.timer_class.spc,v $ 
-- $Revision: 1.7 $ $Date: 96/10/23 15:18:17 $ $Author: stt $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with Ada.Unchecked_Conversion; 
with System.RTS.TGT.Kernel.Timers;
with System.RTS.Interface_Types;
with System.Storage_Elements;

package System.RTS.Timer_Class is
    -- Support multiple Time types, including (future) user-defined types,
    -- by providing a Timer_Record'Class container type, with operations
    -- to initialize the container and convert to Timer_Record'Class.

    pragma Suppress( Elaboration_Check );

    type Timer_Class_Container is private;
	-- This type is used to hold objects of type Timer_Record'Class.
	-- Init_Container raises an exception if the size or alignment
	-- of the container type is insufficient to hold the timer_record type.
	-- Package elaboration also raises the same (internal) exception
	-- if the container type is similarly insufficient for
	-- the Timer_Record types for Calendar.Time and Real_Time.Time.
    
    type Container_Access is access all Timer_Class_Container;

    procedure Init_Container(
	Container  : out Timer_Class_Container;
	Time       : Interface_Types.Absolute_Delay_Handle;
	Which_Type : Interface_Types.Time_Type_Enum
    );
	-- Initializes the container with the appropriate kind of Timer_Record
	-- based on Which_Type with the value Time.
	-- Raises a fatal exception if the size or alignment of Time exceeds
	-- the Container size or alignment.
	-- If this happens, the RTS will need to be rebuilt with a
	-- larger Timer_Class_Length or Timer_Class_Alignment.

    type Timer_Record_Ptr is access all TGT.Kernel.Timers.Timer_Record'Class;
	-- This access type is used when accessing the contents of a
	-- Timer_Class_Container as a Timer_Record'Class object
    for Timer_Record_Ptr'Storage_Size use 0;
	-- Suppress registration of collection for finalization

    function To_Timer_Record_Ptr is new Ada.Unchecked_Conversion(
                    Source => Container_Access,
		    Target => Timer_Record_Ptr );
	-- Allows access to Timer_Record'Class object in Timer_Class_Container

private

    Timer_Class_SE_Size : constant := 16;
        -- Number of storage elements to hold any Timer_Record'Class object
        -- Assumptions: 1 Tag word + 2 data words = 3 words * 4 SE/word.
        -- One additional word for NT (address of SO)

    Timer_Class_Alignment : constant := 1;
	-- The biggest alignment needed to hold any Timer_Record'Class object

    type Timer_Class_Container is new
	Storage_Elements.Storage_Array(1 .. Timer_Class_SE_Size);
    
    for Timer_Class_Container'Alignment use Timer_Class_Alignment;

end System.RTS.Timer_Class;
