-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.strings.wide_bounded.bdy,v $ 
-- $Revision: 1.6 $ $Date: 96/04/16 13:49:29 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

pragma Suppress( Elaboration_Check );

with System.RTS.Wide_Strings_Search;
with Ada.Strings.Wide_Fixed;

package body Ada.Strings.Wide_Bounded is
pragma Suppress( Elaboration_Check );

    package body Generic_Bounded_Length is

        package ASM renames Ada.Strings.Wide_Maps; 
        package ASF renames Ada.Strings.Wide_Fixed;
        package RSS renames System.RTS.Wide_Strings_Search;
 
        function Length (Source : in Bounded_Wide_String) return Length_Range is
        begin
            return Source.Length;
        end Length;

        function To_Bounded_Wide_String (Source : in Wide_String;
                                         Drop   : in Truncation := Error)
           return   Bounded_Wide_String is
            SLen   : constant integer := Source'Length;
            Result : Bounded_Wide_String;
        begin
            if Slen <= Max_Length then
                Result.Length := Slen;
                Result.Data (1 .. Slen) := Source;
            else
                case Drop is
                    when Left =>
                        Result.Length := Max_Length;
                        Result.Data := Source (Source'Last - (Max_Length - 1) ..
				               Source'Last);
                    when Right =>
                        Result.Length := Max_Length;
                        Result.Data := Source (Source'First ..
				               Source'First + (Max_Length - 1));
                    when others =>
                        raise Length_Error;
                end case;
            end if;
            return Result;
        end To_Bounded_Wide_String;

        function To_Wide_String (Source : in Bounded_Wide_String)
	   return Wide_String is
        begin
           return Source.Data (1 .. Source.Length);
        end To_Wide_String;

        function Append (Left, Right : in Bounded_Wide_String;
                         Drop        : in Truncation  := Error)
           return Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(
		       To_Wide_String(Left) & To_Wide_String(Right),
		       Drop);
        end Append;

        function Append (Left  : in Bounded_Wide_String;
                         Right : in Wide_String;
                         Drop  : in Truncation := Error)
           return  Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(To_Wide_String(Left) & Right, Drop);
	end Append;

        function Append (Left  : in Wide_String;
                         Right : in Bounded_Wide_String;
                         Drop  : in Truncation := Error)
	   return Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(Left & To_Wide_String(Right), Drop);
        end Append;

        function Append (Left  : in Bounded_Wide_String;
                         Right : in Wide_Character;
                         Drop  : in Truncation := Error)
           return  Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(To_Wide_String(Left) & Right, Drop);
        end Append;

        function Append (Left  : in Wide_Character;
                         Right : in Bounded_Wide_String;
                         Drop  : in Truncation := Error)
           return  Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(Left & To_Wide_String(Right), Drop);
        end Append;

        procedure Append (Source   : in out Bounded_Wide_String;
                          New_Item : in Bounded_Wide_String;
                          Drop     : in Truncation  := Error) is
        begin
	    Source := Append(Source, New_Item, Drop);
        end Append;

        procedure Append (Source   : in out Bounded_Wide_String;
                          New_Item : in Wide_String;
                          Drop     : in Truncation  := Error) is
        begin
	    Source := Append(Source, New_Item, Drop);
        end Append;

        procedure Append (Source   : in out Bounded_Wide_String;
                          New_Item : in Wide_Character;
                          Drop     : in Truncation  := Error) is
        begin
	    Source := Append(Source, New_Item, Drop);
        end Append;

        function "&" (Left, Right : in Bounded_Wide_String)
           return Bounded_Wide_String is
        begin
	    return Append(Left, Right, Error);
        end "&";

        function "&" (Left : in Bounded_Wide_String; Right : in Wide_String)
           return Bounded_Wide_String is
        begin
	    return Append(Left, Right, Error);
        end "&";

        function "&" (Left : in Wide_String; Right : in Bounded_Wide_String)
           return Bounded_Wide_String is
        begin
            return Append(Left, Right, Error);
        end "&";

        function "&" (Left  : in Bounded_Wide_String;
                      Right : in Wide_Character) return  Bounded_Wide_String is
        begin
            return Append(Left, Right, Error);
        end "&";

        function "&" (Left : in Wide_Character; Right : in Bounded_Wide_String)
           return Bounded_Wide_String is
        begin
            return Append(Left, Right, Error);
        end "&";

        function Element (Source : in Bounded_Wide_String;
                          Index  : in Positive) return Wide_Character is
        begin
            if Index <= Length(Source) then
                return Source.Data(Index);
            else
                raise Index_Error;
            end if;
        end Element;

        procedure Replace_Element (Source : in out Bounded_Wide_String;
                                   Index  : in Positive;
                                   By     : in Wide_Character) is
        begin
            if Index <= Length(Source) then
                Source.Data(Index) := By;
            else
                raise Index_Error;
            end if;
        end Replace_Element;

        function Slice (Source : in Bounded_Wide_String;
                        Low    : in Positive;
                        High   : in Natural)
           return Wide_String is
        begin
            if Low > Length(Source) + 1 or else High > Length(Source) then
                raise Index_Error;
            else
                return Source.Data (Low .. High);
            end if;
        end Slice;

        function "=" (Left, Right : in Bounded_Wide_String) 
           return Boolean is
        begin
            return (To_Wide_String(Left) = To_Wide_String(Right));
        end "=";

        function "="  (Left : in Bounded_Wide_String; Right : in Wide_String)
           return Boolean is
        begin
            return (To_Wide_String(Left) = Right);
        end "=";

        function "="  (Left : in Wide_String; Right : in Bounded_Wide_String)
           return Boolean is
        begin
            return (Left = To_Wide_String(Right));
        end "=";

        function "<" (Left, Right : in Bounded_Wide_String)
           return Boolean is
        begin
            return (To_Wide_String(Left) < To_Wide_String(Right));
        end "<";

        function "<"  (Left : in Bounded_Wide_String; Right : in Wide_String)
           return Boolean is
        begin
            return (To_Wide_String(Left) < Right);
        end "<";

        function "<"  (Left : in Wide_String; Right : in Bounded_Wide_String)
           return Boolean is
        begin
            return (Left < To_Wide_String(Right));
        end "<";

        function "<=" (Left, Right : in Bounded_Wide_String)
           return Boolean is
        begin
            return (To_Wide_String(Left) <= To_Wide_String(Right));
        end "<=";

        function "<="  (Left : in Bounded_Wide_String; Right : in Wide_String)
           return Boolean is
        begin
            return (To_Wide_String(Left) <= Right);
        end "<=";

        function "<="  (Left : in Wide_String; Right : in Bounded_Wide_String)
           return Boolean is
        begin
            return (Left <= To_Wide_String(Right));
        end "<=";

        function ">" (Left, Right : in Bounded_Wide_String)
           return Boolean is
        begin
            return (To_Wide_String(Left) > To_Wide_String(Right));
        end ">";

        function ">"  (Left : in Bounded_Wide_String; Right : in Wide_String)
           return Boolean is
        begin
            return (To_Wide_String(Left) > Right);
        end ">";

        function ">"  (Left : in Wide_String; Right : in Bounded_Wide_String)
           return Boolean is
        begin
            return (Left > To_Wide_String(Right));
        end ">";

        function ">=" (Left, Right : in Bounded_Wide_String)
           return Boolean is
        begin
            return (To_Wide_String(Left) >= To_Wide_String(Right));
        end ">=";

        function ">="  (Left : in Bounded_Wide_String; Right : in Wide_String)
           return Boolean is
        begin
            return (To_Wide_String(Left) >= Right);
        end ">=";

        function ">="  (Left : in Wide_String; Right : in Bounded_Wide_String)
           return Boolean is
        begin
            return (Left >= To_Wide_String(Right));
        end ">=";

        function Index (Source   : in Bounded_Wide_String;
                        Pattern  : in Wide_String;
                        Going    : in Strings.Direction := Strings.Forward;
                        Mapping  : in Wide_Maps.Wide_Character_Mapping :=
				      Wide_Maps.Identity)
           return Natural is
        begin
            return RSS.Index (Source.Data (1 .. Source.Length),
                              Pattern,
                              Going, 
                              Mapping);
        end Index;

        function Index (Source   : in Bounded_Wide_String;
                        Pattern  : in Wide_String;
                        Going    : in Direction := Forward;
                        Mapping  : in Wide_Maps.Wide_Character_Mapping_Function)
           return Natural is
        begin
            return RSS.Index (Source.Data (1 .. Source.Length),
                              Pattern,
                              Going,
                              Mapping);
        end Index;

        function Index (Source : in Bounded_Wide_String;
                        Set    : in Wide_Maps.Wide_Character_Set;
                        Test   : in Strings.Membership := Strings.Inside;
                        Going  : in Strings.Direction  := Strings.Forward)
           return Natural is
        begin
            return RSS.Index (Source.Data (1 .. Source.Length),
                              Set,
                              Test,
                              Going);
        end Index;

        function Index_Non_Blank (Source : in Bounded_Wide_String;
                                  Going  : in Direction := Forward)
           return Natural is
        begin
            return RSS.Index_Non_Blank (Source.Data (1 .. Source.Length),
                                        Going);
        end Index_Non_Blank;

        function Count (Source   : in Bounded_Wide_String;
                        Pattern  : in Wide_String;
                        Mapping  : in Wide_Maps.Wide_Character_Mapping := 
				      Wide_Maps.Identity)
           return Natural is
        begin
            return RSS.Count (Source.Data (1 .. Source.Length),
                              Pattern,
                              Mapping);
        end Count;

        function Count (Source   : in Bounded_Wide_String;
                        Pattern  : in Wide_String;
                        Mapping  : in Wide_Maps.Wide_Character_Mapping_Function)
           return Natural is
        begin
            return RSS.Count (Source.Data (1 .. Source.Length),
                              Pattern,
                              Mapping);
        end Count;

        function Count (Source : in Bounded_Wide_String;
                        Set    : in Wide_Maps.Wide_Character_Set)
           return   Natural is
        begin
            return RSS.Count (Source.Data (1 .. Source.Length), Set);
        end Count;

        procedure Find_Token (Source : in Bounded_Wide_String;
                              Set    : in Wide_Maps.Wide_Character_Set;
                              Test   : in Strings.Membership;
                              First  : out Positive;
                              Last   : out Natural) is
        begin
            RSS.Find_Token (Source.Data (1 .. Source.Length),
                            Set,
                            Test,
                            First,
                            Last);
        end Find_Token;

        function Translate (Source  : in Bounded_Wide_String;
                            Mapping : in Wide_Maps.Wide_Character_Mapping)
           return Bounded_Wide_String is
        begin
            return To_Bounded_Wide_String(ASF.Translate(To_Wide_String(Source), 
						        Mapping));
        end Translate;

        procedure Translate (Source : in out Bounded_Wide_String;
                             Mapping  : in Wide_Maps.Wide_Character_Mapping) is
        begin
            Source := To_Bounded_Wide_String(
			  ASF.Translate(To_Wide_String(Source), Mapping));
        end Translate;

        function Translate (
	    Source  : in Bounded_Wide_String;
            Mapping : in Wide_Maps.Wide_Character_Mapping_Function)
           return Bounded_Wide_String is
        begin
            return To_Bounded_Wide_String(ASF.Translate(To_Wide_String(Source),
						        Mapping));
        end Translate;

        procedure Translate (
	    Source  : in out Bounded_Wide_String;
            Mapping : in Wide_Maps.Wide_Character_Mapping_Function) is
        begin
            Source := To_Bounded_Wide_String(
			  ASF.Translate(To_Wide_String(Source), Mapping));
        end Translate;

        function Replace_Slice (Source   : in Bounded_Wide_String;
                                Low      : in Positive;
                                High     : in Natural;
                                By       : in Wide_String;
                                Drop     : in Truncation := Error)
           return Bounded_Wide_String is
        begin
            return To_Bounded_Wide_String(
		       ASF.Replace_Slice(To_Wide_String(Source),
				         Low,
				         High,
				         By),
		       Drop);
        end Replace_Slice;

        procedure Replace_Slice (Source   : in out Bounded_Wide_String;
                                 Low      : in Positive;
                                 High     : in Natural;
                                 By       : in Wide_String;
                                 Drop     : in Truncation := Error) is
        begin
            Source := To_Bounded_Wide_String(
			  ASF.Replace_Slice(To_Wide_String(Source),
					    Low,
					    High,
					    By),
		          Drop);
        end Replace_Slice;

        function Insert (Source   : in Bounded_Wide_String;
                         Before   : in Positive;
                         New_Item : in Wide_String;
                         Drop     : in Truncation := Error)
           return     Bounded_Wide_String is
        begin
            return To_Bounded_Wide_String(ASF.Insert(To_Wide_String(Source),
			                             Before,
				                     New_Item),
				          Drop);
        end Insert;

        procedure Insert (Source   : in out Bounded_Wide_String;
                          Before   : in Positive;
                          New_Item : in Wide_String;
                          Drop     : in Truncation := Error) is
        begin
            Source := To_Bounded_Wide_String(ASF.Insert(To_Wide_String(Source),
			                                Before,
				                        New_Item),
				             Drop);
        end Insert;

        function Overwrite (Source    : in Bounded_Wide_String;
                            Position  : in Positive;
                            New_Item  : in Wide_String;
                            Drop      : in Strings.Truncation := Strings.Error)
           return      Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(ASF.Overwrite(To_Wide_String(Source),
						        Position,
					                New_Item),
				          Drop);
        end Overwrite;

        procedure Overwrite (Source    : in out Bounded_Wide_String;
                             Position  : in Positive;
                             New_Item  : in Wide_String;
                             Drop      : in Truncation := Error) is
        begin
	    Source := To_Bounded_Wide_String(
			  ASF.Overwrite(To_Wide_String(Source),
					Position,
					New_Item),
			  Drop);
        end Overwrite;

        function Delete (Source  : in Bounded_Wide_String;
                         From    : in Positive;
                         Through : in Natural)
           return    Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(ASF.Delete(To_Wide_String(Source),
						     From,
						     Through));
        end Delete;

        procedure Delete (Source  : in out Bounded_Wide_String;
                          From    : in Positive;
                          Through : in Natural) is
        begin
	    Source := To_Bounded_Wide_String(ASF.Delete(To_Wide_String(Source),
						        From,
						        Through));
        end Delete;

        function Trim (Source : in Bounded_Wide_String; Side : in Trim_End)
           return Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(
		ASF.Trim(To_Wide_String(Source), Side));
        end Trim;

        procedure Trim (Source : in out Bounded_Wide_String;
                        Side   : in Trim_End) is
        begin
	    Source := To_Bounded_Wide_String(
		ASF.Trim(To_Wide_String(Source), Side));
        end Trim;

        function Trim (Source : in Bounded_Wide_String;
                       Left   : in Wide_Maps.Wide_Character_Set;
                       Right  : in Wide_Maps.Wide_Character_Set)
           return   Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(ASF.Trim(To_Wide_String(Source), 
						   Left,
						   Right));
        end Trim;

        procedure Trim (Source : in out Bounded_Wide_String;
                        Left   : in Wide_Maps.Wide_Character_Set;
                        Right  : in Wide_Maps.Wide_Character_Set) is
        begin
	    Source := To_Bounded_Wide_String(ASF.Trim(To_Wide_String(Source),
						      Left,
						      Right));

        end Trim;

        function Head (Source : in Bounded_Wide_String;
                       Count  : in Natural;
                       Pad    : in Wide_Character := Wide_Space;
                       Drop   : in Truncation := Error)
           return Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(ASF.Head(To_Wide_String(Source),
				                   Count,
				                   Pad),
		                          Drop);
        end Head;

        procedure Head (Source : in out Bounded_Wide_String;
                        Count  : in Natural;
                        Pad    : in Wide_Character  := Wide_Space;
                        Drop   : in Truncation := Error) is
        begin
	    Source := To_Bounded_Wide_String(ASF.Head(To_Wide_String(Source),
						      Count,
					              Pad),
				             Drop);
        end Head;

        function Tail (Source : in Bounded_Wide_String;
                       Count  : in Natural;
                       Pad    : in Wide_Character := Wide_Space;
                       Drop   : in Truncation := Error)
           return   Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(ASF.Tail(To_Wide_String(Source),
					           Count,
					           Pad),
				          Drop);
        end Tail;

        procedure Tail (Source : in out Bounded_Wide_String;
                        Count  : in Natural;
                        Pad    : in Wide_Character  := Wide_Space;
                        Drop   : in Truncation := Error) is
        begin
	    Source := To_Bounded_Wide_String(ASF.Tail(To_Wide_String(Source),
					              Count,
					              Pad),
					     Drop);
        end Tail;

        function "*" (Left  : in Natural;
                      Right : in Wide_Character)
           return  Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(ASF."*"(Left, Right), Error);
        end "*";

        function "*" (Left  : in Natural;
                      Right : in Wide_String)
           return  Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(ASF."*"(Left, Right), Error);
        end "*";

        function "*" (Left  : in Natural;
                      Right : in Bounded_Wide_String)
           return  Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(ASF."*"(Left,
						  To_Wide_String(Right)),
						  Error);
        end "*";

        function Replicate (Count : in Natural;
                            Item  : in Wide_Character;
                            Drop  : in Truncation := Error)
           return  Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(ASF."*"(Count, Item), Drop);
	end Replicate;

        function Replicate (Count : in Natural;
                            Item  : in Wide_String;
                            Drop  : in Truncation := Error)
           return  Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(ASF."*"(Count, Item), Drop);
	end Replicate;

        function Replicate (Count : in Natural;
                            Item  : in Bounded_Wide_String;
                            Drop  : in Truncation := Error)
           return  Bounded_Wide_String is
        begin
	    return To_Bounded_Wide_String(ASF."*"(Count,
						  To_Wide_String(Item)),
						  Drop);
        end Replicate;

   end Generic_Bounded_Length;

end Ada.Strings.Wide_Bounded;
