with Ada_Store.PoST.Input_Queue;
with Ada_Store.Trading.Item;
with Ada_Store.Trading.Transaction.Sale;
with Ada_Store.User;

private package Ada_Store.PoST.Application.Operations is

   procedure Initialise;
   procedure Finalise;

   type Input_Allowed is array(Positive range <>) 
      of Input_Queue.Input_Action;

   function  Get_Input
     (Allow	     : in Input_Allowed;
      Must_Have_Text : in Boolean	     := False;
      Prompt	     : in String	     := "")
      return Input_Queue.Message_Ptr;

   procedure User_Sign_On
     (Current_User   : in out User.Instance;
      User_As_String : in     String);

   procedure User_Sign_Off
     (Current_User   : in out User.Instance);

   procedure Display_Welcome;

   procedure Error
     (Error_Text  : in String;
      Alarm_Sound : in Boolean := True);

   procedure Transaction_New
     (The_Sale : in out Trading.Transaction.Sale.Instance);

   procedure Transaction_Item
     (Sales_Transaction	: in out Trading.Transaction.Sale.Instance;
      Item_As_String	: in     String);

   procedure Transaction_Department
     (Sales_Transaction	: in out Trading.Transaction.Sale.Instance;
      Dept_As_String	: in     String);

   procedure Transaction_SubTotal
     (Sales_Transaction : in out Trading.Transaction.Sale.Instance);

   procedure Transaction_Pay
     (Sales_Transaction	   : in out Trading.Transaction.Sale.Instance;
      Amount_As_String	   : in     String;
      Payment_Key	   : in     Input_Queue.Input_Action;
      Transaction_Complete :    out Boolean);

   procedure Transaction_Complete
     (Sales_Transaction : in out Trading.Transaction.Sale.Instance);

   procedure Transaction_Void 
     (Sales_Transaction : in out Trading.Transaction.Sale.Instance);

   procedure Price_Enquiry
     (Item_As_String    : in String);

end Ada_Store.PoST.Application.Operations;
