with Ada_Store.PoST.Input_Queue;

package Ada_Store.PoST.Device.Keyboard is

   type Instance 	is new Device.Instance with private;

   type Lock_Status 	is new Positive range 1 .. 4;
   
   function  Current_Status
     (Keyboard : in Instance)
      return Lock_Status;
    
   procedure Open
     (Keyboard : in out Instance);

   procedure Close
     (Keyboard : in out Instance);

   type Function_Key is new Positive range 1 .. 10;
   type Function_Info is
      record
	 Text_Line1 : String(1 .. 6);
	 Text_Line2 : String(1 .. 6);
	 Action     : Input_Queue.Input_Action;
      end record;
   Null_Function : constant Function_Info := ("      ", "      ", Input_Queue.Cancel);

   type Function_Key_Array is array (Function_Key) of Function_Info;

   procedure Set_Function_Keys
     (Keyboard : in out Instance;
      Keys     : in     Function_Key_Array);

   procedure Flush
     (Keyboard	 : in out Instance;
      Flush_Mode : in     Device_Mode);

   procedure Accept_Input
     (Keyboard : in out Instance;
      Input    : in     Input_Queue.Message_Ptr);
	
   Invalid_Input : exception;

private

   Max_Echo_Key : constant Natural := 20;
   Min_Echo_Key : constant Natural := 0;

   type Instance is new Device.Instance with 
      record
	 Function_Keys : Function_Key_Array        := (others => Null_Function);
	 Echo_Keys     : String(1 .. Max_Echo_Key) := (others => ' ');
	 Last_Echo_Key : Natural                   := 0;
      end record;

   procedure Echo_Char
     (Keyboard   : in out Instance;
      Char       : in     Character);

   procedure End_Echo
     (Keyboard   : in out Instance;
      Char       : in     Character);

end Ada_Store.PoST.Device.Keyboard;
