with Ada_Store.Trading.Department;

package Ada_Store.Trading.Item is

   type Instance   is tagged private;
   
   type Identifier is new Natural;
   
   function  Lookup
     (Item_Number : in Identifier)
      return Instance;

   procedure Create_New
     (Item_Number     : in Identifier;
      In_Department   : in Department.Identifier;
      Display_Descr   : in Short_Description;
      Price	      : in Currency;
      Weight_Required : in Boolean := False);

   procedure Remove
     (Item_Number : in Identifier);

   function  In_Department
     (Item : in Instance)
      return Department.Identifier;

   function  Display_Description
     (Item : in Instance)
      return Short_Description;

   function  Price
     (Item : in Instance)
      return Currency;

   function  Weight_Required
     (Item : in Instance)
      return Boolean;

   Invalid_Identifier : exception;
   Invalid_Price      : exception;

private
   Null_Identifier : constant Identifier := 0;

   type Instance is tagged
      record
	 Item_Number	 : Identifier := 0;
	 In_Department	 : Department.Identifier := 0;
	 Display_Descr   : Short_Description := (others => ' ');
	 Price		 : Currency := 0.0;
	 Weight_Required : Boolean := False;
      end record;

end Ada_Store.Trading.Item;    
