-- ***************************************************************************
-- *                       Simple_Tests.adb 
-- * Copyright (c) Simon Johnston & Addison Wesley Longman 1996.
-- *
-- * Description: This example shows various uses of Ada statements.
-- *
-- * Inputs:  None.
-- * Outputs: This program has no outputs, and in fact due to some of the
-- *    loops involved can't even finish.
-- ***************************************************************************
with Ada.Text_IO;
use  Ada.Text_IO;
with Ada.Integer_Text_IO;
use  Ada.Integer_Text_IO;
with Ada.Exceptions;
use  Ada.Exceptions;

procedure Simple_Tests is

   -- please ignore these declarations, see chapter 3.
   Test_1, Test_2, Test_3 : Boolean;
   Test_4 : Integer;
   -- end of declarations.

begin
   pragma Page;
   Assignment_Tests:
      begin
         Put_Line("Assignment_Tests:");

	 Test_1 := True;
	 Test_2 := True;
	 Test_3 := False;
	 Test_4 := 1;
      end Assignment_Tests;

   pragma Page;
   Conditional_Statement_Tests:
      begin
         Put_Line("Conditional_Statement_Tests:");

	 if Test_1 then
	    Put_Line("condition 1");
	 elsif Test_2 then
	    Put_Line("condition 2");
	 else
	    Put_Line("condition 3");
	 end if;

	 if Test_1 and Test_2 then
	    Put_Line("condition 4");
	 end if;

	 if Test_1 and then Test_2 then
	    Put_Line("condition 5");
	 end if;

	 case Test_4 is
	    when 0 => 
               Put_Line("condition 6");
	    when 1 => 
               Put_Line("condition 7");
	    when 2 | 3 | 4 => 
               Put_Line("condition 8");
	    when 5 .. 10 => 
               Put_Line("condition 9");
	    when others => 
               Put_Line("condition 10");
	 end case;
      end Conditional_Statement_Tests;

   pragma Page;
   Loop_Statement_Tests:
      begin
         Put_Line("Loop_Statement_Tests:");

	 loop
	    Put_Line("never ending loop ...");
	 end loop;
	 
	 while Test_2 loop
	    Test_2 := False;
	 end loop;
	 
	 for New_Variable in 1 .. 10 loop
	    Put_Line("You should see 10 of these");
	 end loop;

	 for New_Variable_2 in reverse 1 .. 10 loop
	    Put_Line("You should see 10 of these as well");
	 end loop;

	 Test_2 := True;

         Outer_Loop:
	    while Test_1 loop
	       while Test_2 loop
		  exit Outer_Loop when Test_4 > 0;
	       end loop;
	    end loop Outer_Loop;
      end Loop_Statement_Tests;

   pragma Page;
   Goto_Test:
      begin
         Put_Line("Goto_Test:");

	 goto Goto_Target;
         Put_Line("You should never see this");

	 <<Goto_Target>>
	 null;
      end Goto_Test;

   pragma Page;
   Exception_Tests:
      begin
         Put_Line("Exception_Tests:");

	 begin
	    raise Program_Error;
	 exception
	    when Constraint_Error =>
	       Put_Line("caught Constraint_Error");
	    when Program_Error =>
	       Put_Line("caught Program_Error");
	    when The_Exception : others =>
	       Put_Line("caught " & Exception_Name(The_Exception)); 
	 end ;
      end Exception_Tests;

      Put_Line("Tests Complete.");

end Simple_Tests;
