-- ***************************************************************************
-- *                         Date_Time.adb 
-- * Copyright (c) Simon Johnston & Addison Wesley Longman 1996.
-- *
-- * Description: This example builds upon the smaller examples in section
-- *    3.3.1 on discrete types. It also demonstrates some uses of scalar
-- *    attributes,
-- * Inputs:  None.
-- * Outputs: None.
-- ***************************************************************************

procedure Date_Time is

   type All_Years is new Integer;
   subtype Before_Christ is All_Years range All_Years'First .. 0;
   subtype Ano_Domini is All_Years range 0 .. All_Years'First;

   type All_Months is (January, February, March,
		       April, May, June,
		       July, August, September,
		       October, November, December);

   type Days_In_Month is new Integer range 1 .. 31;
   for Days_In_Month'Size use 8;

   type All_Days is (Monday, Tuesday, Wednesday,
		     Thursday, Friday, Saturday,
		     Sunday);

   subtype Week_Days is All_Days range Monday .. Friday;
   subtype Week_Ends is All_Days range Saturday .. Sunday;

   type A_Complete_Date is 
      record
	 Year  : All_Years  := 0;
	 Month : All_Months := January;
	 Day   : Days_In_Month;
      end record;


   Today : A_Complete_Date := (1970, April, 16);
   A_Day : All_Days := Wednesday;
   Yesterday, Tomorrow : All_Days;
begin
   if A_Day in Week_Days then
      null; -- go to work.
   else
      null; -- something else instead.
   end if;

   if A_Day = Monday then
      Yesterday := Sunday;
   else
      Yesterday := All_Days'Pred(A_Day);
   end if;

   if A_Day = Sunday then
      Tomorrow := Monday;
   else
      Tomorrow := All_Days'Succ(A_Day);
   end if;

   declare
      Days_Name : String := All_Days'Image(A_Day);
   begin
      null;
   end ;
end Date_Time;
