-- ***************************************************************************
-- *                         Card_Pack.adbs
-- * Copyright (c) Simon Johnston & Addison Wesley Longman 1996.
-- *
-- * Description: This is a specification for a package which can represent a
-- *    pack of cards. It demonstrates the use of the standard library random 
-- *    number generator.
-- * Inputs:  None.
-- * Outputs: None.
-- ***************************************************************************

with Ada.Numerics.Discrete_Random;

package Card_Pack is

   type Suits is ( Diamonds, Clubs, Hearts, Spades );
   type Cards is new Positive range 1 .. 13;

   Jack  : constant Cards := 11;
   Queen : constant Cards := 12;
   King  : constant Cards := 13;

   type Pack is limited private;

   procedure Shuffle(The_Pack : in out Pack);

   procedure Draw(The_Pack : in out Pack;
                  The_Suit : out Suits;
                  The_Card : out Cards);

private
   type Index is new Positive range 1 .. 52;
   type Card_Array is array (Index) of Boolean;
   pragma Pack(Card_Array);

   package Random_Card is new Ada.Numerics.Discrete_Random(Index);

   type Pack is limited
      record
         All_Cards : Card_Array := (others => false);
         Gen       : Random_Card.Generator;
      end record;

end Card_Pack;
