-- ***************************************************************************
-- *                         Shared_Poll.ads 
-- * Copyright (c) Simon Johnston & Addison Wesley Longman 1996.
-- *
-- * Description: The specification for a package which implements a storage
-- *    pool. It is expected that pool specific objects are actually allocated
-- *    in some form of shared memory.
-- * Inputs:  None.
-- * Outputs: None.
-- ***************************************************************************

with System.Storage_Pools;
with System.Storage_Elements;
use  System;

package Shared_Pool is

   type Pool_Name is access constant String;

   type Pool(Size : Storage_Elements.Storage_Count;
             Name : Pool_Name) is
      new Storage_Pools.Root_Storage_Pool with private;

   procedure Allocate
     (The_Pool                : in out Pool;
      Storage_Address         : out Address;
      Size_In_Storage_Elements: in Storage_Elements.Storage_Count;
      Alignment               : in Storage_Elements.Storage_Count);

   procedure Deallocate
     (The_Pool                : in out Pool;
      Storage_Address         : in Address;
      Size_In_Storage_Elements: in Storage_Elements.Storage_Count;
      Alignment               : in Storage_Elements.Storage_Count);

   function Storage_Size
     (The_Pool : in Pool)
      return Storage_Elements.Storage_Count;

private

   procedure Initialize (Object : in out Pool);
   procedure Finalize   (Object : in out Pool);

   type Pool(Size : Storage_Elements.Storage_Count;
             Name : Pool_Name) is 
      new Storage_Pools.Root_Storage_Pool with 
      record
         Ex : Integer;
      end record;

end Shared_Pool;
