-- ***************************************************************************
-- *                         String_Test.adb 
-- * Copyright (c) Simon Johnston & Addison Wesley Longman 1996.
-- *
-- * Description: This tests the standard library string functions for fixed
-- *    length strings (ie the builtin type String).
-- * Inputs:  None.
-- * Outputs: See book text.
-- ***************************************************************************

with Ada.Strings; use Ada.Strings;
with Ada.Strings.Fixed; use Ada.Strings.Fixed;
with Ada.Text_IO; use Ada.Text_IO;

procedure String_Test is

   String_1 : String := "Hello";
   String_2 : String(1 .. 20);
   String_3 : String := "1234567890ABCDEFGHIJ";
   Result   : Natural;
begin

   Move(Source => String_1, 
	Target => String_2, 
	Justify => Left);
   Put_Line(" 1: " & String_2 & ".");

   Move(Source => String_1, 
	Target => String_2, 
	Justify => Right);
   Put_Line(" 2: " & String_2 & ".");

   Move(Source => String_1, 
	Target => String_2, 
	Justify => Left,
	Pad => '*');
   Put_Line(" 3: " & String_2 & ".");

   Replace_Slice(Source => String_2, 
		 Low => String_1'First,
		 High => String_1'Last,
		 By => "HELLO");
   Put_Line(" 4: " & String_2 & ".");

   begin
      Insert(Source => String_2, 
	     Before => 3,
	     New_Item => "mum");
   exception
      when Length_Error =>
	 Put_Line(" 5: String is now too long!");
   end ;

   
   Delete(Source => String_2, 
	  From => String_1'Last,
	  Through => String_2'Last);
   Put_Line(" 6: " & String_2 & ".");

   New_Line;

   Put_Line(" 7: " & Head(Source => String_3, Count => 5) & ".");

   Put_Line(" 8: " & Tail(Source => String_3, Count => 5) & ".");

   Put_Line(" 9: " & (20 * '+') & ".");

   Put_Line("10: " & (5 * "Ada ") & ".");

   String_3(11 .. 20) := String_3(1 .. 10);
   New_Line;

   Result := Index(Source => String_3,
		   Pattern => "8");
   Put_Line("11: First '8' in String_3 is " & Natural'Image(Result));
				
   Result := Index(Source => String_3,
		   Pattern => "8",
		   Going=> Backward);
   Put_Line("12: Last '8' in String_3 is " & Natural'Image(Result));
				
   Result := Ada.Strings.Fixed.Count(Source => String_3,
				     Pattern => "8");
   Put_Line("13: Number of '8' in String_3 is " & Natural'Image(Result));
   -- full qualification due to a Count specified in
   -- the package Ada.Text_IO.
end String_Test;
