--[Simple_io.ads] specification
-- Version of March 1995
WITH Ada.Text_io;
PACKAGE Simple_io IS

  PACKAGE Integer_io IS NEW Ada.Text_io.Integer_io (Integer);
  PACKAGE Float_io   IS NEW Ada.Text_io.Float_io (Float);

  SUBTYPE File_type      IS Ada.Text_io.File_type;
  SUBTYPE File_mode      IS Ada.Text_io.File_mode;
  SUBTYPE Count          IS Ada.Text_io.Count;
  SUBTYPE Positive_count IS Ada.Text_io.Positive_count;
  SUBTYPE Field          IS Ada.Text_io.Field;
  SUBTYPE Number_base    IS Ada.Text_io.Number_base;

  IN_FILE:     CONSTANT File_mode := Ada.Text_io.IN_FILE;
  OUT_FILE:    CONSTANT File_mode := Ada.Text_io.OUT_FILE;
  APPEND_FILE: CONSTANT File_mode := Ada.Text_io.APPEND_FILE;

  -- File management

  PROCEDURE create  ( file : IN OUT File_type;
                      mode : IN File_mode := out_file;
                      name : IN String := "";
                      form : IN String := "" )
                             RENAMES Ada.Text_io.create;
  PROCEDURE open    ( file : IN OUT File_type;
                      mode : IN File_mode;
                      name : IN String;
                      form : IN String := "" )
                             RENAMES Ada.Text_io.open;
  PROCEDURE close   ( file : IN OUT File_type )
                             RENAMES Ada.Text_io.close;

  -- Character I/O:

  PROCEDURE get     ( item : OUT character )
                             RENAMES Ada.Text_io.get;
  PROCEDURE put     ( item : IN character )
                             RENAMES Ada.Text_io.put;
  PROCEDURE get     ( file : IN File_type;
                      item : OUT character )
                             RENAMES Ada.Text_io.get;
  PROCEDURE put     ( file : IN File_type;
                      item : IN character )
                             RENAMES Ada.Text_io.put;

  -- String I/O:

  PROCEDURE get     ( item : OUT String )
                             RENAMES Ada.Text_io.get;
  PROCEDURE get_line( item : OUT String;
                      last : OUT Natural )
                             RENAMES Ada.Text_io.get_line;
  PROCEDURE put     ( item : IN String )
                             RENAMES Ada.Text_io.put;
  PROCEDURE put_line( item : IN String )
                             RENAMES Ada.Text_io.put_line;
  PROCEDURE get     ( file : IN File_type;
                      item : OUT String )
                             RENAMES Ada.Text_io.get;
  PROCEDURE get_line( file : IN File_type;
                      item : OUT String;
                      last : OUT Natural )
                             RENAMES Ada.Text_io.get_line;
  PROCEDURE put     ( file : IN File_type;
                      item : IN String )
                             RENAMES Ada.Text_io.put;
  PROCEDURE put_line( file : IN File_type;
                      item : IN String )
                             RENAMES Ada.Text_io.put_line;

  -- Integer I/O:

  PROCEDURE get     ( item : OUT Integer;
                      width: IN Field := 0 )
                             RENAMES Integer_io.get;
  PROCEDURE put     ( item : IN Integer;
                      width: IN Field := Integer'width;
                      base : IN Number_base := 10 )
                             RENAMES Integer_io.put;
  PROCEDURE get     ( file : IN File_type;
                      item : OUT Integer;
                      width: IN Field := 0 )
                             RENAMES Integer_io.get;
  PROCEDURE put     ( file : IN File_type;
                      item : IN Integer;
                      width: IN Field := Integer'width;
                      base : IN Number_base := 10 )
                             RENAMES Integer_io.put;
  PROCEDURE get     ( from : IN String;
                      item : OUT Integer;
                      last : OUT Positive )
                             RENAMES Integer_io.get;
  PROCEDURE put     ( to   : OUT String;
                      item : IN Integer;
                      base : IN Number_base := 10 )
                             RENAMES Integer_io.put;

  -- Floating point I/O:

  PROCEDURE get     ( item : OUT Float;
                      width: IN Field := 0 )
                             RENAMES Float_io.get;
  PROCEDURE put     ( item : IN Float;
                      fore : IN Field := 2;
                      aft  : IN Field := Float'DIGITS-1;
                      exp  : IN Field := 3 )
                             RENAMES Float_io.put;
  PROCEDURE get     ( from : IN String;
                      item : OUT Float;
                      last : OUT Positive )
                             RENAMES Float_io.get;
  PROCEDURE put     ( to   : OUT String;
                      item : IN Float;
                      aft  : IN Field := Float'DIGITS-1;
                      exp  : IN Field := 3 )
                             RENAMES Float_io.put;
  PROCEDURE get     ( file : IN File_type;
                      item : OUT Float;
                      width: IN Field := 0 )
                             RENAMES Float_io.get;
  PROCEDURE put     ( file : IN File_type;
                      item : IN Float;
                      fore : IN Field := 2;
                      aft  : IN Field := Float'DIGITS-1;
                      exp  : IN Field := 3 )
                             RENAMES Float_io.put;

  -- Cursor control

  FUNCTION  end_of_file RETURN boolean
                         RENAMES Ada.Text_io.end_of_file;
  FUNCTION  end_of_line RETURN boolean
                         RENAMES Ada.Text_io.end_of_line;
  PROCEDURE new_line    (spacing: IN Positive_count := 1)
                         RENAMES Ada.Text_io.new_line;
  PROCEDURE skip_line   (spacing: IN Positive_count := 1)
                         RENAMES Ada.Text_io.skip_line;
  FUNCTION  end_of_file (file: IN File_type) RETURN boolean
                         RENAMES Ada.Text_io.end_of_file;
  FUNCTION  end_of_line (file: IN File_type) RETURN boolean
                         RENAMES Ada.Text_io.end_of_line;
  PROCEDURE new_line    (file: IN File_type;
                         spacing: IN Positive_count := 1)
                         RENAMES Ada.Text_io.new_line;
  PROCEDURE skip_line   (file: IN File_type;
                         spacing: IN Positive_count := 1)
                         RENAMES Ada.Text_io.skip_line;

  -- i/o exceptions

  Status_error:  EXCEPTION RENAMES Ada.Text_io.Status_error;
  Mode_error:    EXCEPTION RENAMES Ada.Text_io.Mode_error;
  Name_error:    EXCEPTION RENAMES Ada.Text_io.Name_error;
  Use_error:     EXCEPTION RENAMES Ada.Text_io.Use_error;
  Device_error:  EXCEPTION RENAMES Ada.Text_io.Device_error;
  End_error:     EXCEPTION RENAMES Ada.Text_io.End_error;
  Data_error:    EXCEPTION RENAMES Ada.Text_io.Data_error;
  Layout_error:  EXCEPTION RENAMES Ada.Text_io.Layout_error;

END Simple_io;
