--[class_account.ads] Specification
PACKAGE Class_account02 IS

  TYPE Account IS PRIVATE;
  SUBTYPE Money  IS Float;
  SUBTYPE PMoney IS Float RANGE 0.0 .. Float'Last;

  PROCEDURE statement( the:IN Account );
  PROCEDURE deposit ( the:IN OUT Account; amount:IN PMoney );
  PROCEDURE withdraw( the:IN OUT Account; amount:IN PMoney;
                       get:OUT PMoney );
  FUNCTION  balance ( the:IN Account ) RETURN Money;

PRIVATE
  TYPE Account IS RECORD
     balance_of : Money := 0.00;      -- Amount in account
  END RECORD;
END Class_account02;


--[class_account.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_account02 IS

  PROCEDURE statement( the:IN Account ) IS
  BEGIN
    put("Mini statement: The amount on deposit is $" );
    put( the.balance_of, aft=>2, exp=>0 );
    new_line(2);
  END statement;

  PROCEDURE deposit ( the:IN OUT Account; amount:IN PMoney ) IS
  BEGIN
    the.balance_of := the.balance_of + amount;
  END deposit;

  PROCEDURE withdraw( the:IN OUT Account; amount:IN PMoney;
                      get:OUT PMoney ) IS
  BEGIN
    IF the.balance_of >= amount THEN
      the.balance_of := the.balance_of - amount;
      get := amount;
    ELSE
      get := 0.00;
    END IF;
  END withdraw;

  FUNCTION  balance( the:IN Account ) RETURN Money IS
  BEGIN
    RETURN the.balance_of;
  END balance;

END Class_account02;

--[class_account_other.ads] Specification
PACKAGE Class_account_other IS

  TYPE Account IS PRIVATE;
  SUBTYPE Money  IS Float;

  PROCEDURE statement( the:IN Account );
PRIVATE
  TYPE Account IS RECORD
     amount : Money := 0.00;      -- Amount in account
  END RECORD;
END Class_account_other;


--[class_account_other.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_account_other IS

  PROCEDURE statement( the:IN Account ) IS
  BEGIN
    put("Other account system "); new_line(2);
  END statement;

END Class_account_other;


--[main1.adb] Procedure
WITH Ada.Text_io, Class_account02;
USE  Ada.Text_io, Class_account02;
PROCEDURE main011 IS
  my_account: Account;
  obtain    : Money;
BEGIN
  statement( my_account );

  put("Deposit $100.00 into account"); new_line;  -- Deposit
  deposit( my_account, 100.00 );
  statement( my_account );

  put("Withdraw $80.00 from account"); new_line;  -- Withdraw
  withdraw( my_account, 80.00, obtain );
  statement( my_account );

  put("Deposit $200.00 into account"); new_line;  -- Deposit
  deposit( my_account, 200.00 );
  statement( my_account );

END main011;


--[main2.adb] Procedure
WITH Ada.Text_io, Class_account02;
PROCEDURE main012 IS
  my_account: Class_account02.Account;
  obtain    : Class_account02.Money;
BEGIN
  Class_account02.statement( my_account );

  Ada.Text_io.put("Deposit $100.00 into account");
  Ada.Text_io.new_line;
  Class_account02.deposit( my_account, 100.00 );
  Class_account02.statement( my_account );

  Ada.Text_io.put("Withdraw $80.00 from account");
  Ada.Text_io.new_line;
  Class_account02.withdraw( my_account, 80.00, obtain );
  Class_account02.statement( my_account );

  Ada.Text_io.put("Deposit $200.00 into account");
  Ada.Text_io.new_line;
  Class_account02.deposit( my_account, 200.00 );
  Class_account02.statement( my_account );

END main012;


--[main3.adb] Procedure
WITH Class_account02, Class_account_other;
USE  Class_account02, Class_account_other;
PROCEDURE main013 IS
  my_account    :Class_account02.Account;
  other_account :Class_account_other.Account;
BEGIN
  statement( my_account );    -- statement in Class_account
  statement( other_account ); -- statement in Class_account_other
END main013;

--[main4.adb] Procedure
WITH Class_account02;
USE  Class_account02;
PROCEDURE main014 IS
  my_account   : Account;
  other_account: Account;
  obtain       : PMoney;
BEGIN
  deposit( my_account, 100.00 );
  other_account := my_account;            -- Copy and
  withdraw(other_account, 100.00, obtain);-- Withdraw 100.00

  other_account := my_account;            -- Copy again and
  withdraw(other_account, 100.00, obtain);-- Withdraw 100.00

END main014;

--[main.adb] Procedure
WITH Ada.Text_io, main011, main012, main013, main014;
USE  Ada.Text_io;
PROCEDURE main010 IS
BEGIN
  put("Example Account 1 "); new_line; main011;
  put("Example Account 2 "); new_line; main012;
  put("Example Account 3 "); new_line; main013;
  put("Example Account 4 "); new_line; main014;
END main010;
