--[class_account.ads] Specification
PACKAGE Class_account03 IS
  TYPE Account IS PRIVATE;
  SUBTYPE Money  IS Float;
  SUBTYPE PMoney IS Float RANGE 0.0 .. Float'Last;

  PROCEDURE statement( the:IN Account );
  PROCEDURE deposit( the:IN OUT Account; amount:IN PMoney );
  PROCEDURE withdraw( the:IN OUT Account;
                      amount:IN PMoney; get:OUT PMoney );
  FUNCTION  balance( the:IN Account ) RETURN Money;
PRIVATE
  TYPE Account IS RECORD
     balance_of : Money := 0.00;           -- Amount on deposit
  END RECORD;
END Class_account03;


--[class_account.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_account03 IS

  PROCEDURE statement( the:IN Account ) IS
  BEGIN
    put("Mini statement: The amount on deposit is $" );
    put( the.balance_of, aft=>2, exp=>0 );
    new_line(2);
  END statement;

  PROCEDURE deposit( the:IN OUT Account; amount:IN PMoney ) IS
  BEGIN
    the.balance_of := the.balance_of + amount;
  END deposit;

  PROCEDURE withdraw( the:IN OUT Account;
                      amount:IN PMoney; get:OUT PMoney ) IS
  BEGIN
    IF the.balance_of >= amount THEN
      the.balance_of := the.balance_of - amount;
      get := amount;
    ELSE
      get := 0.00;
    END IF;
  END withdraw;

  FUNCTION  balance( the:IN Account ) RETURN Money IS
  BEGIN
    RETURN the.balance_of;
  END balance;

END Class_account03;


--[class_tui.ads] Specification
PACKAGE Class_tui IS

  TYPE Menu_item IS ( M_1, M_2, M_3, M_4, M_QUIT );
  TYPE TUI IS PRIVATE;

  PROCEDURE menu( the:IN OUT TUI; m1,m2,m3,m4:IN String );
  FUNCTION  event( the:IN TUI ) RETURN Menu_item;
  PROCEDURE message( the:IN TUI; mes:IN String );
  PROCEDURE dialog(the:IN TUI; mes:IN String; res:OUT Float);

PRIVATE
  TYPE TUI IS RECORD
     selection : Menu_item := M_QUIT;
  END RECORD;
END Class_tui;

--[class_tui.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_tui IS
  PROCEDURE menu( the:IN OUT TUI; m1,m2,m3,m4:IN String ) IS

  selection      : Character;
  valid_response : Boolean := FALSE;

  PROCEDURE set_response(choice:IN Menu_item; mes:IN String) IS
  BEGIN
    IF mes /= "" THEN               -- Allowable choice
      the.selection := choice; valid_response := TRUE;
    END IF;
  END set_response;

  PROCEDURE display_menu_item(prompt, name:IN String) IS
  BEGIN
      IF name/="" THEN put(prompt & name); new_line(2); END IF;
  END display_menu_item;

  BEGIN
    WHILE NOT valid_response LOOP
      display_menu_item( "[a]  ", m1 );
      display_menu_item( "[b]  ", m2 );
      display_menu_item( "[c]  ", m3 );
      display_menu_item( "[d]  ", m4 );
      put( "Input selection: "); get( selection ); skip_line;
      CASE selection IS
        WHEN 'a' | 'A' => set_response( M_1, m1 );
        WHEN 'b' | 'B' => set_response( M_2, m2 );
        WHEN 'c' | 'C' => set_response( M_3, m3 );
        WHEN 'd' | 'D' => set_response( M_4, m4 );
        WHEN 'e' | 'E' => set_response( M_QUIT, "Quit" );
        WHEN OTHERS    => valid_response := FALSE;
      END CASE;
      IF NOT valid_response THEN
        message( the, "Invalid response" );
      END IF;
    END LOOP;
  END menu;

  FUNCTION  event( the:IN TUI ) RETURN Menu_item IS
  BEGIN
    return the.selection;
  END;

  PROCEDURE message( the:IN TUI; mes:IN String ) IS
  BEGIN
    new_line; put( mes ); new_line;
  END message;

  PROCEDURE dialog(the:IN TUI; mes:IN String; res:OUT Float) IS
  BEGIN
    new_line(1); put( mes & " : " );
    get( res ); skip_line;
  END dialog;

END Class_tui;

-----------------------------------------------------------------

--[class_container.ads] Specification
PACKAGE Class_container020 IS
  PROCEDURE main1;
  PROCEDURE main2;
END Class_container020;

--[class_container.adb] Implementation
WITH Simple_io, Class_account03, Class_tui;
USE  Simple_io, Class_account03, Class_tui;
PACKAGE BODY Class_container020 IS

PROCEDURE main1 IS
  miles  : Float;
  screen : TUI;
  FUNCTION float_image( f:IN Float ) RETURN String IS
    res : String( 1 .. 10 );        -- String of 10 characters
  BEGIN
    put( res, f, aft=>2, exp=>0 );  -- 2 digits - NO exp
    RETURN res;
  END float_image;
BEGIN
  message( screen, "Distance converter" );
  dialog ( screen, "Enter distance in miles", miles );
  message( screen, "Distance in kilometers is " &
                    Float'Image( miles * 1.6093 )  );
  message( screen, "Distance in kilometers is " &
                    Float_image( miles * 1.6093 )  );
END main1;

PROCEDURE main2 IS
  user     : Account;            -- The users account
  screen   : TUI;                -- The display screen
  cash     : Money;              --
  received : Money;              --

  FUNCTION float_image( f:IN Float ) RETURN String IS
    res : String( 1 .. 10 );     -- String of 10 characters
  BEGIN
    put( res, f, 2, 0 );         -- 2 digits - NO exp
    RETURN res;
  END float_image;

BEGIN
  LOOP
    menu( screen, "Deposit", "Withdraw", "Balance", "" );
    CASE event( screen ) IS
      WHEN M_1 =>                                    -- Deposit
        dialog( screen, "Amount to deposit", cash );
        IF cash <= 0.0 THEN
          message( screen, "Must be >= 0.00" );
        ELSE
          deposit( user, cash );
        END IF;
      WHEN M_2 =>                                     -- Withdraw
        dialog( screen, "Amount to withdraw", cash );
        IF cash <= 0.0 THEN
          message( screen, "Must be >= 0.00" );
        ELSE
          withdraw( user, cash, received );
          IF received <= 0.0 THEN
            message( screen, "Not enough money" );
          END IF;
        END IF;
      WHEN M_3 =>                                    -- Balance
         message( screen, "Balance is " &
                            float_image( balance(user)) );
      WHEN M_QUIT =>                                 -- Exit
         RETURN;
      WHEN OTHERS =>                                 -- Not used
          message( screen, "Program error");         -- oops
    END CASE;
  END LOOP;
END main2;

END Class_container020;

--[main.adb] Procedure
WITH Text_io, Class_container020; 
USE  Text_io, Class_container020;
PROCEDURE main020 IS
BEGIN
  put("Example 1"); new_line; main1;
  put("Example 2"); new_line; main2;
END main020;
