--[class_sort.ads] Specification
GENERIC
  TYPE T         IS PRIVATE;          -- Any non limited type
  TYPE Vec_range IS (<>);             -- Any discrete type
  TYPE Vec       IS ARRAY( Vec_range ) OF T;
  WITH FUNCTION  ">"( first, second:IN T ) RETURN Boolean IS <>;
PROCEDURE sort( items:IN OUT Vec );

--[class_sort.adb] Implementation
PROCEDURE sort( items:IN OUT Vec ) IS
  swaps : Boolean := TRUE;
  tmp   : T;
BEGIN
  WHILE swaps LOOP
    swaps := false;
    FOR i IN items'First .. Vec_range'Pred(items'Last) LOOP
      IF items( i ) > items( Vec_range'Succ(i) ) THEN
        swaps := TRUE;
        tmp := items( Vec_range'Succ(i) );
        items( Vec_range'Succ(i) ) := items( i );
        items( i ) := tmp;
      END IF;
    END LOOP;
  END LOOP;
END sort;

--[main.adb] Procedure
WITH Ada.Text_io, sort;
USE  Ada.Text_io;
PROCEDURE main141 IS

  TYPE Chs_range IS RANGE 1 .. 6;
  TYPE Chs       IS ARRAY( Chs_range ) OF Character; 

  PROCEDURE sort_chs IS NEW sort (
    T         => Character,
    Vec_range => Chs_range,
    Vec       => Chs,
    ">"       => ">" );
  some_characters : Chs := ( 'q', 'w', 'e', 'r', 't', 'y' );
BEGIN
  sort_chs( some_characters );
  FOR i IN Chs_range LOOP
    put( some_characters( i ) ); put( " " );
  END LOOP;
  new_line;
END main141;


--[main2.adb] Procedure
WITH Simple_io, sort;
USE  Simple_io;
PROCEDURE main142 IS
  TYPE Chs_range IS ( RED, BLUE, GREEN );
  TYPE Chs IS ARRAY( Chs_range ) OF Character; 

  PROCEDURE sort_chs IS NEW sort (
    T         => Character,
    Vec_range => Chs_range,
    Vec       => Chs,
    ">"       => ">" );
  some_items : Chs := ( 'q', 'w', 'e' );
BEGIN
  sort_chs( some_items );
  FOR i IN Chs_range LOOP
    put( some_items( i ) ); put( " " );
  END LOOP;
  new_line;
END main142;


--[main3.adb] Procedure
WITH Simple_io, sort;
USE  Simple_io;
PROCEDURE main143 IS
  MAX_CHS : CONSTANT := 7;
  TYPE Height_cm IS RANGE 0 .. 300;
  TYPE Person IS RECORD
    name   : String( 1 .. MAX_CHS );  -- Name as a String
    height : Height_cm := 0;          -- Height in cm.
  END RECORD;
  TYPE People_range IS (FIRST, SECOND, THIRD, FORTH );
  TYPE People       IS ARRAY( People_range ) OF Person; 

  FUNCTION cmp_height(first, second:IN Person) RETURN Boolean IS
  BEGIN
    RETURN first.height > second.height;
  END cmp_height;

  FUNCTION cmp_name( first, second:IN Person ) RETURN Boolean IS
  BEGIN
    RETURN first.name > second.name;
  END cmp_name;

  PROCEDURE sort_people_height IS NEW sort (
    T         => Person,
    Vec_range => People_range,
    Vec       => People,
    ">"       => cmp_height );

  PROCEDURE sort_people_name IS NEW sort (
    T         => Person,
    Vec_range => People_range,
    Vec       => People,
    ">"       => cmp_name );

  friends : People := ( ("Paul   ", 146 ), ("Carol  ", 147 ), 
                        ("Mike   ", 183 ), ("Corinna", 171 ) );
BEGIN
  sort_people_name( friends );                    -- Name order
  put( "The first in ascending name order is   " );
  put( friends( FIRST ).name ); new_line;
  sort_people_height( friends );                  -- Height order
  put( "The first in ascending height order is " );
  put( friends( FIRST ).name ); new_line;
END main143;

--[main4.adb] Procedure
WITH Simple_io, sort;
USE  Simple_io;
PROCEDURE main144 IS
BEGIN
  NULL;
END main144;

--[main.adb] Procedure
WITH Simple_io, main141, main142, main143, main144;
USE  Simple_io;
PROCEDURE main145 IS
BEGIN
  put("Example 1 "); new_line; main141;
  put("Example 2 "); new_line; main142;
  put("Example 3 "); new_line; main143;
  put("Example 4 "); new_line; main144;
END main145;
