--[class_board.ads] Specification
WITH Simple_io; USE Simple_io;
PACKAGE Class_board IS
  TYPE Board IS PRIVATE;
  PROCEDURE add( the:IN OUT Board; pos:IN Integer; 
                 piece:IN Character );
  FUNCTION  valid( the:IN Board; pos:IN Integer ) RETURN Boolean;
  PROCEDURE display( the:IN Board );
PRIVATE
  SIZE_TTT : CONSTANT := 9;                         -- Must be 9
  SUBTYPE Board_index IS Integer RANGE 1 .. SIZE_TTT;
  SUBTYPE Board_range IS Board_index;
  TYPE    Board_grid  IS ARRAY( Board_range ) OF Character;
  TYPE Board IS RECORD
    sqrs : Board_grid := ( OTHERS => ' ');   -- Initialize
  END RECORD;
END Class_board;

--[class_board.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_board IS

  PROCEDURE add( the:IN OUT Board; pos:IN Integer;
                 piece:IN Character ) IS
  BEGIN
    the.sqrs( pos ) := piece;
  END add;

  FUNCTION valid(the:IN Board; pos:IN Integer) RETURN Boolean IS
  BEGIN
    RETURN pos IN Board_range AND THEN the.sqrs( pos ) = ' ';
  END valid;

  PROCEDURE display( the:IN Board ) IS
  BEGIN
    FOR i IN Board_range LOOP          -- All pieces
      put( the.sqrs( i ) );            -- Display counter;
      CASE i IS                        -- after printing counter
        WHEN 3 | 6  =>                 --  print Row Separator
          new_line; put("---------");
          new_line;
        WHEN 9      =>                 --  print new line
          new_line;
        WHEN 1 | 2 | 4 | 5 | 7 | 8 =>  --  print Col separator
          put(" | ");
      END CASE;
    END LOOP;
  END display;
END Class_board;

-- ADA 9X
-- TYPE Player_range IS MOD 2;
-- player := player + 1                    -- Next player

--[play.adb] Procedure
WITH Simple_io, Class_board;
USE  Simple_io, Class_board;
PROCEDURE main100 IS
  player : Character;          -- Either 'X' or 'O'
  game   : Board;              -- An instance of Class Board
  move   : Integer;            -- Move from user
BEGIN
  player := 'X';                             -- Set player
  put("Player "); put( player );             --  Ask player for
  put(" enter move (1-9) : ");               --   move
  WHILE NOT end_of_file LOOP                 -- While moves
    get( move ); skip_line;                  --  Get move
    IF valid( game, move ) THEN              -- Valid
      add( game, move, player );             --  Add to board
      display( game );                       --  Display board
      CASE player IS                         --  Next Player
        WHEN 'X'      => player := 'O';      --   'X' -> 'O'
        WHEN 'O'      => player := 'X';      --   'O' -> 'X'
        WHEN OTHERS   => NULL;               --
      END CASE;
    ELSE
      put("Move invalid"); new_line;         -- for board
    END IF;
    put("Player "); put( player );           --  Ask player for
    put(" enter move (1-9) : ");             --   move
  END LOOP;
  new_line(2);
END main100;
