--[class_string.ads] Specification
WITH Simple_io; USE Simple_io;
PACKAGE Class_bounded_string IS
  TYPE Bounded_string IS PRIVATE;

  FUNCTION to_bounded_string(str:IN String)
    RETURN Bounded_string;

  FUNCTION to_string(the:IN Bounded_string) RETURN String;

  FUNCTION "&" (f:IN Bounded_string; s:IN Bounded_string)
    RETURN Bounded_string;
  FUNCTION "&" (f:IN Bounded_string; s:IN String)
    RETURN Bounded_string;
  FUNCTION "&" (f:IN String; s:IN Bounded_string)
    RETURN Bounded_string;

  FUNCTION slice( the:IN Bounded_string;
                  low:IN Positive; high:IN Natural )
    RETURN String;

  FUNCTION "="  ( f:IN Bounded_string; s:IN Bounded_string )
    RETURN Boolean;

  FUNCTION ">"  ( f:IN Bounded_string; s:IN Bounded_string )
    RETURN Boolean;
  FUNCTION ">=" ( f:IN Bounded_string; s:IN Bounded_string )
    RETURN Boolean;
  FUNCTION "<"  ( f:IN Bounded_string; s:IN Bounded_string )
    RETURN Boolean;
  FUNCTION "<=" ( f:IN Bounded_string; s:IN Bounded_string )
    RETURN Boolean;
PRIVATE
  MAX_STRING: CONSTANT := 80;
  SUBTYPE Str_range IS Natural RANGE 0 .. MAX_STRING;
  TYPE A_Bounded_string( length: Str_range := 0 ) IS RECORD
    chrs: String( 1 .. length );  -- Stored string
  END RECORD;
  TYPE Bounded_string IS RECORD
    v_str : A_Bounded_string;
  END RECORD;
END Class_bounded_string;

--[class_string.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_bounded_string IS

  FUNCTION to_bounded_string( str:IN String )
    RETURN Bounded_string IS
  BEGIN
    RETURN (v_str=>(str'Length, str));
  END to_bounded_string;

  FUNCTION to_string(the:IN Bounded_string) RETURN String IS
  BEGIN
    RETURN the.v_str.chrs( 1 .. the.v_str.length );
  END to_string;

  FUNCTION "&" ( f:IN Bounded_string; s:IN Bounded_string )
    RETURN Bounded_string IS
  BEGIN
    RETURN (v_str=>(f.v_str.chrs'Length + s.v_str.chrs'Length,
                    f.v_str.chrs & s.v_str.chrs));
  END "&";

  FUNCTION "&" ( f:IN Bounded_string; s:IN String )
    RETURN Bounded_string IS
  BEGIN
    RETURN (v_str=>(f.v_str.chrs'Length + s'Length,
                    f.v_str.chrs & s ) );
  END "&";

  FUNCTION "&" ( f:IN String; s:IN Bounded_string )
    RETURN Bounded_string IS
  BEGIN
    RETURN ( v_str=>(f'Length + s.v_str.chrs'Length,
                     f & s.v_str.chrs ) );
  END "&";

  FUNCTION slice( the:IN Bounded_string;
                  low:IN Positive; high:IN Natural)
    RETURN String IS
  BEGIN
    IF low <= high AND THEN high <= the.v_str.length THEN
        RETURN the.v_str.chrs( low .. high );
     END IF;
     RETURN "";
  END slice;

  FUNCTION "="  ( f:IN Bounded_string; s:IN Bounded_string )
    RETURN Boolean IS
  BEGIN
    RETURN f.v_str.chrs = s.v_str.chrs;
  END "=";

  FUNCTION ">"  ( f:IN Bounded_string; s:IN Bounded_string )
    RETURN Boolean IS
  BEGIN
    RETURN f.v_str.chrs > s.v_str.chrs;
  END ">";

  FUNCTION ">=" ( f:IN Bounded_string; s:IN Bounded_string )
    RETURN Boolean IS
  BEGIN
    RETURN f.v_str.chrs >= s.v_str.chrs;
  END ">=";

  FUNCTION "<"  ( f:IN Bounded_string; s:IN Bounded_string )
    RETURN Boolean IS
  BEGIN
    RETURN f.v_str.chrs < s.v_str.chrs;
  END "<";

  FUNCTION "<=" ( f:IN Bounded_string; s:IN Bounded_string )
    RETURN Boolean IS
  BEGIN
    RETURN f.v_str.chrs <= s.v_str.chrs;
  END "<=";

END Class_bounded_string;

--[class_container.ads] Specification
PACKAGE Class_container130 IS
  PROCEDURE main1;
  PROCEDURE main2;
END Class_container130;

--[class_container.adb] Implementation
WITH Simple_io, Class_bounded_string;
USE  Simple_io, Class_bounded_string;
PACKAGE BODY Class_container130 IS

PROCEDURE main1 IS
  town, county, address : Bounded_string;
BEGIN
  town   := to_bounded_string( "Brighton" );
  county := to_bounded_string( "East Sussex" );

  address := town & " " & county;

  put( to_string(address) ); new_line;
  put( slice( county & " UK", 6, 14 ) );
  new_line;

END main1;

PROCEDURE main2 IS
  text: Bounded_string;
  PROCEDURE check(b:IN Boolean; str1,str2:IN String) IS
  BEGIN
    IF b THEN put(str1); ELSE put(str2); END IF;
    put(" ");
  END check;
BEGIN
  put("=    /=   >    >=   >=   <    <=   <=    "); new_line;
  check(to_bounded_string("ABC") =  to_bounded_string("ABC") , "Pass",  "Fail");
  check(to_bounded_string("ABC") /= to_bounded_string("AB ") , "Pass",  "Fail");
  check(to_bounded_string("ABC") >  to_bounded_string("ABBB"), "Pass",  "Fail");
  check(to_bounded_string("ABC") >= to_bounded_string("ABBB"), "Pass",  "Fail");
  check(to_bounded_string("ABC") >= to_bounded_string("ABC") , "Pass",  "Fail");
  check(to_bounded_string("ABBB")<  to_bounded_string("ABC") , "Pass",  "Fail");
  check(to_bounded_string("ABBB")<= to_bounded_string("ABC") , "Pass",  "Fail");
  check(to_bounded_string("ABC") <= to_bounded_string("ABC") , "Pass",  "Fail");
  new_line;

  put("/=    =   <=   <    <    >=   >    >     "); new_line;
  check(to_bounded_string("ABC") /= to_bounded_string("ABC") , "Fail",  "Pass");
  check(to_bounded_string("ABC") =  to_bounded_string("AB ") , "Fail",  "Pass");
  check(to_bounded_string("ABC") <= to_bounded_string("ABBB"), "Fail",  "Pass");
  check(to_bounded_string("ABC") <  to_bounded_string("ABBB"), "Fail",  "Pass");
  check(to_bounded_string("ABC") <  to_bounded_string("ABC") , "Fail",  "Pass");
  check(to_bounded_string("ABBB")>= to_bounded_string("ABC") , "Fail",  "Pass");
  check(to_bounded_string("ABBB")>  to_bounded_string("ABC") , "Fail",  "Pass");
  check(to_bounded_string("ABC") >  to_bounded_string("ABC") , "Fail",  "Pass");
  new_line;

  put("Equality test"); new_line;
  FOR i IN 1 .. 6 LOOP
    text := to_bounded_string("ABCDEFGH-ABCDEFGH");
    IF slice(text,i,4) = slice(text,9+i,13) THEN
      put("Pass");
    ELSE
      put("Fail");
    END IF;
    put(" ");
  END LOOP;
  new_line;

END main2;

END Class_container130;

WITH Ada.Text_io, Class_bounded_string;
USE  TYPE Class_bounded_string.Bounded_string;
PROCEDURE main133 IS
  town  : Class_bounded_string.Bounded_string := 
    Class_bounded_string.to_bounded_string("Brighton");
  county: Class_bounded_string.Bounded_string := 
    Class_bounded_string.to_bounded_string("E Sussex");
BEGIN
  Ada.Text_io.put( 
       Class_bounded_string.to_string( town & " " & county ) 
  );
END main133;

--[main.adb] Procedure
WITH Class_container130, main133; 
USE  Class_container130;
PROCEDURE main130 IS
BEGIN
  main1; main2; main133;
END main130;
