-- In Ada 9X the Iterator should be a child of the class list

-- list  -> ( first_node, last_node ) ( prev,item,next)
-- Iter  -> ( cur_list_first, cur_list_last, cur_node )

--[class_list.ads] Specification
WITH Ada.Finalization, Unchecked_deallocation;
USE  Ada.Finalization;
GENERIC
  TYPE T IS PRIVATE;                  -- Any type
PACKAGE Class_list5 IS
  TYPE List IS NEW Controlled WITH PRIVATE;

  PROCEDURE initialize( the:IN OUT List );
  PROCEDURE initialize( the:IN OUT List; data:IN T );
  PROCEDURE finalize( the:IN OUT List );
  PROCEDURE adjust( the:IN OUT List );
  FUNCTION "="  ( f:IN List; s:IN List ) RETURN Boolean;
  PROCEDURE xyz(the:IN OUT List ); -- DEBUG TRACE
PRIVATE
  PROCEDURE release_storage( the:IN OUT List );

  TYPE Node;                       -- Tentative declaration
  TYPE P_Node IS ACCESS ALL Node;  -- Pointer to Node

  TYPE Node IS RECORD
    prev    : P_Node;              -- Previous Node
    item    : T;                   -- The physical item
    next    : P_Node;              -- Next Node
  END RECORD;

  PROCEDURE dispose_node IS
    NEW unchecked_deallocation( Node, P_Node );
  TYPE List IS NEW Controlled WITH RECORD
    first_node : ALIASED P_Node := NULL;   -- First item in list
    last_node  : ALIASED P_Node := NULL;   -- First item in list
  END RECORD;

END Class_list5;

--[class_list.adb] Implementation
--WITH Simple_io; USE  Simple_io;
PACKAGE BODY Class_list5 IS

  PROCEDURE initialize( the:IN OUT List ) IS
  BEGIN
--  put("Initialization C[List]"); new_line; -- DEBUG TRACE
    the.first_node := NULL;   -- Empty list
    the.last_node  := NULL;   -- Empty list
    xyz( the ); -- DEBUG TRACE
  END initialize;

  PROCEDURE initialize( the:IN OUT List; data:IN T ) IS
  BEGIN
--  put("Initialization [List,data]"); new_line; -- DEBUG TRACE
    the.first_node := NEW Node'(NULL, data, NULL);
    the.last_node  := the.first_node;
    xyz( the ); -- DEBUG TRACE
  END initialize;

  PROCEDURE finalize( the:IN OUT List ) IS
  BEGIN
--  put("Finalization C[List]"); new_line; -- DEBUG TRACE
    IF the.first_node /= NULL THEN
      release_storage( the );
      the.first_node := NULL;
    END IF;
    --xyz( the ); -- Can not do this here DEBUG TRACE
  END finalize;

  PROCEDURE adjust( the:IN OUT List ) IS
    cur : P_Node := the.first_node;  -- Original list
    lst : P_Node := NULL;            -- Last created node
    prv : P_Node := NULL;            -- Previously created node
    fst : P_Node := NULL;            -- The first node

  BEGIN
--  put("Adjust list "); new_line; -- DEBUG TRACE
    WHILE cur /= NULL LOOP
      lst := NEW Node'( prv, cur.item, NULL );
      IF fst =  NULL THEN fst := lst; END IF;
      IF prv /= NULL THEN prv.next := lst; END IF;
      prv := lst;
      cur := cur.next;               -- Next node
    END LOOP;
    the.first_node := fst;           -- Update
    the.last_node  := lst;
    xyz( the ); -- DEBUG TRACE
  END adjust;

  FUNCTION "="  ( f:IN List; s:IN List ) RETURN Boolean IS
    f_node : P_Node := f.first_node;  -- First list
    s_node : P_Node := s.first_node;  -- Second list
  BEGIN
    WHILE f_node /= NULL AND s_node /= NULL LOOP
      IF f_node.item /= s_node.item THEN
        RETURN FALSE;                 -- Different items
      END IF;
      f_node := f_node.next; s_node := s_node.next;
    END LOOP;
    RETURN f_node = s_node;           -- Both NULL if equal
  END "=";

  PROCEDURE release_storage( the:IN OUT List ) IS
    cur : P_Node := the.first_node; -- Pointer to curr node
    tmp : P_Node;                   -- Node to dispose
  BEGIN
--  put("Release storage "); new_line; -- DEBUG TRACE
    WHILE cur /= NULL LOOP          -- For each item in list
      tmp := cur;                   -- Item to dispose
      cur := cur.next;              -- Next node
--    put("Dispose node");new_line; -- DEBUG TRACE;
      dispose_node( tmp );          -- Dispose of item
    END LOOP;
  END release_storage;

  PROCEDURE xyz(the:IN OUT List ) IS -- DEBUG TRACE
    cur : P_Node := the.first_node;  -- DEBUG TRACE
  BEGIN                              -- DEBUG TRACE
    WHILE cur /= NULL LOOP           -- DEBUG TRACE
      cur := cur.next;               -- DEBUG TRACE
    END LOOP;                        -- DEBUG TRACE
    cur := the.last_node;            -- DEBUG TRACE
    WHILE cur /= NULL LOOP           -- DEBUG TRACE
      cur := cur.prev;               -- DEBUG TRACE
    END LOOP;                        -- DEBUG TRACE
  END xyz;                           -- DEBUG TRACE
                                     -- DEBUG TRACE
END Class_list5;

--[class_list-iterator.ads] Specification
--WITH Ada.Finalization; USE  Ada.Finalization;
GENERIC
PACKAGE Class_list5.Iterator IS

  TYPE List_iter IS LIMITED PRIVATE;

  PROCEDURE initialize( the:IN OUT List_iter );
  PROCEDURE finalize( the:IN OUT List_iter );
  PROCEDURE first( the:IN OUT List_iter; l:IN OUT List );
  PROCEDURE last( the:IN OUT List_iter; l:IN OUT List );

  FUNCTION  deliver( the:IN List_iter) RETURN T;
  PROCEDURE insert( the:IN OUT List_iter; data:IN T );
  PROCEDURE delete( the:IN OUT List_iter );
  FUNCTION  is_end( the:IN List_iter ) RETURN Boolean;
  PROCEDURE next( the:IN OUT List_iter );
  PROCEDURE prev( the:IN OUT List_iter );
  PROCEDURE xyz( the:IN OUT List_iter ); -- DEBUG TRACE
PRIVATE
  TYPE P_P_Node IS ACCESS ALL P_Node;
  TYPE List_iter IS RECORD
    cur_list_first: P_P_Node := NULL;    -- First in chain
    cur_list_last : P_P_Node := NULL;    -- Last in chain
    cur_node      : P_Node   := NULL;    -- Current item
  END RECORD;
END Class_list5.Iterator;

--[class_list-iterator.adb] Implementation
--WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_list5.Iterator IS

  PROCEDURE xyz( the:IN OUT List_iter ) IS  -- DEBUG TRACE
    cur : P_Node := the.cur_list_first.ALL; -- DEBUG TRACE
  BEGIN                                     -- DEBUG TRACE
    WHILE cur /= NULL LOOP                  -- DEBUG TRACE
      cur := cur.next;                      -- DEBUG TRACE
    END LOOP;                               -- DEBUG TRACE
    cur := the.cur_list_last.ALL;           -- DEBUG TRACE
    WHILE cur /= NULL LOOP                  -- DEBUG TRACE
      cur := cur.prev;                      -- DEBUG TRACE
    END LOOP;                               -- DEBUG TRACE
  END xyz;                                  -- DEBUG TRACE
                                            -- DEBUG TRACE
  PROCEDURE initialize( the:IN OUT List_iter ) IS
  BEGIN
--  put("Initialize C[Iter]"); new_line;    -- DEBUG TRACE
    the.cur_node       := NULL;   -- Iterator not setup
    the.cur_list_first := NULL;
    the.cur_list_last  := NULL;
    xyz( the );                             -- DEBUG TRACE
  END initialize;

  PROCEDURE finalize( the:IN OUT List_iter ) IS
  BEGIN
--  put("Finalize C[Iter]"); new_line;      -- DEBUG TRACE
    xyz( the );                             -- DEBUG TRACE
    NULL;
  END finalize;

  PROCEDURE first( the:IN OUT List_iter; l:IN OUT List ) IS
  BEGIN
--  put("First C[Iter]"); new_line;         -- DEBUG TRACE
    the.cur_node      := l.first_node;       -- Set to first
    the.cur_list_first:= l.first_node'Unchecked_Access;
    the.cur_list_last := l.last_node'Unchecked_Access;
    xyz( the ); -- DEBUG TRACE
  END first;

  PROCEDURE last( the:IN OUT List_iter; l:IN OUT List ) IS
  BEGIN
--  put("Last C[Iter]"); new_line;           -- DEBUG TRACE
    the.cur_node      := l.last_node;        -- Set to last
    the.cur_list_first:= l.first_node'Unchecked_Access;
    the.cur_list_last := l.last_node'Unchecked_Access;
    xyz( the ); -- DEBUG TRACE
  END last;

  FUNCTION deliver( the:IN List_iter ) RETURN T IS
  BEGIN
--  put("deliver [iter]"); new_line;          -- DEBUG TRACE
    RETURN the.cur_node.item;  -- The current item
  END deliver;

  PROCEDURE insert( the:IN OUT List_iter; data:IN T ) IS
    tmp   : P_Node;
    cur   : P_Node   := the.cur_node;   -- Current element
    first : P_P_Node := the.cur_list_first;
    last  : P_P_Node := the.cur_list_last;
  BEGIN
--  put("insert [iter]"); new_line;           -- DEBUG TRACE
    IF cur = NULL THEN            -- Empty or last item
      IF first.ALL = NULL THEN    --  Empty list
        tmp := NEW Node'( NULL, data, NULL );
        first.ALL := tmp;
        last.ALL  := tmp;
        the.cur_node := tmp;
      ELSE                        --  Last
        tmp := NEW Node'( last.ALL, data, NULL );
        last.ALL.next := tmp;
        last.ALL      := tmp;
        the.cur_node := tmp;
       END IF;
    ELSE
      tmp := NEW Node'( cur.prev, data, cur );
      IF cur.prev = NULL THEN      -- First item
        first.ALL := tmp;
      ELSE
        cur.prev.next := tmp;
      END IF;
      cur.prev := tmp;
    END IF;
    xyz( the );                            -- DEBUG TRACE
  END insert;

  PROCEDURE delete( the:IN OUT List_iter) IS
    cur   : P_Node   := the.cur_node;   -- Current element
    first : P_P_Node := the.cur_list_first;
    last  : P_P_Node := the.cur_list_last;
  BEGIN
--  put("delete [iter]"); new_line; -- DEBUG TRACE
    IF cur /= NULL THEN             -- Something to delete
      IF cur.prev /= NULL THEN      -- Fix forward pointer;
        cur.prev.next := cur.next;  --  Not first in chain
      ELSE
        first.ALL := cur.next;      --  First in chain
        IF first.ALL = NULL THEN
          last.ALL := NULL;         --   Empty list
        END IF;
      END IF;
      IF cur.next /= NULL THEN      -- Fix backward pointer;
        cur.next.prev := cur.prev;  --  Not last in chain
      ELSE
        last.ALL := cur.prev;       --  Last in chain
        IF last.ALL = NULL THEN
          first.ALL := NULL;        --   Empty list
        END IF;
      END IF;
      IF cur.next /= NULL THEN      -- Fix current pointer
        the.cur_node := cur.next;   --  next
      ELSIF cur.prev /= NULL THEN
        the.cur_node := cur.prev;   --  previous
      ELSE
        the.cur_node := NULL;       --  none empty list
      END IF;
      dispose_node( cur );          -- Release storage
      xyz( the );                   -- DEBUG TRACE
    END IF;
  END delete;

  FUNCTION  is_end( the:IN List_iter ) RETURN Boolean IS
  BEGIN
--  put("is end [iter]"); new_line; -- DEBUG TRACE
    RETURN the.cur_node = NULL;               -- True if end
  END is_end;

  PROCEDURE next( the:IN OUT List_iter ) IS
  BEGIN
--  put("next [iter]"); new_line; -- DEBUG TRACE
    IF the.cur_node /= NULL THEN               --
      the.cur_node  := the.cur_node.next;      -- Next
    END IF;
    xyz( the ); -- DEBUG TRACE
  END next;

  PROCEDURE prev( the:IN OUT List_iter ) IS
  BEGIN
--  put("prev [iter]"); new_line; -- DEBUG TRACE
    IF the.cur_node /= NULL THEN               --
      the.cur_node  := the.cur_node.prev;      -- Previous
    END IF;
    xyz( the ); -- DEBUG TRACE
  END prev;

END Class_list5.Iterator;

--[class_list_nat.ADS] Implementation Instantiation
WITH Class_list5;
PRAGMA ELABORATE_ALL( Class_list5 );
  PACKAGE Class_list_nat IS NEW Class_list5(Natural);

--[class_list-Nat_Iterator.ADS] Implementation Instantiation
WITH Class_list_nat, Class_list5.Iterator;
  PRAGMA ELABORATE_ALL( Class_List_nat, Class_list5.Iterator );
  PACKAGE Class_list_nat_iterator IS NEW Class_list_nat.Iterator;


--[pack_procedures.ads] Specification
WITH Simple_io, Class_list_nat, Class_list_nat_iterator;
USE  Simple_io, Class_list_nat, Class_list_nat_iterator;
PACKAGE Pack_procedures160 IS
  PROCEDURE ex1;
  PROCEDURE ex2;
  PROCEDURE ex3;
  PROCEDURE ex4;
  PROCEDURE ex5;
  PROCEDURE ex6;
END Pack_procedures160;

--[pack_procedures.adb] Implementation
WITH Simple_io;
USE  Simple_io;
PACKAGE BODY Pack_procedures160 IS

PROCEDURE ex1 IS
  numbers    : List;            -- List of Natural numbers
  numbers_it : List_iter;       -- Iterator for list
BEGIN
  first( numbers_it, numbers );
  insert( numbers_it, 50 );
  insert( numbers_it, 5 );
  insert( numbers_it, 40 );
  last( numbers_it, numbers );
  next( numbers_it );
  insert( numbers_it, 100 );
  prev( numbers_it );
  prev( numbers_it );
  insert( numbers_it, 30 );
  first( numbers_it, numbers );
  WHILE NOT is_end( numbers_it ) LOOP
    put( deliver( numbers_it ) );
    next( numbers_it );
  END LOOP;
  new_line;
END ex1;

PROCEDURE ex2 IS
  numbers    : List;            -- List of Natural numbers
  numbers_it : List_iter;       -- Iterator for list

PROCEDURE insert( pos:IN Integer; num:IN Integer ) IS
  count : Integer := pos;
BEGIN
  first( numbers_it, numbers );
  WHILE count >= 1 LOOP
    next( numbers_it ); count := count-1;
  END LOOP;
  insert( numbers_it, num );
END insert;

PROCEDURE delete( pos:IN Integer ) IS
  count : Integer := pos;
BEGIN
  first( numbers_it, numbers );
  WHILE count >= 1 LOOP
    next( numbers_it ); count := count-1;
  END LOOP;
  delete( numbers_it );
END delete;

PROCEDURE print IS
BEGIN
  first( numbers_it, numbers );
  WHILE NOT is_end(numbers_it) LOOP
    put( deliver(numbers_it), width=>3 );
    next( numbers_it );
  END LOOP;
  new_line;
END print;

BEGIN
  insert( 0, 50 ); print;
  put(" 50 <Should be>"); new_line;
  insert( 0, 30 ); print;
  put(" 30 50 <Should be>"); new_line;
  insert( 1, 40 ); print;
  put(" 30 40 50 <Should be>"); new_line;
  insert( 2, 45 ); print;
  put(" 30 40 45 50 <Should be>"); new_line;
  insert( 2, 42 ); print;
  put(" 30 40 42 45 50 <Should be>"); new_line;
  insert( 5, 99 ); print;
  put(" 30 40 42 45 50 99<Should be>"); new_line;

  delete( 5 ); print;
  put(" 30 40 42 45 50 <Should be>"); new_line;
  delete( 0 ); print;
  put(" 40 42 45 50 <Should be>"); new_line;
  delete( 1 ); print;
  put(" 40 45 50 <Should be>"); new_line;
  delete( 2 ); print;
  put(" 40 45 <Should be>"); new_line;
  delete( 1 ); print;
  put(" 40 <Should be>"); new_line;
  delete( 0 ); print;
  put(" <Should be>"); new_line;
END ex2;

PROCEDURE ex3 IS
  numbers    : List;            -- List of Natural numbers
  numbers_it : List_iter;       -- Iterator for list
BEGIN
  first( numbers_it, numbers );
  insert( numbers_it, 100 );
  insert( numbers_it, 50 );

  first( numbers_it, numbers );
  WHILE NOT is_end(numbers_it) LOOP
    put( deliver(numbers_it) ); next( numbers_it );
  END LOOP;
  new_line;
END ex3;

PROCEDURE ex4 IS
  numbers    : List;               -- List of Natural numbers
  numbers_it : List_iter;          -- Iterator for list
BEGIN
  first( numbers_it, numbers );    -- Initialize
  FOR number IN 1 .. 10 LOOP
    insert( numbers_it, number );  -- Insert before
    next( numbers_it );            -- Next item
  END LOOP;

  first(numbers_it,numbers);                -- Set to start
  WHILE NOT is_end( numbers_it ) LOOP       -- Not end of list
    put( deliver(numbers_it) ); put(" ");   --  Print
    next( numbers_it );                     -- Next item
  END LOOP;
  new_line;
END ex4;

PROCEDURE ex5 IS
  original: List;                      -- List of numbers
  copy    : List;                      -- List of numbers
  num_it  : List_iter;                 -- Iterator
BEGIN
  FOR number IN 1 .. 10 LOOP
    last( num_it, original );          -- Set iterator last
    next( num_it );                    -- So can insert
    insert( num_it, number );          --   after last item
  END LOOP;

  put("copy := original "); new_line;
  copy := original;

  put("copy = original ");
  IF copy = original THEN put("True"); ELSE put("False"); END IF;
  new_line;

  put("Append 99 to copy "); new_line;
  last( num_it, copy ); next( num_it );
  insert( num_it, 99 );

  put("copy = original ");
  IF copy = original THEN put("True"); ELSE put("False"); END IF;
  new_line;

  first(num_it,original);               -- Set to start
  WHILE NOT is_end( num_it ) LOOP       -- Not end of list
    put( deliver(num_it), width=>4 );   --  Print
    next( num_it );                     -- Next item
  END LOOP;

  new_line;
  first(num_it,copy);                   -- Set to start
  WHILE NOT is_end( num_it ) LOOP       -- Not end of list
    put( deliver(num_it), width=>4 );   --  Print
    next( num_it );                     -- Next item
  END LOOP;
  new_line;
END ex5;

PROCEDURE ex6 IS
  numbers: List;                       -- List of numbers
  num_it : List_iter;                  --  Iterator
  num,in_list: Natural;
BEGIN
  first( num_it, numbers );

  WHILE NOT end_of_file LOOP           -- Data available
    WHILE NOT end_of_line LOOP
      get(num); first(num_it,numbers); -- Read number
      WHILE NOT is_end( num_it ) LOOP  -- Scan through list
          in_list := deliver(num_it);
          EXIT WHEN in_list > num;     -- Exit when larger no.
          next( num_it );              -- Next item
      END LOOP;
      insert( num_it, num );           -- Before larger no.
    END LOOP;
    skip_line;                         -- Next line
  END LOOP;

  put("Numbers sorted are: ");
  first(num_it,numbers);               -- Set at start
  WHILE NOT is_end( num_it ) LOOP
    in_list := deliver( num_it );      -- Current number
    put( in_list,width=>1 ); put(" "); --  Print
    next( num_it );                    -- Next number
  END LOOP;
  new_line;
END ex6;


END Pack_procedures160;

--[main.adb] Procedure
WITH Simple_io, Pack_procedures160;
USE  Simple_io, Pack_procedures160;
PROCEDURE main160 IS
BEGIN
  put("Example 1"); new_line; ex1;
  put("Example 2"); new_line; ex2;
  put("Example 3"); new_line; ex3;
  put("Example 4"); new_line; ex4;
  put("Example 5"); new_line; ex5;
  put("Example 6"); new_line; ex6;
END main160;
