WITH Ada.Finalization, Unchecked_deallocation;
USE  Ada.Finalization;
GENERIC
  TYPE T IS PRIVATE;                  -- Any type
PACKAGE Class_list IS
  TYPE List IS NEW Controlled WITH PRIVATE;

  PROCEDURE initialize( the:IN OUT List );
  PROCEDURE initialize( the:IN OUT List; data:IN T );
  PROCEDURE finalize( the:IN OUT List );
  PROCEDURE adjust( the:IN OUT List );
  FUNCTION "="  ( f:IN List; s:IN List ) RETURN Boolean;
  PROCEDURE xyz(the:IN OUT List ); -- DEBUG TRACE
PRIVATE
  PROCEDURE release_storage( the:IN OUT List );

  TYPE Node;                       -- Tentative declaration
  TYPE P_Node IS ACCESS ALL Node;  -- Pointer to Node

  TYPE Node IS RECORD
    prev    : P_Node;              -- Previous Node
    item    : T;                   -- The physical item
    next    : P_Node;              -- Next Node
  END RECORD;

  PROCEDURE dispose_node IS
    NEW unchecked_deallocation( Node, P_Node );
  TYPE List IS NEW Controlled WITH RECORD
    first_node : ALIASED P_Node := NULL;   -- First item in list
    last_node  : ALIASED P_Node := NULL;   -- First item in list
  END RECORD;

END Class_list;

--[class_list.adb] Implementation
--WITH Simple_io; USE  Simple_io;
PACKAGE BODY Class_list IS

  PROCEDURE initialize( the:IN OUT List ) IS
  BEGIN
--  put("Initialization C[List]"); new_line; -- DEBUG TRACE
    the.first_node := NULL;   -- Empty list
    the.last_node  := NULL;   -- Empty list
    xyz( the ); -- DEBUG TRACE
  END initialize;

  PROCEDURE initialize( the:IN OUT List; data:IN T ) IS
  BEGIN
--  put("Initialization [List,data]"); new_line; -- DEBUG TRACE
    the.first_node := NEW Node'(NULL, data, NULL);
    the.last_node  := the.first_node;
    xyz( the ); -- DEBUG TRACE
  END initialize;

  PROCEDURE finalize( the:IN OUT List ) IS
  BEGIN
--  put("Finalization C[List]"); new_line; -- DEBUG TRACE
    IF the.first_node /= NULL THEN
      release_storage( the );
      the.first_node := NULL;
    END IF;
    --xyz( the ); -- Can not do this here DEBUG TRACE
  END finalize;

  PROCEDURE adjust( the:IN OUT List ) IS
    cur : P_Node := the.first_node;  -- Original list
    lst : P_Node := NULL;            -- Last created node
    prv : P_Node := NULL;            -- Previously created node
    fst : P_Node := NULL;            -- The first node

  BEGIN
--  put("Adjust list "); new_line; -- DEBUG TRACE
    WHILE cur /= NULL LOOP
      lst := NEW Node'( prv, cur.item, NULL );
      IF fst =  NULL THEN fst := lst; END IF;
      IF prv /= NULL THEN prv.next := lst; END IF;
      prv := lst;
      cur := cur.next;               -- Next node
    END LOOP;
    the.first_node := fst;           -- Update
    the.last_node  := lst;
    xyz( the ); -- DEBUG TRACE
  END adjust;

  FUNCTION "="  ( f:IN List; s:IN List ) RETURN Boolean IS
    f_node : P_Node := f.first_node;  -- First list
    s_node : P_Node := s.first_node;  -- Second list
  BEGIN
    WHILE f_node /= NULL AND s_node /= NULL LOOP
      IF f_node.item /= s_node.item THEN
        RETURN FALSE;                 -- Different items
      END IF;
      f_node := f_node.next; s_node := s_node.next;
    END LOOP;
    RETURN f_node = s_node;           -- Both NULL if equal
  END "=";

  PROCEDURE release_storage( the:IN OUT List ) IS
    cur : P_Node := the.first_node; -- Pointer to curr node
    tmp : P_Node;                   -- Node to dispose
  BEGIN
--  put("Release storage "); new_line; -- DEBUG TRACE
    WHILE cur /= NULL LOOP          -- For each item in list
      tmp := cur;                   -- Item to dispose
      cur := cur.next;              -- Next node
--    put("Dispose node");new_line; -- DEBUG TRACE;
      dispose_node( tmp );          -- Dispose of item
    END LOOP;
  END release_storage;

  PROCEDURE xyz(the:IN OUT List ) IS -- DEBUG TRACE
    cur : P_Node := the.first_node;  -- DEBUG TRACE
  BEGIN                              -- DEBUG TRACE
    WHILE cur /= NULL LOOP           -- DEBUG TRACE
      cur := cur.next;               -- DEBUG TRACE
    END LOOP;                        -- DEBUG TRACE
    cur := the.last_node;            -- DEBUG TRACE
    WHILE cur /= NULL LOOP           -- DEBUG TRACE
      cur := cur.prev;               -- DEBUG TRACE
    END LOOP;                        -- DEBUG TRACE
  END xyz;                           -- DEBUG TRACE
                                     -- DEBUG TRACE
END Class_list;

--[class_list-iterator.ads] Specification
--WITH Ada.Finalization; USE  Ada.Finalization;
GENERIC
PACKAGE Class_list.Iterator IS

  TYPE List_iter IS LIMITED PRIVATE;

  PROCEDURE initialize( the:IN OUT List_iter );
  PROCEDURE finalize( the:IN OUT List_iter );
  PROCEDURE first( the:IN OUT List_iter; l:IN OUT List );
  PROCEDURE last( the:IN OUT List_iter; l:IN OUT List );

  FUNCTION  deliver( the:IN List_iter) RETURN T;
  PROCEDURE insert( the:IN OUT List_iter; data:IN T );
  PROCEDURE delete( the:IN OUT List_iter );
  FUNCTION  is_end( the:IN List_iter ) RETURN Boolean;
  PROCEDURE next( the:IN OUT List_iter );
  PROCEDURE prev( the:IN OUT List_iter );
  PROCEDURE xyz( the:IN OUT List_iter ); -- DEBUG TRACE
PRIVATE
  TYPE P_P_Node IS ACCESS ALL P_Node;
  TYPE List_iter IS RECORD
    cur_list_first: P_P_Node := NULL;    -- First in chain
    cur_list_last : P_P_Node := NULL;    -- Last in chain
    cur_node      : P_Node   := NULL;    -- Current item
  END RECORD;
END Class_list.Iterator;

--[class_list-iterator.adb] Implementation
--WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_list.Iterator IS

  PROCEDURE xyz( the:IN OUT List_iter ) IS  -- DEBUG TRACE
    cur : P_Node := the.cur_list_first.ALL; -- DEBUG TRACE
  BEGIN                                     -- DEBUG TRACE
    WHILE cur /= NULL LOOP                  -- DEBUG TRACE
      cur := cur.next;                      -- DEBUG TRACE
    END LOOP;                               -- DEBUG TRACE
    cur := the.cur_list_last.ALL;           -- DEBUG TRACE
    WHILE cur /= NULL LOOP                  -- DEBUG TRACE
      cur := cur.prev;                      -- DEBUG TRACE
    END LOOP;                               -- DEBUG TRACE
  END xyz;                                  -- DEBUG TRACE
                                            -- DEBUG TRACE
  PROCEDURE initialize( the:IN OUT List_iter ) IS
  BEGIN
--  put("Initialize C[Iter]"); new_line;    -- DEBUG TRACE
    the.cur_node       := NULL;   -- Iterator not setup
    the.cur_list_first := NULL;
    the.cur_list_last  := NULL;
    xyz( the );                             -- DEBUG TRACE
  END initialize;

  PROCEDURE finalize( the:IN OUT List_iter ) IS
  BEGIN
--  put("Finalize C[Iter]"); new_line;      -- DEBUG TRACE
    xyz( the );                             -- DEBUG TRACE
    NULL;
  END finalize;

  PROCEDURE first( the:IN OUT List_iter; l:IN OUT List ) IS
  BEGIN
--  put("First C[Iter]"); new_line;         -- DEBUG TRACE
    the.cur_node      := l.first_node;       -- Set to first
    the.cur_list_first:= l.first_node'Unchecked_Access;
    the.cur_list_last := l.last_node'Unchecked_Access;
    xyz( the ); -- DEBUG TRACE
  END first;

  PROCEDURE last( the:IN OUT List_iter; l:IN OUT List ) IS
  BEGIN
--  put("Last C[Iter]"); new_line;           -- DEBUG TRACE
    the.cur_node      := l.last_node;        -- Set to last
    the.cur_list_first:= l.first_node'Unchecked_Access;
    the.cur_list_last := l.last_node'Unchecked_Access;
    xyz( the ); -- DEBUG TRACE
  END last;

  FUNCTION deliver( the:IN List_iter ) RETURN T IS
  BEGIN
--  put("deliver [iter]"); new_line;          -- DEBUG TRACE
    RETURN the.cur_node.item;  -- The current item
  END deliver;

  PROCEDURE insert( the:IN OUT List_iter; data:IN T ) IS
    tmp   : P_Node;
    cur   : P_Node   := the.cur_node;   -- Current element
    first : P_P_Node := the.cur_list_first;
    last  : P_P_Node := the.cur_list_last;
  BEGIN
--  put("insert [iter]"); new_line;           -- DEBUG TRACE
    IF cur = NULL THEN            -- Empty or last item
      IF first.ALL = NULL THEN    --  Empty list
        tmp := NEW Node'( NULL, data, NULL );
        first.ALL := tmp;
        last.ALL  := tmp;
        the.cur_node := tmp;
      ELSE                        --  Last
        tmp := NEW Node'( last.ALL, data, NULL );
        last.ALL.next := tmp;
        last.ALL      := tmp;
        the.cur_node := tmp;
       END IF;
    ELSE
      tmp := NEW Node'( cur.prev, data, cur );
      IF cur.prev = NULL THEN      -- First item
        first.ALL := tmp;
      ELSE
        cur.prev.next := tmp;
      END IF;
      cur.prev := tmp;
    END IF;
    xyz( the );                            -- DEBUG TRACE
  END insert;

  PROCEDURE delete( the:IN OUT List_iter) IS
    cur   : P_Node   := the.cur_node;   -- Current element
    first : P_P_Node := the.cur_list_first;
    last  : P_P_Node := the.cur_list_last;
  BEGIN
--  put("delete [iter]"); new_line; -- DEBUG TRACE
    IF cur /= NULL THEN             -- Something to delete
      IF cur.prev /= NULL THEN      -- Fix forward pointer;
        cur.prev.next := cur.next;  --  Not first in chain
      ELSE
        first.ALL := cur.next;      --  First in chain
        IF first.ALL = NULL THEN
          last.ALL := NULL;         --   Empty list
        END IF;
      END IF;
      IF cur.next /= NULL THEN      -- Fix backward pointer;
        cur.next.prev := cur.prev;  --  Not last in chain
      ELSE
        last.ALL := cur.prev;       --  Last in chain
        IF last.ALL = NULL THEN
          first.ALL := NULL;        --   Empty list
        END IF;
      END IF;
      IF cur.next /= NULL THEN      -- Fix current pointer
        the.cur_node := cur.next;   --  next
      ELSIF cur.prev /= NULL THEN
        the.cur_node := cur.prev;   --  previous
      ELSE
        the.cur_node := NULL;       --  none empty list
      END IF;
      dispose_node( cur );          -- Release storage
      xyz( the );                   -- DEBUG TRACE
    END IF;
  END delete;

  FUNCTION  is_end( the:IN List_iter ) RETURN Boolean IS
  BEGIN
--  put("is end [iter]"); new_line; -- DEBUG TRACE
    RETURN the.cur_node = NULL;               -- True if end
  END is_end;

  PROCEDURE next( the:IN OUT List_iter ) IS
  BEGIN
--  put("next [iter]"); new_line; -- DEBUG TRACE
    IF the.cur_node /= NULL THEN               --
      the.cur_node  := the.cur_node.next;      -- Next
    END IF;
    xyz( the ); -- DEBUG TRACE
  END next;

  PROCEDURE prev( the:IN OUT List_iter ) IS
  BEGIN
--  put("prev [iter]"); new_line; -- DEBUG TRACE
    IF the.cur_node /= NULL THEN               --
      the.cur_node  := the.cur_node.prev;      -- Previous
    END IF;
    xyz( the ); -- DEBUG TRACE
  END prev;

END Class_list.Iterator;

--[class_set.ads] Specification
WITH Class_list, Class_list.Iterator;
PRAGMA ELABORATE_ALL( Class_List, Class_List.Iterator );
GENERIC
  TYPE T IS PRIVATE;
  WITH PROCEDURE put( item:IN T ) IS <>;
  WITH FUNCTION  ">" (first,second:IN T ) RETURN Boolean IS <>;
  WITH FUNCTION  "<" (first,second:IN T ) RETURN Boolean IS <>;
PACKAGE Class_set IS
  TYPE Set IS PRIVATE;
  PROCEDURE put( the:IN Set );
  FUNCTION "+"( f:IN Set; s:IN Set ) RETURN Set;
  FUNCTION set_const( item: IN T )   RETURN Set;
  FUNCTION members( the:IN Set )     RETURN Positive;
PRIVATE
  PACKAGE Class_list_t          IS NEW Class_list(T);
  PACKAGE Class_list_t_iterator IS NEW Class_list_t.Iterator;
  TYPE Set IS NEW Class_list_t.List WITH RECORD
    elements : Natural := 0;             -- Elements in set
  END RECORD;
END Class_set;

--[class_set.adb] Implementation
WITH Simple_io; USE Simple_io;
PACKAGE BODY Class_set IS
  USE Class_list_t, Class_list_t_iterator;

  PROCEDURE put( the:IN Set ) IS
    it    : List_iter;
    c_the : List := List(the);
  BEGIN
    put("("); first( it, c_the );
    FOR i IN 1 .. the.elements LOOP
      put( deliver(it) ); next( it );
      IF i /= the.elements THEN put(","); END IF;
    END LOOP;
    put(")");
  END put;

  FUNCTION "+"  ( f:IN Set; s:IN Set ) RETURN Set IS
    res_it    : List_iter;
    f_it,s_it : List_iter;
    res       : Set;
    f_list, s_list: List;
  BEGIN
    f_list := List(f); s_list := List(s);
    first( f_it, List(f_list) );
    first( s_it, List(s_list) );
    first( res_it, List(res) );

    WHILE (NOT is_end(f_it)) OR (NOT is_end(s_it)) LOOP
      IF is_end(f_it) THEN
--        put("END f"); new_line;           -- DEBUG
        next(res_it); insert(res_it, deliver(s_it));
        next(s_it);
      ELSIF is_end(s_it) THEN
--        put("END s"); new_line; -- DEBUG
        next(res_it); insert(res_it, deliver(f_it));
        next(f_it);
      ELSIF deliver(f_it) < deliver(s_it) THEN
--        put("f < s"); new_line; -- DEBUG
        next(res_it); insert(res_it, deliver(f_it));
        next(f_it);
      ELSIF deliver(f_it) > deliver(s_it) THEN
--        put("f > s"); new_line; -- DEBUG
        next(res_it); insert(res_it, deliver(s_it));
        next(s_it);
      ELSIF deliver(f_it) = deliver(s_it) THEN
--        put("f = s"); new_line; -- DEBUG
        next(res_it); insert(res_it, deliver(f_it) );
        next(f_it); next(s_it);
      END IF;
      res.elements := res.elements + 1;
--      put("Res["); put(res); put("]"); new_line; -- DEBUG
    END LOOP;
    RETURN res;
  END "+";


  FUNCTION set_const( item: IN T ) RETURN Set IS
    res : Set;
  BEGIN
    initialize( res, item ); res.elements := 1;
    RETURN res;
  END set_const;

  FUNCTION members( the:IN Set ) RETURN Positive IS
  BEGIN
    RETURN the.elements;
  END members;

END Class_set;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---
-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

PACKAGE Pack_types300 IS
  TYPE Filling IS ( CHEESE, ONION, HAM, TOMATO );
END Pack_types300;

WITH Simple_io, Pack_types300; 
USE  Simple_io, Pack_types300;
PROCEDURE put_filling( c:IN Filling ) IS
BEGIN
  put( Filling'Image( c ) );
END put_filling;

WITH Pack_types300, Class_set, put_filling;
USE  Pack_types300;                          -- **** Object Ada
PRAGMA ELABORATE_ALL( Class_set );
PACKAGE Class_set_sandwich IS 
  NEW Class_set( T => Pack_types300.Filling, put => put_filling );

--[main1.adb] Procedure
WITH Pack_types300, Simple_io, Class_set_sandwich;
USE  Pack_types300, Simple_io, Class_set_sandwich;
PROCEDURE main301 IS
  sandwich : Class_set_sandwich.Set;
BEGIN
  sandwich := sandwich + set_const(CHEESE);
  sandwich := sandwich + set_const(ONION) ;
  put("Contents of sandwich are : ");
  put( sandwich ); new_line;
  put("Number of ingredients is : ");
  put( members(sandwich) ); new_line;
  NULL;
END main301;

-- ---- --- --- --- --- --- --- --- --- --- --- --- --- ---

WITH Simple_io;
PROCEDURE put_natural( n:IN Natural ) IS
BEGIN
  Simple_io.put( n, width=>2 );
END put_natural;

WITH Class_set, put_natural;
PRAGMA ELABORATE_ALL( Class_set );
PACKAGE Class_set_naturals IS 
  NEW Class_set( T => Natural, put => put_natural );

--[main2.adb] Procedure
WITH Simple_io, Class_set_naturals;
USE  Simple_io, Class_set_naturals;
PROCEDURE main302 IS
  set1 : Class_set_naturals.Set;
  set2 : Class_set_naturals.Set;
  set3 : Class_set_naturals.Set;
BEGIN
  FOR i IN 1 .. 5 LOOP
    set1 := set1 + set_const(i*2);
  END LOOP;
  put( "Set 1 =       "); put( set1 ); new_line;
  FOR i IN 1 .. 5 LOOP
    set2 := set2 + set_const(i*2+1);
  END LOOP;
  put( "Set 2 =       "); put( set2 ); new_line;
  put( "Set 1 + Set 2 ");
  put( set1+set2 ); new_line;
  FOR i IN 4 .. 15 LOOP
    set3 := set3 + set_const(i);
  END LOOP;
  put( "Set 3 =       "); put( set3 ); new_line;
  put( "Set 2 + Set 3 ");
  put( set2+set3 ); new_line;
END main302;


--[main3.adb] Procedure
WITH Simple_io;
USE  Simple_io;
PROCEDURE main303 IS
BEGIN
  NULL;
END main303;

--[main4.adb] Procedure
WITH Simple_io;
USE  Simple_io;
PROCEDURE main304 IS
BEGIN
  NULL;
END main304;

--[main.adb] Procedure
WITH Simple_io, main301, main302, main303, main304;
USE  Simple_io;
PROCEDURE main300 IS
BEGIN
  put("Example 1 "); new_line; main301;
  put("Example 2 "); new_line; main302;
  put("Example 3 "); new_line; main303;
  put("Example 4 "); new_line; main304;
END main300;
