--[class_account.ads] Specification
WITH Simple_io; USE Simple_io;
PACKAGE Class_account61 IS
  TYPE Account IS TAGGED PRIVATE;
  SUBTYPE Money  IS Float;
  SUBTYPE PMoney IS Float RANGE 0.0 .. Float'Last;

  PROCEDURE statement( the:IN Account );
  PROCEDURE deposit( the:IN OUT Account; amount:IN PMoney );
  PROCEDURE withdraw( the:IN OUT Account;
                      amount:IN PMoney; get:OUT PMoney );
  FUNCTION  balance( the:IN Account ) RETURN Money;
PRIVATE
  TYPE Account IS TAGGED RECORD
     balance_of : Money := 0.00;           -- Amount on deposit
  END RECORD;
END Class_account61;


--[class_account.adb] Implementation
PACKAGE BODY Class_account61 IS

  PROCEDURE statement( the:IN Account ) IS
  BEGIN
    put("Mini statement: The amount on deposit is $" );
    put( the.balance_of, aft=>2, exp=>0 );
    new_line(2);
  END statement;

  PROCEDURE deposit( the:IN OUT Account; amount:IN PMoney ) IS
  BEGIN
    the.balance_of := the.balance_of + amount;
  END deposit;

  PROCEDURE withdraw( the:IN OUT Account;
                      amount:IN PMoney; get:OUT PMoney ) IS
  BEGIN
    IF the.balance_of >= amount THEN
      the.balance_of := the.balance_of - amount;
      get := amount;
    ELSE
      get := 0.00;
    END IF;
  END withdraw;

  FUNCTION  balance( the:IN Account ) RETURN Money IS
  BEGIN
    RETURN the.balance_of;
  END balance;

END Class_account61;

-- Generic IN        constant
--         IN OUT    Renaming

--[class_object_rc.ads] Specification
WITH Ada.Finalization; USE Ada.Finalization;
GENERIC
  TYPE T IS PRIVATE;                    -- The type
  NULL_VALUE:IN T;                      -- Identity element
PACKAGE Class_object_rc IS
  TYPE Object IS NEW Controlled WITH PRIVATE;
  TYPE P_T IS ACCESS ALL T;

  PROCEDURE initialize( the:IN OUT Object );
  PROCEDURE initialize( the:IN OUT Object; data:IN T );
  PROCEDURE finalize( the:IN OUT Object );
  PROCEDURE adjust( the:IN OUT Object );
  FUNCTION  deliver( the:IN Object) RETURN T;
  FUNCTION  deliver_ref( the:IN Object) RETURN P_T;
  PROCEDURE unique( the:IN OUT Object);
PRIVATE
  PROCEDURE build_storage ( the:IN OUT Object; value:IN T );
  PROCEDURE release_storage( the:IN OUT Object );

  TYPE Descriptor;
  TYPE P_Descriptor IS ACCESS ALL Descriptor;

  TYPE Descriptor IS RECORD
    refs   : Natural;       -- References to this data
    object : ALIASED T;     -- The physical data
  END RECORD;

  TYPE Object IS NEW Controlled WITH RECORD
     p_desc : P_Descriptor:= NULL; -- Descriptor for a number
  END RECORD;

END Class_object_rc;


--[class_object_rc.adb] Implementation
WITH Simple_io; USE Simple_io;
WITH unchecked_deallocation;
PACKAGE BODY Class_object_rc IS

  PROCEDURE initialize( the:IN OUT Object ) IS
  BEGIN
    put("Initialize"); new_line;  -- DEBUG
    build_storage( the, NULL_VALUE );
  END initialize;

  PROCEDURE initialize( the:IN OUT Object; data:IN T ) IS
  BEGIN
    put("Initialize( data )"); new_line;  -- DEBUG
    build_storage( the, data );
  END initialize;

  PROCEDURE build_storage ( the:IN OUT Object; value:IN T ) IS
  BEGIN
    put("Build_storage"); new_line;  -- DEBUG
    the.p_desc := NEW Descriptor'(1,value);
  END build_storage;

  PROCEDURE finalize( the:IN OUT Object ) IS
  BEGIN
    put("Finalize"); new_line;  -- DEBUG
    IF the.p_desc /= NULL THEN
      release_storage( the );
      the.p_desc := NULL;
    END IF;
  END finalize;

  PROCEDURE dispose IS
    NEW unchecked_deallocation( Descriptor, P_Descriptor );

  PROCEDURE release_storage( the:IN OUT Object ) IS
  BEGIN
    the.p_desc.refs := the.p_desc.refs-1;
    IF the.p_desc.refs = 0 THEN
      put("Release storage "); --put( the.p_desc.object );  -- DEBUG
      new_line;  -- DEBUG
      dispose( the.p_desc );
    ELSE
      put("Release storage "); --put( the.p_desc.object );  -- DEBUG
      put(" <shared> so not done"); new_line;  -- DEBUG
      NULL;
    END IF;
  END release_storage;

  PROCEDURE adjust( the:IN OUT Object ) IS
  BEGIN
    put("Adjust"); new_line;  -- DEBUG
    the.p_desc.refs := the.p_desc.refs+1;
  END adjust;

  FUNCTION  deliver( the:IN Object) RETURN T IS
  BEGIN
    put("Deliver"); new_line;  -- DEBUG
    RETURN the.p_desc.object;
  END deliver;

  FUNCTION  deliver_ref( the:IN Object) RETURN P_T IS
  BEGIN
    put("Deliver ref"); new_line;  -- DEBUG
    RETURN the.p_desc.object'Access;
  END deliver_ref;

  PROCEDURE unique( the:IN OUT Object) IS
    tmp : P_Descriptor;
  BEGIN
    put("Unique"); new_line; -- DEBUG
    IF the.p_desc.refs > 1 THEN
      the.p_desc.refs := the.p_desc.refs-1;
      tmp := NEW Descriptor'(1,the.p_desc.object);
      the.p_desc := tmp;
    END IF;
  END unique;
END Class_object_rc;

----[class_object_rc_int.ADS] Specification Instantiation
--WITH Class_object_rc;
--  PACKAGE Class_rc_integer IS
--    NEW Class_object_rc( Integer, 0 );

----[class_ref_integer.ads] Specification
--WITH Class_rc_integer; USE Class_rc_integer;
--PACKAGE Class_ref_integer IS
--  TYPE Ref_integer IS NEW Object WITH NULL RECORD;
--  FUNCTION  number_const( value:IN Integer ) RETURN Ref_integer;
--  FUNCTION "+" ( l,r:IN Ref_integer ) RETURN Ref_integer;
--  FUNCTION "=" ( l,r:IN Ref_integer ) RETURN Boolean;
--END Class_ref_integer;
--
----[class_ref_integer.adb] Implementation
--PACKAGE BODY Class_ref_integer IS
--  FUNCTION  number_const( value:IN Integer ) RETURN Ref_integer IS
--    res: Ref_integer;
--  BEGIN
--    initialize( res, value );
--    RETURN res;
--  END number_const;
--
--  FUNCTION "+" ( l,r:IN Ref_integer ) RETURN Ref_integer IS
--    res : Ref_integer;
--  BEGIN
--    initialize( res, deliver(l) + deliver(r) );
--    RETURN res;
--  END "+";
--
--  FUNCTION "=" ( l,r:IN Ref_integer ) RETURN Boolean IS
--  BEGIN
--    RETURN deliver(l) = deliver(r);
--  END "=";
--END Class_ref_integer;

WITH Class_object_rc;
PRAGMA ELABORATE_ALL( Class_object_rc );
PACKAGE Class_rc_integer IS
  NEW Class_object_rc(Integer, 0);

--[pack_procedures.adb] Implementation
WITH Simple_io, Class_rc_integer;
USE  Simple_io, Class_rc_integer;
PROCEDURE main171 IS
  a,b,c : Class_rc_integer.Object;
BEGIN
  initialize( b, 10 );
  initialize( c, 20 );

  put("a := b; "); new_line;
  a := b;

  put("b := c; "); new_line;
  b := c;

  put("c := a; "); new_line;
  c := a;

  put("a = "); put( deliver( a ) ); new_line;
  put("b = "); put( deliver( b ) ); new_line;
  put("c = "); put( deliver( c ) ); new_line;

END main171;

PACKAGE Pack_types170 IS
  TYPE P_Integer IS ACCESS ALL Integer;
END Pack_types170;

WITH Pack_types170, Class_object_rc;
PACKAGE Class_object_rc_p_int IS
  NEW Class_object_rc(Pack_types170.P_Integer, NULL);

WITH Simple_io, Class_object_rc_p_int;
USE  Simple_io, Class_object_rc_p_int;
PROCEDURE main172 IS
  a,b,c  : Object;
BEGIN
  initialize( a, NEW Integer'(10) );
  initialize( b, NEW Integer'(20) );

  put("a := b; "); new_line;
  a := b;
  put("a = "); put( deliver( a ).ALL ); new_line;
  put("b = "); put( deliver( b ).ALL ); new_line;
END main172;

PROCEDURE main173 IS
BEGIN
  NULL;
END main173;

--WITH Class_account61;
--PACKAGE Pack_consts IS
--  null_account: Class_account61.Account;
--END Pack_consts;
--
--WITH Pack_consts, Class_object_rc, Class_account61;
--PACKAGE Class_rc_account IS
--  NEW Class_object_rc(Class_account61.Account, Pack_consts.null_account);
--
PROCEDURE main174 IS
--  original,copy : Class_rc_account.Object;
--BEGIN
--  deposit( deliver_ref(original).ALL, 100.00 );
--  put("copy := original; (Shallow copy)"); new_line;
--  copy := original;                       -- Shallow copy
--  statement( deliver_ref(original).ALL ); -- The same object
--  statement( deliver_ref(copy).ALL );     --  "  "
--  put("Make copy unique (Deep copy if necessary)"); new_line;
--  unique( copy );                         -- Deep copy
--  deposit( deliver_ref(copy).ALL, 20.00 );-- copy only
--  statement( deliver_ref(original).ALL ); -- Unique object
--  statement( deliver_ref(copy).ALL );     --   "   "
BEGIN
  NULL;
END main174;

WITH Simple_io, Class_rc_integer;
USE  Simple_io, Class_rc_integer;
PROCEDURE main175 IS
  a : Class_rc_integer.Object;
BEGIN
  initialize( a, 20 );
  put("a = "); put( deliver( a ) ); new_line;

  put("a := a; "); new_line;

  put("a = "); put( deliver( a ) ); new_line;

END main175;

--[main.adb] Procedure
WITH Simple_io, main171, main172, main173, main174, main175;
USE  Simple_io;
PROCEDURE main170 IS
BEGIN
  put("Example 1"); new_line; main171;
  put("Example 2"); new_line; main172;
  put("Example 3"); new_line; main173;
  put("Example 4"); new_line; main174;
  put("Example 5"); new_line; main175;
END main170;
