with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int; 
  use type Win32.Long;
--  use type Win32.Uint; 

   Help_Message_Text      : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "Help not yet implimented");

   True                   : constant Win32.Bool := 1;

   False                  : constant Win32.Bool := 0;

   About_Message_Text     : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "A Poor-Person's Menu Program");

   About_Text             : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "About...");

   Help_Text              : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "Help...");

   Remove_Text            : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "Remove Additions");

   Idm_About              : constant Win32.Uint := 1;             

   Idm_Help               : constant Win32.Uint := 2;             

   Idm_Remove             : constant Win32.Uint := 3;             

   Menu_Handle           : Win32.Windef.Hmenu;

   Window_Class          : aliased Win32.Winuser.Wndclass;

--   Device_Context_Handle : aliased Win32.Windef.Hdc;

   Message               : aliased Win32.Winuser.Msg;

   Integer_Result        : Win32.Int;
     
   Dword_Result          : Win32.Dword;

   Uint_Result           : Win32.Uint;

   Bool_Result           : Win32.Bool;
                     
   Long_Result           : Win32.Long;

   Atom_Result           : Win32.Windef.Atom;

   Menu_Result           : Win32.Windef.Hmenu;

   function Convert_Chars_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Convert_Integer_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Lpcstr);

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is

   begin

      case Message is

	 when Win32.Winuser.Wm_Syscommand =>

           case
             Win32.Uint
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Wparam))) is

             when
               Idm_About   =>

               Integer_Result :=
                 Win32.Winuser.MessageBox
                   (Hwnd      => 
                     Window_Handle,
                    Lptext    => 
                     Convert_Chars_Ptr_To_Lpcstr
                       (S => 
                         About_Message_Text),
                    Lpcaption =>
                     Convert_Chars_Ptr_To_Lpcstr
                       (S => 
                         Windows.Data.Class_Name),
                    Utype     =>
                     Win32.Uint
                       (Win32.Winuser.Mb_Iconexclamation or
                        Win32.Winuser.Mb_Ok));

             when
               Idm_Help            =>

               Integer_Result :=
                 Win32.Winuser.MessageBox
                   (Hwnd      => 
                     Window_Handle,
                    Lptext    => 
                     Convert_Chars_Ptr_To_Lpcstr
                       (S => 
                         Help_Message_Text),
                    Lpcaption =>
                     Convert_Chars_Ptr_To_Lpcstr
                       (S => 
                         Windows.Data.Class_Name),
                    Utype     =>
                     Win32.Uint
                       (Win32.Winuser.Mb_Iconexclamation or
                        Win32.Winuser.Mb_Ok));

             when
               Idm_Remove          =>

               Menu_Result :=
                 Win32.Winuser.Getsystemmenu
                   (Hwnd    =>
                     Window_Handle,
                    Brevert =>
                     True);               

             when
               others              =>
             
               return
                 Win32.Winuser.Defwindowproc
	           (Hwnd   =>
                     Window_Handle, 
	            Msg    => 
                     Message, 
        	    Wparam => 
                     Wparam,
	            Lparam => 
                     Lparam);

           end case;
           
 	   return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground =>
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Menu_Handle :=
      Win32.Winuser.Getsystemmenu
        (Hwnd    =>
          Windows.Data.Window_Handle,
         Brevert =>
          False);
       
    Bool_Result :=
      Win32.Winuser.Appendmenu
        (Hmenu      =>
          Menu_Handle,
         Uflags     =>
          Win32.Winuser.Mf_Separator,
         Uidnewitem =>
          0,
         LpnewItem  =>
          Convert_Integer_To_Lpcstr
            (S =>
              0));

    Bool_Result :=
      Win32.Winuser.Appendmenu
        (Hmenu      =>
          Menu_Handle,
         Uflags     =>
          Win32.Winuser.Mf_String,
         Uidnewitem =>
          Idm_About,
         LpnewItem  =>
          Convert_Chars_Ptr_To_Lpcstr
            (S =>
              About_Text));

    Bool_Result :=
      Win32.Winuser.Appendmenu
        (Hmenu      =>
          Menu_Handle,
         Uflags     =>
          Win32.Winuser.Mf_String,
         Uidnewitem =>
          Idm_Help,
         LpnewItem  =>
          Convert_Chars_Ptr_To_Lpcstr
            (S =>
              Help_Text));

    Bool_Result :=
      Win32.Winuser.Appendmenu
        (Hmenu      =>
          Menu_Handle,
         Uflags     =>
          Win32.Winuser.Mf_String,
         Uidnewitem =>
          Idm_Remove,
         LpnewItem  =>
          Convert_Chars_Ptr_To_Lpcstr
            (S =>
              Remove_Text));

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
