with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with Poppad2;
with System;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int; 
  use type Win32.Long;
  use type Win32.Word;
  use type Win32.Uint; 

   Space_Text            : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "Edit Control out of space");

   Help_Text             : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "Help not yet implimented");

   About_Text            : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "PopPad2 (c) Charles Petzold, 1996");

   Ask_Confirmation_Text : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "Do you really want to close PopPad2");

   Edit_Text             : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "edit");

   Accelerator_Handle    : Win32.Windef.Haccel;

   Edit_Window_Handle    : Win32.Windef.Hwnd;

   Selection             : Win32.Long;

   Is_Enabled            : Win32.Uint;

   True                  : constant Win32.Bool := 1;

   Message                  : aliased Win32.Winuser.Msg;

   Window_Class          : aliased Win32.Winuser.Wndclass;

   Integer_Result        : Win32.Int;
     
   Dword_Result          : Win32.Dword;

   Uint_Result           : Win32.Uint;

   Bool_Result           : Win32.Bool;
                     
   Long_Result           : Win32.Long;

   Hgdiobj_Result        : Win32.Windef.Hgdiobj;

   Atom_Result           : Win32.Windef.Atom;

   Window_Handle_Result  : Win32.Windef.Hwnd;

   function Convert_Chars_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source =>
         Interfaces.C.Strings.Chars_Ptr,
        Target =>
         Win32.Lpcstr);

   function Convert_Integer_To_Hmenu is new
     Ada.Unchecked_Conversion
       (Source =>
         Integer,
        Target =>
         Win32.Windef.Hmenu);

   function Convert_Wparam_To_Hmenu is new
     Ada.Unchecked_Conversion
       (Source =>
         Win32.Wparam,
        Target =>
         Win32.Windef.Hmenu);

   function Convert_Lparam_To_Lpcreatestruct is new
     Ada.Unchecked_Conversion
       (Source =>
         Win32.Lparam,
        Target =>
         Win32.Winuser.Lpcreatestruct);

   function Ask_Confirmation
     (Window_Handle : Win32.Windef.Hwnd) return Win32.Int is

   begin

     return
       Win32.Winuser.Messagebox
         (Hwnd      =>
           Window_Handle,
          Lptext    =>
           Convert_Chars_Ptr_To_Lpcstr
             (S =>
               Ask_Confirmation_Text),
          Lpcaption =>
           Convert_Chars_Ptr_To_Lpcstr
             (S =>
               Windows.Data.Class_Name),
          Utype     =>
           Win32.Uint
             (Win32.Winuser.Mb_Yesno) or
           Win32.Uint
              (Win32.Winuser.Mb_Iconquestion));

   end Ask_Confirmation;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
        if
          Win32.Winuser.Translateaccelerator
            (Hwnd      =>
              Windows.Data.Window_Handle,
             Hacctable =>
              Accelerator_Handle,
             Lpmsg     =>
              Message'Access) = 0 then

      	    Bool_Result := 
              Win32.Winuser.Translatemessage
	        (Lpmsg => 
                 Message'Access);
	 
   	    Long_Result := 
              Win32.Winuser.Dispatchmessage
	        (Lpmsg => 
                 Message'Access);
	 
        end if;

      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is

   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

           Edit_Window_Handle := 
             Win32.WinUser.Createwindow
               (Lpclassname => 
                 Convert_Chars_Ptr_To_Lpcstr
                   (S => 
                     Edit_Text),
                Lpwindowname => 
                 null,
                Dwstyle      => 
                 Win32.Winuser.Ws_Child or
                 Win32.Winuser.Ws_Visible or
                 Win32.Winuser.Ws_Hscroll or
                 Win32.Winuser.Ws_Vscroll or
                 Win32.Winuser.Ws_Border or
                 Win32.Winuser.Es_Left or
                 Win32.Winuser.Es_Multiline or
                 Win32.Winuser.Es_Autohscroll or
                 Win32.Winuser.Es_Autovscroll,
                X            => 
                 0, 
                Y            => 
                 0,
                Nwidth       => 
                 0,
                Nheight      => 
                 0,
                Hwndparent   => 
                 Window_Handle,
                Hmenu        => 
                 Convert_Integer_To_Hmenu
                   (S =>
                     1),
                Hinstance    => 
                 Convert_Lparam_To_Lpcreatestruct
                   (S =>
                     Lparam).Hinstance, 
                Lpparam      =>
                 Null);

 	   return (0);

	 when Win32.Winuser.Wm_Setfocus =>

           Window_Handle_Result :=
             Win32.Winuser.Setfocus
               (Hwnd =>
                 Edit_Window_Handle);

 	   return (0);

	 when Win32.Winuser.Wm_Size =>

           Bool_Result :=
             Win32.Winuser.Movewindow
               (Hwnd     =>
                 Edit_Window_Handle,
                X        =>
                 0,
                Y        =>
                 0,
                Nwidth   =>
                 Win32.Int
                   (Win32.Utils.Loword
                     (Win32.Dword
                       (Lparam))),
                Nheight  =>
                 Win32.Int
                   (Win32.Utils.Hiword
                     (Win32.Dword
                       (Lparam))),
                Brepaint =>
                 True);

 	   return (0);

	 when Win32.Winuser.Wm_Initmenupopup =>

           if Lparam = 1 then

             if
               Win32.Winuser.Sendmessage
                 (Hwnd   =>
                   Edit_Window_Handle,
                  Msg    =>
                   Win32.Winuser.Em_Canundo,
                  Wparam =>
                   0,
                  Lparam =>
                   0) /= 0 then
      
               Bool_Result :=
                 Win32.Winuser.Enablemenuitem
                   (Hmenu         =>
                     Convert_Wparam_To_Hmenu
                       (S =>
                         Wparam),
                    Uidenableitem =>
                     Win32.Uint
                       (Poppad2.Idm_Undo),
                    Uenable       =>
                     Win32.Winuser.Mf_Enabled);

             else

               Bool_Result :=
                 Win32.Winuser.Enablemenuitem
                   (Hmenu         =>
                     Convert_Wparam_To_Hmenu
                       (S =>
                         Wparam),
                    Uidenableitem =>
                     Win32.Uint
                       (Poppad2.Idm_Undo),
                    Uenable       =>
                     Win32.Winuser.Mf_Grayed);
     
             end if;

             if
               Win32.Winuser.Isclipboardformatavailable
                  (Format =>
                   Win32.Winuser.Cf_Text) = True then
      
               Bool_Result :=
                 Win32.Winuser.Enablemenuitem
                   (Hmenu         =>
                     Convert_Wparam_To_Hmenu
                       (S =>
                         Wparam),
                    Uidenableitem =>
                     Win32.Uint
                       (Poppad2.Idm_Paste),
                    Uenable       =>
                     Win32.Winuser.Mf_Enabled);

             else

               Bool_Result :=
                 Win32.Winuser.Enablemenuitem
                   (Hmenu         =>
                     Convert_Wparam_To_Hmenu
                       (S =>
                         Wparam),
                    Uidenableitem =>
                     Win32.Uint
                       (Poppad2.Idm_Paste),
                    Uenable       =>
                     Win32.Winuser.Mf_Grayed);
     
             end if;

             Selection :=         
               Win32.Winuser.Sendmessage
                 (Hwnd   =>
                   Edit_Window_Handle,
                  Msg    =>
                   Win32.Winuser.Em_Getsel,
                  Wparam =>
                   0,
                  Lparam =>
                   0);

             if
               Win32.Utils.Hiword
                 (Win32.Dword
                   (Selection)) =
               Win32.Utils.Loword
                 (Win32.Dword
                   (Selection)) then

               Is_Enabled :=
                 Win32.Winuser.Mf_Grayed;

             else

               Is_Enabled :=
                 Win32.Winuser.Mf_Enabled;

             end if;

             Bool_Result :=
               Win32.Winuser.Enablemenuitem
                (Hmenu         =>
                  Convert_Wparam_To_Hmenu
                    (S =>
                      Wparam),
                  Uidenableitem =>
                    Win32.Uint
                      (Poppad2.Idm_Cut),
                  Uenable       =>
                    Is_Enabled);
     
             Bool_Result :=
               Win32.Winuser.Enablemenuitem
                (Hmenu         =>
                  Convert_Wparam_To_Hmenu
                    (S =>
                      Wparam),
                  Uidenableitem =>
                    Win32.Uint
                      (Poppad2.Idm_Copy),
                  Uenable       =>
                    Is_Enabled);
     
             Bool_Result :=
               Win32.Winuser.Enablemenuitem
                (Hmenu         =>
                  Convert_Wparam_To_Hmenu
                    (S =>
                      Wparam),
                  Uidenableitem =>
                    Win32.Uint
                      (Poppad2.Idm_Del),
                  Uenable       =>
                    Is_Enabled);
     
           end if;

 	   return (0);

	 when Win32.Winuser.Wm_Command =>

           if
             Lparam /= 0 then

             if

               Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam)) = 1 AND
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Wparam)) = Win32.Winuser.En_Errspace or
                Win32.Utils.Hiword
                 (Win32.Dword
                   (Wparam)) = Win32.Winuser.En_Maxtext) then

               Integer_Result :=
                 Win32.Winuser.Messagebox
                   (Hwnd      =>
                     Window_Handle,
                    Lptext    =>
                     Convert_Chars_Ptr_To_Lpcstr
                       (S =>
                         Space_Text),
                    Lpcaption =>
                     Convert_Chars_Ptr_To_Lpcstr
                       (S =>
                         Windows.Data.Class_Name),
                    Utype     =>
                     Win32.Uint
                       (Win32.Winuser.Mb_Ok) or
                     Win32.Uint
                       (Win32.Winuser.Mb_Iconstop));

             end if;

           else 

             case
               Win32.Int
                 (Win32.Utils.Loword
                   (Win32.Dword
                     (Wparam))) is

               when
                 Poppad2.Idm_New    |
                 Poppad2.Idm_Open   |
                 Poppad2.Idm_Save   |
                 Poppad2.Idm_Saveas |
                 Poppad2.Idm_Print  =>
     
                 Bool_Result :=
                   Win32.Winuser.Messagebeep
                     (Utype =>
                       0);
              
               when
                 Poppad2.Idm_Exit  =>

                 Selection :=         
                   Win32.Winuser.Sendmessage
                     (Hwnd   =>
                       Window_Handle,
                      Msg    =>
                       Win32.Winuser.Wm_Close,
                      Wparam =>
                       0,
                      Lparam =>
                       0);

               when
                 Poppad2.Idm_Undo  =>

                 Selection :=         
                   Win32.Winuser.Sendmessage
                     (Hwnd   =>
                       Edit_Window_Handle,
                      Msg    =>
                       Win32.Winuser.Wm_Undo,
                      Wparam =>
                       0,
                      Lparam =>
                       0);

               when
                 Poppad2.Idm_Cut  =>

                 Selection :=         
                   Win32.Winuser.Sendmessage
                     (Hwnd   =>
                       Edit_Window_Handle,
                      Msg    =>
                       Win32.Winuser.Wm_Cut,
                      Wparam =>
                       0,
                      Lparam =>
                       0);

               when
                 Poppad2.Idm_Copy  =>

                 Selection :=         
                   Win32.Winuser.Sendmessage
                     (Hwnd   =>
                       Edit_Window_Handle,
                      Msg    =>
                       Win32.Winuser.Wm_Copy,
                      Wparam =>
                       0,
                      Lparam =>
                       0);

               when
                 Poppad2.Idm_Paste  =>

                 Selection :=         
                   Win32.Winuser.Sendmessage
                     (Hwnd   =>
                       Edit_Window_Handle,
                      Msg    =>
                       Win32.Winuser.Wm_Paste,
                      Wparam =>
                       0,
                      Lparam =>
                       0);

               when
                 Poppad2.Idm_Del  =>

                 Selection :=         
                   Win32.Winuser.Sendmessage
                     (Hwnd   =>
                       Edit_Window_Handle,
                      Msg    =>
                       Win32.Winuser.Wm_Clear,
                      Wparam =>
                       0,
                      Lparam =>
                       0);

               when
                 Poppad2.Idm_Selall  =>

                 Selection :=         
                   Win32.Winuser.Sendmessage
                     (Hwnd   =>
                       Edit_Window_Handle,
                      Msg    =>
                       Win32.Winuser.Em_Setsel,
                      Wparam =>
                       0,
                      Lparam =>
                       -1);

               when
                 Poppad2.Idm_Help  =>

                 Integer_Result :=
                   Win32.Winuser.Messagebox
                     (Hwnd      =>
                       Window_Handle,
                      Lptext    =>
                       Convert_Chars_Ptr_To_Lpcstr
                         (S =>
                           Help_Text),
                      Lpcaption =>
                       Convert_Chars_Ptr_To_Lpcstr
                         (S =>
                           Windows.Data.Class_Name),
                      Utype     =>
                       Win32.Uint
                         (Win32.Winuser.Mb_Ok) or
                       Win32.Uint
                          (Win32.Winuser.Mb_Iconstop));

               when
                 Poppad2.Idm_About  =>

                 Integer_Result :=
                   Win32.Winuser.Messagebox
                     (Hwnd      =>
                       Window_Handle,
                      Lptext    =>
                       Convert_Chars_Ptr_To_Lpcstr
                         (S =>
                           About_Text),
                      Lpcaption =>
                       Convert_Chars_Ptr_To_Lpcstr
                         (S =>
                           Windows.Data.Class_Name),
                      Utype     =>
                       Win32.Uint
                         (Win32.Winuser.Mb_Ok) or
                       Win32.Uint
                         (Win32.Winuser.Mb_Iconinformation));

               when
                 others              =>
     
                   return
                     Win32.Winuser.Defwindowproc
	               (Hwnd   =>
                         Window_Handle, 
	                Msg    => 
                         Message, 
	                Wparam => 
                         Wparam,
	                Lparam => 
                         Lparam);

             end case;
           
           end if;

 	   return (0);

	 when Win32.WinUser.Wm_Close =>

           if
             Win32.Winuser.Idyes =
             Ask_Confirmation
               (Window_Handle =>
                 Window_Handle) then

             Bool_Result :=
               Win32.Winuser.Destroywindow
                 (Hwnd =>
                   Window_Handle);

           end if;

 	   return (0);

	 when Win32.WinUser.Wm_Queryendsession =>

           if
             Win32.Winuser.Idyes =
             Ask_Confirmation
               (Window_Handle =>
                 Window_Handle) then

  	     return (1);

           else

  	     return (0);

           end if;

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground =>
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Convert_Chars_Ptr_To_Lpcstr
               (S =>
                 Windows.Data.Class_Name),
	    Lpszclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.CLass_Name),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

     Bool_Result := 
       Win32.Winuser.Showwindow
         (Hwnd     => 
           Windows.Data.Window_Handle, 
	  Ncmdshow =>
           Win32.Winmain.Get_Ncmdshow);

     Bool_Result := 
       Win32.Winuser.Updatewindow
         (Hwnd => 
           Windows.Data.Window_Handle);

     Accelerator_Handle :=
       Win32.Winuser.Loadaccelerators
         (Hinstance =>
           Instance,
          Lptablename =>
           Convert_Chars_Ptr_To_Lpcstr
             (S =>
               Windows.Data.Class_Name));

     Windows.Routines.Main_Windows_Message_Loop;

     return Message.Wparam;

  end Main;

end Windows.Routines;
