With Ada.Unchecked_Conversion;
with Win32.Wingdi;
with Win32.Utils;
with Windows.Data;
package body Windows.Child_Routines is
  use type Win32.Int;
  use type Win32.Uint;
  use type Win32.Wparam;

  True                        : constant Win32.Bool := 1;

  False                       : constant Win32.Bool := 0;

  Rectangle                   : aliased Win32.Windef.Rect;

  Color                       : Win32.Int;

  Figure                      : Win32.Int;

  Control_Block_Window_Handle : Win32.Windef.Hwnd;

  Color_Array                 : array(0 .. 7) of Win32.Windef.Colorref :=
    (0 =>
      Win32.Wingdi.Rgb
        (Bred   =>
          0,
         Bgreen =>
          0,
         Bblue  =>
          0),
     1 =>
      Win32.Wingdi.Rgb
        (Bred   =>
          0,
         Bgreen =>
          0,
         Bblue  =>
          255),
     2 =>
      Win32.Wingdi.Rgb
        (Bred   =>
          0,
         Bgreen =>
          255,
         Bblue  =>
          0),
     3 =>
      Win32.Wingdi.Rgb
        (Bred   =>
          0,
         Bgreen =>
          255,
         Bblue  =>
          255),
     4 =>
      Win32.Wingdi.Rgb
        (Bred   =>
          255,
         Bgreen =>
          0,
         Bblue  =>
          0),
     5 =>
      Win32.Wingdi.Rgb
        (Bred   =>
          255,
         Bgreen =>
          0,
         Bblue  =>
          255),
     6 =>
      Win32.Wingdi.Rgb
        (Bred   =>
          255,
         Bgreen =>
          255,
         Bblue  =>
          0),
     7 =>
      Win32.Wingdi.Rgb
        (Bred   =>
          255,
         Bgreen =>
          255,
         Bblue  =>
          255));

  Bool_Result                 : Win32.Bool;

  Integer_Result              : Win32.Int;

  Window_Handle_Result        : Win32.Windef.Hwnd;

  procedure Paint_The_Block
    (Window_Handle : Win32.Windef.Hwnd;
     Color         : Win32.Int;
     Figure        : Win32.Int) is

    function Convert_Integer_To_Ac_Rect_t is new
      Ada.Unchecked_Conversion
        (Source =>
          Integer,
         Target =>
          Win32.Wingdi.Ac_Rect_T);

  begin

    Bool_Result :=
      Win32.Winuser.Invalidaterect
        (Hwnd =>
          Window_Handle,
         Lprect =>
          Convert_Integer_To_Ac_Rect_T
            (S =>
              0),
         Berase =>
          True);

    Bool_Result :=
      Win32.Winuser.Updatewindow
        (Hwnd =>
          Window_Handle);

    Paint_The_Window
      (Window_Handle =>
        Window_Handle,
       Color         =>
        Color,
       Figure        =>
        Figure);
   
  end;

  procedure Paint_The_Window
    (Window_Handle : Win32.Windef.Hwnd;
     Color         : Win32.Int;
     Figure        : Win32.Int) is

    Brush_Handle          : Win32.Windef.Hbrush;

    Device_Context_Handle : Win32.Windef.Hdc;

  begin
    
    Device_Context_Handle :=
      Win32.Winuser.Getdc
        (Hwnd =>
          Window_Handle);

    Bool_Result :=
      Win32.Winuser.Getclientrect
        (Hwnd   =>
          Window_Handle,
         Lprect =>
          Rectangle'Access);

    Brush_Handle :=
      Win32.Wingdi.Createsolidbrush
        (Clrref =>
          Color_Array(Integer(Color) - Integer(About2.Idd_Black)));

    Brush_Handle :=
      Win32.Wingdi.Selectobject
        (Hdc     =>
          Device_Context_Handle,
         Hgdiobj =>
          Brush_Handle);
     
    if

      Figure =
      About2.Idd_Rect then

      Bool_Result :=
        Win32.Wingdi.Rectangle
          (Hdc         =>
            Device_Context_Handle,
           Nleftrect   =>
            Win32.Int
              (Rectangle.Left),
           Ntoprect    =>
            Win32.Int
              (Rectangle.Top),
           Nrightrect  =>
            Win32.Int
              (Rectangle.Right),
           Nbottomrect =>
            Win32.Int
              (Rectangle.Bottom));      

    else

      Bool_Result :=
        Win32.Wingdi.Ellipse
          (Hdc         =>
            Device_Context_Handle,
           Nleftrect   =>
            Win32.Int
              (Rectangle.Left),
           Ntoprect    =>
            Win32.Int
              (Rectangle.Top),
           Nrightrect  =>
            Win32.Int
              (Rectangle.Right),
           Nbottomrect =>
            Win32.Int
              (Rectangle.Bottom));      

    end if;

    Bool_Result :=
      Win32.Wingdi.Deleteobject
        (Hgdiobj =>
          Win32.Wingdi.Selectobject
            (Hdc     =>
              Device_Context_Handle,
             Hgdiobj =>
              Brush_Handle)); 

    Integer_Result :=
      Win32.Winuser.Releasedc
        (Hwnd =>
          Window_Handle,
         Hdc  =>
          Device_Context_Handle);

  end;

  function About_Dialog
    (Dialog_Handle : Win32.Windef.Hwnd;
     Message       : Win32.Uint;
     Wparam        : Win32.Wparam;
     Lparam        : Win32.Lparam) return Win32.Bool is
  begin

    case
      Message is

      when
        Win32.Winuser.Wm_Initdialog =>
       
          Color :=
            Current_Color;

          Figure :=
            Current_Figure;

          Bool_Result :=
            Win32.Winuser.Checkradiobutton
              (Hdlg           =>
                Dialog_Handle,
               Nidfirstbutton =>
                About2.Idd_Black,
               Nidlastbutton  =>
                About2.Idd_White,
               Nidcheckbutton =>
                Color);

          Bool_Result :=
            Win32.Winuser.Checkradiobutton
              (Hdlg           =>
                Dialog_Handle,
               Nidfirstbutton =>
                About2.Idd_Rect,
               Nidlastbutton  =>
                About2.Idd_Ell,
               Nidcheckbutton =>
                Figure);

          Control_Block_Window_Handle :=
            Win32.Winuser.Getdlgitem
              (Hdlg       =>
                Dialog_Handle,
               Niddlgitem =>
                About2.Idd_Paint);

          Window_Handle_Result :=
            Win32.Winuser.Setfocus
              (Hwnd =>
                Win32.Winuser.Getdlgitem
                  (Hdlg       =>
                    Dialog_Handle,
                   Niddlgitem =>
                    Color));

        return False;

      when
        Win32.Winuser.Wm_Command    =>

        case
          Win32.Int
            (Win32.Utils.Loword
              (Win32.Dword
                (Wparam))) is
         
          when
            Win32.Winuser.Idok     =>
       
            Current_Color :=
              Color;

            Current_Figure :=
              Figure;

            Bool_Result :=
              Win32.Winuser.Enddialog
                (Hdlg    =>
                  Dialog_Handle,
                 Nresult =>
                  1);

            return True;

          when
            Win32.Winuser.Idcancel =>
        
            Bool_Result :=
              Win32.Winuser.Enddialog
                (Hdlg    =>
                  Dialog_Handle,
                 Nresult =>
                  0);

            return (1);

          when
            About2.Idd_Black   | 
            About2.Idd_Red     | 
            About2.Idd_Green   | 
            About2.Idd_Yellow  | 
            About2.Idd_Blue    |
            About2.Idd_Magenta | 
            About2.Idd_Cyan    | 
            About2.Idd_White   =>
        
            Color :=
              Win32.Int
                (Win32.Utils.Loword
                   (Win32.Dword
                     (Wparam)));

            Bool_Result :=
              Win32.Winuser.Checkradiobutton
                (Hdlg           =>
                  Dialog_Handle,
                 Nidfirstbutton =>
                  About2.Idd_Black,
                 Nidlastbutton  =>
                  About2.Idd_White,
                 Nidcheckbutton =>
                  Win32.Int
                    (Win32.Utils.Loword
                      (Win32.Dword
                        (Wparam))));

            Paint_The_Block
              (Window_Handle =>
                Control_Block_Window_Handle,
               Color         =>
                Color,
               Figure        =>
                Figure);

            return True;

          when
            About2.Idd_Rect | 
            About2.Idd_Ell  =>

            Figure :=
              Win32.Int
                (Wparam);
       
            Bool_Result :=
              Win32.Winuser.Checkradiobutton
                (Hdlg           =>
                  Dialog_Handle,
                 Nidfirstbutton =>
                  About2.Idd_Rect,
                 Nidlastbutton  =>
                  About2.Idd_Ell,
                 Nidcheckbutton =>
                  Win32.Int
                    (Win32.Utils.Loword
                      (Win32.Dword
                        (Wparam))));

            Paint_The_Block
              (Window_Handle =>
                Control_Block_Window_Handle,
               Color         =>
                Color,
               Figure        =>
                Figure);

            return True;

          when
            others =>

            return False;

        end case;

      when
        Win32.Winuser.Wm_Paint      =>

        Paint_The_Block
          (Window_Handle =>
            Control_Block_Window_Handle,
           Color         =>
            Color,
           Figure        =>
            Figure);

        return False;

      when
        others =>

        return False;

    end case;      

    return False;

  end About_Dialog;

end Windows.Child_Routines;

