With About2;
with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Winbase;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Child_Routines;
with Windows.Data;
package body Windows.Routines is
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int; 
  use type Win32.Long;

   Paint_Record          : aliased win32.Winuser.Paintstruct;

   About_Text            : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str => "AboutBox");

   Window_Class          : aliased Win32.Winuser.Wndclass;

   Message               : aliased Win32.Winuser.Msg;

   True                  : constant Win32.Bool := 1;

   Instance              : Win32.Windef.Hinstance;

   Integer_Result        : Win32.Int;
     
   Dword_Result          : Win32.Dword;

   Uint_Result           : Win32.Uint;

   Bool_Result           : Win32.Bool;
                     
   Long_Result           : Win32.Long;

   Atom_Result           : Win32.Windef.Atom;

   Device_Context_Result : Win32.Windef.Hdc;

   function Convert_Chars_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source =>
         Interfaces.C.Strings.Chars_Ptr,
        Target =>
         Win32.Lpcstr);

   function Convert_Lparam_To_Lpcreatestruct is new
     Ada.Unchecked_Conversion
       (Source =>
         Win32.Lparam,
        Target =>
         Win32.Winuser.Lpcreatestruct);

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is

     function Convert_Integer_To_Ac_Rect_t is new
       Ada.Unchecked_Conversion
         (Source =>
           Integer,
          Target =>
           Win32.Wingdi.Ac_Rect_T);

   begin

      case Message is

	 when
           Win32.Winuser.Wm_Create =>

           Instance := 
             Convert_Lparam_To_Lpcreatestruct
               (S =>
                 Lparam).Hinstance; 

 	   return (0);

	 when
           Win32.Winuser.Wm_Command =>
      
           case 
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Wparam))) is

             when
               About2.Idm_About =>

               if
                 Win32.Winuser.Dialogbox
                   (Hinstance      =>
                     Instance,
                    Lptemplatename =>
                     Convert_Chars_Ptr_To_Lpcstr
                       (S =>
                         About_Text),
                    Hwndparent     =>
                     Window_Handle,
                    Lpdialogfunc   =>
                     Windows.Child_Routines.About_Dialog'Access) /= 0 then

               Bool_Result :=
                 Win32.Winuser.Invalidaterect
                   (Hwnd =>
                     Window_Handle,
                    Lprect =>
                     Convert_Integer_To_Ac_Rect_T
                       (S =>
                         0),
                    Berase =>
                     True);

               end if;

     	       return (0);

             when
               others =>
           
               return
                 Win32.Winuser.Defwindowproc
	           (Hwnd   =>
                     Window_Handle, 
	            Msg    => 
                     Message, 
	            Wparam => 
                     Wparam,
	            Lparam => 
                     Lparam);

           end case;

	 when
           Win32.WinUser.Wm_Paint   =>

	   Device_Context_Result := 
             Win32.Winuser.Beginpaint
	       (Hwnd    => 
                 Window_Handle,
	        Lppaint => 
                 Paint_Record'Access);

	   Bool_Result := 
             Win32.Winuser.Endpaint
	       (Hwnd    => 
                 Window_Handle,
	        Lppaint => 
                 Paint_Record'Access);

           Windows.Child_Routines.Paint_The_Window
             (Window_Handle =>
               Window_Handle,
              Color         =>
               Windows.Child_Routines.Current_Color,
              Figure        =>
               Windows.Child_Routines.Current_Figure);

	   return (0);


	 when
           Win32.WinUser.Wm_Destroy =>

	   Win32.WinUser.Postquitmessage
	     (Nexitcode => 
              0);

	   return (0);

	 when
           others =>

	   return
             Win32.Winuser.Defwindowproc
	       (Hwnd   =>
                 Window_Handle, 
	        Msg    => 
                 Message, 
	        Wparam => 
                 Wparam,
	        Lparam => 
                 Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Win32.Winuser.Loadicon
               (Hinstance => 
                 Instance,
                Lpiconname =>
                 Convert_Chars_Ptr_To_Lpcstr
                   (S =>
                     Windows.Data.Class_Name)),
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground =>
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Convert_Chars_Ptr_To_Lpcstr
               (S =>
                 Windows.Data.Class_Name),
	    Lpszclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Chars_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
