with Interfaces.C.Strings;
With Ada.Unchecked_Conversion;
with Win32.Wingdi;
with Win32.Utils;
with Windows.Data;
package body Windows.Child_Routines is
  use type Win32.Int;
  use type Win32.Uint;
  use type Win32.Wparam;

  Parent_Window_Handle  : Win32.Windef.Hwnd;

  True                        : constant Win32.Bool := 1;

  False                       : constant Win32.Bool := 0;

  Color_Array                 : array(0 .. 2) of Win32.Int;

  Bool_Result                 : Win32.Bool;

  Integer_Result              : Win32.Int;

  Long_Result                 : Win32.Long;

  function Color_Scroll
    (Dialog_Handle : Win32.Windef.Hwnd;
     Message       : Win32.Uint;
     Wparam        : Win32.Wparam;
     Lparam        : Win32.Lparam) return Win32.Bool is

    Control_Window_Handle : Win32.Windef.Hwnd;

    Control_Id            : Win32.Int;

    Index                 : Integer;

   function Convert_Dword_To_Hgdiobj is new
     Ada.Unchecked_Conversion
       (Source =>
         Win32.Dword,
        Target =>
         Win32.Windef.Hgdiobj);

   function Convert_Hbrush_To_Long is new
     Ada.Unchecked_Conversion
       (Source =>
         Win32.Windef.Hbrush,
        Target =>
         Win32.Long);

   function Convert_Lparam_To_Hwnd is new
     Ada.Unchecked_Conversion
       (Source =>
         Win32.Lparam,
        Target =>
         Win32.Windef.Hwnd);

    function Min
      (L,R : Win32.Int) return Win32.Int is
    begin

      if L < R then

        return L;

      else

        return R;

      end if;

    end Min;

    function Max
      (L,R : Win32.Int) return Win32.Int is
    begin

      if L > R then

        return L;

      else

        return R;

      end if;

    end Max;

  begin

    case
      Message is

      when
        Win32.Winuser.Wm_Initdialog =>

        for i in 10 .. 12 loop

          Control_Window_Handle :=
            Win32.Winuser.Getdlgitem
              (Hdlg       =>
                Dialog_Handle,
               Niddlgitem =>
                Win32.Int
                  (i));

          Bool_Result :=
            Win32.Winuser.Setscrollrange
              (Hwnd    =>
                Control_Window_Handle,
               Nbar    =>
                Win32.Winuser.Sb_Ctl,
               Nminpos =>
                0,
               NMaxpos =>
                255,
               Bredraw =>
                False);

        Integer_Result :=
          Win32.Winuser.Setscrollpos
            (Hwnd    =>
              Control_Window_Handle,
             Nbar    =>
              Win32.Winuser.Sb_Ctl,
             Npos    =>
              0,
             Bredraw =>
              False);

        end loop;

        return True;

      when
        Win32.Winuser.Wm_Vscroll    =>

        Control_Window_Handle :=
          Convert_Lparam_To_Hwnd
            (S =>
              Lparam);

        Control_Id :=
          Win32.Int
            (Win32.Winuser.Getwindowlong
              (Hwnd   =>
                Control_Window_Handle,
               Nindex =>
                Win32.Winuser.Gwl_Id));

        Index :=
          Integer
            (Control_Id - 10);

        Parent_Window_Handle :=
          Win32.Winuser.Getparent
            (Hwnd =>
              Dialog_Handle);

        case
          Win32.Utils.Loword
            (Win32.Dword
              (Wparam)) is

          when
            Win32.Winuser.Sb_Pagedown      =>

            Color_Array(Index) :=
              Color_Array(Index) + 15;

            Color_Array(Index) :=
              Min
                (L =>
                  255,
                 R =>
                  Color_Array(Index) + 1);            
            
          when
            Win32.Winuser.Sb_Linedown =>

            Color_Array(Index) :=
              Min
                (L =>
                  255,
                 R =>
                  Color_Array(Index) + 1);            
            
          when
            Win32.Winuser.Sb_Pageup        =>

            Color_Array(Index) :=
              Color_Array(Index) - 15;

            Color_Array(Index) :=
              Max
                (L =>
                  0,
                 R =>
                  Color_Array(Index) - 1);            
            
          when
            Win32.Winuser.Sb_Lineup   =>

            Color_Array(Index) :=
              Max
                (L =>
                  0,
                 R =>
                  Color_Array(Index) - 1);            
            
          when
            Win32.Winuser.Sb_Top           =>

            Color_Array(Index) :=
              0;

          when
            Win32.Winuser.Sb_Bottom        =>

            Color_Array(Index) :=
              255;

          when
            Win32.Winuser.Sb_Thumbposition |
            Win32.Winuser.Sb_Thumbtrack    =>

            Color_Array(Index) :=
              Win32.Int
                (Win32.Utils.Hiword
                  (Win32.Dword
                    (Wparam)));

          when
            others                   =>

            return False;

        end case;

        Integer_Result :=
          Win32.Winuser.Setscrollpos
            (Hwnd    =>
              Control_Window_Handle,
             Nbar    =>
              Win32.Winuser.Sb_Ctl,
             Npos    =>
              Color_Array(Index),
             Bredraw =>
              True);

        Bool_Result :=
          Win32.Winuser.Setdlgitemint
            (Hdlg       =>
              Dialog_Handle,
             Niddlgitem =>
              Control_Id + 3,
             Uvalue     =>
              Win32.Uint
                (Color_Array(Index)),
             Bsigned    =>
              False);

        Bool_Result :=
          Win32.Wingdi.Deleteobject
            (Hgdiobj =>
              Convert_Dword_To_Hgdiobj
                (S =>
                  Win32.Winuser.Setclasslong
                   (Hwnd      =>
                     Parent_Window_Handle,
                    Nindex    =>
                     Win32.Winuser.Gcl_Hbrbackground,
                    Dwnewlong =>
                     Convert_Hbrush_To_Long
                       (S =>
                         Win32.Wingdi.Createsolidbrush
                           (Clrref =>
                             Win32.Wingdi.Rgb
                               (Bred   =>
                                 Win32.Byte
                                   (Color_Array(0)),
                                Bgreen =>
                                 Win32.Byte
                                   (Color_Array(1)),
                                Bblue  =>
                                 Win32.Byte
                                   (Color_Array(2))))))));
              
        Bool_Result :=
          Win32.Winuser.Invalidaterect
            (Hwnd =>
              Parent_Window_Handle,
             Lprect =>
              null,
             Berase =>
              True);

      when
        others =>

        null;

    end case;      

    return False;

  end;

end Windows.Child_Routines;

