with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
With System;
with Win32.Strings;
with Win32.Winuser;
package body Windows.Pop.Find is
  use type Win32.Dword;
  use type Win32.Int;
  use type Win32.Pstr;

  Maximum_String_Length : constant integer := 256;

  Temp_String           : String(1 .. Maximum_String_Length) :=(others => ' ');

  Find_Text_String      : Interfaces.C.Strings.Chars_Ptr :=
    Interfaces.C.Strings.New_String
      (Str =>
        Temp_String);

  Replace_Text_String   : Interfaces.C.Strings.Chars_Ptr :=
    Interfaces.C.Strings.New_String
      (Str =>
        Temp_String);

  Fr                    : aliased Win32.Commdlg.Findreplace;

  Integer_Result        : Win32.Int;

  Long_Result           : Win32.Lresult;

  function Convert_Chars_Ptr_To_Lpstr is new
    Ada.Unchecked_Conversion
      (Source =>
        Interfaces.C.Strings.Chars_Ptr,
       Target =>
        Win32.Lpstr);

  function
    Find_Dialog
      (Window_Handle :
        Win32.Windef.Hwnd)
      return Win32.Windef.Hwnd is

  begin

    Fr.Lstructsize      :=
      Win32.Dword
        (Win32.Commdlg.Findreplace'Size/8);

    Fr.Hwndowner        :=
      Window_Handle;

    Fr.Hinstance        :=
      null;

    Fr.Flags            :=
      Win32.Dword
        (Win32.Commdlg.Fr_Hideupdown)    or
      Win32.Dword
        (Win32.Commdlg.Fr_Hidematchcase) or
      Win32.Dword
        (Win32.Commdlg.Fr_Hidewholeword);

    Fr.Lpstrfindwhat    :=
      Convert_Chars_Ptr_To_Lpstr
        (S =>
          Find_Text_String);

    Fr.Lpstrreplacewith :=
      null;

    Fr.Wfindwhatlen      :=
      Win32.Word
        (Interfaces.C.Strings.Strlen
          (Item =>
            Find_Text_String));

    Fr.Wreplacewithlen   :=
      0;

    Fr.Lcustdata         :=
      0;

    Fr.Lpfnhook          :=
      null;

    Fr.Lptemplatename    :=
      null;

    return
      Win32.Commdlg.Findtext
        (Lpfr =>
          Fr'Access);

  end;

  function
    Replace_Dialog
      (Window_Handle :
        Win32.Windef.Hwnd)
      return Win32.Windef.Hwnd is

  begin

    Fr.Lstructsize      :=
      Win32.Dword
        (Win32.Commdlg.Findreplace'Size/8);

    Fr.Hwndowner        :=
      Window_Handle;

    Fr.Hinstance        :=
      null;

    Fr.Flags            :=
      Win32.Dword
        (Win32.Commdlg.Fr_Hideupdown)    or
      Win32.Dword
        (Win32.Commdlg.Fr_Hidematchcase) or
      Win32.Dword
        (Win32.Commdlg.Fr_Hidewholeword);

    Fr.Lpstrfindwhat    :=
      Convert_Chars_Ptr_To_Lpstr
        (S =>
          Find_Text_String);

    Fr.Lpstrreplacewith :=
      Convert_Chars_Ptr_To_Lpstr
        (S =>
          Replace_Text_String);

    Fr.Wfindwhatlen      :=
      Win32.Word
        (Interfaces.C.Strings.Strlen
          (Item =>
            Find_Text_String));

    Fr.Wreplacewithlen   :=
      Win32.Word
        (Interfaces.C.Strings.Strlen
          (Item =>
            Replace_Text_String));

    Fr.Lcustdata         :=
      0;

    Fr.Lpfnhook          :=
      null;

    Fr.Lptemplatename    :=
      null;

    return
      Win32.Commdlg.Replacetext
        (Lpfr =>
          Fr'Access);

  end;

  function
    Text
      (Window_Handle :
        Win32.Windef.Hwnd;
       Search_Offset :
        Int_Ptr;
       Pfr           :
        Win32.Commdlg.Lpfindreplace)
      return Boolean is
 
    Length   : Win32.Int :=
      Win32.Winuser.Getwindowtextlength
        (Hwnd =>
          Window_Handle);

    Temp_String      : String(1 .. Integer(Length) + 1) := (others => ' ');

    Document_Pointer : Interfaces.C.Strings.Chars_Ptr :=
      Interfaces.C.Strings.New_String
        (Str =>
          Temp_String);

    Position         : Win32.Int;

    Temp_Offset      : Win32.Int;

    Pos              : Win32.Pstr;
    
    function Convert_Chars_Ptr_To_Int is new
      Ada.Unchecked_Conversion
        (Source =>
          Interfaces.C.Strings.Chars_Ptr,
         Target =>
          Win32.Int);

    function Convert_Lpstr_To_Chars_Ptr is new
      Ada.Unchecked_Conversion
        (Source =>
          Win32.Lpstr,
         Target =>
          Interfaces.C.Strings.Chars_Ptr);

    function Convert_Pstr_To_Int is new
      Ada.Unchecked_Conversion
        (Source =>
          Win32.Pstr,
         Target =>
          Win32.Int);

  begin

    Integer_Result :=
      Win32.Winuser.Getwindowtext
        (Hwnd      =>
          Window_Handle,
         Lpstring  =>
          Convert_Chars_Ptr_To_Lpstr
            (S =>
              Document_Pointer),
         Nmaxcount =>
          Length + 1);
    
    Pos :=
      Win32.Strings.Strstr
        (String1 =>
          Win32.Pcstr(Pfr.Lpstrfindwhat),  
         String2 =>
          Win32.Pcstr
            (Pfr.Lpstrfindwhat));

    if
      Pos = null then

      return
        False;

    else

      Position :=
        Convert_Pstr_To_Int
          (S =>
            Pos) -
        Convert_Chars_Ptr_To_Int
          (S =>
            Document_Pointer);
 
      Interfaces.C.Strings.Free
        (Item =>
          Document_Pointer);

    end if;  

    Search_Offset.all :=
      Convert_Pstr_To_Int
        (S =>
          Pos) -
      Win32.Int
        (Interfaces.C.Strings.Strlen
          (Item =>
            Convert_Lpstr_To_Chars_Ptr
              (S =>
                Pfr.Lpstrfindwhat)));

    Temp_Offset :=
      Search_Offset.all;

    Long_Result :=
      Win32.Winuser.Sendmessage
        (Hwnd   =>
          Window_Handle,
         Msg    =>
          Win32.Winuser.Em_Setsel,
         Wparam =>
          Win32.Wparam
            (Position),
         Lparam =>
          Win32.Lparam
            (Temp_Offset));  

    Long_Result :=
      Win32.Winuser.Sendmessage
        (Hwnd   =>
          Window_Handle,
         Msg    =>
          Win32.Winuser.Em_Setsel,
         Wparam =>
          0,
         Lparam =>
          0);  

    return
      True;

  end;

  function
    Next_Text
      (Window_Handle :
        Win32.Windef.Hwnd;
       Search_Offset :
        Int_Ptr)
      return Boolean is

    Fr   : Win32.Commdlg.Findreplace;

    function Convert_Address_To_Lpfindreplace is new
      Ada.Unchecked_Conversion
        (Source =>
          System.Address,
         Target =>
          Win32.Commdlg.Lpfindreplace);

  begin

    Fr.Lpstrfindwhat    :=
      Convert_Chars_Ptr_To_Lpstr
        (S =>
          Find_Text_String);

    return
      Text
        (Window_Handle =>
          Window_Handle,
         Search_Offset =>
          Search_Offset,
         Pfr           =>
          Convert_Address_To_Lpfindreplace
            (S =>
              Fr'Address));

  end;

  function
    Replace_Text
      (Window_Handle :
        Win32.Windef.Hwnd;
       Search_Offset :
        Int_Ptr;
       Pfr           :
        Win32.Commdlg.Lpfindreplace)
      return Boolean is

    function Convert_Lpstr_To_Lparam is new
      Ada.Unchecked_Conversion
        (Source =>
          Win32.Lpstr,
         Target =>
          Win32.Lparam);

  begin

    if 
      Text
        (Window_Handle =>
          Window_Handle,
         Search_Offset =>
          Search_Offset,
         Pfr           =>
          Pfr) then

      Long_Result :=
        Win32.Winuser.Sendmessage
          (Hwnd   =>
            Window_Handle,
           Msg    =>
            Win32.Winuser.Em_Replacesel,
           Wparam =>
            0,
           Lparam =>
            Convert_Lpstr_To_Lparam
              (S =>
                Pfr.Lpstrreplacewith));  

      return
        True;

    else 

      return
        False;

    end if;

  end;

  function
    Valid
      return Boolean is

  begin

    if
      Interfaces.C.Strings.Value
        (Item =>
          Find_Text_String)(1) /= Character'Val(0) then

      return
        True;

    else

      return
        False;

    end if;

  end;

end;
        
