with Ada.Unchecked_Conversion;
with Win32.Commdlg;
with Win32.Wingdi;
with Win32.Winuser;
package body Windows.Pop.Font is
  use type Win32.Dword;
 
  Font_Handle    : aliased Win32.Windef.Hfont;

  Log_Font       : aliased Win32.Wingdi.Logfont;

  Cf             : aliased Win32.Commdlg.Choosefont;

  Rectangle      : aliased Win32.Windef.Rect;

  Bool_Result    : Win32.Bool;

  Integer_Result : Win32.Int;

  Long_Result    : Win32.Lresult;

  function
    Choose
      (Window_Handle :
        Win32.Windef.Hwnd)
      return Boolean is

    function Convert_Ac_Logfont_T_To_Wparam is new
      Ada.Unchecked_Conversion
        (Source =>
          Win32.Wingdi.Ac_Logfont_T,
         Target =>
          Win32.Wparam);

  begin

    Cf.LstructSize    :=
      Win32.Dword
        (Win32.Commdlg.Choosefont'Size/8);

    Cf.Hwndowner      :=
      Window_Handle;

    Cf.Hdc            :=
      null;

    Cf.Lplogfont      :=
      Log_Font'Access;

    Cf.Ipointsize     :=
      0;

    Cf.Flags          :=
      Win32.Dword
        (Win32.Commdlg.Cf_Inittologfontstruct) or
      Win32.Dword
        (Win32.Commdlg.Cf_Screenfonts)         or
      Win32.Dword
        (Win32.Commdlg.Cf_Effects);

    Cf.Rgbcolors      :=
      Win32.Wingdi.Rgb
        (Bred   =>
          0,
         Bgreen =>
          0,
         Bblue  =>
          0);
    Cf.lcustdata       :=
      0;

    Cf.Lpfnhook       :=
      null;

    Cf.Lptemplatename :=
      null;
    
    Cf.Hinstance      :=
      null;

    Cf.Lpszstyle      :=
      null;

    Cf.Nfonttype       :=
      0;

    Cf.Nsizemin        :=
      0;

    Cf.Nsizemax        :=
      0;

    Bool_Result :=
      Win32.Commdlg.Choosefont_Func
        (Lpcf =>
          Cf'Access);

    if
      Integer
        (Bool_Result) = 
      0 then
      
      return
        False;

    else

      return
        True;

    end if;

  end;

  procedure 
    Initialize
      (Window_Handle :
        Win32.Windef.Hwnd) is

    function Convert_Ac_Logfont_T_To_Lpvoid is new
      Ada.Unchecked_Conversion
        (Source =>
          Win32.Wingdi.Ac_Logfont_T,
         Target =>
          Win32.Lpvoid);

    function Convert_Ac_Logfont_T_To_Wparam is new
      Ada.Unchecked_Conversion
        (Source =>
          Win32.Wingdi.Ac_Logfont_T,
         Target =>
          Win32.Wparam);

  begin

    Integer_Result :=
      Win32.Wingdi.Getobject
        (Hgdiobj   =>
          Win32.Wingdi.Getstockobject
            (Fnobject =>
              Win32.Wingdi.System_Font),
         Cbbuffer  =>
          Win32.Int
            (Win32.Wingdi.Logfont'Size/8),
         Lpvobject =>
          Convert_Ac_Logfont_T_To_Lpvoid
            (S =>
              Log_Font'Access));

    Long_Result :=
      Win32.Winuser.Sendmessage
        (Hwnd =>
          Window_Handle,
         Msg  =>
          Win32.Winuser.Wm_Setfont,
         Wparam =>
          Convert_Ac_Logfont_T_To_Wparam
            (S =>
              Log_Font'Access),
         Lparam =>
          0);

  end;

  procedure
    Set
      (Window_Handle :
        Win32.Windef.Hwnd) is

    New_Font_Handle : Win32.Windef.Hfont;

    function Convert_Ac_Logfont_T_To_Ac_Textmetric_T is new
      Ada.Unchecked_Conversion
        (Source =>
          Win32.Wingdi.Ac_Logfont_T,
         Target =>
          Win32.Wingdi.Ac_Textmetric_T);

    function Convert_Hfont_To_Wparam is new
      Ada.Unchecked_Conversion
        (Source =>
          Win32.Windef.Hfont,
         Target =>
          Win32.Wparam);

  begin

    New_Font_Handle :=
      Win32.Wingdi.Createfontindirect
        (Lplf =>
          Convert_Ac_Logfont_T_To_Ac_Textmetric_T
            (S =>
               Log_Font'Access));

    Long_Result :=
      Win32.Winuser.Sendmessage
        (Hwnd =>
          Window_Handle,
         Msg  =>
          Win32.Winuser.Wm_Setfont,
         Wparam =>
          Convert_Hfont_To_Wparam
            (S =>
              New_Font_Handle),
         Lparam =>
          0);

    Bool_Result :=
      Win32.Wingdi.Deleteobject
        (Hgdiobj =>
          Font_Handle);

    Font_Handle.all :=
      New_Font_Handle.all;

    Bool_Result :=
      Win32.Winuser.Getclientrect
        (Hwnd   =>
          Window_Handle,
         Lprect =>
          Rectangle'Access);

    Bool_Result :=
      Win32.Winuser.Invalidaterect
        (Hwnd   =>
          Window_Handle,
         Lprect =>
          Rectangle'Access,
         Berase =>
          Win32.False);

  end;

  procedure
    Uninitialize is
  begin

    Bool_Result :=
      Win32.Wingdi.Deleteobject
        (Hgdiobj =>
          Font_Handle);
  end;

end;
        
