with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with System_Metrics;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
   use type System.Address;
   use type Win32.Dword;
   use type Win32.Bool;
   use type Win32.Byte;
   use type Win32.Int;
   use type Win32.Long;
   use type Win32.Uint;

   Text_Metric : aliased win32.Wingdi.Textmetric;

   Character_Width : win32.Long;

   Character_Height : win32.Long;

   Cap_Width : win32.Long;

   Temporary_String_Pointer : Interfaces.C.Strings.Chars_Ptr;

   Align_Top_Right : constant Win32.Uint := 
     Win32.Wingdi.Ta_Right or 
     Win32.Wingdi.Ta_Top;

   Align_Top_Left  : constant Win32.Uint := 
     Win32.Wingdi.Ta_Left or 
     Win32.Wingdi.Ta_Top;

   Vertical_Scroll_Position : win32.Uint := 0;

   Vertical_Position : win32.Uint := 0;

   True : constant Win32.bool := 1;

   False : constant Win32.bool := 0;

   Minimum_Vertical_Scroll_Position : constant Win32.Int := 0;

   Client_Height : win32.Long;

   Device_Context_Handle : aliased Win32.Windef.Hdc;

   Paint_Record : aliased win32.Winuser.Paintstruct;

   Window_Class : aliased Win32.Winuser.Wndclass;

   Message : aliased Win32.Winuser.Msg;

   Integer_Result        : Win32.Int;
   Bool_Result           : Win32.Bool;
   Long_Result           : Win32.Long;
   Atom_Result           : Win32.Windef.Atom;
   Uint_Result           : Win32.Uint;

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   procedure Convert_Integer_To_Char_Array
     (Int : in Integer;
      Ptr : in out Interfaces.C.Strings.Chars_Ptr) is

     Blank_String : string := "         ";

   begin

     Blank_String
       ((Blank_String'Length - integer'Image(Int)'Length) + 1 .. 
         Blank_String'Length) :=
          integer'Image(Int);

     Ptr := 
       Interfaces.C.Strings.New_String
         (Str => 
           Blank_String);

   end Convert_Integer_To_Char_Array;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => Message'Access,
	      Hwnd =>System.Null_Address, 
	      Wmsgfiltermin => Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;
   end Main_Windows_Message_Loop;

--   function Min is new Win32.Windef.Min(T => integer, "<" => "<" );

--   function Max is new Win32.Windef.Max(T => integer, ">" => ">" );

   function Min(l,r : integer) return integer is
   begin
     if l < r then
       return l;
     else
       return r;
     end if;
   end Min;

   function Max(l,r : integer) return integer is
   begin
     if l > r then
       return l;
     else
       return r;
     end if;
   end Max;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message        : Win32.Uint;
      Wparam         : Win32.Wparam;
      Lparam         : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

	    Device_Context_Handle := 
              Win32.Winuser.Getdc
               (Hwnd => 
                 Window_Handle);

	    Bool_Result :=
              Win32.Wingdi.Gettextmetrics
               (Hdc  => 
                 Device_Context_Handle,
                Lptm => 
                 Text_Metric'Access);

            Character_Width := 
              Text_Metric.Tmavecharwidth;

            if Text_Metric.Tmpitchandfamily mod 2 > 0 then

              Cap_width :=
                (3 * Character_Width)/2;

            else

              Cap_width :=
                (2 * Character_Width)/2;

            end if;

            Character_Height := 
              Text_Metric.TmHeight +
              Text_Metric.Tmexternalleading;

            Integer_Result := 
              Win32.Winuser.Releasedc
                (Hwnd => 
                  Window_Handle,
                 Hdc  => 
                  Device_Context_Handle);

            Bool_Result := 
              Win32.Winuser.Setscrollrange
                (Hwnd => 
                  Window_Handle,
                 Nbar => 
                  Win32.Winuser.Sb_Vert,
                 Nminpos => 
                  Minimum_Vertical_Scroll_Position,
                 Nmaxpos =>
                  System_Metrics.Sm'Pos(System_Metrics.Metrics_Record_Array'Last),
                 Bredraw => 
                  False);

            Integer_Result :=
              Win32.Winuser.Setscrollpos
                (Hwnd    => 
                  Window_Handle,
                 Nbar    => 
                  Win32.Winuser.Sb_Vert,
                 Npos    => 
                  Win32.Int
                    (Vertical_Scroll_Position),
                 Bredraw => 
                  True);

            return (0);

	 when Win32.Winuser.Wm_Size =>

            Client_Height := 
              Win32.Long
                (Win32.Utils.Hiword
                  (Win32.Dword
                    (Lparam)));

            return (0);

	 when Win32.Winuser.Wm_Vscroll =>

            case Win32.Utils.Loword
              (Win32.Dword
                (Wparam)) is

              when Win32.Winuser.Sb_Lineup =>

                 Vertical_Scroll_Position := 
                   Vertical_Scroll_Position - 1;

              when Win32.Winuser.Sb_Linedown =>

                 Vertical_Scroll_Position := 
                   Vertical_Scroll_Position + 1;

              when Win32.Winuser.Sb_Pageup =>

                 Vertical_Scroll_Position := 
                   Vertical_Scroll_Position -
                   Win32.Uint
                     (Client_Height /
                      Character_Height);

              when Win32.Winuser.Sb_Pagedown =>

                 Vertical_Scroll_Position := 
                   Vertical_Scroll_Position +
                   Win32.Uint
                     (Client_Height /
                      Character_Height);

              when Win32.Winuser.Sb_Thumbposition =>

                 Vertical_Scroll_Position := 
                   Win32.Uint
                     (Win32.Utils.Hiword
                       (Win32.Dword
                         (Wparam)));
                
              when others =>

                 null;

            end case;

            Vertical_Scroll_Position :=
               Win32.Uint
                 (Max
                   (L => 
                     0,
                    R => 
                     Min
                      (L => 
                        integer
                          (Vertical_Scroll_Position),
                       R => 
                        System_Metrics.Sm'POS
                          (System_Metrics.Sm'Last))));

            if Vertical_Scroll_Position /=
               Win32.Uint
                 (Win32.Winuser.Getscrollpos
                  (Hwnd => 
                    Window_Handle,
                   Nbar => 
                    Win32.Winuser.Sb_Vert)) then

               Integer_Result := 
                 Win32.Winuser.Setscrollpos
                  (Hwnd    =>
                    Window_Handle,
                   Nbar    => 
                    Win32.Winuser.Sb_Vert,
                   Npos    =>
                    Win32.Int
                      (Vertical_Scroll_Position),
                   Bredraw => 
                      True);
               
               Bool_Result := 
                 Win32.Winuser.InvalidateRect
                  (Hwnd   => 
                    Window_Handle,
                   Lprect => 
                    null,
                   Berase  => 
                    True);

            end if;

            return (0);

	 when Win32.Winuser.Wm_Paint =>

	    Device_Context_Handle := 
              Win32.Winuser.Beginpaint
	        (Hwnd    =>
                  Window_Handle,
	         Lppaint =>
                  Paint_Record'Access);

            for i in System_Metrics.Sm'First .. 
                     System_Metrics.Sm'Last loop

              Vertical_Position :=
                (Win32.Uint
                  (Character_Height) *
                  ( 1 -  Vertical_Scroll_Position +
                  Win32.Uint(
                    System_Metrics.Sm'Pos(i))));

	      Bool_Result := 
                Win32.Wingdi.Textout
	        (Hdc   =>
                  Device_Context_Handle,
                 Nxstart  => 
                  Win32.Int
                    (Character_Width),
                 Nystart  => 
                  Win32.Int
                    (Vertical_Position),
                 Lpstring => 
                  Convert_Char_Ptr_To_Lpcstr
                    (S => 
                      System_Metrics.Metrics_Record_Array(i).Label),
                 Cbstring => 
                   Win32.Int
                     (Interfaces.C.Strings.Strlen
                       (Item => 
                         System_Metrics.Metrics_Record_Array(i).Label)));

	      Bool_Result := 
                Win32.Wingdi.Textout
	          (Hdc   => Device_Context_Handle,
                   Nxstart  => 
                     Win32.Int
                       (Character_Width + 22 
                        * Cap_Width),
                   Nystart  => 
                     Win32.Int
                       (Vertical_Position),
                   Lpstring => 
                    Convert_Char_Ptr_To_Lpcstr
                      (S => 
                        System_Metrics.Metrics_Record_Array(i).Description),
                   Cbstring => 
                     Win32.Int
                       (Interfaces.C.Strings.Strlen
                         (Item => 
                           System_Metrics.Metrics_Record_Array(i).Description)));

              Uint_Result := 
                Win32.Wingdi.Settextalign
                  (Hdc   => 
                    Device_Context_Handle,
                   Fmode =>
                    Align_Top_Right);

	      Convert_Integer_To_Char_Array
                (Int => 
                  integer(Win32.Winuser.GetsystemMetrics
                    (System_Metrics.Sm'Pos(i))),
                 Ptr => 
                  Temporary_String_Pointer);

              Bool_Result :=
                Win32.Wingdi.Textout
	          (Hdc   => 
                    Device_Context_Handle,
                   Nxstart  => 
                    Win32.Int(
                      Character_Width + 22 * 
                      Cap_Width + 60 * 
                      Character_Width),
                   Nystart  => 
                    Win32.Int
                      (Vertical_Position),
                   Lpstring => 
                    Convert_Char_Ptr_To_Lpcstr
                      (S => 
                        Temporary_String_Pointer),
                   Cbstring => 
                     Win32.Int
                       (Interfaces.C.Strings.Strlen
                         (Item => 
                           Temporary_String_Pointer)));

              Interfaces.C.Strings.Free
                (Item =>
                  Temporary_String_Pointer);

              Uint_Result := 
                Win32.Wingdi.Settextalign
                  (Hdc => 
                    Device_Context_Handle,
                   Fmode =>
                    Align_Top_Left);

            end loop;

	    Bool_Result := 
              Win32.Winuser.Endpaint
               (Hwnd   => 
                 Window_Handle,
                Lppaint => 
                 Paint_Record'Access);

	    return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
                0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => Window_Handle, 
	       Msg    => Message, 
	       Wparam => Wparam,
	       Lparam => Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

   end Main;

end Windows.Routines;
