-- Filename: routines.adb
--
-- This is the conversion of Petzold's Win 95
-- Chapter 4 program bezier.
-- 
-- Contains : 1. main_windows_message_loop
--            2. function Main_Windows  (wndproc)
--            3. function main          (winmain)
--            4. DrawBezier
--            
with System;
with Unchecked_Conversion;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Winnt;
with Win32.Winuser;

with Bezier;
with Windows.Data;

use win32;  -- put in after all dot notation completed
            -- needed to see "and" and "or" for wparam
-- use type win32.wparam;



package body Windows.Routines is
   use type System.Address;
   use type Win32.Bool;
   use type Win32.Byte;
   use type Win32.Dword;
   use type Win32.Float;
   use type Win32.Int;
   use type Win32.Long;
   use type Win32.Uint;
   use type Win32.Word;

   Device_Handle : aliased Win32.Windef.Hdc;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => Windows.Data.Message'Access,
	      Hwnd =>System.Null_Address, 
	      Wmsgfiltermin => Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Windows.Data.Bresult := Win32.Winuser.Translatemessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
   	 Windows.Data.Lresult := Win32.Winuser.Dispatchmessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
      end loop;
   end Main_Windows_Message_Loop;



   procedure DrawBezier (Device_Handle : in Win32.Windef.Hdc) is
   begin

      -- example of gdi call PolyBezier
      Windows.Data.Bresult := Win32.Wingdi.PolyBezier
            (Hdc => Device_Handle,
             lppt => Bezier.Bezier_Points(Bezier.Bezier_Points'First)'access,
             Cpoints => Win32.Dword(4));

      -- examples of gdi calls: MoveToEx, LineTo
      -- The following lines are not part of the bezier curve
      -- but can be put back in to see clearly which point is
      -- being referenced by the left or right mouse button.
--      Windows.Data.Bresult := Win32.Wingdi.MoveToEx
--                      (Hdc => Device_Handle,
--                       X => Win32.Int(Bezier.Bezier_Points(0).X),
--                       Y => Win32.Int(Bezier.Bezier_Points(0).Y),
--                       Lppoint => Null);
--      Windows.Data.Bresult := Win32.Wingdi.LineTo
--                      (Hdc => Device_Handle,
--                       Xend => Win32.Int(Bezier.Bezier_Points(1).X),
--                       Yend => Win32.Int(Bezier.Bezier_Points(1).Y));
--
--      Windows.Data.Bresult := Win32.Wingdi.MoveToEx
--                      (Hdc => Device_Handle,
--                       X => Win32.Int(Bezier.Bezier_Points(2).X),
--                       Y => Win32.Int(Bezier.Bezier_Points(2).Y),
--                       Lppoint => Null);
--      Windows.Data.Bresult := Win32.Wingdi.LineTo
--                      (Hdc => Device_Handle,
--                       Xend => Win32.Int(Bezier.Bezier_Points(3).X),
--                       Yend => Win32.Int(Bezier.Bezier_Points(3).Y));

   end DrawBezier;



   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message : Win32.Uint;
      Wparam : Win32.Wparam;
      Lparam : Win32.Lparam) return Win32.Lresult is

   Hgdi_Object : Win32.Winnt.Handle;

   begin
      case Message is
         when Win32.Winuser.Wm_Size =>
            -- get client area width and height
            Bezier.Client_Width  := Win32.Int(Win32.Windef.LoWord
                                                (Win32.Dword(Lparam)));
            Bezier.Client_Height := Win32.Int(Win32.Windef.HiWord
                                                (Win32.Dword(Lparam)));

            -- assign default curve points
            Bezier.Bezier_Points(0).X := Win32.Long(Bezier.Client_Width / 4);
            Bezier.Bezier_Points(0).Y := Win32.Long(Bezier.Client_Height / 2);

            Bezier.Bezier_Points(1).X := Win32.Long(Bezier.Client_Width / 2);
            Bezier.Bezier_Points(1).Y := Win32.Long(Bezier.Client_Height / 4);

            Bezier.Bezier_Points(2).X := Win32.Long(Bezier.Client_Width / 2);
            Bezier.Bezier_Points(2).Y := Win32.Long(3 * Bezier.Client_Height / 4);

            Bezier.Bezier_Points(3).X := Win32.Long(3 * Bezier.Client_Width / 4);
            Bezier.Bezier_Points(3).Y := Win32.Long(Bezier.Client_Height / 2);

            return (0);

         -- mouse movement processing
	 when Win32.Winuser.Wm_Mousemove =>
            -- is it a left or right mouse button?
            if ( ((Wparam and Win32.Wparam(Win32.Winuser.Mk_Lbutton)) /= 0) or
                 ((Wparam and Win32.Wparam(Win32.Winuser.Mk_Rbutton)) /= 0)
               ) then
               Device_Handle := Win32.Winuser.Getdc
                                 (Hwnd => Window_Handle);
               Hgdi_Object := Win32.Wingdi.SelectObject
                   (Hdc => Device_Handle,
                   hgdiobj => Win32.Wingdi.GetStockObject
                                 (Win32.Wingdi.White_Pen));
               DrawBezier (Device_Handle => Device_Handle);

               -- is it the left mouse button?
               if (Wparam and Win32.Winuser.Mk_Lbutton) /= 0 then
                  Bezier.Bezier_Points(1).X := Win32.Long(Win32.Windef.Loword
                                                  (Win32.Dword(Lparam)));
                  Bezier.Bezier_Points(1).Y := Win32.Long(Win32.Windef.Hiword
                                                  (Win32.Dword(Lparam)));
               end if;

               -- is it the right mouse button?
               if (Wparam and Win32.Winuser.Mk_Rbutton) /= 0 then
                  Bezier.Bezier_Points (2).X := Win32.Long(Win32.Windef.Loword
                                                   (Win32.Dword(Lparam)));
                  Bezier.Bezier_Points (2).Y := Win32.Long(Win32.Windef.Hiword
                                                   (Win32.Dword(Lparam)));
                end if;

               -- redraw the curve
               Hgdi_Object := Win32.Wingdi.SelectObject
                      (Hdc => Device_Handle,
                       hgdiobj => Win32.Wingdi.GetStockObject
                                  (Win32.Wingdi.Black_Pen));
               DrawBezier (Device_Handle => Device_Handle);

               Windows.Data.Iresult := Win32.Winuser.Releasedc
                                       (Hwnd => Window_Handle,
                                        Hdc => Device_Handle);
            end if;
            return (0);


         -- Wm_Paint processing
	 when Win32.Winuser.Wm_Paint =>
            Windows.Data.Bresult := Win32.Winuser.InvalidateRect
                (Hwnd => Window_Handle,
                 Lprect => Null,
                 Berase => Win32.True);
	    Device_Handle := Win32.Winuser.Beginpaint -- BEGINPAINT
	               (Hwnd    => Window_Handle,
	                Lppaint => Windows.Data.Paint_Record'Access);

            DrawBezier (Device_Handle => Device_Handle);

            Windows.Data.Bresult := Win32.Winuser.EndPaint  -- ENDPAINT
                    (Hwnd => Window_Handle,
                     Lppaint => Windows.Data.Paint_Record'Access);
	    return (0);

	 when Win32.WinUser.Wm_Destroy =>
	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 0);
	    return (0);

	 when others =>
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => Window_Handle, 
	       Msg    => Message, 
	       Wparam => Wparam,
	       Lparam => Lparam);
      end case;

      return (0);
   end Main_Windows;



   function Main
     (Instance : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line : Win32.Pcstr;
      Show_Command : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Windows.Data.Window_Class :=      
	   (Style         => Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => Instance,
	    Hicon         => Windows.Data.Window_Class_Icon,
	    Hcursor       => Windows.Data.Window_Class_Cursor,
	    Hbrbackground => Windows.Data.Window_Class_Background,
	    Lpszmenuname  => Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => Windows.Data.Class_Name
                                (Windows.Data.Class_Name'First)'Access);

	 Windows.Data.Aresult :=
	   win32.winuser.Registerclass
	   (Lpwndclass => Windows.Data.Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := Win32.WinUser.CreateWindow
	(Lpclassname => Windows.Data.Class_Name
                           (Windows.Data.Class_Name'First)'Access,
            Lpwindowname => Windows.Data.Caption
                           (Windows.Data.Caption'First)'Access,
            Dwstyle      => Windows.Data.Window_Style,
            X            => Windows.Data.Window_Horizontal_Start, 
            Y            => Windows.Data.Window_Vertical_Start,
            Nwidth       => Windows.Data.Window_Width,
            Nheight      => Windows.Data.Window_Height,
            Hwndparent   => Windows.Data.Window_Parent,
            Hmenu        => Windows.Data.Window_Menu,
            Hinstance    => Instance,
            Lpparam      => Windows.Data.Window_Parameters);

    Windows.Data.Bresult := Win32.Winuser.Showwindow
      (Hwnd => Windows.Data.Window_Handle, 
	 Ncmdshow =>win32.Winmain.Get_Ncmdshow);

    Windows.Data.Bresult := Win32.Winuser.Updatewindow
      (Hwnd => Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Windows.Data.Message.Wparam;

   end Main;

end Windows.Routines;

