-- Filename: routines.adb
--
-- This is the conversion of Petzold's Win 95
-- Chapter 4 program Clover.
-- 
-- Contains : 1. main_windows_message_loop
--            2. function Main_Windows  (wndproc)
--            3. function main          (winmain)
--
--            



with Ada.Numerics.Elementary_Functions;
with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;

with Win32;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Winuser;

with Clover;
with Windows.Data;



package body Windows.Routines is
--use Win32;  -- put in after all dot notation completed
             -- needed to see "and" and "or" for wparam
   use type Win32.Bool;
   use type Win32.Int;
   use type Win32.Uint;
   use type Win32.Windef.Hrgn;



    function SetViewportOrgEx(hdc    : Win32.Windef.HDC;
                              X      : Win32.INT;
                              Y      : Win32.INT;
                              lpPoint: Win32.Windef.PPOINT)
                                  return Win32.BOOL;
    pragma Import(Stdcall, SetViewportOrgEx, "SetViewportOrgEx");



   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => Windows.Data.Message'Access,
	      Hwnd =>System.Null_Address, 
	      Wmsgfiltermin => Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => Windows.Data.Message_Filter_Maximum) /= 
             Win32.False) loop
	 
      	 Windows.Data.Bresult := Win32.Winuser.Translatemessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
   	 Windows.Data.Lresult := Win32.Winuser.Dispatchmessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
      end loop;
   end Main_Windows_Message_Loop;



   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message : Win32.Uint;
      Wparam : Win32.Wparam;
      Lparam : Win32.Lparam) return Win32.Lresult is

      Cursor_Handle : Win32.Windef.Hcursor;
      Device_Context_Handle : Win32.Windef.Hdc;
      Paint_Record : aliased win32.Winuser.Paintstruct;

      Radius, Angle : float;
      X, Y : float;
   
      type Temp_Region_Handle_Arr is array (1..6) of Win32.Windef.Hrgn;
      Temp_Region_Handles : Temp_Region_Handle_Arr;
      
      function Convert_Sys_Adr_To_Ppoint is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Windef.Ppoint);
      Null_Ppoint : Win32.Windef.Ppoint;



   begin

      case Message is
         when Win32.WinUser.Wm_Size =>
            -- get client area width and height
            Clover.Client_Width  := Win32.Int(Win32.Windef.Loword
                                         (L => Win32.Dword(Lparam)));
            Clover.Client_Height := Win32.Int(Win32.Windef.Hiword
                                         (L => Win32.Dword(Lparam)));

            Cursor_Handle := Win32.Winuser.SetCursor
               (Win32.Winuser.LoadCursor 
                   (System.Null_Address,
                    Win32.Lpcstr(Win32.Winuser.Idc_Wait)
                   )
               );
            Windows.Data.Iresult := Win32.Winuser.ShowCursor (Win32.True);

            if (Clover.Handle_Clip_Region /= System.Null_Address) then
               Windows.Data.Bresult := Win32.Wingdi.DeleteObject
                                       (Clover.Handle_Clip_Region);
            end if;

            -- create 4 elliptical regions
            Temp_Region_Handles (1) := Win32.Wingdi.CreateEllipticRgn
                       (Nleftrect   => 0,
                        Ntoprect    => Clover.Client_Height / 3,
                        Nrightrect  => Clover.Client_Width / 2,
                        Nbottomrect => 2 * Clover.Client_Height / 3
                       );
            Temp_Region_Handles (2) := Win32.Wingdi.CreateEllipticRgn
                       (Nleftrect   => Clover.Client_Width / 2,
                        Ntoprect    => Clover.Client_Height / 3,
                        Nrightrect  => Clover.Client_Width,
                        Nbottomrect => 2 * Clover.Client_Height / 3
                       );
            Temp_Region_Handles (3) := Win32.Wingdi.CreateEllipticRgn
                       (Nleftrect   => Clover.Client_Width / 3,
                        Ntoprect    => 0,
                        Nrightrect  => 2 * Clover.Client_Width / 3,
                        Nbottomrect => Clover.Client_Height / 2
                       );
            Temp_Region_Handles (4) := Win32.Wingdi.CreateEllipticRgn
                       (Nleftrect   => Clover.Client_Width / 3,
                        Ntoprect    => Clover.Client_Height / 2,
                        Nrightrect  => 2 * Clover.Client_Width / 3,
                        Nbottomrect => Clover.Client_Height
                       );

            -- create 3 dummy regions
            Temp_Region_Handles (5) := Win32.Wingdi.CreateRectRgn
                       (Nleftrect   => 0,
                        Ntoprect    => 0,
                        Nrightrect  => 1,
                        Nbottomrect => 1
                       );
            Temp_Region_Handles (6) := Win32.Wingdi.CreateRectRgn
                       (Nleftrect   => 0,
                        Ntoprect    => 0,
                        Nrightrect  => 1,
                        Nbottomrect => 1
                       );
            Clover.Handle_Clip_Region := Win32.Wingdi.CreateRectRgn
                       (Nleftrect   => 0,
                        Ntoprect    => 0,
                        Nrightrect  => 1,
                        Nbottomrect => 1
                       );

            -- combine regions to get resultant region
            Windows.Data.Iresult := Win32.Wingdi.CombineRgn
                       (HRgnDest => Temp_Region_Handles(5),
                        HRgnSrc1 => Temp_Region_Handles(1),
                        HRgnSrc2 => Temp_Region_Handles(2),
                        FCombineMode => Win32.Wingdi.Rgn_Or
                       );
            Windows.Data.Iresult := Win32.Wingdi.CombineRgn
                       (HRgnDest => Temp_Region_Handles(6),
                        HRgnSrc1 => Temp_Region_Handles(3),
                        HRgnSrc2 => Temp_Region_Handles(4),
                        FCombineMode => Win32.Wingdi.Rgn_Or
                       );
            Windows.Data.Iresult := Win32.Wingdi.CombineRgn
                       (HRgnDest => Clover.Handle_Clip_Region,
                        HRgnSrc1 => Temp_Region_Handles(5),
                        HRgnSrc2 => Temp_Region_Handles(6),
                        FCombineMode => Win32.Wingdi.Rgn_Xor
                       );

            -- delete temporary region handles
            for I in 1 .. 6 loop
               Windows.Data.Bresult := Win32.Wingdi.DeleteObject
                                       (Temp_Region_Handles(I));
            end loop;

            -- finally setcursor and showcursor
            Cursor_Handle := Win32.Winuser.SetCursor (Cursor_Handle);
            Windows.Data.Iresult := Win32.Winuser.ShowCursor (Win32.False);            

            return (0);




         when Win32.WinUser.Wm_Paint =>
            -- get handle to the device context
            Device_Context_Handle := Win32.Winuser.BeginPaint
                                     (HWnd    => Window_Handle,
                                      LpPaint => Paint_Record'Access);

            -- set the view port origin and the clipping region
            Null_Ppoint := Convert_Sys_Adr_To_Ppoint(System.Null_Address);
            Windows.Data.Bresult := SetViewportOrgEx
                (hdc    => Device_Context_Handle,
                 X      => Clover.Client_Width / 2,
                 Y      => Clover.Client_Height / 2,
                 lpPoint=> Null_Ppoint);
            Windows.Data.Iresult := Win32.Wingdi.SelectClipRgn
                          (Hdc  => Device_Context_Handle,
                           Hrgn => Clover.Handle_Clip_Region);

            -- determine radius size
            X := (float(Clover.Client_Width) / 2.0);
            Y := (float(Clover.Client_Height) / 2.0);
            Radius := Ada.Numerics.Elementary_Functions.Sqrt (X * X + Y * Y);

            -- draw 360 lines
            Angle := 0.0;
            for I in 1 .. 360 loop
               Windows.Data.Bresult := Win32.Wingdi.MoveToEx
                   (Hdc     => Device_Context_Handle,
                    X       => 0,
                    Y       => 0,
                    LpPoint => Convert_Sys_Adr_To_Ppoint(System.Null_Address));

               X := Radius * Ada.Numerics.Elementary_Functions.Cos (Angle) +
                    0.5;
               Y := -Radius * Ada.Numerics.Elementary_Functions.Sin (Angle) +
                    0.5;
               Windows.Data.Bresult := Win32.Wingdi.LineTo
                                       (Hdc     => Device_Context_Handle,
                                        Xend    => Win32.Int(X),
                                        Yend    => Win32.Int(Y));
               Angle := Angle + Clover.One_Degree;
            end loop;

            -- release the device context
	    Windows.Data.Bresult := Win32.Winuser.Endpaint
                                    (hwnd   => Window_Handle,
                                     Lppaint => Paint_Record'Access);
         return (0);



	 when Win32.WinUser.Wm_Destroy =>
            Windows.Data.Bresult := Win32.Wingdi.DeleteObject
                                    (Clover.Handle_Clip_Region);
	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 0);
	    return (0);




	 when others =>
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => Window_Handle, 
	       Msg    => Message, 
	       Wparam => Wparam,
	       Lparam => Lparam);
      end case;

      return (0);
   end Main_Windows;



   function Main
     (Instance : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line : Win32.Pcstr;
      Show_Command : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Windows.Data.Window_Class :=      
	   (Style         => Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => Instance,
	    Hicon         => Windows.Data.Window_Class_Icon,
	    Hcursor       => Windows.Data.Window_Class_Cursor,
	    Hbrbackground => Windows.Data.Window_Class_Background,
	    Lpszmenuname  => Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => Windows.Data.Class_Name
                                (Windows.Data.Class_Name'First)'Access);

	 Windows.Data.Aresult :=
	   win32.winuser.Registerclass
	   (Lpwndclass => Windows.Data.Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := Win32.WinUser.CreateWindow
	(Lpclassname => Windows.Data.Class_Name
                           (Windows.Data.Class_Name'First)'Access,
            Lpwindowname => Windows.Data.Caption
                           (Windows.Data.Caption'First)'Access,
            Dwstyle      => Windows.Data.Window_Style,
            X            => Windows.Data.Window_Horizontal_Start, 
            Y            => Windows.Data.Window_Vertical_Start,
            Nwidth       => Windows.Data.Window_Width,
            Nheight      => Windows.Data.Window_Height,
            Hwndparent   => Windows.Data.Window_Parent,
            Hmenu        => Windows.Data.Window_Menu,
            Hinstance    => Instance,
            Lpparam      => Windows.Data.Window_Parameters);

    Windows.Data.Bresult := Win32.Winuser.Showwindow
      (Hwnd => Windows.Data.Window_Handle, 
	 Ncmdshow =>win32.Winmain.Get_Ncmdshow);

    Windows.Data.Bresult := Win32.Winuser.Updatewindow
      (Hwnd => Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Windows.Data.Message.Wparam;

   end Main;

end Windows.Routines;
