-- Filename: Routines.adb
--
-- This is the conversion of Petzold's Win 95
-- Chapter 4 program EMF1 (Enhanced Metafile 1).
-- 
-- Contains : 1. main_windows_message_loop
--            2. function Main_Windows  (wndproc)
--            3. function main          (winmain)
--
--            



with Ada.Unchecked_Conversion;
with System;

-- needed for all programs so far
with Win32;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Winuser;

with EMF1;
with Windows.Data;


package body Windows.Routines is
   use type Win32.Bool;
   use type Win32.Long;
   use type Win32.Windef.Hrgn;



   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => Windows.Data.Message'Access,
	      Hwnd =>System.Null_Address, 
	      Wmsgfiltermin => Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => Windows.Data.Message_Filter_Maximum) /= 
             Win32.False) loop
	 
      	 Windows.Data.Bresult := Win32.Winuser.Translatemessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
   	 Windows.Data.Lresult := Win32.Winuser.Dispatchmessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
      end loop;
   end Main_Windows_Message_Loop;



   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message : Win32.Uint;
      Wparam : Win32.Wparam;
      Lparam : Win32.Lparam) return Win32.Lresult is

      Device_Context_Handle : Win32.Windef.Hdc;
      EMF_Context_Handle : Win32.Windef.Hdc;
      Rect : aliased Win32.Windef.Rect;

      function Convert_Sys_Adr_To_Lpcstr is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Lpcstr);
      Null_Lpcstr : Win32.Lpcstr := Convert_Sys_Adr_To_Lpcstr
                                    (System.Null_Address);

      function Convert_Sys_Adr_To_PPoint is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Windef.Ppoint);
      Null_Ppoint : Win32.Windef.Ppoint := Convert_Sys_Adr_To_PPoint
                                           (System.Null_Address);

--      function Convert_Sys_Adr_To_PSize is new
--         Ada.Unchecked_Conversion
--                  (Source => System.Address,
--                   Target => Win32.Windef.PSize);
--      Null_LpSize : Win32.Windef.PSize;
--
      function Convert_Sys_Adr_To_Ac_Rect_T is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Wingdi.Ac_Rect_T);
      Ac_Rect_T : Win32.Wingdi.Ac_Rect_T;
      Null_Ac_Rect_T : Win32.Wingdi.Ac_Rect_T := Convert_Sys_Adr_To_Ac_Rect_T
                                             (System.Null_Address);

      function Convert_Sys_Adr_To_Hdc is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Windef.HDC);
      Null_Hdc : Win32.Windef.HDC := Convert_Sys_Adr_To_Hdc
                                           (System.Null_Address);



   begin

      case Message is

         when Win32.WinUser.Wm_Create =>

            -- get metafile device context handle,
            EMF_Context_Handle := Win32.Wingdi.CreateEnhmetaFile
                                  (Hdcref     => Null_Hdc,
                                   lpFilename => Null_Lpcstr,
                                   LpRect     => Null_Ac_Rect_T,
                                   lpDescription => Null_Lpcstr
                                  );

            -- draw rectangle and lines
            Windows.Data.Bresult := Win32.Wingdi.Rectangle
                        (Hdc => EMF_Context_Handle,
                         NLeftRect    => Win32.Int(100),
                         NTopRect     => Win32.Int(100),
                         NRightRect   => Win32.Int(200),
                         NBottomRect  => Win32.Int(200));

               Windows.Data.Bresult := Win32.Wingdi.MoveToEx
                   (Hdc     => EMF_Context_Handle,
                    X       => Win32.Int(100),
                    Y       => Win32.Int(100),
                    LpPoint => Null_Ppoint);
               Windows.Data.Bresult := Win32.Wingdi.LineTo
                   (Hdc     => EMF_Context_Handle,
                    Xend    => Win32.Int(200),
                    Yend    => Win32.Int(200));
               Windows.Data.Bresult := Win32.Wingdi.MoveToEx
                   (Hdc     => EMF_Context_Handle,
                    X       => Win32.Int(200),
                    Y       => Win32.Int(100),
                    LpPoint => Null_Ppoint);
               Windows.Data.Bresult := Win32.Wingdi.LineTo
                   (Hdc     => EMF_Context_Handle,
                    Xend    => Win32.Int(100),
                    Yend    => Win32.Int(200));

            EMF1.EMF_Handle := Win32.Wingdi.CloseEnhMetaFile
                               (Hdc => EMF_Context_Handle);

            return (0);


         when Win32.WinUser.Wm_Paint =>
            -- get handle to the device context
            Device_Context_Handle := Win32.Winuser.BeginPaint
                                     (HWnd    => Window_Handle,
                                      LpPaint => EMF1.Paint_Record'Access);
            Windows.Data.Bresult := Win32.Winuser.GetClientRect
                           (HWnd => Window_Handle,
                            LpRect => EMF1.Rect'Access);

            EMF1.Rect.Left   := EMF1.Rect.Right / 4;
            EMF1.Rect.Right  := 3 * EMF1.Rect.Right / 4;
            EMF1.Rect.Top    := EMF1.Rect.Bottom / 4;
            EMF1.Rect.Bottom := 3 * EMF1.Rect.Bottom / 4;

            Ac_Rect_T := Convert_Sys_Adr_To_Ac_Rect_T (EMF1.Rect.Left'Address);
            Windows.Data.Bresult := Win32.Wingdi.PlayEnhMetaFile
                           (Hdc    => Device_Context_Handle,
                            Hemf   => EMF1.EMF_Handle,
                            LpRect => Ac_Rect_T
                           );

            -- release the device context
	    Windows.Data.Bresult := Win32.Winuser.Endpaint
                                    (hwnd   => Window_Handle,
                                     Lppaint => EMF1.Paint_Record'Access);
         return (0);



	 when Win32.WinUser.Wm_Destroy =>
            Windows.Data.Bresult := Win32.Wingdi.DeleteEnhMetaFile
                                    (Hemf => EMF1.EMF_Handle);
 	    Win32.WinUser.Postquitmessage
	                  (Nexitcode => 0);
	    return (0);



	 when others =>
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => Window_Handle, 
	       Msg    => Message, 
	       Wparam => Wparam,
	       Lparam => Lparam);
      end case;

      return (0);
   end Main_Windows;



   function Main
     (Instance : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line : Win32.Pcstr;
      Show_Command : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Windows.Data.Window_Class :=      
	   (Style         => Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => Instance,
	    Hicon         => Windows.Data.Window_Class_Icon,
	    Hcursor       => Windows.Data.Window_Class_Cursor,
	    Hbrbackground => Windows.Data.Window_Class_Background,
	    Lpszmenuname  => Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => Windows.Data.Class_Name
                                (Windows.Data.Class_Name'First)'Access);

	 Windows.Data.Aresult :=
	   win32.winuser.Registerclass
	   (Lpwndclass => Windows.Data.Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := Win32.WinUser.CreateWindow
	(Lpclassname => Windows.Data.Class_Name
                           (Windows.Data.Class_Name'First)'Access,
            Lpwindowname => Windows.Data.Caption
                           (Windows.Data.Caption'First)'Access,
            Dwstyle      => Windows.Data.Window_Style,
            X            => Windows.Data.Window_Horizontal_Start, 
            Y            => Windows.Data.Window_Vertical_Start,
            Nwidth       => Windows.Data.Window_Width,
            Nheight      => Windows.Data.Window_Height,
            Hwndparent   => Windows.Data.Window_Parent,
            Hmenu        => Windows.Data.Window_Menu,
            Hinstance    => Instance,
            Lpparam      => Windows.Data.Window_Parameters);

    Windows.Data.Bresult := Win32.Winuser.Showwindow
      (Hwnd => Windows.Data.Window_Handle, 
	 Ncmdshow =>win32.Winmain.Get_Ncmdshow);

    Windows.Data.Bresult := Win32.Winuser.Updatewindow
      (Hwnd => Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Windows.Data.Message.Wparam;

   end Main;

end Windows.Routines;
