-- Filename: Routines.adb
--
-- This is the conversion of Petzold's Win 95
-- Chapter 4 program Endjoin.
-- 
-- Contains : 1. main_windows_message_loop
--            2. function Main_Windows  (wndproc)
--            3. function main          (winmain)
--
--            



-- with Ada.Numerics.Elementary_Functions;
with Ada.Unchecked_Conversion;
-- with Interfaces.C.Strings;
with System;

-- needed for all programs so far
with Win32;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Winuser;

with Endjoin;
with Windows.Data;


package body Windows.Routines is
--use Win32;  -- put in after all dot notation completed
             -- needed to see "and" and "or" for wparam
   use type Win32.Bool;
   use type Win32.DWord;

--   use type Win32.Int;
--   use type Win32.Uint;
   use type Win32.Windef.Hrgn;



    function SetWindowExtEx(Hdc     : Win32.Windef.HDC; 
                            NxExtent: Win32.INT; 
                            NyExtent: Win32.INT; 
                            LpSize  : Win32.Windef.PSIZE) 
                                return Win32.BOOL;
    pragma Import(Stdcall, SetWindowExtEx, "SetWindowExtEx");


    function SetViewportExtEx(Hdc     : Win32.Windef.HDC; 
                              NxExtent: Win32.INT; 
                              NyExtent: Win32.INT; 
                              LpSize  : Win32.Windef.PSIZE) 
                                  return Win32.BOOL;
    pragma Import(Stdcall, SetViewportExtEx, "SetViewportExtEx");




   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => Windows.Data.Message'Access,
	      Hwnd =>System.Null_Address, 
	      Wmsgfiltermin => Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => Windows.Data.Message_Filter_Maximum) /= 
             Win32.False) loop
	 
      	 Windows.Data.Bresult := Win32.Winuser.Translatemessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
   	 Windows.Data.Lresult := Win32.Winuser.Dispatchmessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
      end loop;
   end Main_Windows_Message_Loop;



   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message : Win32.Uint;
      Wparam : Win32.Wparam;
      Lparam : Win32.Lparam) return Win32.Lresult is

      Device_Context_Handle : Win32.Windef.Hdc;
      Hgd_Iobj : Win32.Windef.Hgdiobj;
      Log_Brush : Win32.Wingdi.Logbrush;
      Paint_Record : aliased win32.Winuser.Paintstruct;
      Pen_Style : Win32.Dword;

      function Convert_Sys_Adr_To_PSize is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Windef.PSize);
      Null_LpSize : Win32.Windef.PSize;

      function Convert_Sys_Adr_To_PcdWord is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.PcdWord);
      Null_PcdWord : Win32.PcdWord;

      function Convert_Sys_Adr_To_Ppoint is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Windef.Ppoint);
      Null_Ppoint : Win32.Windef.Ppoint;

      function Convert_Sys_Adr_To_PLog_Brush is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Wingdi.Ac_LogBrush_T);
      PLog_Brush : Win32.Wingdi.Ac_LogBrush_T;



   begin

      case Message is
         when Win32.WinUser.Wm_Size =>
            -- get client area width and height
            Endjoin.Client_Width  := Win32.Int(Win32.Windef.Loword
                                         (L => Win32.Dword(Lparam)));
            Endjoin.Client_Height := Win32.Int(Win32.Windef.Hiword
                                         (L => Win32.Dword(Lparam)));
            return (0);



         when Win32.WinUser.Wm_Paint =>
            -- get handle to the device context
            Device_Context_Handle := Win32.Winuser.BeginPaint
                                     (HWnd    => Window_Handle,
                                      LpPaint => Paint_Record'Access);

            -- set mapping mode to anisotropic, then set
            -- window extent and viewport extent
            Windows.Data.Iresult := Win32.Wingdi.SetMapMode
                            (Hdc => Device_Context_Handle,
                             FnMapMode => Win32.Wingdi.Mm_Anisotropic);
            Null_Lpsize := Convert_Sys_Adr_To_PSize(System.Null_Address);
            Windows.Data.Bresult := SetWindowExtEx
                (Hdc      => Device_Context_Handle,
                 NxExtent => Win32.Int(100),
                 NyExtent => Win32.Int(100),
                 LpSize   => Null_LpSize);
            Windows.Data.Bresult := SetViewportExtEx
                (Hdc      => Device_Context_Handle,
                 NxExtent => Endjoin.Client_Width,
                 NyExtent => Endjoin.Client_Height,
                 LpSize   => Null_LpSize);

            Log_Brush.LbStyle := Win32.Wingdi.Bs_Solid;
            Log_Brush.LbColor := Win32.Wingdi.RGB (128, 128, 128);
            Log_Brush.LbHatch := 0;

            -- draw object with 3 path styles
            for I in 1 .. 3 loop
               Pen_Style := Win32.Wingdi.Ps_Solid or
                            Win32.Wingdi.Ps_Geometric or
                            Win32.Dword(Endjoin.Ends (I)) or
                            Win32.Dword(Endjoin.Join (I));
               Null_PcdWord := Convert_Sys_Adr_To_PcdWord
                               (System.Null_Address);
               PLog_Brush := Convert_Sys_Adr_To_PLog_Brush (Log_Brush'Address);
               Hgd_Iobj := Win32.Wingdi.SelectObject
                       (Hdc => Device_Context_Handle,
                        Hgdiobj => Win32.Wingdi.ExtCreatePen
                                  (DwPenStyle => Pen_Style,
                                   DwWidth => 10,
                                   Lplb => PLog_Brush,
                                   DwStyleCount => 0,
                                   LpStyle => Null_PcdWord
                                  )
                       );
               Windows.Data.Bresult := Win32.Wingdi.BeginPath
                                       (Device_Context_Handle);
                             
               Windows.Data.Bresult := Win32.Wingdi.MoveToEx
                   (Hdc     => Device_Context_Handle,
                    X       => Win32.Int(10 + 30 * (I - 1)),
                    Y       => Win32.Int(25),
                    LpPoint => Null_Ppoint);
               Windows.Data.Bresult := Win32.Wingdi.LineTo
                                       (Hdc     => Device_Context_Handle,
                                        Xend    => Win32.Int(20 + 30 * (I - 1)),
                                        Yend    => Win32.Int(75));
               Windows.Data.Bresult := Win32.Wingdi.LineTo
                                       (Hdc     => Device_Context_Handle,
                                        Xend    => Win32.Int(30 + 30 * (I - 1)),
                                        Yend    => Win32.Int(25));

               Windows.Data.Bresult := Win32.Wingdi.EndPath
                                       (Hdc => Device_Context_Handle);
               Windows.Data.Bresult := Win32.Wingdi.StrokePath
                                       (Hdc => Device_Context_Handle);
               Windows.Data.Bresult := Win32.Wingdi.DeleteObject
               (Win32.Wingdi.SelectObject
                   (Hdc => Device_Context_Handle,
                    Hgdiobj => Win32.Wingdi.GetStockObject  --Win32.Windef.Hgdiobj(
                               (Win32.Wingdi.Black_Pen)
                   )
               );

               Windows.Data.Bresult := Win32.Wingdi.MoveToEx
                   (Hdc     => Device_Context_Handle,
                    X       => Win32.Int(10 + 30 * (I - 1)),
                    Y       => Win32.Int(25),
                    LpPoint => Convert_Sys_Adr_To_Ppoint(System.Null_Address));
               Windows.Data.Bresult := Win32.Wingdi.LineTo
                                       (Hdc     => Device_Context_Handle,
                                        Xend    => Win32.Int(20 + 30 * (I - 1)),
                                        Yend    => Win32.Int(75));
               Windows.Data.Bresult := Win32.Wingdi.LineTo
                                       (Hdc     => Device_Context_Handle,
                                        Xend    => Win32.Int(30 + 30 * (I - 1)),
                                        Yend    => Win32.Int(25));

            end loop;

            -- release the device context
	    Windows.Data.Bresult := Win32.Winuser.Endpaint
                                    (hwnd   => Window_Handle,
                                     Lppaint => Paint_Record'Access);
         return (0);



	 when Win32.WinUser.Wm_Destroy =>
	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 0);
	    return (0);




	 when others =>
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => Window_Handle, 
	       Msg    => Message, 
	       Wparam => Wparam,
	       Lparam => Lparam);
      end case;

      return (0);
   end Main_Windows;



   function Main
     (Instance : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line : Win32.Pcstr;
      Show_Command : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Windows.Data.Window_Class :=      
	   (Style         => Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => Instance,
	    Hicon         => Windows.Data.Window_Class_Icon,
	    Hcursor       => Windows.Data.Window_Class_Cursor,
	    Hbrbackground => Windows.Data.Window_Class_Background,
	    Lpszmenuname  => Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => Windows.Data.Class_Name
                                (Windows.Data.Class_Name'First)'Access);

	 Windows.Data.Aresult :=
	   win32.winuser.Registerclass
	   (Lpwndclass => Windows.Data.Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := Win32.WinUser.CreateWindow
	(Lpclassname => Windows.Data.Class_Name
                           (Windows.Data.Class_Name'First)'Access,
            Lpwindowname => Windows.Data.Caption
                           (Windows.Data.Caption'First)'Access,
            Dwstyle      => Windows.Data.Window_Style,
            X            => Windows.Data.Window_Horizontal_Start, 
            Y            => Windows.Data.Window_Vertical_Start,
            Nwidth       => Windows.Data.Window_Width,
            Nheight      => Windows.Data.Window_Height,
            Hwndparent   => Windows.Data.Window_Parent,
            Hmenu        => Windows.Data.Window_Menu,
            Hinstance    => Instance,
            Lpparam      => Windows.Data.Window_Parameters);

    Windows.Data.Bresult := Win32.Winuser.Showwindow
      (Hwnd => Windows.Data.Window_Handle, 
	 Ncmdshow =>win32.Winmain.Get_Ncmdshow);

    Windows.Data.Bresult := Win32.Winuser.Updatewindow
      (Hwnd => Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Windows.Data.Message.Wparam;

   end Main;

end Windows.Routines;
