-- Filename: Routines.adb
--
-- This is the conversion of Petzold's Win 95
-- Chapter 4 program metafile.
-- 
-- Contains : 1. main_windows_message_loop
--            2. function Main_Windows  (wndproc)
--            3. function main          (winmain)
--
--            



with Ada.Unchecked_Conversion;
with System;

-- needed for all programs so far
with Win32;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Winuser;

with Metafile;
with Windows.Data;


package body Windows.Routines is
   use type Win32.Bool;
   use type Win32.Windef.Hrgn;



    function SetWindowExtEx(Hdc     : Win32.Windef.HDC; 
                            NxExtent: Win32.Int;
                            NyExtent: Win32.Int;
                            LpSize  : Win32.Windef.Psize) 
                                return Win32.BOOL;
    pragma Import(Stdcall, SetWindowExtEx, "SetWindowExtEx");


    function SetViewportExtEx(Hdc     : Win32.Windef.HDC; 
                              NxExtent: Win32.Int;
                              NyExtent: Win32.Int;
                              LpSize  : Win32.Windef.Psize) 
                                  return Win32.BOOL;
    pragma Import(Stdcall, SetViewportExtEx, "SetViewportExtEx");

    function SetWindowOrgEx(hdc    : Win32.Windef.HDC; 
                            X      : Win32.Int;
                            Y      : Win32.Int;
                            lpPoint: Win32.Windef.PPoint)
                                return Win32.BOOL;
    pragma Import(Stdcall, SetWindowOrgEx, "SetWindowOrgEx");




   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => Windows.Data.Message'Access,
	      Hwnd =>System.Null_Address, 
	      Wmsgfiltermin => Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => Windows.Data.Message_Filter_Maximum) /= 
             Win32.False) loop
	 
      	 Windows.Data.Bresult := Win32.Winuser.Translatemessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
   	 Windows.Data.Lresult := Win32.Winuser.Dispatchmessage
	   (Lpmsg => Windows.Data.Message'Access);
	 
      end loop;
   end Main_Windows_Message_Loop;



   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message : Win32.Uint;
      Wparam : Win32.Wparam;
      Lparam : Win32.Lparam) return Win32.Lresult is

      Brush_Handle : Win32.Windef.HBrush;
      Device_Context_Handle : Win32.Windef.Hdc;
      Metafile_Context_Handle : Win32.Windef.Hdc;
      Hgd_Iobj : Win32.Windef.Hgdiobj;
      Paint_Record : aliased win32.Winuser.Paintstruct;

      function Convert_Sys_Adr_To_Lpcstr is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Lpcstr);
      Null_Lpcstr : Win32.Lpcstr := Convert_Sys_Adr_To_Lpcstr
                                    (System.Null_Address);

      function Convert_Sys_Adr_To_PPoint is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Windef.Ppoint);
      Null_Ppoint : Win32.Windef.Ppoint := Convert_Sys_Adr_To_PPoint
                                           (System.Null_Address);

      function Convert_Sys_Adr_To_PSize is new
         Ada.Unchecked_Conversion
                  (Source => System.Address,
                   Target => Win32.Windef.PSize);
      Null_LpSize : Win32.Windef.PSize;



   begin

      case Message is

         when Win32.WinUser.Wm_Create =>

            -- get metafile device context handle,
            -- handle to a brush ...
            Metafile_Context_Handle := Win32.Wingdi.CreatemetaFile
                                       (lpszFile => Null_Lpcstr);

            Brush_Handle := Win32.Wingdi.CreateSolidBrush
                            (Clrref => Win32.Wingdi.RGB (0, 0, 255));

            Windows.Data.Bresult := Win32.Wingdi.Rectangle
                        (Hdc => Metafile_Context_Handle,
                         NLeftRect    => Win32.Int(0),
                         NTopRect     => Win32.Int(0),
                         NRightRect   => Win32.Int(100),
                         NBottomRect  => Win32.Int(100));

               Windows.Data.Bresult := Win32.Wingdi.MoveToEx
                   (Hdc     => Metafile_Context_Handle,
                    X       => Win32.Int(0),
                    Y       => Win32.Int(0),
                    LpPoint => Null_Ppoint);
               Windows.Data.Bresult := Win32.Wingdi.LineTo
                   (Hdc     => Metafile_Context_Handle,
                    Xend    => Win32.Int(100),
                    Yend    => Win32.Int(100));
               Windows.Data.Bresult := Win32.Wingdi.MoveToEx
                   (Hdc     => Metafile_Context_Handle,
                    X       => Win32.Int(0),
                    Y       => Win32.Int(100),
                    LpPoint => Null_Ppoint);
               Windows.Data.Bresult := Win32.Wingdi.LineTo
                   (Hdc     => Metafile_Context_Handle,
                    Xend    => Win32.Int(100),
                    Yend    => Win32.Int(0));

               Hgd_Iobj := Win32.Wingdi.SelectObject
                           (Hdc => Metafile_Context_Handle,
                            Hgdiobj => Brush_Handle);

               Windows.Data.Bresult :=Win32.Wingdi.Ellipse
                    (hdc        => Metafile_Context_Handle,
                     NLeftRect  => Win32.Int(20), 
                     NTopRect   => Win32.Int(20),
                     NRightRect => Win32.Int(80),
                     NBottomRect=> Win32.Int(80));

            Metafile.Metafile_Handle := Win32.Wingdi.CloseMetaFile
                               (Hdc => Metafile_Context_Handle);

            Windows.Data.Bresult := Win32.Wingdi.DeleteObject
                      (hgdiobj => Brush_Handle);

            return (0);




         when Win32.WinUser.Wm_Size =>
            -- get client area width and height
            Metafile.Client_Width  := Win32.Int(Win32.Windef.Loword
                                         (L => Win32.Dword(Lparam)));
            Metafile.Client_Height := Win32.Int(Win32.Windef.Hiword
                                         (L => Win32.Dword(Lparam)));
            return (0);



         when Win32.WinUser.Wm_Paint =>
            -- get handle to the device context
            Device_Context_Handle := Win32.Winuser.BeginPaint
                                     (HWnd    => Window_Handle,
                                      LpPaint => Paint_Record'Access);

            -- set mapping mode to anisotropic, then set
            -- window extent and viewport extent
            Windows.Data.Iresult := Win32.Wingdi.SetMapMode
                            (Hdc => Device_Context_Handle,
                             FnMapMode => Win32.Wingdi.Mm_Anisotropic);
            Null_Lpsize := Convert_Sys_Adr_To_PSize(System.Null_Address);
            Windows.Data.Bresult := SetWindowExtEx
                (Hdc      => Device_Context_Handle,
                 NxExtent => Win32.Int(1000),
                 NyExtent => Win32.Int(1000),
                 LpSize   => Null_LpSize);
            Windows.Data.Bresult := SetViewportExtEx
                (Hdc      => Device_Context_Handle,
                 NxExtent => Metafile.Client_Width,
                 NyExtent => Metafile.Client_Height,
                 LpSize   => Null_LpSize);

            -- for 10 by 10 table in window, play metafile
            for X in 0 .. 9 loop
               for Y in 0 .. 9 loop
                  Windows.Data.Bresult := SetWindowOrgEx
                           (hdc    => Device_Context_Handle,
                            X      => Win32.Int (-100 * X),
                            Y      => Win32.Int (-100 * Y),
                            lpPoint=> Null_PPoint);

                  Windows.Data.Bresult := Win32.Wingdi.PlayMetaFile
                           (Hdc => Device_Context_Handle,
                            Hmf => Metafile.Metafile_Handle);
               end loop;
            end loop;

            -- release the device context
	    Windows.Data.Bresult := Win32.Winuser.Endpaint
                                    (hwnd   => Window_Handle,
                                     Lppaint => Paint_Record'Access);
         return (0);



	 when Win32.WinUser.Wm_Destroy =>
            Windows.Data.Bresult := Win32.Wingdi.DeleteMetaFile
                                    (Hmf => Metafile.Metafile_Handle);
 	    Win32.WinUser.Postquitmessage
	                  (Nexitcode => 0);
	    return (0);



	 when others =>
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => Window_Handle, 
	       Msg    => Message, 
	       Wparam => Wparam,
	       Lparam => Lparam);
      end case;

      return (0);
   end Main_Windows;



   function Main
     (Instance : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line : Win32.Pcstr;
      Show_Command : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Windows.Data.Window_Class :=      
	   (Style         => Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => Instance,
	    Hicon         => Windows.Data.Window_Class_Icon,
	    Hcursor       => Windows.Data.Window_Class_Cursor,
	    Hbrbackground => Windows.Data.Window_Class_Background,
	    Lpszmenuname  => Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => Windows.Data.Class_Name
                                (Windows.Data.Class_Name'First)'Access);

	 Windows.Data.Aresult :=
	   win32.winuser.Registerclass
	   (Lpwndclass => Windows.Data.Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := Win32.WinUser.CreateWindow
	(Lpclassname => Windows.Data.Class_Name
                           (Windows.Data.Class_Name'First)'Access,
            Lpwindowname => Windows.Data.Caption
                           (Windows.Data.Caption'First)'Access,
            Dwstyle      => Windows.Data.Window_Style,
            X            => Windows.Data.Window_Horizontal_Start, 
            Y            => Windows.Data.Window_Vertical_Start,
            Nwidth       => Windows.Data.Window_Width,
            Nheight      => Windows.Data.Window_Height,
            Hwndparent   => Windows.Data.Window_Parent,
            Hmenu        => Windows.Data.Window_Menu,
            Hinstance    => Instance,
            Lpparam      => Windows.Data.Window_Parameters);

    Windows.Data.Bresult := Win32.Winuser.Showwindow
      (Hwnd => Windows.Data.Window_Handle, 
	 Ncmdshow =>win32.Winmain.Get_Ncmdshow);

    Windows.Data.Bresult := Win32.Winuser.Updatewindow
      (Hwnd => Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Windows.Data.Message.Wparam;

   end Main;

end Windows.Routines;
