with Ada.Unchecked_Conversion;
With Interfaces.C.Strings;
with System;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
   use type Interfaces.C.Size_T;
   use type System.Address;
   use type Win32.Dword;
   use type Win32.Bool;
   use type Win32.Byte;
   use type Win32.Int;
   use type Win32.Long;
   use type Win32.Uint;

   Text_Metric : aliased win32.Wingdi.Textmetric;

   Rectangle : aliased Win32.Windef.Rect;

   Character_Width : win32.Long;

   Character_Height : win32.Long;

   True : constant Win32.bool := 1;

   Top_Line : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       ("Message        Key Char  Repeat Scan Ext ALT Prev Tran");
 
   Underline_Line : constant Interfaces.C.STrings.Chars_Ptr  :=
     Interfaces.C.Strings.New_String
       ("_______        ___ _____ ______ ____ ___ ___ ____ ____");

   Message_Start : constant integer := 1;

   Key_Start : constant integer := 16;

   Key_Length : constant integer := 3;

   Character_Start : constant integer := 19;

   Character_Length : constant integer := 6;

   Repeat_Start : constant integer := 25;

   Repeat_Length : constant integer := 7;

   Scan_Start : constant integer := 33;

   Scan_Length : constant integer := 4;

   Ext_Start : constant integer := 38;

   Ext_Length : constant integer := 3;

   Alt_Start : constant integer := 42;

   Alt_Length : constant integer := 3;

   Prev_Start : constant integer := 45;

   Prev_Length : constant integer := 5;

   Tran_Start : constant integer := 51;

   Tran_Length : constant integer := 4;

   No_String : constant string := "No";

   Yes_String : constant string := "Yes";

   Down_String : constant string := "Down";

   Up_String  : constant string := "Up";

   Paint_Record : aliased win32.Winuser.Paintstruct;

   Window_Class : aliased Win32.Winuser.Wndclass;

   Message : aliased Win32.Winuser.Msg;

   Device_Context_Handle : aliased Win32.Windef.Hdc;

   Integer_Result        : Win32.Int;
   Bool_Result           : Win32.Bool;
   Long_Result           : Win32.Long;
   Atom_Result           : Win32.Windef.Atom;
   Uint_Result           : Win32.Uint;
   Object_Result : Win32.Windef.Hgdiobj;

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               System.Null_Address, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   procedure Show_Key
     (Window_Handle : Win32.Windef.Hwnd;
      Itype         : Win32.Int;
      Message       : String;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) is



     procedure Set_Output_Text
       (Window_Handle : Win32.Windef.Hwnd;
        Itype         : Win32.Int;
        Message       : String;
        Wparam        : Win32.Wparam;
        Lparam        : Win32.Lparam) is

       Scratch_Line_Pointer : Interfaces.C.Strings.Chars_Ptr;

       Scratch_Line : String :=
           ("                                                      ");

        procedure Set_Message_In_Scratch_Line
          (Start   : integer;
           Input   : String;
           Line    : in out String) is

        begin

          Line(Start .. (start + input'Length) - 1) := input;
      
        end Set_Message_In_Scratch_Line;

        procedure Set_Message
          (Message : String;
           Line    : in out String ) is
        begin

          Set_Message_In_Scratch_Line
            (Start   => 
               Message_Start,
             Input   => 
               Message,
             Line =>
               Line);

        end Set_Message;

        procedure Set_Key
          (Key   : Win32.Wparam;
           Itype : Win32.Int;
           Line  : in out String ) is
        
        begin

          if Itype = 0 then

            Set_Message_In_Scratch_Line
              (Start      =>
                (Key_Start +
                 Key_Length) - 
                 Integer'Image
                   (Integer
                      (Key))'Length,
               Input      =>
                 Integer'Image
                   (Integer
                     (Key)),
               Line => Line);

       end if;

     end Set_Key;

     procedure Set_Itype
       (Itype  : win32.Int;
        Wparam : Win32.Wparam;
        Line   : in out String) is

        Temporary_String : constant String(1 .. 1) :=
          (1 => Character'Val(Wparam));

     begin

       if Itype /= 0 then

         Set_Message_In_Scratch_Line
           (Start      => 
             (Character_Start +
             (Character_Length - 2)) -
             Integer'Image
               (Integer
                 (Wparam))'Length,
            Input      => 
             Integer'Image
               (Integer
                 (Wparam)),
            Line =>
              Line);

         Set_Message_In_Scratch_Line
           (Start      => 
             Integer
              (Character_Start +
               Character_Length) -
             Temporary_String'Length,
            Input      =>
             Temporary_String,
            Line => 
             Line);

       end if;

     end Set_Itype;


     procedure Set_Repeat
       (Lparam : Win32.Lparam;
        Line   : in out String) is

     begin

       Set_Message_In_Scratch_Line
         (Start      => 
           (Repeat_Start +
            Repeat_Length) -
            Integer'Image
              (Integer
                (Win32.Utils.Loword
                  (Win32.Dword
                    (Lparam))))'Length,
          Input      => 
           Integer'Image
             (Integer
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam)))),
          Line =>
           Line);

     end Set_Repeat;

     procedure Set_Scan
       (Lparam : Win32.Lparam;
        Line   : in out String) is

     begin

       Set_Message_In_Scratch_Line
         (Start => 
           Integer
             (Scan_Start +
              Scan_Length) -
              Win32.Byte'Image
               (Win32.Utils.Lobyte
                 (Win32.Utils.Hiword
                   (Win32.Dword
                     (Lparam))))'Length,
          Input =>  
           Win32.Byte'Image
             (Win32.Utils.Lobyte
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam)))),
          Line => 
           Line);

     end Set_Scan;

     procedure Set_Ext
       (Lparam : Win32.Lparam;
        Line   : in out String) is

     begin

       if integer(Lparam)/ 16#1000000# /= 1 then

         Set_Message_In_Scratch_Line
           (Start => 
             (Ext_Start +
              Ext_Length)-
              No_String'Length,
            Input => 
              No_String,
            Line  => 
              Line);

       else

         Set_Message_In_Scratch_Line
           (Start => 
             (Ext_Start +
              Ext_Length) -
              Yes_String'Length,
            Input => 
             Yes_String,
            Line  => 
             Line);

       end if;

     end Set_Ext;

     procedure Set_Alt
       (Lparam : Win32.Lparam;
        Line   : in out String) is

     begin

       if integer(Lparam)/ 16#20000000# /= 1 then

         Set_Message_In_Scratch_Line
           (Start => 
             (Alt_Start +
              Alt_Length)-
             No_String'Length,
            Input => 
             No_String,
            Line  => 
             Line);

       else

         Set_Message_In_Scratch_Line
           (Start => 
             (Alt_Start +
              Alt_Length) -
              Yes_String'Length,
            Input => 
             Yes_String,
            Line  => 
             Line);

       end if;

     end Set_Alt;

     procedure Set_Prev
       (Lparam : Win32.Lparam;
        Line   : in out String) is

     begin

       if integer(Lparam)/ 16#40000000# /= 1 then

         Set_Message_In_Scratch_Line
           (Start => 
             (Prev_Start +
              Prev_Length)-
              Up_String'Length,
            Input => 
             Up_String,
            Line  => 
             Line);

       else

         Set_Message_In_Scratch_Line
           (Start => 
             (Prev_Start +
              Prev_Length) -
              Down_String'Length,
            Input => 
             Down_String,
            Line  => 
             Line);

       end if;

     end Set_Prev;

     procedure Set_Tran
       (Lparam : Win32.Lparam;
        Line   : in out String) is

     begin

       if integer(Lparam) > 1 then

         Set_Message_In_Scratch_Line
           (Start => 
             (Tran_Start +
              Tran_Length)-
              Down_String'Length,
            Input => 
             Down_String,
            Line  => 
             Line);

       else

         Set_Message_In_Scratch_Line
           (Start => 
             (Tran_Start +
              Tran_Length) -
              Up_String'Length,
            Input => 
             Up_String,
            Line  => 
             Line);

       end if;

     end Set_Tran;

    begin

      Set_Message
        (Message =>
          Message,
         Line    => 
          Scratch_Line);

      Set_Key
        (Key   => Wparam,
         Itype => Itype,
         Line  => Scratch_Line);

      Set_Itype
        (Itype  => Itype,
         Wparam => Wparam,
         Line   => Scratch_Line);

      Set_Repeat
        (Lparam => Lparam,
         Line   => Scratch_Line);

      Set_Scan
        (Lparam => Lparam,
         Line   => Scratch_Line);

      Set_Ext
        (Lparam => Lparam,
         Line   => Scratch_Line);

      Set_Alt
        (Lparam => Lparam,
         Line   => Scratch_Line);

      Set_Prev
        (Lparam => Lparam,
         Line   => Scratch_Line);

      Set_Tran
        (Lparam => Lparam,
         Line   => Scratch_Line);

      Scratch_Line_Pointer := 
        Interfaces.C.Strings.New_String
         (Str => Scratch_Line);

      Bool_Result := 
        Win32.Wingdi.Textout
	  (Hdc      =>
            Device_Context_Handle,
           Nxstart  => 
             Win32.Int
               (Character_Width),
           Nystart  => 
             Win32.Int
               (Rectangle.Bottom -
                Character_Height),
           Lpstring => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Scratch_Line_Pointer),
           Cbstring => 
             Win32.Int
               (Interfaces.C.Strings.Strlen
                 (Item => 
                   Scratch_Line_Pointer)));

        Interfaces.C.Strings.Free
          (Item =>
            Scratch_Line_Pointer);

    end Set_Output_Text;

   begin

     Bool_Result := 
       Win32.Winuser.Scrollwindow
       (Hwnd       => 
         Window_Handle,
        Xamount    => 0,
        Yamount    => 
          Win32.Int
            (- Character_Height),
        Lprect     => 
          Rectangle'Access,
        Lpcliprect =>
          Rectangle'Access);

     Device_Context_Handle := 
       Win32.Winuser.Getdc
         (Hwnd =>
           Window_Handle);

     Object_Result :=
       Win32.Wingdi.SelectObject
         (Hdc    =>
           Device_Context_Handle,
          Hgdiobj =>
           Win32.Wingdi.Getstockobject
            (Fnobject => 
              Win32.Wingdi.System_Fixed_Font));

     Set_Output_Text
       (Window_Handle => 
         Window_Handle,
        Itype         => 
         Itype,
        Message       => 
         Message,
        Wparam        => 
         Wparam,
        Lparam        => 
         Lparam);

     Integer_Result :=
       Win32.Winuser.Releasedc
         (Hwnd => 
           Window_Handle,
          Hdc  => 
           Device_Context_Handle);

     Bool_Result :=
       Win32.Winuser.ValidateRect
         (Hwnd   =>
            Window_Handle,
          Lprect => 
           null);

   end Show_Key;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message : Win32.Uint;
      Wparam : Win32.Wparam;
      Lparam : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

	    Device_Context_Handle := 
              Win32.Winuser.Getdc
                (Hwnd => Window_Handle);

            Object_Result :=
              Win32.Wingdi.SelectObject
                (Hdc    =>
                  Device_Context_Handle,
                 Hgdiobj => 
                  Win32.Wingdi.Getstockobject
                    (Fnobject => 
                      Win32.Wingdi.System_Fixed_Font));

	    Bool_Result := 
               Win32.Wingdi.Gettextmetrics
                 (Hdc => 
                   Device_Context_Handle,
                  Lptm => 
                   Text_Metric'Access);

            Character_Width :=
              Text_Metric.Tmavecharwidth;

            Character_Height := 
              Text_Metric.Tmheight;

            Integer_Result := 
              Win32.Winuser.Releasedc
                (Hwnd =>
                  Window_Handle,
                 Hdc  => 
                  Device_Context_Handle);

            Rectangle.Top :=
              3 * Character_Height / 2;

            return (0);

	 when Win32.Winuser.Wm_Size =>

            Rectangle.Right := 
              Win32.Long
                (Win32.Utils.Loword
                  (Win32.Dword
                    (Lparam)));

            Rectangle.Bottom := 
              Win32.Long
                (Win32.Utils.Hiword
                  (Win32.Dword
                    (Lparam)));

            Bool_Result := 
              Win32.Winuser.Updatewindow
                (Hwnd => 
                  Window_Handle);

            return (0);

	 when Win32.Winuser.Wm_Paint =>

            Bool_Result :=
              Win32.Winuser.Invalidaterect
                (Hwnd   => 
                  Window_Handle,
                 Lprect => 
                  null,
                 Berase =>
                  True);

	    Device_Context_Handle :=
              Win32.Winuser.Beginpaint
	        (Hwnd    => 
                  Window_Handle,
	         Lppaint => 
                  Paint_Record'Access);

            Object_Result :=
              Win32.Wingdi.SelectObject
                (Hdc    => 
                  Device_Context_Handle,
                 Hgdiobj => 
                  Win32.Wingdi.Getstockobject
                   (Fnobject => 
                     Win32.Wingdi.System_Fixed_Font));

            Integer_Result :=
              Win32.Wingdi.Setbkmode
                (Hdc      =>
                  Device_Context_Handle,
                 Fnbkmode => 
                  Win32.Wingdi.Transparent);

	    Bool_Result := 
              Win32.Wingdi.Textout
	        (Hdc   => 
                  Device_Context_Handle,
                 Nxstart  => 
                  Win32.Int
                     (Character_Width),
                 Nystart  => 
                   Win32.Int
                     (Character_Height / 2),
                 Lpstring => 
                  Convert_Char_Ptr_To_Lpcstr
                    (S => 
                      Top_Line),
                 Cbstring => 
                  Win32.Int
                    (Interfaces.C.Strings.Strlen
                      (Item => 
                        Top_Line)));

	    Bool_Result := 
              Win32.Wingdi.Textout
	        (Hdc   => 
                  Device_Context_Handle,
                 Nxstart  => 
                   Win32.Int
                     (Character_Width),
                 Nystart  => 
                   Win32.Int
                     (Character_Height / 2),
                 Lpstring => 
                  Convert_Char_Ptr_To_Lpcstr
                    (S => 
                      Underline_Line),
                 Cbstring => 
                  Win32.Int
                    (Interfaces.C.Strings.Strlen
                      (Item => 
                        Underline_Line)));

	    Bool_Result :=
              Win32.Winuser.Endpaint
                (Hwnd   => 
                  Window_Handle,
                 Lppaint =>
                  Paint_Record'Access);

	    return (0);

	 when Win32.Winuser.Wm_Keydown =>

            Show_Key
              (Window_Handle =>
                Window_Handle,
               Itype         => 
                0,
               Message       =>    
                "Wm_Keydown",
               Wparam        => 
                Wparam,
               Lparam        => 
                Lparam);
            
            return (0);

	 when Win32.Winuser.Wm_Keyup  =>

            Show_Key
              (Window_Handle => 
                Window_Handle,
               Itype         => 
                0,
               Message       => 
                "Wm_Keyup",
               Wparam        => 
                Wparam,
               Lparam        => 
                Lparam);
            
            return (0);

	 when Win32.Winuser.Wm_Char =>

            Show_Key
              (Window_Handle => 
                Window_Handle,
               Itype         => 
                1,
               Message       => 
                "Wm_Char",
               Wparam        => 
                Wparam,
               Lparam        =>
                Lparam);
            
            return (0);

	 when Win32.Winuser.Wm_Deadchar =>

            Show_Key
              (Window_Handle => 
                Window_Handle,
               Itype   => 
                1,
               Message => 
                "Wm_Deadchar",
               Wparam  => 
                Wparam,
               Lparam  => 
                Lparam);
            
            return (0);

	 when Win32.Winuser.Wm_Syskeydown  =>

            Show_Key
              (Window_Handle =>
                Window_Handle,
               Itype         => 
                0,
               Message       => 
                "Wm_Syskeydown",
               Wparam        => 
                Wparam,
               Lparam        => 
                Lparam);
            
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => 
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

	 when Win32.Winuser.Wm_Syskeyup  =>

            Show_Key
              (Window_Handle => 
                Window_Handle,
               Itype         => 
                0,
               Message       => 
                "Wm_Syskeyup",
               Wparam        => 
                Wparam,
               Lparam        => 
                Lparam);
            
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => 
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

	 when Win32.Winuser.Wm_Syschar =>

            Show_Key
              (Window_Handle => 
                Window_Handle,
               Itype         => 
                1,
               Message       => 
                "Wm_Syschar",
               Wparam        => 
                Wparam,
               Lparam        => 
                Lparam);
            
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => 
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

	 when Win32.Winuser.Wm_Sysdeadchar =>

            Show_Key
              (Window_Handle => 
                Window_Handle,
               Itype         => 
                1,
               Message       => 
                "Wm_Sysdeadchar",
               Wparam        => 
                Wparam,
               Lparam        => 
                Lparam);
            
	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => 
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => 
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
