with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with System_Metrics;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
   use type System.Address;
   use type Win32.Dword;
   use type Win32.Bool;
   use type Win32.Byte;
   use type Win32.Int;
   use type Win32.Long;
   use type Win32.Uint;

   Text_Metric : aliased win32.Wingdi.Textmetric;

   Character_Width : win32.Long;

   Character_Height : win32.Long;

   Cap_Width : win32.Long;

   Temporary_String_Pointer : Interfaces.C.Strings.Chars_Ptr ;

   Align_Top_Right : constant Win32.Uint := 
     Win32.Wingdi.Ta_Right or 
     Win32.Wingdi.Ta_Top;

   Align_Top_Left  : constant Win32.Uint := 
     Win32.Wingdi.Ta_Left or 
     Win32.Wingdi.Ta_Top;

   Vertical_Scroll_Position : win32.Uint := 0;

   Temporary_Vertical_Scroll_Position : win32.Uint := 0;

   Temporary_Horizontal_Scroll_Position : win32.Uint := 0;

   Vertical_Position : win32.Uint := 0;

   Horizontal_Scroll_Position : win32.Uint := 0;

   Horizontal_Position : win32.Uint := 0;

   Begin_Paint : win32.Uint := 0;

   End_Paint : win32.Uint := 0;

   Vertical_Scroll_Increment : win32.Uint := 0;

   Horizontal_Scroll_Increment : win32.Uint := 0;

   True : constant Win32.bool := 1;

   False : constant Win32.bool := 0;

   Minimum_Vertical_Scroll_Position : constant Win32.Int := 0;

   Maximum_Vertical_Scroll : Win32.Int := 0;

   Minimum_Horizontal_Scroll_Position : constant Win32.Int := 0;

   Maximum_Horizontal_Scroll : Win32.Int := 0;

   Client_Width : win32.Long;

   Client_Height : win32.Long;

   Maximum_Width : win32.Long;

   Paint_Record : aliased win32.Winuser.Paintstruct;

   Window_Class : aliased Win32.Winuser.Wndclass;

   Message : aliased Win32.Winuser.Msg;

   Device_Context_Handle : aliased Win32.Windef.Hdc;

   Integer_Result        : Win32.Int;
   Bool_Result           : Win32.Bool;
   Long_Result           : Win32.Long;
   Atom_Result           : Win32.Windef.Atom;
   Uint_Result           : Win32.Uint;

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   procedure Convert_Integer_To_Char_Array
     (Int : in Integer;
      Ptr : in out Interfaces.C.Strings.Chars_Ptr) is

     Blank_String : string := "         ";

   begin

     Blank_String
       ((Blank_String'Length - integer'Image(Int)'Length) + 1 .. 
         Blank_String'Length) :=
          integer'Image(Int);

     Ptr := 
       Interfaces.C.Strings.New_String
         (Str => 
           Blank_String);

   end Convert_Integer_To_Char_Array;

--  function Min is new Win32.Windef.Min(T => integer, "<" => "<" );
--
--    function Max is new Win32.Windef.Max(T => integer, ">" => ">" );

   function Min(l,r : integer) return integer is
   begin
     if l < r then
       return l;
     else
       return r;
     end if;
   end Min;

   function Max(l,r : integer) return integer is
   begin
     if l > r then
       return l;
     else
       return r;
     end if;
   end Max;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               System.Null_Address, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

	    Device_Context_Handle := 
              Win32.Winuser.Getdc
                (Hwnd => Window_Handle);

	    Bool_Result := 
              Win32.Wingdi.Gettextmetrics
                (Hdc => 
                  Device_Context_Handle,
                 Lptm => 
                  Text_Metric'Access);

            Character_Width :=
              Text_Metric.Tmavecharwidth;

            if Text_Metric.Tmpitchandfamily mod 2 > 0 then

              Cap_Width :=
                (3 * Character_Width)/2;

            else

              Cap_Width :=
                (2 * Character_Width)/2;

            end if;

            Character_Height := 
              Text_Metric.TmHeight +
              Text_Metric.Tmexternalleading;

            Integer_Result := 
              Win32.Winuser.Releasedc
                (Hwnd => 
                  Window_Handle,
                 Hdc  => 
                  Device_Context_Handle);

            Maximum_Width :=
              60 * 
              Character_Width + 
              22 * 
              Cap_Width; 

            return (0);

	 when Win32.Winuser.Wm_Size =>

            Client_Width := 
              Win32.Long
                (Win32.Utils.Loword
                  (Win32.Dword
                    (Lparam)));

            Client_Height := 
              Win32.Long
                (Win32.Utils.Hiword
                  (Win32.Dword
                    (Lparam)));

            Maximum_Vertical_Scroll :=
              Win32.Int
                (Max
                  (L => 
                    0,
                   R => Integer
                    (System_Metrics.Sm'Pos
                      (System_Metrics.Sm'Last) + 2 - 
                     Client_Height / 
                     Character_Height)));

            Vertical_Scroll_Position :=
              Win32.Uint
                (Min
                  (L => 
                    Integer(Vertical_Scroll_Position),
                   R => 
                    Integer(Maximum_Vertical_Scroll)));

            Bool_Result := 
              Win32.Winuser.Setscrollrange
                (Hwnd => 
                  Window_Handle,
                 Nbar => 
                  Win32.Winuser.Sb_Vert,
                 Nminpos => 
                  0,
                 Nmaxpos => 
                  Maximum_Vertical_Scroll,
                 Bredraw => 
                  False);

            Integer_Result := 
              Win32.Winuser.Setscrollpos
                (Hwnd    => 
                  Window_Handle,
                 Nbar    => 
                  Win32.Winuser.Sb_Vert,
                 Npos    => 
                  Win32.Int
                    (Vertical_Scroll_Position),
                 Bredraw =>
                    True);

            Maximum_Horizontal_Scroll :=
              Win32.Int
                (Max
                  (L => 
                    0,
                   R => 
                     Integer
                       (2 + (Maximum_Width -
                        Client_Width) 
                        / Character_Width)));

            Vertical_Scroll_Position :=
              Win32.Uint
                (Min
                  (L => 
                    Integer
                      (Horizontal_Scroll_Position),
                   R => 
                    Integer
                      (Maximum_Horizontal_Scroll)));

            Bool_Result := 
              Win32.Winuser.Setscrollrange
                (Hwnd => 
                  Window_Handle,
                 Nbar => 
                  Win32.Winuser.Sb_Horz,
                 Nminpos => 
                  0,
                 Nmaxpos => 
                  Maximum_Horizontal_Scroll,
                 Bredraw => 
                  False);

            Integer_Result := 
              Win32.Winuser.Setscrollpos
                (Hwnd    =>
                  Window_Handle,
                 Nbar    => 
                  Win32.Winuser.Sb_Horz,
                 Npos    => 
                  Win32.Int
                    (Horizontal_Scroll_Position),
                 Bredraw =>
                    True);

            return (0);

	 when Win32.Winuser.Wm_Vscroll =>

            case Win32.Utils.Loword
                   (Win32.Dword
                     (Wparam)) is

              when Win32.Winuser.Sb_Top =>

                 Vertical_Scroll_Increment :=
                   - Vertical_Scroll_Position;

              when Win32.Winuser.Sb_Bottom =>

                 Vertical_Scroll_Increment :=
                    Win32.Uint
                      (Maximum_Vertical_Scroll) -
                       Vertical_Scroll_Position;

              when Win32.Winuser.Sb_Lineup =>

                 Vertical_Scroll_Increment := 
                   - 1;

              when Win32.Winuser.Sb_Linedown =>

                 Vertical_Scroll_Increment := 1; 

              when Win32.Winuser.Sb_Pageup =>

                 Vertical_Scroll_Increment := 
                   Win32.Uint
                     (Min
                       (L => 
                         -1,
                        R => 
                         Integer
                           (- Client_Height 
                            / Character_Height)));

              when Win32.Winuser.Sb_Pagedown =>

                 Vertical_Scroll_Increment := 
                   Win32.Uint
                     (Max
                       (L => 
                         1,
                        R => 
                         Integer
                           (Client_Height / 
                            Character_Height)));

              when Win32.Winuser.Sb_Thumbtrack =>

                 Vertical_Scroll_Increment := 
                   Win32.Uint
                     (Win32.Utils.Hiword
                       (Win32.Dword
                         (Wparam))) - 
                      Vertical_Scroll_Position;
                
              when others =>

                 Vertical_Scroll_Increment :=  0;

            end case;

            Vertical_Scroll_Increment :=
               Win32.Uint
                 (Max
                   (L =>
                     Integer
                       (-Vertical_Scroll_Position),
                    R => 
                     Min
                      (L => 
                        Integer
                          (Vertical_Scroll_Increment),
                       R => 
                        Integer
                          (Win32.Uint
                            (Maximum_Vertical_Scroll) - 
                           Vertical_Scroll_Position))));

            if Vertical_Scroll_Increment /= 0  then

               Vertical_Scroll_Position :=
                 Vertical_Scroll_Position + 
                 Vertical_Scroll_Increment;

               Bool_Result := 
                 Win32.Winuser.Scrollwindow
                   (Hwnd       =>
                     Window_Handle,
                    Xamount    => 0,
                    Yamount    => 
                     Win32.Int
                       (- Win32.Uint
                         (Character_Height) * 
                        Vertical_Scroll_Increment),
                    Lprect     => 
                     null,
                  Lpcliprect => 
                     null);

            Integer_Result := 
              Win32.Winuser.Setscrollpos
                (Hwnd    => 
                  Window_Handle,
                 Nbar    => 
                  Win32.Winuser.Sb_Vert,
                 Npos    => 
                  Win32.Int
                    (Vertical_Scroll_Position),
                 Bredraw => True);

            Bool_Result := 
              Win32.Winuser.Updatewindow
                (Hwnd => Window_Handle);

            end if;

            return (0);

	 when Win32.Winuser.Wm_Hscroll =>

            case Win32.Utils.Loword
                   (Win32.Dword
                     (Wparam)) is

              when Win32.Winuser.Sb_Lineup =>

                 Horizontal_Scroll_Increment :=
                   - 1;

              when Win32.Winuser.Sb_Linedown =>

                 Horizontal_Scroll_Increment := 
                   1; 

              when Win32.Winuser.Sb_Pageup =>

                 Horizontal_Scroll_Increment := 
                   -8; 

              when Win32.Winuser.Sb_Pagedown =>

                 Horizontal_Scroll_Increment := 
                   8;

              when Win32.Winuser.Sb_Thumbposition =>

                 Horizontal_Scroll_Increment := 
                   Win32.Uint
                     (Win32.Utils.Hiword
                       (Win32.Dword
                         (Wparam))) - 
                   Horizontal_Scroll_Position;
                
              when others =>

                 Horizontal_Scroll_Increment :=  0;

            end case;

            Horizontal_Scroll_Increment :=
               Win32.Uint
                 (Max
                   (L =>  
                     Integer
                       (-Horizontal_Scroll_Position),
                    R => 
                     Min
                       (L => 
                         Integer
                           (Horizontal_Scroll_Increment),
                        R => 
                         Integer
                           (Win32.Uint
                             (Maximum_Horizontal_Scroll) - 
                            Horizontal_Scroll_Position))));

            if Horizontal_Scroll_Increment /= 0 then

               Horizontal_Scroll_Position :=
                 Horizontal_Scroll_Position + 
                 Horizontal_Scroll_Increment;

               Bool_Result := 
                 Win32.Winuser.Scrollwindow
                   (Hwnd       =>
                     Window_Handle,
                    Xamount    => 
                     Win32.Int
                       (- Win32.Uint
                         (Character_Width) * 
                        Horizontal_Scroll_Increment),
                    Yamount    => 
                     0,
                    Lprect     => 
                     null,
                    Lpcliprect => 
                     null);

            Integer_Result := 
              Win32.Winuser.Setscrollpos
                (Hwnd    =>
                  Window_Handle,
                 Nbar    => 
                  Win32.Winuser.Sb_Horz,
                 Npos    => 
                  Win32.Int
                    (Horizontal_Scroll_Position),
                 Bredraw =>
                  True);

            end if;

            return (0);

	 when Win32.Winuser.Wm_Keydown =>

            case Wparam is

              when Win32.Winuser.Vk_Home =>

                Long_Result :=
                  Win32.Winuser.SendMessage
                    (Hwnd   =>
                      Window_Handle,
                     Msg    => 
                      Win32.Winuser.Wm_Vscroll,
                     Wparam => 
                      Win32.Winuser.Sb_Top,
                     Lparam => 
                      0);
                
              when Win32.Winuser.Vk_End =>

                Long_Result := 
                  Win32.Winuser.SendMessage
                    (Hwnd   => 
                      Window_Handle,
                     Msg    => 
                      Win32.Winuser.Wm_Vscroll,
                     Wparam => 
                      Win32.Winuser.Sb_Bottom,
                     Lparam => 
                      0);
                
              when Win32.Winuser.Vk_Prior =>

                Long_Result := 
                  Win32.Winuser.SendMessage
                    (Hwnd   => 
                      Window_Handle,
                     Msg    => 
                      Win32.Winuser.Wm_Vscroll,
                     Wparam => 
                      Win32.Winuser.Sb_Pageup,
                     Lparam => 
                      0);
                
              when Win32.Winuser.Vk_Next =>

                Long_Result := 
                  Win32.Winuser.SendMessage
                    (Hwnd   => 
                      Window_Handle,
                     Msg    => 
                      Win32.Winuser.Wm_Vscroll,
                     Wparam => 
                      Win32.Winuser.Sb_Pagedown,
                     Lparam => 
                      0);
                
              when Win32.Winuser.Vk_Up =>

                Long_Result :=
                  Win32.Winuser.SendMessage
                    (Hwnd   => 
                      Window_Handle,
                     Msg    => 
                      Win32.Winuser.Wm_Vscroll,
                     Wparam => 
                      Win32.Winuser.Sb_Lineup,
                     Lparam => 
                      0);
                
              when Win32.Winuser.Vk_Down =>

                Long_Result := 
                  Win32.Winuser.SendMessage
                    (Hwnd   =>
                      Window_Handle,
                     Msg    => 
                      Win32.Winuser.Wm_Vscroll,
                     Wparam => 
                      Win32.Winuser.Sb_Linedown,
                     Lparam => 
                      0);
                
              when Win32.Winuser.Vk_Left =>

                Long_Result := 
                  Win32.Winuser.SendMessage
                    (Hwnd   =>
                      Window_Handle,
                     Msg    => 
                      Win32.Winuser.Wm_Hscroll,
                     Wparam => 
                      Win32.Winuser.Sb_Pageup,
                     Lparam => 
                      0);
                
              when Win32.Winuser.Vk_Right =>

                Long_Result := 
                  Win32.Winuser.SendMessage
                    (Hwnd   =>
                      Window_Handle,
                     Msg    => 
                      Win32.Winuser.Wm_Hscroll,
                     Wparam => 
                      Win32.Winuser.Sb_Pagedown,
                     Lparam => 
                      0);
                
              when others =>

                 null;

            end case;

            return (0);

	 when Win32.Winuser.Wm_Paint =>

	    Device_Context_Handle := 
              Win32.Winuser.Beginpaint
	        (Hwnd    =>
                  Window_Handle,
	         Lppaint => 
                  Paint_Record'Access);

            Begin_Paint := 
              Win32.Uint
                (Max
                  (L => 
                    0,
                   R => 
                    Integer
                      (Win32.Long
                        (Vertical_Scroll_Position) + 
                       Paint_Record.Rcpaint.Top / 
                       Character_Height - 1 )));

            End_Paint := 
              Win32.Uint
                (Min
                  (L => 
                    System_Metrics.Sm'Pos
                      (System_Metrics.Sm'Last),
                   R => 
                     Integer
                       (Win32.long
                         (Vertical_Scroll_Position) + 
                        Paint_Record.Rcpaint.Bottom / 
                        Character_Height)));

            for i in System_Metrics.Sm'Val
                       (Begin_Paint) .. 
                     System_Metrics.Sm'Val
                       (End_Paint) loop

              Temporary_Horizontal_Scroll_Position :=
                Win32.Uint
                  (Character_Width) *
                  (1 - Horizontal_Scroll_Position);

              Temporary_Vertical_Scroll_Position :=
                Win32.Uint
                  (Character_Height) *
                  (1 - 
                   Vertical_Scroll_Position + 
                     System_Metrics.Sm'Pos
                       (i));

	      Bool_Result := 
                Win32.Wingdi.Textout
	          (Hdc   => 
                    Device_Context_Handle,
                   Nxstart  => 
                    Win32.Int
                     (Temporary_Horizontal_Scroll_Position),
                   Nystart  => 
                     Win32.Int
                       (Temporary_Vertical_Scroll_Position),
                   Lpstring => 
                    Convert_Char_Ptr_To_Lpcstr
                      (S => 
                        System_Metrics.Metrics_Record_Array(i).Label),
                   Cbstring => 
                     Win32.Int
                       (Interfaces.C.Strings.Strlen
                         (Item => 
                           System_Metrics.Metrics_Record_Array(i).Label)));

	      Bool_Result := 
                Win32.Wingdi.Textout
	          (Hdc   =>
                    Device_Context_Handle,
                   Nxstart  => 
                    Win32.Int
                      (Temporary_Horizontal_Scroll_Position + 
                       22 * Win32.Uint
                         (Cap_Width)),
                   Nystart  => 
                     Win32.Int
                       (Temporary_Vertical_Scroll_Position),
                   Lpstring => 
                    Convert_Char_Ptr_To_Lpcstr
                      (S => 
                        System_Metrics.Metrics_Record_Array(i).Description),
                   Cbstring => 
                     Win32.Int
                       (Interfaces.C.Strings.Strlen
                         (Item => 
                           System_Metrics.Metrics_Record_Array(i).Description)));

              Uint_Result :=
                Win32.Wingdi.Settextalign
                  (Hdc => 
                    Device_Context_Handle,
                   Fmode =>
                    Align_Top_Right);

	      Convert_Integer_To_Char_Array
                (Int => 
                  Integer(Win32.Winuser.GetsystemMetrics
                    (System_Metrics.Sm'Pos(i))),
                 Ptr => 
                  Temporary_String_Pointer);

              Bool_Result := 
                Win32.Wingdi.Textout
	          (Hdc   =>
                    Device_Context_Handle,
                   Nxstart  => 
                    Win32.Int
                      (Temporary_Horizontal_Scroll_Position + 
                       22 * Win32.Uint
                              (Cap_Width) + 
                       60 * Win32.Uint
                              (Character_Width)),
                   Nystart  => 
                    Win32.Int
                      (Temporary_Vertical_Scroll_Position),
                   Lpstring => 
                    Convert_Char_Ptr_To_Lpcstr
                      (S => 
                        Temporary_String_Pointer),
                   Cbstring => 
                     Win32.Int
                       (Interfaces.C.Strings.Strlen
                         (Item => 
                           Temporary_String_Pointer)));

              Interfaces.C.Strings.Free
                (Item =>
                  Temporary_String_Pointer);

              Uint_Result := 
                Win32.Wingdi.Settextalign
                  (Hdc =>
                    Device_Context_Handle,
                   Fmode => 
                    Align_Top_Left);

            end loop;

	    Bool_Result := 
              Win32.Winuser.Endpaint
               (Hwnd   => 
                 Window_Handle,
                Lppaint =>
                 Paint_Record'Access);

	    return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode =>
                0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   => 
                Window_Handle, 
	       Msg    =>
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
