with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type Interfaces.C.Unsigned;
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Int;

   Rectangle                : aliased win32.Windef.Rect;

   Begining_Point           : Win32.Windef.Point;

   Ending_Point             : Win32.Windef.Point;

   Box_Begining_Point       : Win32.Windef.Point;

   Box_Ending_Point         : Win32.Windef.Point;

   True                     : constant Win32.Bool := 1;

   False                    : constant Win32.Bool := 0;

   Is_Blocking              : Win32.Bool := False;

   Is_Valid_Box             : Win32.Bool := False;

   Paint_Record             : aliased win32.Winuser.Paintstruct;

   Window_Class             : aliased Win32.Winuser.Wndclass;

   Message                  : aliased Win32.Winuser.Msg;

   Device_Context_Handle    : aliased Win32.Windef.Hdc;

   Integer_Result  : Win32.Int;

   Bool_Result     : Win32.Bool;

   Long_Result     : Win32.Long;

   Atom_Result     : Win32.Windef.Atom;

   Object_Result   : Win32.Windef.Hgdiobj;

   Cursor_Result   : Win32.Windef.Hcursor;

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   procedure Draw_Box_Outline
     (Window_Handle  : Win32.Windef.Hwnd;
      Begining_Point : Win32.Windef.Point;
      Ending_Point   : Win32.Windef.Point) is

     Device_Context_Handle    : aliased Win32.Windef.Hdc;

   begin 

     Device_Context_Handle := 
       Win32.Winuser.Getdc
         (Hwnd =>
           Window_Handle);

     Integer_Result :=
       Win32.Wingdi.Setrop2
         (Hdc        =>
           Device_Context_Handle,
          Fndrawmode => 
           Win32.Wingdi.R2_Not);

     Object_Result :=
       Win32.Wingdi.Selectobject
         (Hdc    =>
           Device_Context_Handle,
          Hgdiobj =>
           Win32.Wingdi.Getstockobject
            (Fnobject => 
              Win32.Wingdi.Null_Brush));

     Bool_Result :=
       Win32.Wingdi.Rectangle
         (Hdc         => 
           Device_Context_Handle,
          Nleftrect   =>
           Win32.Int
             (Begining_Point.X),
          Ntoprect    =>
           Win32.Int
             (Begining_Point.Y),
          Nrightrect  =>
           Win32.Int
             (Ending_Point.X),
          Nbottomrect =>
           Win32.Int
             (Ending_Point.Y));

     Integer_Result := 
       Win32.Winuser.Releasedc
         (Hwnd =>
           Window_Handle,
          Hdc  =>
           Device_Context_Handle);

   end Draw_Box_Outline;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.Winuser.Wm_Lbuttondown =>

           Begining_Point.X :=
             Win32.Long
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam)));

           Ending_Point.X :=
             Begining_Point.X;

           Begining_Point.Y :=
             Win32.Long
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam)));

           Ending_Point.Y :=
             Begining_Point.X;
           
           Draw_Box_Outline
             (Window_Handle  =>
               Window_Handle,
              Begining_Point =>
               Begining_Point,
              Ending_Point   =>
               Ending_Point);

           Cursor_Result := 
             Win32.Winuser.Setcursor
               (Hcursor =>
                 Win32.Winuser.Loadcursor
                   (Hinstance =>
                     null,
                    Lpcursorname => 
                     Win32.Lpcstr
                       (Win32.Winuser.Idc_Arrow)));

           Is_Blocking :=
             True;

           return (0);

	 when Win32.Winuser.Wm_Mousemove =>

           If Is_Blocking = True then

             Cursor_Result := 
               Win32.Winuser.Setcursor
                 (Hcursor =>
                   Win32.Winuser.Loadcursor
                     (Hinstance =>
                       null,
                      Lpcursorname => 
                       Win32.Lpcstr
                         (Win32.Winuser.Idc_Arrow)));

             Draw_Box_Outline
               (Window_Handle  =>
                 Window_Handle,
                Begining_Point =>
                 Begining_Point,
                Ending_Point   =>
                 Ending_Point);
             

             Ending_Point.X :=
               Win32.Long
                 (Win32.Utils.Loword
                   (Win32.Dword
                     (Lparam)));

             Ending_Point.Y :=
               Win32.Long
                 (Win32.Utils.Hiword
                   (Win32.Dword
                     (Lparam)));

             Draw_Box_Outline
               (Window_Handle  =>
                 Window_Handle,
                Begining_Point =>
                 Begining_Point,
                Ending_Point   =>
                 Ending_Point);
                 
           end if;

           return (0);

	 when Win32.Winuser.Wm_Lbuttonup =>

           If Is_Blocking = True then

             Draw_Box_Outline
               (Window_Handle  =>
                 Window_Handle,
                Begining_Point =>
                 Begining_Point,
                Ending_Point   =>
                 Ending_Point);
             
             Box_Begining_Point :=
               Begining_Point;
   
             Box_Ending_Point.X :=
               Win32.Long
                 (Win32.Utils.Loword
                   (Win32.Dword
                     (Lparam)));

             Box_Ending_Point.Y :=
               Win32.Long
                 (Win32.Utils.Hiword
                   (Win32.Dword
                     (Lparam)));

             Cursor_Result := 
               Win32.Winuser.Setcursor
                 (Hcursor =>
                   Win32.Winuser.Loadcursor
                     (Hinstance =>
                       null,
                      Lpcursorname => 
                       Win32.Lpcstr
                         (Win32.Winuser.Idc_Arrow)));


             Is_Blocking :=
               False;

             Is_Valid_Box :=
               True;

             Bool_Result := 
               Win32.Winuser.InvalidateRect
                 (Hwnd   => 
                   Window_Handle,
                  Lprect => 
                   null,
                  Berase  => 
                   True);

           end if;

           return (0);

	 when Win32.Winuser.Wm_Char =>

           if Is_Blocking = True and Wparam = 27 then

             Draw_Box_Outline
               (Window_Handle  =>
                 Window_Handle,
                Begining_Point =>
                 Begining_Point,
                Ending_Point   =>
                 Ending_Point);

             Cursor_Result := 
               Win32.Winuser.Setcursor
                 (Hcursor =>
                   Win32.Winuser.Loadcursor
                     (Hinstance =>
                       null,
                      Lpcursorname => 
                       Win32.Lpcstr
                         (Win32.Winuser.Idc_Arrow)));

             Is_Blocking :=
               False;

           end if;

           return (0);

	 when Win32.Winuser.Wm_Paint =>

	   Device_Context_Handle :=
             Win32.Winuser.Beginpaint
	       (Hwnd    =>                                                       
                 Window_Handle,
	        Lppaint =>
                 Paint_Record'Access);

             if Is_Valid_Box = True then

               Object_Result :=
                 Win32.Wingdi.Selectobject
                   (Hdc    =>
                     Device_Context_Handle,
                    Hgdiobj =>
                     Win32.Wingdi.Getstockobject
                       (Fnobject => 
                         Win32.Wingdi.Black_Brush));

               Bool_Result :=
                 Win32.Wingdi.Rectangle
                   (Hdc         => 
                     Device_Context_Handle,
                    Nleftrect   =>
                     Win32.Int
                       (Box_Begining_Point.X),
                    Ntoprect    =>
                     Win32.Int
                       (Box_Begining_Point.Y),
                    Nrightrect  =>
                     Win32.Int
                       (Box_Ending_Point.X),
                    Nbottomrect =>
                     Win32.Int
                       (Box_Ending_Point.Y));

             end if;

             if Is_Blocking = True then

               Integer_Result :=
                 Win32.Wingdi.Setrop2
                   (Hdc        =>
                     Device_Context_Handle,
                    Fndrawmode => 
                     Win32.Wingdi.R2_Not);

               Object_Result :=
                 Win32.Wingdi.Selectobject
                   (Hdc    =>
                     Device_Context_Handle,
                    Hgdiobj =>
                     Win32.Wingdi.Getstockobject
                       (Fnobject => 
                         Win32.Wingdi.Null_Brush));

               Bool_Result :=
                 Win32.Wingdi.Rectangle
                   (Hdc         => 
                     Device_Context_Handle,
                    Nleftrect   =>
                     Win32.Int
                       (Begining_Point.X),
                    Ntoprect    =>
                     Win32.Int
                       (Begining_Point.Y),
                    Nrightrect  =>
                     Win32.Int
                       (Ending_Point.X),
                    Nbottomrect =>
                     Win32.Int
                       (Ending_Point.Y));

             end if;

    	     Bool_Result :=
               Win32.Winuser.Endpaint
                 (Hwnd    =>
                   Window_Handle,
                  Lppaint =>
                   Paint_Record'Access);

	    return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
                0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
