with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type Interfaces.C.Unsigned;
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Int;

   type Bool_Array_Type is array ( Win32.Int range <>,Win32.Int Range <>) of Win32.Bool;

   Number_Of_Boxes         : constant Win32.Int := 5;   

   Bool_Array              : Bool_Array_Type(0 .. Number_Of_Boxes - 1,
                                             0 .. Number_Of_Boxes - 1);

   Current_X                : Win32.Int := 0;

   Current_Y                : Win32.Int := 0;

   Box_Width                : Win32.Int := 0;

   Box_Height               : Win32.Int := 0;

   Rectangle                : aliased win32.Windef.Rect;

   Point                    : aliased win32.Windef.Point;

   True                     : constant Win32.Bool := 1;

   False                    : constant Win32.Bool := 0;

   Paint_Record             : aliased win32.Winuser.Paintstruct;

   Window_Class             : aliased Win32.Winuser.Wndclass;

   Message                  : aliased Win32.Winuser.Msg;

   Device_Context_Handle    : aliased Win32.Windef.Hdc;

   Integer_Result  : Win32.Int;

   Bool_Result     : Win32.Bool;

   Long_Result     : Win32.Long;

   Atom_Result     : Win32.Windef.Atom;

   function Min(l,r : Win32.Int) return Win32.Int is
   begin
     if l < r then
       return l;
     else
       return r;
     end if;
   end Min;

   function Max(l,r : Win32.Int) return Win32.Int is
   begin
     if l > r then
       return l;
     else
       return r;
     end if;
   end Max;

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.Winuser.Wm_Size =>

           Box_Width :=
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam))) / 
             Number_Of_Boxes;

           Box_Height :=
             Win32.Int
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam))) /
             Number_Of_Boxes;

           return (0);

	 when Win32.Winuser.Wm_Setfocus =>

           Integer_Result :=                 
             Win32.Winuser.Showcursor
               (Bshow =>
                 True);

           return (0);

	 when Win32.Winuser.Wm_Killfocus =>

           Integer_Result :=                 
             Win32.Winuser.Showcursor
               (Bshow =>
                 False);

           return (0);

	 when Win32.Winuser.Wm_Keydown =>

           Bool_Result :=
             Win32.Winuser.Getcursorpos
               (Lppoint => Point'Access);
      
           Bool_Result :=
             Win32.Winuser.Screentoclient
               (Hwnd    => 
                 Window_Handle,
                Lppoint =>
                Point'Access);

           Current_X :=
             Max
               (L =>
                 0,
                R =>
                 Min
                   (L =>
                     Number_Of_Boxes -1,
                    R =>
                     Win32.Int
                       (Point.X) /
                     Box_Width));

           Current_Y :=
             Max
               (L =>
                 0,
                R =>
                 Min
                   (L =>
                     Number_Of_Boxes -1,
                    R =>
                     Win32.Int
                       (Point.Y) /
                     Box_Height));

           case Wparam is

             when Win32.Winuser.Vk_Up =>

               Current_Y :=
                 Current_Y - 1;

             when Win32.Winuser.Vk_Down =>

               Current_Y :=
                 Current_Y + 1;

             when Win32.Winuser.Vk_Left =>

               Current_X :=
                 Current_X - 1;

             when Win32.Winuser.Vk_Right =>

               Current_X :=
                 Current_X + 1;

             when Win32.Winuser.Vk_Home =>

               Current_X :=
                 0;

               Current_Y :=
                 0;

             when Win32.Winuser.Vk_End =>

               Current_X :=
                 Number_Of_Boxes - 1;

               Current_Y :=
                 Number_Of_Boxes - 1;

             when Win32.Winuser.Vk_Return |
                  Win32.Winuser.Vk_Space =>

               Long_Result :=
                 Win32.Winuser.Sendmessage
                   (Hwnd   => 
                     Window_Handle,
                    Msg    => 
                     Win32.Winuser.Wm_Lbuttondown,
                    Wparam => 
                     Win32.Winuser.Mk_Lbutton,
                    Lparam =>
                     Win32.Lparam
                       (Win32.Utils.Makelong
                         (Low  =>
                           Win32.Word
                             (Current_X * Box_Width),
                          High =>
                           Win32.Word
                             (Current_Y * Box_Height))));

             when others =>

               null;

           end case;

           Current_X :=
             (Current_X + Number_Of_Boxes) mod Number_Of_Boxes;

           Current_Y :=
             (Current_Y + Number_Of_Boxes) mod Number_Of_Boxes;

           Point.X :=
             Win32.Long
               (Current_X * Box_Width + Box_Width / 2 );

           Point.Y :=
             Win32.Long
               (Current_Y * Box_Height + Box_Height / 2 );

           Bool_Result :=                 
             Win32.Winuser.Clienttoscreen
               (Hwnd    => 
                 Window_Handle,
                Lppoint =>
                 Point'Access);

           Bool_Result :=
             Win32.Winuser.Setcursorpos
              (X =>
                Win32.Int
                  (Point.X),
               Y =>
                Win32.Int
                  (Point.Y));

           return (0);

	 when Win32.Winuser.Wm_Lbuttondown =>

           Current_X :=
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam))) / 
              Box_Width;

           Current_Y :=
             Win32.Int
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam))) / 
              Box_Height;

           if Current_X < Number_Of_Boxes and 
              Current_Y < Number_Of_Boxes then
       
             if Bool_Array(Current_X,Current_Y) = True then 

               Bool_Array(Current_X,Current_Y) :=
                 False;

             else

               Bool_Array(Current_X,Current_Y) :=
                 True;

             end if;

             Rectangle.Left :=
               Win32.Long
                 (Current_X * Box_Width);

             Rectangle.Top :=
               Win32.Long
                 (Current_Y * Box_Height);

             Rectangle.Right :=
               Win32.Long
                 ((Current_X + 1) * Box_Width);

             Rectangle.Bottom :=
               Win32.Long
                 ((Current_Y + 1) * Box_Height);

             Bool_Result :=
               Win32.Winuser.Invalidaterect
                 (Hwnd   => 
                   Window_Handle,
                  Lprect => 
                   null,
                  Berase => 
                   False);

           else

             Bool_Result :=
               Win32.Winuser.Messagebeep
                 (Utype =>
                   0);

           end if;

           return (0);

	 when Win32.Winuser.Wm_Paint =>

	   Device_Context_Handle :=
             Win32.Winuser.Beginpaint
	       (Hwnd    =>                                                       
                 Window_Handle,
	        Lppaint =>
                 Paint_Record'Access);

           for x in 0 .. Number_Of_Boxes - 1 loop 

             for y in 0  .. Number_Of_Boxes - 1 loop 

               Bool_Result :=
                 Win32.Wingdi.Rectangle
                   (Hdc         => 
                     Device_Context_Handle,
                    Nleftrect   =>
                     x * Box_Width,
                    Ntoprect    =>
                     y * Box_Height,
                    Nrightrect  =>
                     (x + 1) * Box_Width,
                    Nbottomrect =>
                     (y + 1) * Box_Height);

               if Bool_Array(x,y) = True then

                 Bool_result :=
                   Win32.Wingdi.Movetoex
                     (Hdc     =>
                       Device_Context_Handle,
                      X       =>
                       x * Box_Width,
                      Y       => 
                       y * Box_Height,
                      Lppoint => 
                       null);

                 Bool_Result :=
                   Win32.Wingdi.Lineto
                     (Hdc  =>
                       Device_Context_Handle,
                      Xend =>
                       (x + 1) * Box_Width,
                      Yend =>
                       (y + 1) * Box_Height);

                 Bool_result :=
                   Win32.Wingdi.Movetoex
                     (Hdc     =>
                       Device_Context_Handle,
                      X       =>
                       x * Box_Width,
                      Y       => 
                       (y + 1) * Box_Height,
                      Lppoint => 
                       null);

                 Bool_Result :=
                   Win32.Wingdi.Lineto
                     (Hdc  =>
                       Device_Context_Handle,
                      Xend =>
                       (x + 1) * Box_Width,
                      Yend =>
                       y * Box_Height);

               end if;

             end loop;

           end loop;

    	     Bool_Result :=
               Win32.Winuser.Endpaint
                 (Hwnd    =>
                   Window_Handle,
                  Lppaint =>
                   Paint_Record'Access);

	    return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
                0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
