with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type Interfaces.C.Unsigned;
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Int;
  use type Win32.Windef.Hbitmap;

   Pixel_Width              : Win32.Int;

   Pixel_Height             : Win32.Int;

   Center_Width             : Win32.Int;

   Center_Height            : Win32.Int;

   Client_Width             : Win32.Int;

   Client_Height            : Win32.Int;

   Scale                    : Win32.Int;

   Radius_Width             : Win32.Int;

   Radius_Height            : Win32.Int;

   Move_Width               : Win32.Int;

   Move_Height              : Win32.Int;

   Total_Width              : Win32.Int;

   Total_Height             : Win32.Int;

   Bitmap_Handle            : Win32.Windef.Hbitmap;

   Brush_Handle             : Win32.Windef.Hbrush;

   Id_Timer                 : constant Win32.Uint := 1;

   Message_Box_Text         : constant Interfaces.C.Strings.Chars_Ptr := 
     Interfaces.C.Strings.New_String
       (Str =>
         "Too many clocks or timers!");

   Window_Class             : aliased Win32.Winuser.Wndclass;

   Message                  : aliased Win32.Winuser.Msg;

   Device_Context_Handle    : aliased Win32.Windef.Hdc;

   Device_Context_Memory_Handle    : aliased Win32.Windef.Hdc;

   Integer_Result           : Win32.Int;

   Bool_Result              : Win32.Bool;

   Long_Result              : Win32.Long;

   Atom_Result              : Win32.Windef.Atom;

   Object_Result            : Win32.Windef.Hgdiobj;

   Uint_Result              : Win32.Uint;

   Colorref_Result          : Win32.Windef.Colorref;

   Window_Handle_Result     : Win32.Windef.Hwnd;

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Min(L,R : Win32.Int) return Win32.Int is

   begin

     if L < R then

       return L;

     else

       return R;

     end if;

   end Min;

   function Max(L,R : Win32.Int) return Win32.Int is

   begin

     if L > R then

       return L;

     else

       return R;

     end if;

   end Max;

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

	   Device_Context_Handle := 
             Win32.Winuser.Getdc
               (Hwnd =>
                 Window_Handle);

           Pixel_Width :=
             Win32.Wingdi.Getdevicecaps
               (Hdc         =>
                 Device_Context_Handle,
                Icapability =>
                 Win32.Wingdi.Aspectx);

           Pixel_Height :=
             Win32.Wingdi.Getdevicecaps
               (Hdc         =>
                 Device_Context_Handle,
                Icapability =>
                 Win32.Wingdi.AspectY);

           Integer_Result := 
             Win32.Winuser.Releasedc
               (Hwnd =>
                 Window_Handle,
                Hdc  =>
                 Device_Context_Handle);

	   return (0);

	 when Win32.Winuser.Wm_Size =>

           Client_Width :=
             Win32.Int
               (Win32.Utils.Loword
                 (Win32.Dword
                   (Lparam)));

           Client_Height :=
             Win32.Int
               (Win32.Utils.Hiword
                 (Win32.Dword
                   (Lparam)));

           Center_Width := 
             Client_Width / 2;

           Center_Height := 
             Client_Height / 2;

           Scale :=
             Min
               (L => 
                 Client_Width * Pixel_Width,
                R => 
                 Client_Height * Pixel_Height) /
             16;

           Radius_Width := 
             Scale / Pixel_Width;

           Radius_Height := 
             Scale / Pixel_Height;

           Move_Width :=
             Max
               (L =>
                 1,
                R =>
                 Radius_Width / 2 );  

           Move_Height :=
             Max
               (L =>
                 1,
                R =>
                 Radius_Height / 2 );  

           Total_Width :=
             2 * (Radius_Width + Move_Width);

           Total_Height :=
             2 * (Radius_Height + Move_Height);

           if Bitmap_Handle /= System.Null_Address then

             Bool_Result :=
               Win32.Wingdi.Deleteobject
                 (Hgdiobj =>
                   Bitmap_Handle);  

           end if;

	   Device_Context_Handle := 
             Win32.Winuser.Getdc
               (Hwnd =>
                 Window_Handle);

           Device_Context_Memory_Handle :=
             Win32.Wingdi.Createcompatibledc
               (Hdc     =>
                 Device_Context_Handle);

           Bitmap_Handle :=
             Win32.Wingdi.Createcompatiblebitmap
               (Hdc     =>
                 Device_Context_Handle,
                Nwidth  =>
                 Total_Width,
                Nheight =>
                 Total_Height);

           Integer_Result := 
             Win32.Winuser.Releasedc
               (Hwnd =>
                 Window_Handle,
                Hdc  =>
                 Device_Context_Handle);

           Object_Result :=
             Win32.Wingdi.Selectobject
               (Hdc    => 
                 Device_Context_Memory_Handle,
                Hgdiobj => 
                 Bitmap_Handle);

           Bool_Result :=
             Win32.Wingdi.Rectangle
               (Hdc         =>
                 Device_Context_Memory_Handle,
                Nleftrect   =>
                 -1,
                Ntoprect    =>
                 -1,
                Nrightrect  =>
                 Total_Width + 1,
                Nbottomrect =>
                 Total_Height + 1);

           Object_Result :=
             Win32.Wingdi.Selectobject
               (Hdc    => 
                 Device_Context_Memory_Handle,
                Hgdiobj => 
                 Bitmap_Handle);

           Brush_Handle :=
             Win32.Wingdi.Createhatchbrush
               (Fnstyle =>
                 Win32.Wingdi.Hs_Diagcross,
                Clrref  =>
                 Win32.Wingdi.Rgb
                   (Bred   => 
                     0,
                    Bgreen => 
                     0,
                    Bblue  => 
                     0));

           Object_Result :=
             Win32.Wingdi.Selectobject
               (Hdc    =>                               
                 Device_Context_Memory_Handle,
                Hgdiobj => 
                 Brush_Handle);

           Colorref_Result :=
             Win32.Wingdi.Setbkcolor
               (Hdc => 
                 Device_Context_Memory_Handle,
                Clrref =>
                 Win32.Wingdi.Rgb
                   (Bred =>
                     255,
                    Bgreen =>
                     0,
                    Bblue =>
                     255));
        
           Bool_Result :=
             Win32.Wingdi.Ellipse
               (Hdc =>
                 Device_Context_Memory_Handle,
                Nleftrect =>
                 Move_Width,
                Ntoprect  =>
                 Move_Height,
                Nrightrect  =>
                 Total_Width - Move_Width,
                Nbottomrect =>
                 Total_Height - Move_Height);
           
           Bool_Result :=
             Win32.Wingdi.Deletedc
               (Hdc =>
                 Device_Context_Memory_Handle);

           Bool_Result :=
             Win32.Wingdi.Deleteobject
               (Hgdiobj =>
                 Brush_Handle);

	   return (0);

	 when Win32.Winuser.Wm_Timer =>

           if Bitmap_Handle /= System.Null_Address then

             Device_Context_Handle := 
               Win32.Winuser.Getdc
                 (Hwnd =>
                   Window_Handle);

             Device_Context_Memory_Handle :=
               Win32.Wingdi.Createcompatibledc
                 (Hdc     =>
                   Device_Context_Handle);

             Object_Result :=
               Win32.Wingdi.Selectobject
                 (Hdc    => 
                   Device_Context_Memory_Handle,
                  Hgdiobj => 
                   Bitmap_Handle);
          
             Bool_Result :=
               Win32.Wingdi.Bitblt
                 (Hdcdest =>
                   Device_Context_Handle,
                  Nxdest  =>
                   Center_Width - Total_Width /2 ,
                  Nydest  =>
                   Center_Height - Total_Height /2 ,
                  Nwidth  =>
                   Total_Width,
                  Nheight =>
                   Total_Height,
                  Hdcsrc  =>
                   Device_Context_Memory_Handle,
                  Nxsrc   =>
                   0,
                  Nysrc   =>
                   0,
                  Dwrop   =>
                   Win32.Wingdi.Srccopy);

             Integer_Result := 
               Win32.Winuser.Releasedc
                 (Hwnd =>
                   Window_Handle,
                  Hdc  =>
                   Device_Context_Handle);

             Bool_Result :=
               Win32.Wingdi.Deletedc
                 (Hdc =>
                   Device_Context_Memory_Handle);

             Center_Width :=
               Center_Width + Move_Width;

             Center_Height :=
               Center_Height + Move_Height;

             if Center_Width + Radius_Width >= Client_Width or
                Center_Width - Radius_Width <= 0 then

               Move_Width :=
                 - Move_Width;

             end if;

             if Center_Height + Radius_Height >= Client_Height or
                Center_Height - Radius_Height <= 0 then

               Move_Height :=
                 - Move_Height;

             end if;

           end if;

	   return (0);

	 when Win32.WinUser.Wm_Destroy =>

           if Bitmap_Handle /= System.Null_Address then

             Bool_Result :=
               Win32.Wingdi.Deleteobject
                 (Hgdiobj =>
                   Bitmap_Handle);  

           end if;

           Bool_Result :=
             Win32.Winuser.Killtimer
               (Hwnd     =>
                 Window_Handle,
                Uidevent =>
                 Id_Timer);

	   Win32.WinUser.Postquitmessage
	     (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Windows.Data.Window_Class_Background,
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Caption),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

        Uint_Result :=
          0;

    while Uint_Result = 0 loop 

      Uint_Result :=
        Win32.Winuser.Settimer
          (Hwnd        =>
            Windows.Data.Window_Handle,
           Nidevent    =>
            Id_Timer,
           Uelapse     =>
            1,
           Lptimerfunc =>
            null);

      if Uint_Result = 0 then

        Integer_Result :=
          Win32.Winuser.Messagebox
             (Hwnd      => 
               Windows.Data.Window_Handle,
              Lptext    => 
               Convert_Char_Ptr_To_Lpcstr
                 (S => 
                   Message_Box_Text),
              Lpcaption =>
               Convert_Char_Ptr_To_Lpcstr
                 (S => 
                   Windows.Data.Class_Name),
              Utype     =>
               Win32.Winuser.Mb_Iconexclamation or
               Win32.Winuser.Mb_Ok);

        return(0);     

      end if;

    end loop;

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
