with Ada.Unchecked_Conversion;
with Interfaces.C.Strings;
with System;
with Win32.Windef;
with Win32.Wingdi;
with Win32.Winmain;
with Win32.Utils;
with Win32.Winuser;
with Windows.Data;
package body Windows.Routines is
  use type Interfaces.C.Unsigned;
  use type System.Address;
  use type Win32.Bool;
  use type Win32.Dword;
  use type Win32.Int;
  use type Win32.Long;

   Edit_Window_Handle         : Win32.Windef.Hwnd;

   Edit_Text                  : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "edit");

   Message_Box_Text                  : constant Interfaces.C.Strings.Chars_Ptr :=
     Interfaces.C.Strings.New_String
       (Str =>
         "Edit control out of space");

   True                     : constant Win32.Bool := 1;

   Window_Class             : aliased Win32.Winuser.Wndclass;

   Message                  : aliased Win32.Winuser.Msg;

   Device_Context_Handle    : aliased Win32.Windef.Hdc;

   Integer_Result           : Win32.Int;

   Dword_Result             : Win32.Dword;

   Bool_Result              : Win32.Bool;

   Long_Result              : Win32.Long;

   Atom_Result              : Win32.Windef.Atom;

   Object_Result            : Win32.Windef.Hgdiobj;

   Colorref_Result          : Win32.Windef.Colorref;

   Window_Handle_Result     : Win32.Windef.Hwnd;
 
   function Convert_Integer_To_Hmenu is new
     Ada.Unchecked_Conversion
       (Source => Integer,
        Target => Win32.Windef.Hmenu);

   function Convert_Char_Ptr_To_Lpcstr is new
     Ada.Unchecked_Conversion
       (Source => Interfaces.C.Strings.Chars_Ptr,
        Target => Win32.Lpcstr);

   function Convert_Lparam_To_Lpcreatestruct is new
     Ada.Unchecked_Conversion
       (Source => Win32.Lparam,
        Target => Win32.Winuser.Lpcreatestruct);

   procedure Main_Windows_Message_Loop is
   begin
      while (Win32.Winuser.Getmessage 
	     (Lpmsg => 
               Message'Access,
	      Hwnd  => 
               null, 
	      Wmsgfiltermin => 
               Windows.Data.Message_Filter_Minimum,
	      Wmsgfiltermax => 
               Windows.Data.Message_Filter_Maximum) /= 0) loop
	 
      	 Bool_Result := 
           Win32.Winuser.Translatemessage
	     (Lpmsg => 
               Message'Access);
	 
   	 Long_Result := 
           Win32.Winuser.Dispatchmessage
	     (Lpmsg => 
               Message'Access);
	 
      end loop;

   end Main_Windows_Message_Loop;

   function Main_Windows
     (Window_Handle : Win32.Windef.Hwnd;
      Message       : Win32.Uint;
      Wparam        : Win32.Wparam;
      Lparam        : Win32.Lparam) return Win32.Lresult is
   begin

      case Message is

	 when Win32.Winuser.Wm_Create =>

      Edit_Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Edit_Text),
            Lpwindowname => 
             null,
            Dwstyle      => 
             Win32.Winuser.Ws_Child or
             Win32.Winuser.Ws_Visible or
             Win32.Winuser.Ws_Hscroll or
             Win32.Winuser.Ws_Vscroll or
             Win32.Winuser.Ws_Border or
             Win32.Winuser.Es_Left or
             Win32.Winuser.Es_Multiline or
             Win32.Winuser.Es_Autohscroll or
             Win32.Winuser.Es_Autovscroll,
            X            => 
             0, 
            Y            => 
             0,
            Nwidth       => 
             0,
            Nheight      => 
             0,
            Hwndparent   => 
             Window_Handle,
            Hmenu        => 
             Convert_Integer_To_Hmenu
               (S =>
                 1),
            Hinstance    => 
             Convert_Lparam_To_Lpcreatestruct
               (S =>
                 Lparam).Hinstance, 
            Lpparam      =>
             Null);

	   return (0);

	 when Win32.Winuser.Wm_Setfocus =>

           Window_Handle_Result :=
             Win32.Winuser.Setfocus
               (Hwnd =>
                 Edit_Window_Handle);
           
	   return (0);

	 when Win32.Winuser.Wm_Size =>

           Bool_Result :=
             Win32.Winuser.Movewindow
               (Hwnd     =>
                 Edit_Window_Handle,
                X        =>
                 0,
                Y        =>
                 0,
                Nwidth   =>
                 Win32.Int
                   (Win32.Utils.Loword
                     (Win32.Dword
                       (Lparam))),
                NHeight  =>
                 Win32.Int
                   (Win32.Utils.Hiword
                     (Win32.Dword
                       (Lparam))),
                Brepaint =>
                 True);

	   return (0);

	 when Win32.Winuser.Wm_Command =>

           if Integer
                (Win32.Utils.Loword
                  (Win32.Dword
                    (Wparam))) = 1 then

             if Integer
                  (Win32.Utils.Hiword
                    (Win32.Dword
                      (Wparam))) = Win32.Winuser.En_Errspace or
                Integer
                  (Win32.Utils.Hiword
                    (Win32.Dword
                      (Wparam))) = Win32.Winuser.En_Maxtext then

               Integer_Result :=
                 Win32.Winuser.Messagebox
                   (Hwnd      => 
                     Window_Handle,
                    Lptext    => 
                     Convert_Char_Ptr_To_Lpcstr
                      (S => 
                        Message_Box_Text),
                    Lpcaption =>
                     Convert_Char_Ptr_To_Lpcstr
                      (S => 
                        Windows.Data.Class_Name),
                    Utype     =>
                     Win32.Winuser.Mb_Ok or
                     Win32.Winuser.Mb_Iconstop);

             end if;

           end if;

	   return (0);

	 when Win32.WinUser.Wm_Destroy =>

	    Win32.WinUser.Postquitmessage
	      (Nexitcode => 
               0);

	    return (0);

	 when others =>

	    return Win32.Winuser.Defwindowproc
	      (Hwnd   =>
                Window_Handle, 
	       Msg    => 
                Message, 
	       Wparam => 
                Wparam,
	       Lparam => 
                Lparam);

      end case;

      return (0);

   end Main_Windows;

   function Main
     (Instance          : Win32.Windef.Hinstance;
      Previous_instance : Win32.Windef.Hinstance;
      Command_Line      : Win32.Pcstr;
      Show_Command      : Win32.Int ) return Win32.Uint is
   begin

      if Previous_Instance = System.Null_Address then

	 Window_Class :=      
	   (Style         => 
             Windows.Data.Window_Class_Style,
	    Lpfnwndproc   => 
             Windows.Routines.Main_Windows'Access, 
	    Cbclsextra    => 
             Windows.Data.Window_Class_Cbclsextra,
	    Cbwndextra    => 
             Windows.Data.Window_Class_Cbwndextra,
	    Hinstance     => 
             Instance,
	    Hicon         => 
             Windows.Data.Window_Class_Icon,
	    Hcursor       => 
             Windows.Data.Window_Class_Cursor,
	    Hbrbackground => 
             Win32.Wingdi.Createsolidbrush
               (Clrref =>
                 Win32.Wingdi.Rgb
                   (Bred  =>
                     0,
                    Bgreen =>
                     0,
                    Bblue  =>
                     0)),
	    Lpszmenuname  => 
             Windows.Data.Window_Class_Menu_Name,
	    Lpszclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name));

	 Atom_Result :=
	   win32.winuser.Registerclass
	   (Lpwndclass => 
             Window_Class'Access);

      end if;

      Windows.Data.Window_Handle := 
        Win32.WinUser.Createwindow
	  (Lpclassname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Lpwindowname => 
             Convert_Char_Ptr_To_Lpcstr
               (S => 
                 Windows.Data.Class_Name),
            Dwstyle      => 
             Windows.Data.Window_Style,
            X            => 
             Windows.Data.Window_Horizontal_Start, 
            Y            => 
             Windows.Data.Window_Vertical_Start,
            Nwidth       => 
             Windows.Data.Window_Width,
            Nheight      => 
             Windows.Data.Window_Height,
            Hwndparent   => 
             Windows.Data.Window_Parent,
            Hmenu        => 
             Windows.Data.Window_Menu,
            Hinstance    => 
             Instance,
            Lpparam      => 
             Windows.Data.Window_Parameters);

    Bool_Result := 
      Win32.Winuser.Showwindow
        (Hwnd     => 
          Windows.Data.Window_Handle, 
	 Ncmdshow =>
          Win32.Winmain.Get_Ncmdshow);

    Bool_Result := 
      Win32.Winuser.Updatewindow
        (Hwnd => 
          Windows.Data.Window_Handle);

    Windows.Routines.Main_Windows_Message_Loop;

    return Message.Wparam;

  end Main;

end Windows.Routines;
