
--
-- Author : Pascal Obry
--
-- This is a sample program to show some OpenGL features.
--
-- It compile fine under GNAT 3.04a. You must add the following
-- libraries : OPENGL32.LIB GLU32.LIB GLAUX.LIB
--


with Win32.Gl;
with Win32.Glu;
with Win32.Glaux;
with Interfaces.C;
with Interfaces.C.Strings;
with System;
with Unchecked_Conversion;

procedure OGLDepthCue is

   use Win32.Gl;
   use Win32.Glu;
   use Win32.Glaux;
   use Interfaces;

   function Conv is new Unchecked_Conversion (System.Address, AUXRESHAPEPROC);
   function Conv is new Unchecked_Conversion (System.Address, AUXMAINPROC);

   type Glfloat_4 is array (1 .. 4) of aliased Glfloat;

   Window_Name : aliased C.Char_Array := C.To_C ("Distance Using Fog");
   Res : Glenum;

   --  callbacks must be defined with the Stdcall Convention
   procedure MyReshape (W, H : in Glsizei);
   pragma Convention (Stdcall, MyReshape);

   procedure Display;
   pragma Convention (Stdcall, Display);

   --  here start the definition of the gl scene.
   procedure MyInit is
      FogColor : GLfloat_4 := (0.0, 0.0, 0.0, 1.0);
   begin
    glEnable (GL_FOG);
    glFogi (GL_FOG_MODE, GL_LINEAR);
    glHint (GL_FOG_HINT, GL_NICEST);
    glFogf (GL_FOG_START, 3.0);
    glFogf (GL_FOG_END, 5.0);
    glFogfv (GL_FOG_COLOR, FogColor (1)'Unchecked_Access);
    glClearColor (0.0, 0.0, 0.0, 1.0);

    glDepthFunc(GL_LESS);
    glEnable(GL_DEPTH_TEST);
    glShadeModel(GL_FLAT);
   end MyInit;

   procedure MyReshape (W, H : in Glsizei) is
   begin
      GlViewPort (0, 0, W, H);
      GlMatrixMode (GL_PROJECTION);
      GlLoadIdentity;
      GluPerspective (45.0, GLdouble (W)/ GLdouble (H), 3.0, 5.0);
      GlMatrixMode (GL_MODELVIEW);
      GlLoadIdentity;
      glTranslatef (0.0, 0.0, -4.0);
   end MyReshape;

   procedure Display is
   begin
      GlClear (GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT);
      GlColor3f (1.0, 1.0, 1.0);
      AuxWireIcosahedron (1.0);
      GlFlush;
   end Display;

begin
   AuxInitDisplayMode (AUX_SINGLE + AUX_RGB + AUX_DEPTH16);
   AuxInitPosition (0, 0, 400, 400);
   Res := AuxInitWindow (Window_Name (Window_Name'First)'Unchecked_Access);
   MyInit;
   AuxReshapeFunc (Conv(MyReshape'Address));
   Auxmainloop (Conv(Display'Address));
end OGLDepthCue;
