
--
-- Author : Pascal Obry
--
-- This is a sample program to show some OpenGL features.
--
-- It compile fine under GNAT 3.04a. You must add the following
-- libraries : OPENGL32.LIB GLU32.LIB GLAUX.LIB
--

with Win32.Gl;
with Win32.Glu;
with Win32.Glaux;
with Interfaces.C;
with Interfaces.C.Strings;
with System;
with Unchecked_Conversion;

procedure OGLLight is

   use Win32.Gl;
   use Win32.Glu;
   use Win32.Glaux;
   use Interfaces;

   function Conv is new Unchecked_Conversion (System.Address, AUXRESHAPEPROC);
   function Conv is new Unchecked_Conversion (System.Address, AUXMAINPROC);

   Window_Name : aliased C.Char_Array := C.To_C ("Lighting");
   Res : Glenum;

   type Glfloat_4 is array (1 .. 4) of aliased Glfloat;
   type Gldouble_4 is array (1 .. 4) of aliased Gldouble;

   --  callbacks must be defined with the Stdcall Convention
   procedure MyReshape (W, H : in Glsizei);
   pragma Convention (Stdcall, MyReshape);

   procedure Display;
   pragma Convention (Stdcall, Display);

   --  here start the definition of the gl scene.
   procedure MyInit is

      Mat_Specular   : Glfloat_4 := (1.0, 0.5, 0.1, 1.0);
      Mat_Shininess  : Glfloat   := 50.0;

      Light_Position : Glfloat_4 := (1.0, 1.0, 1.0, 0.0);

   begin
      GlMaterialfv (GL_FRONT, GL_SPECULAR, Mat_Specular (1)'Unchecked_Access);
      GlMaterialf  (GL_FRONT, GL_SHININESS, Mat_Shininess);

      GlLightfv      (GL_LIGHT0, GL_POSITION,
                      Light_Position (1)'Unchecked_Access);

      GlEnable (GL_LIGHTING);
      GlEnable (GL_LIGHT0);
      GlDepthFunc (GL_LESS);
      GlEnable (GL_DEPTH_TEST);
   end MyInit;

   procedure MyReshape (W, H : in Glsizei) is
      LH : Gldouble := Gldouble (H);
      LW : Gldouble := Gldouble (W);
   begin
      if H = 0 then
         LH := 1.0;
      end if;
      GlViewPort (0, 0, Glsizei (LW), Glsizei (LH));
      GlMatrixMode (GL_PROJECTION);
      GlLoadIdentity;

      if LW <= LH then
         GlOrtho (-1.5, 1.5, -1.5 * LH/LW, 1.5 * LH/LW, -10.0, 10.0);
      else
         GlOrtho (-1.5 * LW/LH, 1.5 * LW/LH, -1.5, 1.5, -10.0, 10.0);
      end if;

      GlMatrixMode (GL_MODELVIEW);
      GlLoadIdentity;
   end MyReshape;

   procedure Display is
   begin
      GlClear (GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT);
      AuxSolidSphere (1.2);
      GlFlush;
   end Display;

begin
   AuxInitDisplayMode (AUX_SINGLE + AUX_RGB + AUX_DEPTH16);
   AuxInitPosition (0, 0, 500, 500);
   Res := AuxInitWindow (Window_Name (Window_Name'First)'Unchecked_Access);
   MyInit;
   AuxReshapeFunc (Conv(MyReshape'Address));
   Auxmainloop (Conv(Display'Address));
end OGLLight;
