
--
-- Author : Pascal Obry
--
-- This is a sample program to show some OpenGL features.
--
-- It compile fine under GNAT 3.04a. You must add the following
-- libraries : OPENGL32.LIB GLU32.LIB GLAUX.LIB
--

with Win32.Gl;
with Win32.Glu;
with Win32.Glaux;
with Interfaces.C;
with Interfaces.C.Strings;
with System;
with Unchecked_Conversion;
procedure OGLSurface is

   use Win32.Gl;
   use Win32.Glu;
   use Win32.Glaux;
   use Interfaces;

   function Conv is new Unchecked_Conversion (System.Address, AUXRESHAPEPROC);
   function Conv is new Unchecked_Conversion (System.Address, AUXMAINPROC);

   type Glfloat_4 is array (1 .. 4) of aliased Glfloat;
   type Glfloat_8 is array (1 .. 8) of aliased Glfloat;

   Window_Name : aliased C.Char_Array := C.To_C ("NURBS Surface");
   Res : Glenum;

   CtlPoints : array (1 .. 4, 1 .. 4, 1 .. 3) of aliased Glfloat;
   TheNurb   : PGLUnurbsObj;

   --  callbacks must be defined with the Stdcall Convention
   procedure MyReshape (W, H : in Glsizei);
   pragma Convention (Stdcall, MyReshape);

   procedure Display;
   pragma Convention (Stdcall, Display);

   --  here start the definition of the gl scene.
   procedure MyInit is

      mat_diffuse   : Glfloat_4 := ( 0.7, 0.7, 0.7, 1.0 );
      mat_specular  : Glfloat_4 := ( 1.0, 1.0, 1.0, 1.0 );

      procedure Init_Surface is
         U, V : Integer;
      begin
         for U in CtlPoints'Range (1) loop
            for V in CtlPoints'Range (2) loop
               CtlPoints (U, V, 1) := 2.0 * (GLfloat (U) - 1.5);
               CtlPoints (U, V, 2) := 2.0 * (GLfloat (V) - 1.5);

               if (U = 1 or U = 2) and  (V = 1 or V = 2) then
                  CtlPoints (U, V, 3) := 3.0;
               else
                  CtlPoints (U, V, 3) := -3.0;
               end if;
            end loop;
         end loop;
      end Init_Surface;

   begin
    glClearColor (0.0, 0.0, 0.0, 1.0);

    GlMaterialfv (GL_FRONT, GL_DIFFUSE, Mat_Diffuse (1)'Unchecked_Access);
    GlMaterialfv (GL_FRONT, GL_SPECULAR, Mat_Specular (1)'Unchecked_Access);
    GlMaterialf (GL_FRONT, GL_SHININESS, 100.0);

    GlEnable (GL_LIGHTING);
    GlEnable (GL_LIGHT0);
    GlDepthFunc (GL_LESS);
    GlEnable (GL_DEPTH_TEST);
    GlEnable (GL_AUTO_NORMAL);
    GlEnable (GL_NORMALIZE);

    init_surface;

    theNurb := GluNewNurbsRenderer;
    GluNurbsProperty (TheNurb, GLU_SAMPLING_TOLERANCE, 25.0);
    GluNurbsProperty (theNurb, GLU_DISPLAY_MODE, Glfloat (GLU_FILL));
   end MyInit;

   procedure MyReshape (W, H : in Glsizei) is
   begin
      GlViewPort (0, 0, W, H);
      GlMatrixMode (GL_PROJECTION);
      GlLoadIdentity;
      GluPerspective (45.0, GLdouble (W)/ GLdouble (H), 3.0, 8.0);
      GlMatrixMode (GL_MODELVIEW);
      GlLoadIdentity;
      glTranslatef (0.0, 0.0, -5.0);
   end MyReshape;

   procedure Display is
      Knots : Glfloat_8 := (0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0);
   begin
      GlClear (GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT);
      GlPushMatrix;
      GlRotatef (330.0, 1.0, 0.0, 0.0);
      glScalef (0.5, 0.5, 0.5);

      GluBeginSurface (theNurb);
      GluNurbsSurface (theNurb,
                       8, Knots(1)'Unchecked_Access,
                       8, Knots(1)'Unchecked_Access,
                       4 * 3,
                       3,
                       Ctlpoints (1,1,1)'Unchecked_Access,
                       4, 4,
                       GL_MAP2_VERTEX_3);
      GluEndSurface (theNurb);

      glPopMatrix;
      glFlush;
      --  Auxswapbuffers;
   end Display;

begin
   AuxInitDisplayMode (AUX_SINGLE + AUX_RGB);
   AuxInitPosition (0, 0, 500, 500);
   Res := AuxInitWindow (Window_Name (Window_Name'First)'Unchecked_Access);
   MyInit;
   AuxReshapeFunc (Conv(MyReshape'Address));
   Auxmainloop (Conv(Display'Address));
end OGLSurface;
