-- $Source: /home/harp/1/proto/monoBANK/winnt/pcxchild.adb,v $ 
-- $Revision: 1.7 $ $Date: 96/09/03 17:23:50 $ $Author: mg $ 
-- $Id: pcxchild.adb 1.4 1995/01/24 16:59:28 mps Exp mps $
--
-- This packages manages the low level details of the child windows.
-- It also performs a 'use' clause on Win32 AND Win32.WinDef to improve 
-- readability, unlike the main procedure.
--
with Ada.Unchecked_Conversion;
with Interfaces.C;
with PcxOps;
with System;
with Win32.Utils;
with Win32.WinDef;
with Win32.WinUser; 
with Win32.WinBase;
with Win32.WinGdi;
with Win32.WinNT;

use Win32;
use Win32.WinDef;

package body PcxChild is

    function To_DWORD is new Ada.Unchecked_Conversion(
	Win32.LPARAM, Win32.DWORD);

    cursor_offset_left : constant := 10;
    cursor_offset_top  : constant := 10;

    WM_STARTRESIZE     : constant := 561;
    WM_ENDRESIZE       : constant := 562;

    type CHILD_ARRAY is array(INT range 1..max_images) of HWND; 

    CHILDREN         : CHILD_ARRAY;
    REPAINT_WANTED   : BOOLEAN;               -- repaint during resizing?
    BOX_ACTIVE       : BOOLEAN;               -- is rectangle being sized?
    RECTANGLE_DRAWN  : BOOLEAN;               -- was rectangle drawn? (to erase)
    GLOBAL_HDC       : HDC;                   -- program-wide display context 
    XPOS             : LONG;                  -- final X position of rectangle
    YPOS             : LONG;                  -- final Y position of rectangle
    XPOS_START       : LONG;                  -- start X position of rectangle
    YPOS_START       : LONG;                  -- start Y position of rectangle


    -- aliased for function ChildWndProc
    CLIENT_RECT : aliased RECT;
    CURSOR_RECT : aliased RECT;
    CUR_POINT   : aliased POINT;
    TMP_POINT   : aliased POINT;
    NEW_POINT   : aliased POINT;
    TL_POINT    : aliased POINT;
    BR_POINT    : aliased POINT;


    -- aliased for function CreateWindow
    title_str  : aliased constant Win32.CHAR_Array := "PcxChild" & Nul;
    BUFFER     : aliased Win32.CHAR_Array(0..1023);



    function CP(S : Win32.CHAR_Array) return Win32.LPCSTR is
	function UC is new Ada.Unchecked_Conversion(
	    System.Address, Win32.LPCSTR);
    begin
	return UC(S(S'First)'Address);
    end CP;

    --
    -- This is the standard windows procedure which accepts and handles
    -- the various child messages.
    --
    function ChildWndProc(HWND    : WinDef.HWND;
			  MESSAGE : UINT;
			  WPARAM  : Win32.WPARAM;
			  LPARAM  : Win32.LPARAM) return LRESULT is

	use type Win32.INT;
	use type Win32.LONG;

	BRESULT     : BOOL;        
	IRESULT     : INT;
	LRESULT     : LONG;
	PREVMIXMODE : INT;
	CURSOR      : HCURSOR;

    begin
      case MESSAGE is
    --
    -- "Turn off" the repainting to let the user resize the window.  When the
    -- resize operation is completed, the image will be redrawn.
    --
	when WM_STARTRESIZE =>
	  REPAINT_WANTED := FALSE;
	  LRESULT := Win32.WinUser.DefWindowProc(HWND, MESSAGE, WPARAM, LPARAM);

    --
    -- If a repaint message, redraw the image.  Note that the default windows 
    -- procedure must still be called.
    --
	when Win32.WinUser.WM_PAINT =>
	  if REPAINT_WANTED then
    --
    -- Only move the cursor if it is in the window.
    --
	    BRESULT := Win32.WinUser.GetCursorPos(CUR_POINT'access);
	    BRESULT := Win32.WinUser.GetWindowRect(HWND, CURSOR_RECT'access);
	    if Win32.WinUser.PtInRect(CURSOR_RECT'access, 
				      CUR_POINT) = Win32.TRUE then
	      TMP_POINT.X := CURSOR_RECT.LEFT + cursor_offset_left;
	      TMP_POINT.Y := CURSOR_RECT.TOP + cursor_offset_top;
	      BRESULT := Win32.WinUser.SetCursorPos(INT(TMP_POINT.X), 
						    INT(TMP_POINT.Y));
	      CURSOR := Win32.WinUser.LoadCursor(
	          System.Null_Address, LPCSTR(Win32.WinUser.IDC_WAIT));
	      CURSOR := Win32.WinUser.SetCursor(CURSOR);
	      PcxOps.Display(HWND);
	      CURSOR := Win32.WinUser.SetCursor(CURSOR);
    --
    -- Only restore cursor if it hasn't moved from its original location.  If it
    -- has moved, the user may be doing some other application.
    --
	      BRESULT := Win32.WinUser.GetCursorPos(NEW_POINT'access);
	      if NEW_POINT.X in TMP_POINT.X - 5..TMP_POINT.X + 5 and then 
		 NEW_POINT.Y in TMP_POINT.Y - 5..TMP_POINT.Y + 5 then 
		BRESULT := Win32.WinUser.SetCursorPos(INT(CUR_POINT.X), 
						      INT(CUR_POINT.Y));
	      end if;
	    else
	      PcxOps.Display(HWND);
	    end if;
	  end if;
	  return Win32.WinUser.DefWindowProc(HWND, MESSAGE, WPARAM, LPARAM);

    --
    -- If the user resizes, redraw the image.
    --
	when WM_ENDRESIZE =>
	  REPAINT_WANTED := TRUE;
	  LRESULT := Win32.WinUser.SendMessage(HWND,
					       Win32.WinUser.WM_PAINT,
					       WPARAM,
					       LPARAM);
	  LRESULT := Win32.WinUser.DefWindowProc(HWND, MESSAGE, WPARAM, LPARAM);

    --
    -- If the user presses the left mouse button down, start drawing the 
    -- box that will be magnified.
    --
	when Win32.WinUser.WM_LBUTTONDOWN => 
	  BOX_ACTIVE := TRUE;
    --
    -- This call sets the Device Context to display the XNOR of the pixel to 
    -- draw and the pixel that was there.  This allows us to erase by simply 
    -- redrawing the previous rectangle and has the advantage of not erasing 
    -- anything "beneath" the rectangle.
    --
	  GLOBAL_HDC := Win32.WinUser.GetDC(HWND);
	  PREVMIXMODE := Win32.WinGdi.SetRop2(GLOBAL_HDC, 
					      Win32.WinGdi.R2_NOTXORPEN);
    --
    -- Prevent the mouse from going outside the rectangle.
    --
	  BRESULT := Win32.WinUser.GetClientRect(HWND, CLIENT_RECT'access);
	  TL_POINT.X := CLIENT_RECT.LEFT;
	  TL_POINT.Y := CLIENT_RECT.TOP;
	  BR_POINT.X := CLIENT_RECT.RIGHT;
	  BR_POINT.Y := CLIENT_RECT.BOTTOM;
	  BRESULT := Win32.WinUser.ClientToScreen(HWND, TL_POINT'access);
	  BRESULT := Win32.WinUser.ClientToScreen(HWND, BR_POINT'access);
	  BRESULT := Win32.WinUser.SetRect(CLIENT_RECT'access, 
					   INT(TL_POINT.X), 
					   INT(TL_POINT.Y),
					   INT(BR_POINT.X), 
					   INT(BR_POINT.Y));
	  BRESULT := Win32.WinUser.ClipCursor(CLIENT_RECT'access);
    --
    -- Save the initial points.
    --
	  XPOS_START := LONG(Win32.Utils.Loword(To_DWORD(LPARAM)));
	  YPOS_START := LONG(Win32.Utils.Hiword(To_DWORD(LPARAM)));
      
    --
    -- When the left mouse button is released, redraw the image within the 
    -- box scaled to the client window.
    --
	when Win32.WinUser.WM_LBUTTONUP =>
	  BRESULT := Win32.WinUser.ClipCursor(null); -- release mouse

	  if RECTANGLE_DRAWN then
	    BRESULT := Win32.WinGdi.Rectangle(GLOBAL_HDC, 
					      INT(XPOS_START), 
					      INT(YPOS_START), 
					      INT(XPOS),       
					      INT(YPOS));
	  end if;

	  XPOS := LONG(Win32.Utils.Loword(To_DWORD(LPARAM)));
	  YPOS := LONG(Win32.Utils.Hiword(To_DWORD(LPARAM)));

	  BOX_ACTIVE      := FALSE;
	  RECTANGLE_DRAWN := FALSE;

	  CLIENT_RECT.LEFT   := XPOS_START;
	  CLIENT_RECT.TOP    := YPOS_START;
	  CLIENT_RECT.RIGHT  := XPOS;
	  CLIENT_RECT.BOTTOM := YPOS;

	  if Win32.WinUser.IsRectEmpty(CLIENT_RECT'access) = Win32.FALSE then
	    BRESULT := Win32.WinUser.GetCursorPos(CUR_POINT'access);
	    BRESULT := Win32.WinUser.GetWindowRect(HWND, CURSOR_RECT'access);
	    if Win32.WinUser.PtInRect(CURSOR_RECT'access, 
				      CUR_POINT) = Win32.TRUE then
	      TMP_POINT.X := CURSOR_RECT.LEFT + cursor_offset_left;
	      TMP_POINT.Y := CURSOR_RECT.TOP + cursor_offset_top;
	      BRESULT := Win32.WinUser.SetCursorPos(INT(TMP_POINT.X), 
						    INT(TMP_POINT.Y));
	      CURSOR := Win32.WinUser.LoadCursor(System.Null_Address, 
						 LPCSTR(Win32.WinUser.IDC_WAIT));
	      CURSOR := Win32.WinUser.SetCursor(CURSOR);
	      PcxOps.Zoom_In(HWND, CLIENT_RECT);
	      CURSOR := Win32.WinUser.SetCursor(CURSOR);
	--
	-- Only restore cursor if it hasn't moved from it's original location.  
	-- If it has moved, the user may be doing some other application.
	--
	      BRESULT := Win32.WinUser.GetCursorPos(NEW_POINT'access);
	      if NEW_POINT.X in TMP_POINT.X - 5..TMP_POINT.X + 5 and then 
		 NEW_POINT.Y in TMP_POINT.Y - 5..TMP_POINT.Y + 5 then 
		BRESULT := Win32.WinUser.SetCursorPos(INT(CUR_POINT.X), 
						      INT(CUR_POINT.Y));
	      end if;
	    else
	      PcxOps.Zoom_In(HWND, CLIENT_RECT);
	    end if;
	  end if;

	  IRESULT := Win32.WinUser.ReleaseDC(HWND, GLOBAL_HDC);

	--
	-- Whenever there is mousemovement, if the box is currently being sized, 
	-- erase it and redraw it based on the new mouse coordinates.
	--
	when Win32.WinUser.WM_MOUSEMOVE =>
	  if BOX_ACTIVE then
	    if RECTANGLE_DRAWN then
	      BRESULT := Win32.WinGdi.Rectangle(GLOBAL_HDC, -- erases
					       INT(XPOS_START), 
					       INT(YPOS_START), 
					       INT(XPOS),       
					       INT(YPOS));
	    end if;
	    XPOS := LONG(Win32.Utils.Loword(To_DWORD(LPARAM)));
	    YPOS := LONG(Win32.Utils.Hiword(To_DWORD(LPARAM)));
	--
	-- Now draw the new rectangle.
	--
	    BRESULT := Win32.WinGdi.Rectangle(GLOBAL_HDC, 
					      INT(XPOS_START), 
					      INT(YPOS_START), 
					      INT(XPOS),       
					      INT(YPOS));
	    RECTANGLE_DRAWN := TRUE;
	  end if;

	--
	-- Zoom out.
	--
	when Win32.WinUser.WM_RBUTTONUP =>
	  BRESULT := Win32.WinUser.GetCursorPos(CUR_POINT'access);
	  BRESULT := Win32.WinUser.GetWindowRect(HWND, CURSOR_RECT'access);
	  if Win32.WinUser.PtInRect(CURSOR_RECT'access, 
				    CUR_POINT) = Win32.TRUE then
	    TMP_POINT.X := CURSOR_RECT.LEFT + cursor_offset_left;
	    TMP_POINT.Y := CURSOR_RECT.TOP + cursor_offset_top;
	    BRESULT := Win32.WinUser.SetCursorPos(INT(TMP_POINT.X), 
						  INT(TMP_POINT.Y));
	    CURSOR := Win32.WinUser.LoadCursor(System.Null_Address, 
					       LPCSTR(Win32.WinUser.IDC_WAIT));
	    CURSOR := Win32.WinUser.SetCursor(CURSOR);
	    PcxOps.Zoom_Out(HWND);
	    CURSOR := Win32.WinUser.SetCursor(CURSOR);
	--
	-- Only restore cursor if it hasn't moved from it's original location.  
	-- If it has moved, the user may be doing some other application.
	--
	    BRESULT := Win32.WinUser.GetCursorPos(NEW_POINT'access);
	    if NEW_POINT.X in TMP_POINT.X - 5..TMP_POINT.X + 5 and then 
	       NEW_POINT.Y in TMP_POINT.Y - 5..TMP_POINT.Y + 5 then 
	      BRESULT := Win32.WinUser.SetCursorPos(INT(CUR_POINT.X), 
						    INT(CUR_POINT.Y));
	    end if;
	  else
	    PcxOps.Zoom_Out(HWND);
	  end if;

	when others =>
	  return Win32.WinUser.DefWindowProc(HWND, MESSAGE, WPARAM, LPARAM);

      end case;
      return Win32.FALSE;
    end ChildWndProc;

    function CreateWindow(HINST         : HINSTANCE;
			  PARENT_HWND   : HWND;
			  CREATE_SIZE_X : INT;
			  CREATE_SIZE_Y : INT;
			  TITLE         : LPCSTR) return HWND is

	use type Win32.INT;
	use type Win32.DWORD;
	use type System.Address;

	CHILDHWND  : HWND;
	IRESULT    : INT;
	SYSTEM_MSG : LPSTR;
	ERROR_LEN  : DWORD;

    begin
      --
      -- Create a child window for this application instance.
      --
      NUM_CHILDREN := NUM_CHILDREN + 1;
      BOX_ACTIVE := FALSE;
      RECTANGLE_DRAWN := FALSE;
      REPAINT_WANTED := TRUE;
      CHILDHWND := Win32.WinUser.CreateWindow(
			lpClassName  => SZCHILDNAME, 
			lpWindowName => TITLE,
			dwStyle      => Win32.WinUser.WS_OVERLAPPED or 
					Win32.WinUser.WS_THICKFRAME or
					Win32.WinUser.WS_VISIBLE or
					Win32.WinUser.WS_CAPTION or
					Win32.WinUser.WS_SYSMENU,
			X            => Win32.WinUser.CW_USEDEFAULT,
			Y            => Win32.WinUser.CW_USEDEFAULT,
			nWidth       => CREATE_SIZE_X,
			nHeight      => CREATE_SIZE_Y,
			hWndParent   => PARENT_HWND,
			hMenu        => System.Null_Address,
			hInstance    => HINST,
			lpParam      => System.Null_Address);
      --
      -- If window could not be created, return "failure", else add it to the list 
      -- of child windows.
      --
      if CHILDHWND = System.Null_Address then
	SYSTEM_MSG := BUFFER(BUFFER'First)'access;
	ERROR_LEN := Win32.WinBase.FormatMessage(
				    Win32.WinBase.format_message_from_system,
				    System.Null_Address,
				    Win32.WinBase.GetLastError,
				    Win32.DWORD(Win32.WinNT.MakeLangId(
					       Win32.WinNT.lang_english,
					       Win32.WinNT.sublang_english_us)),
				    SYSTEM_MSG,
				    DWORD(BUFFER'last));
	IRESULT := Win32.WinUser.MessageBox(
				      Win32.WinUser.GetFocus,
				      LPCSTR(SYSTEM_MSG),
				      CP(title_str),
				      Win32.WinUser.MB_OK);
	return System.Null_Address;
      else
	CHILDREN(NUM_CHILDREN) := CHILDHWND;
      end if;
      return CHILDHWND;
    end CreateWindow;

begin
       SZCHILDNAME  := CP(childname);


-------------------------------------------------------------------------------
--
-- THIS FILE AND ANY ASSOCIATED DOCUMENTATION IS FURNISHED "AS IS" WITHOUT 
-- WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
-- TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR 
-- PURPOSE.  The user assumes the entire risk as to the accuracy and the 
-- use of this file.
--
-- Copyright (c) Intermetrics, Inc. 1995
-- Royalty-free, unlimited, worldwide, non-exclusive use, modification, 
-- reproduction and further distribution of this file is permitted.
--
-------------------------------------------------------------------------------

end PcxChild;
