-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.numerics.generic_elementary_functions.bdy,v $ 
-- $Revision: 1.7 $ $Date: 96/10/25 14:03:57 $ $Author: stt $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System_Pure;

package body Ada.Numerics.Generic_Elementary_Functions is
 pragma Suppress( Elaboration_Check );

 subtype Assert_True is Boolean range True .. True;
    -- Used to check assumptions

 Max_Digits : constant := System_Pure.Max_Digits;  -- To shorten code below
 function Min(Left, Right : Integer) return Integer renames Integer'Min;
					      -- To shorten code below

 subtype Common_Float  is  Float_Type'Base;
       

 type    Common_Int    is  range  System_Pure.Min_Int..System_Pure.Max_Int;
 type    Position      is  (Lead, Trail);
 type    Quadrant_Flag is  range  0..3;

 function "rem" (X, Y : Common_Float) return Common_Float 
    renames Common_Float'Remainder;

 Radix : constant Integer := Common_Float'Machine_Radix;

 Radix_Is_Appropriate : constant Assert_True := Radix = 2 or else Radix = 16;

 N_Bits_Per_Radix_Digit : constant Integer := -- 1 or 4, depending on radix
   Boolean'Pos(Radix = 2) * 1 + Boolean'Pos(Radix = 16) * 4;


 Mant : constant Integer := Common_Float'Machine_Mantissa;
 Man2 : constant Integer := Mant/2;

 Min_Mantissa_Bits       : constant := 12;

 Mantissa_Big_Enough     : constant Assert_True :=
   Mant * N_Bits_Per_Radix_Digit >= Min_Mantissa_Bits;

 -- =======================================================================--
 --          DECLARATION RELATED TO TRIGONOMETRIC FUNCTIONS                --
 -- =======================================================================--

     Two_Pi : constant := 2.0 * Pi;

     Large_Trig_Threshold : constant Common_Float :=
	     Common_Float(Radix)**(Float_Type'Machine_Mantissa/2);
		-- Values greater than this are not as accurately
		-- reduced; simply use "'Remainder" to handle these.

     Two_Pi_Inv : constant := 1.0 / Two_Pi;


 -- The following declarations are needed for our trigonometric argument
 -- reduction scheme, which needs many digits of the transcendental number
 -- pi/2. That value is represented  by the sum
 --    Piby2_Piece(1) + Piby2_Piece(2) + ... 
 --             ... + Piby2_Piece( N_Pieces_of_Piby2 ).
 -- Each piece is a Common_Float type with the lesser significant half of 
 -- its mantissa being zero. Since we cannot determine a priori the mantissa
 -- length of Common_Float, the value of the Piby2_Pieces are initialized by
 -- a sequence of statements.

   Piby2 : constant :=  -- Pi/2, to 90 hex digits
      (16#3.243F6A8885A308D313198A2E03707344A4093822299F# +
       16#3.1D0082EFA98EC4E6C89452821E638D01377BE5466CF3#E-45)/2.0;


   N_Hex_Digits_of_Piby2   : constant := 90;
   N_Bits_of_Piby2	   : constant := N_Hex_Digits_of_Piby2 * 4;

   Min_Bits_per_Piece      : constant := Min_Mantissa_Bits/2;
   Max_Pieces_of_Piby2     : constant := N_Bits_of_Piby2 / Min_Bits_per_Piece;

   N_Bits_per_Piece      : constant Integer := Man2 * N_Bits_Per_Radix_Digit;
   N_Pieces_of_Piby2     : constant Integer := N_Bits_of_Piby2 /
                                               N_Bits_per_Piece;

   Piby2_Piece : constant array(1..Max_Pieces_of_Piby2) of Common_Float :=
     (Common_Float'Leading_Part(Piby2, Man2*1),
      Common_Float'Leading_Part(Piby2, Man2*2) -
	Common_Float'Leading_Part(Piby2, Man2*1),
      Common_Float'Leading_Part(Piby2, Man2*3) -
	Common_Float'Leading_Part(Piby2, Man2*2),
      Common_Float'Leading_Part(Piby2, Man2*4) -
	Common_Float'Leading_Part(Piby2, Man2*3),
      Common_Float'Leading_Part(Piby2, Man2*5) -
	Common_Float'Leading_Part(Piby2, Man2*4),
      Common_Float'Leading_Part(Piby2, Man2*6) -
	Common_Float'Leading_Part(Piby2, Man2*5),
      Common_Float'Leading_Part(Piby2, Man2*7) -
	Common_Float'Leading_Part(Piby2, Man2*6),
      Common_Float'Leading_Part(Piby2, Man2*8) -
	Common_Float'Leading_Part(Piby2, Man2*7),
      Common_Float'Leading_Part(Piby2, Man2*9) -
	Common_Float'Leading_Part(Piby2, Man2*8),
      others => 0.0);

-- NOTE: *** Remaining commented out because problems with underflow 
--       *** at compile-time.

--       Common_Float'Leading_Part(Piby2, Man2*10) -
-- 	Common_Float'Leading_Part(Piby2, Man2*9),
--       Common_Float'Leading_Part(Piby2, Man2*11) -
-- 	Common_Float'Leading_Part(Piby2, Man2*10),
--       Common_Float'Leading_Part(Piby2, Man2*12) -
-- 	Common_Float'Leading_Part(Piby2, Man2*11),
--       Common_Float'Leading_Part(Piby2, Man2*13) -
-- 	Common_Float'Leading_Part(Piby2, Man2*12),
--       Common_Float'Leading_Part(Piby2, Man2*14) -
-- 	Common_Float'Leading_Part(Piby2, Man2*13),
--       Common_Float'Leading_Part(Piby2, Man2*15) -
-- 	Common_Float'Leading_Part(Piby2, Man2*14),
--       Common_Float'Leading_Part(Piby2, Man2*16) -
-- 	Common_Float'Leading_Part(Piby2, Man2*15),
--       Common_Float'Leading_Part(Piby2, Man2*17) -
-- 	Common_Float'Leading_Part(Piby2, Man2*16),
--       Common_Float'Leading_Part(Piby2, Man2*18) -
-- 	Common_Float'Leading_Part(Piby2, Man2*17),
--       Common_Float'Leading_Part(Piby2, Man2*19) -
-- 	Common_Float'Leading_Part(Piby2, Man2*18),
--       Common_Float'Leading_Part(Piby2, Man2*20) -
-- 	Common_Float'Leading_Part(Piby2, Man2*19),
--       Common_Float'Leading_Part(Piby2, Man2*21) -
-- 	Common_Float'Leading_Part(Piby2, Man2*20),
--       Common_Float'Leading_Part(Piby2, Man2*22) -
-- 	Common_Float'Leading_Part(Piby2, Man2*21),
--       Common_Float'Leading_Part(Piby2, Man2*23) -
-- 	Common_Float'Leading_Part(Piby2, Man2*22),
--       Common_Float'Leading_Part(Piby2, Man2*24) -
-- 	Common_Float'Leading_Part(Piby2, Man2*23),
--       Common_Float'Leading_Part(Piby2, Man2*25) -
-- 	Common_Float'Leading_Part(Piby2, Man2*24),
--       Common_Float'Leading_Part(Piby2, Man2*26) -
-- 	Common_Float'Leading_Part(Piby2, Man2*25),
--       Common_Float'Leading_Part(Piby2, Man2*27) -
-- 	Common_Float'Leading_Part(Piby2, Man2*26),
--       Common_Float'Leading_Part(Piby2, Man2*28) -
-- 	Common_Float'Leading_Part(Piby2, Man2*27),
--       Common_Float'Leading_Part(Piby2, Man2*29) -
-- 	Common_Float'Leading_Part(Piby2, Man2*28),
--       Common_Float'Leading_Part(Piby2, Man2*30) -
-- 	Common_Float'Leading_Part(Piby2, Man2*29),
--       Common_Float'Leading_Part(Piby2, Man2*31) -
-- 	Common_Float'Leading_Part(Piby2, Man2*30),
--       Common_Float'Leading_Part(Piby2, Man2*32) -
-- 	Common_Float'Leading_Part(Piby2, Man2*31),
--       Common_Float'Leading_Part(Piby2, Man2*33) -
-- 	Common_Float'Leading_Part(Piby2, Man2*32),
--       Common_Float'Leading_Part(Piby2, Man2*34) -
-- 	Common_Float'Leading_Part(Piby2, Man2*33),
--       Common_Float'Leading_Part(Piby2, Man2*35) -
-- 	Common_Float'Leading_Part(Piby2, Man2*34),
--       Common_Float'Leading_Part(Piby2, Man2*36) -
-- 	Common_Float'Leading_Part(Piby2, Man2*35),
--       Common_Float'Leading_Part(Piby2, Man2*37) -
-- 	Common_Float'Leading_Part(Piby2, Man2*36),
--       Common_Float'Leading_Part(Piby2, Man2*38) -
-- 	Common_Float'Leading_Part(Piby2, Man2*37),
--       Common_Float'Leading_Part(Piby2, Man2*39) -
-- 	Common_Float'Leading_Part(Piby2, Man2*38),
--       Common_Float'Leading_Part(Piby2, Man2*40) -
-- 	Common_Float'Leading_Part(Piby2, Man2*39),
--       Common_Float'Leading_Part(Piby2, Man2*41) -
-- 	Common_Float'Leading_Part(Piby2, Man2*40),
--       Common_Float'Leading_Part(Piby2, Man2*42) -
-- 	Common_Float'Leading_Part(Piby2, Man2*41),
--       Common_Float'Leading_Part(Piby2, Man2*43) -
-- 	Common_Float'Leading_Part(Piby2, Man2*42),
--       Common_Float'Leading_Part(Piby2, Man2*44) -
-- 	Common_Float'Leading_Part(Piby2, Man2*43),
--       Common_Float'Leading_Part(Piby2, Man2*45) -
-- 	Common_Float'Leading_Part(Piby2, Man2*44),
--       Common_Float'Leading_Part(Piby2, Man2*46) -
-- 	Common_Float'Leading_Part(Piby2, Man2*45),
--       Common_Float'Leading_Part(Piby2, Man2*47) -
-- 	Common_Float'Leading_Part(Piby2, Man2*46),
--       Common_Float'Leading_Part(Piby2, Man2*48) -
-- 	Common_Float'Leading_Part(Piby2, Man2*47),
--       Common_Float'Leading_Part(Piby2, Man2*49) -
-- 	Common_Float'Leading_Part(Piby2, Man2*48),
--       Common_Float'Leading_Part(Piby2, Man2*50) -
-- 	Common_Float'Leading_Part(Piby2, Man2*49),
--       Common_Float'Leading_Part(Piby2, Man2*51) -
-- 	Common_Float'Leading_Part(Piby2, Man2*50),
--       Common_Float'Leading_Part(Piby2, Man2*52) -
-- 	Common_Float'Leading_Part(Piby2, Man2*51),
--       Common_Float'Leading_Part(Piby2, Man2*53) -
-- 	Common_Float'Leading_Part(Piby2, Man2*52),
--       Common_Float'Leading_Part(Piby2, Man2*54) -
-- 	Common_Float'Leading_Part(Piby2, Man2*53),
--       Common_Float'Leading_Part(Piby2, Man2*55) -
-- 	Common_Float'Leading_Part(Piby2, Man2*54),
--       Common_Float'Leading_Part(Piby2, Man2*56) -
-- 	Common_Float'Leading_Part(Piby2, Man2*55),
--       Common_Float'Leading_Part(Piby2, Man2*57) -
-- 	Common_Float'Leading_Part(Piby2, Man2*56),
--       Common_Float'Leading_Part(Piby2, Man2*58) -
-- 	Common_Float'Leading_Part(Piby2, Man2*57),
--       Common_Float'Leading_Part(Piby2, Man2*59) -
-- 	Common_Float'Leading_Part(Piby2, Man2*58),
--       Common_Float'Leading_Part(Piby2, Man2*60) -
-- 	Common_Float'Leading_Part(Piby2, Man2*59));


 -- =======================================================================--
 --          DECLARATION RELATED TO EXPONENTIAL FUNCTIONS                  --
 -- =======================================================================--

    Two_to           : constant array (-3..3) of Common_Float := 
			    ( 0.125, 0.25, 0.5, 1.0, 2.0, 4.0, 8.0 );

    Log2             : constant :=
		         16#0.B17217F7D1CF79ABC9E3B39803F2F6AF40#;

    Digs : constant Integer := Common_Float'Digits;
    Digit_Range_Required : constant Assert_True := Digs in 4..33;

    Log2_by_32 : constant :=
	  16#5.8B90B_FBE8E_7BCD5_E4F1D_9CC01_F97B5_7A079_A1933_94C#E-2;

    Log2_by_32_Lead : constant Common_Float :=
	Common_Float'Leading_Part(Log2_by_32, Man2);
    Log2_by_32_Trail : constant Common_Float := Log2_by_32 - Log2_by_32_Lead;

    Pow_Log2_Tmp : constant := 16#0.B17217#;

    Pow_Log2_Lead : constant Common_Float := 
      Common_Float'Leading_Part(Pow_Log2_Tmp, Mant - Man2);
    Pow_Log2_Trail: constant Common_Float :=
      16#0.F7D1C_F79AB_C9E3B_39803_F2F6A_F40F34#E-6 + 
	(Pow_Log2_Tmp - Pow_Log2_Lead);


 -- Two_to_Jby32 is an array of 32 pairs of Common_Float numbers
 -- representing 2**(j/32) for j = 0, 1, 2, ..., 31. Each such value
 -- is represented by Lead + Trail. The leading parts contain 13 bits
 -- of information and are consequently model numbers as long as
 -- System.Max_Digits is >= 4. The trailing parts contain roughly
 -- Common_Float'Mantissa bits of information, under the assumption
 -- that System.Max_Digits is <= 35. So, when the assumptions are met,
 -- Lead + Trail represents 2**(j/32) to roughly 13 extra bits.

   Two_to_Jby32 : constant array(0..31, Position) of Common_Float := (
   (16#1.000#,    16#0.000000000000000000000000000000000#),
   (16#1.059#,    16#0.000B0D31585743AE7C548EB68CA417FE5#),
   (16#1.0B5#,    16#0.000586CF9890F6298B92B71842A983642#),
   (16#1.113#,    16#0.00001D0125B50A4EBBF1AED9318CEAC5C#),
   (16#1.172#,    16#0.000B83C7D517ADCDF7C8C50EB14A79203#),
   (16#1.1D4#,    16#0.000873168B9AA7805B8028990F07A98B4#),
   (16#1.238#,    16#0.0007A6E75623866C1FADB1C15CB593B03#),
   (16#1.29E#,    16#0.0009DF51FDEE12C25D15F5A24AA3BCA88#),
   (16#1.306#,    16#0.000FE0A31B7152DE8D5A46305C85EDECB#),
   (16#1.371#,    16#0.000A7373AA9CAA7145502F4547987E3E1#),
   (16#1.3DE#,    16#0.000A64C12342235B41223E13D773FBA2C#),
   (16#1.44E#,    16#0.000086061892D03136F409DF019FBD4F3#),
   (16#1.4BF#,    16#0.000DAD5362A271D4397AFEC42E20E0363#),
   (16#1.534#,    16#0.0002B569D4F81DF0A83C49D86A63F4E67#),
   (16#1.5AB#,    16#0.00007DD48542958C93015191EB345D88D#),
   (16#1.624#,    16#0.0007EB03A5584B1F0FA06FD2DA42BB1CE#),
   (16#1.6A0#,    16#0.0009E667F3BCC908B2FB1366EA957D3E3#),
   (16#1.71F#,    16#0.00075E8EC5F73DD2370F2EF0ACD6CB434#),
   (16#1.7A1#,    16#0.0001473EB0186D7D51023F6CDA1F5EF42#),
   (16#1.825#,    16#0.00089994CCE128ACF88AFAB34A010F6AD#),
   (16#1.8AC#,    16#0.000E5422AA0DB5BA7C55A192C9BB3E6ED#),
   (16#1.937#,    16#0.00037B0CDC5E4F4501C3F2540A22D2FC4#),
   (16#1.9C4#,    16#0.0009182A3F0901C7C46B071F2BE58DDAD#),
   (16#1.A55#,    16#0.00003B23E255C8B424491CAF87BC8050A#),
   (16#1.AE8#,    16#0.0009F995AD3AD5E8734D1773205A7FBC3#),
   (16#1.B7F#,    16#0.00076F2FB5E46EAA7B081AB53C5354C88#),
   (16#1.C19#,    16#0.0009BDD85529C2220CB12A091BA667944#),
   (16#1.CB7#,    16#0.00020DCEF90691503CBD1E949DB761D95#),
   (16#1.D58#,    16#0.00018DCFBA48725DA05AEB66E0DCA9F58#),
   (16#1.DFC#,    16#0.00097337B9B5EB968CAC39ED291B7225A#),
   (16#1.EA4#,    16#0.000AFA2A490D9858F73A18F5DB301F86D#),
   (16#1.F50#,    16#0.000765B6E4540674F84B762862BAFF98F#) );

   Log_of_Jby64 : constant array(64..128, Position )
                  of Common_Float := (
   (0.0,          0.0),
   (16#0.03F#,    16#0.000815161F807C79F3DB4E9A6F57AADBEA#),
   (16#0.07E#,    16#0.0000A6C39E0CC0133E3F04F1EF229FAEAE#),
   (16#0.0BB#,    16#0.000A2C7B196E7E231A7950F7252C163C9B#),
   (16#0.0F8#,    16#0.0005186008B15330BE64B8B775997898D3#),
   (16#0.134#,    16#0.0001D7961BD1D092998376104D1375021A#),
   (16#0.16F#,    16#0.0000D28AE56B4B9BE499B9ED19B640CE50#),
   (16#0.1A9#,    16#0.00026D3A4AD563650BD22A9C3AA4C79A9F#),
   (16#0.1E2#,    16#0.0007076E2AF2E5E9EA87FFE1FE9E155DB9#),
   (16#0.21A#,    16#0.000EFCF9A11CB2CD2EE2F481855D1C484C#),
   (16#0.252#,    16#0.000AA5F03FEA46980BB8E203EDF4D109FB#),
   (16#0.289#,    16#0.000A56D996FA3CCFA7B2A1F0FC3C1882CA#),
   (16#0.2BF#,    16#0.000E60E14F27A790E7C4140E424775FCD5#),
   (16#0.2F5#,    16#0.0007120421B21237C6D65AD40C100C8FFC#),
   (16#0.32A#,    16#0.0004B539E8AD68EC8260EA71712CEC4CA0#),
   (16#0.35E#,    16#0.0007929D017FE5B19CC0326F99EB976769#),
   (16#0.391#,    16#0.000FEF8F353443584BB03DE5FF734495C7#),
   (16#0.3C4#,    16#0.000E0EDC55E5CBD3D50FFFC3FD3C2ABB72#),
   (16#0.3F7#,    16#0.000230DABC7C551AAA8CD86F29A5941240#),
   (16#0.428#,    16#0.000C9389CE438D7DCFDE8061C030E28DE0#),
   (16#0.459#,    16#0.000D72AEAE98380E731F55C41B8B823F06#),
   (16#0.48A#,    16#0.000507EF3DE596890A14F69D750CBD2E9A#),
   (16#0.4BA#,    16#0.00038AEB8474C270B3246A14206CF37B77#),
   (16#0.4E9#,    16#0.00093155A517A71CBCD735D034237D6F47#),
   (16#0.518#,    16#0.00062F08717B09F42DECDECCF1CD105772#),
   (16#0.546#,    16#0.000AB61CB7E0B42724F5833EABC623A9E9#),
   (16#0.574#,    16#0.0006F6FD6027294236383DC7FE1159F380#),
   (16#0.5A1#,    16#0.000B207A6C52BB110AF840538E1A592DED#),
   (16#0.5CE#,    16#0.00075FDAEF401A7389314FEB4FBDE5AADD#),
   (16#0.5FA#,    16#0.000BE0EE0ABF0D92CE979ED2950437160C#),
   (16#0.626#,    16#0.0008CE1B05096AD69C620440F055B3FFC6#),
   (16#0.651#,    16#0.000E5070845BEAE9337451F441BABA929C#),
   (16#0.67C#,    16#0.000C8FB2FE612FCADA35D9BD014886067D#),
   (16#0.6A7#,    16#0.0003B26A68212635213FD4BC950D7BE11F#),
   (16#0.6D1#,    16#0.0003DDEF323D8A32FBB6ABA63878EF2053#),
   (16#0.6FA#,    16#0.000D36769C6DEFDE1874DEAEF06B25B52C#),
   (16#0.723#,    16#0.000FDF1E6A6886B097607BCBFEE6892B8E#),
   (16#0.74C#,    16#0.000BF9F803AF55877B232FAFA36FD18AB4#),
   (16#0.775#,    16#0.0001A813071282FB989A927476E1FE9F50#),
   (16#0.79D#,    16#0.000109875A1E1F8DF68DBCF2ED1BB404A1#),
   (16#0.7C4#,    16#0.000A3D7EBC1BB2CD720EC44C73D75CF564#),
   (16#0.7EB#,    16#0.000D623DE3CC7B66BECF93AA1AFEC6D4CD#),
   (16#0.812#,    16#0.000A952D2E87F634E34AEBF73FFE346E4B#),
   (16#0.839#,    16#0.0001F2E0E6FA0272BCB1C488B755B2B7A5#),
   (16#0.85F#,    16#0.00039721295415B4C4BDD99EFFE69B6436#),
   (16#0.884#,    16#0.000F9CF16A64B7EF1F64D85BC8C5F24163#),
   (16#0.8AA#,    16#0.00061E97A6AF4D4C799D1CB2F14054ED3A#),
   (16#0.8CF#,    16#0.000735A33E4B7662E5EEBBC0EF3D571078#),
   (16#0.8F4#,    16#0.0002FAF3820681EF62CD2F9F1E35F2E7CA#),
   (16#0.918#,    16#0.000986BDF5FA1416F1B439165240A471BC#),
   (16#0.93C#,    16#0.000AF0944D88D75BC1F9EDCB438FFC0352#),
   (16#0.960#,    16#0.00074F6A24745DCBD4E18DD14F312A40A5#),
   (16#0.983#,    16#0.000EB99A7885F0FDAC850FAB36CDEE180B#),
   (16#0.9A7#,    16#0.000144ECE70E98B75C96C42E727572531D#),
   (16#0.9C9#,    16#0.000F069AB150CD4E221301B6F8C38F6217#),
   (16#0.9EC#,    16#0.000813538AB7D5202131E85693CF6B809D#),
   (16#0.A0E#,    16#0.000C7F423395732325E617A300BBCA9C44#),
   (16#0.A30#,    16#0.000C5E10E2F613E85BD9BD99E39A20AEE5#),
   (16#0.A52#,    16#0.0007C2ED81F5D8113DFA3D3761B6316E6F#),
   (16#0.A73#,    16#0.000EC08DBADD84E584C2B22C2AEE1A18BD#),
   (16#0.A95#,    16#0.00016932DE2D5773BE4578AD97AEA7BED0#),
   (16#0.AB5#,    16#0.000FCEAD9F9CCA08E310B9B1FE59CDC156#),
   (16#0.AD6#,    16#0.000A0261ACF967D94D552F811CD4084583#),
   (16#0.AF7#,    16#0.0000154920B3AB86B04AFE92103EF4C629#),
   (16#0.B17#,    16#0.000217F7D1CF79ABC9E3B39803F2F6AF40#) );

   Pow_Log_of_Jby64 : constant array(64..128, Position )
                  of Common_Float := (
   (0.0,          0.0),
   (16#0.03F8151#, 16#0.61F807C79F3DB4E9A6F57AADBEB03BE903DDC50#E-7),
   (16#0.07E0A6C#, 16#0.39E0CC0133E3F04F1EF229FAEAEFAE14CDDF350#E-7),
   (16#0.0BBA2C7#, 16#0.B196E7E231A7950F7252C163C9BF701B2A89D80#E-7),
   (16#0.0F85186#, 16#0.008B15330BE64B8B775997898D3474D3375B520#E-7),
   (16#0.1341D7#,  16#0.961BD1D092998376104D1375021A0ECA5B78467#E-6),
   (16#0.16F0D2#,  16#0.8AE56B4B9BE499B9ED19B640CE50C1EF65087FD#E-6),
   (16#0.1A926D#,  16#0.3A4AD563650BD22A9C3AA4C79A9F67E22ED398C#E-6),
   (16#0.1E2707#,  16#0.6E2AF2E5E9EA87FFE1FE9E155DB94EBC4017F6F#E-6),
   (16#0.21AEFC#,  16#0.F9A11CB2CD2EE2F481855D1C484CB4FD8D03860#E-6),
   (16#0.252AA5#,  16#0.F03FEA46980BB8E203EDF4D109FBC9070F7E29F#E-6),
   (16#0.289A56#,  16#0.D996FA3CCFA7B2A1F0FC3C1882CAF99174F60AA#E-6),
   (16#0.2BFE60#,  16#0.E14F27A790E7C4140E424775FCD55C7355FDF3E#E-6),
   (16#0.2F5712#,  16#0.0421B21237C6D65AD40C100C8FFC2929B102165#E-6),
   (16#0.32A4B5#,  16#0.39E8AD68EC8260EA71712CEC4CA0BED3CF71766#E-6),
   (16#0.35E792#,  16#0.9D017FE5B19CC0326F99EB976769B8B9A5D50C9#E-6),
   (16#0.391FEF#,  16#0.8F353443584BB03DE5FF734495C765EA7411ADB#E-6),
   (16#0.3C4E0E#,  16#0.DC55E5CBD3D50FFFC3FD3C2ABB729D78802FEDF#E-6),
   (16#0.3F7230#,  16#0.DABC7C551AAA8CD86F29A5941240584455B22C8#E-6),
   (16#0.428C93#,  16#0.89CE438D7DCFDE8061C030E28DE035AFBE0972D#E-6),
   (16#0.459D72#,  16#0.AEAE98380E731F55C41B8B823F067D04A43C19F#E-6),
   (16#0.48A507#,  16#0.EF3DE596890A14F69D750CBD2E9AAD37A78762E#E-6),
   (16#0.4BA38A#,  16#0.EB8474C270B3246A14206CF37B77AD6FB226F14#E-6),
   (16#0.4E9931#,  16#0.55A517A71CBCD735D034237D6F479DCFC053C8D#E-6),
   (16#0.51862F#,  16#0.08717B09F42DECDECCF1CD105772CD24C00B443#E-6),
   (16#0.546AB6#,  16#0.1CB7E0B42724F5833EABC623A9E9E6AF97F5C12#E-6),
   (16#0.5746F6#,  16#0.FD6027294236383DC7FE1159F380B4A6B429A4B#E-6),
   (16#0.5A1B20#,  16#0.7A6C52BB110AF840538E1A592DEDED1C3395996#E-6),
   (16#0.5CE75F#,  16#0.DAEF401A7389314FEB4FBDE5AADDE10DCEA5975#E-6),
   (16#0.5FABE0#,  16#0.EE0ABF0D92CE979ED2950437160CBFCBF71EE8D#E-6),
   (16#0.6268CE#,  16#0.1B05096AD69C620440F055B3FFC63281B40515A#E-6),
   (16#0.651E50#,  16#0.70845BEAE9337451F441BABA929CC25DCA0FA1A#E-6),
   (16#0.67CC8F#,  16#0.B2FE612FCADA35D9BD014886067D20FFB34547D#E-6),
   (16#0.6A73B2#,  16#0.6A68212635213FD4BC950D7BE11FC8EE26768C4#E-6),
   (16#0.6D13DD#,  16#0.EF323D8A32FBB6ABA63878EF2053AB4D08603CF#E-6),
   (16#0.6FAD36#,  16#0.769C6DEFDE1874DEAEF06B25B52C1BE100233B3#E-6),
   (16#0.723FDF#,  16#0.1E6A6886B097607BCBFEE6892B8ECBD4E8235B8#E-6),
   (16#0.74CBF9#,  16#0.F803AF55877B232FAFA36FD18AB4DFF5EFD6E58#E-6),
   (16#0.7751A8#,  16#0.13071282FB989A927476E1FE9F50684CE6BAFCF#E-6),
   (16#0.79D109#,  16#0.875A1E1F8DF68DBCF2ED1BB404A18E2AA5EE014#E-6),
   (16#0.7C4A3D#,  16#0.7EBC1BB2CD720EC44C73D75CF5649117429EC74#E-6),
   (16#0.7EBD62#,  16#0.3DE3CC7B66BECF93AA1AFEC6D4CDE2EF184DC7B#E-6),
   (16#0.812A95#,  16#0.2D2E87F634E34AEBF73FFE346E4B8B8C4C34EBB#E-6),
   (16#0.8391F2#,  16#0.E0E6FA0272BCB1C488B755B2B7A5D75211210F7#E-6),
   (16#0.85F397#,  16#0.21295415B4C4BDD99EFFE69B64366FBBF35D3ED#E-6),
   (16#0.884F9C#,  16#0.F16A64B7EF1F64D85BC8C5F24163E6F9907E4AD#E-6),
   (16#0.8AA61E#,  16#0.97A6AF4D4C799D1CB2F14054ED3A330F341CF1F#E-6),
   (16#0.8CF735#,  16#0.A33E4B7662E5EEBBC0EF3D571078EA06C2C371D#E-6),
   (16#0.8F42FA#,  16#0.F3820681EF62CD2F9F1E35F2E7CA4F4817696AD#E-6),
   (16#0.918986#,  16#0.BDF5FA1416F1B439165240A471BCDFCC1F5B0C6#E-6),
   (16#0.93CAF0#,  16#0.944D88D75BC1F9EDCB438FFC03527D7309433BB#E-6),
   (16#0.96074F#,  16#0.6A24745DCBD4E18DD14F312A40A546F842B7451#E-6),
   (16#0.983EB9#,  16#0.9A7885F0FDAC850FAB36CDEE180B7013338119B#E-6),
   (16#0.9A7144#,  16#0.ECE70E98B75C96C42E727572531DDFD382B6BE3#E-6),
   (16#0.9C9F06#,  16#0.9AB150CD4E221301B6F8C38F621717C9E700413#E-6),
   (16#0.9EC813#,  16#0.538AB7D5202131E85693CF6B809D96954ADF1FF#E-6),
   (16#0.A0EC7F#,  16#0.423395732325E617A300BBCA9C4486EA2756F59#E-6),
   (16#0.A30C5E#,  16#0.10E2F613E85BD9BD99E39A20AEE59A498016887#E-6),
   (16#0.A527C2#,  16#0.ED81F5D8113DFA3D3761B6316E6F7191D8AF47E#E-6),
   (16#0.A73EC0#,  16#0.8DBADD84E584C2B22C2AEE1A18BD794408F7745#E-6),
   (16#0.A95169#,  16#0.32DE2D5773BE4578AD97AEA7BED0920F6A4C39B#E-6),
   (16#0.AB5FCE#,  16#0.AD9F9CCA08E310B9B1FE59CDC15631BF5C35094#E-6),
   (16#0.AD6A02#,  16#0.61ACF967D94D552F811CD40845839E04578161C#E-6),
   (16#0.AF7015#,  16#0.4920B3AB86B04AFE92103EF4C629F04AE4E7A29#E-6),
   (16#0.B17217#,  16#0.F7D1CF79ABC9E3B39803F2F6AF40F343267298B#E-6) );

   
 --=====================================================================--
 --        DECLARATION OF KERNEL FUNCTIONS AND PROCEDURES               --
 --=====================================================================--

 -----------------------------------------------------------------------------
 --    Kernel functions and procedures for the exp/log/hyp family.          --
 -----------------------------------------------------------------------------
    function KF_R1pSm( Y : Common_Float ) return Common_Float is

    -- On input, Y lies in the interval [-1/64, 1/64]
    -- On output, the value sqrt( 1 + Y ) - 1 is returned.
    -- The approximation employs both minmax polynomial and Newton Iteration.
    --     
    -- First, sqrt(1+Y)-1 is approximated by a polynomial of the form
    --    0.5 Y - Y^2*(A1 - Y*(A2 - Y*(A3 ... ))).
    -- Depending of the number of digits required, this polynomial may or may
    -- not be the value return. Should the polynomial fall short of the
    -- accuracy requirement, one step of the Newton iteration is applied to
    -- the equation
    --      (P + 1)^2   =   1 + Y, where P is the unknown.
    -- Thus,
    --      new P  :=   P   -  (P^2 + 2P - Y)/(2P + 2)
    -- where the initial guess P is the value of the polynomial.

       Poly : Common_Float;

    begin


       case Float_Type'Base'Digits is

	  when 1..6 =>

	     declare
		type Working_Float is digits Min(6, Max_Digits);
		R, P : Working_Float;
	     begin
		R := Working_Float( Y );
		P := R*0.5 -
		     R*R*( 0.12500_79870 -
		       R*( 0.62505_22999E-01 ));
		Poly := Common_Float( P );
	     end;

	  when 7..15 =>

	     declare
		type Working_Float is digits Min(15, Max_Digits);
		R, P, Two_P, Correction : Working_Float;
	     begin
		R := Working_Float( Y );
		P := R*0.5 -
		     R*R*( 0.12500_79870 -
		       R*( 0.62505_22999E-01 ));
		Two_P := P + P;
		Correction := (P*P + (Two_P-R)) / (Two_P + 2.0);
		Poly := Common_Float(P) - Common_Float(Correction);
	     end;

	  when 16 =>

	     declare
		type Working_Float is digits Min(16, Max_Digits);
		R, P, Two_P, Correction : Working_Float; 
	     begin
		R := Working_Float( Y );
		P := R*0.5 -
		     R*R*( 0.12500_79870 -
		       R*( 0.62505_22999E-01 ));
		Two_P := P + P;
		Correction := (P*P + (Two_P-R)) / (Two_P + 2.0);
		Poly := Common_Float(P) - Common_Float(Correction);
	     end;

	  when 17..18 =>

	     declare
		type Working_Float is digits Min(18, Max_Digits);
		R, P, Two_P, Correction : Working_Float; 
	     begin
		R := Working_Float( Y );
		P := R*0.5 -
		     R*R*( 0.125 -
		       R*( 0.62505_85095_32379_40606E-01 -
		       R*( 0.39067_55576_62793_58660E-01 )));
		Two_P := P + P;
		Correction := (P*P + (Two_P-R)) / (Two_P + 2.0);
		Poly := Common_Float(P) - Common_Float(Correction);
	     end;

	  when 19..27 =>

	     declare
		type Working_Float is digits Min(27, Max_Digits);
		R, P, Two_P, Correction : Working_Float; 
	     begin
		R := Working_Float( Y );
		P := R*0.5 -
		     R*R*( 0.12500_00000_00043_29869_79603     -
		       R*( 0.62499_99949_58313_28657_64839E-01 -
		       R*( 0.39062_49897_81141_25807_82311E-01 -
		       R*( 0.27349_66395_87080_51587_13279E-01 -
		       R*( 0.20514_47322_06561_78273_82000E-01 )))));
		Two_P := P + P;
		Correction := (P*P + (Two_P-R)) / (Two_P + 2.0);
		Poly := Common_Float(P) - Common_Float(Correction);
	     end;

	  when 28..33 =>

	     declare
		type Working_Float is digits Min(33, Max_Digits);
		R, P, Two_P, Correction : Working_Float; 
	     begin
		R := Working_Float( Y );
		P := R*0.5 -
		     R*R*( 0.12500_00000_00035_79677_13859_89202_617 -
		       R*( 0.62500_00000_00664_48622_98045_60336E-01 -
		       R*( 0.39062_49911_61750_24807_32583_41508E-01 -
		       R*( 0.27343_74885_03551_93697_91233_69204E-01 -
		       R*( 0.20514_00699_96599_52762_98109_70261E-01 -
		       R*( 0.16119_54134_09746_99174_73737_89068E-01 ))))));
		Two_P := P + P;
		Correction := (P*P + (Two_P-R)) / (Two_P + 2.0);
		Poly := Common_Float(P) - Common_Float(Correction);
	     end;

	  when others =>

	     raise Program_Error;  -- assumption (1) is violated.

       end case;

       return( Poly );

    end KF_R1pSm;

    -- Rt_Jby32 is an array of pairs of Common_Float numbers
    -- representing sqrt(j/32) for j = 32, 33, ..., 128. Each such value
    -- is represented by Lead + Trail. The leading parts contain 13 bits
    -- of information and are consequently model numbers as long as
    -- System.Max_Digits is >= 4. The trailing parts contain roughly
    -- Common_Float'Mantissa bits of information, under the assumption
    -- that System.Max_Digits is <= 33. 

    Rt_Jby32 : constant array( Integer range 32..128, 
				  Position ) of Common_Float := (
     (16#1.000#,   16#0.000000000000000000000000000000000000#),
     (16#1.03F#,   16#0.00081F636B80BBC40DD15FB55CFE8794F1E3#),
     (16#1.07E#,   16#0.0000F66AFED06F5AC8F119530A885D60CA10#),
     (16#1.0BB#,   16#0.000B307ACAFDAEAD568979062D2866CD0BAA#),
     (16#1.0F8#,   16#0.00076CCDF6CD96C6863C4E8D2FF01DEEAC27#),
     (16#1.134#,   16#0.00063FA37014E0D94ADD534C66998A640703#),
     (16#1.16F#,   16#0.0008334644DF885E619FD78EDC88849950ED#),
     (16#1.1A9#,   16#0.000DC8F6DF10380EB98F9C8F8ADA2DC38B3F#),
     (16#1.1E3#,   16#0.000779B97F4A7C15F39CC0605CEDC8341082#),
     (16#1.21C#,   16#0.0005B70D9F8239EEC19844640F5C4DA7CCEA#),
     (16#1.254#,   16#0.0008EB9151E8533304014EE90F133D57DEFE#),
     (16#1.28C#,   16#0.00017B9337833E50EAF5F17768E0292FE925#),
     (16#1.2C2#,   16#0.000FC595456A6CD213E9E0702D4D3664BE11#),
     (16#1.2F9#,   16#0.000422C23C47DED837A0D6249A34FEFC03D4#),
     (16#1.32E#,   16#0.000EE757704167B639DCC3ABFC978928FCD9#),
     (16#1.364#,   16#0.0000630445305A1C9E77362F3D783E11EA98#),
     (16#1.398#,   16#0.0008E1409212E7D0321914321A556473DB02#),
     (16#1.3CC#,   16#0.0008A99AF5452FE79C9BB0FA0D42CD967382#),
     (16#1.400#,   16#0.000000000000000000000000000000000000#),
     (16#1.432#,   16#0.000F24FB01C7A4E2FF83804B06E49D480837#),
     (16#1.465#,   16#0.000655F122FF6618B174BC3292ED999CC170#),
     (16#1.497#,   16#0.0005CD5768087BA33850FF422DE5ECD200DE#),
     (16#1.4C8#,   16#0.000DC2E42397FD6C5C1722145A8A2CA6DC62#),
     (16#1.4F9#,   16#0.000E6BBC4ECB3073969806124FC607D66B4B#),
     (16#1.52A#,   16#0.0007FA9D2F8E9B78E753F30FE1BD106AA53B#),
     (16#1.55A#,   16#0.000AA002A9D59FE40AFB89DBE06CA9CB3B5F#),
     (16#1.58A#,   16#0.00068A4A8D9F3541E6399C04B6432EDB7BB5#),
     (16#1.5B9#,   16#0.000BE5D52A9DA21319A3675EEEEF9BBBF815#),
     (16#1.5E8#,   16#0.000ADD236A58EF0839C1F97009CF6309A3C9#),
     (16#1.617#,   16#0.000398F2AAA47C931A88BB3F196BFAB425E5#),
     (16#1.645#,   16#0.000640568C1C3745DE1F93225C0898BA66A4#),
     (16#1.673#,   16#0.0002F8D0E2F76D8400571F7BA3A3BAD60731#),
     (16#1.6A0#,   16#0.0009E667F3BCC908B2FB1366EA957D3E3ADE#),
     (16#1.6CD#,   16#0.000B2BBB212EB580CCB36EAE8B5B6FFC91B7#),
     (16#1.6FA#,   16#0.0006EA162D0F00C8D121EB14BF48BC6E0A0B#),
     (16#1.726#,   16#0.000D41832A0BDFF13272AB89A7D4425F8E90#),
     (16#1.752#,   16#0.000E50DB3A3A1B1B33B0456F1FBAC9056B03#),
     (16#1.77E#,   16#0.000A35D632E434BDC52688BE93AD9C570D1E#),
     (16#1.7AA#,   16#0.00010D193C22CE6F126BFC5017AED875D8CB#),
     (16#1.7D5#,   16#0.0002F244809E94207DBCEE6757BB7A4A324D#),
     (16#1.800#,   16#0.000000000000000000000000000000000000#),
     (16#1.82A#,   16#0.0008500794E6BDA24AB1432A741CE963313E#),
     (16#1.854#,   16#0.000BFB363DC3964E67F28E1E1F9F6E29BE6E#),
     (16#1.87E#,   16#0.000B1990B697A1C43E9F593EA0EABD90D1C2#),
     (16#1.8A8#,   16#0.0005C24F706597DB81DAD62293FF086ABE21#),
     (16#1.8D1#,   16#0.000C0BE7F20ABC848EB241FBC1A45E32BEE8#),
     (16#1.8FA#,   16#0.000E0C15AD389E24852497E80935E4B5704C#),
     (16#1.923#,   16#0.000BD7E25164CB61CB9BF8B12FE273885183#),
     (16#1.94C#,   16#0.000583ADA5B529204A2BC830CD9BFEA55A70#),
     (16#1.974#,   16#0.000B2334F2346229C95A75D3C7E82CE6023A#),
     (16#1.99C#,   16#0.000CC999FFF002CC4680C7711DC346B5EE9B#),
     (16#1.9C4#,   16#0.000A8969B7076A4D5868CAEBA1823A72BA0F#),
     (16#1.9EC#,   16#0.000474A261263D57E76EC8B5F1811E33D9DA#),
     (16#1.A13#,   16#0.000A9CB99665081951BC6C5E8F43A079BFF3#),
     (16#1.A3A#,   16#0.000D12A1DA1605445A3D2F96610292C13A91#),
     (16#1.A61#,   16#0.000BE6CFEC996C7902A9A8946BF287D23F2D#),
     (16#1.A88#,   16#0.0007293FD6F34168EF24F1987215A65D94DD#),
     (16#1.AAE#,   16#0.000EE979B4837B83DD44184250109D8B3E77#),
     (16#1.AD5#,   16#0.000336963EEFBA20ED6B20908B64AC4E18C3#),
     (16#1.AFB#,   16#0.00041F432002DE536B5020173A54A81E71A1#),
     (16#1.B21#,   16#0.0001B1C70D02300F262F33757177947B7458#),
     (16#1.B46#,   16#0.000BFC05AEB896C71F30E6B727BA1B823BF4#),
     (16#1.B6C#,   16#0.00030B83593E63E9C7FA51B0978580314306#),
     (16#1.B91#,   16#0.0006ED68964EC24D4BABB755C51692DDAC14#),
     (16#1.BB6#,   16#0.0007AE8584CAA73B25742D7078B83B8925D8#),
     (16#1.BDB#,   16#0.00055B550FDBC1365E96AA5FAE317999642D#),
     (16#1.C00#,   16#0.000000000000000000000000000000000000#),
     (16#1.C24#,   16#0.0007A85FE81FA33B1DDED0A843F3D1971D1A#),
     (16#1.C48#,   16#0.000C6001F0ABFB4ADFB9D840A53ADC8DC996#),
     (16#1.C6C#,   16#0.000E322982A3F32365BE7C2BE5138018AE92#),
     (16#1.C90#,   16#0.000D29D2D43CD849493E03BB218420C5CA72#),
     (16#1.CB4#,   16#0.000951B558D173F97966AFE461040C6E3CF4#),
     (16#1.CD8#,   16#0.0002B446159F360FEDECCF37F9E485EE26DE#),
     (16#1.CFB#,   16#0.00095BB9DCC0C4B3A88215EB80C48475FE6B#),
     (16#1.D1E#,   16#0.000D52076FBE93E9150D6E50EBEC6D31C346#),
     (16#1.D41#,   16#0.000EA0E98AF91248C8006132C5DA1A797190#),
     (16#1.D64#,   16#0.000D51E0DB1C5BB84B259E4B278016ADC883#),
     (16#1.D87#,   16#0.00096E35DDBB227FBB235D6C6F3DEC157D48#),
     (16#1.DAA#,   16#0.0002FEFAAE1D7F600F8FE05DD35E1E018509#),
     (16#1.DCC#,   16#0.000A0D0CBF4081A6D162310DB56C5AB234A6#),
     (16#1.DEE#,   16#0.000EA11683F4920555C97F4F84DA4E72B441#),
     (16#1.E11#,   16#0.0000C39105FAF1761614C513963B3FA95DCD#),
     (16#1.E33#,   16#0.00007CC56CF5BDC1F6279CEDD9E4EA5CD9FE#),
     (16#1.E54#,   16#0.000DD4CE75F1DDDC9CD44073A363E611E0E7#),
     (16#1.E76#,   16#0.0008D399DC46F95CDFB2FE8FF8F93FCA3195#),
     (16#1.E98#,   16#0.000180E9B47F19250A2F1A4BDC64666B2228#),
     (16#1.EB9#,   16#0.0007E455B9EDB666D150215E20B45D93E9DB#),
     (16#1.EDA#,   16#0.000C054C8F94C42D7045695881AC1491A88A#),
     (16#1.EFB#,   16#0.000DEB14F4ED9B17AE807907E1E88FE92C4D#),
     (16#1.F1C#,   16#0.000D9CCEEF23946BF6193AAD828D8C3590F2#),
     (16#1.F3D#,   16#0.000B2174E746878544CB5AB14A71F75D3401#),
     (16#1.F5E#,   16#0.00067FDCBDF439D43E79ED8A41009E5B0A26#),
     (16#1.F7E#,   16#0.000FBEB8D4F11E17B175900CDA9F1B891674#),
     (16#1.F9F#,   16#0.0006E4990F2273685CB6BA3D0102FE4EB10C#),
     (16#1.FBF#,   16#0.000BF7EBC755E9355AFDEC97D29B989FF7D9#),
     (16#1.FDF#,   16#0.000EFEFEBE3D5BD940C0E0F83137D7F3C73B#),
     (16#2.000#,   16#0.000000000000000000000000000000000000#) );

    function KF_Sqrt( Y : Common_Float ) return Common_Float is

    -- On input, Y is a floating-point value in Common_Float;
    -- On output, the square root of Y is returned

    -- The caculation is carried out in three steps.
    --
    -- Step 1. Reduction.
    -- The input argument is scaled to the interval [1, 4) thus:
    --     input argument = 2^L * X, X in [1, 4).
    -- Furthermore, X is decomposed as X = C + T where
    --       C = 1 + j/32, j = 0,1,..,96; and |T| <= 1/64.
    --
    -- Step 2. Approximation.
    -- An approximation Q = sqrt(1 + (T/C)) - 1  is obtained. This is
    -- performed by the kernel function  kf_R1pSm.
    --
    -- Step 3. Reconstruction.
    -- The value of sqrt(Y) is reconstructed via
    --  sqrt(Y) = 2^(L/2) * sqrt(X)
    --          = 2^(L/2) * sqrt(C) * sqrt(X/C)
    --          = 2^(L/2) * sqrt(C) * sqrt(1 + T/C)
    --          = 2^(L/2) * [ sqrt(C)  + sqrt(C)*Q ]
    --

       X, X_Frac, X_Trail, R, C                  : Common_Float;
       Z, Z1, Z2, Q, RtC, RtC_Lead, RtC_Trail    : Common_Float;
       X_Exp, L, L1, L2, Index                   : Integer;
       

    begin

    -- Filter out exceptional cases.

       X := Y;

       if X < 0.0 then
	  raise Argument_Error;
       end if;

       if X = 0.0 then
	  return( Y );
       end if;


    -- Step 1. Argument Reduction. Must be extra careful for base 16 machines.

       X_Frac := Common_Float'Fraction( X );
       X_Exp := Common_Float'Exponent( X );
       if (Radix = 2) then

	  if (X_Exp mod 2  =  0) then
	     X := 4.0 * X_Frac;  L := X_Exp - 2;
	  else
	     X := 2.0 * X_Frac;  L := X_Exp - 1;
	  end if;
	  Index := Integer( 2#1.0#E5 * X );
	  C     := Common_Float(Index)*2#1.0#E-5;
	  R     := (X - C)/C;

       else

	  X_Exp := X_Exp * 4;
	  if (X_Frac < 0.25) then
	     X := X_Frac * 16.0;  L := X_Exp - 4;  X_Trail := 0.0;
	  else
	     X := X_Frac * 4.0;   L := X_Exp - 2;  
	     X_Trail := 4.0*( X_Frac - X*0.25 );
	  end if;
	  Index := Integer( 2#1.0#E5 * X );
	  C     := Common_Float(Index)*2#1.0#E-5;
	  R     := ((X-C) + X_Trail) / C;

       end if;
	


    -- Step 2. Get sqrt(1 + R) - 1
       
       Q := KF_R1pSm ( R );


    -- Step 3. Reconstruction.

       RtC_Lead  := Rt_Jby32( Index, Lead ); 
       RtC_Trail := Rt_Jby32( Index, Trail );
       RtC := RtC_Lead + RtC_Trail;

       if (Radix = 2) then
	  L := L / 2;
	  Z := RtC_Lead + (RtC * Q  +  RtC_Trail);
	  return( Common_Float'Scaling(Z,L) );
       else
	  L := L/2; L2 := L mod 4; L1 := (L - L2)/4;
	  -- now L = 4L1 + L2; 2^L = 16^L1 * 2^L2
	  Z1 := Two_to(L2) * RtC_Lead;
	  Z2 := Two_to(L2) * (RtC * Q  +  RtC_Trail);
	  Z  := Z1 + Z2;
	  return( Common_Float'Scaling(Z,L1) );
       end if;
       

    end KF_Sqrt;

    function KF_L1pSm( Y : Common_Float ) return Common_Float is

    -- On input, Y lies in the interval ( exp(-1/16)-1, exp(1/16)-1 ).
    -- On output, the value log( 1 + Y ) is returned.
    -- The approximation exploit the identity
    --     log( 1 + Y ) = log( 1 + u/2 )  /  log( 1 - u/2 ), where
    --     u  = 2Y / (2+Y).
    -- Note that the right hand side has an odd Taylor series expansion
    -- which converges much faster than the Taylor series expansion of
    -- log( 1 + Y ) in Y. Thus, we approximate log( 1 + Y ) by
    --     u + A1 * u^3 + A2 * u^5 + ... + An * u^(2n+1).

    -- One subtlety is that since   u    cannot be calculated from
    -- Y exactly, the rounding error in the first   u   should be 
    -- avoided if possible. To accomplish this, we observe that
    --               u  =  Y  -  Y*Y/(2+Y).
    -- Since Y is the input argument, and thus presumed exact, the
    -- formula above approximates u accurately because
    --               u  =  Y  -  correction,
    -- and the magnitude of "correction" (of the order of Y*Y)
    -- is small.
    -- With these observations, we will approximate log( 1 + Y ) by
    --    Y + (  (A1*u^3 + ... + An*u^(2n+1)) - correction ).

    -- All calculations will be carried out in Common_Float.
    -- Since this functions does not need to perform any argument
    -- reduction or reconstruction, it will be faster than the
    -- calculation of  log   in the normal case, despite computations
    -- done solely in Common_Float.

       U, V, Correction, Poly : Common_Float;

    begin

    -- We approximate log(1+Y) by a odd polynomial in U, where
    --          U = 2Y/(2+Y) = Y - Y*Y/(2+Y).
    -- We now calculate U and other related quantities.

       U := Y/(2.0+Y);
       Correction := Y*U;
       U := U + U;
       V := U * U;

    -- We use an appropriate polynomial approximation that will deliver 
    -- a result accurate enough with respect to Float_Type'Base'Digits.
    -- Note that the upper bounds of the cases below (6, 15, 16, 18, 
    -- 27, and 33) are attributes of predefined floating types of
    -- common systems.

       case Float_Type'Base'Digits is

	 when 1..6 =>

	    Poly := U*V*( 8.33333_18167E-02 +
		      V*( 1.25123_45996E-02 ) );


	  when 7..15 =>

	     Poly := U*V*( 8.33333_33333_33179_23934E-02 +
		       V*( 1.25000_00003_77175_09602E-02 +
		       V*( 2.23213_99879_19448_06202E-03 +
		       V*( 4.34887_77770_76145_52256E-04 ))));

	  when 16 =>

	     Poly := U*V*( 8.33333_33333_33333_37909E-02 +
		       V*( 1.24999_99999_99836_60424E-02 +
		       V*( 2.23214_28590_37388_98829E-03 +
		       V*( 4.34026_81922_36633_88536E-04 +
		       V*( 8.89985_05326_15867_46686E-05 )))));

	  when 17..18 =>

	     Poly := U*V*( 8.33333_33333_33333_37909_47954E-02 +
		       V*( 1.24999_99999_99836_60424_94790E-02 +
		       V*( 2.23214_28590_37388_98829_86093E-03 +
		       V*( 4.34026_81922_36633_88536_93268E-04 +
		       V*( 8.89985_05326_15867_46686_13783E-05 )))));

	  when 19..27 =>

	     Poly := 
	   U*V*( 8.33333_33333_33333_33333_33333_32289_687E-02 +
	     V*( 1.25000_00000_00000_00000_00085_22215_240E-02 +
	     V*( 2.23214_28571_42857_14261_92866_48258_201E-03 +
	     V*( 4.34027_77777_77780_98809_33315_23070_295E-04 +
	     V*( 8.87784_09090_66940_14314_76792_89545_412E-05 +
	     V*( 1.87800_48181_19495_76549_39975_36152_064E-05 +
	     V*( 4.06898_41092_88362_34597_46674_51012_081E-06 +
	     V*( 9.01142_71591_15642_85666_93997_67835_754E-07 
		  ))))))));

	  when 28..33 =>

	     Poly := 
	   U*V*( 8.33333_33333_33333_33333_33333_33333_62086_64017E-02 +
	     V*( 1.24999_99999_99999_99999_99999_97094_37287_04079E-02 +
	     V*( 2.23214_28571_42857_14285_72439_65284_28894_60352E-03 +
	     V*( 4.34027_77777_77777_77605_29930_95879_387E-04 +
	     V*( 8.87784_09090_90925_73327_23371_16332_464E-05 +
	     V*( 1.87800_48076_82610_80293_27069_90675_944E-05 +
	     V*( 4.06901_04514_65706_22856_93057_91655_559E-06 +
	     V*( 8.97568_30488_20751_96312_20868_52977_742E-07 +
	     V*( 2.01671_52613_96766_05830_17681_29488_302E-07
		  )))))))));
	  when others =>

	     raise PROGRAM_ERROR;  -- assumption (1) is violated.

       end case;

    -- return the result

       return( Y + ( Poly - Correction ) );

    end KF_L1pSm;

    function KF_Atanh( Y : Common_Float ) return Common_Float is

    -- On input, |Y| <= 2(exp(1/16)-1) / (exp(1/16)+1).
    -- On output, the value of [log(1 + Y/2) - log(1 - Y/2)]/Y  -  1 
    -- is returned.

    -- The core approximation calculates 
    --     Poly = [log(1 + Y/2) - log(1 - Y/2)]/Y  -  1 
    -- in the type Working_Float, which is a type chosen to
    -- have accuracy comparable to the base type of Float_Type.

       Result : Common_Float;

    begin

       
    -- Approximation.


    -- The following is the core approximation. We approximate
    --         [log(1 + Y/2) - log(1 - Y/2)]/Y  -  1 
    -- by a polynomial Poly. The case analysis finds both a suitable 
    -- floating-point type (less expensive to use than LONGEST_FLOAT)
    -- and an appropriate polynomial approximation that will deliver 
    -- a result accurate enough with respect to Float_Type'Base'Digits.
    -- Note that the upper bounds of the cases below (6, 15, 16, 18, 
    -- 27, and 33) are attributes of predefined floating types of
    -- common systems.

       case Float_Type'Base'Digits is

	  when 1..6 =>

	     declare
		type Working_Float is digits Min(6, Max_Digits);
		R, Poly : Working_Float;
	     begin
		R := Working_Float( Y * Y );
		Poly :=  R * 8.33340_08285_51364E-02;
		Result := Common_Float( Poly );
	     end;

	  when 7..15 =>

	     declare
		type Working_Float is digits  Min(15, Max_Digits);
		R, Poly : Working_Float;
	     begin
		R := Working_Float( Y * Y );
		Poly := R*( 8.33333_33333_33335_93622E-02 +
			R*( 1.24999_99997_81386_68903E-02 +
			R*( 2.23219_81075_85598_51206E-03 )));

		Result := Common_Float( Poly );
	     end;

	  when 16 =>

	     declare
		type Working_Float is digits Min(16, Max_Digits);
		R, Poly : Working_Float; 
	     begin
		R :=  Working_Float( Y * Y );
		Poly := R*( 8.33333_33333_33335_93622E-02 +
			R*( 1.24999_99997_81386_68903E-02 +
			R*( 2.23219_81075_85598_51206E-03 )));

		Result := Common_Float( Poly );
	     end;

	  when 17..18 =>

	     declare
		type Working_Float is digits Min(18, Max_Digits);
		R, Poly : Working_Float;
	     begin
		R := Working_Float( Y * Y );
		Poly := R*( 8.33333_33333_33335_93622E-02 +
			R*( 1.24999_99997_81386_68903E-02 +
			R*( 2.23219_81075_85598_51206E-03 )));

		Result := Common_Float( Poly );
	     end;

	  when 19..27 =>

	     declare
		type Working_Float is digits Min(27, Max_Digits);
		R, Poly : Working_Float; 
	     begin
		R := Working_Float( Y * Y );
		Poly := R*( 8.33333_33333_33333_33333_33334_07301_529E-02 +
			R*( 1.24999_99999_99999_99998_61732_74718_869E-02 +
			R*( 2.23214_28571_42866_13712_34336_23012_985E-03 +
			R*( 4.34027_77751_26439_67391_35491_00214_979E-04 +
			R*( 8.87820_39767_24501_02052_39367_49695_054E-05
			     )))));
		Result := Common_Float( Poly );
	     end;

	  when 28..33 =>

	     declare
		type Working_Float is digits Min(33, Max_Digits);
		R, Poly : Working_Float; 
	     begin
		R := Working_Float( Y * Y );
		Poly := 
		 R*( 8.33333_33333_33333_33333_33333_33332_96298_39318E-02 +
		 R*( 1.25000_00000_00000_00000_00000_93488_19499_40702E-02 +
		 R*( 2.23214_28571_42857_14277_26598_59261_40273_30694E-03 +
		 R*( 4.34027_77777_77814_30973_20354_95180_362E-04 +
		 R*( 8.87784_09009_03777_78533_78449_15942_610E-05 +
		 R*( 1.87809_65740_24066_11924_19609_24471_232E-05
		     ))))));
		Result := Common_Float( Poly );
	     end;

	  when others =>

	     raise PROGRAM_ERROR;  -- assumption (1) is violated.

       end case;

    -- This completes the core approximation.

       return( Result );

    end KF_Atanh;

    procedure KP_Log ( Y         : in  Common_Float;
			     M, Z1, Z2 : out Common_Float ) is
     
    -- On input, Y is a floating-point value in Common_Float;
    -- On output, the values M, Z1, and Z2 are returned such that
    --      log(Y) = M*log(2) + Z1 + Z2 , where
    -- log(Y) is the natural log of Y.

    -- Except for arguments Y that are very close to 1, computations
    -- needed in order to obtain M, Z1, and Z2 involve two steps. 

    -- First, we decompose the argument Y to the form
    --          Y  =  2**M  *  (F1  +  F2),
    -- where  1 <= F1+F2 < 2, M has the value of an integer,
    -- F1 = 1 + j/64, j ranges from 0 to 64, and |F2| <= 1/128.

    -- Second, we approximate log( 1 + F2/F1 ) by an odd polynomial
    -- in U, where
    --              U  =  2 F2 / (2 F2 + F1).
    -- Note that 
    --     log( 1 + F2/F1 ) = log( 1 + U/2 ) - log( 1 - U/2 ).
    -- The core approximation calculates
    --     Poly = [log( 1 + U/2 ) - log( 1 - U/2 )]/U   -   1.
    -- Note that  log(1 + U/2) - log(1 - U/2) = 2 arctanh ( U/2 ),
    -- thus,
    --     Poly =  2 arctanh( U/2 ) / U    -     1.
    -- That value is computed by the kernel function   KF_Atanh.

    -- It is not hard to see that
    --    log(Y) = M*log(2) + log(F1) + log( 1 + F2/F1 ).
    -- Hence, we return Z1 := log(F1), and  Z2 := log( 1 + F2/F1).
    -- The values of log(F1) are calculated beforehand and stored in the program.

    -- If Y is close to 1, satisfying |log(Y)| < 1/16, then we use
    -- an approximation similar to the core approximation in Step 2
    -- above to calculate log( 1 + (Y-1) ). For details of the
    -- computation, refer to the code and comments in the 
    -- body of the function KF_L1pSm.

    -- As far as the types used for internal computation are concerned,
    -- we use Common_Float to carry out Step 1,  and use 
    -- Working_Float to carry out Step 2. See the code in the 
    -- body for the function KF_Atanh for details about Working_Float.

       X, F, F1, F2, U,
       K, Half_F2, Poly                     : Common_Float;
       Index, I                             : Integer;
       Thres1                               : constant := 16#0.F07D5#;
       Thres2                               : constant := 16#1.1082C#;

       Log2_Lead : constant Common_Float :=
		   16#0.B17#;
       Log2_Trail : constant Common_Float :=
		   16#0.000217F7D1CF79ABC9E3B39803F2F6AF40#;

    begin

       
    -- I. Filter out exceptional arguments.

       if ( Y <= 0.0 ) then
	  raise Argument_Error;
       end if;

       X := Y;

    -- II. Check if the input argument is very close to 1.0

       if ( X >= Thres1 and X <= Thres2 ) then

    -- Invoke the function KF_L1pSm to compute log(1 + (X-1))
    -- for the value of Z1 and set M and Z2 to zero.
    -- Although the value of X-1 is representable, on machines
    -- without a guard digit for subtraction, X-1 may not be calculated
    -- exactly.  Consequently, we have to take extra precaution.
	  if ( X >= 1.0 ) then
	     X := X - 1.0;
	  else
	     X := X - 0.5; X := X - 0.5;
	  end if;
    -- On machines with guard digit for subtraction, the conditions
    -- above can be replaced by   X := X - 1.0

	  M  := 0.0;
	  Z1 := KF_L1pSm( X );
	  Z2 := 0.0;
	  return;

       end if;

    -- III. General case.

    -- Step 1. Decomposition.
       
    -- Use the primitive function to extract K and F where
    --     X  =  radix**K   *   F,   1/radix <= F < 1.
       F := Common_Float'Fraction( X );
       I := Common_Float'Exponent( X );
       K := Common_Float( I );
       
       if ( Common_Float'Machine_Radix = 16 ) then
	  K := 4.0 * K;
	  while F < 0.5 loop
	     F := F + F;    K := K - 1.0;
	  end loop;
       elsif ( Common_Float'Machine_Radix /= 2 ) then
	  raise Program_Error; -- Assumption (1) is violated.
       end if;
    -- Now  X = 2**K  * F,  1/2 <= F < 1.

       Index := Integer( 2#1.0#E7 * F );
       F1 := Common_Float( Index ) * 2#1.0#E-7;
    -- Again, on machines with gaurd digit for subtraction, the
    -- statements below can be replaced by  Half_F2 := F - F1.
       if ( F >= F1 ) then
	  Half_F2 := F - F1;
       else
	  F2 := F1 * 0.5;
	  Half_F2 := (F - F2) - F2;
       end if;

       F1 := F1 + F1;   F2 := Half_F2 + Half_F2;   K := K - 1.0;
    -- At this point, X = 2**K * ( F1  +  F2 ) where
    -- F1 := j/64, j = 64, 65, ..., 128 and |F2| <= 1/128.

    -- Step 2. Approximation.

    -- Calculate U := 2 F2 / ( 2 F1 + F2 ) = F2 / ( F1 + Half_F2 )
       U := F2 / ( F1 + Half_F2 );

    -- Approximate [log(1+U/2)-log(1-U/2)]/U  -   1.
       Poly := KF_Atanh( U );

       M  := K;
       Z1 := Log_of_Jby64( Index, Lead);
       Z2 := U + ( Log_of_Jby64( Index, Trail ) + U * Poly );
       return;

    end KP_Log;

    function KF_L1p ( Y : Common_Float) return Common_Float is
     
    -- On input, Y is a floating-point value in Common_Float;
    -- On output, the value of log(1 + Y) is returned.

    -- Except for arguments Y that are near zero, log( 1 + Y ) is 
    -- calculated by determining M, Z1, and Z2  such that 
    -- log( 1 + Y ) = M*Log2 + Z1 + Z2.  Then, except for arguments Y
    -- that are very large numbers, computations needed in order to 
    -- retrieve the values of M, Z1, and Z2 involve two steps.

    -- First, we decompose one plus the argument Y to the form
    --          1 + Y =  2**M  *  (F1  +  F2),
    -- where  1 <= F1+F2 < 2, M has the value of an integer,
    -- F1 = 1 + j/64, j ranges from 0 to 64, and |F2| <= 1/128.

    -- Second, we approximate log( 1 + F2/F1 ) by an odd polynomial
    -- in U, where
    --              U  =  2 F2 / (2 F2 + F1).
    -- Note that 
    --     log( 1 + F2/F1 ) = log( 1 + U/2 ) - log( 1 - U/2 ).
    -- The core approximation calculates
    --     Poly = [log( 1 + U/2 ) - log( 1 - U/2 )]/U   -   1.
    -- Note that  log(1 + U/2) - log(1 - U/2) = 2 arctanh ( U/2 ),
    -- thus,
    --     Poly =  2 arctanh( U/2 ) / U    -     1.
    -- That value is computed by the kernel function   KF_Atanh.

    -- It is not hard to see that
    --    log(Y) = M*log(2) + log(F1) + log( 1 + F2/F1 ).
    -- Hence, we return Z1 := log(F1), and  Z2 := log( 1 + F2/F1).
    -- The values of log(F1) are calculated beforehand and 
    -- stored in the program.

    -- If Y is near zero, satisfying |log(Y)| < 1/16 - 1, then we use
    -- an approximation similar to the core approximation in Step 2
    -- above to calculate log( 1 + Y ). For details of the
    -- computation, refer to the code and comments in the 
    -- body of the function KF_L1pSm.

    -- As far as the types used for internal computation are concerned,
    -- we use Common_Float to carry out Step 1,  and use 
    -- Working_Float to carry out Step 2. See the code in the 
    -- body for the function KF_Atanh for details about Working_Float.

       F, F1, F2, U, M, Z, Z1, Z2,
       Half_F2, Poly, Result                : Common_Float;
       Index, I, J                          : Integer;
       Thres1                               : constant := 16#0.F07D5# - 1.0;
       Thres2                               : constant := 16#1.1082C# - 1.0;

       Base_Digits : constant Common_Float :=
		     Common_Float( 3 * Float_Type'Base'Digits );

       Log2_Lead   : constant Common_Float :=
		     16#0.B17#;
       Log2_Trail  : constant Common_Float :=
		     16#0.000217F7D1CF79ABC9E3B39803F2F6AF40#;

    begin

       
    -- I. Filter out exceptional case.

       if ( Y <= -1.0 ) then
	  raise Argument_Error;
       end if;

    -- II. Check if the input argument is near zero.

       if ( Y >= Thres1 and Y <= Thres2 ) then

    -- Invoke the function KF_L1pSm to compute log(1 + Y).

	  return ( KF_L1pSm ( Y ) );

       end if;

    -- III. General case.

       If ( Y >= ( 0.5 * Float_Type'Safe_Last ) ) then

	  KP_Log ( Y, M, Z1, Z2 );

       else

    --    Calculate M, Z1, and Z2 not using KP_Log

    --    Step 1.  Decomposition.

	  Z := 1.0 + Y;

    --    Use the primitive function to extract M and F where
    --        Z  =  radix**M   *   F,   1/radix <= F < 1.
	  F := Common_Float'Fraction ( Z );
	  I := Common_Float'Exponent ( Z );
	  M := Common_Float( I );
	  J := 0;
      
	  if ( Common_Float'Machine_Radix = 16 ) then

	     M := 4.0 * M;
	     while F < 0.5 loop
		J := J - 1;    M := M - 1.0;
		F := F + F;
	     end loop;
	  end if;

    --    Now, regardless if Radix is 2 or 16, we have
    --       Z = 2**M * F = (radix**I) * (2**J) * F,  1/2 <= F < 1.

    --    Extract the leading 7 bits of F.

	  Index := Integer(  F * 2#1.0#E7 );
	  F1 := Common_Float( Index ) * 2#1.0#E-7;

    --    We now calculate  F2  such that
    --          2**M * (F1 + F2)                = 1 + Y  accurately.
    --    i.e.  (radix**I) * (2**J) * (F1 + F2) = 1 + Y  accurately.

	  if ( M <= Base_Digits ) then

    --       F2 = 2**(-J) * ( ( r**(-I) - 2**(J) * F1 ) + r**(-I) * Y )

	     if ( J = 0 ) then
		F2 := Common_Float'Scaling ( 1.0, -I )  -  F1;
		F2 := F2 + Common_Float'Scaling ( Y, -I );
	     else
		F2 := Common_Float'Scaling ( 1.0, -I )  -  Two_to ( J )  *  F1;
		F2 := ( F2 + Common_Float'Scaling ( Y, -I ) ) * Two_to ( -J );
	     end if;

	  else

    --       M is large.
    --       F2 = 2**(-J) * ( ( r**(-I) * Y - (2**J) * F1 ) + r**(-I) )

	     if ( J = 0 ) then
		F2 := Common_Float'Scaling( Y, -I )  -  F1;
		F2 := F2 + Common_Float'Scaling( 1.0, -I );
	     else
		F2 := Common_Float'Scaling ( Y, -I )  -  Two_to ( J )  *  F;
		F2 := ( F2 + Common_Float'Scaling ( 1.0, -I ) ) * Two_to ( -J );
	     end if;

	  end if;

    --    Now   2**M * (F1 + F2) = 1 + Y   accurately,
    --    but   F1  = J/128, j = 64, ... , 128   and
    --         |F2| < 1/256.

	  M       := M - 1.0;
	  F1      := F1 + F1;
	  Half_F2 := F2;
	  F2      := F2 + F2;

    --    Now   2**M * (F1 + F2) = 1 + Y
    --          F1  = j/64,  j = 64, ... , 128   and
    --         |F2| < 1/128.

    --    Step 2. Approximation.

    --    Calculate U := 2 F2 / ( 2 F1 + F2 ) = F2 / ( F1 + Half_F2 )
	  U := F2 / ( F1 + Half_F2 );

    --    Approximate [log(1+U/2)-log(1-U/2)]/U  -   1.
	  Poly := KF_Atanh( U );

	  Z1 := Log_of_Jby64( Index, Lead);
	  Z2 := U + ( Log_of_Jby64( Index, Trail ) + U * Poly );

       end if;

    -- Calculate log( 1 + Y ) from M, Z1, and Z2  such that
    --    log(1 + Y ) = M*Log2 + Z1 + Z2.

       if ( M = 0.0 ) then
	  Result := Z1 + Z2;
       else
	  Result := M * Log2_Trail + Z2;
	  Result := (M * Log2_Lead + Z1 ) + Result;
       end if;

       return (Result);

    end KF_L1p;

    function KF_Em1Sm( Y1, Y2 : Common_Float ) return Common_Float is

    -- On input, Y1 and Y2 are floating point values in Common_Float;
    --   These two variables represent the remainder of the reduced argument
    --   X = N * log2/32 + remainder, where |remainder| <= log2/64.
    -- On output, a Common_Float value is returned which represents the 
    --   approximation of exp(Y1+Y2)-1 for small numbers.

       R1, R2, Q                    : Common_Float;

    begin

       R1 := Y1;
       R2 := Y2;

    -- The following is the core approximation. We approximate
    -- exp(R1+R2)-1 by a polynomial. The case analysis finds both
    -- a suitable floating-point type (less expensive to use than 
    -- Common_Float) and an appropriate polynomial approximation
    -- that will deliver a result accurate enough with respect to
    -- Float_Type'Base'Digits. Note that the upper bounds of the
    -- cases below (6, 15, 16, 18, 27, and 33) are attributes
    -- of predefined floating types of common systems.

       case Float_Type'Base'Digits is

	  when 1..6 =>

	     declare
		type Working_Float is digits Min(6, Max_Digits);
		R, Poly : Working_Float;
	     begin
		R := Working_Float( R1 + R2 );
		Poly := R*R*( 5.00004_0481E-01 + R * 1.66667_6443E-01 );
		Q := R1 + ( R2 + Common_Float( Poly ) );
	     end;

	  when 7..15 =>

	     declare
		type Working_Float is digits Min(15, Max_Digits);
		R, Poly : Working_Float;
	     begin
		R := Working_Float( R1 + R2 );
		Poly := R*R*( 5.00000_00000_00000_08883E-01 + 
			  R*( 1.66666_66666_52608_78863E-01 + 
			  R*( 4.16666_66666_22607_95726E-02 + 
			  R*( 8.33336_79843_42196_16221E-03 + 
			  R*( 1.38889_49086_37771_99667E-03 )))));
		Q := R1 + ( R2 + Common_Float( Poly ) );
	     end;

	  when 16 =>

	     declare
		type Working_Float is digits Min(16, Max_Digits);
		R, Poly : Working_Float; 
	     begin
		R :=  Working_Float( R1 + R2 );
		Poly := R*R*( 5.00000_00000_00000_08883E-01 + 
			  R*( 1.66666_66666_52608_78863E-01 + 
			  R*( 4.16666_66666_22607_95726E-02 + 
			  R*( 8.33336_79843_42196_16221E-03 + 
			  R*( 1.38889_49086_37771_99667E-03 )))));
		Q := R1 + ( R2 + Common_Float( Poly ) );
	     end;

	  when 17..18 =>

	     declare
		type Working_Float is digits Min(18, Max_Digits);
		R, Poly : Working_Float;
	     begin
		R := Working_Float( R1 + R2 );
		Poly := R*R*( 5.00000_00000_00000_07339E-01 + 
			  R*( 1.66666_66666_66666_69177E-01 + 
			  R*( 4.16666_66666_28680_32559E-02 + 
			  R*( 8.33333_33332_52083_91118E-03 + 
			  R*( 1.38889_44766_51246_30293E-03 + 
			  R*( 1.98413_53190_32208_33704E-04 ))))));
		Q := R1 + ( R2 + Common_Float( Poly ) );
	     end;

	  when 19..27 =>

	     declare
		type Working_Float is digits Min(27, Max_Digits);
		R, Poly : Working_Float; 
	     begin
		R := Working_Float( R1 + R2 );
		Poly := R*R*( 4.99999_99999_99999_99999_99636_21075E-01 +
			  R*( 1.66666_66666_66666_66666_66512_04136E-01 +
			  R*( 4.16666_66666_66666_69681_59325_03184E-02 +
			  R*( 8.33333_33333_33333_40906_33326_46233E-03 +
			  R*( 1.38888_88888_81124_92492_26093_01620E-03 +
			  R*( 1.98412_69841_13983_54303_59568_15543E-04 +
			  R*( 2.48016_66086_20855_39725_92760_56125E-05 +
			  R*( 2.75574_13983_51388_82843_29291_74995E-06
			      ))))))));
		Q := R1 + ( R2 + Common_Float( Poly ) );
	     end;

	  when 28..33 =>

	     declare
		type Working_Float is digits Min(33, Max_Digits);
		R, Poly : Working_Float; 
	     begin
		R := Working_Float( R1 + R2 );
		Poly := R*R*( 5.0E-01 +
	      R*( 1.66666_66666_66666_66666_66666_66668_18891E-01 +
	      R*( 4.16666_66666_66666_66666_66666_66671_98062E-02 +
	      R*( 8.33333_33333_33333_33333_33182_72433_96473E-03 +
	      R*( 1.38888_88888_88888_88888_88860_77788_96115E-03 +
	      R*( 1.98412_69841_26984_13216_98830_39302_820E-04 +
	      R*( 2.48015_87301_58730_16549_32617_44006_810E-05 +
	      R*( 2.75573_19223_90497_50521_23337_44713_411E-06 +
	      R*( 2.75573_19223_90383_09381_24531_22474_208E-07 +
	      R*( 2.50521_67036_89710_14700_24557_88635_351E-08 +
	      R*( 2.08768_06002_87469_73970_46716_40247_597E-09
			 )))))))))));
		Q := R1 + ( R2 + Common_Float( Poly ) );
	     end;

	  when others =>

	     raise Program_Error;  -- assumption (1) is violated.

       end case;

    -- This completes the core approximation.

       return( Q );


    end KF_Em1Sm;

    procedure KP_Exp( Y      : in  Common_Float;
			    M      : out Integer;
			    Z1, Z2 : out Common_Float) is

    -- On input, Y is a floating-point value in Common_Float;
    -- On output, the values M, Z1, and Z2 are returned such that
    --      exp(Y) = 2**M * (Z1 + Z2),  where
    -- exp(Y) is the natural exponential of Y.

    -- Computations needed in order to obtain M, Z1, and Z2 involve three steps.

    -- First, we reduce the argument Y to the form
    --          Y = N * log2/32 + remainder,
    -- where N has the value of an integer and |remainder| <= log2/64.
    -- The value of N = Y * 32/log2 rounded to the nearest integer and
    -- the remainder = Y - N*log2/32.

    -- Second, we approximate exp(Y) - 1 by using KF_Em1Sm (Y1, Y2)
    -- where Y1 is the leading part of the remainder and Y2 is the trailing part
    -- of the remainder.  The approximation is carried out by using a polynomial
    -- and Working_Float.  See the code in KF_Em1Sm for more details.

    -- Third, we reconstruct the exponential of Y so that
    --   exp(Y) = 2**M * (Z1 + Z2).

       F, X, R1, R2, Q, Tmp        : Common_Float;

       Two_to_6            : constant := 64;
       Two_to_8            : constant := 256;

       Log2_by_32          : constant := 
	  16#5.8B90B_FBE8E_7BCD5_E4F1D_9CC01_F97B5_7A079_A1933_94C#E-2; 

       Thirty_Two_by_Log2  : constant :=
	  16#2E.2A8EC_A5705_FC2EE_FA1FF_B41A4_74FA2_3AD5E#;

       I, N, N_1, N_2, J              : Integer;

    begin

       X := Y;

    -- Step 1. Reduce the argument.

    -- To perform argument reduction, we find the integer N such that
    --   X = N * log2/32 + remainder, |remainder| <= log2/64.
    -- N is defined by round-to-nearest-integer( X*32/log2 ) and
    -- remainder by X - N*log2/32. The calculation of N is 
    -- straightforward whereas the computation of X - N*log2/32
    -- must be carried out carefully.
    -- log2/32 is so represented in two pieces that
    -- (1) log2/32 is known to extra precision, (2) the product
    -- of N and the leading piece is a model number and is hence 
    -- calculated without error, and (3) the subtraction of the value
    -- obtained in (2) from X is a model number and is hence again obtained
    -- without error.

    -- Perform argument reduction in Common_Float.

       N := Integer( X * Thirty_Two_by_Log2 );
       if abs(N) >= Two_to_8 then
	  N_2 := N mod Two_to_6;
	  N_1 := N - N_2;
       else
	  N_2 := 0;
	  N_1 := N;
       end if;
       Tmp := Common_Float( N_1 ) * Log2_by_32_Lead;
       if abs( X ) >= abs( Tmp ) then
	  R1 := X - Tmp;
       else
	  Tmp := 0.5 * Tmp;
	  R1 := (X - Tmp) - Tmp;
       end if;
       if N_2 /= 0 then
	  R1 := R1 - Common_Float(N_2) * Log2_by_32_Lead;
       end if;
       R2 := -Common_Float(N) * Log2_by_32_Trail;
       J := N mod 32;
       M := (N - J)/32;
      
    -- Step 2. Approximation.

       Q := KF_Em1Sm( R1, R2 );

    -- Step 3. Function value reconstruction.

    -- We now reconstruct the exponential of the input argument
    -- so that Exp(Y) = 2**M * (Z1 + Z2).
    -- The order of the computation below must be strictly observed.

       F  := Two_to_Jby32( J, Lead ) + Two_to_Jby32( J, Trail );

       Z1 := Two_to_Jby32( J, Lead );
       Z2 := Two_to_Jby32( J, Trail ) + F*Q;

       return;


    end KP_Exp;

    function KF_Em1( Y : Common_Float ) return Common_Float is

    -- On input, Y is a floating point value in Common_Float;
    -- On output, a Common_Float value is returned that represents
    --   exp(Y) - 1  for general cases.

       X, X2, Z1, Z2, Z1_Prime : Common_Float;
       Z2_Prime, Cond, Result  : Common_Float;
       M, L, L_Rem, D          : Integer;

       Log2_by_64: constant :=
			     16#0.02C5C85FDF473DE6AF278ECE600FCBDAB#;

    begin

       X    := Y;
       D    := Float_Type'Base'Digits;
       Cond := -6.0 * Common_Float(D) * Log2;

       if ( X < Cond ) then
	  return ( -1.0 );
       end if;


       if (abs(X) < Log2_by_64) then
	  X2 := 0.0;
	  return KF_Em1Sm ( X, X2 );
       end if;

    -- Get M, Z1, and Z2 so that  Exp(X) = 2**M * (Z1 + Z2)

       KP_Exp (X, M, Z1, Z2);

    -- Calculate L, Z1_Prime, and Z2_Prime such that
    --     2**M * (Z1 + Z2) = Radix**L * (Z1_Prime + Z2_Prime)

       case Common_Float'Machine_Radix is

	  when 2  =>
	     L        := M;
	     Z1_Prime := Z1;
	     Z2_prime := Z2;

	  when others =>     -- Radix is 16 
	     L_Rem    := M rem 4;
	     L        := (M - L_Rem) / 4;
	     Z1_Prime := Z1 * Two_To(L_Rem);
	     Z2_Prime := Z2 * Two_To(L_Rem);

       end case;


    -- Now Exp(X) - 1 = Radix**L * (Z1_Prime + Z2_Prime) - 1

    -- The following is the computation of Exp(X) - 1

       if ( M = 0 ) then
	  Result := ( Z1_Prime - 1.0 ) + Z2_Prime;

       elsif ( abs(M) <= D ) then
	  Result := Common_Float'Scaling ( Z2_Prime, L );
	  Result := ( Common_Float'Scaling ( Z1_Prime, L ) - 1.0) + Result;

       elsif ( M < -D  ) then
	  Result := Z1_Prime + Z2_Prime;
	  Result := Common_Float'Scaling ( Result, L ) - 1.0;

       else
	  Result := Z2_Prime - Common_Float'Scaling( 1.0, -L );
	  Result := Z1_Prime + Result;
	  Result := Common_Float'Scaling ( Result, L );
       end if;


       return( Result );


    end KF_Em1;

    procedure KP_LogPw ( X      : in  Common_Float;
			 Z1, Z2 : out Common_Float ) is
     
    -- On input, X is a floating-point value in Common_Float;
    -- On output, the values Z1, and Z2 are returned such that
    --      log(X) =  Z1 + Z2 , where
    -- log(X) is the natural log of X. Moreover, Z1 only has a few
    -- digits followed by trailing zeros.

    -- Except for arguments Y that are very close to 1, computations
    -- needed in order to obtain Z1, and Z2 involve two steps. 

    -- First, we decompose the argument X to the form
    --          X  =  2**M  *  (F1  +  F2),
    -- where  1 <= F1+F2 < 2, M has the value of an integer,
    -- F1 = 1 + j/64, j ranges from 0 to 64, and |F2| <= 1/128.

    -- Second, we approximate log( 1 + F2/F1 ) by an odd polynomial
    -- in U, where
    --              U  =  2 F2 / (2 F2 + F1).
    -- Note that 
    --     log( 1 + F2/F1 ) = log( 1 + U/2 ) - log( 1 - U/2 ).
    -- The core approximation calculates
    --     Poly = [log( 1 + U/2 ) - log( 1 - U/2 )]/U   -   1.
    --
    -- It is not hard to see that
    --    log(X) = M*log(2) + log(F1) + log( 1 + F2/F1 ).
    -- Hence, we return Z1 + Z2 as an accurate approximation to the
    -- sum above. The idea is that Z1 is the first few digits of
    -- the sum. Z2 is the correction to the sum accurate to full machine
    -- precision.
    --
    -- 

    -- If X is close to 1, satisfying |log(X)| < 1/64, then we use
    -- an approximation similar to the core approximation in Step 2
    -- above to calculate log( 1 + (X-1) ). 

    -- As far as the types used for internal computation are concerned,
    -- we use Common_Float to carry out Step 1,  and use 
    -- Working_Float to carry out Step 2. 

       Y, F, F1, F2, G, U, U1, U2, V, W1, W2,
       Half_F2, Half_F2_H, Half_F2_L, Poly,
       A1, A2, T1, T2                       : Common_Float;
       Index, I, K                          : Integer;
       L, LL                                : Positive;
       Thres1                               : constant := 16#0.FC07F560#;
       Thres2                               : constant := 16#1.04080AB5#;

    begin

       L  := (2*Common_Float'Machine_Mantissa) / 5; 
       LL := Common_Float'Machine_Mantissa;

    -- I. Filter out exceptional arguments.

       if ( X <= 0.0 ) then
	  raise Argument_Error;
       end if;


    -- II. Check if the input argument is very close to 1.0

       if ( X >= Thres1 and X <= Thres2 ) then

	  if ( X >= 1.0 ) then
	     F := X - 1.0;
	  else
	     F := X - 0.5; F := F - 0.5;
	  end if;
    -- On machines with guard digit for subtraction, the conditions
    -- above can be replaced by   X := X - 1.0

	  
	  F1 := Common_Float'Leading_Part( F, L );
	  F2 := F - F1;
	  G  := 1.0 / (2.0 + F);
	  U  := 2.0*F*G;
	  V  := U*U;
	  U1 := Common_Float'Leading_Part( U, L );
	  U2 := G*( ( 2.0*(F-U1) - U1*F1 )  -  U1*F2 );

    -- We now compute a polynomial approximation to log(1 + F2/F1)
	  case Float_Type'Base'Digits is
	  when 1..6 =>
	     declare
		type Working_Float is digits Min(6, Max_Digits);
		Q, R, S : Working_Float;
	     begin
		R := Working_Float(U); S := Working_Float(V);
		Q :=  S * 8.33340_08285_51364E-02;
		Z2 := U2 + Common_Float(R*Q);
	     end;

	  when 7..15 =>
	     declare
		type Working_Float is digits Min(15, Max_Digits);
		Q, R, S : Working_Float;
	     begin
		R := Working_Float(U); S := Working_Float(V);
		Q := S*( 8.33333_33333_33335_93622E-02 +
		     S*( 1.24999_99997_81386_68903E-02 +
		     S*( 2.23219_81075_85598_51206E-03 )));

		Z2 := U2 + Common_Float(R*Q);
	     end;

	  when 16 =>
	     declare
		type Working_Float is digits Min(16, Max_Digits);
		Q, R, S : Working_Float; 
	     begin
		R := Working_Float(U); S := Working_Float(V); 
		Q := S*( 8.33333_33333_33335_93622E-02 +
		     S*( 1.24999_99997_81386_68903E-02 +
		     S*( 2.23219_81075_85598_51206E-03 )));

		Z2 := U2 + Common_Float(R*Q);
	     end;

	  when 17..18 =>
	     declare
		type Working_Float is digits Min(18, Max_Digits);
		Q, R, S : Working_Float;
	     begin
		R := Working_Float(U); S := Working_Float(V);
		Q := S*( 8.33333_33333_33335_93622E-02 +
		     S*( 1.24999_99997_81386_68903E-02 +
		     S*( 2.23219_81075_85598_51206E-03 )));

		Z2 := U2 + Common_Float(R*Q);
	     end;

	  when 19..27 =>
	     declare
		type Working_Float is digits Min(27, Max_Digits);
		Q, R, S : Working_Float; 
	     begin
		R := Working_Float(U); S := Working_Float(V);
		Q := S*( 8.33333_33333_33333_33333_33334_07301_529E-02 +
		     S*( 1.24999_99999_99999_99998_61732_74718_869E-02 +
		     S*( 2.23214_28571_42866_13712_34336_23012_985E-03 +
		     S*( 4.34027_77751_26439_67391_35491_00214_979E-04 +
		     S*( 8.87820_39767_24501_02052_39367_49695_054E-05
			     )))));
		Z2 := U2 + Common_Float(R*Q);
	     end;

	  when 28..33 =>
	     declare
		type Working_Float is digits Min(33, Max_Digits);
		Q, R, S : Working_Float; 
	     begin
		R := Working_Float(U); S := Working_Float(V);
		Q := 
		 S*( 8.33333_33333_33333_33333_33333_33332_96298_39318E-02 +
		 S*( 1.25000_00000_00000_00000_00000_93488_19499_40702E-02 +
		 S*( 2.23214_28571_42857_14277_26598_59261_40273_30694E-03 +
		 S*( 4.34027_77777_77814_30973_20354_95180_362E-04 +
		 S*( 8.87784_09009_03777_78533_78449_15942_610E-05 +
		 S*( 1.87809_65740_24066_11924_19609_24471_232E-05
		     ))))));
		Z2 := U2 + Common_Float(R*Q);
	     end;

	  when others =>
	     raise PROGRAM_ERROR;  -- assumption (1) is violated.

	  end case;

	  Z1 := U1;
	  Return;

       end if;
    -- End of the case when X is close to 1

    -- III. General case.

    -- Step 1. Decomposition.
       
    -- Use the primitive function to extract K and F where
    --     X  =  radix**K   *   F,   1/radix <= F < 1.
       F := Common_Float'Fraction( X );
       K := Common_Float'Exponent( X );
       
       if ( Radix = 16 ) then
	  K := 4 * K;
	  while F < 0.5 loop
	     F := F + F;    K := K - 1;
	  end loop;
       end if;
    -- Now  X = 2**K  * F,  1/2 <= F < 1.

       Index := Integer( 2#1.0#E7 * F );
       F1 := Common_Float( Index ) * 2#1.0#E-7;
    -- Again, on machines with gaurd digit for subtraction, the
    -- statements below can be replaced by  Half_F2 := F - F1.
       if ( F >= F1 ) then
	  Half_F2 := F - F1;
       else
	  F2 := F1 * 0.5;
	  Half_F2 := (F - F2) - F2;
       end if;

       F1 := F1 + F1;   F2 := Half_F2 + Half_F2;   K := K - 1;
    -- At this point, X = 2**K * ( F1  +  F2 ) where
    -- F1 := j/64, j = 64, 65, ..., 128 and |F2| <= 1/128.

    -- Step 2. Approximation.

    -- Calculate U := 2 F2 / ( 2 F1 + F2 ) = F2 / ( F1 + Half_F2 )
       G  := 1.0 / (F1 + Half_F2);
       U  := F2*G;
       V  := U*U;
       U1 := Common_Float'Leading_Part( U, L );
       Half_F2_H := Common_Float'Leading_Part( Half_F2, L );
       Half_F2_L := Half_F2 - Half_F2_H;
       U2 := G*( ( (F2 - U1*F1) - U1*Half_F2_H ) - U1*Half_F2_L );

    -- Approximate [log(1+U/2)-log(1-U/2)]/U  -   1.
       case Float_Type'Base'Digits is
	  when 1..6 =>
	     declare
		type Working_Float is digits Min(6, Max_Digits);
		Q, R, S : Working_Float;
	     begin
		R := Working_Float(U); S := Working_Float(V);
		Q :=  S * 8.33340_08285_51364E-02;
		W2 := U2 + Common_Float(R*Q);
	     end;

	  when 7..15 =>
	     declare
		type Working_Float is digits Min(15, Max_Digits);
		Q, R, S : Working_Float;
	     begin
		R := Working_Float(U); S := Working_Float(V);
		Q := S*( 8.33333_33333_33335_93622E-02 +
		     S*( 1.24999_99997_81386_68903E-02 +
		     S*( 2.23219_81075_85598_51206E-03 )));

		W2 := U2 + Common_Float(R*Q);
	     end;

	  when 16 =>
	     declare
		type Working_Float is digits Min(16, Max_Digits);
		Q, R, S : Working_Float; 
	     begin
		R := Working_Float(U); S := Working_Float(V);
		Q := S*( 8.33333_33333_33335_93622E-02 +
		     S*( 1.24999_99997_81386_68903E-02 +
		     S*( 2.23219_81075_85598_51206E-03 )));

		W2 := U2 + Common_Float(R*Q);
	     end;

	  when 17..18 =>
	     declare
		type Working_Float is digits Min(18, Max_Digits);
		Q, R, S : Working_Float;
	     begin
		R := Working_Float(U); S := Working_Float(V);
		Q := S*( 8.33333_33333_33335_93622E-02 +
		     S*( 1.24999_99997_81386_68903E-02 +
		     S*( 2.23219_81075_85598_51206E-03 )));

		W2 := U2 + Common_Float(R*Q);
	     end;

	  when 19..27 =>
	     declare
		type Working_Float is digits Min(27, Max_Digits);
		Q, R, S : Working_Float; 
	     begin
		R := Working_Float(U); S := Working_Float(V);
		Q := S*( 8.33333_33333_33333_33333_33334_07301_529E-02 +
		     S*( 1.24999_99999_99999_99998_61732_74718_869E-02 +
		     S*( 2.23214_28571_42866_13712_34336_23012_985E-03 +
		     S*( 4.34027_77751_26439_67391_35491_00214_979E-04 +
		     S*( 8.87820_39767_24501_02052_39367_49695_054E-05
			     )))));
		W2 := U2 + Common_Float(R*Q);
	     end;

	  when 28..33 =>
	     declare
		type Working_Float is digits Min(33, Max_Digits);
		Q, R, S : Working_Float; 
	     begin
		R := Working_Float(U); S := Working_Float(V);
		Q := 
		 S*( 8.33333_33333_33333_33333_33333_33332_96298_39318E-02 +
		 S*( 1.25000_00000_00000_00000_00000_93488_19499_40702E-02 +
		 S*( 2.23214_28571_42857_14277_26598_59261_40273_30694E-03 +
		 S*( 4.34027_77777_77814_30973_20354_95180_362E-04 +
		 S*( 8.87784_09009_03777_78533_78449_15942_610E-05 +
		 S*( 1.87809_65740_24066_11924_19609_24471_232E-05
		     ))))));
		W2 := U2 + Common_Float(R*Q);
	     end;

	  when others =>
	     raise PROGRAM_ERROR;  -- assumption (1) is violated.

       end case;

       A2 := Common_Float(K);
       A1 := A2*Pow_Log2_Lead  + Pow_Log_of_Jby64(Index, Lead);
       A2 := A2*Pow_Log2_Trail + Pow_Log_of_Jby64(Index, Trail);
       T1 := Common_Float'Leading_Part( A1 + U1, LL-2 );	-- force storage
       T2 := (A1 - T1) + U1;
       T2 := T2 + A2;
       W1 := Common_Float'Leading_Part( T1 + W2, L );
       Z2 := ((T1 - W1) + W2) + T2;
       Z1 := W1;

       return;

    end KP_LogPw;

 -----------------------------------------------------------------------------
 --     Kernel functions and procedures for the trigonometric family.       --
 -----------------------------------------------------------------------------

    procedure KP_Pi2Rd ( Y      : in  Common_Float;
			 R1, R2 : out Common_Float;
			 I_Flag : out Quadrant_Flag ) is

    -- On input, Y is a Common_Float number such that
    --       |Y| <= 2**( Common_Float'Mantissa / 2 ).
    -- On return, the sum R1 and R2 represents the reduced
    -- argument (with respect to pi/2) to at least 3/2 of the precision
    -- offered by Common_Float. Precisely
    -- 
    --  | R2 | <= Common_Float'Model_Epsilon * | R1 |,
    --  
    -- there is an integer value k such that 
    --
    --  Y - k( pi/2 ) = (1 + delta) * ( R1 + R2 ),
    --  | delta | <= (Common_Float'Model_Epsilon) ** (3/2),
    --
    -- and
    --
    --  | R1 + R2 | <= (pi/4) * ( 1 + sqrt(Common_Float'Model_Epsilon) ).
    --
    --
    -- There are two major steps in the algorithm.
    --
    -- First, the value k is obtained by
    --      k := round-to-nearest-integer( Y * Two_by_Pi ),
    -- where Two_by_Pi is the working-precise value of (2/pi).
    -- Because the magnitude of Y is never too large, the value k
    -- obtained is very accurate. Moreover, note also that the value
    -- k need not be identical to the integer closest to to
    -- Y*(2/pi), provided that 
    --                    Y - k * (pi/2) 
    -- can be computed accurately and that value is small in magnitude
    -- ( <= pi/4 roughly ).
    --
    -- Second, the difference Y - k*(pi/2) is calculated. In this
    -- program, (pi/2) is represented by 
    --   pi/2 = Piby2_Piece(1) + Piby2_Piece(2) + ... +
    --
    --              Piby2_Piece( N_Pieces_of_Piby2 )
    --
    -- in a way such that  k * Piby2_Piece(J)  is exact for all 
    -- possible J's. Thus, to compute the reduced argument accurately,
    -- we need only subtract correctly:
    --
    --     Y - k*Piby2_Piece(1) - k*Piby2_Piece(2) - ...
    --             ... - k*Piby2_Piece( N_Pieces_of_Piby2 ).
    --
    -- Three steps are needed to perform the subtraction.
    --
    -- Step 1. Let X := Y. For J = 1, 2, ....,  perform
    --            X := X - k * Piby2_Piece( J ) 
    -- as long as the subtraction is exact. That is the case if
    --    (k*Piby2_Piece(J))/2 <= X <= 2*(k*Piby2_Piece(J)).
    --
    -- Step 2. Suppose Step 1 terminates at J = N. Then,
    --    X = Y - k*(Piby2_Piece(1) + ... + Piby2_Piece(N))
    -- exactly. It can be shown that
    --             X - k*Piby2_Piece(N+1)
    -- can be represented exactly as
    --       X - k*Piby2_Piece = X_Lead  +  X_Trail,
    -- where
    --     | X_Trail | <= Common_Float'Epislon * | X_Lead |.
    --
    -- Step 3. At this stage, X_Lead is the reduced argument
    -- to the precision of Common_Float. We must now calculate
    -- correction to X_Lead+X_Trail so that the reduced argument
    -- can be (3/2) the accuracy of Common_Float. Therefore, we
    -- modify X_Trail by
    --    X_Trail := X_Trail - k*( Piby2_Piece(N+2) + ...
    --                              + Piby2_Piece( good enough ) ).
    --

       X, X_Lead, X_Trail, X_Extra,
       Temp, F, K, K_mod16, P, Two_P, Half_P  : Common_Float;
       J, L1, L2                              : Integer;
       Same_Sign                              : Boolean;
       Tmp_R1                                 : Common_Float;
       Two_by_Pi : constant :=
	  16#A.2F983_6E4E4_41529_FC275_7D1F5_34DDC_0DB63#E-1;

    begin

    -- Initialization
       if Y < -Large_Trig_Threshold or else Y > Large_Trig_Threshold then
	   -- Use "rem" to do initial reduction
	   X := Y rem Two_Pi;
       else
	   X := Y;
       end if;

    -- Calculation of integer quotient K

       K := Common_Float'Rounding ( X * Two_by_Pi );

       if K = 0.0 then
	  R1 := X;
	  R2 := 0.0;
	  I_Flag := 0;
	  return;
       end if;

       K_mod16 := K - 16.0*Common_Float'Truncation(K * (1.0/16.0));
       I_Flag  := Quadrant_Flag( Integer(K_mod16) mod 4 );
     
    -- Step 1. Calculate X := X - k*Piby2_Piece(J) whenever exact 
       -- Initialization of Step 1
       J := 1;
       P := K * Piby2_Piece ( J );
       TWO_P := P + P;
       HALF_P := P * 2#1.0#E-1;

       -- The loop
       while X >= HALF_P and then X <= TWO_P and then 
	 J <= N_PIECES_OF_PIBY2-2 loop

	  if P > X then
	     X := (X - HALF_P) - HALF_P;
	  else
	     X := X - P;
	  end if;
	  J := J + 1;
	  P := K * Piby2_Piece( J );
	  TWO_P := P + P;
	  HALF_P := P * 2#1.0#E-1;

       end loop;
    -- End of Step 1

    -- Step 2. Represent X - k*Piby2_Piece( J ) in two pieces 
       P := -K * Piby2_Piece( J );
       X_Lead := X + P;
       if X < 0.0  then
	  Same_Sign := TRUE;
       else
	  Same_Sign := FALSE;
       end if;
       if abs(X) < abs(P) then
	  TEMP := X;
	  X := P;
	  P := TEMP;
       end if;

    -- Ideally, if the arithmetic of the underlying machine is good,
    -- X_Trail := (X - X_Lead) + P suffices for our purpose because
    -- X - X_Lead must be exact. Unforturnately, there are machines
    -- without a guard bit for subtraction. The trick below ensures
    -- that  (X-F), (X_Lead-F), and (X-F) - (X_Lead-F) are all exact.
    -- For a detailed explanation of why this trick work, refer to
    -- pages 10-3 to 10-9 of the notes IMPLEMENTATION OF ALGORITHMS
    -- by W. Kahan. This set of notes can be obtained through NTIS
    -- in Virginia. Order number AD769 124 / LP, 1986.
       F := 0.0;
       if Same_Sign then
	  F := (0.46 * X_Lead - X_Lead) + X_Lead;
       end if;
       X_Trail := ( (X - F) - (X_Lead - F) ) + P;

    -- Step 3. Calcuate to higher precision.
       J := J + 1;
       L1 := 2 * 4 * Common_Float'Base'Digits - N_Bits_per_Piece;
       L2 := L1 / N_Bits_per_Piece + 1;
       X_Extra := 0.0;
       if (J+L2 > N_Pieces_of_Piby2) then
	  L2 := N_Pieces_of_Piby2 - J;
       end if;
       for I in reverse J..J+L2 loop
	  X_Extra := X_Extra + Piby2_Piece( I );
       end loop;
       X_Trail := X_Trail - K*X_Extra;

    -- Finally, recast the answer in Common_Float.

       Tmp_R1 := X_Lead;
       R2     := ((X_Lead-F) - (Tmp_R1-F)) +  X_Trail ;
       R1     := Tmp_R1;

       return;

    end  KP_Pi2Rd;

    function KF_Sin( Y1, Y2 : Common_Float ) return Common_Float is

    -- On input, Y1 and Y2 are floating point values in Common_Float;
    --   These two variables represent the remainder of the reduced argument
    --   X = N * (pi/2) + remainder, where |remainder| <= pi/4.
    -- On output, a Common_Float value is returned which represents the 
    --   approximation of sin( Y1+Y2 ).

       R1, R2, Q                    : Common_Float;

    begin

       R1 := Y1;
       R2 := Y2;

    -- The following is the core approximation. We approximate
    -- sin(Y1+Y2) by an odd polynomial. The case analysis finds both
    -- a suitable floating-point type (less expensive to use than 
    -- Common_Float) and an appropriate polynomial approximation
    -- that will deliver a result accurate enough with respect to
    -- Float_Type'Base'Digits. Note that the upper bounds of the
    -- cases below (6, 15, 16, 18, 27, and 33) are attributes
    -- of predefined floating types of common systems.

       case Float_Type'Base'Digits is

	  when 1..6 =>

	     declare
		type Working_Float is digits Min(6, Max_Digits);
		R, S, Poly : Working_Float;
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := S*( 0.16666_66269E-0 -
			S*( 0.83329_37955E-2 -
			S*( 0.19729_49430E-3 -
			S*( 0.17898_67484E-5 ))));
		Q := R1 + ( R2 - (R1+R2)*Common_Float( Poly ) );
	     end;

	  when 7..15 =>

	     declare
		type Working_Float is digits Min(15, Max_Digits);
		R, S, Poly : Working_Float;
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := S*( 0.16666_66666_66666_62965E-00 -
			S*( 0.83333_33333_33216_57487E-02 -
			S*( 0.19841_26984_00425_50051E-03 -
			S*( 0.27557_31863_01054_79460E-05 -
			S*( 0.25051_96268_35442_39751E-07 -
			S*( 0.16041_33183_44428_30041E-09 -
			S*( 0.67810_12820_36054_53944E-12 ))))))); 
		Q := R1 + ( R2 - (R1+R2)*Common_Float( Poly ) );
	     end;

	  when 16 =>

	     declare
		type Working_Float is digits Min(16, Max_Digits);
		R, S, Poly : Working_Float; 
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := S*( 0.16666_66666_66666_62965E-00 -
			S*( 0.83333_33333_33216_57487E-02 -
			S*( 0.19841_26984_00425_50051E-03 -
			S*( 0.27557_31863_01054_79460E-05 -
			S*( 0.25051_96268_35442_39751E-07 -
			S*( 0.16041_33183_44428_30041E-09 -
			S*( 0.67810_12820_36054_53944E-12 ))))))); 
		Q := R1 + ( R2 - (R1+R2)*Common_Float( Poly ) );
	     end;

	  when 17..18 =>

	     declare
		type Working_Float is digits Min(18, Max_Digits);
		R, S, Poly : Working_Float;
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := S*( 0.16666_66666_66666_66663_05265E-00 -
			S*( 0.83333_33333_33333_18353_27331E-02 -
			S*( 0.19841_26984_12677_42397_98061E-03 -
			S*( 0.27557_31922_25915_00034_88464E-05 -
			S*( 0.25052_10788_96438_73197_48920E-07 -
			S*( 0.16058_94677_96247_63281_41150E-09 -
			S*( 0.76372_69506_69707_85652_27420E-12 - 
			S*( 0.23950_00674_30593_53449_39104E-14 ))))))));
		Q := R1 + ( R2 - (R1+R2)*Common_Float( Poly ) );

	     end;

	  when 19..27 =>

	     declare
		type Working_Float is digits Min(27, Max_Digits);
		R, S, Poly : Working_Float; 
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := S*( 0.16666_66666_66666_66666_66666_66633_0086E+00 -
		   S*( 0.83333_33333_33333_33333_33330_76943_0210E-02 -
		   S*( 0.19841_26984_12698_41269_83450_93722_6766E-03 -
		   S*( 0.27557_31922_39858_90643_73301_78259_2770E-05 -
		   S*( 0.25052_10838_54417_12158_37226_81690_8857E-07 -
		   S*( 0.16059_04383_68185_46636_41083_91388_2129E-09 -
		   S*( 0.76471_63730_91063_93957_96779_08149_5627E-12 - 
		   S*( 0.28114_57081_79321_12323_34834_60010_1450E-14 -
		   S*( 0.82204_32293_08044_34287_39069_99726_9855E-17 - 
		   S*( 0.19438_17832_99374_63087_47164_67848_4712E-19 
		       ))))))))));
		Q := R1 + ( R2 - (R1+R2)*Common_Float( Poly ) );
	     end;

	  when 28..33 =>

	     declare
		type Working_Float is digits Min(33, Max_Digits);
		R, S, Poly : Working_Float; 
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := S*( 0.16666_66666_66666_66666_66666_66666_6666E+00 -
		    S*( 0.83333_33333_33333_33333_33333_33333_0274E-02 -
		    S*( 0.19841_26984_12698_41269_84126_98311_2600E-03 -
		    S*( 0.27557_31922_39858_90652_55730_24207_5442E-05 -
		    S*( 0.25052_10838_54417_18775_03589_32984_9329E-07 -
		    S*( 0.16059_04383_68216_14589_51327_14046_5961E-09 -
		    S*( 0.76471_63731_81981_25083_98331_27914_0063E-12 - 
		    S*( 0.28114_57254_34445_07256_92700_63990_6368E-14 -
		    S*( 0.82206_35244_69072_20561_89193_97777_7182E-17 - 
		    S*( 0.19572_93878_24940_42631_49760_79163_7901E-19 - 
		    S*( 0.38680_05154_37551_58372_98445_80258_9098E-22 -
		    S*( 0.63830_99586_87078_27791_97302_21097_2554E-25 
			    ))))))))))));
		Q := R1 + ( R2 - (R1+R2)*Common_Float( Poly ) );
	     end;

	  when others =>

	     raise Program_Error;  -- assumption (1) is violated.

       end case;

    -- This completes the core approximation.

       return( Q );


    end KF_Sin;

    function KF_Cos( Y1, Y2 : Common_Float ) return Common_Float is

    -- On input, Y1 and Y2 are floating point values in Common_Float;
    --   These two variables represent the remainder of the reduced argument
    --   X = N * (pi/2) + remainder, where |remainder| <= pi/4.
    -- On output, a Common_Float value is returned which represents the 
    --   approximation of cos( Y1+Y2 ).

       R1, R2, Rsq, Q                  : Common_Float;

    begin

       R1  := Y1;
       R2  := Y2;
       Rsq := R1 + R2;
       Rsq := Rsq * Rsq;

    -- The following is the core approximation. We approximate
    -- cos(Y1+Y2) by an even polynomial. The case analysis finds both
    -- a suitable floating-point type (less expensive to use than 
    -- Common_Float) and an appropriate polynomial approximation
    -- that will deliver a result accurate enough with respect to
    -- Float_Type'Base'Digits. Note that the upper bounds of the
    -- cases below (6, 15, 16, 18, 27, and 33) are attributes
    -- of predefined floating types of common systems.

       case Float_Type'Base'Digits is

	  when 1..6 =>

	     declare
		type Working_Float is digits Min(6, Max_Digits);
		R, S, Poly : Working_Float;
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := 0.5 -
			S*( 0.41666_65673E-01 - 
			S*( 0.13887_77331E-02 -
			S*( 0.24478_79547E-04 )));
		Q := 1.0 - Rsq*Common_Float( Poly );
	     end;

	  when 7..15 =>

	     declare
		type Working_Float is digits Min(15, Max_Digits);
		R, S, Poly : Working_Float;
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := 0.5 -
		   S*( 0.41666_66666_66666_01903E-01 -
		   S*( 0.13888_88888_88744_48186E-02 -
		   S*( 0.24801_58728_96438_74032E-04 -
		   S*( 0.27557_31439_04201_86748E-06 -
		   S*( 0.20875_72754_71895_52619E-08 -
		   S*( 0.11359_83163_10688_30184E-10 ))))));
		Q := 1.0 - Rsq*Common_Float( Poly );
	     end;

	  when 16 =>

	     declare
		type Working_Float is digits Min(16, Max_Digits);
		R, S, Poly : Working_Float; 
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := 0.5 -
		   S*( 0.41666_66666_66666_01903E-01 -
		   S*( 0.13888_88888_88744_48186E-02 -
		   S*( 0.24801_58728_96438_74032E-04 -
		   S*( 0.27557_31439_04201_86748E-06 -
		   S*( 0.20875_72754_71895_52619E-08 -
		   S*( 0.11359_83163_10688_30184E-10 ))))));
		Q := 1.0 - Rsq*Common_Float( Poly );
	     end;

	  when 17..18 =>

	     declare
		type Working_Float is digits Min(18, Max_Digits);
		R, S, Poly : Working_Float;
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := 0.5 -
			S*( 0.41666_66666_66666_66603_42153E-01 - 
			S*( 0.13888_88888_88888_71536_63177E-02 -
			S*( 0.24801_58730_15691_85047_02943E-04 -
			S*( 0.27557_31921_43827_45008_49677E-06 -
			S*( 0.20876_75414_25307_03561_14757E-08 -
			S*( 0.11470_26790_89435_73177_12619E-10 -
			S*( 0.47369_66690_47318_75384_42068E-13 ))))))); 
		Q := 1.0 - Rsq*Common_Float( Poly );
	     end;

	  when 19..27 =>

	     declare
		type Working_Float is digits Min(27, Max_Digits);
		R, S, Poly : Working_Float; 
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := 0.5 -
		     S*( 0.41666_66666_66666_66666_66665_16383_4051E-01 -
		     S*( 0.13888_88888_88888_88888_88245_47342_9026E-02 -
		     S*( 0.24801_58730_15873_01576_57181_53770_2548E-04 -
		     S*( 0.27557_31922_39858_81219_61255_06316_8107E-06 -
		     S*( 0.20876_75698_78631_80489_36726_20113_1863E-08 -
		     S*( 0.11470_74559_61289_73694_98585_54814_7595E-10 -
		     S*( 0.47794_77003_63678_43164_38976_00318_0721E-13 -
		     S*( 0.15618_79265_92016_38912_62591_16219_0901E-15 -
		     S*( 0.40810_43877_26966_16816_80251_59059_3738E-18 )
			    )))))))); 

		Q := 1.0 - Rsq*Common_Float( Poly );

	     end;

	  when 28..33 =>

	     declare
		type Working_Float is digits Min(33, Max_Digits);
		R, S, Poly : Working_Float; 
	     begin
		R := Working_Float( R1 + R2 );
		S := R * R;
		Poly := 0.5 -
		     S*( 0.41666_66666_66666_66666_66666_66666_6480E-01 -
		     S*( 0.13888_88888_88888_88888_88888_88877_6335E-02 -
		     S*( 0.24801_58730_15873_01587_30158_46180_0299E-04 -
		     S*( 0.27557_31922_39858_90652_55387_96436_4088E-06 -
		     S*( 0.20876_75698_78680_98976_51897_22915_8020E-08 -
		     S*( 0.11470_74559_77297_23343_14028_68248_0321E-10 -
		     S*( 0.47794_77332_38691_60941_21737_53668_2651E-13 -
		     S*( 0.15619_20696_74971_44814_63427_34672_9062E-15 -
		     S*( 0.41103_17454_13508_37011_88930_50799_6253E-18 -
		     S*( 0.88966_22926_27803_79129_98598_30659_9363E-21 -
		     S*( 0.16020_11193_30748_03892_30511_97354_2056E-23
			     ))))))))))); 
		Q := 1.0 - Rsq*Common_Float( Poly );

	     end;

	  when others =>

	     raise Program_Error;  -- assumption (1) is violated.

       end case;

    -- This completes the core approximation.

       return( Q );


    end KF_Cos;

    function KF_AtnSm( Y : Common_Float ) return Common_Float is

    -- On input, Y is a floating point value in Common_Float, |Y| <= 1/16;
    -- On output, a Common_Float value is returned which represents the 
    -- approximation of atan(Y)-Y.

       Q                    : Common_Float;

    begin


    -- The following is the core approximation. We approximate
    -- exp(R1+R2)-1 by a polynomial. The case analysis finds both
    -- a suitable floating-point type (less expensive to use than 
    -- Common_Float) and an appropriate polynomial approximation
    -- that will deliver a result accurate enough with respect to
    -- Float_Type'Base'Digits. Note that the upper bounds of the
    -- cases below (6, 15, 16, 18, 27, and 33) are attributes
    -- of predefined floating types of common systems.

       case Float_Type'Base'Digits is

	  when 1..6 =>

	     declare
		type Working_Float is digits Min(6, Max_Digits);
		R, S, Poly : Working_Float;
	     begin
		R := Working_Float( Y );
		S := R * R;
		Poly := -R*S*( 0.33333_23001 -
			   S*( 0.19918_59719 ));
		Q := Common_Float( Poly );
	     end;

	  when 7..15 =>

	     declare
		type Working_Float is digits Min(15, Max_Digits);
		R, S, Poly : Working_Float;
	     begin
		R := Working_Float( Y );
		S := R * R;
		Poly := -R*S*( 0.33333_33333_33331_70500 -
			   S*( 0.19999_99999_93932_23405 -
			   S*( 0.14285_71356_18071_69030 -
			   S*( 0.11110_73628_35145_25407 -
			   S*( 0.90029_81028_54497_84439E-01 ))))); 
		Q := Common_Float( Poly );
	     end;

	  when 16 =>

	     declare
		type Working_Float is digits Min(16, Max_Digits);
		R, S, Poly : Working_Float; 
	     begin
		R :=  Working_Float( Y );
		S := R * R;
		Poly := -R*S*( 0.33333_33333_33331_70500 -
			   S*( 0.19999_99999_93932_23405 -
			   S*( 0.14285_71356_18071_69030 -
			   S*( 0.11110_73628_35145_25407 -
			   S*( 0.90029_81028_54497_84439E-01 ))))); 
		Q := Common_Float( Poly );

	     end;

	  when 17..18 =>

	     declare
		type Working_Float is digits Min(18, Max_Digits);
		R, S, Poly : Working_Float;
	     begin
		R := Working_Float( Y );
		S := R * R;
		Poly := -R*S*( 0.33333_33333_33333_33130_94892 -
			   S*( 0.19999_99999_99989_99552_86683 -
			   S*( 0.14285_71428_40674_81920_91353 -
			   S*( 0.11111_10986_45979_04500_50686 -
			   S*( 0.90904_31858_09744_75086_35684E-01 - 
			   S*( 0.76023_22269_40038_42302_02382E-01 ))))));
		Q := Common_Float( Poly );
	     end;

	  when 19..27 =>

	     declare
		type Working_Float is digits Min(27, Max_Digits);
		R, S, Poly : Working_Float; 
	     begin
		R := Working_Float( Y );
		S := R * R;
		Poly := -R*S*( 0.33333_33333_33333_33333_33333_30640_688 -
			   S*( 0.19999_99999_99999_99999_99725_07169_093 -
			   S*( 0.14285_71428_57142_85704_61906_79692_957 -
			   S*( 0.11111_11111_11110_94447_44685_62289_640 -
			   S*( 0.90909_09090_89284_08030_74591_43510_263E-01 -
			   S*( 0.76923_07682_73618_54039_14363_83712_288E-01 -
			   S*( 0.66666_63192_87231_75829_95036_25183_458E-01 -
			   S*( 0.58815_92881_06830_95028_64129_28595_505E-01 -
			   S*( 0.51711_22330_76482_53731_52208_70654_932E-01 
				  )))))))));
		Q := Common_Float( Poly );
	     end;

	  when 28..33 =>

	     declare
		type Working_Float is digits Min(33, Max_Digits);
		R, S, Poly : Working_Float; 
	     begin
		R := Working_Float( Y );
		S := R * R;
		Poly := -R*S*( 0.33333_33333_33333_33333_33333_33333_3304 -
			   S*( 0.19999_99999_99999_99999_99999_99956_0187 -
			   S*( 0.14285_71428_57142_85714_28569_15903_2129 -
			   S*( 0.11111_11111_11111_11111_05278_66512_9097 -
			   S*( 0.90909_09090_90909_08220_63173_05946_2434E-01 -
			   S*( 0.76923_07692_30761_06801_67642_90322_8423E-01 -
			   S*( 0.66666_66666_61651_60744_77042_68877_1500E-01 -
			   S*( 0.58823_52920_68496_03403_53270_56134_3084E-01 -
			   S*( 0.52631_52374_60356_91863_62352_54421_7989E-01 -
			   S*( 0.47609_62465_86213_53088_72992_19065_8779E-01 -
			   S*( 0.42553_42964_37218_11922_99573_45059_6737E-01 
				 )))))))))));
		Q := Common_Float( Poly );
	     end;

	  when others =>

	     raise Program_Error;  -- assumption (1) is violated.

       end case;

    -- This completes the core approximation.

       return( Q );


    end KF_AtnSm;

    Atan_Jby256 : constant array( Integer range 16..256, Position ) of
		 Common_Float := ( 
      (16#0.0FFA#,   16#0.0000ADDB967EF4E36CB2792DC0E2E0D51319C12CF#),
      (16#0.10F9#,   16#0.00009EA71D52A6F6FCB0089E4ED8919D8CFB4CED4#),
      (16#0.11F8#,   16#0.00006DBF082D58DED447119A20FDB0FE529D558C1#),
      (16#0.12F7#,   16#0.000019318A4A9A027FA2EF373003E5CAA22519EC5#),
      (16#0.13F5#,   16#0.00009F0E7C559D6B1338A177E11CD9BEC9EB30FB4#),
      (16#0.14F3#,   16#0.0000FD677292FB2011A6C4C92F3B2F1BE4D45EA9E#),
      (16#0.15F2#,   16#0.0000324FD2D7B262A3691004C637EACE735FA0C51#),
      (16#0.16F0#,   16#0.00003BDCEA4B0CEC0FF1AED05E835FD94922F357B#),
      (16#0.17EE#,   16#0.0000182602F10E8C126ACFCF099F06CECFC1508F3#),
      (16#0.18EB#,   16#0.0000C54478FB282E6510195A499750DEC78B76238#),
      (16#0.19E9#,   16#0.00004153CFDCF168CCB875A711F8151E9FA90CA27#),
      (16#0.1AE6#,   16#0.00008A71C722B838029CD22173B6E5BFBBFE48A9D#),
      (16#0.1BE3#,   16#0.00009EBE6F07C37DEE3CA681661CBB3DD21AFCA1D#),
      (16#0.1CE0#,   16#0.00007C5C3CCA322271CC84B2314D28C1967B9DD82#),
      (16#0.1DDD#,   16#0.000021701EBA6E653BFF35DBF95FAB59910B169AF#),
      (16#0.1ED9#,   16#0.00008C2190043AF62D33D369C276A948E9E36D846#),
      (16#0.1FD5#,   16#0.0000BA9AAC2F6DC65912F313E7D111DEF1672AFB2#),
      (16#0.20D1#,   16#0.0000AB0842567B30E96BE7F320176B74FCB8FBED6#),
      (16#0.21CD#,   16#0.00005B99E811031472E72233979639684D87AF78F#),
      (16#0.22C8#,   16#0.0000CA820C10A0B5E774573F548D5E3062929DB77#),
      (16#0.23C3#,   16#0.0000F5F6086E4DC96F4DD64A60E82BE678A856B0A#),
      (16#0.24BE#,   16#0.0000DC2E34A6B8BE36BDEF0D1883CE1CAD71A1221#),
      (16#0.25B9#,   16#0.00007B65F743FE63CA1C155854D14FCAB40FD271A#),
      (16#0.26B3#,   16#0.0000D1DBD733472D01458DCFD9CE2BD30B5BD245B#),
      (16#0.27AD#,   16#0.0000DDD18CC4D8B0D1D8674940D83FA15DD4BD3E2#),
      (16#0.28A7#,   16#0.00009D8C12553C8FB29D9D2D3C5B858FD95008299#),
      (16#0.29A1#,   16#0.00000F53B49E2E8F991F8D6F8ABCC1F785F820A0A#),
      (16#0.2A9A#,   16#0.0000317422AE148C14179442039BACA952F68C3D1#),
      (16#0.2B93#,   16#0.0000023C7D84D3BEAD534FFBC30B7A650B4F9B754#),
      (16#0.2C8B#,   16#0.00007FFF6754E7E0AC949883D34C6F1118FDDF54C#),
      (16#0.2D83#,   16#0.0000A9131267B1B5AAD627F0EF6549A0B4FFB8C87#),
      (16#0.2E7B#,   16#0.00007BD14FA403A94BC24AD6668D5F078B6224297#),
      (16#0.2F72#,   16#0.0000F6979CB6044D1EC2D3E207271D21E4EB4035A#),
      (16#0.306A#,   16#0.000017C731D78FA95BB33518DBCC20FB83AA95920#),
      (16#0.3160#,   16#0.0000DDC50F38517739967E7B9D85F2C5A797D90C3#),
      (16#0.3257#,   16#0.000046FA0A04E37961097805C4E1EC71BFD3E8E91#),
      (16#0.334D#,   16#0.000051D2D90C4C39EC03CF68691BBACEAAFC49930#),
      (16#0.3442#,   16#0.0000FCC021034B79083ACFA86463CF3B8D8A7AD78#),
      (16#0.3538#,   16#0.000046368064F28745DF44E6542A87586909C1451#),
      (16#0.362D#,   16#0.00002CAE9AF0179E951C0FA96633F48AC9F4E6AC1#),
      (16#0.3721#,   16#0.0000AEA524C14408BD88697072D54BC0A19144A34#),
      (16#0.3815#,   16#0.0000CA9AED08CD7D99205506C9213BED92F30BC53#),
      (16#0.3909#,   16#0.00007F14E85CDB9A908BE5F45766D521969F79C2C#),
      (16#0.39FC#,   16#0.0000CA9C3AA72AA3DCD1FB8F57DAC766CC23F3B11#),
      (16#0.3AEF#,   16#0.0000ABBE40AE6CE32468A9A2CBEF5E39EC4B3B0A8#),
      (16#0.3BE2#,   16#0.0000210C993B3BEA7E74AC40B87F65091C68ABE98#),
      (16#0.3CD4#,   16#0.0000291D2DD89AC33591014FBB281A0733BF6DAF4#),
      (16#0.3DC5#,   16#0.0000C28A3B301999623DCEE515BD423C91D7DACDB#),
      (16#0.3EB6#,   16#0.0000EBF25901BAC55B71E7BD7DE885F96A9FEA40E#),
      (16#0.3FA7#,   16#0.0000A3F881B7C826E28DE7438BD4F2B7344F7EA1B#),
      (16#0.4097#,   16#0.0000E9441996D698D2097CD05B85A52F129D011DA#),
      (16#0.4187#,   16#0.0000BA80F58A43D4113E49D579FE16A0FF1FDDAF1#),
      (16#0.4277#,   16#0.0000165F618D8962E47390CB8655E9D1571285505#),
      (16#0.4365#,   16#0.0000FB9426B2CD47C317BD5A3EAE0BB79AF3FD40A#),
      (16#0.4454#,   16#0.000068D890C726B237236F884613C81A979D9A18B#),
      (16#0.4542#,   16#0.00005CEA73951A8694580635CD86108B4BF1726D0#),
      (16#0.462F#,   16#0.0000D68C2FC5E0986523A458DFC414C687E9714DE#),
      (16#0.471C#,   16#0.0000D484B7620F41329C39A686926C0D1131740EF#),
      (16#0.4809#,   16#0.0000559F91F25773E6E6B85EB78CCC11F2ADF0AEA#),
      (16#0.48F5#,   16#0.000058ACE041078EB3A3A77FFDDEBE99A0D090CF4#),
      (16#0.49E0#,   16#0.0000DC815FBD16F88322C92037F0A23D223E10CF9#),
      (16#0.4ACB#,   16#0.0000DFF66D7F880A11E7E0E16A2148AAF4CF1A52A#),
      (16#0.4BB6#,   16#0.000061EA08F3F8DC892C7500964D0DE34F9E3EF24#),
      (16#0.4CA0#,   16#0.0000613ED6254656A0F566C60D0DD218A1F7C145D#),
      (16#0.4D89#,   16#0.0000DCDC1FAF2F34E2D5DA4C693D7994045247C28#),
      (16#0.4E72#,   16#0.0000D3ADD855EED0B25A5DEB8EC8B08A1AA6B1C0E#),
      (16#0.4F5B#,   16#0.000044A49C44D1137CA41CC9589E8CE198E65EDF2#),
      (16#0.5043#,   16#0.00002EB5B1F4CA4F37451EC4AEDBD671CD697057E#),
      (16#0.512A#,   16#0.000090DB0ABC26A2A1BC3AA4C45C6CF1A7413C521#),
      (16#0.5211#,   16#0.00006A1343086D1E28A7563C6A594645CB14D6EA6#),
      (16#0.52F7#,   16#0.0000B961A2439B0D91C41EEAC549D5FA0148A6DA0#),
      (16#0.53DD#,   16#0.00007DCE1A65E39A978C2003BBDE5D5D587C4CF43#),
      (16#0.54C2#,   16#0.0000B6654735276D4CDBFBBDFBECF46090961CE98#),
      (16#0.55A7#,   16#0.000062386D335F009CF0C76180EC7A655DBB882D3#),
      (16#0.568B#,   16#0.0000805D783D3913B7A8F82E45741DEB4EDA313C6#),
      (16#0.576F#,   16#0.00000FEEF9DA34F50760DBB7BECE25FB2CB9C490E#),
      (16#0.5852#,   16#0.0000100C273F8658DA8EA8EE100507E15A042E6F4#),
      (16#0.5934#,   16#0.00007FD8D7071605BFC183F0242309F5848AF9637#),
      (16#0.5A16#,   16#0.00005E7D7E9BF7DB50FD65CA4AC7B49AB58574B2D#),
      (16#0.5AF7#,   16#0.0000AB272F5DB3968EA706485F6062FA576944ACA#),
      (16#0.5BD8#,   16#0.00006507937BC239C55190916E7F22419EC21CBBD#),
      (16#0.5CB8#,   16#0.00008B54EA8AA62636CB1860D84315183DF5116DB#),
      (16#0.5D98#,   16#0.00001D4A05D407C45DD0DC6713FFFA13E3D8A9869#),
      (16#0.5E77#,   16#0.00001A264463440DBA09D077B0366780E2AD83C47#),
      (16#0.5F55#,   16#0.0000812D8ECFDD69C885C2B249A0881312E09E0EA#),
      (16#0.6033#,   16#0.000051A852C74218606697B05E6CC5EC618C28F83#),
      (16#0.6110#,   16#0.00008AE37E575DD76A0299B41B5C3A3B8175A01C8#),
      (16#0.61ED#,   16#0.00002C307AFB6E923055D05B8F0B9465D580574F7#),
      (16#0.62C9#,   16#0.000034E5286C95B6D0BA3748FA85146EE25BE4F28#),
      (16#0.63A4#,   16#0.0000A45BD737A14C0A7E12BFAFAF4343EE0DD669E#),
      (16#0.647F#,   16#0.000079F34319891074188054B536BEC637D321248#),
      (16#0.6559#,   16#0.0000B50E8D241CCD80A9BF05795AF8A9973E149E6#),
      (16#0.6633#,   16#0.0000551535AC619E6C988FD0A76CDBE1C93D002A4#),
      (16#0.670C#,   16#0.0000597316041C36CD2C044A19F5325F29C10727E#),
      (16#0.67E4#,   16#0.0000C1985A000637D8FB836BC6581A7B995F498A0#),
      (16#0.68BC#,   16#0.00008CF9794C2C605357DE5C21AC153C672881904#),
      (16#0.6993#,   16#0.0000BB0F308FF2DB213E4AF4800F389B3700206E9#),
      (16#0.6A6A#,   16#0.00004B567A633C25C85BFEEBB02D4A70C3F91D5AC#),
      (16#0.6B40#,   16#0.00003D5088162DFC4C33891D2E7EEF7E0A43580C0#),
      (16#0.6C15#,   16#0.00009082BA4D0E6CEC6B9C9CBAAA6BAEFF35AAF24#),
      (16#0.6CEA#,   16#0.000044769971B1AE187B1CA504031A2EAAA4088C5#),
      (16#0.6DBE#,   16#0.000058B9CDFBEF8D6B2E6D9F863D50C8CD7949F86#),
      (16#0.6E91#,   16#0.0000CCDE189295519A1B46E4AFE929CEC1897CA0D#),
      (16#0.6F64#,   16#0.0000A0794A0646B4EABC6509499B66209B4BDED17#),
      (16#0.7036#,   16#0.0000D3253B27BE33E318F6CB3CC65C01DB0A5F97A#),
      (16#0.7108#,   16#0.0000647FC47ADA4E820BCB4C1123A70868DB9C1EF#),
      (16#0.71D9#,   16#0.0000542AB5C7E28B474008FBC1B87EA7BF8B16E66#),
      (16#0.72A9#,   16#0.0000A1CBCD8C6C0CA833C0B1B7A3F6A7561939859#),
      (16#0.7379#,   16#0.00004D0CB04D425D305BBE70E536E164325927439#),
      (16#0.7448#,   16#0.0000559ADFCAB5BE67835886C302BBADCEBB3F7CE#),
      (16#0.7516#,   16#0.0000BB27B218ACC4A108B1F7C969A4A369AE94166#),
      (16#0.75E4#,   16#0.00007D68489BD35FF40AD24DAC4FDE79CB45B4D2D#),
      (16#0.76B1#,   16#0.00009C1586ED3DA2B7F222F65E1D4681B70A0AC39#),
      (16#0.777E#,   16#0.000016EC09A5D0A1DDD86050912E14CB66EFB5308#),
      (16#0.7849#,   16#0.0000EDAC1D12BFB53D9E719CAD55185BC787457EF#),
      (16#0.7915#,   16#0.00002019B3D468274754259D31E3BA15B11E90DA1#),
      (16#0.79DF#,   16#0.0000ADFC5D68D10E53DC1BF34356F9FD1790505C4#),
      (16#0.7AA9#,   16#0.0000971F3CA31097D967651DF82B549A759EDD0F0#),
      (16#0.7B72#,   16#0.0000DB50FE10D380DA2733DDEC71326E2C085D461#),
      (16#0.7C3B#,   16#0.00007A63CE4F3ED0BF57834E76BBE5183EFAC40FF#),
      (16#0.7D03#,   16#0.0000742D50505F2E33691E3EAEE476610806496FC#),
      (16#0.7DCA#,   16#0.0000C8869392545227083DFADD004A57B9B4C8A2F#),
      (16#0.7E91#,   16#0.0000774C0A496235A9C23D7274B5310EE6B4F66CB#),
      (16#0.7F57#,   16#0.0000805D7F7E0C9D43D8F624D8A9D6729F93B5A82#),
      (16#0.801C#,   16#0.0000E39E0D205C99A6D6C6C54D938596692486326#),
      (16#0.80E1#,   16#0.0000A0F412116A7B689F99521DBB92B1AD73C7082#),
      (16#0.81A5#,   16#0.0000B8492824418F9B38D85540FF5431C9474F206#),
      (16#0.8269#,   16#0.0000298A1A172DC6F39377A7131D6A27BBFF43FF8#),
      (16#0.832B#,   16#0.0000F4A6D9867E2A4B6A09CB61A515C0F1155CD87#),
      (16#0.83EE#,   16#0.0000199274D9C1B70C9E04450AC73920C96BA6DC7#),
      (16#0.84AF#,   16#0.000098430D2C7EEDE4DF5EA560D2AAEAAEA472105#),
      (16#0.8570#,   16#0.000070B1CC3361075ABF2DE445A53E2C75D49076C#),
      (16#0.8630#,   16#0.0000A2DADA1ED065D3E84ED5013CA37D92A950DA9#),
      (16#0.86F0#,   16#0.00002EBD537BE67C97B4E3AD655622BA7C75090D3#),
      (16#0.87AF#,   16#0.0000145B3F14A800988D8BF64C8681429BDD6BF9F#),
      (16#0.886D#,   16#0.000053B983D06AD3867A898F60AF9F5DBCE1325F9#),
      (16#0.892A#,   16#0.0000ECDFDE9547B5094478FC472B4AFB8FBE7B9FB#),
      (16#0.89E7#,   16#0.0000DFD8D82B726221CDF87DA6F989C0B2E59EEFA#),
      (16#0.8AA4#,   16#0.00002CB1BB234D68B0756E81879E8439FE76F7ECC#),
      (16#0.8B5F#,   16#0.0000D37A89BF099606FE141BD348634D99492CF6B#),
      (16#0.8C1A#,   16#0.0000D445F3E09B8C439D801860205920F8E244490#),
      (16#0.8CD5#,   16#0.00002F294CFCCCB3F27A74E0F9602B3B8709D0F21#),
      (16#0.8D8E#,   16#0.0000E43C8214276F0B9BA88386B8A2CCDFC2B8615#),
      (16#0.8E47#,   16#0.0000F39A0FB27928BBC9D5E7929F8ECDEF9CEC1A8#),
      (16#0.8F00#,   16#0.00005D5EF7F59F9B5C835E1665C43747918A67E06#),
      (16#0.8FB8#,   16#0.000021AAB89C516C659F6D7DD47136A9B668D8770#),
      (16#0.906F#,   16#0.0000409F411D8D0DC515828917850AB599F0A4844#),
      (16#0.9125#,   16#0.0000BA60E8C953ADA7BF114C1880F6A2B0B51D132#),
      (16#0.91DB#,   16#0.00008F1664F350E210E4F9C1126E021FD995E8D1F#),
      (16#0.9290#,   16#0.0000BEE8BF280AAD5511836CF3797D76EA50B3E72#),
      (16#0.9345#,   16#0.00004A034B6D30744D228131CF31C981C4DB4F92C#),
      (16#0.93F9#,   16#0.000030939E8D9A877E6C2A3BB3E2ECDA573DC02FF#),
      (16#0.94AC#,   16#0.000072C9847186F618C4F393F78A32F8F38AE0F47#),
      (16#0.955F#,   16#0.000010D6F6839B85180035A045C3B111748DBE470#),
      (16#0.9611#,   16#0.00000AF012232FD68FDE4B8B683E268649AF3B919#),
      (16#0.96C2#,   16#0.0000614B0F245E0DABDF5C91AFA42AA81FD293F46#),
      (16#0.9773#,   16#0.00001420365E538BABD3FE19F1AEB6B29798DB274#),
      (16#0.9823#,   16#0.000023A9D84856C18ED9E25E70CDBA0ED14FC5826#),
      (16#0.98D2#,   16#0.0000902443A5F2819570F4F584892A42FDCC108F9#),
      (16#0.9981#,   16#0.000059CDBC42B2BBA5DB0F7505678AD6593E8D1B4#),
      (16#0.9A2F#,   16#0.000080E671BDDA204226F8E2204FF3BCDAE46F061#),
      (16#0.9ADD#,   16#0.000005B0766673C77433726D172B70996C03D1D89#),
      (16#0.9B89#,   16#0.0000E86FB6281FAC21D87F3A7E4B6DD3C5DAAC0ED#),
      (16#0.9C36#,   16#0.00002969ED88F490E3C682832BFBDA16E92930016#),
      (16#0.9CE1#,   16#0.0000C8E6A0B8CDB9F799C4E8174CF11C5A2AC6A3B#),
      (16#0.9D8C#,   16#0.0000C72F12B256D06DDE0F8AD5420D93FEEBFB012#),
      (16#0.9E37#,   16#0.0000248E3C6E243D75ED605E8B128B9C394F7AC85#),
      (16#0.9EE0#,   16#0.0000E150C4282259CC6E1E9D2E1DFDF5925A09FFF#),
      (16#0.9F89#,   16#0.0000FDC4F4B7A1ECF8B492644F0701DF9D743D1BC#),
      (16#0.A032#,   16#0.00007A3AB4FA44AA4EE0DC81A9BE5A70912C563A9#),
      (16#0.A0DA#,   16#0.000057037F52089EB052993BB28EC56839662ECBF#),
      (16#0.A181#,   16#0.000094725936ADD9D4DDF1E8CC2EF763F0F535930#),
      (16#0.A228#,   16#0.000032DBCADAAE0892FE9C08637AF0E5D084146D4#),
      (16#0.A2CE#,   16#0.00003295D6E3FA332D31D1C26B7A893D3F56D575E#),
      (16#0.A373#,   16#0.000093F7F238AF63232723DD99B9EFD38377A1EC5#),
      (16#0.A418#,   16#0.0000575AFBDFEF9766A66626D63334D8D43B4E063#),
      (16#0.A4BC#,   16#0.00007D1934F7092419A87F2A457DAC9EE3F08689E#),
      (16#0.A560#,   16#0.0000058E38BB13641CE470572003EB4629EF1ECA0#),
      (16#0.A602#,   16#0.0000F116F4A7247EA7C1EC1A242451D4464264C5C#),
      (16#0.A6A5#,   16#0.00004011A0A740F1D448ED544580EE24A1872A95C#),
      (16#0.A746#,   16#0.0000F2DDB760229467B7D66F2D74E01921B81774D#),
      (16#0.A7E8#,   16#0.000009DBEE8BF1D513F3E7C24B4D1412B0D212CAE#),
      (16#0.A888#,   16#0.0000856E2F6C0923DCD6832A63DE1EE8E17AB4A5A#),
      (16#0.A928#,   16#0.000065F78F4FD7AB19217375DE41B9DF73CA99A35#),
      (16#0.A9C7#,   16#0.0000ABDC4830F5C8916A84B5BE7933F5F9971655E#),
      (16#0.AA66#,   16#0.00005781B1647B14648FF1E8782ACC5172CDE5487#),
      (16#0.AB04#,   16#0.0000694E3861A332739E1BD011000FD4FA1DB9E78#),
      (16#0.ABA1#,   16#0.0000E1A9599DCC2BFE5966F02B469FB603728B70E#),
      (16#0.AC3E#,   16#0.0000C0FB997DD6A1A36273A56AFA8EF4183DB5406#),
      (16#0.ACDB#,   16#0.000007AE7D5CEDC9F677DFB28769F24D1951BB831#),
      (16#0.AD76#,   16#0.0000B62C84A8BAE625EE295286CDB591529F4F435#),
      (16#0.AE11#,   16#0.0000CCE12213059C8562C352995154ABC1476866C#),
      (16#0.AEAC#,   16#0.00004C38B4D8C08014725E2F3E52070A03742B464#),
      (16#0.AF46#,   16#0.000034A0821E7EF408A41BAE1904F9164736FC44B#),
      (16#0.AFDF#,   16#0.00008686AE624F92CC0BFFD23AA0AE627E411319F#),
      (16#0.B078#,   16#0.0000425A3702F4488F2CCD75CD9CDAF2A5B0F42A1#),
      (16#0.B110#,   16#0.0000688AEBDC6F6A43D65788B9F6A7B509E2828D4#),
      (16#0.B1A7#,   16#0.0000F98968F9DB3A5E62BE56B2F9E94A3AE3C1F67#),
      (16#0.B23E#,   16#0.0000F5C7105C7F84D7EFF716B8C33242B2A3BDCA5#),
      (16#0.B2D5#,   16#0.00005DB603D8174363262866C6B9DC4BA2D0847E9#),
      (16#0.B36B#,   16#0.000031C91F043691590141744462F939E469FF280#),
      (16#0.B400#,   16#0.00007273F142BF9C56579305BC3F157669A530952#),
      (16#0.B495#,   16#0.0000202AB7DB53B2972B51D45971CA787426F3BB8#),
      (16#0.B529#,   16#0.00003B62582BAC12934C772B2C3F989BC0998AF1F#),
      (16#0.B5BC#,   16#0.0000C49059ECC4AFF8F3CEE75E3907D575216F47B#),
      (16#0.B64F#,   16#0.0000BC2AE18CC1C09DE29BD280C8A0B6EE7E3ADE5#),
      (16#0.B6E2#,   16#0.000022A8AA9D77901DE72A31F5C864980AD7A1EC7#),
      (16#0.B773#,   16#0.0000F88102577AD25DD460CC4E3EF828F91A49550#),
      (16#0.B805#,   16#0.00003E2BC2319E73CB2DA55210A4443D3D7AECC11#),
      (16#0.B895#,   16#0.0000F4214A8CC2B9B2066FB1102F77175A4D5A41A#),
      (16#0.B926#,   16#0.00001ADA7D73D86522CCB6CD266BC46005FDE833F#),
      (16#0.B9B5#,   16#0.0000B2D0B96FF9775FD06A892D100E70F01F06D4D#),
      (16#0.BA44#,   16#0.0000BC7DD470782F654C2CB10942E38623228454D#),
      (16#0.BAD3#,   16#0.0000385C16C6C3DC95898E5210DA150B48CCA7310#),
      (16#0.BB61#,   16#0.000026E636360232A4AA35B907915AE61342DEFF6#),
      (16#0.BBEE#,   16#0.0000889751163AE64C11F821A7AE3E62740E9AAFA#),
      (16#0.BC7B#,   16#0.00005DEAE98AF280D4113006E80FB29013FAB81F5#),
      (16#0.BD07#,   16#0.0000A75CE0CD1089E33E6C0AFABB5D7415D7D6A58#),
      (16#0.BD93#,   16#0.000065697287EC62FE419F7DC09B179189FB1E26C#),
      (16#0.BE1E#,   16#0.0000988D30495B767965A6065109B7436BBB76D27#),
      (16#0.BEA9#,   16#0.00004144FD049AAC1043C5E755282E7D01438341F#),
      (16#0.BF33#,   16#0.0000600E08A7EC6FA8EE913E8E50ADA66F6BFD26A#),
      (16#0.BFBC#,   16#0.0000F565CBC4C3FAB9501D5A441A24B5CE2085DF5#),
      (16#0.C046#,   16#0.000001CA034A55FE19B11D412B630D4D59538BABA#),
      (16#0.C0CE#,   16#0.000085B8AC52664089DD62C46E92FA24D58EE867A#),
      (16#0.C156#,   16#0.000081B00000294592FCE924D2425EA8695BF27C7#),
      (16#0.C1DD#,   16#0.0000F62E6F711098838827FE361448E71812B904E#),
      (16#0.C264#,   16#0.0000E3B29FBF57E7C9EE01F5C1713E2F8039A8EC1#),
      (16#0.C2EB#,   16#0.00004ABB661628B5B373FE45C61BB9FAE970EC0E0#),
      (16#0.C371#,   16#0.00002BC7C3D72A045626EB9FC678FDE093A72F30D#),
      (16#0.C3F6#,   16#0.00008756E2D1510AF5AD957F4BC61565CE4AA623C#),
      (16#0.C47B#,   16#0.00005DE81188C7B34B8B6951667E250297192BB86#),
      (16#0.C4FF#,   16#0.0000AFFABF8FBD548CB43D10BC9E02214DA621B60#),
      (16#0.C583#,   16#0.00007E0E79EFF5E099DB35249263D1D46837D6DA1#),
      (16#0.C606#,   16#0.0000C8A2E7A4EB7D480EDB88665164556F74B5DA8#),
      (16#0.C689#,   16#0.00009037C626564FE7345576574CA266DD13C472E#),
      (16#0.C70B#,   16#0.0000D54CE602EE13E7D54FBD09F2BE380E9C986EA#),
      (16#0.C78D#,   16#0.00009862278B38EE8E55C9C99FE946B23C9C7639E#),
      (16#0.C80E#,   16#0.0000D9F7778C3AD0CCBCAC0791EF71D43B7D29BEC#),
      (16#0.C88F#,   16#0.00009A8CCC19D89D745076DFA90D530C167B4C95E#),
      (16#0.C90F#,   16#0.0000DAA22168C234C4C6628B80DC1CD129024E088#) );

    procedure KP_Atn ( Y,  X   : in  Common_Float;
		       Z1, Z2  : out Common_Float ) is
     
    -- On input, Y and X are floating-point values in Common_Float such that
    -- 0 < Y < X. 
    -- On output, the values Z1, and Z2 are returned such that
    --      atan(Y/X) =  Z1 + Z2 , where
    -- atan is the inverse tangent. Whenever Z1 is non-zero, |Z| >= 1/32 and
    -- has at most 4 hex digits.

    -- Two methods are used to calculate atan(Y/X). If Y/X <= 1/16, then
    -- simple polynomial approximations are used. These calculations are 
    -- performed in the kernel function  KF_AtnSm  which returns atan(Y)-Y
    -- for an input argument |Y| <= 1/16.
    --
    -- For Y/X > 1/16, a table driven technique is used. First, a break point
    -- C = j/256, j = 16, 17, ..., 256 is found such that 
    -- | Y/X  -  C | < 1/512. This C can be obtained by Round( 256*Y/X ).
    -- Second, atan((Y-CX)/(X+CY)) is calculated via polynomial approximations.
    -- Because the argument's magnitude in question is small, the polynomials
    -- required are short. Finally, the value  atan(Y/X) is reconstructed as
    --    atan(Y/X) = atan(C)  +  atan( (Y-CX)/(X+CY) ).
    --

       V, U, V_by_U, C, U_Lead, U_Trail, Q, R     :  Common_Float;
       Index                                      :  Integer;

       Zero      : constant := 0.0;
       One       : constant := 1.0;
       One_16    : constant := 16#1.0#E-1;
       Two_Neg80 : constant := 2#1.0#E-80;
       Two_80    : constant := 2#1.0#E80;


    begin

       U := X; V := Y; 
       if U = One then
	  V_by_U := V;
       else
	  V_by_U := V/U;
       end if;

       if V_by_U <= One_16 then
	  Z1 := Zero;  Z2 := V_by_U + KF_AtnSm( V_by_U );
	  return;
       end if;

       
       Index := Integer( 16#1.0#E2 * V_by_U );
       C     := Common_Float( Index ) * 16#1.0#E-2;
       
       if V < Two_Neg80 then
	  V := Two_80 * V;  U := Two_80 * U;
       end if;

       if U = One then
	  R := (V - C)/(U + C*V);
       else
	  U_Lead  := Common_Float'Leading_Part( 
	    U, Common_Float'Machine_Mantissa/2 );
	  U_Trail := U - U_Lead;
	  R := ( (V - C*U_Lead) - C*U_Trail ) / ( U + C*V );
       end if;


     --Polynomial approximation of  atan(R)

       case Float_Type'Base'Digits is

	  when 1..6 =>

	     declare
		type Working_Float is digits Min(6, Max_Digits);
		S, P : Working_Float;
	     begin
		S := Working_Float(R);
		S := S*S;
		P := S*0.33333_26634;
		Q := R - R*Common_Float( P );
	     end;

	  when 7..15 =>

	     declare
		type Working_Float is digits Min(15, Max_Digits);
		S, P : Working_Float;
	     begin
		S := Working_Float(R);
		S := S*S;
		P := S*( 0.33333_33333_32240_95522 -
		     S*( 0.19999_91803_89891_43496 ));
		Q := R - R*Common_Float( P );
	     end;

	  when 16 =>

	     declare
		type Working_Float is digits Min(16, Max_Digits);
		S, P : Working_Float; 
	     begin
		S := Working_Float(R);
		S := S*S;
		P := S*( 0.33333_33333_32240_95522 -
		     S*( 0.19999_91803_89891_43496 ));
		Q := R - R*Common_Float( P );
	     end;

	  when 17..18 =>

	     declare
		type Working_Float is digits Min(18, Max_Digits);
		S, P : Working_Float;
	     begin
		S := Working_Float(R);
		S := S*S;
		P := S*( 0.33333_33333_33333_33174_03042 -
		     S*( 0.19999_99999_97844_27116_94195 -
		     S*( 0.14285_62650_27453_42297_36256 )));
		Q := R - R*Common_Float( P );
	     end;

	  when 19..27 =>

	     declare
		type Working_Float is digits Min(27, Max_Digits);
		S, P : Working_Float; 
	     begin
		S := Working_Float(R);
		S := S*S;
		P := S*( 0.33333_33333_33333_33333_33311_72058_56 -
		     S*( 0.19999_99999_99999_99544_27301_91257_38 -
		     S*( 0.14285_71428_53959_54585_25767_15009_50 -
		     S*( 0.11111_02043_07258_54991_41929_62303_48 ))));
		Q := R - R*Common_Float( P );
	     end;

	  when 28..33 =>

	     declare
		type Working_Float is digits Min(33, Max_Digits);
		S, P : Working_Float; 
	     begin
		S := Working_Float(R);
		S := S*S;
		P := S*( 0.33333_33333_33333_33333_33333_33330_5509 -
		     S*( 0.19999_99999_99999_99999_99916_05331_9450 -
		     S*( 0.14285_71428_57142_84833_81100_15518_0603 -
		     S*( 0.11111_11111_06928_08149_95978_85354_6911 -
		     S*( 0.09090_81679_15943_49047_00774_95327_5560 )))));
		Q := R - R*Common_Float( P );
	     end;

	  when others =>

	     raise Program_Error;  -- assumption (1) is violated.

       end case;
    -- This completes the core approximation.

       Z1 := Atan_Jby256( Index, Lead );
       Z2 := Atan_Jby256( Index, Trail ) + Q;

       return;

    end KP_Atn;


 --=====================================================================--
 --            DECLARATION OF THE ELEMENTARY FUNCTIONS                  --
 --=====================================================================--


    function Sqrt( X : Float_Type ) return Float_Type is

       Y       : Common_Float;

    begin

    -- Filter out exceptional cases.

       Y := Common_Float( X );

       if Y < 0.0 then
	  raise Argument_Error;
       end if;

       if Y = 0.0 then
	  return( X );
       end if;

       return( Float_Type( kf_Sqrt( Y ) ) );

    end Sqrt;

    function Log ( X : Float_Type ) return Float_Type is
     
    -- On input, X is a floating-point value in Float_Type;
    -- on output, the value of log(X) (natural log of X) is returned.

    -- The bulk of the computations are performed by the procedure
    -- KP_Log( Y, M, Z1, Z2 ) which returns log(Y) in M, Z1, and Z2
    -- where  
    --               log(Y) = M * log2  +  Z1  +  Z2,
    -- M of integer value, and Z1 only has at most 12 significant bits.


       Y, M, Z1, Z2, Result                 : Common_Float;

       Log2_Lead : constant Common_Float :=
		   16#0.B17#;
       Log2_Trail : constant Common_Float :=
		   16#0.000217F7D1CF79ABC9E3B39803F2F6AF40#;

    begin

       Y := Common_Float( X );

       if ( Y = 0.0 ) then
	  raise Constraint_Error;
       end if;

       if ( Y < 0.0 ) then
	  raise Argument_Error;
       end if;

    -- Get values of M, Z1, and Z2 so that the natural log of X
    -- can be calculated by log(X) = M*log(2) + Z1 + Z2

       KP_Log ( Y, M, Z1, Z2 );

       if ( M = 0.0 ) then
	  Result := Z1 + Z2;
       else
	  Result :=  M*Log2_Trail + Z2;
	  Result := (M*Log2_Lead + Z1) + Result;
       end if;

       return( Float_Type(Result) );

    end Log;

    function Log ( X, Base : Float_Type ) return Float_Type is
     
    -- On input, X and Base are floating-point values in Float_Type;
    -- on output, the value of log(X) (log of X to the Base) is returned.

    -- The bulk of the computations are performed by the procedure
    -- KP_Log( Y, M, Z1, Z2 ) which returns log(Y) in M, Z1, and Z2
    -- where  
    --               log(Y) = M * log2  +  Z1  +  Z2,
    -- M of integer value, and Z1 only has at most 12 significant bits.

    -- Two special cases are considered : Base = 2.0 and Base = 10.0.
    -- Since  log_2(X) = log(X) * (1/log(2)) and 
    --        log(X)   = M log 2 + Z1 + Z2.
    -- Thus,  log_2(X) = M  +  (Z1 + Z2) * (1/log(2)).
    -- We store 1/log(2) as  Log2_Inv, Log2_Inv_Lead, and Log2_Inv_Trail.
    -- If M = 0, we recast (Z1,Z2) as (W1,W2) so that W1*Log2_Inv_Lead is
    -- exact. If M is non-zero, we simply compute 
    --        M  +  (Z1+Z2)*Log2_Inv.

    -- Next, for log_10(X), we store Log10_Inv, Log10_Inv_Lead, and 
    -- Log10_Inv_Trail. The trick is the same for M = 0. For M /= 0,
    -- we need to compute  M * log(2)/log(10)  +  (Z1+Z2)/log(10).
    -- The constants  Log2div10_Lead and Log2div10_Trail are the leading
    -- and trailing part of log(2)/log(10).


       Y, Z, M, Z1, Z2, W1, W2, Temp, Result      : Common_Float;
       Ndigits      : Integer := Common_Float'Machine_Mantissa/2;

       Log2_Lead       : constant Common_Float :=
			 16#0.B17#;
       Log2_Trail      : constant Common_Float :=
			 16#0.00021_7F7D1_CF79A_BC9E3_B3980_3F2F6_AF40#;

       Log2_Inv_Lead   : constant Common_Float :=
			 16#1.71#;
       Log2_Inv_Trail  : constant Common_Float :=
			 16#0.00547_652B8_2FE17_77D0F_FDA0D_23A7D_11D6A_EF55#;
       Log2_Inv        : constant Common_Float :=
			 16#1.71547_652B8_2FE17_77D0F_FDA0D_23A7D_11D6A_EF55#;

       Log10_Inv_Lead  : constant Common_Float :=
			 16#0.6F2#;
       Log10_Inv_Trail : constant Common_Float :=
			 16#0.000DE_C549B_9438C_A9AAD_D557D_699EE_191F7_1A301#;
       Log10_Inv       : constant Common_Float :=
			 16#0.6F2DE_C549B_9438C_A9AAD_D557D_699EE_191F7_1A301#;

       Log2div10_Lead  : constant Common_Float :=
			 16#0.4D1#;
       Log2div10_Trail : constant Common_Float :=
			 16#0.00004_D427D_E7FBC_C47C4_ACD60_5BE48_BC135_69862#;

       Log16_Inv_Lead  : constant Common_Float :=
			 16#0.5C5#;
       Log16_Inv_Trail : constant Common_Float :=
			 16#0.00051_D94AE_0BF85_DDF43_FF683_48E9F_4475A_BBD54#;
       Log16_Inv       : constant Common_Float :=
			 16#0.5C551_D94AE_0BF85_DDF43_FF683_48E9F_4475A_BBD54#;


    begin

       Y := Common_Float( X );

       if ( Base <= 0.0  or  Base = 1.0 ) then
	  raise Argument_Error;
       end if;

       if ( Y = 0.0 ) then
	  raise Constraint_Error;
       end if;

       if ( Y < 0.0 ) then
	  raise Argument_Error;
       end if;

    -- Get values of M, Z1, and Z2 so that the log of X to the base
    -- can be calculated by log (x) = log(x) / log(base)

       KP_Log ( Y, M, Z1, Z2 );

       if ( Base  = 2.0 ) then 

	    W1 := Common_Float'Leading_Part( Z1+Z2, Ndigits );
	    W2 := (Z1 - W1) + Z2;
	    W2 := W1*Log2_Inv_Trail + W2*Log2_Inv;
	    W1 := W1*Log2_Inv_Lead;
	    if ( M = 0.0 ) then
	       Result := W1 + W2;
	    else
	       Z1 := M + W1;
	       Z2 := ((M - Z1) + W1) + W2;
	       Result := Z1 + Z2;
	    end if;

       elsif ( Base = 10.0 ) then

	     W1 := Common_Float'Leading_Part( Z1+Z2, Ndigits ); 
	     W2 := (Z1 - W1) + Z2;
	     W2 := (W1*Log10_Inv_Trail + W2*Log10_Inv);
	     W1 := W1*Log10_Inv_Lead;

	     if ( M = 0.0 ) then
		Result := W1 + W2;
	     else
		Y := M * Log2div10_Lead;
		Z1 := Y + W1;
		Z2 := ((Y - Z1) + W1) + (W2 + M*Log2div10_Trail);
		Result := Z1 + Z2;
	     end if;

       elsif ( Base = 16.0 ) then

	    W1 := Common_Float'Leading_Part( Z1+Z2, Ndigits );
	    W2 := (Z1 - W1) + Z2;
	    W2 := W1*Log16_Inv_Trail + W2*Log16_Inv;
	    W1 := W1*Log16_Inv_Lead;
	    if ( M = 0.0 ) then
	       Result := W1 + W2;
	    else
	       M  := M * 0.25;
	       Z1 := M + W1;
	       Z2 := ((M - Z1) + W1) + W2;
	       Result := Z1 + Z2;
	    end if;

       else

	     if ( M = 0.0 ) then
		Result := Z1 + Z2;
	     else
		Result := M * Log2_Trail + Z2; 
		Result := (M * Log2_Lead  + Z1) + Result;
	     end if;
	     Z         := Common_Float (Base);
	     KP_Log (Z, M, Z1, Z2 );
	     if ( M = 0.0 ) then
		Temp   := Z1 + Z2;
	     else
		Temp   := M * Log2_Trail + Z2;
		Temp   := (M * Log2_Lead  + Z1) + Temp;
	     end if;
	     Result := Result / Temp;

       end if;
       return( Float_Type(Result) );

    end Log;

    function Exp( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of exp(X) (the natural exponential of X) 
    -- is returned.

    -- The bulk of the computations are performed by the procedure
    -- KP_Exp (Y, M, Z1, Z2) which returns exp(Y) in M, Z1, and Z2
    -- where
    --              exp(Y) = 2**M * ( Z1 + Z2 )
    -- M of integer value, and Z1 only has at most 12 significant bits.

       Result                             : Float_Type;

       Y, Z1, Z2                          : Common_Float;
       M, J                               : Integer;

       Large_Threshold  : constant Common_Float :=
	  4.0 * Common_Float(Float_Type'Machine_Emax) * 6.931471806E-1;

       Small_Threshold  : constant Common_Float :=
	  Float_Type'Model_Epsilon;
	  

    begin

    -- Filter out exceptional cases.

       Y := Common_Float( X );

       if abs(Y) >= Large_Threshold then
	  if ( Y > 0.0 ) then
	     raise Constraint_Error;
	  else
	     return( Float_Type( 0.0 ) );
	  end if;
       elsif abs(Y) <= SMALL_THRESHOLD then
	  return( Float_Type( 1.0 + Y ) );
       end if;

    -- Get the values of M, Z1, and Z2  so that the natural exponential of Y
    -- can be calculated by  Exp(Y) = 2**M * (Z1 + Z2)

       KP_Exp (Y, M, Z1, Z2);

       case Common_Float'Machine_Radix is

	  when 2 =>
	     Y      := Z1 + Z2;

	  when others =>
	     J      := M rem 4;
	     M      := (M - J) / 4;
	     Z1     := Z1 * Two_to(J);
	     Z2     := Z2 * Two_to(J);
	     Y      := Z1 + Z2;

       end case;

       Result := Float_Type( Common_Float'Scaling( Y, M ) );

       return( Result );


    end Exp;

    function "**"( Left, Right : Float_Type ) return Float_Type is

    -- On input, Left and Right are floating-point values in Float_Type;
    -- On output, the value of Left**Right is returned.

       X : Float_Type renames Left;
       Y : Float_Type renames Right;

       Result                             : Float_Type;

       XX, YY, R1, R2, Z1, Z2, W1, W2, Q,
       Y1, Y2, Tmp, F                     : Common_Float;
       N, M, J                            : Integer;
       L, LL                              : Positive;

       Thirty_Two_by_Log2  : constant :=
	  16#2E.2A8EC_A5705_FC2EE_FA1FF_B41A4_74FA2_3AD5E#;

       Large_Threshold  : constant Common_Float :=
	  4.0 * Common_Float(Float_Type'Machine_Emax) /
		Common_Float(Float_Type'Model_Epsilon);


    begin

    -- Filter out exceptional cases.

       XX := Common_Float( X );
       YY := Common_Float( Y );

       if ( XX = 0.0  ) then
	  if ( Y < 0.0 ) then
	       raise Constraint_Error;
	  elsif ( Y > 0.0 ) then
	       return ( 0.0 );
	  else
	       raise Argument_Error;
	  end if;
       end if;

       if ( XX < 0.0  ) then
	  raise Argument_Error;
       end if;

       if ( XX = 1.0 or Y = 0.0 ) then
	  return ( 1.0 );
       end if;

       if ( YY = 1.0 ) then
	  return ( X );
       end if;

       if abs(YY) >= Large_Threshold then
	  if ( YY > 0.0 ) then
	       raise Constraint_Error;
	    else
	       return( Float_Type( 0.0 ) );
	  end if;
       end if;

       L  := (2*Common_Float'Machine_Mantissa) / 5;
       LL := Common_Float'Machine_Mantissa;

    -- Get log(X) to high accuracy

       KP_LogPw (XX, Z1, Z2);

       Y1 := Common_Float'Leading_Part( YY, L );
       Y2 := YY - Y1;

       W1 := Y1*Z1;
       W2 := Y1*Z2 + ( Y2*Z1 + Y2*Z2);

       Z1 := Common_Float'Leading_Part(W1 + W2, LL-2);	-- force storage
       Z2 := W1 - Z1;
       Z2 := W2 + Z2;
    -- Z1 + Z2 is Y*log(X) to high precision


    -- Now get exp(Z1+Z2)

    -- Perform argument reduction in Common_Float.

       N   := Integer( Z1 * Thirty_Two_by_Log2 );
       Tmp := Common_Float( N ) * Log2_by_32_Lead;
       if abs( Z1 ) >= abs( Tmp ) then
	  R1 := Z1 - Tmp;
       else
	  Tmp := 0.5 * Tmp;
	  R1 := (Z1 - Tmp) - Tmp;
       end if;
       R2 := Z2 -Common_Float(N) * Log2_by_32_Trail;
       J := N mod 32;
       M := (N - J)/32;
      
    -- Step 2. Approximation.

       Q := KF_Em1Sm( R1, R2 );

    -- Step 3. Function value reconstruction.

    -- We now reconstruct the exponential of the input argument
    -- so that Exp(Z1+Z2) = 2**M * (W1 + W2).
    -- The order of the computation below must be strictly observed.

       F  := Two_to_Jby32( J, Lead ) + Two_to_Jby32( J, Trail );

       W1 := Two_to_Jby32( J, Lead );
       W2 := Two_to_Jby32( J, Trail ) + F*Q;


       case Radix is

	  when 2 =>
	     Tmp := W1 + W2;

	  when others =>
	     J      := M rem 4;
	     M      := (M - J) / 4;
	     W1     := W1 * Two_to(J);
	     W2     := W2 * Two_to(J);
	     Tmp    := W1 + W2;

       end case;

       Result := Float_Type( Common_Float'Scaling( Tmp, M ) );

       return( Result );


    end "**";

 -----------------------------------------------------------------------

    function Sin( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of sin(X) (the sine of X) is returned.

       Y, R1, R2, Z : Common_Float;
       I_Flag       : Quadrant_Flag;

    begin

    -- Argument reduction by KP_Pi2Rd

       if ( X = 0.0 ) then
	  return( X );
       end if;

       Y := Common_Float( X );

       KP_Pi2Rd( Y, R1, R2, I_Flag );

       case I_Flag is
	 when 0 => Z := + KF_Sin(R1, R2);
	 when 1 => Z := + KF_Cos(R1, R2);
	 when 2 => Z := - KF_Sin(R1, R2);
	 when 3 => Z := - KF_Cos(R1, R2);
       end case;

       return Float_Type(Z);
    end Sin;

    function Sin( X, Cycle : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type; Cycle is a
    -- user specificed period.
    -- On output, the value of sin(X*2pi/Cycle) is returned.

       R, Y, Z, User_Period, P_Lead, P_Trail, K  : Common_Float;
       Zero                                      : Common_Float := 0.0;
       I_Flag					 : Quadrant_Flag;

    begin

    -- Argument reduction by  "rem"

       User_Period := Common_Float( Cycle );

       if User_Period <= 0.0 then
	  raise Argument_Error;
       end if;

       Y := Common_Float( X ) rem User_Period;
       R := Y;

       K := Common_Float'Rounding( 4.0*R / User_Period );
       I_Flag := Quadrant_Flag( Integer(K) mod 4 );

       P_Lead  := Common_Float'Leading_Part( User_Period, 4 );
       P_Trail := User_Period - P_Lead;

       K := K * 0.25;

       R := (R - K*P_Lead) - K*P_Trail;
       R := (R/User_Period) * Two_Pi;

       case I_Flag is
	 when 0 => Z := + KF_Sin(R, Zero);
	 when 1 => Z := + KF_Cos(R, Zero);
	 when 2 => Z := - KF_Sin(R, Zero);
	 when 3 => Z := - KF_Cos(R, Zero);
       end case;

       return Float_Type(Z);
    end Sin; -- with Cycle

    function Cos( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of cos(X) (the cosine of X) is returned.

       Y, R1, R2 : Common_Float;
       I_Flag    : Quadrant_Flag;

    begin

    -- Argument reduction by KP_Pi2Rd

       Y := Common_Float( X );

       KP_Pi2Rd( Y, R1, R2, I_Flag );

       case I_Flag is
	 when 0 => return( + KF_Cos(R1, R2) );
	 when 1 => return( - KF_Sin(R1, R2) );
	 when 2 => return( - KF_Cos(R1, R2) );
	 when 3 => return( + KF_Sin(R1, R2) );
       end case;

    end Cos;

    function Cos( X, Cycle : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type; Cycle is a
    -- user specificed period.
    -- On output, the value of cos(X*2pi/Cycle) is returned.

       R, Y, Z, User_Period, P_Lead, P_Trail, K  : Common_Float;
       Zero                                      : Common_Float := 0.0;
       I_Flag                                    : Quadrant_Flag;

    begin

    -- Argument reduction by  "rem"

       User_Period := Common_Float( Cycle );

       if User_Period <= 0.0 then
	  raise Argument_Error;
       end if;

       Y := Common_Float( X ) rem User_Period;
       R := Y;

       K := Common_Float'Rounding( 4.0*R / User_Period );
       I_Flag := Quadrant_Flag( Integer(K) mod 4 );

       P_Lead  := Common_Float'Leading_Part( User_Period, 4 );
       P_Trail := User_Period - P_Lead;

       K := K * 0.25;

       R := (R - K*P_Lead) - K*P_Trail;
       R := (R/User_Period) * Two_Pi;

       case I_Flag is
	 when 0 => return( + KF_Cos(R, Zero) );
	 when 1 => return( - KF_Sin(R, Zero) );
	 when 2 => return( - KF_Cos(R, Zero) );
	 when 3 => return( + KF_Sin(R, Zero) );
       end case;

    end Cos; -- with Cycle

    function Tan( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of tan(X) (the tangent of X) is returned.

       Y, R1, R2, Z : Common_Float;
       I_Flag       : Quadrant_Flag;

    begin

    -- Argument reduction by KP_Pi2Rd

       if ( X = 0.0 ) then
	  return( X );
       end if;

       Y := Common_Float( X );

       KP_Pi2Rd( Y, R1, R2, I_Flag );

       case I_Flag is
	 when 0|2 => Z :=  KF_Sin(R1, R2) / KF_Cos(R1, R2);
	 when 1|3 => Z := -KF_Cos(R1, R2) / KF_Sin(R1, R2);
       end case;

       return Float_Type(Z);
    end Tan;

    function Tan( X, Cycle : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type; Cycle is a
    -- user specificed period.
    -- On output, the value of tan(X*2pi/Cycle) is returned.

       R, Y, Z, User_Period, P_Lead, P_Trail, K  : Common_Float;
       Zero                                      : Common_Float := 0.0;
       I_Flag                                    : Quadrant_Flag;

    begin

    -- Argument reduction by  "rem"

       User_Period := Common_Float( Cycle );

       if User_Period <= 0.0 then
	  raise Argument_Error;
       end if;

       Y := Common_Float( X ) rem User_Period;

       R := Y;

       K := Common_Float'Rounding( 4.0*R / User_Period );
       I_Flag := Quadrant_Flag( Integer(K) mod 4 );

       P_Lead  := Common_Float'Leading_Part( User_Period, 4 );
       P_Trail := User_Period - P_Lead;

       K := K * 0.25;

       R := (R - K*P_Lead) - K*P_Trail;
       R := (R/User_Period) * Two_Pi;

       case I_Flag is
	 when 0|2 => Z :=  KF_Sin(R, Zero) / KF_Cos(R, Zero);
	 when 1|3 => Z := -KF_Cos(R, Zero) / KF_Sin(R, Zero);
       end case;

       return Float_Type(Z);
    end Tan; -- with Cycle

    function Cot( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of cot(X) (the cotangent of X) is returned.

       Y, R1, R2, Z : Common_Float;
       I_Flag       : Quadrant_Flag;

    begin

    -- Argument reduction by KP_Pi2Rd

       if ( X = 0.0 ) then
	  raise Constraint_Error;
       end if;

       Y := Common_Float( X );

       KP_Pi2Rd( Y, R1, R2, I_Flag );

       case I_Flag is
	 when 0|2 => Z :=  KF_Cos(R1, R2) / KF_Sin(R1, R2);
	 when 1|3 => Z := -KF_Sin(R1, R2) / KF_Cos(R1, R2);
       end case;

       return Float_Type(Z);
    end Cot;

    function Cot( X, Cycle : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type; Cycle is a
    -- user specificed period.
    -- On output, the value of cot(X*2pi/Cycle) is returned.

       R, Y, Z, User_Period, P_Lead, P_Trail, K  : Common_Float;
       Zero                                      : Common_Float := 0.0;
       I_Flag					 : Quadrant_Flag;

    begin

    -- Argument reduction by  "rem"

       User_Period := Common_Float( Cycle );

       if User_Period <= 0.0 then
	  raise Argument_Error;
       end if;

       if ( X = 0.0 ) then
	  raise Constraint_Error;
       end if;

       Y := Common_Float( X ) rem User_Period;

       R := Y;

       K := Common_Float'Rounding( 4.0*R / User_Period );
       I_Flag := Quadrant_Flag( Integer(K) mod 4 );

       P_Lead  := Common_Float'Leading_Part( User_Period, 4 );
       P_Trail := User_Period - P_Lead;

       K := K * 0.25;

       R := (R - K*P_Lead) - K*P_Trail;
       R := (R/User_Period) * Two_Pi;

       case I_Flag is
	 when 0|2 => Z :=  KF_Cos(R, Zero) / KF_Sin(R, Zero);
	 when 1|3 => Z := -KF_Sin(R, Zero) / KF_Cos(R, Zero);
       end case;

       return Float_Type(Z);

    end Cot; -- with Cycle

 -----------------------------------------------------------------------

    function Arcsin(X : Float_Type) return Float_Type is

       Y, sign_X, U, V, One_m_Y, Z1, Z2, Result  : Common_Float;

       Zero    : constant := 0.0;
       One     : constant := 1.0; 
       Two     : constant := 2.0;
       Half    : constant := 0.5;

       Pi_by_2 : constant := 1.57079_63267_94896_61923_13216_91639_75144_2098;

       Piby2_Lead  : constant Common_Float := 16#1.921#;
       Piby2_Trail : constant Common_Float := 
	  16#0.FB544_42D18_46989_8CC51_701B8_39A25_2049C#E-3;

    begin

    -- Filter out exceptional cases.
       if X > One or else x < -One then
	  raise Argument_Error;
       end if;
       Y := Common_Float(X); 
       Sign_X := Common_Float'Copy_Sign(One, Y); 
       Y := abs(Y);

       V := Y;
       if Y < Half then
	  U := One - Y*Y;
       else
	  One_m_Y := Half + (Half - Y);
	  U := Two*One_m_Y  -  One_m_Y*One_m_Y;
       end if;

       U := KF_Sqrt( U );

       if U >= V then
	  KP_Atn( V, U, Z1, Z2 );
	  Result := Z1 + Z2;
       else
	  KP_Atn( U, V, Z1, Z2 );
	  Result := (Piby2_Lead - Z1)  +  (Piby2_Trail - Z2);
       end if;

       return( Float_Type(Common_Float'Copy_Sign(Result,Sign_X)) );

    end Arcsin;

    function Arcsin(X, Cycle : Float_Type) return Float_Type is

       Y, sign_X, U, V, One_m_Y, Z1, Z2, Result, Cy  : Common_Float;

       Zero    : constant := 0.0;
       One     : constant := 1.0; 
       Two     : constant := 2.0;
       Four    : constant := 4.0;
       Half    : constant := 0.5;

    begin

    -- Filter out exceptional cases.
       Cy := Common_Float(Cycle);
       if Cy <= 0.0 then
	  raise Argument_Error;
       end if;

       if X > One or else x < -One then
	  raise Argument_Error;
       end if;
       Y := Common_Float(X); 
       Sign_X := Common_Float'Copy_Sign(One, Y);
       Y := abs(Y);
     
       if Y = One then
	  Result := Cy/Four;
	  return( Float_Type( Common_Float'Copy_Sign(Result,Sign_X)) );
       end if;

       V := Y;
       if Y < Half then
	  U := One - Y*Y;
       else
	  One_m_Y := Half + (Half - Y);
	  U := Two*One_m_Y  -  One_m_Y*One_m_Y;
       end if;

       U := KF_Sqrt( U );

       if U >= V then
	  KP_Atn( V, U, Z1, Z2 );
	  Result := ((Z1 + Z2)*Two_Pi_Inv)*Cy;
       else
	  KP_Atn( U, V, Z1, Z2 );
	  Result := Cy*0.25 - ((Z1 + Z2)*Two_Pi_Inv)*Cy;
       end if;

       return( Float_Type(Common_Float'Copy_Sign(Result,Sign_X)) );

    end Arcsin; -- with Cycle

    function Arccos(X : Float_Type) return Float_Type is

       Y, Sign_X, Z1, Z2, U, V, Result  : Common_Float;

       Zero     : constant := 0.0;
       One      : constant := 1.0; 
       Two      : constant := 2.0;

       Pi_Lead  : constant Common_Float := 16#3.243#;
       Pi_Trail : constant Common_Float :=
	16#0.F6A88_85A30_8D313_198A2_E0370_7344A_40938#E-3;

    begin

    -- Filter out exceptional cases.
       if X > One or else x < -One then
	  raise Argument_Error;
       end if;
       Y := Common_Float(X); 
       Sign_X := Common_Float'Copy_Sign(One, Y); Y := abs(Y);

       V := (One - Y) / (One + Y);
       V := KF_Sqrt( V );  U := One;
       KP_Atn( V, U, Z1, Z2 ); 
       Z1 := Z1 + Z1;  Z2 := Z2 + Z2;


       if Sign_X = One then
	  Result := Z1 + Z2;
       else
	  Result := (Pi_Lead - Z1)  +  (Pi_Trail - Z2);
       end if;

       return( Float_Type(Result) );

    end Arccos;

    function Arccos(X, Cycle : Float_Type) return Float_Type is

       Y, Sign_X, Z1, Z2, U, V, Cy, Result  : Common_Float;

       Zero     : constant := 0.0;
       One      : constant := 1.0; 
       Two      : constant := 2.0;

    begin

    -- Filter out exceptional cases.
       Cy := Common_Float(Cycle);
       if Cy <= Zero then
	  raise Argument_Error;
       end if;

       if X > One or else x < -One then
	  raise Argument_Error;
       end if;
       Y := Common_Float(X); 
       Sign_X := Common_Float'Copy_Sign(One, Y); Y := abs(Y);

       if (Y = One) then
	  if (Sign_X = One) then
	     Result := 0.0;
	  else
	     Result := Cy / Two;
	  end if;
	  return( Float_Type( Result ) );
       end if;

       if (Y = Zero) then
	  Result := Cy / 4.0;
	  return( Float_Type( Result ) );
       end if;

       V := (One - Y) / (One + Y);
       V := KF_Sqrt( V );  U := One;
       KP_Atn( V, U, Z1, Z2 ); 
       Z1 := Z1 + Z1;  Z2 := Z2 + Z2;


       if Sign_X = One then
	  Result := ((Z1 + Z2)*Two_Pi_Inv)*Cy;
       else
	  Result := Cy*0.5 - ((Z1 + Z2)*Two_Pi_Inv)*Cy;
       end if;

       return( Float_Type(Result) );

    end Arccos; -- with Cycle

    function Arctan(Y : Float_Type; X : Float_Type := 1.0) return Float_Type is

       U, V, sign_X, sign_Y, Buffer, Z1, Z2, Result  : Common_Float;
       Swap_Flag                                     : Boolean;

       Zero    : constant := 0.0;
       One     : constant := 1.0; 
       Pi_by_2 : constant := Pi / 2.0;
       Pi_by_4 : constant := Pi / 4.0;
       Pi_3_4  : constant := 3.0 * Pi / 4.0;

       Pi_Lead    : constant Common_Float := 16#3.243#;
       Piby2_Lead : constant Common_Float := 16#1.921#;

       Pi_Trail    : constant Common_Float := Pi - Pi_Lead;
       Piby2_Trail : constant Common_Float := Pi_by_2 - Piby2_Lead;

    begin

    -- Filter out exceptional cases.
       U := Common_Float(X); 
       Sign_X := Common_Float'Copy_Sign(One, U);
       if U = Common_Float'Safe_First then
	  U := Common_Float'Safe_Last;
       else
          U := abs(U); 
       end if;

       V := Common_Float(Y); 
       Sign_Y := Common_Float'Copy_Sign(One, V);
       if V = Common_Float'Safe_First then
	  V := Common_Float'Safe_Last;
       else
          V := abs(V); 
       end if;

       if V = Zero then
	  if U = Zero then
	     raise Argument_Error;
	  else
	     if Sign_X = One then
		return( Float_Type(Common_Float'Copy_Sign(Zero,Sign_Y)) );
	     else
		return( Float_Type(Sign_Y*Pi) );
	     end if;
	  end if;
       end if;

       if U = Zero then
	  return( Float_Type(Sign_Y*Pi_by_2) );
       end if;

       if U = V then
	  if Sign_X = One then
	     return( Float_Type(Sign_Y*Pi_by_4) );
	  else
	     return( Float_Type(Sign_Y*Pi_3_4) );
	  end if;
       end if;


    -- Step 1. Argument Reduction. 
       if U < V then
	  Swap_Flag := True;
	  Buffer := U;
	  U := V;
	  V := Buffer;
       else
	  Swap_Flag := False;
       end if;


    -- Step 2. Approximation. Obtain atan(V/U). This is performed by KP_Atn
    -- which returns atan(V/U) as Z1 + Z2. Moreover, whenever Z1 is non-zero,
    -- it has at most 4 hex digits and satisfies |Z1| >= 1/16.

       KP_Atn( V, U, Z1, Z2 ); 


    -- Step 3. Reconstruction. Obtain atan(Y,X) via Sign_X, Sign_Y, Swap_Flag,
    -- and atan(V/U). The reconstruction is based on three relations:
    --   atan(Y,X)     = sign(Y) * atan(|Y|,X),
    --   atan(|Y|,X)   = pi - atan(|Y|,-X),         
    --   atan(|Y|,|X|) = pi/2 - atan(|X|,|Y|).

       if Swap_Flag = False then

	  if Sign_X = One then
	  -- atan(|Y|,X) = atan(V,U)
	     Result := Z1 + Z2;
	  else
	  -- atan(|Y|,X) = atan(V,-U) = pi - atan(V,U)
	     Result := (Pi_Lead - Z1) + (Pi_Trail - Z2);
	  end if;

       else

	  if Sign_X = One then
	  -- atan(|Y|,X) = atan(U,V) = pi/2 - atan(V,U)
	     Result := (Piby2_Lead - Z1) + (Piby2_Trail - Z2);
	  else 
	  -- atan(|Y|,X) = atan(U,-V) = pi - atan(U,V) = pi - (pi/2 - atan(V,U))
	     Result := (Piby2_Lead + Z1) + (Piby2_Trail + Z2);
	  end if;

       end if;

       return( Float_Type(Common_Float'Copy_Sign(Result,Sign_Y)) );


    end Arctan;

    function Arctan(Y : Float_Type; 
		    X : Float_Type := 1.0;
		    Cycle : Float_Type )  return Float_Type is

       U, V, sign_X, sign_Y, Buffer, Z1, Z2                 : Common_Float;
       Cy, Cy_Lead, Cy_Trail, Result                        : Common_Float;
       Swap_Flag                                            : Boolean;

       Zero       : constant := 0.0;
       One        : constant := 1.0; 

    begin

    -- Filter out exceptional cases.
       Cy := Common_Float(Cycle);
       if Cy <= Zero then
	  raise Argument_Error;
       end if;

       U := Common_Float(X); 
       Sign_X := Common_Float'Copy_Sign(One, U);
       if U = Common_Float'Safe_First then
          U := Common_Float'Safe_Last;
       else
          U := abs(U);
       end if;

       V := Common_Float(Y); 
       Sign_Y := Common_Float'Copy_Sign(One, V);
       if V = Common_Float'Safe_First then
          V := Common_Float'Safe_Last;
       else
          V := abs(V);
       end if;

       if V = Zero then
	  if U = Zero then
	     raise Argument_Error;
	  else
	     if Sign_X = One then
		return( Float_Type(Common_Float'Copy_Sign(Zero,Sign_Y)) );
	     else
		return( Float_Type(Sign_Y*Cy*0.5) );
	     end if;
	  end if;
       end if;

       if U = Zero then
	  return( Float_Type(Sign_Y*Cy*0.25) );
       end if;

       if U = V then
	  if Sign_X = One then
	     return( Float_Type(Sign_Y*Cy*0.125) );
	  else
	     return( Float_Type(Sign_Y*Cy*0.375) );
	  end if;
       end if;


    -- Step 1. Argument Reduction. 
       if U < V then
	  Swap_Flag := True; Buffer := U; U := V; V := Buffer;
       else
	  Swap_Flag := False;
       end if;


    -- Step 2. Approximation. Obtain atan(V/U). This is performed by KP_Atn
    -- which returns atan(V/U) as Z1 + Z2. Moreover, whenever Z1 is non-zero,
    -- it has at most 4 hex digits and satisfies |Z1| >= 1/16.

       KP_Atn( V, U, Z1, Z2 ); 
       Z1 := (Z1*Two_Pi_Inv)*Cy;  Z2 := (Z2*Two_Pi_Inv)*Cy;
     
       Cy_Lead  := Common_Float'Leading_Part( Cy, 
         Common_Float'Machine_Mantissa/2 );
       Cy_Trail := Cy - Cy_Lead;


    -- Step 3. Reconstruction. Obtain atan(Y,X) via Sign_X, Sign_Y, Swap_Flag,
    -- and atan(V/U). The reconstruction is based on three relations:
    --   atan(Y,X,Cycle)     = sign(Y) * atan(|Y|,X,Cycle),
    --   atan(|Y|,X,Cycle)   = Cycle/2 - atan(|Y|,-X,Cycle),         
    --   atan(|Y|,|X|,Cycle) = Cycle/4 - atan(|X|,|Y|,Cycle).

       if Swap_Flag = False then

	  if Sign_X = One then
	  -- atan(|Y|,X,Cycle) = atan(V,U,Cycle)
	     Result := Z1 + Z2;
	  else
	  -- atan(|Y|,X,Cycle) = atan(V,-U,Cycle) = Cycle/2 - atan(V,U,Cycle)
	     Result := (Cy_Lead*0.5 - Z1) + (Cy_Trail*0.5 - Z2);
	  end if;

       else

	  if Sign_X = One then
	  -- atan(|Y|,X,Cycle) = atan(U,V,Cycle) = Cycle/4 - atan(V,U,Cycle)
	     Result := (Cy_Lead*0.25 - Z1) + (Cy_Trail*0.25 - Z2);
	  else 
	  -- atan(|Y|,X,Cycle) = atan(U,-V,Cycle) 
	  --                   = Cycle/2 - atan(U,V,Cycle) 
	  --                   = Cycle/2 - (Cycle/4 - atan(V,U,Cycle))
	     Result := (Cy_Lead*0.25 + Z1) + (Cy_Trail*0.25 + Z2);
	  end if;

       end if;

       return( Float_Type(Common_Float'Copy_Sign(Result,Sign_Y)) );


    end Arctan;  -- with Cycle

    function Arccot(X     : Float_Type;
                    Y     : Float_Type := 1.0) return Float_Type is
           begin return( Arctan(Y,X) ); end Arccot;

    function Arccot(X     : Float_Type;
                    Y     : Float_Type := 1.0;
                    Cycle : Float_Type)        return Float_Type is
           begin return( Arctan(Y,X,Cycle) ); end Arccot;

 -----------------------------------------------------------------------


    function Sinh( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of sinh(X) (the hyperbolic sine of X) is returned.

    -- The definition of sinh(Y) is (exp(Y) - exp(-Y))/2, therefore
    -- the bulk of the computations are performed by the procedure
    -- KP_Exp (Y, M, Z1, Z2) which returns exp(Y) in M, Z1, and Z2
    -- where
    --              exp(Y) = 2**M * ( Z1 + Z2 )
    -- M of integer value, and Z1 only has at most 12 significant bits.

       Z, Sign_Y                           : Common_Float;

       Y, Abs_Y, Z1, Z2, Cond              : Common_Float;
       M, J                                : Integer;

       One         : constant Common_Float := 1.0;

       Base_Digits : constant Common_Float :=
		     Common_Float( 6 * Float_Type'Base'Digits );

       Large_Threshold  : constant Common_Float :=
	  8.0 * Common_Float(Float_Type'Machine_Emax) * 0.6931471806;


    begin

    -- Filter out exceptional cases.

       if ( X = 0.0 ) then
	  return(X);
       end if;

       Y  := Common_Float( X );
       if ( Y > 0.0 ) then
	  Sign_Y := One;
       else
	  Sign_Y := -One;
       end if;

       if Y < -Large_Threshold or else Y > Large_Threshold then
	  raise Constraint_Error;
       end if;

       Abs_Y := abs(Y);

       Cond := Base_Digits * Log2;
       if (Abs_Y >= Cond) then
    --    Get the values of M, Z1, and Z2  so that the natural exponential of Y
    --    can be calculated by  Exp(Y) = 2**M * (Z1 + Z2)

	  KP_Exp (Abs_Y, M, Z1, Z2);

	  M := M - 1;

	  case Radix is
	     when 2 =>
		Y      := Z1 + Z2;

	     when others =>
		J      := M rem 4;
		M      := (M - J) / 4;
		Z1     := Z1 * Two_to(J);
		Z2     := Z2 * Two_to(J);
		Y      := Z1 + Z2;
	  end case;

	  Z    := Sign_Y * Common_Float'Scaling( Y, M );  

    --    Now,  Z = sign(X) * 1/2 * exp( abs(X) ).
    --    When abs(Y) gets so big, subtracting (1/4)/Z will not make
    --    a difference in the outcome of the sinh(X).

	  return ( Float_Type(Z) );

       else

	  Z :=  KF_Em1 (Abs_Y);
	  return ( Float_Type( Sign_Y * 0.5 * (Z+(Z/(Z+1.0))) ) );

       end if;

    end Sinh;

    function Cosh( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of cosh(X) (the hyperbolic cosine of X) is returned.

    -- The definition of cosh(Y) is (exp(Y) + exp(-Y))/2, therefore
    -- the bulk of the computations are performed by the procedure
    -- KP_Exp (Y, M, Z1, Z2) which returns exp(Y) in M, Z1, and Z2
    -- where
    --              exp(Y) = 2**M * ( Z1 + Z2 )
    -- M of integer value, and Z1 only has at most 12 significant bits.

       Z                                   : Common_Float;

       Y, Abs_Y, Z1, Z2                    : Common_Float;
       M, J                                : Integer;

       Base_Digits : constant Common_Float :=
		     Common_Float( 6 * Float_Type'Base'Digits );

       Large_Threshold  : constant Common_Float :=
	  8.0 * Common_Float(Float_Type'Machine_Emax) * 0.6931471806;

       Cond         : constant Common_Float := Base_Digits * Log2;

    begin

    -- Filter out exceptional cases.

       Y     := Common_Float( X );

       if Y < -Large_Threshold or else Y > Large_Threshold then
          raise Constraint_Error;
       end if;
 
       Abs_Y := abs(Y);
 
    -- Get the values of M, Z1, and Z2  so that the natural exponential of Y
    -- can be calculated by  Exp(Y) = 2**M * (Z1 + Z2)

       KP_Exp (Abs_Y, M, Z1, Z2);

       M := M - 1;

       case Radix is
	  when 2 =>
	     Y      := Z1 + Z2;
	  when others =>
	     J      := M rem 4;
	     M      := (M - J) / 4;
	     Z1     := Z1 * Two_to(J);
	     Z2     := Z2 * Two_to(J);
	     Y      := Z1 + Z2;
       end case;

    -- Now,  Z = 1/2 * exp( abs(X) ).

       Z    := Common_Float'Scaling( Y, M );  


       if (Abs_Y >= Cond) then
    -- When abs(Y) gets so big, adding (1/4)/Z will not make a difference 
    -- in the outcome of cosh(X).
	  return ( Float_Type(Z) );
       else
	  return ( Float_Type( Z + 0.25/Z ) );
       end if;

    end Cosh;

    function Tanh( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of tanh(X) (the hyperbolic tangent of X) 
    -- is returned.

    -- The definition of tanh(Y) is sinh(Y)/cosh(Y), which is also equivalent
    -- to the following three formulas.
    --      1.  ( exp(Y) - exp(-Y) ) / ( exp(Y) + exp(-Y) )
    --      2.  ( 1 - ( 2 / ( exp(2*Y) + 1 ) ) )
    --      3.  ( exp(2*Y) - 1 ) / ( exp(2*Y) + 1 ).
    -- but computationally, some formulas are better on some ranges.

       Z, Sign_Y                            : Common_Float;

       Y, Abs_Y                             : Common_Float;

       Base_Digits  : constant Common_Float :=
		      Common_Float( 6 * Float_Type'Base'Digits );

       Log2_Times_2 : constant Common_Float := ( 2.0 * Log2 );

       Cond         : constant Common_Float := ( Base_Digits * Log2 );


    begin

    -- Filter out exceptional cases.

       if (X = 0.0) then
	  return( X );
       end if;

       Y     := Common_Float( X );
       if Y = Common_Float'Safe_First then
          Abs_Y := Common_Float'Safe_Last;
       else
          Abs_Y := abs(Y);
       end if;

       if ( Y >= 0.0 ) then
	  Sign_Y := 1.0;
       else
	  Sign_Y := -1.0;
       end if;


       if ( Abs_Y <= ( Log2_Times_2 ) ) then
    --    Formula 3 should be used in this situation to guarantee accuracy.

	  Z      := KF_Em1 ( 2.0 * Abs_Y );
	  Z      := Sign_Y * ( Z / ( Z + 2.0 ) );
	  return ( Float_Type(Z) );

       elsif (Abs_Y > Cond) then
    --    Formula 2 should be used in this situation to guarantee accuracy,
    --    but observe that 2/(exp(2*Y) + 1) will be so small compared to 1
    --    that it is negligible.

	  return ( Float_Type(Sign_Y) );

       else
    --   When ( Log2_Times_2 < Abs_Y <= Cond ), use formula 2 for best accuracy. 

	  Z := KF_Em1 ( 2.0 * Abs_Y );
	  Z := Sign_Y * ( 1.0 - 2.0 / ( Z + 2.0 ) );
	  return ( Float_Type(Z) );

       end if;


    end Tanh;

    function Coth( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of coth(X) (the hyperbolic cotangent of X) 
    -- is returned.

    -- The definition of coth(Y) is 1/tanh(Y) = cosh(Y)/sinh(Y), which is also
    -- equivalent to the following three formulas.
    --      1.  ( exp(Y) + exp(-Y) ) / ( exp(Y) - exp(-Y) )
    --      2.  ( 1 + ( 2 / ( exp(2*Y) - 1 ) ) )
    --      3.  ( exp(2*Y) + 1 ) / ( exp(2*Y) - 1 ).
    -- but computationally, some formulas are better on some ranges.

       Z, Sign_Y                            : Common_Float;

       Y, Abs_Y                             : Common_Float;


       Base_Digits  : constant Common_Float :=
		      Common_Float( 6 * Float_Type'Base'Digits );

       Log2_Times_2 : constant Common_Float := ( 2.0 * Log2 );

       Cond         : constant Common_Float := ( Base_Digits * Log2 );


    begin

    -- Filter out exceptional cases.

       Y     := Common_Float( X );

       if ( Y = 0.0 ) then
	  raise Constraint_Error;
       end if;

       if Y = Common_Float'Safe_First then
          Abs_Y := Common_Float'Safe_Last;
       else
          Abs_Y := abs(Y);
       end if;

    -- Calculate the coth(X).

       if ( Y >= 0.0 ) then
	  Sign_Y := 1.0;
       else
	  Sign_Y := -1.0;
       end if;


       if ( Abs_Y <= ( Log2_Times_2 ) ) then
    --    Formula 3 should be used in this situation to guarantee accuracy.

	  Z  := KF_Em1 ( 2.0 * Abs_Y );
	  Z  := Sign_Y * ( ( Z + 2.0 ) / Z );
	  return ( Float_Type(Z) );

       elsif (Abs_Y > Cond) then
    --    Formula 2 should be used in this situation to guarantee accuracy,
    --    but observe that 2/(exp(2*Y) - 1) will be so small compared to 1
    --    that it is negligible.

	  return ( Float_Type(Sign_Y) );

       else
    --   When ( Log2_Times_2 < Abs_Y <= Cond ), use formula 2 for best accuracy. 

	  Z := KF_Em1 ( 2.0 * Abs_Y );
	  Z := Sign_Y * ( 1.0 + 2.0 / Z );
	  return ( Float_Type(Z) );

       end if;


    end Coth;

    function Arcsinh( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of Arcsinh(X) (the inverse hyperbolic sin of X)
    --            is returned.

    -- The definition of Arcsinh(Y) is log( Y + sqrt(Y*Y + 1) )
    -- For symmetry, we return sign(Y)*Arcsinh(|Y|). The discussions below
    -- therefore assume Y >= 0.
    -- To obtain good accuracy, we consider several cases:
    -- 1) Y <= epsilon, simply return Y.
    -- 2) epsilon < Y <= 0.5, 
    --    Y + sqrt(Y*Y+1) = 1 + ( Y  +  Y*Y/[1 + sqrt(Y*Y+1)] )
    --                    = 1 + ( Y  +  Y / [ (1/Y) + sqrt(1 + [1/Y*Y]) ] )
    --    A formula best suited for the kernel function L1p.
    -- 3) 0.5 < Y < 10/epsilon,
    --    Y + sqrt(Y*Y+1) = 2( Y  +  0.5/[ sqrt(Y*Y+1) + Y ] ).
    -- 4) 10/epsilon <= Y, then 
    --    Y + sqrt(Y*Y+1) = 2Y for practical purposes.
    -- Note that (3) and (4) are suited for invoking the kernel procedure
    -- KP_Log(Input) which returns M, Z1, and Z2 where 
    --    log(Input) = M * log(2)   +   Z1   +   Z2.
    --
      
       Y, Sign_X, V, M, Z1, Z2, Result                   : Common_Float;

       One  : constant := 1.0;
       Half : constant := 0.5;

       Small_Threshold : constant Common_Float := 
				     Common_Float'Model_Epsilon;
       Large_Threshold : constant Common_Float :=
				     10.0 / Common_Float'Model_Epsilon;

       Log2_Lead  : constant Common_Float := 16#0.B17#;
       Log2_Trail : constant Common_Float :=
		   16#0.000217F7D1CF79ABC9E3B39803F2F6AF40#;

    begin


       Y := Common_Float(X); 
       Sign_X := Common_Float'Copy_Sign(One,Y);
       if Y = Common_Float'Safe_First then 
          Y := Common_Float'Safe_Last;
       else
          Y := abs(Y);
       end if;
 
       if (Y <= Half) then

	  if (Y < Small_Threshold) then
	     return( Float_Type( Common_Float'Copy_Sign(Y,Sign_X) ) );
	  else
	     V := Y + Y/( (One/Y) + KF_Sqrt(One + (One/(Y*Y))) );
	     Result := KF_L1p( V );
	  end if;

       else

	  if (Y < Large_Threshold) then
	     Y := Y + Half/( Y + KF_Sqrt(One + Y*Y) );
	  end if;
	  KP_Log(Y, M, Z1, Z2); M := M + One;
	  Result := M*Log2_Lead + (Z1 + (Z2 + M*Log2_Trail));

       end if;

       return( Float_Type( Common_Float'Copy_Sign( Result, Sign_X ) ) );

    end Arcsinh;

    function Arccosh( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of Arccosh(X) (the inverse hyperbolic cos of X)
    --            is returned.

    -- The definition of Arccosh(Y) is log( Y + sqrt(Y*Y - 1) ), Y >= 1.
    -- To obtain good accuracy, we consider several cases:
    -- 1) Y = 1, simply return 0.
    -- 2) 1 < Y <= sqrt(2)
    --    Y + sqrt(Y*Y-1) = Y + sqrt( (Y-1)*(Y+1) )
    -- 3) sqrt(2) < Y < 10/epsilon,
    --    Y + sqrt(Y*Y-1) = 2( Y  -  0.5/[ sqrt(Y*Y-1) + Y ] ).
    -- 4) 10/epsilon <= Y, then 
    --    Y + sqrt(Y*Y-1) = 2Y for practical purposes.
    -- Note that (3) and (4) are suited for invoking the kernel procedure
    -- KP_Log(Input) which returns M, Z1, and Z2 where 
    --    log(Input) = M * log(2)   +   Z1   +   Z2.
    --
      
       Y1, Y, M, Z1, Z2, Result                   : Common_Float;

       Zero  : constant := 0.0;
       One   : constant := 1.0;
       Half  : constant := 0.5;
       Root2 : constant := 1.41421_35623_73095_04880_16887_24209_69807;
       
       Large_Threshold : constant Common_Float :=
			   10.0 / Common_Float'Model_Epsilon;
       Log2_Lead  : constant Common_Float := 16#0.B17#;
       Log2_Trail : constant Common_Float :=
		   16#0.000217F7D1CF79ABC9E3B39803F2F6AF40#;

    begin


       Y := Common_Float(X); 
       if (Y < One) then
	  raise Argument_Error;
       end if;

       if (Y <= Root2) then

	  if (Y = One) then
	     return( Float_Type(Zero) );
	  else
	     Y1 := Y - One;
	     Y := Y1 + KF_Sqrt( Y1*(Y+One) );
	     return( Float_Type(KF_L1p( Y )) );
	  end if;

       else

	  if (Y < Large_Threshold) then
	     Y := Y - Half/(Y + KF_Sqrt( (Y-One)*(Y+One) ));
	  end if;
	  KP_Log( Y, M, Z1, Z2 );
	  M := M + One;

       end if;

       Result := M*Log2_Lead + (Z1 + (Z2 + M*Log2_Trail));


       return( Float_Type(Result) );

    end Arccosh;

    function Arctanh( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of Arctanh(X) (the inverse hyperbolic tangent of X)
    --            is returned.

    -- The definition of Arctanh(Y) is log((1+Y)/(1-Y)) / 2, which is also 
    -- equivalent to the following three formulas:
    --      1.  ( log(1+Y) - log(1-Y) ) / 2
    --      2.  ( log(Y+1) - log(-Y+1) ) / 2.
    --      3.  log( 1 + ( (2*Y) / (1-Y) ) ) / 2.
    -- but computationally, the last formula is better.

       Z, Sign_Y                            : Common_Float;

       Y, Abs_Y, Temp                       : Common_Float;

       Log2_Times_2 : constant Common_Float := ( 2.0 * Log2 );


    begin

    -- Filter out exceptional cases.

       if ( X = 0.0 ) then
	  return( X );
       end if;

       Y     := Common_Float( X );
       if Y = Common_Float'Safe_First then
          Abs_Y := Common_Float'Safe_Last;
       else
          Abs_Y := abs(Y);
       end if;

       if ( Abs_Y = 1.0 ) then
	  raise Constraint_Error;
       end if;

       if ( Abs_Y > 1.0 ) then
	  raise Argument_Error;
       end if;

    -- Calculate Arctanh(Y) by using KF_L1p.

       if ( Y >= 0.0 ) then
	  Sign_Y := 1.0;
       else
	  Sign_Y := -1.0;
       end if;

       Temp   := ( 2.0 * Abs_Y ) / ( 1.0 - Abs_Y );
       Temp   := KF_L1p ( Temp );
       Z      := Sign_Y *  0.5 * Temp;
       return ( Float_Type(Z) );

    end Arctanh;

    function Arccoth( X : Float_Type ) return Float_Type is

    -- On input, X is a floating-point value in Float_Type;
    -- On output, the value of Arccoth(X) (the inverse hyperbolic 
    --            cotangent of X) is returned.

    -- The definition of Arccoth(Y) is log((Y+1)/(Y-1)) / 2, which is also 
    -- equivalent to the following three formulas:
    --      1.  ( log(Y+1) - log(Y-1) ) / 2
    --      2.  ( log(Y+1) - log(Y+1-2) ) / 2.
    --      3.  log( 1 + ( 2 / (Y-1) ) ) / 2.
    -- but computationally, the last formula is better.

       Z, Sign_Y                            : Common_Float;

       Y, Abs_Y, Temp                       : Common_Float;

       Log2_Times_2 : constant Common_Float := ( 2.0 * Log2 );


    begin

    -- Filter out exceptional cases.

       Y     := Common_Float( X );
       if Y = Common_Float'Safe_First then
          Abs_Y := Common_Float'Safe_Last;
       else
          Abs_Y := abs(Y);
       end if;
 
       if ( Abs_Y = 1.0 ) then
	  raise Constraint_Error;
       end if;

       if ( Abs_Y < 1.0 ) then
	  raise Argument_Error;
       end if;

    -- Calculate Arccoth(Y) by using KF_L1p.

       if ( Y >= 0.0 ) then
	  Sign_Y := 1.0;
       else
	  Sign_Y := -1.0;
       end if;

       Temp   := 2.0 / ( Abs_Y - 1.0 );
       Temp   := KF_L1p ( Temp );
       Z      := Sign_Y * 0.5 * Temp;
       return ( Float_Type(Z) );

    end Arccoth;

end Ada.Numerics.Generic_Elementary_Functions;
